Package: Gmisc
Version: 3.4.0
Title: Descriptive Statistics, Transition Plots, and More
Authors@R: c(
    person(given = "Max", family = "Gordon",
           email = "max@gforge.se", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8080-5815")))
Maintainer: Max Gordon <max@gforge.se>
Description: Utilities for common medical-statistics graphics
    and tables, including automatic creation of publication-ready "Table 1"
    summaries; transition (Sankey) plots; flow-chart helpers that extend the
    grid package; Bézier lines and arrows; and a singular-value-decomposition-
    based variable-selection method.
License: GPL (>= 3)
URL: https://gforge.se
BugReports: https://github.com/gforge/Gmisc/issues
Depends: R (>= 4.1.0), htmlTable (>= 2.0.0)
Imports: Rcpp (>= 0.11.4), abind, checkmate, forestplot, Hmisc, glue,
        grid, grDevices, graphics, knitr, lattice, lubridate, magrittr,
        methods, rlang, rmarkdown, stringr, stats, XML, yaml, utils
Suggests: bench, vdiffr, datasets, dplyr, jsonlite, testthat,
        tidyselect, webshot2
Encoding: UTF-8
NeedsCompilation: yes
LinkingTo: Rcpp
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-20 10:49:57 UTC; max
Author: Max Gordon [aut, cre] (ORCID: <https://orcid.org/0000-0002-8080-5815>)
Repository: CRAN
Date/Publication: 2026-06-20 11:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-20 15:21:16 UTC; unix
Archs: Gmisc.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(align,Gmisc_list_of_boxes)
S3method(align,default)
S3method(align,list)
S3method(append,Gmisc_list_of_boxes)
S3method(append,default)
S3method(connect,Gmisc_list_of_boxes)
S3method(connect,default)
S3method(getDescriptionStatsBy,data.frame)
S3method(getDescriptionStatsBy,default)
S3method(grid.draw,connect_boxes_list)
S3method(heightDetails,box)
S3method(htmlTable,Gmisc_getDescriptionStatsBy)
S3method(htmlTable,descMrg)
S3method(insert,Gmisc_list_of_boxes)
S3method(insert,default)
S3method(knit_print,Gmisc_getDescriptionStatsBy)
S3method(knit_print,descMrg)
S3method(latex,descMrg)
S3method(length,Gmisc_getDescriptionStatsBy)
S3method(move,Gmisc_list_of_boxes)
S3method(move,default)
S3method(phaseLabel,Gmisc_list_of_boxes)
S3method(phaseLabel,default)
S3method(plot,box)
S3method(plot,connect_boxes)
S3method(plot,connect_boxes_list)
S3method(position,box)
S3method(position,coords)
S3method(position,default)
S3method(prCalculateConnector,ConnectorStrategy)
S3method(prCalculateConnector,ManyToOneFanInCenterConnectorStrategy)
S3method(prCalculateConnector,ManyToOneFanInTopConnectorStrategy)
S3method(prCalculateConnector,ManyToOneNConnectorStrategy)
S3method(prCalculateConnector,OneToManyNConnectorStrategy)
S3method(print,Gmisc_connector_label)
S3method(print,Gmisc_extended_connector_labels)
S3method(print,Gmisc_getDescriptionStatsBy)
S3method(print,Gmisc_list_of_boxes)
S3method(print,Gmisc_unit)
S3method(print,box)
S3method(print,connect_boxes)
S3method(print,connect_boxes_list)
S3method(print,descMrg)
S3method(spread,Gmisc_list_of_boxes)
S3method(spread,default)
S3method(spread,list)
S3method(widthDetails,box)
S3method(yamlDump,character)
S3method(yamlDump,default)
S3method(yamlDump,json)
export(MidDistanceX)
export(MidDistanceY)
export(align)
export(alignHorizontal)
export(alignVertical)
export(append)
export(bezierArrowGradient)
export(bezierArrowSmpl)
export(boxDatabaseGrob)
export(boxDiamondGrob)
export(boxDocumentGrob)
export(boxDocumentsGrob)
export(boxEllipseGrob)
export(boxGrob)
export(boxHeaderGrob)
export(boxPropGrob)
export(boxRackGrob)
export(boxServerGrob)
export(boxTapeGrob)
export(connect)
export(connectGrob)
export(convertShowMissing)
export(coords)
export(copyAllNewAttributes)
export(database_box_fn)
export(descGetMissing)
export(describeFactors)
export(describeMean)
export(describeMedian)
export(describeProp)
export(diamond_rounded_box_fn)
export(diamond_sharp_box_fn)
export(distance)
export(document_box_fn)
export(documents_box_fn)
export(docx_document)
export(ellipse_box_fn)
export(equalizeHeights)
export(equalizeWidths)
export(fastDoCall)
export(figCapNo)
export(figCapNoLast)
export(figCapNoNext)
export(flowchart)
export(forestplot2)
export(getDescriptionStatsBy)
export(getPvalAnova)
export(getPvalChiSq)
export(getPvalFisher)
export(getPvalKruskal)
export(getPvalWilcox)
export(getSvdMostInfluential)
export(gnrlBezierPoints)
export(has)
export(insert)
export(insertRowAndKeepAttr)
export(labelConnector)
export(mergeDesc)
export(mergeLists)
export(move)
export(moveBox)
export(pathJoin)
export(phaseLabel)
export(position)
export(rack_box_fn)
export(retrieve)
export(server_box_fn)
export(setConnectorLabels)
export(set_column_labels)
export(set_column_units)
export(spread)
export(spreadHorizontal)
export(spreadVertical)
export(tape_box_fn)
export(time2spanTxt)
export(transitionPlot)
export(yamlDump)
exportClasses(Transition)
import(XML)
import(abind)
import(forestplot)
import(grid)
import(htmlTable)
import(lattice)
import(magrittr)
importFrom(Hmisc,"label<-")
importFrom(Hmisc,capitalize)
importFrom(Hmisc,label)
importFrom(Hmisc,latex)
importFrom(Hmisc,units)
importFrom(Rcpp,evalCpp)
importFrom(checkmate,assert)
importFrom(checkmate,assert_class)
importFrom(checkmate,assert_list)
importFrom(checkmate,assert_number)
importFrom(checkmate,checkCharacter)
importFrom(checkmate,checkClass)
importFrom(checkmate,checkList)
importFrom(checkmate,checkNumeric)
importFrom(checkmate,checkString)
importFrom(checkmate,checkTRUE)
importFrom(glue,glue)
importFrom(grDevices,as.raster)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,grey)
importFrom(grDevices,rgb)
importFrom(graphics,par)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,is.unit)
importFrom(knitr,asis_output)
importFrom(knitr,knit_print)
importFrom(lubridate,as_date)
importFrom(lubridate,mday)
importFrom(lubridate,month)
importFrom(lubridate,year)
importFrom(methods,formalArgs)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setRefClass)
importFrom(rlang,enquo)
importFrom(rlang,expr)
importFrom(rmarkdown,html_document)
importFrom(stats,anova)
importFrom(stats,chisq.test)
importFrom(stats,fisher.test)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,wilcox.test)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_trim)
importFrom(utils,as.roman)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(yaml,as.yaml)
useDynLib(Gmisc)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# Gmisc

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/Gmisc)](https://CRAN.R-project.org/package=Gmisc)
[![CRAN
downloads](https://cranlogs.r-pkg.org/badges/Gmisc)](https://CRAN.R-project.org/package=Gmisc)
[![R-CMD-check](https://github.com/gforge/Gmisc/actions/workflows/ci.yaml/badge.svg)](https://github.com/gforge/Gmisc/actions/workflows/ci.yaml)
<!-- badges: end -->

**Gmisc** collects utilities for the graphics and tables that recur in
medical research papers — built so they compose with the native R pipe
(`|>`):

- **Descriptive “Table 1”** — `getDescriptionStatsBy()` + `htmlTable()`
  for publication-ready, copy-paste descriptive tables.
- **Grid-based flowcharts** — a
  `flowchart() |> spread() |> move() |> connect()` pipeline for
  CONSORT-style diagrams, including `phaseLabel()` headings.
- **Transition plots** — the `Transition` class for visualising how
  observations move between categories over time.
- **SVD variable selection** — `getSvdMostInfluential()` for picking
  influential variables.
- **Bézier arrows** — smooth arrows complementing the `grid` package.

## Installation

``` r
# From CRAN
install.packages("Gmisc")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("gforge/Gmisc")
```

## Flowcharts & CONSORT diagrams

Build a flowchart as a named list of boxes, position the columns with
`spread()`/`move()`, then add the connectors. Parallel arms are just
lists, and `phaseLabel()` drops a CONSORT phase heading (“Allocation”,
“Follow-up”, …) between the arms — centred, slightly overlapping, and
drawn on top.

``` r
library(Gmisc)
library(grid)

main_gp <- gpar(fill = "white", col = "black", lwd = 1)
head_gp <- gpar(fill = "#c8daf7", col = "#2f5f9f", lwd = 1)
con_gp  <- gpar(col = "#4f86c6", fill = "#4f86c6", lwd = 1.8)
sw      <- unit(70, "mm")

flowchart(
  rando = boxGrob("Randomised\nN = 100", box_gp = main_gp),
  groups = list(
    boxGrob("Allocated to intervention\nn = 50", width = sw, box_gp = main_gp),
    boxGrob("Allocated to control\nn = 50", width = sw, box_gp = main_gp)
  ),
  followup = list(
    boxGrob("Lost to follow-up\nn = 1", width = sw, box_gp = main_gp),
    boxGrob("Lost to follow-up\nn = 2", width = sw, box_gp = main_gp)
  ),
  analysis = list(
    boxGrob("Analysed\nn = 49", width = sw, box_gp = main_gp),
    boxGrob("Analysed\nn = 48", width = sw, box_gp = main_gp)
  )
) |>
  spread(axis = "y", margin = unit(0.04, "npc")) |>
  move(subelement = list(c("groups", 1), c("followup", 1), c("analysis", 1)), x = 0.27) |>
  move(subelement = list(c("groups", 2), c("followup", 2), c("analysis", 2)), x = 0.73) |>
  phaseLabel("groups",   "Allocation", box_gp = head_gp) |>
  phaseLabel("followup", "Follow-up",  box_gp = head_gp) |>
  phaseLabel("analysis", "Analysis",   box_gp = head_gp) |>
  connect("rando",    "groups",   type = "N", lty_gp = con_gp, arrow_size = 3, smooth = TRUE) |>
  connect("groups",   "followup", type = "v", lty_gp = con_gp, arrow_size = 3) |>
  connect("followup", "analysis", type = "v", lty_gp = con_gp, arrow_size = 3)
```

<img src="man/figures/README-flowchart-1.png" alt="" width="100%" />

See `vignette("Grid-based_flowcharts", package = "Gmisc")` for the full
API.

## Descriptive “Table 1”

`getDescriptionStatsBy()` summarises variables split by a grouping
column. It is often used with `mergeDesc()` to group related variables
into sections, and pipes straight into `htmlTable()` for a
publication-ready table.

``` r
library(dplyr)
library(Gmisc)

# A custom wrapper to keep statistics and formatting consistent
# (e.g., same digits, p-values, and header count)
get_stats <- function(data, ...) {
  res <- data |>
    getDescriptionStatsBy(...,
                          by = am,
                          statistics = TRUE,
                          digits = 1,
                          header_count = TRUE)
  if (is.list(res)) {
    return(do.call(rbind, res))
  }
  return(res)
}

mtcars_prep <- mtcars |>
  mutate(am = factor(am, labels = c("Automatic", "Manual")),
         gear = factor(gear),
         cyl = factor(cyl)) |>
  set_column_labels(mpg = "Gas",
                    wt = "Weight",
                    hp = "Horsepower",
                    cyl = "Cylinders",
                    gear = "Gears") |>
  set_column_units(mpg = "Miles/gallon",
                   wt = "10<sup>3</sup> lbs",
                   hp = "hp")

# Group variables and merge them into a single table
mergeDesc(
  "Main" = mtcars_prep |> get_stats(mpg, wt),
  "Engine" = mtcars_prep |> get_stats(hp, cyl),
  "Transmission" = mtcars_prep |> get_stats(gear)
) |>
  htmlTable(caption = "Baseline characteristics by transmission",
            tfoot = "&dagger; Statistics: Mean (SD) for continuous; n (%) for categorical")
```

<img src="man/figures/README-table1.png" alt="" width="70%" />

See `vignette("Descriptives", package = "Gmisc")` for the many
formatting options.

## Transition plots

The `Transition` class shows how observations move between classes over
time; a third dimension can be encoded as a colour split within each
box.

``` r
set.seed(1)
n <- 100
sex    <- sample(c("Male", "Female"), n, replace = TRUE)
before <- sample(1:3, n, replace = TRUE)
# Most cases improve one class, some stay, a few worsen
after  <- pmin(pmax(before - sample(c(-1, 0, 1), n, replace = TRUE, prob = c(.15, .35, .5)), 1), 3)

lbl <- c("A", "B", "C")
tbl <- table(factor(before, 1:3, lbl), factor(after, 1:3, lbl), sex)

transitions <- getRefClass("Transition")$new(tbl, label = c("Before surgery", "1 year after"))
transitions$title   <- "Charnley class before vs. after surgery"
transitions$clr_bar <- "bottom"
transitions$render()
```

<img src="man/figures/README-transition-1.png" alt="" width="70%" />

See `vignette("Transition-class", package = "Gmisc")` for customisation.

## Learn more

- Project page: <https://gforge.se>
- Vignettes: `browseVignettes("Gmisc")`
- Bugs & feature requests: <https://github.com/gforge/Gmisc/issues>

library(grid)

# In interactive sessions, pause between pages produced by this example.
if (interactive()) {
  old_ask <- grDevices::devAskNewPage(TRUE)
  on.exit(grDevices::devAskNewPage(old_ask), add = TRUE)
}

grid.newpage()

# Build a flowchart object
boxes <- flowchart(
  start = boxGrob("Top", x = .5, y = .8),
  end = boxGrob("Bottom", x = .5, y = .2),
  side = boxPropGrob("Side", "Left", "Right", prop = .3, x = .2, y = .8),
  exclude = boxGrob("Exclude:\n - Too sick\n - Prev. surgery", x = .8, y = .5, just = "left")
)

# Connect using the pipe-friendly S3 API
# Note: type = "vertical" connects top/bottom faces with a straight segment;
# if x positions differ, the segment may be diagonal.
boxes <- boxes |>
  connect(from = "start", to = "end", type = "vertical") |>
  connect(from = "start", to = "side", type = "horizontal") |>
  connect(from = "start", to = "exclude", type = "L")

print(boxes)

# Start a fresh page for split-box connector examples.
grid.newpage()

# We can also connect to/from lists
side_boxes <- list(
  left = boxGrob("Left", x = attr(boxes$side, "coords")$left_x, y = .5),
  right = boxGrob("Right", x = attr(boxes$side, "coords")$right_x, y = .5)
)

connectGrob(boxes$side, side_boxes$left, "v", "l")
connectGrob(boxes$side, side_boxes$right, "v", "r")

# Start a fresh page for fan-in example.
grid.newpage()

# Fan-in center example: multiple starts into one center bus and single trunk
flowchart(
  start = boxes$start,
  S2 = boxGrob("S2", x = .3, y = .7),
  S3 = boxGrob("S3", x = .7, y = .7),
  end = boxes$end
) |>
  connect(from = c("start", "S2", "S3"), to = "end", type = "fan_in_center") |>
  print()

# Start a fresh page for axis-preserving connector examples.
grid.newpage()

# vertical_axis is the axis-preserving alternative: it preserves each source's
# x-position when landing on the boundary of a wide shared target.
flowchart(
  A = boxGrob("A", x = .25, y = .75),
  B = boxGrob("B", x = .50, y = .75),
  C = boxGrob("C", x = .75, y = .75),
  target = boxGrob("Shared target", x = .5, y = .2, width = unit(.75, "npc"))
) |>
  connect(from = c("A", "B", "C"), to = "target", type = "vertical_axis") |>
  print()

grid.newpage()

# horizontal_axis is the axis-preserving alternative: it preserves each
# source's y-position when landing on the boundary of a tall shared target.
flowchart(
  A = boxGrob("A", x = .20, y = .25),
  B = boxGrob("B", x = .20, y = .50),
  C = boxGrob("C", x = .20, y = .75),
  target = boxGrob("Shared\ntarget", x = .8, y = .5, height = unit(.75, "npc"))
) |>
  connect(from = c("A", "B", "C"), to = "target", type = "horizontal_axis") |>
  print()

# Start a fresh page for spread/assignment example.
grid.newpage()

# When using spread/align, use the returned objects for connectors
visits <- flowchart(
  visit1 = boxGrob("Visit 1", x = .1, y = .35),
  visit2 = boxGrob("Visit 2", x = .2, y = .35),
  visit3 = boxGrob("Visit 3", x = .8, y = .35)
)

# Incorrect pattern (no assignment): spread result is discarded
visits |>
  spread(axis = "x", from = .05, to = .95, type = "between")

# This connector uses original coordinates because 'visits' was unchanged
connectGrob(visits$visit1, visits$visit2, type = "horizontal")

# Correct pattern: assign returned boxes and connect those
visits_spread <- visits |>
  spread(axis = "x", from = .05, to = .95, type = "between")
connectGrob(visits_spread$visit1, visits_spread$visit2, type = "horizontal")

# Print the boxes
boxes
side_boxes
visits
visits_spread

library(grid)
grid.newpage()

# Create a set of boxes
start <- boxGrob("Top", x = .5, y = .8)
end <- boxGrob("Bottom", x = .5, y = .2)
side <- boxPropGrob("Side", "Left", "Right", prop = .3, x = .2, y = .8)
exclude <- boxGrob("Exclude:\n - Too sick\n - Prev. surgery", x = .8, y = .5, just = "left")

# We can chain the spread operations from a flowchart object
flowchart(
  start = start,
  middle = list(side, exclude),
  end = end
) |>
  spread(axis = "y") |>
  spread(axis = "x", subelement = "middle", from = 0.2, to = 0.8)

# Use device-level paging in interactive sessions so users can inspect the first plot
# This will prompt before creating the next page; we restore the previous setting afterwards
if (interactive()) {
  old_ask <- grDevices::devAskNewPage(TRUE)
  on.exit(grDevices::devAskNewPage(old_ask), add = TRUE)
  grid.newpage() # will prompt on interactive devices
} else {
  # non-interactive: just start a fresh page for the next example
  grid.newpage()
}

# Example: spread a nested subelement by deep path and print sequentially
flowchart(
  anchor = boxGrob("Anchor", x = .5, y = .15),
  grp = list(middle = list(side, exclude))
) |>
  spread(axis = "x", subelement = c("grp", "middle"), from = 0.2, to = 0.8)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

library(Gmisc, quietly = TRUE)
library(glue)
library(htmlTable)
library(grid)

## ----flowchart-example, fig.height = 7, fig.width = 8-------------------------
# Shared styling
main_box_gp <- gpar(fill = "#ddeeff", col = "#336699", lwd = 1.5)
group_box_gp <- gpar(fill = "#e8f4e8", col = "#2e7d32", lwd = 1.5)
excl_box_gp  <- gpar(fill = "#fff8e1", col = "#cc8800", lwd = 1.2)
main_con_gp  <- gpar(col = "#336699", lwd = 1.5, fill = "#336699")
excl_con_gp  <- gpar(col = "#cc8800", lwd = 1.2, fill = "#cc8800")

grid.newpage()
flowchart(
  source = boxGrob(glue("Stockholm population\nn = {pop}", pop = txtInt(1632798)),
                   box_gp = main_box_gp),
  eligible = boxGrob(glue("Eligible\nn = {pop}", pop = txtInt(10032)),
                     box_gp = main_box_gp),
  included = boxGrob(glue("Randomized\nn = {incl}", incl = txtInt(122)),
                     box_gp = main_box_gp),
  groups = list(
    boxGrob(glue("Treatment A\nn = {n}", n = txtInt(43)),        box_gp = group_box_gp),
    boxGrob(glue("Treatment B\nn = {n}", n = txtInt(122-43-30)), box_gp = group_box_gp)
  )) |>
  spread(axis = "y") |>
  spread(subelement = "groups", axis = "x") |>
  equalizeWidths(subelement = list("source", "eligible", "included")) |>
  equalizeWidths(subelement = "groups") |>
  insert(list(excluded = boxHeaderGrob(
                header = glue("Excluded (n = {tot}):", tot = 30),
                body   = glue(" - not interested: {n1}\n - contra-indicated: {n2}",
                              n1 = 12, n2 = 18),
                bjust      = "left",
                box_gp     = excl_box_gp,
                header_gp  = gpar(col = "#cc8800", cex = 1))),
         after = "eligible",
         name  = "excluded") |>
  move(subelement = "excluded", x = .8) |>
  connect("eligible", "excluded", type = "L",    lty_gp = excl_con_gp, arrow_size = 3,
          label = "Excluded") |>
  connect("source",   "eligible", type = "vert", lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE) |>
  connect("eligible", "included", type = "vert", lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE) |>
  connect("included", "groups",   type = "N",    lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE)

## ----consort-stage-headings, fig.height = 7, fig.width = 8--------------------
old_opts <- options(boxGrobTxtPadding = unit(2, "mm"))

main_box_gp <- gpar(fill = "white", col = "black", lwd = 1)
heading_gp  <- gpar(fill = "#c8daf7", col = "#2f5f9f", lwd = 1)
con_gp      <- gpar(col = "#4f86c6", fill = "#4f86c6", lwd = 1.8)
side_width  <- unit(70, "mm")

flowchart(
  rando = boxGrob("Randomised\nN = 100", box_gp = main_box_gp),
  groups = list(
    boxGrob("Allocated to intervention\nn = 50",
            width = side_width, box_gp = main_box_gp),
    boxGrob("Allocated to control\nn = 50",
            width = side_width, box_gp = main_box_gp)
  ),
  followup = list(
    boxGrob("Lost to follow-up\nn = 1",
            width = side_width, box_gp = main_box_gp),
    boxGrob("Lost to follow-up\nn = 2",
            width = side_width, box_gp = main_box_gp)
  ),
  analysis = list(
    boxGrob("Analysed\nn = 49",
            width = side_width, box_gp = main_box_gp),
    boxGrob("Analysed\nn = 48",
            width = side_width, box_gp = main_box_gp)
  )
) |>
  spread(axis = "y", margin = unit(0.04, "npc")) |>
  # Spread each stage's arms within the viewport, leaving an outer margin —
  # this gives the two arms a clear central gap for the labels to sit in
  spread(subelement = "groups",   axis = "x", margin = unit(0.05, "npc")) |>
  spread(subelement = "followup", axis = "x", margin = unit(0.05, "npc")) |>
  spread(subelement = "analysis", axis = "x", margin = unit(0.05, "npc")) |>
  # One call per stage: centred between the arms, slightly above, drawn on top
  phaseLabel("groups",   "Allocation", box_gp = heading_gp) |>
  phaseLabel("followup", "Follow-up",  box_gp = heading_gp) |>
  phaseLabel("analysis", "Analysis",   box_gp = heading_gp) |>
  connect("rando",    "groups",   type = "N", lty_gp = con_gp, arrow_size = 3,
          smooth = TRUE) |>
  connect("groups",   "followup", type = "v", lty_gp = con_gp, arrow_size = 3) |>
  connect("followup", "analysis", type = "v", lty_gp = con_gp, arrow_size = 3)

options(old_opts)

## ----consort_example, fig.width = 9, fig.height = 6---------------------------
old_opts <- options(boxGrobTxtPadding = unit(3, "mm"))

box_fill     <- gpar(fill = "#ddeeff", col = "#336699", lwd = 1.5)
con_gp       <- gpar(col = "#336699", lwd = 1.5, fill = "#336699")
side_gp      <- gpar(col = "#cc8800", lwd = 1.2, fill = "#cc8800")
excl_fill    <- gpar(fill = "#fff8e1", col = "#cc8800", lwd = 1.2)
badge_gp     <- gpar(fill = "#336699", col = NA)
badge_txt_gp <- gpar(col = "white", cex = 0.65)

# Main arms use an inner horizontal span; exclusions use the outer span.
main_arm_margin  <- 0.28
exclusion_margin <- 0.05
main_x           <- 0.5
exclusion_to     <- 1 - exclusion_margin

grid.newpage()
flowchart(
  assessed = boxGrob(
    "Patients assessed for eligibility",
    x = main_x, box_gp = box_fill,
    badge_label = "840", badge_gp = badge_gp, badge_txt_gp = badge_txt_gp
  ),
  randomised = boxGrob(
    "Randomised",
    x = main_x, box_gp = box_fill,
    badge_label = "126", badge_gp = badge_gp, badge_txt_gp = badge_txt_gp
  ),
  arms = list(
    cast     = boxGrob("Randomised to\ncast immobilisation",
                       box_gp = box_fill,
                       badge_label = "62",
                       badge_gp = badge_gp, badge_txt_gp = badge_txt_gp),
    surgical = boxGrob("Randomised to\nsurgery",
                       box_gp = box_fill,
                       badge_label = "64",
                       badge_gp = badge_gp, badge_txt_gp = badge_txt_gp)
  ),
  # Lost-to-follow-up: one per arm, placed on the outer exclusion span
  # and centred vertically between allocation and analysis.
  lost = list(
    lost_cast     = boxGrob("Lost to follow-up (n = 2)\n  1 No response\n  1 Other surgery",
                            just = "left", box_gp = excl_fill),
    lost_surgical = boxGrob("Lost to follow-up (n = 3)\n  2 No response\n  1 Other surgery",
                            just = "left", box_gp = excl_fill)
  ),
  analysis = list(
    analysis_cast     = boxGrob("Included in\nprimary analysis",
                                box_gp = box_fill,
                                badge_label = "60",
                                badge_gp = badge_gp, badge_txt_gp = badge_txt_gp),
    analysis_surgical = boxGrob("Included in\nprimary analysis",
                                box_gp = box_fill,
                                badge_label = "61",
                                badge_gp = badge_gp, badge_txt_gp = badge_txt_gp)
  )
) |>
  # Vertical spacing — lost boxes are side branches, not main pathway rows
  spread(axis = "y", margin = unit(5, "mm"), exclude = "lost") |>
  align(axis = "y", subelement = "lost",
        references = list("arms", "analysis")) |>
  # Make arms and analysis boxes the same width so the same from/to spread
  # places their centres at matching x positions (otherwise the wider arm text
  # shifts centres relative to the narrower analysis text).
  # Alternative: skip equalizeWidths() and explicitly align x-centres,
  # e.g. with `align(axis = "x", reference = "analysis", subelement = "arms")`
  # (equivalent to `alignHorizontal(reference = "analysis", subelement = "arms")`).
  equalizeWidths(subelement = list("arms", "analysis")) |>
  # Arms and analysis in the inner span; lost boxes use the outer span
  # so that lost_cast lands left of cast and lost_surgical right of surgical
  spread(axis = "x", subelement = "arms",     margin = main_arm_margin) |>
  spread(axis = "x", subelement = "analysis", margin = main_arm_margin) |>
  spread(axis = "x", subelement = "lost",     margin = exclusion_margin) |>
  # Exclusion box: right edge on the outer span, vertically centred between rows
  insert(list(excluded = boxGrob(
    "Excluded (n = 714)\n  477 Stable ankle mortise\n   64 Incongruent ankle mortise\n   30 Previous serious trauma\n  143 Other reasons",
    just = "left", box_gp = excl_fill
  )), after = "assessed") |>
  move(subelement = "excluded", x = exclusion_to, just = "right") |>
  align(axis = "y", subelement = "excluded",
        references = list("assessed", "randomised"))  |>
  # type = "L": exits assessed's *bottom* then turns right — the "down then right" branch
  connect("assessed", "excluded", type = "L", lty_gp = side_gp, arrow_size = 3, smooth = TRUE) |>
  # Pairwise arm -> lost: sharp corners (smooth = FALSE) avoid a colour-transition
  # artefact where the orange arc would diverge from the shared blue vertical path
  # a few mm above the junction, making the line appear doubled.
  connect("arms", "lost", type = "L", lty_gp = side_gp, arrow_size = 3, smooth = TRUE) |>
  # Main-flow connectors, put last so that the main lines end up on top of the exclusion/lost lines
  connect("assessed",   "randomised", type = "v", lty_gp = con_gp,  arrow_size = 3, smooth = TRUE) |>
  connect("randomised", "arms",       type = "N", lty_gp = con_gp,  arrow_size = 3, smooth = TRUE) |>
  connect("arms",       "analysis",   type = "v", lty_gp = con_gp,  arrow_size = 3)

options(old_opts)

## ----consort_dotted_return_arrows, fig.width = 10, fig.height = 9-------------
old_opts <- options(boxGrobTxtPadding = unit(1, "mm"))

main_gp <- gpar(fill = "white", col = "black")
main_con_gp <- gpar(col = "black", fill = "black")
dotted_gp <- gpar(col = "black", fill = "black", lty = 2)
arm_from <- .05
arm_to <- .75
box_width <- unit(56, "mm")
ex_width <- unit(36, "mm")
ex_gap <- unit(5, "mm")
ex_offset <- box_width / 2 + ex_width / 2 + ex_gap

grid.newpage()
flowchart(
  rando = boxGrob("Randomised\nN = 197", box_gp = main_gp),
  groups = list(
    boxGrob("96 assigned to\ndecompressive craniectomy\nplus best medical treatment\n95 received allocated\nintervention",
            box_gp = main_gp),
    boxGrob("101 assigned to best medical\ntreatment alone\n93 received allocated\nintervention",
            box_gp = main_gp)
  ),
  ex1 = list(
    boxGrob("8 died\n1 withdrew\nconsent", just = "left", box_gp = main_gp),
    boxGrob("18 died\n1 withdrew\nconsent", just = "left", box_gp = main_gp)
  ),
  groups1 = list(
    boxGrob("87 completed day 30\nfollow-up", box_gp = main_gp),
    boxGrob("79 completed day 30\nfollow-up", box_gp = main_gp)
  ),
  ex2 = list(
    boxGrob("8 died", just = "left", box_gp = main_gp),
    boxGrob("9 died\n1 withdrew\nconsent\n2 lost to follow-up",
            just = "left", box_gp = main_gp)
  ),
  groups2 = list(
    boxGrob("79 completed day 180\nfollow-up", box_gp = main_gp),
    boxGrob("68 completed day 180\nfollow-up", box_gp = main_gp)
  ),
  ex3 = list(
    boxGrob("5 died\n2 lost to\nfollow-up", just = "left", box_gp = main_gp),
    boxGrob("3 died\n2 withdrew\nconsent\n6 lost to follow-up",
            just = "left", box_gp = main_gp)
  ),
  groups3 = list(
    boxGrob("95 included in the primary\noutcome analysis\n1 withdrew consent",
            box_gp = main_gp),
    boxGrob("95 included in the primary\noutcome analysis\n2 withdrew consent\n2 lost to follow-up",
            box_gp = main_gp)
  )
) |>
  spread(axis = "y", margin = unit(0.02, "npc")) |>
  equalizeWidths(subelement = stringr::regex("^groups"), width = box_width) |>
  equalizeHeights(subelement = stringr::regex("^groups")) |>
  equalizeWidths(subelement = stringr::regex("^ex"), width = ex_width) |>
  spread(subelement = stringr::regex("^groups"), axis = "x", from = arm_from, to = arm_to,
         type = "center") |>
  move(subelement = "rando",
       x = position("groups", position = "center", type = "x")) |>
  move(subelement = list(c("ex1", 1), c("ex2", 1), c("ex3", 1)),
       x = position(c("groups", 1), position = "center", type = "x") + ex_offset) |>
  move(subelement = list(c("ex1", 2), c("ex2", 2), c("ex3", 2)),
       x = position(c("groups", 2), position = "center", type = "x") + ex_offset) |>
  connect("rando", "groups", type = "N", lty_gp = main_con_gp, arrow_size = 3) |>
  connect("groups", "groups1", type = "vertical", lty_gp = main_con_gp, arrow_size = 3) |>
  connect("groups1", "groups2", type = "vertical", lty_gp = main_con_gp, arrow_size = 3) |>
  connect("groups2", "groups3", type = "vertical", lty_gp = main_con_gp, arrow_size = 3) |>
  connect("groups", "ex1", type = "L", lty_gp = main_con_gp, arrow_size = 3) |>
  connect("groups1", "ex2", type = "L", lty_gp = main_con_gp, arrow_size = 3) |>
  connect("groups2", "ex3", type = "L", lty_gp = main_con_gp, arrow_size = 3) |>
  connect(stringr::regex("^ex"), "groups3", type = "side",
          lty_gp = dotted_gp, arrow_size = 3,
          side = "right",
          end_side = "right",
          side_route = "outside",
          side_offset = ex_gap)

options(old_opts)

## ----flowchart-group-width-and-padding, fig.height = 5.5, fig.width = 8-------
old_opts <- options(boxGrobTxtPadding = unit(2, "mm"))

flowchart(
  rando = glue("Randomised\nN = 100"),
  groups = list(
    glue("Group 1\nn = 50"),
    glue("Group 2\nn = 50")
  ),
  groups2 = list(
    glue("Analysed\nn = 49"),
    glue("Analysed\nn = 48")
  )
) |>
  spread(axis = "y", margin = unit(0.02, "npc")) |>
  spread(subelement = "groups", axis = "x", margin = unit(.05, "npc")) |>
  spread(subelement = "groups2", axis = "x", margin = unit(.05, "npc")) |>
  equalizeWidths(subelement = list("groups", "groups2")) |>
  connect("rando", "groups", type = "N") |>
  connect("groups", "groups2", type = "vertical")

options(old_opts)

## ----axis_preserving_connectors, fig.width = 10, fig.height = 7---------------
input_gp     <- gpar(fill = "#F3F8FF", col = "#3B73C5", lwd = 1.3)
process_gp   <- gpar(fill = "#FFF4C7", col = "#C69214", lwd = 1.3)
issue_gp     <- gpar(fill = "#FCE4EC", col = "#AD1457", lwd = 1.2)
output_gp    <- gpar(fill = "#E8F5E9", col = "#2E7D32", lwd = 1.3)
con_gp       <- gpar(col = "#555555", fill = "#555555", lwd = 1.3)
issue_con_gp <- gpar(col = "#AD1457", fill = "#AD1457", lwd = 1.1)
main_path    <- list("validation", "clean")

grid.newpage()
flowchart(
  inputs = list(
    web    = boxEllipseGrob("REDCap\nform",   width = unit(42, "mm"), height = unit(24, "mm"), box_gp = input_gp),
    import = boxDatabaseGrob("CSV\nimport",    width = unit(42, "mm"), height = unit(24, "mm"), box_gp = input_gp),
    manual = boxDocumentGrob("Manual\nentry",  width = unit(42, "mm"), height = unit(24, "mm"), box_gp = input_gp)
  ),
  validation = boxTapeGrob(
    "Validation queue\nIDs, dates, ranges, missingness",
    width = unit(.58, "npc"), height = unit(.14, "npc"), box_gp = process_gp
  ),
  issues = list(
    missing   = boxDiamondGrob("Missing\nfields", width = unit(48, "mm"), height = unit(14, "mm"), box_gp = issue_gp),
    duplicate = boxDiamondGrob("Duplicate\nID",   width = unit(48, "mm"), height = unit(14, "mm"), box_gp = issue_gp),
    outlier   = boxDiamondGrob("Outlier\nvalue",  width = unit(48, "mm"), height = unit(14, "mm"), box_gp = issue_gp)
  ),
  log = boxDocumentsGrob(
    "Issue log\nqueries sent\nchanges reviewed",
    width = unit(48, "mm"), height = unit(.44, "npc"), box_gp = issue_gp
  ),
  clean = boxDatabaseGrob(
    "Analysis-ready dataset\nlocked for report",
    width = unit(.44, "npc"), height = unit(.16, "npc"), box_gp = output_gp
  )
) |>
  # Main vertical flow (inputs -> validation -> clean); issues are a side column
  spread(axis = "y", margin = unit(7, "mm"), exclude = "issues") |>
  spread(axis = "x", subelement = "inputs", from = 0, to = 0.7, margin = 0.05, type = "center") |>
  equalizeWidths(subelement = main_path) |>
  align(axis = "x", subelement = "validation", reference = "inputs") |>
  align(axis = "x", subelement = "clean", reference = "validation") |>
  # Place the issue log beside the main flow and stack the diamonds along it
  align(axis = "y", subelement = "log", references = list("validation", "clean")) |>
  spread(axis = "y", subelement = "issues",
         from = position("log", position = "top", type = "y"),
         to   = position("log", position = "bottom", type = "y"),
         margin = unit(2, "mm")) |>
  move(subelement = "issues", x = 0.08, just = "left") |>
  move(subelement = "log",    x = 0.92, just = "right") |>
  connect("inputs", "validation", type = "vertical_axis",   lty_gp = con_gp,       arrow_size = 3) |>
  connect("issues", "log",        type = "horizontal_axis", lty_gp = issue_con_gp, arrow_size = 3) |>
  connect("validation", "clean",  type = "vertical_axis",   lty_gp = con_gp,       arrow_size = 3, smooth = TRUE) |>
  print()

## ----basic_box, fig.height = 1.5, fig.width = 3, message = FALSE--------------
grid.newpage()
txt <-
"Just a plain box
with some text
- Note that newline is OK"
boxGrob(txt)

## ----styled_box, fig.height = 3, fig.width = 3--------------------------------
grid.newpage()
boxGrob("A large\noffset\nyellow\nbox",
        width = .8, height = .8,
        x = 0, y = 0,
        bjust = c("left", "bottom"),
        txt_gp = gpar(col = "darkblue", cex = 2),
        box_gp = gpar(fill = "lightyellow", col = "darkblue"))

## ----prop_box, fig.height = 2, fig.width = 4----------------------------------
grid.newpage()
boxPropGrob("A box with proportions",
            "Left side", "Right side",
            prop = .7)

## ----box_coordinates, fig.height = 3, fig.width = 4---------------------------
grid.newpage()
smpl_bx <- boxGrob(
  label = "A simple box",
  x = .5,
  y = .9,
  just = "center")

prop_bx <- boxPropGrob(
  label = "A split box",
  label_left = "Left side",
  label_right = "Right side",
  x = .5,
  y = .3,
  prop = .3,
  just = "center")

plot(smpl_bx)
plot(prop_bx)

smpl_bx_coords <- coords(smpl_bx)
grid.circle(y = smpl_bx_coords$y,
            x = smpl_bx_coords$x,
            r = unit(2, "mm"),
            gp = gpar(fill = "#FFFFFF99", col = "black"))
grid.circle(y = smpl_bx_coords$bottom,
            x = smpl_bx_coords$right,
            r = unit(1, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = smpl_bx_coords$top,
            x = smpl_bx_coords$right,
            r = unit(1, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = smpl_bx_coords$bottom,
            x = smpl_bx_coords$left,
            r = unit(1, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = smpl_bx_coords$top,
            x = smpl_bx_coords$left,
            r = unit(1, "mm"),
            gp = gpar(fill = "orange"))

prop_bx_coords <- coords(prop_bx)
grid.circle(y = prop_bx_coords$y,
            x = prop_bx_coords$x,
            r = unit(2, "mm"),
            gp = gpar(fill = "#FFFFFF99", col = "black"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$right_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$right_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$left_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$left_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "orange"))

grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$right,
            r = unit(2, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$right,
            r = unit(2, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$left,
            r = unit(2, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$left,
            r = unit(2, "mm"),
            gp = gpar(fill = "orange"))

## ----extra_shapes, fig.height = 3, fig.width = 6------------------------------
# --- Branch labels on alternate shapes ---
grid.newpage()

decision <- boxDiamondGrob("Decision", box_gp = gpar(fill = "#FFF4E6"))
local    <- boxEllipseGrob("Local", box_gp = gpar(fill = "#E6FFF4"))
server   <- boxServerGrob("Server", box_gp = gpar(fill = "#E8F0FF"))

# Position the boxes, then draw them. The ellipse carries extra padding so its
# text fits the curved outline, making it taller than the server box; equalize
# the outcome heights so both branches line up symmetrically.
boxes <- list(decision = decision, outcomes = list(local, server)) |>
  equalizeHeights(subelement = "outcomes") |>
  spreadHorizontal(from = unit(.1, "npc"), to = unit(.9, "npc"), subelement = "outcomes") |>
  spreadVertical() |>
  print()

# Build one N connector for both branches and attach a label to each with
# setConnectorLabels(); print() then draws the lines and labels together.
connectGrob(boxes$decision, boxes$outcomes, type = "N") |>
  setConnectorLabels(c("Local", "Server")) |>
  print()

## ----standard_shapes, fig.height = 3.5, fig.width = 8-------------------------
# Arrange shapes in three rows for better readability
# 1) Grid-based objects (basic boxGrob / boxPropGrob / rect)
row1 <- list(
  boxGrob("Box (default)", box_gp = gpar(fill = "#EFEFEF"), y = unit(.85, "npc")),
  boxPropGrob("Prop", "Left", "Right", prop = .4, box_left_gp = gpar(fill = "#EFEFAF"), box_right_gp = gpar(fill = "#EFAFEF"), y = unit(.85, "npc")),
  boxGrob("Rectangle", box_fn = rectGrob, box_gp = gpar(fill = "#EFEFEF"), y = unit(.85, "npc"))
)

# 2) Gmisc row 1 (rounded/sharp diamond + ellipse + rack + server)
row2 <- list(
  boxDiamondGrob("Diamond\n(rounded)", box_gp = gpar(fill = "#FFF4E6"), y = unit(.55, "npc")),
  boxDiamondGrob("Diamond\n(sharp)", rounded = FALSE, box_gp = gpar(fill = "#FFF4E6"), y = unit(.55, "npc")),
  boxEllipseGrob("Ellipse", box_gp = gpar(fill = "#E6FFF4"), y = unit(.55, "npc")),
  boxRackGrob("Rack", box_gp = gpar(fill = "#E8F0FF"), y = unit(.55, "npc")),
  boxServerGrob("Server", box_gp = gpar(fill = "#E8F0FF"), y = unit(.55, "npc"))
)

# 3) Gmisc row 2 (database, document, documents, tape)
row3 <- list(
  boxDatabaseGrob("Database", box_gp = gpar(fill = "#DFF4E6"), y = unit(.25, "npc")),
  boxDocumentGrob("Document", box_gp = gpar(fill = "#FFF6E6"), y = unit(.25, "npc")),
  boxDocumentsGrob("Documents", box_gp = gpar(fill = "#FFF6E6"), y = unit(.25, "npc")),
  boxTapeGrob("Tape", box_gp = gpar(fill = "#E6F0FF"), y = unit(.25, "npc"))
)

# Spread each row across the horizontal span
spreadHorizontal(row1, from = unit(.05, "npc"), to = unit(.95, "npc"))
spreadHorizontal(row2, from = unit(.05, "npc"), to = unit(.95, "npc"))
spreadHorizontal(row3, from = unit(.05, "npc"), to = unit(.95, "npc"))

## ----"Connected boxes", fig.width = 7, fig.height = 5-------------------------
grid.newpage()
# Initiate the boxes that we want to connect
side <- boxPropGrob("Side", "Left", "Right",
                    prop = .3,
                    x = 0, y = .9,
                    bjust = c(0,1))
start <- boxGrob("Top",
                 x = .6, y = coords(side)$y,
                 box_gp = gpar(fill = "yellow"))
bottom <- boxGrob("Bottom",
                  x = .6, y = 0,
                  bjust = "bottom")


sub_side_left <- boxGrob("Left",
                         x = coords(side)$left_x,
                         y = 0,
                         bjust = "bottom")
sub_side_right <- boxGrob("Right",
                          x = coords(side)$right_x,
                          y = 0,
                          bjust = "bottom")

odd <- boxGrob("Odd\nbox",
               x = coords(side)$right,
               y = .5)

odd2 <- boxGrob("Also odd",
               x = coords(odd)$right +
                 distance(bottom, odd, type = "h", half = TRUE) -
                 unit(2, "mm"),
               y = 0,
               bjust = c(1,0))

exclude <- boxGrob("Exclude:\n - Too sick\n - Prev. surgery",
                   x = 1,
                   y = coords(bottom)$top +
                     distance(start, bottom, type = "v", half = TRUE),
                   just = "left", bjust = "right")

# Connect the boxes and print/plot them
connectGrob(start, bottom, "vertical")
connectGrob(start, side, "horizontal")
connectGrob(bottom, odd, "Z", "l")
connectGrob(odd, odd2, "N", "l")
connectGrob(side, sub_side_left, "v", "l")
connectGrob(side, sub_side_right, "v", "r")
connectGrob(start, exclude, "-",
            lty_gp = gpar(lwd = 2, col = "darkred", fill = "darkred"))

# Print the grobs
start
bottom
side
exclude
sub_side_left
sub_side_right
odd
odd2

## ----connect_multi, fig.width = 4, fig.height = 4-----------------------------
grid.newpage()

# Three upstream boxes + one side box
a_boxes <- paste("A", 1:3) |>
  lapply(\(x) boxGrob(x, box_gp = gpar(fill = "#E6F2FF"))) |>
  spreadHorizontal(from = unit(.1, "npc"), to = unit(1, "npc") - unit(1, "cm")) |>
  alignVertical(position="top",
                reference = unit(1, "npc")) |>
  print()

b_side <- boxGrob("B",  y = .70, box_gp = gpar(fill = "#FFF3BF")) |>
  moveBox(x = unit(1, "npc"),
          just = 1) |>
  print()

# Target box
c <- boxGrob("C", x = .50, box_gp = gpar(fill = "#D3F9D8"), width = unit(4, "cm")) |>
  moveBox(y = unit(0, "npc"),
          just = "bottom") |>
  print()


# Many -> one: merge on top with evenly distributed attachment points + margin
connectGrob(c(a_boxes, list(b_side)), c,
            type = "fan_in_top",
            margin = 4)

## ----horizontal_alignment, fig.width=10, fig.height=6-------------------------
align_1 <- boxGrob("Align 1",
                   y = .9,
                   x = 0,
                   bjust = c(0),
                   box_gp = gpar(fill = "#E6E8EF"))

align_2 <- boxPropGrob("Align 2",
                       "Placebo",
                       "Treatment",
                       prop = .7,
                       y = .8,
                       x = .5)

align_3 <- boxGrob("Align 3\nvertical\ntext",
                   y = 1,
                   x = 1,
                   bjust = c(1, 1),
                   box_gp = gpar(fill = "#E6E8EF"))

b1 <- boxGrob("B1",
              y = .3,
              x = .1,
              bjust = c(0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .6,
              x = .5)
b3 <- boxGrob("B3",
              y = .2,
              x = .8,
              bjust = c(0, 1))

grid.newpage()
align_1
alignHorizontal(reference = align_1,
                b1, b2, b3,
                position = "left")

align_2
alignHorizontal(reference = align_2,
                b1, b2, b3,
                position = "center",
                sub_position = "left")
alignHorizontal(reference = align_2,
                b1, b2, b3,
                position = "left",
                sub_position = "right")

align_3
alignHorizontal(reference = align_3,
                b1, b2, b3,
                position = "right")

## ----vertical_alignment, fig.width=10, fig.height=6---------------------------
align_1 <- boxGrob("Align 1\nvertical\ntext",
                   y = 1,
                   x = 1,
                   bjust = c(1, 1),
                   box_gp = gpar(fill = "#E6E8EF"))

align_2 <- boxPropGrob("Align 2",
                       "Placebo",
                       "Treatment",
                       prop = .7,
                       y = .5,
                       x = .6)

align_3 <- boxGrob("Align 3",
                   y = 0,
                   x = 0,
                   bjust = c(0, 0),
                   box_gp = gpar(fill = "#E6E8EF"))


b1 <- boxGrob("B1",
              y = .3,
              x = 0.1,
              bjust = c(0, 0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .6,
              x = .3)
b3 <- boxGrob("B3",
              y = .2,
              x = .85,
              bjust = c(0, 1))

grid.newpage()
align_1
alignVertical(reference = align_1,
              b1, b2, b3,
              position = "top")

align_2
alignVertical(reference = align_2,
              b1, b2, b3,
              position = "center")

align_3
alignVertical(reference = align_3,
              b1, b2, b3,
              position = "bottom")

## ----horizontal_spread, fig.width = 11, fig.height = 8------------------------
b1 <- boxGrob("B1", y = .85, x = .1, bjust = c(0, 0))
b2 <- boxGrob("B2", y = .65, x = .6)
b3 <- boxGrob("B3", y = .45, x = .6)
b4 <- boxGrob("B4 with long\ndescription", y = .7, x = .8)

from <- boxGrob("from",
                y = .25,
                x = .05,
                box_gp = gpar(fill = "darkgreen"),
                txt_gp = gpar(col = "white"))
to <- boxGrob("to this wide box",
              y = coords(from)$y,
              x = .95,
              bjust = "right",
              box_gp = gpar(fill = "darkred"),
              txt_gp = gpar(col = "white"))
txtOut <- function(txt, y_top) {
  grid.text(txt,
            x = unit(2, "mm"),
            y = y_top + unit(2, "mm"),
            just = c("left", "bottom"))
  grid.lines(y = y_top + unit(1, "mm"),
             gp = gpar(col = "grey"))
}

drawRow <- function(label, row_y, spread_args = list()) {
  row <- alignVertical(reference = row_y, b1, b2, b3, b4, position = "top")
  txtOut(label, coords(row[[1]])$top)
  do.call(spreadHorizontal, c(list(row), spread_args))
}

rowYs <- unit(c(.93, .76, .59, .42, .25, .12), "npc")

grid.newpage()

drawRow("Basic (viewport)", rowYs[1])
drawRow("From–to + margin (numeric = npc)", rowYs[2],
        spread_args = list(from = .2, to = .7, margin = .05))
drawRow("Only to (defaults from = 0)", rowYs[3],
        spread_args = list(to = .7))
drawRow("Only from (defaults to = 1)", rowYs[4],
        spread_args = list(from = .2))

# Row 5: Between boxes (box-to-box span)
row5_y <- rowYs[5]
row5 <- alignVertical(reference = row5_y, b1, b2, b3, b4, position = "top")
txtOut("Between boxes", coords(row5[[1]])$top)

span <- alignVertical(reference = row5_y, from  = from, to = to, position = "top")
span
spreadHorizontal(row5, from = span$from, to = span$to)

# Row 6: Reverse box order + center distribution
row6_y <- unit(.10, "npc")

bottom_from <- moveBox(from, x = coords(to)$right, y = 0, just = c(1, 0))
bottom_to <- moveBox(to, x = coords(from)$left, y = 0, just = c(0, 0))
bottom_from
bottom_to

row6 <- alignVertical(reference = bottom_from, b1, b2, b3, b4, position = "bottom")
txtOut("Reverse box order + center", coords(row6[[4]])$top)

spreadHorizontal(row6,
                 from = bottom_from,
                 to = bottom_to,
                 type = "center")



## ----vertical_spread, fig.width=6, fig.height=6-------------------------------
b1 <- boxGrob("B1",
              y = .8,
              x = 0.1,
              bjust = c(0, 0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .5,
              x = .5)
b3 <- boxGrob("B3",
              y = .2,
              x = .8)
b4 <- boxGrob("B4",
              y = .7,
              x = .8)


txtOut <- function(txt, refBx) {
  grid.text(txt,
            x = coords(refBx)$left - unit(2, "mm"),
            y = .5,
            just = c("center", "bottom"),
            rot = 90)
  grid.lines(x = coords(refBx)$left - unit(1, "mm"),
             gp = gpar(col = "grey"))
}

grid.newpage()
txtOut("Basic", b1)
alignHorizontal(reference = b1,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical()

txtOut("From-to", b2)
alignHorizontal(reference = b2,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical(from = .2,
                 to = .7)

txtOut("From-to with center and reverse the box order", b3)
alignHorizontal(reference = b3,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical(from = .7,
                 to = .2,
                 type = "center")

## ----complex_nested, fig.width = 7, fig.height = 8----------------------------
# Helper function to convert nested structure to grobs
make_boxes <- function(x) {
  if (is.list(x) && !inherits(x, "box_header")) {
    return(lapply(x, make_boxes))
  }

  if (inherits(x, "box_header")) {
    return(do.call(boxHeaderGrob, x))
  }

  # Simple text box fallback
  args <- attr(x, "args")
  if (is.null(args)) return(boxGrob(label = x))

  args$label <- x
  do.call(boxGrob, args)
}

# Define styling for different elements
arm_a_style <- list(
  header = gpar(fill = "#E8F5E9", col = "#2E7D32", lwd = 1.4),
  box = gpar(fill = "#F1F8E9", col = "#43A047")
)

arm_b_style <- list(
  header = gpar(fill = "#FFF8E1", col = "#EF6C00", lwd = 1.4),
  box = gpar(fill = "#FFFDE7", col = "#F9A825")
)

# Build flowchart structure
flowchart <- list(
  # Shared inclusion criteria
  criteria = structure(
    list(
      header = "Inclusion Criteria",
      body = paste(
        "• Adults aged 18-65",
        "• Confirmed diagnosis",
        "• Written informed consent",
        "• No contraindications",
        "• Available for 6-month follow-up",
        sep = "\n"
      ),
      box_gp = gpar(fill = "#E3F2FD", col = "#1E88E5", lwd = 1.4),
      body_gp = gpar(fontsize = 10)
    ),
    class = "box_header"
  ),

  # Two treatment arms
  arms = list(
    arm_a = list(
      # Arm header
      structure("Intensive Protocol", args = list(
        box_gp = arm_a_style$header,
        txt_gp = gpar(fontsize = 11, fontface = "bold")
      )),

      # Timeline boxes
      structure(list(
        header = "Week 0-1",
        body = "• Daily sessions\n• Supervised therapy\n",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Week 2-4",
        body = "• 3× weekly sessions\n• Progressive loading",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Week 5-8",
        body = "• Home program\n• Monthly check-ins\n• Return to activity",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header")
    ),

    arm_b = list(
      # Arm header
      structure("Standard Care", args = list(
        box_gp = arm_b_style$header,
        txt_gp = gpar(fontsize = 11, fontface = "bold")
      )),

      # Timeline boxes - different schedule
      structure(list(
        header = "Month 0",
        body = "• Initial consultation\n• Exercise booklet",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Month 3",
        body = "• Follow-up visit\n• Progress review",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Month 6",
        body = "• Final assessment\n• Discharge planning",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header")
    )
  )
)

# Convert to grobs and layout
grid.newpage()
boxes <- flowchart |>
  make_boxes() |>
  spreadVertical() |>
  spreadHorizontal(subelement = "arms", from = 0.15, to = 0.85) |>
  spreadVertical(subelement = c("arms", "arm_a"), from = 0.65) |>
  spreadVertical(subelement = c("arms", "arm_b"), from = 0.65) |>
  print()

# Connect criteria to both arms
connectGrob(boxes$criteria, boxes$arms, type = "N")

# Connect timeline within each arm
for (arm_name in names(boxes$arms)) {
  arm_boxes <- boxes$arms[[arm_name]]
  for (i in 2:length(arm_boxes)) {
    connectGrob(arm_boxes[[i-1]], arm_boxes[[i]], type = "v") |> print()
  }
}

## ----s3_api_example, fig.height=8, fig.width=6, eval=FALSE--------------------
# grid.newpage()
# 
# # Define the nodes
# b1 <- boxGrob("Start", y = 0.8)
# b2 <- boxGrob("Process", y = 0.5)
# b3 <- boxGrob("End", y = 0.2)
# 
# # Pipeline: list -> align -> connect -> print
# list(start = b1, process = b2, end = b3) |>
#   align(axis = "y") |>
#   spread(axis = "x") |>
#   connect("start", "process", type = "horizontal") |>
#   connect("process", "end", type = "horizontal") |>
#   print()

## ----math_expressions, fig.width=6, fig.height=3------------------------------
grid.newpage()
###############
# Expressions #
###############
# Font style
list(expression(bold("Bold text")),
     expression(italic("Italics text")),
     expression(paste("Mixed: ", italic("Italics"), " and ", bold("bold")))) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(1, "npc"),
                position = "top") |>
  spreadHorizontal()

# Math
list(expression(paste("y = ", beta[0], " + ", beta[1], X[1], " + ", beta[2], X[2]^2)),
     expression(paste(hat(mu) == sum(frac(x[i], n), i == 1, n))),
     expression(paste(int(a, b, f(x) * dx) == F(b) - F(a)))) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(0.5, "npc"),
                position = "center") |>
  spreadHorizontal()

##########
# Quotes #
##########
a = 5
list(bquote(alpha == theta[1] * .(a) + ldots),
     paste("argument", sQuote("x"), "\nmust be non-zero")) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(0, "npc"),
                position = "bottom") |>
  spreadHorizontal(from = .2, to = .8)

## ----basic_plot, fig.height = 2, fig.width = 2--------------------------------
# Load the grid library
# part of standard R libraries so no need installing
library(grid)

# Create a new graph
grid.newpage()

pushViewport(viewport(width = .5, height = .8))

grid.rect(gp = gpar(fill = "#D8F0D1"))

popViewport()

## ----relative_lines, fig.height = 3, fig.width = 3----------------------------
grid.newpage()
pushViewport(viewport(width = .5, height = .8, clip = "on"))
grid.rect(gp = gpar(lty = 2, fill = "lightyellow"))
lg <- linesGrob(x = unit(c(.2, 1), "npc"),
                y = unit(c(.2, 1), "npc"),
                gp = gpar(lwd = 2))
grid.draw(lg)
pushViewport(viewport(x = 0, y = .6, just = "left", width = .4, height = .4, angle = 20))
grid.rect(gp = gpar(fill = "lightblue")) # A translucent box to indicate the new viewport
grid.draw(lg)
popViewport()

## ----absolute_lines, fig.height = 3, fig.width = 3----------------------------
grid.newpage()
pushViewport(viewport(width = .5, height = .8, clip = "on"))
grid.rect(gp = gpar(lty = 2, fill = "lightyellow"))
lg <- linesGrob(x = unit(c(2, 10), "mm"),
                y = unit(c(2, 10), "mm"),
                gp = gpar(lwd = 2))
grid.draw(lg)
pushViewport(viewport(x = 0, y = .6, just = "left", width = .4, height = .4, angle = 20))
grid.rect(gp = gpar(fill = "lightblue")) # A translucent box to indicate the new viewport
grid.draw(lg)
popViewport()

## ----complex_example, fig.height = 9, fig.width = 9---------------------------
# Define the boxes
org_cohort <- glue("Proximal humerus fracture",
                   "  - \u2265 18 years",
                   "  - \u2264 4 weeks of trauma",
                   "  - Not pathological",
                   .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#E3F2FD"))

surgery <- glue("Surgery",
                "  - Direct (\u2248 4%)",
                "  - Delayed (\u2248 4%)",
                .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#F8BBD0"))

randomize <- boxGrob("Non-surgical\nRandomise",
                     box_gp = gpar(fill = "#FFF3E0"))

treatments <- list(early = boxGrob("Early rehab",
                                   box_gp = gpar(fill = "#DCEDC8")),
                   late = boxGrob("Late rehab",
                                  box_gp = gpar(fill = "#DCEDC8")),
                   obs = boxGrob("Observation",
                                 box_gp = gpar(fill = "#E0E0E0")))

early_followup <- glue("Early follow-up",
                       "  - 2 weeks [PNRS]",
                       "  - 4 weeks [PNRS]",
                       .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#E0F7FA"))

late_followup <- glue("Late follow-up",
                      "  - 2-10 months (random) [OSS, PNRS]",
                      "  - 1 year [OSS, PNRS, accelerometer]",
                      "  - 2 years [OSS, PNRS]",
                      "  - 5 years [OSS, PNRS]",
                      .sep = "\n") |>
  boxGrob(just = "left",
                       box_gp = gpar(fill = "#E0F7FA"))

# Create the flowchart
grid.newpage()
flowchart(start = org_cohort,
          step_1 = list(surgery = surgery,
                        `non-surgical` = randomize),
          treatment = treatments,
          early_followup = early_followup,
          followup = late_followup) |>
  spread(axis = "y") |>
  spread(axis = "x", subelement = "step_1") |>
  spread(axis = "x", subelement = "treatment", from = 0.35) |>
  align(axis = "x",
        reference = c("treatment", "late"),
        subelement = c("step_1", "non-surgical")) |>
  connect(from = "start", to = "step_1", type = "N") |>
  connect(from = "step_1$non-surgical", to = "treatment", type = "N") |>
  connect(from = "treatment", to = "early_followup", type = "fan_in_center") |>
  connect(from = "early_followup", to = "followup", type = "v") |>
  connect(from = "early_followup", to = "step_1$surgery", type = "Z",
          label = "Crossover\nto surgery") |>
  connect(from = "step_1$surgery", to = "followup", type = "L") |>
  print()

library(grid)
grid.newpage()

# Create a reference box
box <- boxGrob("A cool reference box",
  x = .5, y = .8,
  box_gp = gpar(fill = "#ADB5C7")
)

# Create a group of boxes to align
boxes <- flowchart(
  another_box = boxGrob("A horizontal box", x = .1, y = .5),
  yet_another_box = boxGrob("Another horizontal box", x = .8, y = .3)
)

# Align the group in pipe style
aligned_boxes <- boxes |>
  align(axis = "x", reference = box, position = "right") |>
  align(axis = "y", reference = .5, position = "center")

# Example: align a nested element inside a complex list using a deep path
complex_list <- flowchart(
  arms = list(
    early = list(boxGrob("Early", x = .2, y = .4)),
    late = list(boxGrob("Late", x = .8, y = .2))
  ),
  detail = list(
    list(boxGrob("D_early", x = .1, y = .6)),
    list(boxGrob("D_late", x = .9, y = .1))
  )
)

# Align the first detail element to the early arm by deep path
complex_list <- complex_list |>
  align(
    axis = "x",
    reference = c("arms", "early"),
    position = "center",
    subelement = c("detail", 1)
  )

# Center a side branch between the boxes before and after it
side_branch <- flowchart(
  assessed = boxGrob("Assessed", y = .9),
  randomised = boxGrob("Randomised", y = .5)
) |>
  insert(list(excluded = boxGrob("Excluded")), after = "assessed") |>
  move(subelement = "excluded", x = .85) |>
  align(
    axis = "y",
    subelement = "excluded",
    references = list("assessed", "randomised")
  )

# Print the reference and the aligned boxes
box
aligned_boxes
complex_list
side_branch

#' Move a boxGrob
#'
#' Moves a [`boxGrob`]/[`boxPropGrob`] by modifying it's [`viewport`][grid::viewport].
#' This can be useful if you want to create a series of boxes whose position are relative
#' to each other and depend on each box's width/height.
#'
#' @param element A [`boxGrob`]/[`boxPropGrob`] object.
#' @param x A [`unit`][grid::unit] element or a numeric that can be converted to a `npc` unit object.
#' @param y A [`unit`][grid::unit] element or a numeric that can be converted to a `npc` unit object.
#' @param space We can provide `absolute` that confers the box absolute position within the parent
#'  [`viewport`][grid::viewport]. If `relative` the movement is related to the current position.
#' @param just The justification of an argument as used by [`viewport`][grid::viewport] some tiny differences:
#'  (1) you only want to change the justification in the vertical direction you can retain the
#'  existing justification by using `NA`, e.g. `c(NA, 'top')`, (2) if you specify only one string
#'  and that string is either `top` or `bottom` it will assume vertical justification.
#' @param subelement If a `list` of boxes is provided, this can be a name, index, a deep path (e.g. `c("detail", 1)`) or a
#'  vector of names/indices to target a single nested element to move. You can also pass
#'  multiple targets as a list of paths (e.g. `list(c("detail", 1), c("followup", 1))`).
#'  The function will return the original list with the targeted element(s) replaced by their moved version(s).
#' @return The element with updated viewport and coordinates
#'
#' @md
#' @importFrom checkmate assert_class assert checkString checkNumeric checkCharacter
#' @export
#' @example inst/examples/moveBox_ex.R
#' @importFrom checkmate assert_class assert_list
#' @family flowchart components
moveBox <- function(element,
                    x = NULL, y = NULL,
                    space = c("absolute", "relative"),
                    just = NULL,
                    subelement = NULL) {
  space <- match.arg(space)

  to_unit <- function(u) if (is.unit(u) || is.null(u)) u else unit(u, "npc")

  if (is.list(element) && !inherits(element, "box")) {
    if (is.null(x) && is.null(y)) {
      stop("You have to specify at least x or y move parameters")
    }

    if (!is.null(subelement)) {
      has_nested_first_container <- function(x) {
        is.list(x) && length(x) > 1 && is.list(x[[1]]) && !inherits(x[[1]], "box")
      }

      # Normalize into list of paths
      paths <- if (is.list(subelement) && all(sapply(subelement, is.atomic))) subelement else list(subelement)

      for (path in paths) {
        norm_seg <- function(s) {
          if (is.numeric(s) || (is.character(s) && grepl("^[0-9]+$", s))) {
            return(as.integer(s))
          }
          s
        }

        exists_nested <- function(el, path) {
          cur <- el
          for (p in path) {
            pp <- norm_seg(p)
            if (is.null(cur[[pp]])) {
              return(FALSE)
            }
            cur <- cur[[pp]]
          }
          TRUE
        }

        get_nested <- function(el, path) {
          cur <- el
          for (p in path) {
            cur <- cur[[norm_seg(p)]]
          }
          cur
        }

        set_nested <- function(el, path, value) {
          if (length(path) == 1) {
            el[[norm_seg(path[[1]])]] <- value
            return(el)
          }
          idx <- norm_seg(path[[1]])
          el[[idx]] <- set_nested(el[[idx]], path[-1], value)
          el
        }

        if (!exists_nested(element, path)) {
          if (has_nested_first_container(element) && exists_nested(element[[1]], path)) {
            target <- get_nested(element[[1]], path)
            moved_target <- moveBox(target, x = x, y = y, space = space, just = just)
            element[[1]] <- set_nested(element[[1]], path, moved_target)
            next
          }

          stop("The subelement '", paste(path, collapse = "/"), "' was not found in the provided boxes.",
            call. = FALSE
          )
        }

        target <- get_nested(element, path)
        moved_target <- moveBox(target, x = x, y = y, space = space, just = just)
        element <- set_nested(element, path, moved_target)
      }

      return(prExtendClass(element, "Gmisc_list_of_boxes"))
    }

    if (space == "absolute") {
      # For absolute, we need to know where the group is currently to calculate relative shift
      grp_coords <- prConvert2Coords(element)

      target_x <- to_unit(x)
      target_y <- to_unit(y)

      if (!is.null(just)) {
        if (!is.null(x)) target_x <- prAdjustPos(just, target_x, grp_coords$width, axis = "x")
        if (!is.null(y)) target_y <- prAdjustPos(just, target_y, grp_coords$height, axis = "y")
      }

      shift_x <- if (!is.null(target_x)) target_x - grp_coords$x else unit(0, "npc")
      shift_y <- if (!is.null(target_y)) target_y - grp_coords$y else unit(0, "npc")

      ret <- sapply(element, \(el) moveBox(el, x = shift_x, y = shift_y, space = "relative"), simplify = FALSE)
    } else {
      ret <- sapply(element, \(el) moveBox(el, x = x, y = y, space = "relative"), simplify = FALSE)
    }
    return(prExtendClass(ret, "Gmisc_list_of_boxes"))
  }

  assert_class(element, "box")
  if (is.null(x) && is.null(y)) {
    stop("You have to specify at least x or y move parameters")
  }

  vp_args <- attr(element, "viewport_data")
  assert_list(vp_args)

  if (!is.null(just)) {
    assert(
      checkCharacter(just, min.len = 1, max.len = 2),
      checkNumeric(just, lower = 0, upper = 1, min.len = 1, max.len = 2)
    )
    if (is.null(vp_args$just)) {
      vp_args$just <- just
    } else {
      if (length(just) == 1) {
        if (just %in% c("bottom", "top")) {
          vp_args$just[2] <- just
        } else {
          vp_args$just[1] <- just
        }
      } else {
        for (i in 1:2) {
          if (!is.na(just[i])) {
            vp_args$just[i] <- just[i]
          }
        }
      }
      if (!all(vp_args$just %in% c("center", "centre", "left", "right", "bottom", "top"))) {
        vp_args$just <- sapply(
          vp_args$just,
          function(x) {
            if (x %in% c("center", "centre")) {
              return(0.5)
            }
            if (x %in% c("left", "bottom")) {
              return(0)
            }
            if (x %in% c("right", "top")) {
              return(1)
            }
            if (!is.na(as.numeric(x))) {
              return(as.numeric(x))
            }
            stop("The justification ", x, " has not been implemented")
          }
        )
      }
    }
  }

  # use internal helper `to_unit()` defined above to convert numeric/nil to npc units

  x <- to_unit(x)
  y <- to_unit(y)

  if (space == "relative") {
    if (!is.null(x)) {
      x <- vp_args$x + x
    }
    if (!is.null(y)) {
      y <- vp_args$y + y
    }
  }

  if (!is.null(x)) {
    vp_args$x <- x
  }

  if (!is.null(y)) {
    vp_args$y <- y
  }

  gl <- editGrob(element, vp = do.call(viewport, vp_args))
  attr(gl, "viewport_data") <- vp_args
  attr(gl, "coords") <- prCreateBoxCoordinates(
    viewport_data = vp_args,
    extra_coordinate_functions = attr(element, "extra_coordinate_functions")
  )
  return(gl)
}

testthat::test_that("spread and move accept deep subelement paths", {
    library(grid)
    library(checkmate)
    make_stub_box <- function(label, x = 0.5, y = 0.5) {
        b <- list(label = label)
        class(b) <- "box"
        vp <- list(x = unit(x, "npc"), y = unit(y, "npc"), just = c("center", "center"))
        coords <- list(
            left = unit(x, "npc"), right = unit(x, "npc"),
            top = unit(y, "npc"), bottom = unit(y, "npc"),
            x = unit(x, "npc"), y = unit(y, "npc"),
            width = unit(0, "npc"), height = unit(0, "npc"),
            half_width = unit(0, "npc"), half_height = unit(0, "npc")
        )
        attr(b, "viewport_data") <- vp
        attr(b, "coords") <- structure(coords, class = c("box_coords", "list"))
        b
    }

    # create a nested structure of simple placeholders (we don't need real boxGrob properties)
    nested <- list(
        grp = list(
            sub = list(
                make_stub_box("A", x = 0.1, y = 0.9),
                make_stub_box("B", x = 0.1, y = 0.7)
            )
        ),
        other = list(make_stub_box("C", x = 0.8, y = 0.5))
    )

    # If a path does not exist we get informative error
    expect_error(moveBox(nested, x = 0.5, subelement = c("no", "such")), "The subelement 'no/such' was not found", fixed = TRUE)

    # Multiple paths: if the first path is missing we get an informative error
    expect_error(moveBox(nested, x = 0.5, subelement = list(c("missing", "x"), c("grp", "sub", 1))), "The subelement 'missing/x' was not found", fixed = TRUE)

    # Spread: attempting to target missing path raises same informative error
    expect_error(spreadVertical(nested, subelement = c("no", "pe")), "The subelement 'no -> pe' was not found", fixed = TRUE)
})

testthat::test_that("moveBox preserves sibling branches when subelement is inside first nested container", {
    library(grid)

    wrapped <- list(
        list(
            groups = list(
                boxGrob("A", x = 0.2, y = 0.8),
                boxGrob("B", x = 0.4, y = 0.8)
            ),
            groups2 = list(
                boxGrob("C", x = 0.2, y = 0.6),
                boxGrob("D", x = 0.4, y = 0.6)
            )
        ),
        excluded = list(boxGrob("Excluded", x = 0.85, y = 0.7))
    )

    moved <- moveBox(wrapped, x = 0.6, subelement = "groups2")

    expect_equal(length(moved), 2)
    expect_true(!is.null(moved$excluded))
    expect_true(!is.null(moved[[1]]$groups2))
})

library(testthat)

test_that("Box move relative", {
  box1 <- boxGrob("A simple box", x = .5, y = .8)
  box2 <- moveBox(box1, x = -.2, y = -.1, space = "relative")
  cvt <- function(x, fn = convertX) convertX(x, unitTo = "mm", valueOnly = TRUE)
  expect_gt(cvt(attr(box1, "viewport_data")$x), cvt(attr(box2, "viewport_data")$x))
  expect_gt(cvt(attr(box1, "viewport_data")$y), cvt(attr(box2, "viewport_data")$y))

  box3 <- moveBox(box1, y = 0.1, space = "relative")
  expect_equal(cvt(attr(box1, "viewport_data")$x), cvt(attr(box3, "viewport_data")$x))
  expect_lt(cvt(attr(box1, "viewport_data")$y), cvt(attr(box3, "viewport_data")$y))
})

test_that("Box move absolute", {
  box1 <- boxGrob("A simple box", x = .5, y = .8)
  box2 <- moveBox(box1, x = .2, y = .1, space = "abs")
  cvt <- function(x, fn = convertX) convertX(x, unitTo = "mm", valueOnly = TRUE)
  expect_gt(cvt(attr(box1, "viewport_data")$x), cvt(attr(box2, "viewport_data")$x))
  expect_gt(cvt(attr(box1, "viewport_data")$y), cvt(attr(box2, "viewport_data")$y))

  box3 <- moveBox(box1, x = .3, space = "abs")
  expect_equal(cvt(attr(box1, "viewport_data")$y), cvt(attr(box3, "viewport_data")$y))
  expect_gt(cvt(attr(box1, "viewport_data")$x), cvt(attr(box3, "viewport_data")$x))
})


test_that("subelement moves a single element in a list (shallow)", {
  b1 <- boxGrob("b1", x = .1, y = .1)
  b2 <- boxGrob("b2", x = .2, y = .2)
  L <- list(b1 = b1, b2 = b2)
  L2 <- moveBox(L, x = .1, subelement = "b1", space = "relative")
  expect_s3_class(L2, "Gmisc_list_of_boxes")
  expect_equal(names(L2), names(L))
  expect_false(identical(attr(L2$b1, "viewport_data")$x, attr(L$b1, "viewport_data")$x))
  expect_equal(attr(L2$b2, "viewport_data")$x, attr(L$b2, "viewport_data")$x)
})


test_that("subelement supports nested path", {
  b <- boxGrob("b", x = .5, y = .5)
  L <- list(group = list(sub = b))
  L2 <- moveBox(L, y = -0.2, subelement = c("group", "sub"), space = "relative")
  expect_false(identical(attr(L2$group$sub, "viewport_data")$y, attr(L$group$sub, "viewport_data")$y))
})


test_that("subelement supports numeric index", {
  b1 <- boxGrob("b1", x = .1, y = .1)
  b2 <- boxGrob("b2", x = .2, y = .2)
  L <- list(b1, b2)
  L2 <- moveBox(L, x = .1, subelement = 1, space = "relative")
  expect_false(identical(attr(L2[[1]], "viewport_data")$x, attr(L[[1]], "viewport_data")$x))
  expect_equal(attr(L2[[2]], "viewport_data")$x, attr(L[[2]], "viewport_data")$x)
})

par_org <- par(ask = TRUE)
library(grid)
plot.new()
gl <- bezierArrowGradient(
  x = c(.2, .5, .5, .8),
  y = c(.2, .2, .8, .8),
  width = unit(10, "mm"),
  arrow = list(
    base = unit(.1, "npc"),
    length = unit(.1, "npc")
  ),
  grdt_type = "triangle",
  grdt_prop = .5,
  grdt_decrease_prop = .5,
  grdt_clr_prop = .7,
  grdt_line_width = unit(.7, "cm"),
  grdt_clr = "#2F4F2F"
)
grid.draw(gl)

plot.new()
pg <- bezierArrowGradient(
  x = c(.3, .7, -.1, .1),
  y = c(.5, .1, -1, .6),
  width = unit(10, "pt"),
  grdt_type = "triangle",
  grdt_prop = .9,
  grdt_decrease_prop = .3,
  grdt_clr_prop = .7,
  grdt_line_width = unit(6, "pt"),
  grdt_clr = "#2F4F2F",
  align_2_axis = FALSE
)
grid.draw(pg)

par(par_org)

library(grid)
grid.newpage()

# Create a reference box
box <- boxGrob("A cool reference box",
  x = .5, y = .8,
  box_gp = gpar(fill = "#ADB5C7")
)

# Create a group of boxes to align
boxes <- list(
  another_box = boxGrob("A horizontal box", x = .1, y = .5),
  yet_another_box = boxGrob("Another horizontal box", x = .8, y = .3)
)

# Align the group and then individual boxes within that group
# (do not pipe the list into the function, as the first argument is `reference`)
aligned_boxes <- alignHorizontal(boxes, reference = box, position = "right") |>
  alignVertical(reference = .5, position = "center")

# Example: align a nested element inside a complex list using a deep path
complex_list <- list(
  arms = list(
    early = list(boxGrob("Early", x = .2, y = .4)),
    late = list(boxGrob("Late", x = .8, y = .2))
  ),
  detail = list(
    list(boxGrob("D_early", x = .1, y = .6)),
    list(boxGrob("D_late", x = .9, y = .1))
  )
)

# Align the first detail element to the early arm by deep path
complex_list <- complex_list |>
  alignHorizontal(
    reference = c("arms", "early"),
    position = "center",
    subelement = c("detail", 1)
  )

# Print the reference and the aligned boxes
box
aligned_boxes
complex_list

#' Merging of multiple lists
#'
#' The merge allows for a recursive component
#' where the lists are compared on the subelement.
#' If one does not contain that element it will get NA
#' in for those parameters.
#'
#' @param ... Any number of lists that you want to merge
#' @param lapplyOutput The \code{\link[base]{lapply}} function outputs a number
#'  of lists and this is for specifically merging all of those.
#' @param sortNames Set to false if you don't want the names to be sorted.
#'  This can also be done via the option `Gmisc.mergeList.sort`.
#' @return Returns a list with all the given lists.
#'
#' @example  inst/examples/mergeLists_example.R
#' @export
mergeLists <- function(..., lapplyOutput = NULL, sortNames = getOption("Gmisc.mergeList.sort", default = TRUE)) {
  lst <- list(...)
  if (is.list(lapplyOutput)) {
    lst <- c(lst, lapplyOutput)
  }

  if (length(lst) == 1) {
    warning(
      "You have provided the function with only one list,",
      " are you sure you didn't intend to use the lapplyOutput argument?"
    )
    return(lst)
  } else if (length(lst) == 0) {
    stop("No lists to merge")
  }

  m <- lst[[1]]
  if (is.null(names(m))) {
    for (i in 2:length(lst)) {
      nextList <- lst[[i]]
      if (!is.null(names(nextList))) {
        stop("The list ", nextList, " has names - cannot merge list with no-name elements with named elements.",
             " Handling the merge becomes unpredictable and unsafe. You should consider writing your own routine for this.")
      }
      m <- mapply(mergeLists_internal, m, nextList, SIMPLIFY = FALSE)
    }
  } else {
    for (i in 2:length(lst)) {
      nextList <- lst[[i]]
      if (is.null(names(nextList))) {
        stop("The list ", nextList, " does not have names - cannot merge list with named elements with no-named elements.",
             " Handling the merge becomes unpredictable and unsafe. You should consider writing your own routine for this.")
      }
      elementNames <- union(names(m), names(nextList))
      if (sortNames) {
        elementNames <- sort(elementNames)
      }
      for (n in elementNames) {
        if (is.null(m[[n]])) {
          m[[n]] <- nextList[[n]]
        } else if (!is.null(m[[n]])) {
          m[[n]] <- mergeLists_internal(o_element = m[[n]], n_element = nextList[[n]])
        }
      }
    }
  }
  return(m)
}

#' A helper function for mergeLists that does the actual work.
#'
#' @param o_element Old element
#' @param n_element New element to add to the old element
#' @return list with the two merged
#'
#' @keywords internal
mergeLists_internal <- function(o_element, n_element) {
  if (is.list(o_element) != is.list(n_element)) {
    stop("Can't mix list and non-lists check out the sections containing ", o_element, " and ", n_element)
  }

  if (is.list(n_element)) {
    # Fill in non-existant element with NA elements
    if (length(n_element) != length(o_element)) {
      n_unique <- names(n_element)[!names(n_element) %in% names(o_element)]
      if (length(n_unique) > 0) {
        for (n in n_unique) {
          if (is.matrix(n_element[[n]])) {
            o_element[[n]] <- matrix(NA,
              nrow = nrow(n_element[[n]]),
              ncol = ncol(n_element[[n]])
            )
          } else {
            o_element[[n]] <- rep(NA,
              times = length(n_element[[n]])
            )
          }
        }
      }

      o_unique <- names(o_element)[!names(o_element) %in% names(n_element)]
      if (length(o_unique) > 0) {
        for (n in o_unique) {
          if (is.matrix(n_element[[n]])) {
            n_element[[n]] <- matrix(NA,
              nrow = nrow(o_element[[n]]),
              ncol = ncol(o_element[[n]])
            )
          } else {
            n_element[[n]] <- rep(NA,
              times = length(o_element[[n]])
            )
          }
        }
      }
    }

    # Now merge the two lists
    return(mergeLists(
      o_element,
      n_element
    ))
  }

  if (length(n_element) > 1) {
    new_cols <- ifelse(is.matrix(n_element), ncol(n_element), length(n_element))
    old_cols <- ifelse(is.matrix(o_element), ncol(o_element), length(o_element))
    if (is.null(dim(n_element)) && is.null(dim(o_element))) {
      return(c(o_element, n_element))
    }

    if (new_cols != old_cols) {
      stop(
        "Your length doesn't match on the elements,",
        " new element (", new_cols, ") ! = ",
        " old element (", old_cols, ")"
      )
    }
  }

  if (length(n_element) > 1) {
    return(rbind(o_element,
                 n_element,
                 deparse.level = 0
    ))
  }
  return(c(
    o_element,
    n_element
  ))
}

library("testthat")
context("mergeLists")

test_that("Basic merge", {
  v1 <- list(a = c(3, 2, 1))
  v2 <- list(a = 3)
  merged <- mergeLists(v1, v2)
  expect_equal(length(merged$a), 4)
  expect_equal(all(merged$a %in% 1:3), TRUE)

  v1 <- list(a = c(3, 2, 1), b = 1)
  v2 <- list(a = 3, b = list("incompatible"))
  expect_error(mergeLists(v1, v2),
               regexp = "incompatible",
               info = "Can't merge a list item with a non-list item")

  v1 <- list("a" = c(1, 2), b = "test 1", sublist = list(one = 20:21, two = 21:22))
  v2 <- list("a" = c(3, 4), b = "test 2", sublist = list(one = 10:11, two = 11:12, three = 1:2))
  merged <- mergeLists(v1, v2)
  expect_equal(dim(merged$a), NULL,
    info = "Two vectors should be appended to eachother"
  )
  expect_equal(merged$a[1:2], v1$a,
    info = "The first vector should in the beginning"
  )
  expect_equal(merged$a[3:4], v2$a,
    info = "The second vector should be in the end"
  )
  expect_equal(length(merged$a), 4,
    info = "Two vectors of length 2 should be bound by c generating a 4 length vector"
  )

  expect_equal(length(merged$b), 2,
    info = "Two elements should be c() together into a length of 2"
  )
  expect_equal(merged$b[1], v1$b,
    info = "The first element should be in the first position"
  )
  expect_equal(merged$b[2], v2$b,
    info = "The second element should be in the second position"
  )

  expect_equal(length(merged$sublist), 3,
    info = "Two sub-lists should merge and all elements should be included"
  )

  expect_equal(merged$sublist$one[1:2], v1$sublist$one,
    info = "The vector merge should work with sublists as well"
  )
  expect_equal(merged$sublist$one[3:4], v2$sublist$one,
    info = "The vector merge should work with sublists as well"
  )
  expect_equal(merged$sublist$two[1:2], v1$sublist$two,
    info = "The vector merge should work with sublists as well"
  )
  expect_equal(merged$sublist$two[3:4], v2$sublist$two,
    info = "The vector merge should work with sublists as well"
  )
  expect_equal(merged$sublist$three[3:4], v2$sublist$three,
    info = "The vector merge should work with sublists as well"
  )
  expect_true(all(is.na(merged$sublist$three[1:2])),
    info = "The matrix should be empty at the row for the one without those elements"
  )
})

test_that("Tricky merge", {
  v1 <- list(a = 1)
  v2 <- list(a = matrix(c(3, 4), nrow = 1))
  expect_error(mergeLists(v1, v2),
    info = "It shouldn't work if the elements don't match"
  )

  v1 <- list(a = rbind(c(1, 2), c(11, 12)))
  v2 <- list(a = c(3, 4))
  merged <- mergeLists(v1, v2)
  expect_equal(nrow(merged$a), 3,
    info = "If you have one matrix and one vector that have compatible lengths they should match"
  )

  expect_equal(merged$a[1:2, ], v1$a,
    info = "There is something strange with the matrix + vector match"
  )
  expect_equal(merged$a[3, ], v2$a,
    info = "There is something strange with the matrix + vector match"
  )

  v1 <- list(a = rbind(c(1, 2), c(11, 12), c(13, 14)))
  v2 <- list(a = c(3, 4, 5))
  expect_error(mergeLists(v1, v2),
    info = "It shouldn't work if the elements don't match"
  )

  v1 <- list(a = rbind(c(1, 2), c(11, 12), c(13, 14)))
  v2 <- list(a = rbind(c(3, 4), c(31, 32)))
  merged <- mergeLists(v1, v2)
  expect_equal(merged$a[1:3, ], v1$a)
  expect_equal(merged$a[4:5, ], v2$a)
})

#' Describe the mean
#'
#' A function that returns a description of a continuous variable using
#' the mean together with the standard deviation. The standard deviation
#' is used as it is "industry standard" to use mean with standard deviation
#' and not because it's the only option.
#'
#' @param x The variable that you want the statistics for
#' @param digits The number of decimals used
#' @param digits.nonzero The number of decimals used for values that are close to zero
#' @param html If HTML compatible output should be used. If \code{FALSE}
#'  it outputs LaTeX formatting
#' @param number_first If the number should be given or if the percentage
#'  should be presented first. The second is encapsulated in parentheses ().
#'  This is only used together with the useNA variable.
#' @param useNA This indicates if missing should be added as a separate
#'  row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
#'  \emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.
#' @param useNA.digits The number of digits to use for the
#'  missing percentage, defaults to the overall \code{digits}.
#' @param percentage_sign If you want to suppress the percentage sign you
#'  can set this variable to FALSE. You can also choose something else that
#'  the default \% if you so wish by setting this variable. Note, this is
#'  only used when combined with the missing information.
#' @param plusmin_str Provide if you want anything other than the plus minus sign
#'  suited for the given output format.
#' @param language The ISO-639-1 two-letter code for the language of
#'  interest. Currently only english is distinguished from the ISO
#'  format using a ',' as the separator in the \code{\link[htmlTable]{txtInt}}
#'  function.
#' @param ... Passed on to \code{\link{describeFactors}}
#' @return \code{string} Returns a string formatted for either LaTeX or HTML
#'
#' @seealso \code{\link{getDescriptionStatsBy}}
#'
#' @examples
#' describeMean(1:10)
#' describeMean(c(1:10, NA), useNA = "always")
#' describeMean(c(1:10, NA), useNA = "no")
#' @family descriptive functions
#' @importFrom stats sd
#' @export
describeMean <- function(x,
                         html = TRUE,
                         digits = 1,
                         digits.nonzero = NA,
                         number_first = TRUE,
                         useNA = c("ifany", "no", "always"),
                         useNA.digits = digits,
                         percentage_sign = TRUE,
                         plusmin_str,
                         language = "en",
                         ...) {
  dot_args <- list(...)

  # Warnings due to interface changes in 1.0
  if ("show_missing_digits" %in% names(dot_args)) {
    useNA.digits <- dot_args$show_missing_digits
    dot_args$show_missing_digits <- NULL
    warning("Deprecated: show_missing_digits argument is now useNA.digits as of ver. 1.0")
  }

  if ("show_missing" %in% names(dot_args)) {
    if (missing(useNA)) {
      useNA <- convertShowMissing(dot_args$show_missing)
    }
    dot_args$show_missing <- NULL
    warning("Deprecated: show_missing argument is now useNA as of ver. 1.0")
  }

  useNA <- match.arg(useNA)

  if (missing(plusmin_str)) {
    if (html) {
      plusmin_str <- "&plusmn;"
    } else {
      plusmin_str <- "\\pm"
    }
  }

  ret <- c(sprintf(
    "%s (%s%s)",
    txtRound(mean(x, na.rm = TRUE), digits = digits, digits.nonzero = digits.nonzero),
    plusmin_str,
    txtRound(sd(x, na.rm = TRUE), digits = digits, digits.nonzero = digits.nonzero)
  ))

  # LaTeX complains if any formula isn't encapsulated within $ signs
  if (html == FALSE) {
    ret <- sprintf("$%s$", ret)
  }

  if (useNA %in% c("ifany", "always") && sum(is.na(x)) > 0) {
    ret <- rbind(
      ret,
      descGetMissing(
        x = x,
        html = html,
        number_first = number_first,
        percentage_sign = percentage_sign,
        language = language,
        useNA.digits = useNA.digits,
        digits.nonzero = digits.nonzero,
        dot_args = dot_args
      )
    )
    rownames(ret) <- c("Mean (SD)", "Missing")
  } else if (useNA == "always") {
    if (percentage_sign == TRUE) {
      percentage_sign <- ifelse(html, "%", "\\%")
    } else if (is.character(percentage_sign) == FALSE) {
      percentage_sign <- ""
    }

    empty <- sprintf(ifelse(number_first, "0 (0%s)", "0%s (0)"), percentage_sign)
    ret <- rbind(ret, rep(empty, times = NCOL(ret)))
    rownames(ret) <- c("Mean (SD)", "Missing")
  } else {
    names(ret) <- "Mean (SD)"
  }

  return(ret)
}


#' A function that returns a description median that contains
#' the interquartile range or the full range
#'
#' @param iqr If interquartile range should be used
#' @return \code{string} A string formatted for either LaTeX or HTML
#'
#' @inheritParams describeMean
#' @seealso \code{\link{getDescriptionStatsBy}}
#'
#' @examples
#' describeMedian(1:10)
#' describeMedian(c(1:10, NA), useNA = "ifany")
#' @family descriptive functions
#' @importFrom stats median quantile
#' @export
describeMedian <- function(x,
                           iqr = TRUE,
                           html = TRUE,
                           digits = 1,
                           digits.nonzero = NA,
                           number_first = TRUE,
                           useNA = c("ifany", "no", "always"),
                           useNA.digits = digits,
                           percentage_sign = TRUE,
                           language = "en",
                           ...) {
  dot_args <- list(...)

  # Warnings due to interface changes in 1.0
  if ("show_missing_digits" %in% names(dot_args)) {
    show_missing.digits <- dot_args$show_missing_digits
    dot_args$show_missing_digits <- NULL
    warning("Deprecated: show_missing_digits argument is now show_missing.digits as of ver. 1.0")
  }

  if ("show_missing" %in% names(dot_args)) {
    if (missing(useNA)) {
      useNA <- convertShowMissing(dot_args$show_missing)
    }
    dot_args$show_missing <- NULL
    warning("Deprecated: show_missing argument is now useNA as of ver. 1.0")
  }

  useNA <- match.arg(useNA)

  if (iqr) {
    range_quantiles <- c(1 / 4, 3 / 4)
  } else {
    range_quantiles <- c(0, 1)
  }

  ret <- sprintf(
    "%s (%s - %s)",
    txtRound(median(x, na.rm = TRUE), digits = digits, digits.nonzero = digits.nonzero),
    txtRound(quantile(x, probs = range_quantiles[1], na.rm = TRUE), digits = digits, digits.nonzero = digits.nonzero),
    txtRound(quantile(x, probs = range_quantiles[2], na.rm = TRUE), digits = digits, digits.nonzero = digits.nonzero)
  )

  if (useNA %in% c("ifany", "always") && sum(is.na(x)) > 0) {
    ret <- rbind(
      ret,
      descGetMissing(
        x = x,
        html = html,
        number_first = number_first,
        percentage_sign = percentage_sign,
        language = language,
        useNA.digits = useNA.digits,
        digits.nonzero = digits.nonzero,
        dot_args = dot_args
      )
    )

    rownames(ret) <- c(
      ifelse(iqr, "Median (IQR)", "Median (range)"),
      "Missing"
    )
  } else if (useNA == "always") {
    if (percentage_sign == TRUE) {
      percentage_sign <- ifelse(html, "%", "\\%")
    } else if (is.character(percentage_sign) == FALSE) {
      percentage_sign <- ""
    }

    empty <- sprintf(ifelse(number_first, "0 (0%s)", "0%s (0)"), percentage_sign)
    ret <- rbind(ret, rep(empty, times = NCOL(ret)))
    rownames(ret) <- c(
      ifelse(iqr, "Median (IQR)", "Median (range)"),
      "Missing"
    )
  } else {
    names(ret) <- ifelse(iqr, "Median (IQR)", "Median (range)")
  }

  return(ret)
}

#' A function that returns a description proportion that contains
#' the number and the percentage
#'
#' @return \code{string} A string formatted for either LaTeX or HTML
#'
#' @inheritParams describeMean
#' @inheritParams prDescGetAndValidateDefaultRef
#' @examples
#' describeProp(factor(sample(50, x = c("A", "B", NA), replace = TRUE)))
#' @family descriptive functions
#' @export
describeProp <- function(x,
                         html = TRUE,
                         digits = 1,
                         digits.nonzero = NA,
                         number_first = TRUE,
                         useNA = c("ifany", "no", "always"),
                         useNA.digits = digits,
                         default_ref = NULL,
                         percentage_sign = TRUE,
                         language = "en",
                         ...) {
  dot_args <- list(...)

  # Warnings due to interface changes in 1.0
  if ("show_missing_digits" %in% names(dot_args)) {
    useNA.digits <- dot_args$show_missing_digits
    dot_args$show_missing_digits <- NULL
    warning("Deprecated: show_missing_digits argument is now useNA.digits as of ver. 1.0")
  }

  if ("show_missing" %in% names(dot_args)) {
    if (missing(useNA)) {
      useNA <- convertShowMissing(dot_args$show_missing)
    }
    dot_args$show_missing <- NULL
    warning("Deprecated: show_missing argument is now useNA as of ver. 1.0")
  }

  useNA <- match.arg(useNA)

  default_ref <- prDescGetAndValidateDefaultRef(x, default_ref)

  # If we're to use the horizontal proportions then
  # it's better to report the variable as a factor
  # instead of a single proportion.
  # When we have missing it also gets more difficult
  # to just report one percentage as it suddenly uncertain
  # for what percentage the number applies to
  if ("horizontal_proportions" %in% names(dot_args) ||
    (useNA == "ifany" &&
      any(is.na(x))) ||
    useNA == "always") {
    df_arg_list <- list(
      x = x,
      html = html,
      number_first = number_first,
      percentage_sign = percentage_sign,
      language = language,
      digits = digits,
      digits.nonzero = digits.nonzero,
      useNA = useNA,
      useNA.digits = useNA.digits
    )
    for (n in names(dot_args)) {
      if (!n %in% names(df_arg_list)) {
        df_arg_list[[n]] <- dot_args[[n]]
      }
    }
    return(fastDoCall(describeFactors, df_arg_list))
  }

  if (!is.factor(x)) {
    x <- factor(x)
  }

  reference_name <- levels(x)[default_ref]
  no <- sum(x == reference_name, na.rm = TRUE)

  # Don't count missing since those are treated as factors if any
  percent <- 100 * no / length(x[is.na(x) == FALSE])

  oi_args <- list(
    x = no,
    language = language,
    html = html
  )
  for (n in names(dot_args)) {
    if (!n %in% names(oi_args)) {
      oi_args[[n]] <- dot_args[[n]]
    }
  }
  no <- fastDoCall(txtInt, oi_args)

  # LaTeX treats % as comments unless it's properly escaped
  if (percentage_sign == TRUE) {
    percentage_sign <- ifelse(html, "%", "\\%")
  } else if (!is.character(percentage_sign)) {
    percentage_sign <- ""
  }

  percentage_str <- paste0(txtRound(percent, digits = digits, digits.nonzero = digits.nonzero), percentage_sign)
  if (number_first) {
    ret <- sprintf(
      "%s (%s)",
      no,
      percentage_str
    )
  } else {
    ret <- sprintf(
      "%s (%s)",
      percentage_str,
      no
    )
  }

  return(ret)
}


#' Describes factor variables
#'
#' A function that returns a description of proportions in a
#' factor that contains the number of times a level occurs and the percentage
#'
#' @param ... Passed on to \code{\link[htmlTable]{txtInt}}
#' @param horizontal_proportions Is only active if useNA since this is
#'  the only case of a proportion among continuous variables. This is default NULL and indicates
#'  that the proportions are to be interpreted in a vertical manner.
#'  If we want the data to be horizontal, i.e. the total should be shown
#'  and then how these differ in the different groups then supply the
#'  function with the total number in each group, i.e. if done in a by
#'  manner as in \code{\link{getDescriptionStatsBy}} it needs to provide
#'  the number before the by() command.
#' @return A string formatted for printing either latex by  HTML
#'
#' @inheritParams describeMean
#' @seealso \code{\link{getDescriptionStatsBy}}
#' @family descriptive functions
#'
#' @examples
#' set.seed(1)
#' describeFactors(sample(50, x = c("A", "B", "C"), replace = TRUE))
#'
#' n <- 500
#' my_var <- factor(sample(size = n, x = c("A", "B", "C", NA), replace = TRUE))
#' my_exp <- rbinom(n = n, size = 1, prob = 0.2)
#' total <- table(my_var, useNA = "ifany")
#' by(my_var,
#'   INDICES = my_exp,
#'   FUN = describeFactors,
#'   useNA = "ifany",
#'   horizontal_proportions = total
#' )
#' @export
describeFactors <- function(x,
                            html = TRUE,
                            digits = 1,
                            digits.nonzero = NA,
                            number_first = TRUE,
                            useNA = c("ifany", "no", "always"),
                            useNA.digits = digits,
                            horizontal_proportions,
                            percentage_sign = TRUE,
                            language = "en",
                            ...) {
  dot_args <- list(...)

  # Warnings due to interface changes in 1.0
  if ("show_missing_digits" %in% names(dot_args)) {
    useNA.digits <- dot_args$show_missing_digits
    dot_args$show_missing_digits <- NULL
    warning("Deprecated: show_missing_digits argument is now useNA.digits as of ver. 1.0")
  }

  if ("show_missing" %in% names(dot_args)) {
    if (missing(useNA)) {
      useNA <- convertShowMissing(dot_args$show_missing)
    }
    dot_args$show_missing <- NULL
    warning("Deprecated: show_missing argument is now useNA as of ver. 1.0")
  }

  useNA <- match.arg(useNA)

  # Get basic data
  table_results <- table(x, useNA = useNA)

  # Check if we should relate to an external total
  if (!missing(horizontal_proportions)) {
    # Error check the horizontal_proportions variable
    # First check that it's a table or at least a vector
    if (is.numeric(horizontal_proportions) == FALSE ||
      (!inherits(horizontal_proportions, "table") &&
        is.vector(horizontal_proportions) == FALSE)) {
      stop(
        "You have not provided a proper table/vector variable for the function,",
        " the class you've given is: '", paste(class(horizontal_proportions), collapse = " & "), "'",
        ", instead of numeric vector or table"
      )
    }

    # The original table should always be longer than the subtable
    if (length(horizontal_proportions) < length(table_results)) {
      stop(
        "There is a length discrepancy between the number of groups in the given sample",
        " and the reference sample, ",
        length(horizontal_proportions), " < ", length(table_results), "!"
      )
    }

    # All variables should exist in the horizontal table
    if (all(names(table_results) %in% names(horizontal_proportions)) == FALSE) {
      stop(
        "Your results contain results that are not in ",
        "the reference horizontal_proportions: ",
        paste(names(table_results)[names(table_results) %in% names(horizontal_proportions)],
          collapse = ", "
        )
      )
    }

    # Create a equal length array with the same order
    if (length(horizontal_proportions) > length(table_results)) {
      # There seems to be a few variables more in the horizontal version and
      # then we need to fill in those values with 0
      # Initiate an empty array
      tmp <- rep(0, times = length(horizontal_proportions))
      # The array should have the names of the results
      # to be able to reference them
      names(tmp) <- names(horizontal_proportions)
      # Iterate and copy the values
      for (n in names(tmp)) {
        if (n %in% names(table_results)) {
          tmp[n] <- table_results[n]
        }
      }
      # Reset the table_results to our new array
      table_results <- tmp
      class(tmp) <- "table"
    }

    # Initiate an empty array
    percentages <- rep(NA, times = length(horizontal_proportions))
    # The array should have the names of the results
    # to be able to reference them
    names(percentages) <- names(horizontal_proportions)
    for (n in names(horizontal_proportions)) {
      # Strange things happen with NA
      # Need to find that element in a slightly odd way
      if (is.na(n)) {
        percentages[is.na(names(percentages))] <-
          table_results[is.na(names(table_results))] / horizontal_proportions[is.na(names(horizontal_proportions))] * 100
      } else {
        percentages[n] <- table_results[n] / horizontal_proportions[n] * 100
      }
    }
  } else {
    percentages <- table_results / sum(table_results) * 100
  }

  # Format the values
  sa_args <- list(
    X = table_results,
    FUN = txtInt,
    language = language,
    html = html
  )
  for (n in names(dot_args)) {
    if (!n %in% names(sa_args)) {
      sa_args[[n]] <- dot_args[[n]]
    }
  }
  values <- fastDoCall(sapply, sa_args)

  # LaTeX treats % as comments unless it's properly escaped
  if (percentage_sign == TRUE) {
    percentage_sign <- ifelse(html, "%", "\\%")
  } else if (is.character(percentage_sign) == FALSE) {
    percentage_sign <- ""
  }

  # The is.na(...) is a little overkill
  percentage_str <- paste0(txtRound(percentages,
    digits = ifelse(is.na(names(table_results)), useNA.digits, digits),
    digits.nonzero = digits.nonzero
  ), percentage_sign)

  if (number_first) {
    ret <- matrix(sprintf("%s (%s)", values, percentage_str), ncol = 1)
  } else {
    ret <- matrix(sprintf("%s (%s)", percentage_str, values), ncol = 1)
  }

  rn <- names(table_results)
  rn[is.na(rn)] <- "Missing"
  rownames(ret) <- rn
  return(ret)
}

library("testthat")
library("stringr")

test_that("missing levels are handled correctly when using custom descriptive functions", {
  set.seed(1)
  trial <- data.frame(
    visit = sort(rep(c("randomisation", "week1", "week2", "week3"), 5)),
    arm = sort(rep(c("control", "treatment"))),
    outcome = rnorm(40),
    stringsAsFactors = TRUE
  )
  trial_missing_first <- trial[!((trial$visit == "randomisation") & (trial$arm == "control")), ]
  trial_missing_second <- trial[!((trial$visit == "randomisation") & (trial$arm == "treatment")), ]
  trial_missing_both <- trial[trial$visit != "week3", ]

  trial_2 <- data.frame(
    visit = sort(rep(c("randomisation", "week1", "week2", "week3"), 5)),
    arm = sort(rep(c("control", "standard treatment", "new treatment"))),
    outcome = rnorm(60),
    stringsAsFactors = TRUE
  )
  trial_2_missing_first <- trial_2[!((trial_2$visit == "randomisation") & (trial_2$arm == "control")), ]
  trial_2_missing_second <- trial_2[!((trial_2$visit == "randomisation") & (trial_2$arm == "standard treatment")), ]
  trial_2_missing_outer <- trial_2[!((trial_2$visit == "randomisation") & (trial_2$arm != "new treatment")), ]
  trial_2_missing_all <- trial_2[trial_2$visit != "week3", ]

  descriptive_function <- function(x, ...) {
    result <- c(
      describeMean(x, ...),
      describeMedian(x, ...)
    )
    return(result)
  }

  expected_no_missing <- structure(
    c(
      "0.1 (&plusmn;0.7)", "0.2 (-0.5 - 0.5)", "0.1 (&plusmn;0.5)",
      "0.2 (-0.2 - 0.5)", "0.4 (&plusmn;1.2)", "0.8 (-0.4 - 1.3)",
      "0.4 (&plusmn;0.7)", "0.4 (-0.1 - 0.9)", "0.1 (&plusmn;1.5)",
      "0.5 (-0.4 - 1.0)", "-0.2 (&plusmn;0.8)", "-0.2 (-0.7 - 0.3)",
      "-0.5 (&plusmn;1.2)", "-0.1 (-0.6 - 0.1)", "0.3 (&plusmn;0.5)",
      "0.3 (-0.1 - 0.7)"
    ),
    .Dim = c(8L, 2L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_missing_first <- structure(
    c(
      "-", "-", "0.1 (&plusmn;0.5)", "0.2 (-0.2 - 0.5)",
      "0.4 (&plusmn;1.2)", "0.8 (-0.4 - 1.3)", "0.4 (&plusmn;0.7)",
      "0.4 (-0.1 - 0.9)", "0.1 (&plusmn;1.5)", "0.5 (-0.4 - 1.0)",
      "-0.2 (&plusmn;0.8)", "-0.2 (-0.7 - 0.3)", "-0.5 (&plusmn;1.2)",
      "-0.1 (-0.6 - 0.1)", "0.3 (&plusmn;0.5)", "0.3 (-0.1 - 0.7)"
    ),
    .Dim = c(8L, 2L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_missing_second <- structure(
    c(
      "0.1 (&plusmn;0.7)", "0.2 (-0.5 - 0.5)", "0.1 (&plusmn;0.5)",
      "0.2 (-0.2 - 0.5)", "0.4 (&plusmn;1.2)", "0.8 (-0.4 - 1.3)",
      "0.4 (&plusmn;0.7)", "0.4 (-0.1 - 0.9)", "-", "-", "-0.2 (&plusmn;0.8)",
      "-0.2 (-0.7 - 0.3)", "-0.5 (&plusmn;1.2)", "-0.1 (-0.6 - 0.1)",
      "0.3 (&plusmn;0.5)", "0.3 (-0.1 - 0.7)"
    ),
    .Dim = c(8L, 2L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_missing_both <- structure(
    c(
      "0.1 (&plusmn;0.7)", "0.2 (-0.5 - 0.5)", "0.1 (&plusmn;0.5)",
      "0.2 (-0.2 - 0.5)", "0.4 (&plusmn;1.2)", "0.8 (-0.4 - 1.3)",
      "-", "-", "0.1 (&plusmn;1.5)", "0.5 (-0.4 - 1.0)", "-0.2 (&plusmn;0.8)",
      "-0.2 (-0.7 - 0.3)", "-0.5 (&plusmn;1.2)", "-0.1 (-0.6 - 0.1)",
      "-", "-"
    ),
    .Dim = c(8L, 2L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_no_missing_2 <- structure(
    c(
      "0.2 (&plusmn;0.7)", "0.0 (-0.2 - 0.6)", "0.7 (&plusmn;0.5)",
      "0.4 (0.4 - 0.9)", "0.5 (&plusmn;1.0)", "0.5 (0.3 - 1.2)",
      "0.2 (&plusmn;1.1)", "-0.4 (-0.6 - 0.6)", "-0.5 (&plusmn;0.8)",
      "-0.6 (-0.7 - -0.3)", "0.5 (&plusmn;0.4)", "0.3 (0.2 - 0.8)",
      "-0.3 (&plusmn;1.0)", "-0.7 (-1.1 - 0.4)", "-0.2 (&plusmn;0.7)",
      "-0.1 (-0.4 - 0.0)", "0.7 (&plusmn;1.0)", "0.6 (0.0 - 0.7)",
      "-0.2 (&plusmn;1.5)", "-0.3 (-0.7 - -0.1)", "0.3 (&plusmn;0.9)",
      "0.6 (-0.5 - 0.7)", "-0.5 (&plusmn;0.7)", "-0.5 (-1.0 - 0.1)"
    ),
    .Dim = c(8L, 3L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "new treatment", "standard treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_missing_first_2 <- structure(
    c(
      "-", "-", "0.7 (&plusmn;0.5)", "0.4 (0.4 - 0.9)",
      "0.5 (&plusmn;1.0)", "0.5 (0.3 - 1.2)", "0.2 (&plusmn;1.1)",
      "-0.4 (-0.6 - 0.6)", "-0.5 (&plusmn;0.8)", "-0.6 (-0.7 - -0.3)",
      "0.5 (&plusmn;0.4)", "0.3 (0.2 - 0.8)", "-0.3 (&plusmn;1.0)",
      "-0.7 (-1.1 - 0.4)", "-0.2 (&plusmn;0.7)", "-0.1 (-0.4 - 0.0)",
      "0.7 (&plusmn;1.0)", "0.6 (0.0 - 0.7)", "-0.2 (&plusmn;1.5)",
      "-0.3 (-0.7 - -0.1)", "0.3 (&plusmn;0.9)", "0.6 (-0.5 - 0.7)",
      "-0.5 (&plusmn;0.7)", "-0.5 (-1.0 - 0.1)"
    ),
    .Dim = c(8L, 3L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "new treatment", "standard treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_missing_second_2 <- structure(
    c(
      "0.2 (&plusmn;0.7)", "0.0 (-0.2 - 0.6)", "0.7 (&plusmn;0.5)",
      "0.4 (0.4 - 0.9)", "0.5 (&plusmn;1.0)", "0.5 (0.3 - 1.2)",
      "0.2 (&plusmn;1.1)", "-0.4 (-0.6 - 0.6)", "-0.5 (&plusmn;0.8)",
      "-0.6 (-0.7 - -0.3)", "0.5 (&plusmn;0.4)", "0.3 (0.2 - 0.8)",
      "-0.3 (&plusmn;1.0)", "-0.7 (-1.1 - 0.4)", "-0.2 (&plusmn;0.7)",
      "-0.1 (-0.4 - 0.0)", "-", "-", "-0.2 (&plusmn;1.5)", "-0.3 (-0.7 - -0.1)",
      "0.3 (&plusmn;0.9)", "0.6 (-0.5 - 0.7)", "-0.5 (&plusmn;0.7)",
      "-0.5 (-1.0 - 0.1)"
    ),
    .Dim = c(8L, 3L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "new treatment", "standard treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_missing_outer <- structure(
    c(
      "-", "-", "0.7 (&plusmn;0.5)", "0.4 (0.4 - 0.9)",
      "0.5 (&plusmn;1.0)", "0.5 (0.3 - 1.2)", "0.2 (&plusmn;1.1)",
      "-0.4 (-0.6 - 0.6)", "-0.5 (&plusmn;0.8)", "-0.6 (-0.7 - -0.3)",
      "0.5 (&plusmn;0.4)", "0.3 (0.2 - 0.8)", "-0.3 (&plusmn;1.0)",
      "-0.7 (-1.1 - 0.4)", "-0.2 (&plusmn;0.7)", "-0.1 (-0.4 - 0.0)",
      "-", "-", "-0.2 (&plusmn;1.5)", "-0.3 (-0.7 - -0.1)", "0.3 (&plusmn;0.9)",
      "0.6 (-0.5 - 0.7)", "-0.5 (&plusmn;0.7)", "-0.5 (-1.0 - 0.1)"
    ),
    .Dim = c(8L, 3L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "new treatment", "standard treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  expected_missing_all <- structure(
    c(
      "0.2 (&plusmn;0.7)", "0.0 (-0.2 - 0.6)", "0.7 (&plusmn;0.5)",
      "0.4 (0.4 - 0.9)", "0.5 (&plusmn;1.0)", "0.5 (0.3 - 1.2)", "-",
      "-", "-0.5 (&plusmn;0.8)", "-0.6 (-0.7 - -0.3)", "0.5 (&plusmn;0.4)",
      "0.3 (0.2 - 0.8)", "-0.3 (&plusmn;1.0)", "-0.7 (-1.1 - 0.4)",
      "-", "-", "0.7 (&plusmn;1.0)", "0.6 (0.0 - 0.7)", "-0.2 (&plusmn;1.5)",
      "-0.3 (-0.7 - -0.1)", "0.3 (&plusmn;0.9)", "0.6 (-0.5 - 0.7)", "-", "-"
    ),
    .Dim = c(8L, 3L),
    .Dimnames = list(
      c(
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)",
        "Mean (SD)", "Median (IQR)"
      ),
      c("control", "new treatment", "standard treatment")
    ),
    rgroup = c("randomisation", "week1", "week2", "week3"),
    n.rgroup = c(2, 2, 2, 2),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(lapply(levels(trial$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial$outcome[trial$visit == x],
                              by = trial$arm[trial$visit == x],
                              continuous_fn = descriptive_function
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial$visit),
                     n.rgroup = rep(2, 4))
  )
  expect_identical(out, expected_no_missing)

  out <- mergeDesc(lapply(levels(trial_missing_first$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_missing_first$outcome[trial_missing_first$visit == x],
                              by = trial_missing_first$arm[trial_missing_first$visit == x],
                              continuous_fn = descriptive_function
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_missing_first$visit),
                     n.rgroup = rep(2, 4))
  )
  expect_identical(out, expected_missing_first)

  out <- mergeDesc(lapply(levels(trial_missing_second$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_missing_second$outcome[trial_missing_second$visit == x],
                              by = trial_missing_second$arm[trial_missing_second$visit == x],
                              continuous_fn = descriptive_function
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_missing_second$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_identical(out, expected_missing_second)

  out <- mergeDesc(lapply(levels(trial_missing_both$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_missing_both$outcome[trial_missing_both$visit == x],
                              by = trial_missing_both$arm[trial_missing_both$visit == x],
                              continuous_fn = descriptive_function,
                              names_of_missing = c("Mean (SD)", "Median (IQR)")
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_missing_both$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_identical(out, expected_missing_both)

  out <- mergeDesc(lapply(levels(trial_2$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2$outcome[trial_2$visit == x],
                              by = trial_2$arm[trial_2$visit == x],
                              continuous_fn = descriptive_function
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_identical(out, expected_no_missing_2)

  out <- mergeDesc(lapply(levels(trial_2_missing_first$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2_missing_first$outcome[trial_2_missing_first$visit == x],
                              by = trial_2_missing_first$arm[trial_2_missing_first$visit == x],
                              continuous_fn = descriptive_function
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_first$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_identical(out, expected_missing_first_2)

  out <- mergeDesc(lapply(levels(trial_2_missing_second$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2_missing_second$outcome[trial_2_missing_second$visit == x],
                              by = trial_2_missing_second$arm[trial_2_missing_second$visit == x],
                              continuous_fn = descriptive_function,
                              names_of_missing = c("Mean (SD)", "Median (IQR)")
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_second$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_identical(out, expected_missing_second_2)

  out <- mergeDesc(lapply(levels(trial_2_missing_outer$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2_missing_outer$outcome[trial_2_missing_outer$visit == x],
                              by = trial_2_missing_outer$arm[trial_2_missing_outer$visit == x],
                              continuous_fn = descriptive_function,
                              names_of_missing = c("Mean (SD)", "Median (IQR)")
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_outer$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_identical(out, expected_missing_outer)

  out <- mergeDesc(lapply(levels(trial_2_missing_all$visit),
                          function(x) {
                            getDescriptionStatsBy(trial_2_missing_all$outcome[trial_2_missing_all$visit == x],
                                                  trial_2_missing_all$arm[trial_2_missing_all$visit == x],
                                                  continuous_fn = descriptive_function,
                                                  names_of_missing = c("Mean (SD)", "Median (IQR)")
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_all$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_identical(out, expected_missing_all)

  Hmisc::label(trial_2, self = FALSE) <- c("Study Visit", "Treatment Arm", "Outcome Measure")
  trial_2_missing_first <- trial_2[!((trial_2$visit == "randomisation") & (trial_2$arm == "control")), ]
  trial_2_missing_second <- trial_2[!((trial_2$visit == "randomisation") & (trial_2$arm == "standard treatment")), ]
  trial_2_missing_outer <- trial_2[!((trial_2$visit == "randomisation") & (trial_2$arm != "new treatment")), ]
  trial_2_missing_all <- trial_2[trial_2$visit != "week3", ]

  out <- mergeDesc(lapply(levels(trial_2$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2$outcome[trial_2$visit == x],
                              by = trial_2$arm[trial_2$visit == x],
                              continuous_fn = descriptive_function
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_equivalent(out, expected_no_missing_2)

  out <- mergeDesc(lapply(levels(trial_2_missing_first$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2_missing_first$outcome[trial_2_missing_first$visit == x],
                              by = trial_2_missing_first$arm[trial_2_missing_first$visit == x],
                              continuous_fn = descriptive_function
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_first$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_equivalent(out, expected_missing_first_2)

  out <- mergeDesc(lapply(levels(trial_2_missing_second$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2_missing_second$outcome[trial_2_missing_second$visit == x],
                              by = trial_2_missing_second$arm[trial_2_missing_second$visit == x],
                              continuous_fn = descriptive_function,
                              names_of_missing = c("Mean (SD)", "Median (IQR)")
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_second$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_equivalent(out, expected_missing_second_2)

  out <- mergeDesc(lapply(levels(trial_2_missing_outer$visit),
                          function(x) {
                            getDescriptionStatsBy(
                              x = trial_2_missing_outer$outcome[trial_2_missing_outer$visit == x],
                              by = trial_2_missing_outer$arm[trial_2_missing_outer$visit == x],
                              continuous_fn = descriptive_function,
                              names_of_missing = c("Mean (SD)", "Median (IQR)")
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_outer$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_equivalent(out, expected_missing_outer)

  out <- mergeDesc(lapply(levels(trial_2_missing_all$visit),
                          function(x) {
                            getDescriptionStatsBy(trial_2_missing_all$outcome[trial_2_missing_all$visit == x],
                                                  trial_2_missing_all$arm[trial_2_missing_all$visit == x],
                                                  continuous_fn = descriptive_function,
                                                  names_of_missing = c("Mean (SD)", "Median (IQR)")
                            )
                          }),
                   htmlTable_args = list(
                     rgroup = levels(trial_2_missing_all$visit),
                     n.rgroup = rep(2, 4)
                   )
  )
  expect_equivalent(out, expected_missing_all)
})


test_that("Error when one category has no missing in it", {
  set.seed(1)
  aa <- factor(sample(c("A", "B"), size = 50, replace = TRUE))
  aa[sample(1:50, size = 5)] <- NA
  aaa <- factor(sample(1:3, size = 50, replace = TRUE))
  aa[aaa == 2 & is.na(aa)] <- "B"
  ret <- getDescriptionStatsBy(x = aa, by = aaa, add_total_col = TRUE)

  expect_match(ret["A", "2"], sprintf("^%d", table(aa, aaa)["A", "2"]),
               info = "The value does not seem to match the raw table"
  )
  expect_match(ret["B", "2"], sprintf("^%d", table(aa, aaa)["B", "2"]),
               info = "The value does not seem to match the raw table"
  )
  expect_match(ret["Missing", "2"], "^0")

  ret <-
    getDescriptionStatsBy(x = aa, by = aaa, useNA = "no", add_total_col = TRUE)
  expect_match(ret[1, "2"], sprintf("^%d", table(aa, aaa)["A", "2"]),
               info = "The value does not seem to match the raw table"
  )
  expect_equal(nrow(ret), 1)

  ret <-
    getDescriptionStatsBy(
      x = aa, by = aaa,
      add_total_col = TRUE,
      useNA = "no",
      prop_fn = describeFactors
    )
  expect_match(ret["A", "2"], sprintf("^%d", table(aa, aaa)["A", "2"]),
               info = "The value does not seem to match the raw table"
  )
  expect_match(ret["B", "2"], sprintf("^%d", table(aa, aaa)["B", "2"]),
               info = "The value does not seem to match the raw table"
  )

  cont <- rnorm(length(aa))
  cont[sample(1:50, size = 5)] <- NA
  cont[aaa == 2 & is.na(cont)] <- 0
  ret <- getDescriptionStatsBy(x = cont, by = aaa, useNA = "no", add_total_col = TRUE)
  expect_equal(nrow(ret), 1)

  aa <- factor(sample(LETTERS[1:3], size = 50, replace = TRUE))
  aa[sample(1:50, size = 5)] <- NA
  aaa <- factor(sample(1:2, size = 50, replace = TRUE))
  aa[aaa == 2] <- "B"
  ret <- getDescriptionStatsBy(x = aa, by = aaa, useNA = "no", add_total_col = TRUE)
  expect_match(ret["A", "2"], sprintf("^%d", table(aa, aaa)["A", "2"]),
               info = "The value does not seem to match the raw table"
  )
  expect_match(ret["B", "2"], sprintf("^%d", table(aa, aaa)["B", "2"]),
               info = "The value does not seem to match the raw table"
  )
})

test_that("Error when one continuous variable has no missing in it", {
  set.seed(1)
  aa <- runif(50)
  aa[sample(1:50, size = 5)] <- NA
  aaa <- factor(sample(1:3, size = 50, replace = TRUE))
  aa[aaa == 2 & is.na(aa)] <- 1
  ret <-
    getDescriptionStatsBy(x = aa, by = aaa, html = TRUE)

  expect_match(
    ret["Missing", "2"],
    sprintf("^%d", sum(is.na(aa[aaa == 2])))
  )
})

test_that("Error when a factor variable has an empty level", {
  set.seed(1)
  variable <- factor(sample(LETTERS[1:2], size = 50, replace = TRUE),
                     levels = LETTERS[1:3]
  )
  variable[sample(1:50, size = 5)] <- NA
  by <- factor(sample(1:2, size = 50, replace = TRUE))
  ret <- getDescriptionStatsBy(x = variable, by = by, add_total_col = TRUE, useNA = "no")

  expect_match(
    ret["B", "2"],
    sprintf("^%d", sum(variable[by == 2] == "B", na.rm = TRUE))
  )
})

library("testthat")
library("stringr")
library("dplyr")
# I need to include this for unknown reason or the test fails in R CMD check mode

data("Loblolly")

set.seed(1)
Loblolly$young <- Loblolly$age < 10
Loblolly$young <- factor(Loblolly$young, label = c("Yes", "No"))
Loblolly$fvar <- factor(sample(letters[1:3], size = nrow(Loblolly), replace = TRUE))
Loblolly$young_w_missing <- Loblolly$young
Loblolly$young_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA
Loblolly$fvar_w_missing <- Loblolly$fvar
Loblolly$fvar_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA
Loblolly$height_w_missing <- Loblolly$height
Loblolly$height_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA

test_that("Check mean function", {
  stats <- by(Loblolly$height, Loblolly$young, mean)
  a <- getDescriptionStatsBy(Loblolly$height,
                             Loblolly$young,
                             statistics = TRUE,
                             digits = 2,
                             statistics.sig_lim = 10^-4)
  # Check that it contains the true mean
  expect_true(grepl(round(stats[["No"]], 2), a[1, "No"]),
    info = "Expected the mean"
  )
  expect_true(grepl(round(stats[["Yes"]], 2), a[1, "Yes"]),
    info = "Expected the mean"
  )

  # Check that it contains the sd
  stats <- by(Loblolly$height, Loblolly$young, sd)
  expect_true(grepl(round(stats[["No"]], 2), a[1, "No"]),
    info = "Expected the sd"
  )
  expect_true(grepl(round(stats[["Yes"]], 2), a[1, "Yes"]),
    info = "Expected the sd"
  )

  true_wilc_pv <- txtPval(wilcox.test(Loblolly$height ~ Loblolly$young)$p.value,
    statistics.sig_lim = 10^-4
  )
  expect_equal(
    as.character(a[1, "P-value"]),
    true_wilc_pv
  )

  # Check P-value without truncation
  a <- getDescriptionStatsBy(Loblolly$height, Loblolly$age == 10,
    statistics = TRUE,
    digits = 2, statistics.sig_lim = 10^-4
  )
  true_wilc_pv <- txtPval(wilcox.test(Loblolly$height ~ Loblolly$age == 10)$p.value,
    statistics.sig_lim = 10^-4
  )
  expect_equal(
    as.character(a[1, "P-value"]),
    true_wilc_pv
  )
})

test_that("Check median function", {
  stats <- by(Loblolly$height, Loblolly$young, median)
  a <- getDescriptionStatsBy(Loblolly$height, Loblolly$young,
    continuous_fn = describeMedian,
    statistics = TRUE,
    digits = 2, statistics.sig_lim = 10^-4
  )
  # Check that it contains the true mean
  expect_true(grepl(round(stats[["No"]], 2), a[1, "No"]),
    info = "Expected the median"
  )
  expect_true(grepl(round(stats[["Yes"]], 2), a[1, "Yes"]),
    info = "Expected the median"
  )

  # Check that it contains the sd
  stats <- by(
    Loblolly$height, Loblolly$young,
    function(x) {
      str_trim(paste(format(quantile(x, probs = c(.25, .75)),
        digits = 2,
        nsmall = 2
      ), collapse = " - "))
    }
  )
  expect_true(grepl(stats[["No"]], a[1, "No"]),
    info = "Expected the iqr range"
  )
  expect_true(grepl(stats[["Yes"]], a[1, "Yes"]),
    info = "Expected the iqr range"
  )

  true_wilc_pv <- txtPval(wilcox.test(Loblolly$height ~ Loblolly$young)$p.value,
    statistics.sig_lim = 10^-4
  )
  expect_equal(
    as.character(a[1, "P-value"]),
    true_wilc_pv
  )

  a <- getDescriptionStatsBy(Loblolly$height, Loblolly$young,
    continuous_fn = function(...) {
      describeMedian(..., iqr = FALSE)
    },
    statistics = TRUE,
    digits = 2, statistics.sig_lim = 10^-4
  )

  # Check that it contains the sd
  stats <- by(
    Loblolly$height, Loblolly$young,
    function(x) paste(round(range(x), 2), collapse = " - ")
  )
  expect_true(grepl(stats[["No"]], a[1, "No"]),
    info = "Expected the range"
  )
  expect_true(grepl(stats[["Yes"]], a[1, "Yes"]),
    info = "Expected the range"
  )
})

test_that("Check small proportions", {
  n <- 1e4
  a <- getDescriptionStatsBy(LETTERS[c(rep(1, times = (n - 4)), rep(2, times = 4))],
                             letters[c(rep(1, times = (n - 10)), rep(2, times = 10))],
                             hrzl_prop = TRUE,
                             statistics = TRUE,
                             digits = 0,
                             digits.nonzero = 2,
                             statistics.sig_lim = 10^-4)
  expect_equivalent(as.character(a["A", "b"]), "6 (0.06%)")

  a <- getDescriptionStatsBy(LETTERS[c(rep(1, times = (n - 4)), rep(2, times = 4))],
                             letters[c(rep(1, times = (n - 10)), rep(2, times = 10))],
                             hrzl_prop = TRUE,
                             statistics = TRUE,
                             digits = 0,
                             statistics.sig_lim = 10^-4)
  expect_equivalent(as.character(a["A", "b"]), "6 (0%)")

  # Make sure to handle character input and make it default to the
  # first factor level
  set.seed(1)
  fake_data <- data.frame(Large = sample(LETTERS[1:2], size = n, replace = TRUE),
                          Small = sample(letters[1:2], size = n, replace = TRUE))
  fake_data$Small_factor <- factor(fake_data$Small)
  a <- getDescriptionStatsBy(fake_data$Large,
                             by = fake_data$Small)
  b <- getDescriptionStatsBy(fake_data$Large,
                             by = fake_data$Small_factor)
  attr(a, "raw_data") <- NULL
  attr(b, "raw_data") <- NULL
  expect_identical(a, b)
})

test_that("Check factor function", {
  stats <- table(Loblolly$fvar, Loblolly$young)
  a <- getDescriptionStatsBy(Loblolly$fvar,
                             Loblolly$young,
                             statistics = TRUE,
                             digits = 2,
                             statistics.sig_lim = 10^-4)
  # Check that it contains the true mean
  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], as.character(stats[rn, cn]),
        info = "Factor count don't match"
      )
    }
  }

  # Check that character input is handled similarly
  b <- getDescriptionStatsBy(as.character(Loblolly$fvar),
                             as.character(Loblolly$young),
                             statistics = TRUE,
                             digits = 2,
                             statistics.sig_lim = 10^-4)
  label(a) <- ""
  label(b) <- ""
  for (colname in colnames(b)) {
    expect_equal(a[,colname], b[, colname])
  }
  vertical_perc_stats <- format(apply(stats, 2, function(x) {
    x / sum(x) * 100
  }), nsmall = 2, digits = 2)
  horizontal_perc_stats <- t(format(apply(stats, 1, function(x) {
    x / sum(x) * 100
  }), nsmall = 2, digits = 2))
  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], sprintf("%s%%", vertical_perc_stats[rn, cn]),
        info = "Factor percentagess don't match in vertical mode"
      )
    }
  }

  a <- getDescriptionStatsBy(Loblolly$fvar,
                             Loblolly$young,
                             hrzl_prop = TRUE,
                             statistics = TRUE,
                             digits = 2,
                             statistics.sig_lim = 10^-4)
  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], sprintf("%s%%", horizontal_perc_stats[rn, cn]),
        info = "Factor percentagess don't match in horizontal mode"
      )
    }
  }
})

test_that("Check total column position", {
  a <- getDescriptionStatsBy(Loblolly$fvar,
                             Loblolly$young,
                             hrzl_prop = TRUE, add_total_col = TRUE,
                             continuous_fn = describeMedian,
                             statistics = TRUE,
                             digits = 2, statistics.sig_lim = 10^-4)
  expect_equivalent(colnames(a)[1], "Total")
  expect_equivalent(ncol(a), 4)

  a <- getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
    hrzl_prop = TRUE,
    add_total_col = "last",
    continuous_fn = describeMedian,
    digits = 2, statistics.sig_lim = 10^-4
  )
  expect_equivalent(tail(colnames(a), 1), "Total",
    info = "The last column without statistics should be the total column when the add_total_col is set to last"
  )
  expect_equivalent(ncol(a), 3)

  a <- getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
    statistics = TRUE,
    hrzl_prop = TRUE,
    add_total_col = "last"
  )
  expect_equivalent(tail(colnames(a), 2)[1], "Total",
    info = "The last should be the p-value if statistics is specified"
  )
  expect_equivalent(ncol(a), 4)
})

test_that("Check factor function with missing", {
  stats <- table(Loblolly$fvar, Loblolly$young_w_missing, useNA = "ifany")
  a <- expect_warning(getDescriptionStatsBy(Loblolly$fvar,
                                            Loblolly$young_w_missing,
                                            statistics = TRUE,
                                            digits = 2,
                                            statistics.sig_lim = 10^-4
  ))

  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], as.character(stats[rn, cn]),
        info = "Factor count don't match"
      )
    }
  }


  stats <- table(Loblolly$fvar, Loblolly$young_w_missing, useNA = "no")
  vertical_perc_stats <-
    format(apply(stats, 2, function(x) {
      x / sum(x) * 100
    }), nsmall = 2, digits = 2)
  horizontal_perc_stats <-
    t(format(apply(stats, 1, function(x) {
      x / sum(x) * 100
    }), nsmall = 2, digits = 2))

  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], sprintf("%s%%", vertical_perc_stats[rn, cn]),
        info = "Factor vertical percentages don't match"
      )
    }
  }

  a <- suppressWarnings(getDescriptionStatsBy(Loblolly$fvar,
                                              Loblolly$young_w_missing,
                                              hrzl_prop = TRUE,
                                              statistics = TRUE,
                                              digits = 2,
                                              statistics.sig_lim = 10^-4))

  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], sprintf("%s%%", horizontal_perc_stats[rn, cn]),
        info = "Factor percentages don't match in horizontal mode"
      )
    }
  }

  a <- suppressWarnings(getDescriptionStatsBy(Loblolly$fvar_w_missing,
                                              Loblolly$young_w_missing,
                                              useNA = "no",
                                              digits = 2,
                                              statistics.sig_lim = 10^-4))
  stats <- table(Loblolly$fvar_w_missing, Loblolly$young_w_missing, useNA = "no")
  vertical_perc_stats <-
    format(apply(stats, 2, function(x) {
      x / sum(x) * 100
    }), nsmall = 2, digits = 2)

  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], as.character(stats[rn, cn]),
        info = sprintf(
          "Factor '%s':'%s' count don't match",
          rn, cn
        )
      )
      expect_match(a[rn, cn], sprintf("%s%%", vertical_perc_stats[rn, cn]),
        info = sprintf(
          "Factor '%s':'%s' vertical percentages don't match",
          rn, cn
        )
      )
    }
  }

  a <- suppressWarnings(getDescriptionStatsBy(Loblolly$fvar_w_missing,
                                              Loblolly$young_w_missing,
                                              digits = 2,
                                              statistics.sig_lim = 10^-4))
  stats <- table(Loblolly$fvar_w_missing, Loblolly$young_w_missing, useNA = "ifany")
  stats <- stats[, !is.na(colnames(stats))]
  rownames(stats)[is.na(rownames(stats))] <- "Missing"
  vertical_perc_stats <-
    format(apply(stats, 2, function(x) {
      x / sum(x) * 100
    }), nsmall = 2, digits = 2)
  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], as.character(stats[rn, cn]),
        info = sprintf(
          "Factor '%s':'%s' count don't match",
          rn, cn
        )
      )
      expect_match(a[rn, cn], sprintf("%s%%", str_trim(vertical_perc_stats[rn, cn])),
        info = sprintf(
          "Factor '%s':'%s' vertical percentages don't match",
          rn, cn
        )
      )
    }
  }

  a <- suppressWarnings(getDescriptionStatsBy(Loblolly$fvar_w_missing,
                                              Loblolly$young_w_missing,
                                              hrzl_prop = TRUE,
                                              digits = 2,
                                              statistics.sig_lim = 10^-4))
  horizontal_perc_stats <-
    t(format(apply(stats, 1, function(x) {
      x / sum(x) * 100
    }), nsmall = 2, digits = 2))


  for (rn in rownames(a)) {
    for (cn in levels(Loblolly$young)) {
      expect_match(a[rn, cn], sprintf("%s%%", str_trim(horizontal_perc_stats[rn, cn])),
        info = "Factor vertical percentages don't match"
      )
    }
  }

  # When
  # - `x` has exactly 2 levels and some NAs
  # - add_total_col = TRUE
  # - show_missing = "no"
  # - show_all_values = FALSE
  # Then prGetStatistics should return the count of just the first factor level
  # use example:
  #   a <- getDescriptionStatsBy(Loblolly$young_w_missing, Loblolly$fvar,
  #                              useNA = "no", digits = 2,
  #                              add_total_col = TRUE)
  a <- prGetStatistics(Loblolly$young_w_missing,
                       useNA = "no",
                       show_all_values = FALSE)
  lvl <- levels(Loblolly$young_w_missing)[1]
  target <- sum(stats[, lvl])
  names(target) <- lvl
  expect_equal(a, target)
})

test_that("Check txtInt application", {
  # Check for factors
  set.seed(10)
  test_var <- factor(sample(1:3, size = 10^4, replace = TRUE))
  out <- prGetStatistics(test_var,
                         useNA = "no",
                         show_all_values = FALSE)
  expect_true(any(grepl(",", out)))
})

test_that("Problem with boolean x", {
  set.seed(1)
  aa <- factor(sample(c("No", "Yes"), size = 50, replace = TRUE))
  aaa <- sample(c(TRUE, FALSE), size = 50, replace = TRUE)
  ret <- getDescriptionStatsBy(x = aaa, by = aa, numbers_first = TRUE)
  expect_equivalent(nrow(ret), 1,
    info = "There should only be one alternative returned"
  )
  expect_equivalent(ncol(ret), 2,
    info = "There should be two columns"
  )
  expect_match(ret[TRUE, "No"], sprintf("^%d", table(aaa, aa)["TRUE", "No"]),
    info = "The value does not seem to match the raw table"
  )
})

test_that("test header", {
  data(mtcars)

  mtcars$am <- factor(mtcars$am, levels = 0:1, labels = c("Automatic", "Manual"))
  Hmisc::label(mtcars$am) <- "Transmission"
  set.seed(666)
  mtcars$col <- factor(sample(c("red", "black", "silver"),
    size = NROW(mtcars), replace = TRUE
  ))

  out <- getDescriptionStatsBy(
    x = mtcars$col,
    by = mtcars$am,
    header_count = TRUE,
    add_total_col = TRUE,
    statistics = TRUE
  )
  expect_match(
    colnames(out)[1],
    sprintf("No. %d", nrow(mtcars))
  )
  expect_match(
    colnames(out)[2],
    sprintf(
      "No. %d",
      sum(mtcars$am == levels(mtcars$am)[1])
    )
  )
  expect_match(
    colnames(out)[3],
    sprintf(
      "No. %d",
      sum(mtcars$am != levels(mtcars$am)[1])
    )
  )


  out <- getDescriptionStatsBy(
    x = mtcars$col,
    by = mtcars$am,
    header_count = "(n = %s)", # custom header
    add_total_col = TRUE,
    statistics = TRUE
  )

  expect_match(
    colnames(out)[1],
    sprintf("\\(n = %d\\)", nrow(mtcars))
  )
  expect_match(
    colnames(out)[2],
    sprintf(
      "\\(n = %d\\)",
      sum(mtcars$am == levels(mtcars$am)[1])
    )
  )
  expect_match(
    colnames(out)[3],
    sprintf(
      "\\(n = %d\\)",
      sum(mtcars$am != levels(mtcars$am)[1])
    )
  )
})

test_that("Test use_units", {
  data(mtcars)

  mtcars$am <- factor(mtcars$am, levels = 0:1, labels = c("Automatic", "Manual"))
  Hmisc::label(mtcars$am) <- "Transmission"
  set.seed(666)
  units(mtcars$mpg) <- "mpg"

  out1 <- suppressWarnings(
    getDescriptionStatsBy(
      x = mtcars$mpg,
      by = mtcars$am,
      header_count = TRUE,
      add_total_col = TRUE,
      statistics = TRUE
    )
  )

  out2 <- suppressWarnings(
    getDescriptionStatsBy(
      x = mtcars$mpg,
      by = mtcars$am,
      use_units = TRUE,
      header_count = TRUE,
      add_total_col = TRUE,
      statistics = TRUE
    )
  )

  expect_equal(ncol(out1) + 1, ncol(out2))

  out3 <- suppressWarnings(
    getDescriptionStatsBy(
      x = mtcars$mpg,
      by = mtcars$am,
      use_units = "name",
      header_count = TRUE,
      add_total_col = TRUE,
      statistics = TRUE
    )
  )
  expect_match(Hmisc::label(out3), "\\(mpg\\)")
  expect_false(grepl("\\(mpg\\)", label(out2)))
})

data("mtcars")
test_that("Non-factor variables where values are missing in only one of the by-groups", {
  table(as.character(mtcars$am), mtcars$gear)
  retAll <- getDescriptionStatsBy(
    as.character(mtcars$am), mtcars$gear,
    show_all_values = TRUE
  )
  retOne <- getDescriptionStatsBy(
    as.character(mtcars$am), mtcars$gear,
    show_all_values = FALSE
  )

  retAllRowDefault <- retAll["0", ]
  retAllRowOther <- retAll["1", ]
  # Delete label as the
  attributes(retAllRowDefault) <- NULL
  attributes(retAllRowOther) <- NULL
  attributes(retOne) <- NULL
  expect_equal(retAllRowDefault, retOne)
  expect_false(all(retAllRowOther == retOne))
})

### checks for issue #32: display of p-values for multi-row summaries
cars_missing <- mtcars
cars_missing$mpg[3] <- NA
test_that("p-values are displayed in multi-row summaries when rgroup and n.rgroup are specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "19.7 (&plusmn;1.5)",
      "0 (0%)", "15.1 (&plusmn;2.6)", "0 (0%)", "", ""
    ),
    .Dim = c(2L, 4L),
    .Dimnames = list(
      c("Mean (SD)", "Missing"),
      c("4", "6", "8", "P-value")
    ),
    rgroup = structure("Gas",
      add = structure(list(`1` = "&lt; 0.0001"),
        .Names = "1"
      )
    ),
    n.rgroup = 2,
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(getDescriptionStatsBy(
    x = cars_missing$mpg,
    by = cars_missing$cyl,
    statistics = TRUE
  ),
  htmlTable_args = list(rgroup = c("Gas"), n.rgroup = 2)
  )
  expect_equivalent(out, expected)
})


test_that("Factors with 0 observations should be reported when show_all_values=TRUE, issue #61", {
  set.seed(1)
  n = 20
  d <- data.frame(single = sample(LETTERS[1], size = n, replace = TRUE),
                  two = sample(LETTERS[1:2], size = n, replace = TRUE),
                  three = sample(LETTERS[1:3], size = n, replace = TRUE),
                  by = sample(letters[1:2], size = n, replace = TRUE)) %>%
    mutate(across(everything(), function(x) factor(x, levels = c(unique(x), "no observation"))))

  out <- d %>%
    getDescriptionStatsBy(single,
                          two,
                          three,
                          by = by,
                          show_all_values = TRUE)
  expect_true(all(sapply(out, function(x) "no observation" %in% rownames(x))))

  out <- d %>%
    getDescriptionStatsBy(single,
                          two,
                          three,
                          by = by,
                          show_all_values = FALSE)
  expect_false("no observation" %in% rownames(out[[1]]))
  expect_true(all(sapply(out[2:3], function(x) "no observation" %in% rownames(x))))
})

library("testthat")
library("stringr")
context("descriptionStats")

data("Loblolly")

set.seed(1)
Loblolly$young <- Loblolly$age < 10
Loblolly$young <- factor(Loblolly$young, label = c("Yes", "No"))
Loblolly$fvar <- factor(sample(letters[1:3], size = nrow(Loblolly), replace = TRUE))
Loblolly$young_w_missing <- Loblolly$young
Loblolly$young_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA
Loblolly$fvar_w_missing <- Loblolly$fvar
Loblolly$fvar_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA
Loblolly$height_w_missing <- Loblolly$height
Loblolly$height_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA

test_that("Check describeMean", {
  expect_match(
    describeMean(Loblolly$height, html = TRUE, digits = 1),
    sprintf("^%.1f", mean(Loblolly$height))
  )
  expect_match(
    describeMean(Loblolly$height, html = TRUE, digits = 2),
    sprintf("^%.2f", mean(Loblolly$height))
  )
  expect_match(
    describeMean(Loblolly$height_w_missing, html = TRUE, digits = 2, useNA = "no"),
    sprintf("^%.2f", mean(Loblolly$height_w_missing, na.rm = TRUE))
  )

  dm <- describeMean(Loblolly$height_w_missing, html = TRUE, digits = 2)
  expect_match(
    dm[1],
    sprintf("^%.2f", mean(Loblolly$height_w_missing, na.rm = TRUE))
  )
  expect_match(
    dm[2],
    sprintf(
      "^%d",
      sum(is.na(Loblolly$height_w_missing))
    )
  )
  expect_match(
    dm[2],
    sprintf(
      "\\(%.2f%%\\)",
      mean(is.na(Loblolly$height_w_missing)) * 100
    )
  )

  dm <- describeMean(Loblolly$height_w_missing,
    html = TRUE,
    digits = 2, useNA.digits = 1
  )
  expect_match(
    dm[1],
    sprintf("^%.2f", mean(Loblolly$height_w_missing, na.rm = TRUE))
  )
  expect_match(
    dm[2],
    sprintf(
      "^%d",
      sum(is.na(Loblolly$height_w_missing))
    )
  )
  expect_match(
    dm[2],
    sprintf(
      "\\(%.1f%%\\)",
      mean(is.na(Loblolly$height_w_missing)) * 100
    )
  )
})

test_that("Check describeMedian", {
  expect_match(
    describeMedian(Loblolly$height, html = TRUE, digits = 1),
    sprintf("^%.1f", median(Loblolly$height))
  )
  expect_match(
    describeMedian(Loblolly$height, html = TRUE, digits = 2),
    sprintf("^%.2f", median(Loblolly$height))
  )
  expect_match(
    describeMedian(Loblolly$height_w_missing, html = TRUE, digits = 2, useNA = "no"),
    sprintf("^%.2f", median(Loblolly$height_w_missing, na.rm = TRUE))
  )

  dm <- describeMedian(Loblolly$height_w_missing,
    html = TRUE,
    digits = 2
  )
  expect_match(
    dm[1],
    sprintf("^%.2f", median(Loblolly$height_w_missing, na.rm = TRUE))
  )
  expect_match(
    dm[2],
    sprintf(
      "^%d",
      sum(is.na(Loblolly$height_w_missing))
    )
  )
  expect_match(
    dm[2],
    sprintf(
      "\\(%.2f%%\\)",
      mean(is.na(Loblolly$height_w_missing)) * 100
    )
  )

  dm <- describeMedian(Loblolly$height_w_missing,
    html = TRUE,
    digits = 2, useNA.digits = 1
  )
  expect_match(
    dm[1],
    sprintf("^%.2f", median(Loblolly$height_w_missing, na.rm = TRUE))
  )
  expect_match(
    dm[2],
    sprintf(
      "^%d",
      sum(is.na(Loblolly$height_w_missing))
    )
  )
  expect_match(
    dm[2],
    sprintf(
      "\\(%.1f%%\\)",
      mean(is.na(Loblolly$height_w_missing)) * 100
    )
  )
})

test_that("Check describeFactors", {
  d_f <- describeFactors(Loblolly$fvar, html = TRUE, digits = 1)
  t_f <- table(Loblolly$fvar)
  for (n in names(t_f)) {
    expect_match(
      d_f[n, ],
      sprintf("^%d", t_f[n])
    )
    expect_match(
      d_f[n, ],
      sprintf("\\(%.1f%%\\)", prop.table(t_f)[n] * 100)
    )
  }

  d_f <- describeFactors(Loblolly$fvar, html = TRUE, digits = 1, number_first = FALSE)
  for (n in names(t_f)) {
    expect_match(
      d_f[n, ],
      sprintf("\\(%d\\)", t_f[n])
    )
    expect_match(
      d_f[n, ],
      sprintf("^%.1f%%", prop.table(t_f)[n] * 100)
    )
  }

  d_f <- describeFactors(Loblolly$fvar_w_missing, html = TRUE, digits = 1)
  t_f <- table(Loblolly$fvar_w_missing, useNA = "ifany")
  for (n in names(t_f)) {
    if (is.na(n)) {
      row_no_df <- which(rownames(d_f) == "Missing")
      row_no_tf <- which(is.na(names(t_f)))
    } else {
      row_no_df <- which(rownames(d_f) == n)
      row_no_tf <- which(names(t_f) == n)
    }
    expect_match(
      d_f[row_no_df, ],
      sprintf("^%d", t_f[row_no_tf])
    )
    expect_match(
      d_f[row_no_df, ],
      sprintf("\\(%.1f%%\\)", prop.table(t_f)[row_no_tf] * 100)
    )
  }

  d_f <- describeFactors(Loblolly$fvar_w_missing,
    html = TRUE, digits = 2,
    useNA.digits = 1
  )
  t_f <- table(Loblolly$fvar_w_missing, useNA = "ifany")
  for (n in names(t_f)) {
    if (is.na(n)) {
      row_no_df <- which(rownames(d_f) == "Missing")
      row_no_tf <- which(is.na(names(t_f)))
      prop <- sprintf("%.1f", prop.table(t_f)[is.na(names(t_f))] * 100)
    } else {
      row_no_df <- which(rownames(d_f) == n)
      row_no_tf <- which(names(t_f) == n)
      prop <- sprintf("%.2f", prop.table(t_f)[which(names(t_f) == n)] * 100)
    }
    expect_match(
      d_f[row_no_df, ],
      sprintf("^%d", t_f[row_no_tf])
    )
    expect_match(
      d_f[row_no_df, ],
      sprintf("\\(%s%%\\)", prop)
    )
  }
})

test_that("Check describeProp", {
  d_f <- describeProp(Loblolly$young, html = TRUE, digits = 1)
  expect_equal(length(d_f), 1)

  t_f <- table(Loblolly$young)
  expect_match(
    d_f,
    sprintf("^%d", t_f["Yes"])
  )
  expect_match(
    d_f,
    sprintf("\\(%.1f%%\\)", prop.table(t_f)["Yes"] * 100)
  )

  # Should default to factor function if there are misssing
  d_f <- describeProp(Loblolly$young_w_missing, html = TRUE, digits = 1)
  expect_equal(length(d_f), 3)
})

#' Add a total column to the results
#'
#' @param results The results that we want to add the column to
#'
#' @return results with added column
#' @inheritParams getDescriptionStatsBy
prAddTotalDescColumn <- function(results,
                                 x,
                                 by,
                                 numbers_first,
                                 total_col_show_perc,
                                 show_all_values,
                                 useNA,
                                 useNA.digits,
                                 html,
                                 digits,
                                 continuous_fn,
                                 factor_fn,
                                 prop_fn,
                                 percentage_sign,
                                 default_ref,
                                 header_count = NULL,
                                 add_total_col) {
  org <- results
  column_names <- attr(results, "column_names")
  total_table <- prGetStatistics(x[is.na(by) == FALSE],
                                 numbers_first = numbers_first,
                                 show_perc = total_col_show_perc,
                                 show_all_values = show_all_values,
                                 useNA = useNA,
                                 useNA.digits = useNA.digits,
                                 html = html,
                                 digits = digits,
                                 continuous_fn = continuous_fn,
                                 factor_fn = factor_fn,
                                 prop_fn = prop_fn,
                                 percentage_sign = percentage_sign,
                                 default_ref = default_ref)

  if (!is.matrix(total_table)) {
    total_table <- matrix(total_table, ncol = 1, dimnames = list(names(total_table)))
  }

  if (nrow(total_table) != nrow(results)) {
    stop(
      "There is a discrepancy in the number of rows in the total table",
      " and the by results: ", nrow(total_table), " total vs ", nrow(results), " results",
      "\n Rows total:", paste(rownames(total_table), collapse = ", "),
      "\n Rows results:", paste(rownames(results), collapse = ", ")
    )
  }

  cn_tot <- prGetDescHeader(by = c(Total = length(x[is.na(by) == FALSE])),
                            header_count = header_count,
                            html = html,
                            already_table_format = TRUE)
  if (add_total_col != "last") {
    results <- cbind(total_table, results)
    column_names <- c(cn_tot, column_names)
  } else {
    results <- cbind(results, total_table)
    column_names <- c(column_names, cn_tot)
  }

  org %>%
    copyAllNewAttributes(results) %>%
    structure(column_names = column_names)
}

prConvertListToBoxList <- function(x) {
  if (inherits(x, "box")) {
    x <- list(x)
  }

  if (!is.list(x)) {
    stop("Input must be a list of boxes.")
  }

  # Validate recursively
  validate_structure <- function(el, path_prefix = "") {
    if (inherits(el, "box")) {
      return(NULL)
    }

    if (is.list(el)) {
      invalid_found <- lapply(seq_along(el), function(i) {
        nm <- names(el)[i]
        current_label <- if (!is.null(nm) && nzchar(nm)) nm else i

        # Construct path for error message
        current_path <- if (nzchar(path_prefix)) {
          paste0(path_prefix, "$", current_label)
        } else {
          as.character(current_label)
        }

        validate_structure(el[[i]], current_path)
      })
      return(unlist(invalid_found))
    }

    # If not a box and not a list, it is invalid. Return the path to this element.
    return(path_prefix)
  }

  invalid_elements <- validate_structure(x)

  if (length(invalid_elements) > 0) {
    stop(
      "The following elements are not valid boxes or lists of boxes: ",
      paste(invalid_elements, collapse = ", ")
    )
  }

  if (!inherits(x, "Gmisc_list_of_boxes")) {
    x <- prExtendClass(x, "Gmisc_list_of_boxes")
  }
  x
}

#' Helper to [getDescriptionStatsBy()]
#'
#' @inheritParams getDescriptionStatsBy
#' @param name The name of the row
#'
#' @return A [base::by] list
prPropDescs <- function(x,
                        by,
                        name,
                        default_ref,
                        prop_fn,
                        html,
                        digits,
                        digits.nonzero,
                        numbers_first,
                        useNA,
                        useNA.digits,
                        percentage_sign,
                        missing_value,
                        names_of_missing,
                        NEJMstyle) {
  default_ref <- prDescGetAndValidateDefaultRef(x, default_ref)

  t <- by(x,
          by,
          FUN = prop_fn,
          html = html,
          digits = digits,
          digits.nonzero = digits.nonzero,
          number_first = numbers_first,
          useNA = useNA,
          useNA.digits = useNA.digits,
          default_ref = default_ref,
          percentage_sign = percentage_sign
  )

  missing_t <- sapply(t, is.null)
  if (any(missing_t)) {
    substitute_t <- rep(missing_value, length(t[!missing_t][[1]]))
    names(substitute_t) <- names(t[!missing_t][[1]])
    for (i in seq_along(t[missing_t])) {
      t[missing_t][[i]] <- substitute_t
    }
  }

  if (all(unlist(sapply(t, is.na))) & !is.null(names_of_missing)) {
    substitute_t <- rep(missing_value, length(names_of_missing))
    names(substitute_t) <- names_of_missing
    substitute_list <- vector("list", length = length(t))
    names(substitute_list) <- names(t)
    for (i in seq_along(substitute_list)) {
      substitute_list[[i]] <- substitute_t
    }
    t <- substitute_list
  }

  # Check that we're dealing with only one row
  if (unique(sapply(t, length)) == 1) {
    if (is.factor(x)) {
      factor_name <- levels(x)[default_ref]
    } else {
      factor_name <- levels(factor(x))[default_ref]
    }
    # Set the rowname to a special format
    # if there was missing and this is an matrix
    # then we should avoid using this format
    name <- glue("{capitalize(factor_name)} {tolower(name)}")
  }

  if (NEJMstyle) {
    # LaTeX needs an escape before %
    # or it marks the rest of the line as
    # a comment. This is not an issue with
    # html (markdown)
    percent_sign <- ifelse(html, "%", "\\%")

    if (numbers_first) {
      name <- glue("{name} - no ({percent_sign})")
    } else {
      name <- glue("{name} - {percent_sign} (no)")
    }
  }

  # If this is the only row then set that row to the current name
  if (length(t[[1]]) == 1) {
    names(t[[1]][1]) <- name
  }

  return(t)
}

library(Gmisc)
library(grid)

# Example 1: Simple header + body
grid.newpage()
box1 <- boxHeaderGrob(
    header = "Early rehabilitation",
    body = paste("0-1 weeks", "• Instruction", "• Pendulum + assisted ROM", sep = "\n"),
    header_gp = gpar(fontsize = 11, fontface = "bold", col = "black"),
    body_gp = gpar(fontsize = 9, col = "black"),
    box_gp = gpar(fill = "#E8F5E9", col = "#2E7D32", lwd = 1.4),
    y = 0.7
)

box2 <- boxHeaderGrob(
    header = "Delayed rehabilitation",
    body = paste("0-3 weeks", "• Sling immobilisation", "• Hygiene + distal ROM", sep = "\n"),
    header_gp = gpar(fontsize = 11, fontface = "bold", col = "black"),
    body_gp = gpar(fontsize = 9, col = "black"),
    box_gp = gpar(fill = "#FFF8E1", col = "#EF6C00", lwd = 1.4),
    y = 0.3
)

grid.draw(box1)
grid.draw(box2)
connectGrob(box1, box2, type = "vertical")

# Example 2: Works seamlessly with spread and align
grid.newpage()
boxes <- list(
    early = boxHeaderGrob(
        "Early rehab",
        "• Week 1-2: instruction\n• Week 2-6: active ROM",
        box_gp = gpar(fill = "#E8F5E9", col = "#2E7D32")
    ),
    late = boxHeaderGrob(
        "Delayed rehab",
        "• Week 0-3: immobilisation\n• Week 3-6: active ROM",
        box_gp = gpar(fill = "#FFF8E1", col = "#EF6C00")
    ),
    followup = boxHeaderGrob(
        "Follow-up",
        "• 2 months\n• 1 year\n• 2 years",
        box_gp = gpar(fill = "#E3F2FD", col = "#1976D2")
    )
)

boxes <- boxes |>
    spreadVertical() |>
    print()

# Connect them
connectGrob(boxes$early, boxes$followup, type = "N")
connectGrob(boxes$late, boxes$followup, type = "N")

#' An alternative to the internal \code{do.call}
#'
#' The \code{\link[base]{do.call}} can be somewhat slow, especially when working
#' with large objects. This function is based upon the suggestions from Hadley
#' Wickham on the R mailing list.
#' Also thanks to \emph{Tommy} at StackOverflow for
#' \href{https://stackoverflow.com/questions/10022436/do-call-in-combination-with}{suggesting}
#' how to handle double and triple colon operators, \code{::}, further
#' enhancing the function.
#'
#' @inheritParams base::do.call
#'
#' @section Note:
#'
#' While the function attempts to do most of what \code{\link[base]{do.call}}
#' can it has limitations. It can currently not parse the example code from the
#' original function: \code{do.call(paste, list(as.name("A"), as.name("B")), quote = TRUE)}
#' and the functionality of \code{quote} has not been thoroughly tested.
#'
#' @example inst/examples/fastDoCall_ex.R
#' @export
fastDoCall <- function(what, args, quote = FALSE, envir = parent.frame()) {
  if (quote) {
    args <- lapply(args, enquote)
  }

  if (is.null(names(args)) ||
    is.data.frame(args)) {
    argn <- args
    args <- list()
  } else {
    # Add all the named arguments
    argn <- lapply(names(args)[names(args) != ""], as.name)
    names(argn) <- names(args)[names(args) != ""]
    # Add the unnamed arguments
    argn <- c(argn, args[names(args) == ""])
    args <- args[names(args) != ""]
  }

  if ("character" %in% class(what)) {
    if (is.character(what)) {
      fn <- strsplit(what, "[:]{2,3}")[[1]]
      what <- if (length(fn) == 1) {
        get(fn[[1]], envir = envir, mode = "function")
      } else {
        get(fn[[2]], envir = asNamespace(fn[[1]]), mode = "function")
      }
    }
    call <- as.call(c(list(what), argn))
  } else if ("function" %in% class(what)) {
    f_name <- deparse(substitute(what))
    call <- as.call(c(list(as.name(f_name)), argn))
    args[[f_name]] <- what
  } else if ("name" %in% class(what)) {
    call <- as.call(c(list(what, argn)))
  }

  eval(call,
    envir = args,
    enclos = envir
  )
}

context("Check fastDoCall")

test_that("Compatibility with do.call examples", {
  expect_equal(
    fastDoCall("complex", list(imag = 1:3)),
    do.call("complex", list(imag = 1:3))
  )

  ## if we already have a list (e.g. a data frame)
  ## we need c() to add further arguments
  tmp <- expand.grid(letters[1:2], 1:3, c("+", "-"))
  expect_equal(
    fastDoCall("paste", c(tmp, sep = "")),
    do.call("paste", c(tmp, sep = ""))
  )

  # Does not work: do.call(paste, list(as.name("A"), as.name("B")), quote = TRUE)

  ## examples of where objects will be found.
  A <- 2
  f <- function(x) x^2
  env <- new.env()
  assign("A", 10, envir = env)
  assign("f", f, envir = env)
  f <- function(x) x
  expect_equal(
    fastDoCall("f", list(A)),
    do.call("f", list(A))
  )
  expect_equal(
    fastDoCall("f", list(A), envir = env),
    do.call("f", list(A), envir = env)
  )
  expect_equal(
    fastDoCall(f, list(A), envir = env),
    do.call(f, list(A), envir = env)
  )
  expect_equal(
    fastDoCall("f", list(quote(A)), envir = env),
    do.call("f", list(quote(A)), envir = env)
  )

  expect_equal(
    fastDoCall(f, list(quote(A)), envir = env),
    do.call(f, list(quote(A)), envir = env)
  )
  expect_equal(
    fastDoCall("f", list(as.name("A")), envir = env),
    do.call("f", list(as.name("A")), envir = env)
  )

  expect_equal(
    fastDoCall("base:::range.default", list(1:4)),
    c(1, 4)
  )
  expect_equal(
    fastDoCall(base:::range.default, list(1:4)),
    do.call(base:::range.default, list(1:4))
  )

  # Test for passing a data.frame
  expect_equal(fastDoCall(order, datasets::iris),
    do.call(order, datasets::iris),
    info = "Failed to accept data.frame passed by ::"
  )
})

#' Creates coordinates for box
#'
#' @param viewport_data The arguments that will be used for generating the \code{viewport}
#' @param extra_coordinate_functions A list with named functions if we want additional
#'  parameters
#' @return \code{list} of class \code{coords}
#' @importFrom checkmate assert_list
prCreateBoxCoordinates <- function(viewport_data, extra_coordinate_functions = NULL) {
  # Adjust center depending on the viewport position
  x <- prAdjustPos(viewport_data$just, viewport_data$x, viewport_data$width, axis = "x")
  y <- prAdjustPos(viewport_data$just, viewport_data$y, viewport_data$height, axis = "y")

  height <- viewport_data$height
  if (!is.unit(height)) height <- unit(height, "npc")

  width <- viewport_data$width
  if (!is.unit(width)) width <- unit(width, "npc")

  half_height <- height * 0.5
  half_width <- width * 0.5

  coordinates <- list(
    left = x - half_width,
    right = x + half_width,
    bottom = y - half_height,
    top = y + half_height,
    x = x,
    y = y,
    width = viewport_data$width,
    height = viewport_data$height,
    half_height = half_height,
    half_width = half_width
  )

  if (!is.null(extra_coordinate_functions)) {
    assert_list(extra_coordinate_functions, names = "strict")


    available_inputs <- coordinates
    available_inputs$half_width <- half_width
    available_inputs$half_height <- half_height

    for (n in names(extra_coordinate_functions)) {
      required <- formals(extra_coordinate_functions[[n]]) %>% names()
      missing <- !(required %in% names(available_inputs))
      if (any(missing)) {
        stop(
          "The extra coordinate generated from function '", n, "'",
          " requires '", paste(required[missing], collapse = "', '"), "'",
          " but it is not among the available '", paste(sort(names(available_inputs)), collapse = "', '"), "'"
        )
      }

      args <- list()
      for (argname in required) {
        args[[argname]] <- available_inputs[[argname]]
      }
      coordinates[[n]] <- do.call(extra_coordinate_functions[[n]], args)
    }
  }

  class(coordinates) <- c("coords", class(coordinates))
  return(coordinates)
}

#' Convert the by-list into a matrix compatible format
#'
#' Helper for [getDescriptionStatsBy] that fixes empty values in matrix
#' so that they are compatible with the matrix
#'
#' @param t Output from [prNumericDescs], [prPropDescs], or [prFactorDescs].
#'
#' @inheritParams getDescriptionStatsBy
#' @return A fixed list
prAddEmptyVals <- function(t, missing_value) {
  # Convert the list into a list with vectors instead of matrices
  for (n in names(t)) {
    if (is.matrix(t[[n]])) {
      tmp_names <- rownames(t[[n]])
      t[[n]] <- as.vector(t[[n]])
      names(t[[n]]) <- tmp_names
    }
  }

  # TODO: This function does not respect the order in
  # the factored variable. This could potentially be
  # a problem although probably more theoretical
  # collect unique row names in the order they appear (union() sorts)
  all_row_names <- unique(unlist(lapply(t, names), use.names = FALSE))

  # No rownames exist, this occurs often
  # when there is only one row and that row doesn't
  # have a name
  if (is.null(all_row_names)) {
    return(t)
  }

  # The missing NA element should always be last
  if (any(is.na(all_row_names))) {
    all_row_names <- append(all_row_names[!is.na(all_row_names)], NA)
  }

  ret <- list()
  for (n in names(t)) {
    # Create an empty array
    ret[[n]] <- rep(missing_value, times = length(all_row_names))
    names(ret[[n]]) <- all_row_names
    # Loop and add all the values
    for (nn in all_row_names) {
      if (nn %in% names(t[[n]])) {
        if (is.na(nn)) {
          ret[[n]][is.na(names(ret[[n]]))] <- t[[n]][is.na(names(t[[n]]))]
        } else {
          ret[[n]][nn] <- t[[n]][nn]
        }
      }
    }
  }

  return(ret)
}

#' P-value extractors for \code{\link{getDescriptionStatsBy}}
#'
#' These functions are the base functions for getting the description
#' p-values. You can provide your own functions but all functions should
#' take two arguments and return a p-value (numeric, non-formatted)
#'
#' @param x The main variable of interest
#' @param by The variable for the stratification
#' @return \code{numeric} Returns the p-value from that particular test
#' @rdname getPval
#' @export
#' @family descriptive functions
#'
#' @section getPvalWilcox:
#'
#' Performs a two-sample two-sided Wilcoxon test (also known as the Mann-Whitney test),
#' see \code{\link[stats]{wilcox.test}}.
#'
#' @examples
#' set.seed(123)
#' getPvalFisher(
#'   sample(letters[1:3], size = 100, replace = TRUE),
#'   sample(LETTERS[1:3], size = 100, replace = TRUE)
#' )
#' getPvalWilcox(
#'   rnorm(100),
#'   sample(LETTERS[1:2], size = 100, replace = TRUE)
#' )
#' @importFrom stats wilcox.test
getPvalWilcox <- function(x, by) {
  wilcox.test(x ~ by, alternative = "two.sided")$p.value
}

#' @section getPvalAnova:
#'
#' Performs a standard Analysis of Variance model
#' through \code{\link[stats]{anova}(\link[stats]{lm}(x ~ by))}
#'
#' @rdname getPval
#' @import magrittr
#' @importFrom stats lm anova
#' @export
getPvalAnova <- function(x, by) {
  out <- lm(x ~ by) %>%
    anova()
  out[["Pr(>F)"]][[1]]
}

#' @rdname getPval
#'
#' @section getPvalFisher:
#' Performs Fisher's exact test through the \code{\link[stats]{fisher.test}}.
#'
#' @importFrom stats fisher.test
#' @export
getPvalFisher <- function(x, by) {
  # This is a quick fix in case of large dataset
  workspace <- 10^5
  if (length(x) * length(levels(x)) > 10^4) {
    workspace <- 10^7
  }

  # Large statistics tend to be very heavy and therefore
  # i need to catch errors in fisher and redo by simulation
  tryCatch({
      fisher.test(x, by, workspace = workspace)$p.value
    },
    error = function(err) {
      warning("Simulating p-value for fisher due to high computational demands on the current varible")
      fisher.test(x, by, simulate.p.value = TRUE, B = 10^5)$p.value
    }
  )
}

#' @section getPvalChiSq:
#'
#' Performs a standard Chi-Squares analysis
#' through \code{\link[stats]{chisq.test}}
#'
#' @rdname getPval
#' @importFrom stats chisq.test
#' @export
getPvalChiSq <- function(x, by) {
  chisq.test(x, by)$p.value
}


#' @section getPvalKruskal:
#'
#' Performs a  Kruskal-Wallis rank sum test
#' through \code{\link[stats]{kruskal.test}}
#'
#' @importFrom stats kruskal.test
#' @rdname getPval
#' @export
getPvalKruskal <- function(x, by) {
  kruskal.test(x, by)$p.value
}

library(testthat)
context("getPval wrappers return numeric p-values")

set.seed(42)

# numeric data
x <- rnorm(50)
by2 <- factor(sample(c("A", "B"), size = 50, replace = TRUE))
by3 <- factor(sample(c("A", "B", "C"), size = 50, replace = TRUE))

# Fisher/ChiSq require factors
xf <- sample(letters[1:3], size = 50, replace = TRUE)

test_that("Wilcoxon returns scalar between 0 and 1", {
    p <- getPvalWilcox(x, by2)
    expect_type(p, "double")
    expect_true(length(p) == 1)
    expect_true(p >= 0 && p <= 1)
})

test_that("ANOVA returns scalar between 0 and 1", {
    p <- getPvalAnova(x, by3)
    expect_type(p, "double")
    expect_true(length(p) == 1)
    expect_true(p >= 0 && p <= 1)
})

test_that("Kruskal returns scalar between 0 and 1", {
    p <- getPvalKruskal(x, by3)
    expect_type(p, "double")
    expect_true(length(p) == 1)
    expect_true(p >= 0 && p <= 1)
})

# small tables for Fisher and Chi-square

test_that("Fisher and ChiSq produce identical results on 2x2 data", {
    tblx <- sample(c("a", "b"), 100, replace = TRUE)
    tbly <- sample(c("c", "d"), 100, replace = TRUE)
    p1 <- getPvalFisher(tblx, tbly)
    p2 <- getPvalChiSq(tblx, tbly)
    expect_type(p1, "double")
    expect_type(p2, "double")
    expect_true(p1 >= 0 && p1 <= 1)
    expect_true(p2 >= 0 && p2 <= 1)
})

# error conditions

invalid <- 1:5

# by and x lengths mismatch
expect_error(getPvalWilcox(1:4, factor(1:5)))

#' Checks if unit or number 0-1
#'
#' @param var Variable 2 check
#' @param allow_missing Allow missing variables
#'
#' @return void
#' @rdname assert
#' @keywords internal
#' @importFrom checkmate assert checkClass checkNumeric
assert_unit <- function(var, allow_missing = TRUE) {
  if (missing(var) && allow_missing) {
    return()
  }

  assert(
    checkClass(var, "unit"),
    checkNumeric(var, lower = 0, upper = 1),
    .var.name = dep_var(var)
  )
}

#' Assert that it's a string or number
#'
#' @param var Varible 2 check
#' @param allow_missing Allow missing variables
#'
#' @return void
#' @rdname assert
#' @keywords internal
#' @importFrom checkmate assert checkString checkNumeric
assert_label <- function(var, allow_missing = TRUE) {
  if (missing(var) && allow_missing) {
    return()
  }

  assert(
    checkString(var),
    checkNumeric(var),
    .var.name = dep_var(var)
  )
}

#' Assert valid color
#'
#' @param var Variable 2 check
#' @param allow_missing Allow missing variables
#'
#' @return void
#' @rdname assert
#' @keywords internal
#' @importFrom checkmate assert checkCharacter checkNumeric
assert_just <- function(var, allow_missing = TRUE) {
  if (missing(var) && allow_missing) {
    return()
  }

  assert(
    checkCharacter(var, pattern = "^left|right|top|bottom|center|centre$", max.len = 2),
    checkNumeric(var, lower = 0, upper = 1, max.len = 2),
    .var.name = dep_var(var)
  )
}

dep_var <- function(x) {
  nn <- substitute(x)
  on <- do.call("substitute", list(as.name(nn), parent.frame(1)))
  paste0(deparse(deparse(on), width.cutoff = 500),
    collapse = "\n"
  )
}

#' Helper to [getDescriptionStatsBy()]
#'
#' @inheritParams getDescriptionStatsBy
#'
#' @return A [base::by] list
prFactorDescs <- function(x,
                          by,
                          factor_fn,
                          hrzl_prop,
                          html,
                          digits,
                          digits.nonzero,
                          numbers_first,
                          useNA,
                          useNA.digits,
                          percentage_sign,
                          missing_value,
                          names_of_missing) {
  if (hrzl_prop) {
    t <- by(x, by,
            FUN = factor_fn,
            html = html,
            digits = digits,
            digits.nonzero = digits.nonzero,
            number_first = numbers_first,
            useNA = useNA,
            useNA.digits = useNA.digits,
            horizontal_proportions = table(x, useNA = useNA),
            percentage_sign = percentage_sign
    )
  } else {
    t <- by(x, by,
            FUN = factor_fn,
            html = html,
            digits = digits,
            digits.nonzero = digits.nonzero,
            number_first = numbers_first,
            useNA = useNA,
            useNA.digits = useNA.digits,
            percentage_sign = percentage_sign
    )
  }
  missing_t <- sapply(t, is.null)
  if (any(missing_t)) {
    substitute_t <- rep(missing_value, length(t[!missing_t][[1]]))
    names(substitute_t) <- names(t[!missing_t][[1]])
    for (i in seq_along(t[missing_t])) {
      t[missing_t][[i]] <- substitute_t
    }
  }

  if (all(unlist(sapply(t, is.na))) & !is.null(names_of_missing)) {
    substitute_t <- rep(missing_value, length(names_of_missing))
    names(substitute_t) <- names_of_missing
    substitute_list <- vector("list", length = length(t))
    names(substitute_list) <- names(t)
    for (i in seq_along(substitute_list)) {
      substitute_list[[i]] <- substitute_t
    }
    t <- substitute_list
  }

  return(t)
}

#' Adds a figure caption number
#'
#' The function relies on \code{options("fig_caption_no")}
#' in order to keep track of the last number. If
#' you want to force the caption function to skip
#' captions while still using it in the knitr fig.cap option
#' then simply set \code{options(fig_caption_no = FALSE)}
#'
#' @param str The string that is to be prepended with string
#' @param roman Whether or not to use roman numbers instead
#'  of Arabic. Can also be set through \code{options(fig_caption_no_roman = TRUE)}
#' @param sprintf_str An \code{\link[base:sprintf]{sprintf}} formatted
#'  string where the first argument is reserved for the string
#'  generated by the counter and the second one is for the caption
#'  text. Can also be set through \code{options(fig_caption_no_sprintf = TRUE)}
#'
#' @export
#' @examples
#' \dontrun{
#' ```{r, fig.cap = pigCapNo("My nice plot")}
#' plot(1:10 + rnorm(10), 1:10)
#' ```
#' }
#' org_opts <- options(fig_caption_no = 2,
#'                     fig_caption_no_sprintf = "Figure %s: %s")
#' figCapNo("A plot with caption number = 3")
#'
#' org_opts <- options(fig_caption_no = TRUE)
#' figCapNo("A plot with caption number = 1")
#'
#' # Use default setting
#' options(fig_caption_no_sprintf = NULL)
#' figCapNo("A plot with caption number = 2")
#'
#' # Return the original settings
#' options(org_opts)
#' @family figure caption functions
#' @importFrom utils as.roman
figCapNo <- function(str,
                     roman = getOption("fig_caption_no_roman", default = FALSE),
                     sprintf_str = getOption("fig_caption_no_sprintf", default = "Fig. %s: %s")) {
  last_no <- getOption("fig_caption_no")
  if (is.logical(last_no)) {
    if (last_no == FALSE)
      return(str)

    last_no <- 0
  }else if (is.null(last_no)) {
    last_no <- 0
  }

  current_no <- last_no + 1
  options(fig_caption_no = current_no)

  if (roman)
    current_no <- as.character(as.roman(current_no))

  return(sprintf(sprintf_str,
                 current_no,
                 str))
}


#' Gets the last figure caption number
#'
#' The function relies on \code{options("fig_caption_no")}
#' in order to keep track of the last number.
#'
#' @inheritParams figCapNo
#' @export
#' @examples
#' org_opts <- options(fig_caption_no = 1)
#' figCapNoLast()
#' options(org_opts)
#' @importFrom utils as.roman
#' @family figure caption functions
figCapNoLast <- function(roman = getOption("fig_caption_no_roman", FALSE)) {
  last_no <- getOption("fig_caption_no")
  if (is.logical(last_no) ||
        is.null(last_no)) {
    stop("You cannot call the get last figure number",
         " when there has been no prior figure registered.",
         " In other words, you need to call the fiCapNo()",
         " on a figure before you call this function.",
         " If you want the next number then call figCapNoNext()",
         " instead of this function.")
  }

  if (roman)
    last_no <- as.character(as.roman(last_no))

  return(last_no)
}

#' Gets the next figure caption number
#'
#' The function relies on \code{options("fig_caption_no")}
#' in order to keep track of the last number.
#'
#' @inheritParams figCapNo
#' @export
#' @examples
#' org_opts <- options(fig_caption_no = 1)
#' figCapNoNext()
#' options(org_opts)
#' @importFrom utils as.roman
#' @family figure caption functions
figCapNoNext <- function(roman = getOption("fig_caption_no_roman", default = FALSE)) {
  last_no <- getOption("fig_caption_no")
  if (is.logical(last_no)) {
    if (last_no == FALSE)
      stop("You cannot call the get last figure number",
           " when you have explicitly set the fig_cap_no",
           " option to FALSE.")
    last_no <- 0

  }else if (is.null(last_no)) {
    last_no <- 0
  }

  next_no <- last_no + 1

  if (roman)
    next_no <- as.character(as.roman(next_no))

  return(next_no)
}

context("Figure caption numbers")

test_that("Test figure caption basic functionality",{
  options(fig_caption_no = NULL)
  expect_error(figCapNoLast())
  expect_equivalent(figCapNoNext(), 1)
  no <- figCapNo("a", roman = FALSE, sprintf_str = "%s: %s")
  expect_match(no, "1+: a")

  no <- figCapNo("a", roman = FALSE, sprintf_str = "%s: %s")
  expect_match(no, "2: a")
  
  no <- figCapNo("a", roman = TRUE, sprintf_str = "%s: %s")
  expect_match(no, "III: a")

  options(fig_caption_no = NULL)
  expect_match(figCapNo("a", roman = TRUE, sprintf_str = "%s: %s"),
               "I: a")

  expect_error(figCapNo())

  options(fig_caption_no = FALSE)
  expect_error(figCapNoNext())

  expect_equal(figCapNo("test"), "test")

  options(fig_caption_no = TRUE)
  expect_equal(figCapNoNext(), 1)
  no <- figCapNo("a", roman = FALSE, sprintf_str = "%s: %s")
  expect_equal(no, "1: a")
})

test_that("Test figure caption options",{
  options(fig_caption_no = 2)
  no <- figCapNoLast()
  expect_equivalent(no, 2)
  no <- figCapNoNext()
  expect_equivalent(no, 3)

  options(fig_caption_no_roman = TRUE)
  expect_equivalent(figCapNoLast(), "II")
  expect_equivalent(figCapNoNext(), "III")

  options(fig_caption_no = 10)
  expect_equivalent(figCapNo("test"), "Fig. XI: test")

  options(fig_caption_no_roman = FALSE)
  expect_equivalent(figCapNo("test"), "Fig. 12: test")
})

#' Connect boxes (S3)
#'
#' A convenient way to connect boxes in a `Gmisc_list_of_boxes` or
#' simple `list` context, designed for piping (`|>`).
#'
#' @param x A `list` of boxes (will be converted to `Gmisc_list_of_boxes` if needed).
#' @param from The name (string) or index of the start box in `x`. Multiple values allowed.
#' @param to The name (string) or index of the end box in `x`. Multiple values allowed.
#' @param ... Arguments passed on to [`connectGrob`].
#'
#' @return The original list `x` (upgraded to `Gmisc_list_of_boxes`) with a new
#'   connection appended to its `"connections"` attribute. When printed, these
#'   connections are drawn.
#'
#' @seealso [`connectGrob`]
#' @export
#' @family flowchart components
connect <- function(x, ...) UseMethod("connect")

#' @export
#' @rdname connect
connect.default <- function(x, ...) {
  # If it's a list (but not yet class-extended), treat it as a box list
  if (is.list(x) && !inherits(x, "box")) {
    return(connect(prConvertListToBoxList(x), ...))
  }
  stop("connect() requires a list of boxes (Gmisc_list_of_boxes).")
}

#' @export
#' @rdname connect
connect.Gmisc_list_of_boxes <- function(x, from = NULL, to = NULL, ...) {
  args <- list(...)

  if (is.null(from) || is.null(to)) {
    stop("You must provide both 'from' and 'to' arguments.")
  }

  # Helper to resolve box objects from x using names/indices
  resolve_box <- function(ref) {
    if (is.character(ref)) {
      if (all(ref %in% names(x))) {
        return(x[ref])
      }

      resolved <- lapply(ref, function(r) {
        if (r %in% names(x)) {
          return(x[[r]])
        }

        if (grepl("$", r, fixed = TRUE)) {
          path <- strsplit(r, "$", fixed = TRUE)[[1]]
          el <- get_list_element_by_path(x, path)
          if (!is.null(el)) {
            return(el)
          }
        }

        stop("Could not find box named: ", r)
      })
      return(resolved)
    } else if (is.numeric(ref)) {
      return(x[ref])
    } else if (inherits(ref, "box") || (is.list(ref) && all(vapply(ref, inherits, logical(1), "box")))) {
      # Use directly if passed as object
      return(ref)
    }
    stop("Invalid from/to selector: must be name, index, or box object.")
  }

  start_boxes <- resolve_box(from)
  end_boxes <- resolve_box(to)

  if (length(start_boxes) == 1) start_boxes <- start_boxes[[1]]
  if (length(end_boxes) == 1) end_boxes <- end_boxes[[1]]

  # Support pairwise list-to-list mapping in the S3 flowchart API.
  # This keeps connectGrob() many-to-many unsupported while making
  # connect("groups", "groups2") behave as users expect.
  if (prIsBoxList(start_boxes) && prIsBoxList(end_boxes)) {
    if (length(start_boxes) != length(end_boxes)) {
      stop("When both 'from' and 'to' resolve to lists of boxes, they must have the same length.", call. = FALSE)
    }

    cg <- mapply(function(s, e) {
      do.call(connectGrob, c(list(start = s, end = e), args))
    }, start_boxes, end_boxes, SIMPLIFY = FALSE)
    class(cg) <- c("connect_boxes_list", "list")
  } else {
    # Create the connection grob
    call_args <- c(list(start = start_boxes, end = end_boxes), args)
    cg <- do.call(connectGrob, call_args)
  }

  # Append to attributes
  current_conns <- attr(x, "connections")
  if (is.null(current_conns)) current_conns <- list()

  current_conns <- c(current_conns, list(cg))

  attr(x, "connections") <- current_conns
  x
}

library(grid)

test_that("flowchart() constructor", {
  b1 <- boxGrob("A")
  b2 <- boxGrob("B")

  # Method 1: Named arguments
  fc1 <- flowchart(a = b1, b = b2)
  expect_s3_class(fc1, "Gmisc_list_of_boxes")
  expect_equal(names(fc1), c("a", "b"))
  expect_equal(length(fc1), 2)

  # Method 2: List input
  fc2 <- flowchart(list(a = b1, b = b2))
  expect_s3_class(fc2, "Gmisc_list_of_boxes")
  expect_equal(names(fc2), c("a", "b"))
  expect_equal(length(fc2), 2)

  # Method 3: Unnamed arguments
  fc3 <- flowchart(b1, b2)
  expect_s3_class(fc3, "Gmisc_list_of_boxes")
  expect_equal(length(fc3), 2)
})

test_that("flowchart() auto-converts text to boxes", {
  fc <- flowchart("A", "B", default_box_fn = boxGrob)
  expect_s3_class(fc[[1]], "box")
  expect_s3_class(fc[[2]], "box")
  # Need to check label possibly?
  # attributes(fc[[1]])$"label" ? No, boxGrob stores things in children usually or attributes specific to implementation.
  # boxGrob stores label in `label` attribute? No.
  # But if it didn't crash and is class box, reasonable enough.
})

test_that("flowchart() handles mixed content (boxes + text + nested lists)", {
  b1 <- boxGrob("Box1")
  fc <- flowchart(
    start = b1,
    mid = "Middle",
    end = list(sub1 = "SubA", sub2 = "SubB")
  )
  expect_s3_class(fc$start, "box")
  expect_s3_class(fc$mid, "box")
  expect_type(fc$end, "list")
  expect_s3_class(fc$end$sub1, "box")
  expect_s3_class(fc$end$sub2, "box")
})


test_that("flowchart() pipeline integration", {
  b1 <- boxGrob("Start", y = 0.8)
  b2 <- boxGrob("End", y = 0.2)

  res <- flowchart(start = b1, end = b2) |>
    align(axis = "y") |>
    connect("start", "end", type = "vertical")

  expect_s3_class(res, "Gmisc_list_of_boxes")
  expect_true(!is.null(attr(res, "connections")))
})

#' A bezier arrow with gradient
#'
#' This is an experimental addition to the original \code{\link{bezierArrowSmpl}}
#' with the addition of a gradient in the center of the arrow that fades.
#'
#' @section Note:
#' The triangle section of the arrow is not currently included in the gradient.
#'
#' @inheritParams grid::bezierGrob
#' @inheritParams bezierArrowSmpl
#' @param clr The color of the arrow. This is the main color of the arrow and not the gradient color.
#' @param grdt_type The type of growth and gradient that is to be used,
#'  currently it only supports triangle (I'm considering adding bezier curves
#'  but currently I'm a little tired of coding)
#' @param grdt_prop The proportion of the full length that should be a the gradient. The gradient
#'  consists of three things:
#'  (1) the central band,
#'  (2) the slimming of the central band,
#'  (3) the color shift into the arrow color.
#'  \emph{Note} that the the slimming and color proportions can be overlapping.
#' @param grdt_decrease_prop The proportion of the gradient that should be decreasing,
#'  i.e. narrowing according to the \code{grdt_type} argument.
#' @param grdt_clr_prop The proportion of the gradient that should be converging to the arrow color.
#' @param grdt_line_width The width of the border line. If not specified it defaults to 5 \% of
#'  the original width, note the gradient's width is thus 90 \%.
#' @param grdt_clr The color of the gradient.
#' @param ... Passed on to \code{\link{bezierArrowSmpl}}
#' @return A grob of \code{\link[grid:grid.grob]{gList}}-type
#'
#' @inheritParams calculateLinesAndArrow
#' @examples
#' library(grid)
#' grid::grid.newpage()
#' arrowGrob <- bezierArrowGradient(
#'   x = c(.1, .3, .6, .9),
#'   y = c(0.2, 0.2, 0.9, 0.9)
#' )
#' grid.draw(arrowGrob)
#' @importFrom utils head tail
#' @export
bezierArrowGradient <- function(x = c(0.2, .7, .3, .9),
                                y = c(0.2, .2, .9, .9),
                                width = .05,
                                clr = "#000000",
                                default.units = "npc",
                                align_2_axis = TRUE,
                                grdt_type = c("triangle", "rectangle"),
                                grdt_prop = .8,
                                grdt_decrease_prop = .5,
                                grdt_clr_prop = .7,
                                grdt_line_width,
                                grdt_clr = "#2F4F2F",
                                vp = NULL,
                                gp = gpar(),
                                rm_intersect = 3L,
                                ...) {
  if ("grdt_start_prop" %in% names(list(...))) {
    stop(
      "The grdt_start_prop has been deprecated. Switch to grdt_prop instead.",
      "\n**Note** The grdt_prop is a total of how large proportion of the arrow",
      " should be covered with a gradient. This is different from before where",
      " the grdt_start_prop was added to the grdt_decrease_prop in order to",
      " generate this value."
    )
  }

  if (!length(rm_intersect) == 1L ||
    !is.numeric(rm_intersect) ||
    rm_intersect %% 1 != 0 ||
    rm_intersect < 0L ||
    rm_intersect > 3L) {
    stop("`rm_intersect` must be 0, 1, 2, or 3 (see ?calculateLinesAndArrow).")
  }

  # Get initial values
  grdt_type <- match.arg(grdt_type)

  # Adapt line width to the direction if default.units == "npc"
  width <- getAbsoluteWidth(
    w = width,
    default.units = default.units,
    x = x,
    y = y
  )

  if (missing(grdt_line_width)) {
    grdt_line_width <- unit(getGridVal(width, "mm") * .5, units = "mm")
  }

  grdt_line_width <- getAbsoluteWidth(
    w = grdt_line_width,
    default.units = default.units,
    x = x,
    y = y
  )

  if (!inherits(grdt_line_width, "unit")) {
    grdt_line_width <- unit(grdt_line_width, default.units)
  }

  # Sanity check for input parameters
  if (grdt_prop > 1 &&
    grdt_prop < 0) {
    stop("The gradient proportion (grdt_prop) must be between 0-1! You provided ", grdt_prop)
  }

  if (grdt_decrease_prop > 1 &&
    grdt_decrease_prop < 0) {
    stop("The decrease proportion must be between 0-1! You provided ", grdt_decrease_prop)
  }

  if (grdt_clr_prop > 1 &&
    grdt_clr_prop < 0) {
    stop("The color gradient proportion must be between 0-1! You provided ", grdt_clr_prop)
  }

  if (getGridVal(width, default.units) < getGridVal(grdt_line_width, default.units)) {
    stop("Your gradient width exceeds the width of the arrow")
  }

  # Get the pure arrow
  pg <- bezierArrowSmpl(
    x = x, y = y,
    width = width,
    default.units = default.units,
    clr = clr, align_2_axis = align_2_axis,
    gp = gp, vp = vp,
    rm_intersect = rm_intersect,
    ...
  )

  # Now to the gradient
  bp <- attr(pg, "center_points")
  end_points <- attr(pg, "end_points")

  internal.units <- "mm"
  end_points <- lapply(end_points, function(pt) {
    list(
      x = convertX(pt$x, unitTo = internal.units),
      y = convertY(pt$y, unitTo = internal.units)
    )
  })
  bp$x <- convertX(bp$x, unitTo = internal.units, valueOnly = TRUE)
  bp$y <- convertY(bp$y, unitTo = internal.units, valueOnly = TRUE)
  # Extend to the full line
  bp$x <- c(bp$x, end_points$end$x)
  bp$y <- c(bp$y, end_points$end$y)

  grdt_line_width <- convertX(grdt_line_width, unitTo = internal.units, valueOnly = TRUE)

  # Calculate the lengths
  lenCalc <- function(bp) {
    bp$lengths <-
      c(
        0,
        with(
          bp,
          mapply(
            x1 = x[-length(x)],
            y1 = y[-length(y)],
            x2 = x[-1],
            y2 = y[-1],
            function(x1, y1, x2, y2) sqrt((x2 - x1)^2 + (y2 - y1)^2)
          )
        )
      )
    bp$cumlen <- cumsum(bp$lengths)
    return(bp)
  }
  bp <- lenCalc(bp)

  grdt_length <- sum(bp$lengths) * (grdt_prop)
  end_point <-
    (bp$cumlen - grdt_length) %>%
    abs() %>%
    which.min()

  # Adjust last point if discrepancy > 10%
  if (abs(bp$cumlen[end_point] - grdt_length) > bp$length[end_point] / 10) {
    if (bp$cumlen[end_point] - grdt_length < 0 &&
      end_point <= length(bp$x)) {
      dx <- bp$x[end_point + 1] - bp$x[end_point]
      dy <- bp$y[end_point + 1] - bp$y[end_point]
      adj_prop <- (grdt_length - bp$cumlen[end_point]) / bp$length[end_point + 1]
    } else {
      dx <- bp$x[end_point] - bp$x[end_point - 1]
      dy <- bp$y[end_point] - bp$y[end_point - 1]
      adj_prop <- (grdt_length - bp$cumlen[end_point]) / bp$length[end_point]
    }

    # Desired length vs actual length
    bp$x[end_point] <- bp$x[end_point] +
      dx * adj_prop
    bp$y[end_point] <- bp$y[end_point] +
      dy * adj_prop
    # Adjust the lengths
    bp$lengths[end_point] <- abs(bp$cumlen[end_point] - grdt_length)
    bp$cumlen[end_point] <- grdt_length

    # Shorten to the end
    bp <- lapply(bp, function(x) head(x, n = end_point))

    # If the line was extended with a length beyond the regular
    # length we should split the last element into sub-elements
    avg_len <- mean(bp$lengths[-end_point])
    if (tail(bp$lengths, 1L) > avg_len * 2) {
      no_splits <- round(tail(bp$lengths, 1L) / avg_len)
      last_2_pt <- lapply(bp, tail, n = 2L)
      bp <- lapply(bp, head, n = length(bp$x) - 2)

      spltVals <- function(vals, tail_vals, no_splits) {
        c(
          vals,
          seq(
            from = tail_vals[1],
            to = tail(tail_vals, 1),
            length.out = no_splits
          )
        )
      }
      bp$x <- spltVals(bp$x, last_2_pt$x, no_splits)
      bp$y <- spltVals(bp$y, last_2_pt$y, no_splits)
      bp <- lenCalc(bp)
    }
  } else {
    # Shorten to the end
    bp <- lapply(bp, function(x) head(x, end_point))
  }

  # Add start margin
  margin <- grdt_line_width / 2
  line_start <-
    (bp$cumlen - margin) %>%
    abs() %>%
    which.min()

  if (bp$cumlen[line_start] - margin > 0) {
    line_start <- line_start - 1
  }

  # Remove start
  if (line_start > 1) {
    bp <- lapply(bp, function(x) x[-1 * (1:line_start - 1)])
    if (bp$cumlen[1] != margin) {
      dx <- bp$x[2] - bp$x[1]
      dy <- bp$y[2] - bp$y[1]

      adj_prop <- (bp$cumlen[2] - margin) / bp$length[2]

      bp$x[1] <- bp$x[1] +
        dx * adj_prop
      bp$y[1] <- bp$y[1] +
        dy * adj_prop
    }

    bp <- lenCalc(bp)
  }

  # Defensive cleanup: ensure bp$x and bp$y are numeric and free of NAs
  bp$x <- as.numeric(bp$x)
  bp$y <- as.numeric(bp$y)
  valid_idx <- which(!is.na(bp$x) & !is.na(bp$y))
  if (length(valid_idx) < 2) {
    warning("Gradient aborted: insufficient valid bezier points after cleanup")
    return(gList(pg))
  }
  if (length(valid_idx) < length(bp$x)) {
    bp <- lapply(bp, function(v) v[valid_idx])
    bp <- lenCalc(bp)
  }


  ########################
  # Get the decrease and #
  # color start points   #
  ########################
  start_decrease <-
    (bp$cumlen - tail(bp$cumlen, 1) * (1 - grdt_decrease_prop)) %>%
    abs() %>%
    which.min()

  clr_start <-
    (bp$cumlen - tail(bp$cumlen, 1) * (1 - grdt_clr_prop)) %>%
    abs() %>%
    which.min() %>%
    +1


  # The offset decrease should be relative
  offset <- rep(grdt_line_width / 2, length(bp$x))
  if (grdt_type == "triangle") {
    decr_lengths <- tail(bp$lengths, length(offset) - start_decrease)
    offset[start_decrease:length(offset)] <-
      offset[start_decrease:length(offset)] *
        (1 - c(0, cumsum(decr_lengths)) / sum(decr_lengths))
  }

  grdnt_lines <- calculateLinesAndArrow(x = bp$x, y = bp$y, offset = offset, rm_intersect = rm_intersect)
  if (attr(pg, "axis") != FALSE) {
    grdnt_lines <- align2Axis(
      bp = bp,
      lines = grdnt_lines,
      width = grdt_line_width,
      internal.units = internal.units,
      axis = attr(pg, "axis")
    )
  }

  # Prepare for the correct size
  grdnt_lines$left$x <- convertX(unit(grdnt_lines$left$x, units = internal.units), unitTo = default.units)
  grdnt_lines$right$x <- convertX(unit(grdnt_lines$right$x, units = internal.units), unitTo = default.units)
  grdnt_lines$left$y <- convertY(unit(grdnt_lines$left$y, units = internal.units), unitTo = default.units)
  grdnt_lines$right$y <- convertY(unit(grdnt_lines$right$y, units = internal.units), unitTo = default.units)

  inner_gradient <- gList()
  if (clr_start > 2) {
    reg_size <-
      lapply(
        grdnt_lines,
        function(side) {
          lapply(
            side,
            function(x) head(x, clr_start - 1)
          )
        }
      )
    gradient_pg <-
      with(
        reg_size,
        polygonGrob(
          x = c(left$x, rev(right$x)),
          y = c(left$y, rev(right$y)),
          gp = gpar(fill = grdt_clr, col = grdt_clr)
        )
      )

    inner_gradient <- gList(inner_gradient, gradient_pg)
  }

  # Plot the gradient color - these need to be plotted as
  # one color per polygon
  if (length(bp$x) >= clr_start) {
    elmnts <- (clr_start - 1):(length(grdnt_lines$right$x) - 1)
    g_clrs <- colorRampPalette(colors = c(grdt_clr, clr))(length(elmnts))
    # Speeds signifacantly the process
    decr_polygons <- list()
    for (i in elmnts) {
      col <- g_clrs[i - clr_start + 2]
      x <- with(grdnt_lines, c(left$x[i:(i + 1)], right$x[(i + 1):i]))
      y <- with(grdnt_lines, c(left$y[i:(i + 1)], right$y[(i + 1):i]))
      decr_polygons[[i - clr_start + 2]] <-
        polygonGrob(
          x = x,
          y = y,
          gp = gpar(fill = col, col = col)
        )
    }
    inner_gradient <-
      gList(inner_gradient, fastDoCall(gList, decr_polygons))
  }

  return(gList(pg, inner_gradient))
}

library(testthat)

context("bezier arrow performance")

test_that("arrows compute quickly", {
  skip_on_cran()
  skip_if_not_installed("bench")

  b <- bench::mark(
    simple = bezierArrowSmpl(x = c(.1, .3, .6, .9), y = c(.2, .2, .9, .9)),
    grad   = bezierArrowGradient(x = c(.1, .3, .6, .9), y = c(.2, .2, .9, .9)),
    iterations = 10,
    check = FALSE
  )

  # bench_time -> seconds (numeric)
  med_s <- as.numeric(b$median) / 1e9
  expect_true(all(med_s < 0.5))
})

#' Default One-to-Many connector
#' @noRd
prConnectOneToManyStandard <- function(
  start,
  ends,
  type,
  subelmnt,
  lty_gp,
  arrow_obj,
  smooth = FALSE,
  corner_radius = unit(3, "mm")
) {
  grobs <- lapply(ends, function(e) prConnect1(start, e, type, subelmnt, lty_gp, arrow_obj, smooth = smooth, corner_radius = corner_radius))
  structure(grobs, class = c("connect_boxes_list", "list"))
}

#' One-to-Many N connector (shared bend)
#' @noRd
prConnectOneToManyN <- function(
  start,
  ends,
  type,
  subelmnt,
  lty_gp,
  arrow_obj,
  split_pad = unit(2, "mm"), # only used for shared-bend types
  smooth = FALSE,
  corner_radius = unit(3, "mm")
) {
  assert_class(start, "box")
  if (is.numeric(split_pad)) split_pad <- unit(split_pad, "mm")
  assert_class(split_pad, "unit")

  s <- coords(start)
  end_coords <- lapply(ends, coords)

  # Determine vertical parameters (bend height and attachment edges)
  v_params <- prGetOneToManyNVerticalParameters(s, end_coords, split_pad)

  # Determine horizontal parameters (assigned slots on end boxes)
  x_params <- prGetOneToManyNAssignedX(start, s, ends, end_coords, subelmnt)

  # Create the list of grobs
  grobs <- prMakeOneToManyNGrobs(
    s = s,
    end_coords = end_coords,
    v_params = v_params,
    x_params = x_params,
    lty_gp = lty_gp,
    arrow_obj = arrow_obj,
    smooth = smooth,
    corner_radius = corner_radius
  )

  structure(grobs, class = c("connect_boxes_list", "list"))
}


#' Calculate vertical parameters for One-to-Many N connector
#' @noRd
prGetOneToManyNVerticalParameters <- function(s, end_coords, split_pad) {
  # Use explicit conversion helper for clarity
  s_y <- prConvertHeightToMm(s$y)
  e_y <- vapply(end_coords, \(e) prConvertHeightToMm(e$y), numeric(1))

  dir_down <- s_y > max(e_y)
  dir_up <- s_y < min(e_y)
  if (!dir_down && !dir_up) dir_down <- mean(e_y) < s_y

  d <- lapply(end_coords, function(e) distance(s, e, type = "v", half = TRUE))
  d_min <- Reduce(unit.pmin, d)

  if (dir_down) {
    # midpoint relative to the closest end
    bend_y <- s$bottom - d_min
    bend_y <- unit.pmax(bend_y, unit(0, "npc"))
    bend_y <- unit.pmin(bend_y, s$bottom - split_pad)
    start_attach <- s$bottom
    end_attach_fn <- function(e) e$top
  } else {
    bend_y <- s$top + d_min
    bend_y <- unit.pmin(bend_y, unit(1, "npc"))
    bend_y <- unit.pmax(bend_y, s$top + split_pad)
    start_attach <- s$top
    end_attach_fn <- function(e) e$bottom
  }

  list(
    bend_y = bend_y,
    start_attach = start_attach,
    end_attach_fn = end_attach_fn
  )
}


#' Calculate assigned X positions for One-to-Many N connector
#' @noRd
prGetOneToManyNAssignedX <- function(start, s, ends, end_coords, subelmnt) {
  # subelement x selection (for split boxes)
  sub_prefix <- if (missing(subelmnt)) "" else paste0(match.arg(subelmnt, c("right", "left")), "_")
  x_at <- function(pos, side = c("left", "right", "x")) {
    side <- match.arg(side)
    if (side == "x" && !is.null(pos[[paste0(sub_prefix, side)]])) {
      return(pos[[paste0(sub_prefix, side)]])
    }
    pos[[side]]
  }

  # Map ends left->right and, if centered alignment holds, make middle end align with start
  end_x_vals <- vapply(end_coords, \(e) prGetNpcValue(x_at(e, "x"), "x"), numeric(1))
  # Use shared assignment helper for stable ordering
  slots_res <- prAssignSlots(end_x_vals, end_x_vals)
  assigned_end_vals <- slots_res$assigned
  ord <- slots_res$ord

  centered <- prNShouldUseCenteredBranch(ends, start)
  mid_sorted_idx <- NA
  if (centered) {
    mid_sorted_idx <- slots_res$mid_idx
    assigned_end_vals[mid_sorted_idx] <- prGetNpcValue(s$x, "x")
  }

  list(
    assigned_end_vals = assigned_end_vals,
    mid_sorted_idx = mid_sorted_idx,
    centered = centered,
    x_at_fn = x_at
  )
}


#' Create the list of grobs for One-to-Many N connector
#' @noRd
prMakeOneToManyNGrobs <- function(s, end_coords, v_params, x_params, lty_gp, arrow_obj,
                                  smooth = FALSE, corner_radius = unit(3, "mm")) {
  lapply(seq_along(end_coords), function(i) {
    e <- end_coords[[i]]
    x_end <- unit(x_params$assigned_end_vals[i], "npc")

    # For the centered straight branch, force the start x coords to match the
    # assigned end position so the trunk is perfectly vertical.
    if (x_params$centered && i == x_params$mid_sorted_idx) {
      x_start0 <- x_end
    } else {
      x_start0 <- x_params$x_at_fn(s, "x")
    }

    line <- list(
      x = unit.c(x_start0, x_start0, x_end, x_end),
      y = unit.c(v_params$start_attach, v_params$bend_y, v_params$bend_y, v_params$end_attach_fn(e))
    )
    lg <- prRenderLine(x = line$x, y = line$y, smooth = smooth, corner_radius = corner_radius,
                       gp = lty_gp, arrow = arrow_obj)
    structure(lg, line = line, class = c("connect_boxes", class(lg)))
  })
}

#' Equalize box widths
#'
#' Sets selected boxes to a shared width. This is useful when aligning flowchart
#' levels so corresponding boxes have consistent visual width and center points.
#'
#' @param x A list of boxes (or nested lists of boxes). Can also be a single box.
#' @param subelement Optional target(s) inside `x`.
#'  Can be a single path (e.g. `c("groups", 1)`) or a list of paths
#'  (e.g. `list(c("groups", 1), c("groups2", 1))`).
#'  If `NULL`, all top-level boxes in `x` are used.
#' @param width Optional width to apply. If `NULL`, the maximum current width
#'  among selected boxes is used.
#'
#' @return The updated object with equalized widths.
#' @export
#' @family flowchart components
#' @examples
#' fc <- flowchart(
#'   groups = list("Group 1", "Group 2"),
#'   groups2 = list("Analysed", "Analysed")
#' )
#'
#' fc |>
#'   equalizeWidths(subelement = list(c("groups", 1),
#'                                    c("groups2", 1),
#'                                    c("groups", 2),
#'                                    c("groups2", 2)))
#'
#' # Global fixed width via explicit argument
#' fc |> equalizeWidths(subelement = "groups", width = grid::unit(30, "mm"))
#' @md
equalizeWidths <- function(x, subelement = NULL, width = NULL) {
  if (inherits(x, "box")) {
    if (is.null(width)) {
      return(x)
    }
    return(prSetBoxDimensions(x, width = width))
  }

  if (is.list(x) && !inherits(x, "Gmisc_list_of_boxes")) {
    x <- prConvertListToBoxList(x)
  }

  if (!inherits(x, "Gmisc_list_of_boxes")) {
    stop("equalizeWidths() requires a box or list of boxes.", call. = FALSE)
  }

  if (!is.null(width) && is.numeric(width)) {
    width <- unit(width, "mm")
  }
  if (!is.null(width) && !inherits(width, "unit")) {
    stop("`width` must be a unit or numeric.", call. = FALSE)
  }

  resolve_paths <- function(obj, subelement) {
    if (is.null(subelement)) {
      idx <- which(vapply(obj, inherits, logical(1), "box"))
      return(lapply(idx, as.list))
    }
    if (is.list(subelement) && all(vapply(subelement, is.atomic, logical(1)))) {
      return(subelement)
    }
    list(subelement)
  }

  fetch_target <- function(obj, path) {
    target <- get_list_element_by_path(obj, path)
    if (!is.null(target)) {
      return(list(target = target, first_container = FALSE))
    }

    if (length(obj) > 0 && is.list(obj[[1]]) && !inherits(obj[[1]], "box")) {
      target2 <- get_list_element_by_path(obj[[1]], path)
      if (!is.null(target2)) {
        return(list(target = target2, first_container = TRUE))
      }
    }

    list(target = NULL, first_container = FALSE)
  }

  paths <- resolve_paths(x, subelement)
  if (length(paths) == 0) {
    return(x)
  }

  selected <- list()
  selected_meta <- list()

  for (i in seq_along(paths)) {
    res <- fetch_target(x, paths[[i]])
    if (is.null(res$target)) {
      stop("The subelement '", paste(paths[[i]], collapse = " -> "), "' was not found in the provided boxes.", call. = FALSE)
    }
    if (inherits(res$target, "box")) {
      selected[[length(selected) + 1]] <- res$target
      selected_meta[[length(selected_meta) + 1]] <- list(
        path = paths[[i]],
        first_container = res$first_container
      )
      next
    }

    if (prIsBoxList(res$target)) {
      for (j in seq_along(res$target)) {
        selected[[length(selected) + 1]] <- res$target[[j]]
        selected_meta[[length(selected_meta) + 1]] <- list(
          path = c(paths[[i]], j),
          first_container = res$first_container
        )
      }
      next
    }

    stop("The subelement '", paste(paths[[i]], collapse = " -> "), "' is not a box or a list of boxes.", call. = FALSE)
  }

  target_width <- width
  if (is.null(target_width)) {
    widths_mm <- vapply(selected, function(b) {
      prConvertWidthToMm(coords(b)$width)
    }, numeric(1))
    target_width <- unit(max(widths_mm), "mm")
  }

  for (i in seq_along(selected)) {
    updated <- prSetBoxDimensions(selected[[i]], width = target_width)
    if (isTRUE(selected_meta[[i]]$first_container)) {
      x[[1]] <- set_list_element_by_path(x[[1]], selected_meta[[i]]$path, updated)
    } else {
      x <- set_list_element_by_path(x, selected_meta[[i]]$path, updated)
    }
  }

  prExtendClass(x, "Gmisc_list_of_boxes")
}

prSetBoxDimensions <- function(element, width = NULL, height = NULL) {
  assert_class(element, "box")

  if (is.null(width) && is.null(height)) {
    return(element)
  }

  vp_args <- attr(element, "viewport_data")
  assert_list(vp_args)

  if (!is.null(width)) {
    if (is.numeric(width)) width <- unit(width, "mm")
    if (!inherits(width, "unit")) stop("`width` must be a unit or numeric.", call. = FALSE)
    vp_args$width <- width
  }

  if (!is.null(height)) {
    if (is.numeric(height)) height <- unit(height, "mm")
    if (!inherits(height, "unit")) stop("`height` must be a unit or numeric.", call. = FALSE)
    vp_args$height <- height
  }

  gl <- editGrob(element, vp = do.call(viewport, vp_args))
  attr(gl, "viewport_data") <- vp_args
  attr(gl, "coords") <- prCreateBoxCoordinates(
    viewport_data = vp_args,
    extra_coordinate_functions = attr(element, "extra_coordinate_functions")
  )
  gl
}

library(testthat)
library(grid)

test_that("equalizeWidths() equalizes selected nested boxes and preserves centers", {
  fc <- flowchart(
    groups = list(
      "Short",
      "A much longer label"
    ),
    groups2 = list(
      "Tiny",
      "Another longer label"
    )
  ) |>
    spread(axis = "y", margin = unit(0.02, "npc")) |>
    spread(subelement = "groups", axis = "x", margin = unit(.05, "npc")) |>
    spread(subelement = "groups2", axis = "x", margin = unit(.05, "npc"))

  target_paths <- list(c("groups", 1), c("groups2", 1), c("groups", 2), c("groups2", 2))

  x_before <- vapply(target_paths, function(p) {
    b <- Gmisc:::get_list_element_by_path(fc, p)
    convertX(coords(b)$x, "npc", valueOnly = TRUE)
  }, numeric(1))

  out <- equalizeWidths(fc, subelement = target_paths)

  widths_mm <- vapply(target_paths, function(p) {
    b <- Gmisc:::get_list_element_by_path(out, p)
    convertWidth(coords(b)$width, "mm", valueOnly = TRUE)
  }, numeric(1))

  x_after <- vapply(target_paths, function(p) {
    b <- Gmisc:::get_list_element_by_path(out, p)
    convertX(coords(b)$x, "npc", valueOnly = TRUE)
  }, numeric(1))

  expect_true(all(abs(widths_mm - widths_mm[1]) < 1e-6))
  expect_equal(x_after, x_before, tolerance = 1e-8)
})

test_that("equalizeWidths() supports selecting a list-of-boxes path", {
  fc <- flowchart(
    groups = list("A", "Longer group text")
  )

  out <- equalizeWidths(fc, subelement = "groups", width = unit(30, "mm"))

  widths_mm <- vapply(out$groups, function(b) {
    convertWidth(coords(b)$width, "mm", valueOnly = TRUE)
  }, numeric(1))

  expect_true(all(abs(widths_mm - 30) < 1e-6))
})

#' Fix rownames for descriptive results
#'
#' Helper for [getDescriptionStatsBy] that fixes row names
#'
#' @param results A matrix with the results
#' @param t The [base::by()] output
#' @param name Name if row names are missing or the results is a single row
#'
#' @return The results with fixed names
prFixDescRownames <- function(results, t, name) {
  # Add the proper row names
  if ("matrix" %in% class(t[[1]])) {
    rownames(results) <- rownames(t[[1]])
  } else {
    rownames(results) <- names(t[[1]])
  }

  # This is an effect from the numeric variable not having
  # a naming function
  if (is.null(rownames(results)) && nrow(results) == 1) {
    rownames(results) <- name
  }

  return(results)
}

#' @section The \code{plot}/\code{print}:
#'
#' To output the \code{\link[grid:grid.grob]{grob}} objects to the plot either call \code{plot}
#' on the object or \code{print} it. Note that R automatically prints any object that is outputted
#' to the console. The function calls in turn the \code{\link[grid]{grid.draw}} function on the object.
#'
#' @param ... Passed to \code{\link[grid]{grid.draw}}
#' @rdname box
#' @export
#' @order 2
print.box <- function(x, ...) {
  grid.draw(x, ...)
  invisible(x)
}

#' @rdname box
#' @export
#' @order 3
plot.box <- print.box

#' @section S3 from the \pkg{grid} package:
#'
#' Width and height functions address the \code{\link{coords}} attribute for the corresponding information.
#' The \code{\link[grid:widthDetails]{widthDetails}} and \code{\link[grid:widthDetails]{heightDetails}}
#' that provide information on an object.
#'
#' @rdname box
#' @export
#' @order 4
widthDetails.box <- function(x) {
  coords(x)$width
}

#' @rdname box
#' @export
#' @order 5
heightDetails.box <- function(x) {
  coords(x)$height
}

#' Default Many-to-One connector
#'
#' Loops over starts and connects each to the end using prConnect1.
#' @noRd
prConnectManyToOneStandard <- function(
  starts,
  end,
  type,
  subelmnt,
  lty_gp,
  arrow_obj,
  smooth = FALSE,
  corner_radius = unit(3, "mm")
) {
  grobs <- lapply(starts, function(s) prConnect1(s, end, type, subelmnt, lty_gp, arrow_obj, smooth = smooth, corner_radius = corner_radius))
  structure(grobs, class = c("connect_boxes_list", "list"))
}

#' Many-to-One N connector (shared bend)
#' @noRd
prConnectManyToOneN <- function(
  starts,
  end,
  type,
  subelmnt,
  lty_gp,
  arrow_obj,
  split_pad = unit(2, "mm"),
  smooth = FALSE,
  corner_radius = unit(3, "mm")
) {
  # If `end` is container-like, attempt to find a reasonable boxed target.
  if (!inherits(end, "box")) {
    fb <- prFindFirstBox(end)
    if (!is.null(fb)) end <- fb
  }

  assert_class(end, "box")

  s_coords <- lapply(starts, coords)
  e <- coords(end)

  # Determine vertical parameters (bend height and attachment edges)
  v_params <- prGetManyToOneNVerticalParameters(starts, end, split_pad)

  # Determine horizontal parameters (assigned slots on the end box)
  x_params <- prGetManyToOneNAssignedX(starts, s_coords, e, subelmnt)

  # Create the list of grobs
  grobs <- prMakeManyToOneNGrobs(
    s_coords = s_coords,
    v_params = v_params,
    x_params = x_params,
    lty_gp = lty_gp,
    arrow_obj = arrow_obj,
    smooth = smooth,
    corner_radius = corner_radius
  )

  structure(grobs, class = c("connect_boxes_list", "list"))
}


#' Calculate vertical parameters for Many-to-One N connector
#' @noRd
prGetManyToOneNVerticalParameters <- function(starts, end, split_pad) {
  # Shared bend point calculation
  bend_y <- prCalculateBendY(starts, end, split_pad = split_pad)

  # Attachment edges
  starts_above <- prStartsAbove(starts, end)
  e <- coords(end)
  if (starts_above) {
    end_attach <- e$top
    start_attach_fn <- function(s) s$bottom
  } else {
    end_attach <- e$bottom
    start_attach_fn <- function(s) s$top
  }

  list(
    bend_y = bend_y,
    end_attach = end_attach,
    start_attach_fn = start_attach_fn
  )
}


#' Calculate assigned X positions for Many-to-One N connector
#' @noRd
prGetManyToOneNAssignedX <- function(starts, s_coords, e, subelmnt) {
  # subelement x selection (for split boxes)
  sub_prefix <- if (missing(subelmnt)) "" else paste0(match.arg(subelmnt, c("right", "left")), "_")
  x_at <- function(pos, side = c("left", "right", "x")) {
    side <- match.arg(side)
    if (side == "x" && !is.null(pos[[paste0(sub_prefix, side)]])) {
      return(pos[[paste0(sub_prefix, side)]])
    }
    pos[[side]]
  }

  # Map end attachment slots to starts by left->right order so central alignment
  # can be detected and handled (center straight branch when appropriate).
  starts_x_vals <- vapply(s_coords, function(s) prGetNpcValue(x_at(s, "x"), "x"), numeric(1))
  xs_end_vals <- vapply(prEdgeSlots(e$left, e$right, n = length(starts)), function(u) prGetNpcValue(u, "x"), numeric(1))
  # Use shared assignment helper for stable mapping from starts -> end slots
  slots_res <- prAssignSlots(starts_x_vals, xs_end_vals)
  assigned_end_vals <- slots_res$assigned
  ord <- slots_res$ord

  # If centered straight branch desired, make the middle start attach to end center
  centered <- prNShouldUseCenteredBranch(starts, e)
  mid_sorted_idx <- NA
  if (centered) {
    mid_sorted_idx <- slots_res$mid_idx
    assigned_end_vals[mid_sorted_idx] <- prGetNpcValue(prConvert2Coords(e)$x, "x")
  }

  list(
    assigned_end_vals = assigned_end_vals,
    mid_sorted_idx = mid_sorted_idx,
    centered = centered,
    x_at_fn = x_at
  )
}


#' Create the list of grobs for Many-to-One N connector
#' @noRd
prMakeManyToOneNGrobs <- function(s_coords, v_params, x_params, lty_gp, arrow_obj,
                                  smooth = FALSE, corner_radius = unit(3, "mm")) {
  lapply(seq_along(s_coords), function(i) {
    s <- s_coords[[i]]
    x_end <- unit(x_params$assigned_end_vals[i], "npc")

    # For the centered straight branch, force the start x coords to match the
    # assigned end position so the trunk is perfectly vertical.
    if (x_params$centered && i == x_params$mid_sorted_idx) {
      x_start0 <- x_end
    } else {
      x_start0 <- x_params$x_at_fn(s, "x")
    }

    line <- list(
      x = unit.c(x_start0, x_start0, x_end, x_end),
      y = unit.c(v_params$start_attach_fn(s), v_params$bend_y, v_params$bend_y, v_params$end_attach)
    )
    lg <- prRenderLine(x = line$x, y = line$y, smooth = smooth, corner_radius = corner_radius,
                       gp = lty_gp, arrow = arrow_obj)
    structure(lg, line = line, class = unique(c("connect_boxes", class(lg))))
  })
}

#' Align boxes (S3)
#'
#' Align a list of boxes vertically or horizontally. Designed for piping (`|>`).
#'
#' @param x A `list` of boxes.
#' @param ... Arguments passed to [`alignVertical`] or [`alignHorizontal`].
#' @param axis Orientation: `"y"`/`"vertical"` or `"x"`/`"horizontal"`.
#'
#' @return The aligned list of boxes (class `Gmisc_list_of_boxes`).
#' @export
#' @family flowchart components
align <- function(x, ...) {
  UseMethod("align")
}

#' @export
#' @rdname align
align.default <- function(x, ...) {
  if (is.list(x) && !inherits(x, "box")) {
    return(align(prConvertListToBoxList(x), ...))
  }
  stop("align() requires a list of boxes.")
}

#' @export
#' @rdname align
align.list <- function(x, ...) {
  align.default(x, ...)
}

#' @export
#' @rdname align
align.Gmisc_list_of_boxes <- function(x, ..., axis = c("y", "x", "vertical", "horizontal")) {
  axis <- match.arg(axis)

  if (axis %in% c("y", "vertical")) {
    return(alignVertical(x, ...))
  } else {
    return(alignHorizontal(x, ...))
  }
}

prIsBoxList <- function(x) {
  is.list(x) && length(x) > 0 && all(vapply(x, inherits, logical(1), "box"))
}

# If a list is a single-element wrapper containing a list-of-boxes, unwrap it.
# For example, a `Gmisc_list_of_boxes` object sometimes appears wrapped such
# that the actual boxes are one level deeper. This helper returns the unwrapped
# list-of-boxes when appropriate, or the original object otherwise.
prFlattenBoxListIfNeeded <- function(x) {
  if (length(x) == 1 && is.list(x[[1]]) && all(vapply(x[[1]], inherits, logical(1), "box"))) {
    return(x[[1]])
  }
  x
}

# If the first element of `x` is itself a list-of-boxes and there are other
# top-level elements too, unwrap to the first container. This handles cases
# like `list(list(box1, box2), other = list())` where the user likely passed
# a grouped element as the intended `start`/`end`.
prMaybeUnwrapFirstContainerBoxes <- function(x) {
  if (is.list(x) && length(x) > 1 && is.list(x[[1]]) && all(vapply(x[[1]], inherits, logical(1), "box"))) {
    return(x[[1]])
  }
  x
}

# Normalize list elements so that single-element wrapper elements that
# contain a box are unwrapped. This handles mixed lists like
# `list(box, list(box), box)` which should be treated as a list of boxes.
prNormalizeBoxElements <- function(x) {
  if (!is.list(x)) {
    return(x)
  }
  for (i in seq_along(x)) {
    if (is.list(x[[i]]) && length(x[[i]]) == 1 && inherits(x[[i]][[1]], "box")) {
      x[[i]] <- x[[i]][[1]]
    }
  }
  x
}

# Collapse a single-element list that is itself a boxed element into the bare box.
# This ensures that `connectGrob()` treats `list(box)` as a single `box` input
# rather than as a one-element list-of-boxes (which otherwise may cause both
# start and end to be lists and trigger the unsupported both-list error).
prCollapseSingleBoxList <- function(x) {
  if (is.list(x) && length(x) == 1 && inherits(x[[1]], "box")) {
    return(x[[1]])
  }
  x
}

prEdgeSlots <- function(left, right, n, margin = unit(0, "mm")) {
  if (is.numeric(margin)) margin <- unit(margin, "mm")
  stopifnot(inherits(margin, "unit"))
  stopifnot(n >= 1)

  leftWithMargin <- left + margin
  rightMinusMargin <- right - margin
  if (n == 1) {
    return(unit.c((leftWithMargin + rightMinusMargin) / 2))
  }

  step <- (rightMinusMargin - leftWithMargin) / (n + 1)
  xs <- leftWithMargin + step * seq_len(n)
  unit.c(xs)
}

# Find the first boxed element in a possibly nested container. Useful for
# making many-to-one connectors robust to inputs where the intended `end`
# was wrapped in container lists by layout pipelines.
prFindFirstBox <- function(x) {
  if (inherits(x, "box")) {
    return(x)
  }
  if (!is.list(x)) {
    return(NULL)
  }
  for (el in x) {
    if (inherits(el, "box")) {
      return(el)
    }
    if (is.list(el)) {
      found <- prFindFirstBox(el)
      if (!is.null(found)) {
        return(found)
      }
    }
  }
  NULL
}

#' Determine if start boxes are positioned above the end box
#'
#' Internal helper that compares the mean vertical position of the `starts` boxes
#' with the `end` box y position. Values are converted to millimetres for a
#' device-independent comparison.
#'
#' @param starts A `list` of `boxGrob` objects or a single `boxGrob`.
#' @param end A `boxGrob` object.
#' @return `TRUE` if the mean y of `starts` is above `end`'s y, otherwise `FALSE`.
#' @keywords internal
#' @noRd
prStartsAbove <- function(starts, end) {
  if (inherits(starts, "box")) starts <- list(starts)
  s_coords <- lapply(starts, coords)
  e <- coords(end)
  mean(vapply(s_coords, function(s) prConvertHeightToMm(s$y), numeric(1))) > prConvertHeightToMm(e$y)
}


#' Determine whether an N connector can use a straight center branch
#'
#' For an N (shared-bend) layout, return TRUE when there are an odd number of
#' boxes and the middle box is aligned with the counterpart (within `tolerance`
#' fraction of the counterpart's width). This applies to both many-to-one and
#' one-to-many scenarios.
#'
#' @param items A list of boxes (starts or ends depending on context).
#' @param target The counterpart box (end for many-to-one, start for one-to-many).
#' @param tolerance Fraction of `target` width for allowable offset (default 0.1).
#' @return `TRUE` if the N connector should place a straight centered branch.
#' @keywords internal
#' @noRd
prNShouldUseCenteredBranch <- function(items, target, tolerance = 0.1) {
  n <- length(items)
  if (n %% 2 == 0) {
    return(FALSE)
  }
  coords_items <- lapply(items, coords)

  # Use centralized helper for robust npc extraction
  x_vals <- vapply(coords_items, function(s) prGetNpcValue(s$x, "x"), numeric(1))
  ord <- order(x_vals)
  middle_idx_sorted <- ord[(n + 1) / 2]
  middle_x <- x_vals[middle_idx_sorted]

  targ_coords <- prConvert2Coords(target)
  targ_x <- prGetNpcValue(targ_coords$x, "x")

  # Prefer converting declared width (npc numeric). If that fails (e.g. units
  # not resolvable in the current device/context) fall back to left/right
  # edge difference using `prGetNpcValue`. Finally default to 1 so lack of a
  # convertible width doesn't prevent centered-branch behavior.
  targ_w <- prGetNpcSize(targ_coords$width, "x")
  if (is.na(targ_w) || targ_w <= 0) {
    left <- prGetNpcValue(targ_coords$left, "x")
    right <- prGetNpcValue(targ_coords$right, "x")
    if (!is.na(left) && !is.na(right)) {
      targ_w <- right - left
    } else {
      targ_w <- 1
    }
  }

  abs(middle_x - targ_x) <= tolerance * targ_w
}

# Assign end-slot positions to start positions in a stable, shared way.
# Returns list with `assigned` numeric (length n), `ord` ordering indices,
# and `mid_idx` the index of the chosen center start.
prAssignSlots <- function(starts_x_vals, xs_end_vals) {
  n <- length(starts_x_vals)
  if (n == 0L) {
    return(list(assigned = numeric(0), ord = integer(0), mid_idx = integer(0)))
  }
  ord <- order(starts_x_vals)
  assigned <- numeric(n)
  assigned[ord] <- xs_end_vals
  mid_pos <- ceiling(n / 2)
  mid_idx <- ord[mid_pos]
  list(assigned = assigned, ord = ord, mid_idx = mid_idx)
}

#' Calculate bend Y coordinate for shared-bend connectors
#'
#' Compute a suitable vertical bend point for shared-bend connectors.
#' The function computes the minimum vertical distance between starts and the
#' end box and returns the midpoint, clamped by `split_pad`.
#'
#' @param starts A `list` of start `boxGrob` objects.
#' @param end A single end `boxGrob` object.
#' @param edge Which edge of the end box to use for distance calculation and
#'   clamping: `"auto"` (closest), `"top"`, or `"bottom"`.
#' @param margin A `grid::unit` (unused, kept for API compatibility).
#' @param split_pad Padding to enforce around the shared bend point.
#' @return A `grid::unit` giving the y-coordinate of the shared bend point.
#' @keywords internal
#' @noRd
prCalculateBendY <- function(
  starts,
  end,
  edge = c("auto", "top", "bottom"),
  margin = unit(2, "mm"), split_pad = unit(0, "mm")
) {
  edge <- match.arg(edge)
  if (inherits(starts, "box")) starts <- list(starts)

  # Calculate distances to find closest box
  s_dists <- vapply(starts, function(s) {
    as.numeric(distance(s, end, type = "v", half = FALSE))
  }, numeric(1))

  closest_idx <- which.min(s_dists)
  closest_s <- starts[[closest_idx]]
  s_coords <- coords(closest_s)
  e_coords <- coords(end)

  starts_above <- prStartsAbove(starts, end)

  if (inherits(split_pad, "numeric")) split_pad <- unit(split_pad, "mm")

  if (edge == "top" || (edge == "auto" && starts_above)) {
    # Bend is between starts bottom and end top
    bend_y <- MidDistanceY(s_coords$bottom, e_coords$top)
    bend_y <- unit.pmax(bend_y, e_coords$top + split_pad)
  } else {
    # Bend is between starts top and end bottom
    bend_y <- MidDistanceY(s_coords$top, e_coords$bottom)
    bend_y <- unit.pmin(bend_y, e_coords$bottom - split_pad)
  }

  return(bend_y)
}

#' Determine if start boxes are positioned to the left of the end box
#'
#' Internal helper that compares the mean horizontal position of the `starts` boxes
#' with the `end` box x position. Values are converted to millimetres for a
#' device-independent comparison.
#'
#' @param starts A `list` of `boxGrob` objects or a single `boxGrob`.
#' @param end A `boxGrob` object.
#' @return `TRUE` if the mean x of `starts` is to the left `end`'s x, otherwise `FALSE`.
#' @keywords internal
#' @noRd
prStartsLeft <- function(starts, end) {
  if (inherits(starts, "box")) starts <- list(starts)
  s_coords <- lapply(starts, coords)
  e <- coords(end)
  mean(vapply(s_coords, function(s) prConvertWidthToMm(s$x), numeric(1))) < prConvertWidthToMm(e$x)
}

#' Render a connector line, optionally with rounded corners
#'
#' Replaces `linesGrob()` as the final rendering step for all connector types.
#' When `smooth = FALSE` (default) it behaves identically to the previous
#' `linesGrob()` call. When `smooth = TRUE` each interior orthogonal corner is
#' replaced by a Bézier arc computed with [gnrlBezierPoints()], which is already
#' used elsewhere in the package (transitionPlot etc.).
#'
#' Rules:
#' - Only 3- and 4-point paths have interior corners; 2-point paths are always
#'   straight and ignore `smooth`.
#' - The corner radius is clamped to half the length of each adjacent segment so
#'   the arc cannot overshoot into a neighbouring segment.
#' - The arrow is placed only on the final straight segment.
#'
#' @param x,y `unit` vectors of path points (same length, >= 2).
#' @param smooth Logical; if `TRUE` round interior corners.
#' @param corner_radius A `unit` or numeric (mm) giving the desired corner radius.
#' @param gp [grid::gpar()] for line appearance.
#' @param arrow [grid::arrow()] specification (applied to final segment only).
#' @return A single `linesGrob`.
#' @keywords internal
#' @noRd
prRenderLine <- function(x, y,
                         smooth = FALSE,
                         corner_radius = unit(3, "mm"),
                         gp,
                         arrow) {
  n <- length(x)
  stopifnot(n == length(y), n >= 2)

  if (!smooth || n < 3) {
    return(grid::linesGrob(x = x, y = y, gp = gp, arrow = arrow))
  }

  if (is.numeric(corner_radius)) corner_radius <- unit(corner_radius, "mm")

  # Convert all points to mm for arithmetic
  xmm <- prConvertWidthToMm(x)
  ymm <- prConvertHeightToMm(y)

  # Vectorized mm->npc helpers (prConvertMmToNpc only accepts length-1 inputs)
  mm2npc_x <- function(v) vapply(v, function(m) as.numeric(prConvertMmToNpc(m, "x")), numeric(1))
  mm2npc_y <- function(v) vapply(v, function(m) as.numeric(prConvertMmToNpc(m, "y")), numeric(1))

  # Accumulate the entire smooth path as a single coordinate vector (npc).
  # Using one linesGrob avoids rendering seams at segment/arc joints.
  path_x <- mm2npc_x(xmm[1])
  path_y <- mm2npc_y(ymm[1])

  prev_x <- xmm[1]
  prev_y <- ymm[1]

  r_mm <- as.numeric(convertUnit(corner_radius, "mm"))

  for (i in seq(2, n - 1)) {
    seg1_dx <- xmm[i] - prev_x
    seg1_dy <- ymm[i] - prev_y
    seg1_len <- sqrt(seg1_dx^2 + seg1_dy^2)

    seg2_dx <- xmm[i + 1] - xmm[i]
    seg2_dy <- ymm[i + 1] - ymm[i]
    seg2_len <- sqrt(seg2_dx^2 + seg2_dy^2)

    r <- min(r_mm, seg1_len / 2, seg2_len / 2)

    if (seg1_len <= 0 || seg2_len <= 0 || r <= 0) {
      path_x <- c(path_x, mm2npc_x(xmm[i]))
      path_y <- c(path_y, mm2npc_y(ymm[i]))
      prev_x <- xmm[i]
      prev_y <- ymm[i]
      next
    }

    u1x <- seg1_dx / seg1_len; u1y <- seg1_dy / seg1_len
    u2x <- seg2_dx / seg2_len; u2y <- seg2_dy / seg2_len

    pin_x  <- xmm[i] - r * u1x;  pin_y  <- ymm[i] - r * u1y
    pout_x <- xmm[i] + r * u2x;  pout_y <- ymm[i] + r * u2y

    # Straight portion up to the arc entry point
    path_x <- c(path_x, mm2npc_x(pin_x))
    path_y <- c(path_y, mm2npc_y(pin_y))

    # Bézier arc: control points are P_in, corner, P_out (quadratic)
    ctrl <- data.frame(
      x = mm2npc_x(c(pin_x,  xmm[i], pout_x)),
      y = mm2npc_y(c(pin_y,  ymm[i], pout_y))
    )
    arc_pts <- gnrlBezierPoints(ctrl, length_out = 20L)
    # arc_pts[1,] == P_in (already added above), so skip the first row
    path_x <- c(path_x, arc_pts[-1L, 1])
    path_y <- c(path_y, arc_pts[-1L, 2])

    prev_x <- pout_x
    prev_y <- pout_y
  }

  # Final straight segment to the last point
  path_x <- c(path_x, mm2npc_x(xmm[n]))
  path_y <- c(path_y, mm2npc_y(ymm[n]))

  grid::linesGrob(
    x = unit(path_x, "npc"),
    y = unit(path_y, "npc"),
    gp = gp,
    arrow = arrow
  )
}


#' Calculate bend X coordinate for shared-bend connectors
#'
#' Compute a suitable horizontal bend point for shared-bend connectors.
#'
#' @param starts A `list` of start `boxGrob` objects.
#' @param end A single end `boxGrob` object.
#' @param edge Which edge of the end box to use: `"auto"` (closest), `"left"`, or `"right"`.
#' @param margin A `grid::unit` (unused, kept for API compatibility).
#' @param split_pad Padding to enforce around the shared bend point.
#' @return A `grid::unit` giving the x-coordinate of the shared bend point.
#' @keywords internal
#' @noRd
prCalculateBendX <- function(
  starts,
  end,
  edge = c("auto", "left", "right"),
  margin = unit(2, "mm"), split_pad = unit(0, "mm")
) {
  edge <- match.arg(edge)
  if (inherits(starts, "box")) starts <- list(starts)

  # Calculate distances to find closest box
  s_dists <- vapply(starts, function(s) {
    as.numeric(distance(s, end, type = "h", half = FALSE))
  }, numeric(1))

  closest_idx <- which.min(s_dists)
  closest_s <- starts[[closest_idx]]
  s_coords <- coords(closest_s)
  e_coords <- coords(end)

  starts_left <- prStartsLeft(starts, end)

  if (inherits(split_pad, "numeric")) split_pad <- unit(split_pad, "mm")

  if (edge == "right" || (edge == "auto" && !starts_left)) {
    # Bend is between starts left and end right
    bend_x <- MidDistanceX(s_coords$left, e_coords$right)
    bend_x <- unit.pmax(bend_x, e_coords$right + split_pad)
  } else {
    # Bend is between starts right and end left
    bend_x <- MidDistanceX(s_coords$right, e_coords$left)
    bend_x <- unit.pmin(bend_x, e_coords$left - split_pad)
  }

  return(bend_x)
}

#' A reference class for generating transition plots
#'
#' This class simplifies the creating of transition plots. It also
#' allows for advanced multi-column transitions.
#'
#' Transition plots are a type of \emph{Sankey diagrams}. These are a specific type
#' of flow diagram, in which the width of the arrows is shown proportionally
#' to the flow quantity. See \href{https://en.wikipedia.org/wiki/Sankey_diagram}{Wikipedia}
#' for details.
#'
#' @field id Optional id. The render uses named viewports that require a unique id if multiple transition plots
#'  are combined. In order to avoid having overlapping graphs we need to generate a unique id for each viewport
#'  and thus this variable exists. If left empty it will create a counter that is stored in the \code{\link[base]{options}}
#'  (\code{"Gmisc.transitionClassCounter"}) and each viewport will have the name preceded with \code{tc_[0-9]+}. Set this
#'  if you intend to use \code{\link[grid:viewports]{seekViewport}}.
#' @field transitions This is a >= 3 dimensional array with the transitions. Should not be directly accessed.
#' @field box_width The box width
#' @field box_txt The texts of each box
#' @field box_label Box labels
#' @field box_label_pos The label's positions, either "top"/"bottom"
#' @field box_label_cex The size of the box labels
#' @field box_cex The font-size multiplier for the text within the boxes
#' @field arrow_type The type of arrow to use, defaults to "gradient", but can also be "simple".
#'  The corresponding functions are \code{\link{bezierArrowGradient}}, and
#'  \code{\link{bezierArrowSmpl}}. \emph{Note} The \code{bezierGrob} has been deprecated
#'  as it is no longer faster than the bezier arrows and there is a difference in design.
#' @field arrow_clr The arrow color
#' @field arrow_rez The resolution of the arrow
#' @field vertical_space The space between the boxes
#' @field fill_clr The box fill color
#' @field clr_bar Shows a color bar if there are proportions. Can be \code{"none"}, \code{"top"}, \code{"bottom"}
#' @field clr_bar_clrs Extracts the colors for the color bar from the \code{fill_clr} if none is provided
#' @field clr_bar_cex The size of the ticks in the color bar
#' @field clr_bar_subspace  If little or no difference exists at the low/high proportions of
#'  the spectrum then it can be of interest to focus the color change to the center leaving the tails constant
#' @field clr_bar_labels The labels of the color bars. Defaults to the dim names for the proportions.
#' @field txt_clr The text color within the boxes
#' @field txt_gpar Similar to `txt_clr` but for more advanced styling with fontfamily (see [grid::gpar()]). *Note* that
#'  col & cex are overridden.
#' @field title The plot title if any
#' @field title_cex The font-size multiplier for the title
#' @field skip_shadows Skip the shadow effect on the boxes
#' @field mar The margins for the plot.
#' @field min_lwd The minimum line width that is still shown. The pixels will most likely
#'  not have the same fine resolution as the data and you therefore may want to hide
#'  lines that are smaller than a certain amount.
#' @field max_lwd The maximum line width to show
#' @field lwd_prop_type The line can either be proportional to the \code{"set"} of transitions
#'  (group of two box columns), to \code{"all"} transitions, or to each \code{"box"}. It defaults
#'  to \code{"all"}.
#' @field data Internal storage variable. Should not be accessed directly.
#'
#' @import magrittr
#' @importFrom methods setRefClass new
#' @import abind
#' @importFrom utils head tail
#' @importFrom grDevices grey as.raster
#' @example inst/examples/transitionClass_example.R
#' @export
Transition <-
  setRefClass(
    "Transition",
    fields = list(
      id = "character",
      data = "list",
      transitions = function(value) {
        if (missing(value)) {
          return(data$transitions)
        }

        if (is.null(attr(value, "transition"))) {
          stop("You are only allowed to use new()/addTransitions() method for setting the transitions")
        }
        if (!is.list(value)) {
          value <- list(value)
        }
        dims <- sapply(value, function(mtrx) length(dim(mtrx)))
        if (!all(dims == dims[1])) {
          stop(
            "Your transition matrices have different dimensions: ",
            paste(dims, collapse = ":")
          )
        }
        if (!dims[1] %in% 2:3) {
          stop(
            "The dimension of the transition matrix has to be between 2 and 3.",
            " This means that the stored transitions should have an additional dimension",
            " in order to allow for multiple transitions."
          )
        }
        data$transitions <<- value
      },
      box_width = function(value) {
        if (missing(value)) {
          return(data$box_width)
        }

        if (!inherits(value, "unit") &&
            value > 1 && value < 0) {
          stop("The box width must be grid::unit or a double between 0 and 1")
        }
        if (inherits(value, "unit")) {
          raw_width <- convertUnit(value, unitTo = "npc", axisFrom = "x", axisTo = "x", valueOnly = TRUE)
        } else {
          raw_width <- value
          value <- unit(value, "npc")
        }

        if (raw_width * .self$noCols() * 1.1 > 1) {
          stop("Your box_width leaves less than 10% for arrows assuming you have the current plot size")
        }

        data$box_width <<- value
      },
      box_txt = function(value) {
        if (missing(value)) {
          return(data$box_txt)
        }

        if (!is.list(value)) {
          stop("The value has to be a list")
        }

        if (length(value) != .self$noCols()) {
          stop(
            "Your labels should match the number of rows within the transition matrix.",
            " You have currently ", length(value), " sets of labels while there are ",
            .self$noCols(), " columns"
          )
        }

        for (i in 1:length(value)) {
          if (length(value[[i]]) != .self$noRows(i)) {
            stop(
              "Your labels should match the number of rows within the transition matrix",
              " This is not the case for column no. ", i, " that is ", .self$noRows(i),
              " rows while you've provided ", length(value[[i]]), " texts."
            )
          }
        }

        data$box_txt <<- value
      },
      box_label = function(value) {
        if (missing(value)) {
          return(data$box_label)
        }

        if (length(value) != .self$noCols()) {
          stop(
            "You have provided '", length(value), "' box labels",
            " while there are '", .self$noCols(), "' columns that require a label."
          )
        }
        data$box_label <<- as.character(value)
      },
      box_label_pos = function(value = c("top", "bottom")) {
        if (missing(value)) {
          if (!is.null(data$box_label_pos)) {
            return(data$box_label_pos)
          }

          return("top")
        }

        data$box_label_pos <<- match.arg(value)
      },
      box_label_cex = function(value) {
        if (missing(value)) {
          if (!is.null(data$box_label_cex)) {
            return(data$box_label_cex)
          }

          label_cex <- box_cex * 1.2
          raw_width <- convertWidth(box_width, unitTo = "npc", valueOnly = TRUE)

          for (lab in box_label) {
            r_lab_width <- convertX(unit(1, "strwidth", data = lab), "npc", valueOnly = TRUE)
            size_ratio <- r_lab_width * label_cex * 1.05 / raw_width
            if (size_ratio > 1) {
              label_cex <- label_cex / size_ratio
            }
          }

          # Once used it should save the value for consistency
          data$box_label_cex <<- label_cex

          return(label_cex)
        }

        if (!is.numeric(value) && !is.null(value)) {
          stop("Only numeric cex values are accepted")
        }

        data$box_label_cex <<- value
      },
      box_cex = function(value) {
        if (missing(value)) {
          if (!is.null(data$box_cex)) {
            return(data$box_cex)
          }
          if (is.null(data$box_txt)) {
            return(1)
          }

          base_width <-
            box_txt %>%
            unlist() %>%
            as.vector() %>%
            lapply(function(x) strsplit(x, "\n")[[1]]) %>%
            unlist() %>%
            sapply(function(txt) {
              textGrob(label = txt, gp = gpar(cex = 1)) %>%
                grobWidth() %>%
                convertWidth(unitTo = "npc", valueOnly = TRUE)
            }) %>%
            max()

          width_cex <- convertWidth(box_width, unitTo = "npc", valueOnly = TRUE) * .8 / base_width

          min_height <- Inf
          for (col in 1:.self$noCols()) {
            proportions <- .self$getYProps(col)
            proportions <- proportions[proportions > 0]
            min_height %<>%
              min(proportions)
          }
          max_txt_height <- min_height * .6
          base_height <-
            textGrob(label = "A", gp = gpar(cex = 1)) %>%
            grobHeight() %>%
            convertHeight(unitTo = "npc", valueOnly = TRUE)
          height_cex <- max_txt_height / base_height

          return(max(.75, min(width_cex, height_cex)))
        }

        if (value < 0 && !is.null(value)) {
          stop("The cex is a multiplier of the font size and has to be at minimum 0")
        }

        data$box_cex <<- value
      },
      arrow_type = function(value = c("gradient", "simple")) {
        if (missing(value)) {
          if (!is.null(data$arrow_type)) {
            return(data$arrow_type)
          }
          return("gradient")
        }

        data$arrow_type <<- match.arg(value)
      },
      arrow_clr = function(value) {
        if (missing(value)) {
          if (!is.null(data$arrow_clr)) {
            return(data$arrow_clr)
          }
          clrs <- list()
          for (i in 1:length(transitions)) {
            clrs <- c(
              clrs,
              list(matrix("#000000",
                          nrow = nrow(transitions[[i]]),
                          ncol = ncol(transitions[[i]])
              ))
            )
          }
          return(clrs)
        }

        if (is.matrix(value) && .self$noCols() == 2) {
          if (!all(dim(value) != transitions[[1]])) {
            stop(
              "If you provide an arrow color matrix it must have the same",
              " rows and columns that your transition matrix has: ", paste(.self$getDim(), collapse = "-"),
              " You have provided a matrix of the dimensions: ", paste(dim(value), collapse = "-")
            )
          }
          value <- list(value)
        } else if (!is.list(value) &&
                   length(value) == 1) {
          clrs <- list()
          for (i in 1:length(transitions)) {
            clrs <- c(
              clrs,
              list(matrix(value,
                          nrow = nrow(transitions[[i]]),
                          ncol = ncol(transitions[[i]])
              ))
            )
          }
          value <- clrs
        } else if (is.vector(value) && .self$noCols() == 2) {
          if (length(value) != .self$noRows(1)) {
            stop("Could not match the number of rows in the transition matrix")
          }

          value <- matrix(value, nrow = .self$noRows(1), ncol = .self$noRows(2))
        } else if (is.list(value)) {
          if (length(value) != length(transitions)) {
            stop("The list length for the arrow colors should match the no of transition matrices")
          }

          for (i in 1:length(transitions)) {
            if (nrow(value[[i]]) != nrow(transitions[[i]])) {
              stop("In the ", i, " transitions there is a row mismatch")
            }

            if (ncol(value[[i]]) != ncol(transitions[[i]])) {
              stop("In the ", i, " transitions there is a column mismatch")
            }
          }
        } else {
          stop(
            "The color length '", length(value), "'",
            " could not be interpreted"
          )
        }

        data$arrow_clr <<- value
      },
      arrow_rez = function(value) {
        if (missing(value)) {
          if (!is.null(data$arrow_rez)) {
            return(data$arrow_rez)
          }
          return(400)
        }

        if (length(value) != 1 ||
            value < 1 ||
            round(value) != value) {
          stop("Invalid arrow resolution, needs to be integer between 1 -> Inf")
        }

        data$arrow_rez <<- value
      },
      vertical_space = function(value) {
        if (missing(value)) {
          if (is.null(data$vertical_space)) {
            return(unit(.1, "npc"))
          }

          return(data$vertical_space)
        }

        if (!inherits(value, "unit") &&
            value >= 1 && value < 0) {
          stop("The box width must be grid::unit or a double at least 0 and below 1")
        }
        if (inherits(value, "unit")) {
          raw_space <- convertUnit(value, unitTo = "npc", axisFrom = "y", axisTo = "y", valueOnly = TRUE)
          if (raw_space >= 1) {
            stop("Using your current graph size the provided value is as large as the graph")
          }
          if (raw_space < 0) {
            stop("You cannot have empty space smaller than 0")
          }
        } else {
          value <- unit(value, "npc")
        }

        data$vertical_space <<- value
      },
      fill_clr = function(value) {
        if (missing(value)) {
          return(data$fill_clr)
        }

        value <- prTcValidateAndPrepClr(value,
                                        no_cols = .self$noCols(),
                                        no_rows = .self$noRows(),
                                        is3D = .self$is3D()
        )

        data$fill_clr <<- value
      },
      txt_clr = function(value) {
        if (missing(value)) {
          return(data$txt_clr)
        }

        value <- prTcValidateAndPrepClr(value,
                                        no_cols = .self$noCols(),
                                        no_rows = .self$noRows(),
                                        is3D = .self$is3D()
        )

        data$txt_clr <<- value
      },
      txt_gpar = function(value, row = NULL, col = NULL) {
        if (missing(value)) {
          return(data$txt_gpar)
        }

        if (is.null(data$txt_gpar))
        value <- prTcValidateAndPrepClr(value,
                                        no_cols = .self$noCols(),
                                        no_rows = .self$noRows(),
                                        is3D = .self$is3D()
        )

        data$txt_gpar <<- value
      },
      clr_bar = function(value = c("none", "bottom", "top")) {
        if (missing(value)) {
          # Only show bar if there is actually something to show
          if (.self$is3D()) {
            if (arrow_type == "gradient" &&
                length(clr_bar_clrs) == 2) {
              if (!is.null(data$clr_bar)) {
                return(data$clr_bar)
              }

              return("bottom")
            }
          }

          return("none")
        }

        data$clr_bar <<- match.arg(value)
      },
      clr_bar_clrs = function(value) {
        if (missing(value)) {
          if (!is.null(data$clr_bar_clrs)) {
            return(data$clr_bar_clrs)
          }

          if (.self$is3D()) {
            return(fill_clr[[1]][1, ])
          }

          return(fill_clr[[1]][1])
        }

        data$clr_bar_clrs <<- value
      },
      clr_bar_txt_clr = function(value) {
        if (missing(value)) {
          if (!is.null(data$clr_bar_txt_clr)) {
            return(data$clr_bar_txt_clr)
          }

          if (.self$is3D()) {
            return(txt_clr[[1]][1, ])
          }

          return(rep(txt_clr[[1]][1], length.out = 2))
        }

        data$clr_bar_txt_clr <<- rep(value, length.out = 2)
      },
      clr_bar_cex = function(value) {
        if (missing(value)) {
          if (!is.null(data$clr_bar_cex)) {
            return(data$clr_bar_cex)
          }

          return(box_cex / 3)
        }

        data$clr_bar_cex <<- value
      },
      clr_bar_subspace = function(value) {
        if (missing(value)) {
          if (!is.null(data$clr_bar_subspace)) {
            return(data$clr_bar_subspace)
          }

          return(c(0, 1))
        }

        if (length(value) > 2 &&
            !is.numeric(value)) {
          stop("The color subspace needs to be a numeric vector of length between 1 and 2")
        }
        if (is.list(value)) {
          value <- unlist(value)
        }
        if (any(sapply(value, function(x) (x > 1 || x < 0)))) {
          stop("The color subspace is only defined between 1 and 0")
        }

        if (length(value) == 1) {
          if (value > 0.5) {
            value <- c((1 - value) / 2, 1 - (1 - value) / 2)
          } else {
            value <- c(value, 1 - value)
          }
        } else if (length(unique(value)) == 1) {
          stop("You have provided an undefined color space - the two values should be two unique values between 0 and 1")
        } else {
          value <- sort(value)
        }
        data$clr_bar_subspace <<- value
      },
      clr_bar_labels = function(value) {
        if (missing(value)) {
          if (length(.self$getDim()) == 2) {
            return(NULL)
          }

          if (!is.null(data$clr_bar_labels)) {
            return(data$clr_bar_labels)
          }

          return(dimnames(transitions[[1]])[[3]])
        }

        data$clr_bar_labels <<- value
      },
      title = function(value) {
        if (missing(value)) {
          return(data$title)
        }

        if (is.null(value) ||
            nchar(value) == 0) {
          data$title <<- NULL
        } else {
          data$title <<- value
        }
      },
      min_lwd = function(value) {
        if (missing(value)) {
          if (!is.null(data$min_lwd)) {
            return(data$min_lwd)
          }

          return(unit(1, "pt"))
        }
        data$min_lwd <<- value
      },
      max_lwd = function(value) {
        if (missing(value)) {
          if (!is.null(data$max_lwd)) {
            return(data$max_lwd)
          }

          return(unit(5, "mm"))
        }
        data$max_lwd <<- value
      },
      title_cex = function(value) {
        if (missing(value)) {
          if (!is.null(data$title_cex)) {
            return(data$title_cex)
          }

          return(box_label_cex * 1.2)
        }

        if (!is.numeric(value)) {
          stop("Only numeric cex values are accepted")
        }

        data$title_cex <<- value
      },
      skip_shadows = function(value) {
        if (missing(value)) {
          if (!is.null(data$skip_shadows)) {
            return(data$skip_shadows)
          }

          return(FALSE)
        }

        if (!is.logical(value)) {
          stop("Only logical values are accepted for skip_shadows")
        }

        data$skip_shadows <<- value
      },
      mar = function(value) {
        if (missing(value)) {
          if (!is.null(data$mar)) {
            return(data$mar)
          }
          if (is.null(data$mar)) {
            return(unit(rep(3, times = 4), "mm"))
          }
        }

        if (!is.unit(value)) {
          value <- unit(value, "npc")
        }

        if (length(value) == 1) {
          tmp <- value
          for (i in 1:3) {
            tmp <- unit.c(tmp, value)
          }
          value <- tmp
        }

        if (length(value) != 4) {
          stop("There are 4 margins, you have supplied '", length(value), "'")
        }

        data$mar <<- value
      },
      lwd_prop_type = function(value = c("set", "all", "box")) {
        if (missing(value)) {
          if (!is.null(data$lwd_prop_type)) {
            return(data$lwd_prop_type)
          }

          return("all")
        }

        data$lwd_prop_type <<- match.arg(value)
      }
    ),
    methods = list(
      initialize = function(transitions, label, txt, fill_clr, txt_clr, txt_gpar, id, ...) {
        "Set up a Transition object. The \\code{transitions} should be a 2D or 3D matrix
        as defined in the \\code{$addTransitions} section and not as later internally stored."
        if (missing(transitions)) {
          stop("You must provide a transition matrix when creating a Transition object")
        }

        if (is.character(transitions) &&
            all(transitions == "copy")) {
          return(callSuper(...))
        }

        if (missing(id)) {
          option_name <- "Gmisc.transitionClassCounter"
          last_id_no <- getOption(option_name)
          if (is.null(last_id_no)) {
            last_id_no <- 0
          }
          current_id_no <- last_id_no + 1

          args <- list()
          args[option_name] <- 0
          do.call(options, args)
          getOption(option_name)

          .self$id <- paste0("tc_", current_id_no)
        } else if (is.character(id)) {
          .self$id <- id
        } else {
          stop("The id has to be a string, you provided: ", id)
        }

        .self$addTransitions(transitions, label, txt = txt, fill_clr = fill_clr, txt_clr = txt_clr, txt_gpar = txt_gpar)
        callSuper(...)
      },
      copy = function(shallow = FALSE) {
        "A custom \\code{$copy} function as the initialize requires a transitions argument"
        def <- .refClassDef
        value <- new(def, "copy")
        vEnv <- as.environment(value)
        selfEnv <- as.environment(.self)
        # This object stores everything in the data field
        field <- "data"
        if (shallow) {
          assign(field, get(field, envir = selfEnv), envir = vEnv)
        } else {
          current <- get(field, envir = selfEnv)
          if (is(current, "envRefClass")) {
            current <- current$copy(FALSE)
          }
          assign(field, current, envir = vEnv)
        }
        value
      },
      addTransitions = function(mtrx, label, txt, fill_clr, txt_clr, txt_gpar) {
        "Add a transition matrix. The input has to be a numerical matrix between 2 and 3 dimensions.
        If you don't provide the txt field the box' text field will be deduced from the transition matrix'
        dimnames. The fill_clr and txt_clr are passed on to the \\code{addBoxStyle} function."

        if (is.null(dimnames(mtrx)) ||
            any(sapply(dimnames(mtrx), is.null))) {
          stop(
            "Your transition matrix must have dimension names.",
            " If you use table() for generating your transition matrix",
            " it should automatically add the names.",
            " This feature is for making sure that the names don't",
            " accidentally differ between matrices."
          )
        }

        if (length(transitions) > 0) {
          #           if (!is.null(attr(mtrx, "transition"))) {
          #             transitions <<- list(mtrx)
          #             return()
          #           }
          if (.self$noRows("last") !=
              nrow(mtrx)) {
            stop(
              "The number of elements within the new matrix must be equal to the previous matrix.",
              " You have provided '", nrow(mtrx), "' elements",
              " while there are previously '", .self$noRows("last"), "' elements."
            )
          } else if (sum(.self$boxSizes("last") - rowSums(mtrx)) > .Machine$double.eps * 10 * nrow(mtrx)) {
            stop(
              "You have provided a transition matrix starting with the sizes ", prPasteVec(rowSums(mtrx)),
              " while the previous transition matrix resulted in sizes ", prPasteVec(.self$boxSizes("last")), ".",
              " These two should be equal."
            )
          }

          # Clean from empty rows/columns
          empty_rows <- rowSums(mtrx) == 0
          empty_cols <- colSums(mtrx) == 0
          if (.self$is3D()) {
            # Merge 2D matrix into 1D-vector
            empty_cols <- rowSums(!empty_cols) == 0
            # This is not an issue with rows
          }
          if (any(empty_rows)) {
            if (.self$is3D()) {
              mtrx <- mtrx[!empty_rows, , , drop = FALSE]
            } else {
              mtrx <- mtrx[!empty_rows, , drop = FALSE]
            }
          }

          if (any(empty_cols)) {
            if (.self$is3D()) {
              mtrx <- mtrx[, !empty_cols, , drop = FALSE]
            } else {
              mtrx <- mtrx[, !empty_cols, drop = FALSE]
            }
          }

          # The new matrix must match up to the previous
          prev_match <-
            rownames(mtrx) %in% colnames(.self$getTransitionSet("last"))
          if (any(!prev_match)) {
            stop(
              "Could not find the names ",
              paste(rownames(mtrx)[!prev_match], collapse = ", "),
              " in the previous transition matrix' column names: ",
              paste(colnames(.self$getTransitionSet("last")), collapse = ", ")
            )
          }

          # Align the column order
          prev_order <-
            sapply(
              rownames(mtrx),
              function(n) which(n == colnames(.self$getTransitionSet("last")))
            )
          if (.self$is3D()) {
            mtrx <- mtrx[order(prev_order), , ]
          } else {
            mtrx <- mtrx[order(prev_order), ]
          }

          # Merge the two transitions
          mtrx <- c(transitions, list(mtrx))

          # Update box width
          raw_width <- convertUnit(box_width, unitTo = "npc", axisFrom = "x", axisTo = "x", valueOnly = TRUE)
          shrinkage <- (.self$noCols() * 2 - 1) / ((.self$noCols() + 1) * 2 - 1)
          bw <- unit(raw_width * shrinkage, units = "npc")
        } else {
          mtrx <- list(mtrx)
          bw <- unit(1 / 4, units = "npc")
        }
        attr(mtrx, "transition") <- TRUE

        # Set the transition matrix and the new width
        # Note that more sanity checks are performed when setting the
        # transitions variable
        transitions <<- mtrx
        box_width <<- bw

        # Now adding a label
        if (missing(label)) {
          label <- .self$noCols()
          if (label == 2) {
            label <- 1:2
          }
        }
        if (.self$noCols() > 2) {
          box_label <<- c(box_label, label)
        } else {
          box_label <<- label
        }

        # Next we add txt
        if (missing(txt)) {
          if (.self$noCols() > 2) {
            txt <- colnames(.self$getTransitionSet("last"))
          } else {
            txt <- list(
              rownames(transitions[[1]]),
              colnames(transitions[[1]])
            )
          }
        } else {
          # Prep txt argument
          if (is.vector(txt)) {
            txt <- list(txt)
          } else if (is.matrix(txt)) {
            if (ncol(txt) == 1) {
              txt <- list(txt[, 1])
            } else if (ncol(txt) == 2) {
              if (.self$noCols() > 2) {
                stop("Can only have two txt columns if this is the first transition matrix")
              }
              txt <-
                list(
                  txt[, 1],
                  txt[, 2]
                )
            } else {
              stop("Invalid number of txt columns")
            }
          }

          # If this is the first text we should have a
          # text for both columns and its assumed to be the
          # same
          if (length(txt) == 1 &&
              .self$noCols == 2) {
            txt <- rep(txt, 2)
          }
        }

        if (!is.list(txt)) {
          txt <- list(txt)
        }

        if (.self$noCols() > 2) {
          box_txt <<- c(box_txt, txt)
        } else {
          box_txt <<- txt
        }

        # Now add the colors
        .self$addBoxStyle(
          fill = fill_clr,
          txt = txt_clr,
          gpar = txt_gpar
        )

        invisible(mtrx)
      },
      getTransitionSet = function(no, reduce_dim = FALSE) {
        "Gets a specific set of transitions. If the \\code{reduce_dim} is set
        to TRUE it will only return a 2-dimensional matrix even if the original
        has a 3rd proportions dimension"
        if (no == "last") {
          no <- length(transitions)
        }

        if (no >= .self$noCols() ||
            no < 1) {
          stop("You can only select transition sets ranging from 1 to ", .self$noCols() - 1)
        }

        set <- transitions[[no]]
        if (reduce_dim &&
            length(dim(set)) == 3) {
          set <- set[, , 1, drop = FALSE] + set[, , 2, drop = FALSE]
          set <- set[, , 1]
        }
        return(set)
      },
      addBoxStyle = function(fill, txt, gpar) {
        "Adds colors or extends existing one so that they match the transition matrix.
        The fill corresponds to the fill_clr and txt corresponds to the txt_clr. If
        the colors are missing and the transitions consist of only two columns the default
        colors will be used. If the matrix is being extended and these values are missing the
        values from the previous last column will be used for the default columns."

        addStyle <- function(style,
                             existing_style,
                             generator_2d,
                             generator_3d = generator_2d,
                             is3D = .self$is3D()) {
          # Resolve styles if there already exists one
          if (!is.null(existing_style)) {
            return(prTcMatchClr(add = style,
                                org = existing_style,
                                no_cols = .self$noCols(),
                                no_rows = .self$noRows(),
                                is3D = is3D))
          }

          if (is3D) {
            style_generator <- generator_3d
          } else {
            style_generator <-  generator_2d
          }

          if (missing(style) && .self$noCols() == 2) {
            # This is the color initialization
            style <- lapply(1:.self$noCols(), style_generator)
          } else if (.self$noCols() == 2) {
            is_single_element <- function(x) inherits(x, "gpar") || length(x) == 1
            if (is_single_element(style)) {
               style <- lapply(1:.self$noCols(), style_generator, default = style)
            } else if (length(style) == 2 && all(sapply(style, is_single_element))) {
               style <- lapply(1:.self$noCols(), function(i) style_generator(i, style[[i]]))
            }
          }
          style
        }

        fill_clr <<- addStyle(style = fill,
                              existing_style = fill_clr,
                              generator_2d = function(col, default = "darkgreen") cbind(rep(default, .self$noRows(col))),
                              generator_3d = function(col, default = c("#fdc086", "#386cb0")) cbind(rep(default[[1]], .self$noRows(col)),
                                                                                                    rep(default[[2]], .self$noRows(col))))

        txt_clr <<- addStyle(style = txt,
                             existing_style = txt_clr,
                             generator_2d = function(col, default = "#ffffff") rep(default, .self$noRows(col)),
                             generator_3d = function(col, default = c("#000000", "#ffffff")) cbind(rep(default[[1]], .self$noRows(col)),
                                                                                                   rep(default[[2]], .self$noRows(col))))

        txt_gpar <<- addStyle(style = gpar,
                              existing_style = txt_gpar,
                              generator_2d = function(col, default = list()) rep(list(default), .self$noRows(col)),
                              is3D = FALSE)
      },
      getDim = function() {
        "Gets the current dimensions of the transitions"
        transition_dim <- dim(transitions[[1]])
        if (length(transitions) > 1) {
          if (all(sapply(transitions, nrow) == transition_dim[1])) {
            transition_dim[2] <- length(transitions) + 1
          } else {
            transition_dim[2] <- "x"
          }
        }
        return(transition_dim)
      },
      is3D = function() {
        return(length(.self$getDim()) == 3)
      },
      noRows = function(no) {
        "Gets the number of boxes in each row.
        If multiple rows the number of rows may differ
        betwen each transition matrix we therefore need
        to specify what transitions that we refer to.
        If no value is specified it returns all of them."
        no_rows <-
          c(
            sapply(transitions, nrow),
            ncol(tail(transitions, 1)[[1]])
          )

        if (missing(no)) {
          return(no_rows)
        }

        if (no == "last") {
          return(tail(no_rows, 1))
        }

        if (no < 0 ||
            no > length(transitions) + 1) {
          stop("Invalid column specified")
        }

        return(no_rows[no])
      },
      noCols = function() {
        "Gets the number of columns, i.e. the number of transitions"
        return(length(transitions) + 1)
      },
      arrowWidths = function(set_no, add_width) {
        "Retrieves the details regarding arrow sizes for each arrow within the transition
        group"
        trnstn_set <- .self$getTransitionSet(set_no, reduce_dim = TRUE)

        # Get the maximum transition within the entire plot
        if (lwd_prop_type == "all") {
          max_flow <- -Inf
          for (i in 2:.self$noCols()) {
            mtrx <- trnstnSizes(i - 1)
            max_flow <- max(max_flow, mtrx)
          }
        } else if (lwd_prop_type == "set") {
          max_flow <- max(trnstn_set)
        }

        internal.unit <- "mm"
        raw_max_lwd <- max_lwd
        if (is.unit(raw_max_lwd)) {
          raw_max_lwd <- convertHeight(raw_max_lwd,
                                       unitTo = internal.unit,
                                       valueOnly = TRUE
          )
        }
        raw_min_lwd <- min_lwd
        if (is.unit(raw_min_lwd)) {
          raw_min_lwd <- convertHeight(raw_min_lwd,
                                       unitTo = internal.unit,
                                       valueOnly = TRUE
          )
        }
        arrows <- list()
        for (org_row in 1:.self$noRows(set_no)) {
          if (lwd_prop_type == "box") {
            max_flow <- max(trnstn_set[org_row, ])
          }

          arrows[[org_row]] <-
            lapply(1:.self$noRows(set_no + 1), function(x) list())

          for (targ_row in 1:.self$noRows(set_no + 1)) {
            # Calculate line width
            lwd <- raw_max_lwd * trnstn_set[org_row, targ_row] / max_flow
            if (lwd < raw_min_lwd) {
              # Lines that are below the minimum should not be shown
              next
            }

            adjusted_lwd <- lwd
            if (!missing(add_width)) {
              if ("unit" %in% class(add_width)) {
                adjusted_lwd <- convertY(unit(lwd, "npc") + add_width, unitTo = "npc", valueOnly = TRUE)
              } else if (add_width > 1) {
                adjusted_lwd <- lwd * add_width
              }
            }

            arrows[[org_row]][[targ_row]]$lwd <- unit(adjusted_lwd, internal.unit)
            arrows[[org_row]][[targ_row]]$adj_lwd <- unit(adjusted_lwd, internal.unit)
          }
        }
        return(arrows)
      },
      trnstnSizes = function(set_no) {
        "Gets the transitions per box as a 2D matrix. For the proportions
         it also adds an attribute \\code{attr('props', prop_mtrx)} that
         is a 2D matrix with the corresponding proportions."
        if (set_no >= .self$noCols()) {
          return(NULL)
        }

        mtrx <- .self$getTransitionSet(set_no)
        if (.self$is3D()) {
          props <- mtrx[, , 1] / (mtrx[, , 1] + mtrx[, , 2])
          # Remove the third dimension
          mtrx <- mtrx[, , 1] + mtrx[, , 2]
          attr(mtrx, "props") <- props
        }

        return(mtrx)
      },
      boxSizes = function(col) {
        "Gets the size of the boxes. The \\code{col} argument should
        be either an integer or 'last'"
        if (is.character(col)) {
          if (col == "last") {
            col <- .self$noCols()
          }
        } else if (is.numeric(col)) {
          if (!col %in% 1:.self$noCols()) {
            stop(
              "The column must be within the available columns 1 to ", .self$noCols(),
              " while you have requested '", col, "'"
            )
          }
        }

        if (col == .self$noCols()) {
          # Get last transition matrix and extract the column sums from that one
          mtrx <- .self$getTransitionSet("last")
          if (.self$is3D()) {
            raw_sizes <- apply(mtrx, 3, colSums)
            sizes <- rowSums(raw_sizes)
            attr(sizes, "prop") <-
              raw_sizes[, 1] / sizes
            return(sizes)
          } else if (length(.self$getDim()) == 2) {
            return(colSums(mtrx))
          } else {
            stop("Invalid dimensionality of transition matrix: '", paste(.self$getDim(), collapse = "', '"), "'")
          }
        } else {
          mtrx <- .self$getTransitionSet(col)
          if (.self$is3D()) {
            raw_sizes <- apply(mtrx, 3, rowSums)
            sizes <- rowSums(raw_sizes)
            attr(sizes, "prop") <-
              raw_sizes[, 1] / sizes
            return(sizes)
          } else if (length(.self$getDim()) == 2) {
            return(rowSums(mtrx))
          } else {
            stop("Invalid dimensionality of transition matrix: '", paste(.self$getDim(), collapse = "', '"), "'")
          }
        }
      },
      getYProps = function(col) {
        "Gets the proportions after removing the \\code{vertical_space}
         between the boxes"
        vertical_sizes <- .self$boxSizes(col)
        (1 - convertY(vertical_space,
                      unitTo = "npc",
                      valueOnly = TRUE
        )) *
          vertical_sizes / sum(vertical_sizes)
      },
      boxPositions = function(col) {
        "The box positions as a list with scalars for the positions:
        \\enumerate{
         \\item \\emph{x} The center x-position
         \\item \\emph{y} The center y-position
         \\item \\emph{right} The right edge
         \\item \\emph{left} The left edge
         \\item \\emph{top} The top edge
         \\item \\emph{bottom} The bottom edge
         \\item \\emph{height} The box height
         \\item \\emph{width} The box width
         \\item \\emph{unit} The unit used for the values (npc)
        }"
        raw_width <- convertWidth(box_width, unitTo = "npc", valueOnly = TRUE)
        space_between <- (1 - raw_width * .self$noCols()) / (.self$noCols() - 1)
        x_offset <- (raw_width + space_between) * (col - 1)
        proportions <- .self$getYProps(col)
        raw_v_space <- convertY(vertical_space,
                                unitTo = "npc",
                                valueOnly = TRUE
        )
        y_offset <- 1
        bx <- list()
        for (i in 1:length(proportions)) {
          bx[[i]] <-
            list(
              # Center
              y = y_offset - proportions[i] / 2,
              x = x_offset + raw_width / 2,
              # Borders
              left = x_offset,
              right = x_offset + raw_width,
              top = y_offset,
              bottom = y_offset - proportions[i],
              # Size
              height = proportions[i],
              width = raw_width,
              unit = "npc"
            )
          if (proportions[i] > 0) {
            y_offset <- y_offset - sum(proportions[i], raw_v_space / (sum(proportions > 0) - 1))
          }
        }
        # If there is only one box then we center that box
        if (length(proportions) == 1) {
          box[[1]]$y <- 0.5
          box[[1]]$top <- box[[1]]$height + raw_v_space / 2
          box[[1]]$bottom <- raw_v_space / 2
        }
        return(bx)
      },
      getViewportName = function(name) {
        paste0(.self$id, "_", name)
      },
      render = function(new_page = TRUE) {
        "Call this to render the full graph. The \\code{new_page} argument
        is for creating a new plot, set this to \\code{FALSE}
        if you want to combine this plot with another or if you have
        additional viewports that you intend to use."
        if (new_page) {
          grid.newpage()
        }
        no_upview_ports <- 0

        prPushMarginViewport(
          bottom = mar[1],
          left = mar[2],
          top = mar[3],
          right = mar[4],
          name = .self$getViewportName("main_margins")
        )
        no_upview_ports <- no_upview_ports + 2

        if (!is.null(title)) {
          prGridPlotTitle(title, title_cex, cex_mult = 1)
          no_upview_ports <- no_upview_ports + 2
        }

        raw_width <- convertWidth(box_width, unitTo = "npc", valueOnly = TRUE)
        space_between <- (1 - raw_width * .self$noCols()) / (.self$noCols() - 1)
        .self$render_box_label(raw_width = raw_width,
                               space_between = space_between,
                               no_upview_ports = no_upview_ports)

        if (clr_bar != "none") {
          bar_height <- unit(1, "cm")
          colorAxis <- xaxisGrob(
            at = c(0, .25, .5, .75, 1),
            label = sprintf("%d %%", c(0, .25, .5, .75, 1) * 100),
            main = FALSE, gp = gpar(cex = clr_bar_cex)
          )

          # Add a little space to the actual height
          axis_height <- grobHeight(colorAxis)

          if (clr_bar == "bottom") {
            axis_height <- axis_height
            clr_bar_heights <- unit.c(
              unit(1, "npc") -
                axis_height -
                bar_height,
              unit(3, "mm"),
              axis_height,
              bar_height
            )
            bar_pos <- 4
            graph_pos <- 1
          } else {
            clr_bar_heights <- unit.c(
              axis_height,
              bar_height,
              unit(3, "mm"),
              unit(1, "npc") -
                axis_height -
                bar_height
            )
            bar_pos <- 2
            graph_pos <- 4
          }

          bar_layout <- grid.layout(
            nrow = 4, ncol = 3,
            heights = clr_bar_heights,
            widths = unit.c(
              box_width,
              unit(1, "npc") -
                box_width -
                box_width,
              box_width
            )
          )

          pushViewport(viewport(
            layout = bar_layout,
            name = .self$getViewportName("Bar_layout")
          ))

          pushViewport(viewport(
            layout.pos.row = bar_pos,
            layout.pos.col = 2,
            name = .self$getViewportName("Color_bar")
          ))

          bar_clrs <- prTpGetColors(clr_bar_clrs, space = clr_bar_subspace)
          grid.raster(t(as.raster(bar_clrs)), width = 1, height = 1, interpolate = FALSE)
          grid.draw(colorAxis)

          if (!is.null(clr_bar_labels)) {
            # The height is actually oblivious to upper case and lower case letters
            lab_height <- convertY(unit(1, units = "strheight", clr_bar_labels), "npc", valueOnly = TRUE)
            lab_cex_adjusted <- 1 / (lab_height * 2)

            lab_margin <- .05
            left <- textGrob(clr_bar_labels[1],
                             x = 0 + lab_margin,
                             just = "left",
                             y = .5,
                             gp = gpar(
                               cex = lab_cex_adjusted,
                               col = clr_bar_txt_clr[1]
                             )
            )
            right <- textGrob(clr_bar_labels[2],
                              x = 1 - lab_margin,
                              just = "right",
                              y = .5,
                              gp = gpar(
                                cex = lab_cex_adjusted,
                                col = clr_bar_txt_clr[2]
                              )
            )
            grid.draw(left)
            grid.draw(right)
          }
          popViewport()

          pushViewport(viewport(
            layout.pos.row = graph_pos,
            layout.pos.col = 1:3,
            name = .self$getViewportName("Main_exc_bar")
          ))
          no_upview_ports <- no_upview_ports + 2
        }

        .self$render_boxes(raw_width = raw_width, no_upview_ports = no_upview_ports)
      },
      render_box_label = function(raw_width, space_between, no_upview_ports) {
        if (!is.null(box_label)) {
          raw_height <- 0
          for (i in 1:noCols()) {
            raw_height <-
              unit(1, "strheight", box_label[i]) %>%
              convertHeight(unitTo = "npc", valueOnly = TRUE) %>%
              max(raw_height)
          }

          raw_mm <- convertHeight(unit(raw_height, "npc"), unitTo = "mm", valueOnly = TRUE)

          widths <- c()
          for (i in 1:.self$noCols()) {
            widths %<>% c(raw_width)
            if (i != .self$noCols()) {
              widths %<>% c(space_between)
            }
          }

          box_label_heights <- c(raw_height * box_label_cex * 2.5, 1 - raw_height * 2.5 * box_label_cex)
          label_pos <- 1
          graph_pos <- 2
          if (box_label_pos == "bottom") {
            box_label_heights <- rev(box_label_heights)
            label_pos <- 2
            graph_pos <- 1
          }

          labels <- list()
          for (i in 1:noCols()) {
            labels[[i]] <- textGrob(box_label[i], gp = gpar(cex = box_label_cex))
          }

          pushViewport(viewport(
            layout = grid.layout(
              nrow = 2, ncol = length(widths),
              heights = unit(box_label_heights, "npc"),
              widths = unit(widths, "npc")
            ),
            name = .self$getViewportName("box_label_grid")
          ))
          no_upview_ports <- no_upview_ports + 1

          for (i in 1:.self$noCols()) {
            col_no <- i + (i - 1)
            pushViewport(viewport(
              layout.pos.row = label_pos,
              layout.pos.col = col_no,
              name = .self$getViewportName(paste0("box_label_grid_", label_pos, ":", col_no))
            ))
            grid.draw(labels[[i]])
            upViewport()
          }

          pushViewport(viewport(
            layout.pos.row = graph_pos,
            layout.pos.col = 1:length(widths),
            name = .self$getViewportName(paste0("box_graph_section_", graph_pos, ":1-", col_no))
          ))
          no_upview_ports <- no_upview_ports + 1
        }
      },
      render_boxes = function(raw_width, no_upview_ports) {
        shift <- unit(raw_width * .02, "snpc")
        pushViewport(viewport(
          x = unit(0.5, "npc") + shift,
          y = unit(0.5, "npc") - shift,
          height = unit(1, "npc") - shift - shift,
          width = unit(1, "npc") - shift - shift,
          name = .self$getViewportName("shadows")
        ))
        upViewport()
        pushViewport(viewport(
          x = unit(0.5, "npc") - shift,
          y = unit(0.5, "npc") + shift,
          height = unit(1, "npc") - shift - shift,
          width = unit(1, "npc") - shift - shift,
          name = .self$getViewportName("regular")
        ))
        upViewport()


        for (col in 1:.self$noCols()) {
          proportions <- .self$getYProps(col)

          bx_pos <- .self$boxPositions(col)

          box_args <- list(
            box_positions = bx_pos,
            proportions = as.vector(proportions),
            fill = rep(grey(level = .3), times = .self$noRows(col)),
            txt = rep("", times = .self$noRows(col)),
            txt_clr = rep(grey(level = .3), times = .self$noRows(col)),
            cex = box_cex,
            txt_gpar = txt_gpar[[col]]
          )

          if (!.self$skip_shadows) {
            seekViewport(.self$getViewportName("shadows"))
            fastDoCall(prTcPlotBoxColumn, box_args)
            upViewport()
          }

          seekViewport(.self$getViewportName("regular"))
          box_args[["proportions"]] <- proportions
          box_args[["fill"]] <- fill_clr[[col]]
          box_args[["txt"]] <- box_txt[[col]]
          box_args[["txt_clr"]] <- txt_clr[[col]]

          # Output the transitions
          if (col < .self$noCols()) {
            trnstn_set <- .self$getTransitionSet(col)

            prTcPlotArrows(trnstn_set,
                           widths = .self$arrowWidths(col),
                           type = arrow_type,
                           clr = arrow_clr[[col]],
                           rez = arrow_rez,
                           origin_boxes = bx_pos,
                           target_boxes = .self$boxPositions(col + 1),
                           left_box_clrs = box_args[["fill"]],
                           max_flow = max_flow,
                           min_width = min_width,
                           max_width = max_width,
                           clr_bar_subspace = clr_bar_subspace
            )
          }

          do.call(prTcPlotBoxColumn, box_args)
          upViewport()
        }

        if (no_upview_ports > 0) {
          upViewport(no_upview_ports)
        }
      }
    )
  )

library(testthat)
context("Transition class")

test_that("Check initialization, copy, and dimensions", {
  expect_error(getRefClass("Transition")$new(),
    regexp = "transition matrix"
  )

  trn_mtrx <- matrix(1, ncol = 2, nrow = 2, dimnames = list(LETTERS[1:2], letters[1:2]))
  a <- getRefClass("Transition")$new(trn_mtrx)
  expect_equal(length(a$transitions), 1)
  expect_equal(
    a$getDim(),
    dim(trn_mtrx)
  )

  expect_equal(
    a$noCols(),
    length(a$fill_clr)
  )
  expect_equal(
    a$noCols(),
    length(a$txt_clr)
  )
  expect_equal(
    a$noCols(),
    length(a$box_txt)
  )

  expect_error(a$addTransitions(trn_mtrx))

  trn_mtrx_2 <- matrix(1, ncol = 2, nrow = 2, dimnames = list(letters[1:2], LETTERS[3:4]))
  a$addTransitions(trn_mtrx_2)
  expect_equal(
    a$noCols(),
    3
  )

  trn_mtrx_3 <- matrix(1, ncol = 2, nrow = 2, dimnames = list(LETTERS[3:4], LETTERS[3:4]))
  b <- a$copy()
  a$addTransitions(trn_mtrx_3)
  expect_equal(
    b$getDim(),
    c(2, 3)
  )
  expect_equal(
    length(b$transitions),
    2
  )
  expect_equal(
    length(a$transitions),
    3
  )
})

test_that("Check box size", {
  trn_mtrx <- matrix(1:4, ncol = 2)
  rownames(trn_mtrx) <- c("Test A", "Test B")
  colnames(trn_mtrx) <- c("Test A", "Test B")
  a <- getRefClass("Transition")$new(trn_mtrx)
  expect_error(a$boxSizes())
  expect_equal(
    a$boxSizes(1),
    rowSums(trn_mtrx)
  )
  expect_equal(
    a$boxSizes(2),
    colSums(trn_mtrx)
  )

  trn_mtrx <- array(1:8, dim = c(2, 2, 2), dimnames = list(
    c("Test A", "Test B"),
    c("Test A", "Test B"),
    c("Test A", "Test B")
  ))
  a <- getRefClass("Transition")$new(trn_mtrx)
  expect_equivalent(
    a$boxSizes(1),
    rowSums(trn_mtrx[, , 1]) + rowSums(trn_mtrx[, , 2])
  )
  expect_equivalent(
    attr(a$boxSizes(1), "prop"),
    rowSums(trn_mtrx[, , 1]) / (rowSums(trn_mtrx[, , 1]) + rowSums(trn_mtrx[, , 2]))
  )
  expect_equivalent(
    a$boxSizes(2),
    colSums(trn_mtrx[, , 1]) + colSums(trn_mtrx[, , 2])
  )
  expect_equivalent(
    attr(a$boxSizes(2), "prop"),
    colSums(trn_mtrx[, , 1]) / (colSums(trn_mtrx[, , 1]) + colSums(trn_mtrx[, , 2]))
  )
})

test_that("Check advanced matrix dimensions", {
  # Setup test-data
  set.seed(1)
  library(magrittr)
  n <- 100
  data <-
    data.frame(
      Sex = sample(c("Male", "Female"),
        size = n,
        replace = TRUE,
        prob = c(.4, .6)
      ),
      Charnley_class = sample(c("A", "B", "C"),
        size = n,
        replace = TRUE
      )
    )
  getProbs <- function(Chrnl_name) {
    prob <- data.frame(
      A = 1 / 6 +
        (data$Sex == "Male") * .25 +
        (data$Sex != "Male") * -.25 +
        (data[[Chrnl_name]] %in% "B") * -.5 +
        (data[[Chrnl_name]] %in% "C") * -2,
      B = 2 / 6 +
        (data$Sex == "Male") * .1 +
        (data$Sex != "Male") * -.05 +
        (data[[Chrnl_name]] == "C") * -2,
      C = 3 / 6 +
        (data$Sex == "Male") * -.25 +
        (data$Sex != "Male") * .25
    )

    # Remove negative probabilities
    t(apply(prob, 1, function(x) {
      if (any(x < 0)) {
        x <- x - min(x) + .05
      }
      x
    }))
  }

  Ch_classes <- c("Charnley_class")
  Ch_classes %<>% c(sprintf("%s_%dyr", Ch_classes, c(1, 2, 6)))
  for (i in 1:length(Ch_classes)) {
    if (i == 1) {
      next
    }

    data[[Ch_classes[i]]] <-
      apply(getProbs(Ch_classes[i - 1]), 1, function(p) {
        sample(c("A", "B", "C"),
          size = 1,
          prob = p
        )
      }) %>%
      factor(levels = c("A", "B", "C"))
  }

  test_3D <- with(data, table(Charnley_class, Charnley_class_1yr, Sex))
  transitions <- getRefClass("Transition")$new(test_3D)

  expect_equivalent(
    transitions$getDim(),
    dim(test_3D)
  )

  expect_equivalent(
    dim(transitions$fill_clr[[1]]),
    c(transitions$noRows(1), 2)
  )
  expect_equivalent(
    dim(transitions$txt_clr[[1]]),
    c(
      transitions$noRows(1),
      2
    )
  )

  expect_equivalent(
    sapply(transitions$box_txt, length),
    transitions$noRows()
  )

  add_3D <- with(data, table(Charnley_class_1yr, Charnley_class_2yr, Sex))
  transitions$addTransitions(add_3D)

  expect_equal(
    dim(transitions$fill_clr[[1]]),
    c(
      transitions$noRows(1),
      2
    )
  )
  expect_equal(
    dim(transitions$txt_clr[[1]]),
    c(
      transitions$noRows(1),
      2
    )
  )

  expect_equivalent(
    sapply(transitions$box_txt, length),
    transitions$noRows()
  )


  data$Charnley_class_6yr[data$Charnley_class_6yr == "A"] <- "B"
  add_3D <- with(data, table(Charnley_class_2yr, Charnley_class_6yr, Sex))
  transitions$addTransitions(add_3D)

  expect_equal(
    transitions$noCols(),
    4
  )
})

#' Create a composite box with header and body text
#'
#' Creates a \code{\link[grid:grid.grob]{grob}} box with a centered header and
#' left-justified body text. Useful for flowchart elements that combine a title
#' with bullet points or multi-line descriptions.
#'
#' @param header The header text (centered, typically bold).
#' @param body The body text (left-justified, typically with bullets or multiple lines).
#' @param y The y position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.
#' @param x The x position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.
#' @param width The box automatically adapts the size but you can force by specifying the width
#' @param height The box automatically adapts the size but you can force by specifying the height
#' @param bjust The justification for the box: left, center, right, top or bottom.
#'  See the \code{just} option for the \code{\link[grid]{viewport}}
#' @param header_gp The \code{\link[grid]{gpar}} style to apply to the header text.
#'  Defaults to bold text at slightly larger size.
#' @param body_gp The \code{\link[grid]{gpar}} style to apply to the body text.
#' @param box_gp The \code{\link[grid]{gpar}} style to apply to the box function of `box_fn` below.
#' @param box_fn Function to create box for the text. Parameters of `x=0.5`, `y=0.5` and `box_gp` will
#'  be passed to this function and return a \code{grob} object.
#' @param name a character identifier for the \code{grob}. Used to find the \code{grob} on the display
#'  list and/or as a child of another grob.
#'
#' @return A grob with class "box" compatible with all Gmisc flowchart helpers
#'  (\code{\link{coords}}, \code{\link{distance}}, \code{\link{connectGrob}},
#'  \code{\link{spreadVertical}}, \code{\link{alignHorizontal}}, etc.)
#' @export
#'
#' @rdname boxHeaderGrob
#' @importFrom checkmate assert_class assert checkString checkNumeric
#' @family flowchart components
#' @examples
#' library(grid)
#' grid.newpage()
#'
#' # Simple example with header and bullets
#' box1 <- boxHeaderGrob(
#'   header = "Early rehabilitation",
#'   body = paste("0-1 weeks", "• Instruction", "• Pendulum + assisted ROM", sep = "\n"),
#'   header_gp = gpar(fontsize = 11, fontface = "bold"),
#'   body_gp = gpar(fontsize = 9),
#'   box_gp = gpar(fill = "#E8F5E9", col = "#2E7D32")
#' )
#' grid.draw(box1)
#'
#' # Works with all helpers
#' box2 <- boxHeaderGrob(
#'   header = "Delayed rehabilitation",
#'   body = "0-3 weeks\n• Sling immobilisation",
#'   y = 0.3,
#'   box_gp = gpar(fill = "#FFF8E1", col = "#EF6C00")
#' )
#'
#' # Distance calculation works
#' d <- distance(box1, box2, type = "v")
#'
#' # Connect boxes
#' grid.draw(box2)
#' connectGrob(box1, box2, type = "vertical")
boxHeaderGrob <- function(header,
                          body,
                          y = unit(.5, "npc"),
                          x = unit(.5, "npc"),
                          width,
                          height,
                          bjust = "center",
                          header_gp = getOption("boxHeaderGrobTxt", default = gpar(color = "black", cex = 1)),
                          body_gp = getOption("boxBodyGrobTxt", default = gpar(color = "black", cex = 0.9)),
                          box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                          box_fn = roundrectGrob,
                          name = NULL) {
  assert(
    checkString(header),
    checkNumeric(header),
    is.language(header)
  )
  assert(
    checkString(body),
    checkNumeric(body),
    is.language(body)
  )
  assert_unit(y)
  assert_unit(x)
  assert_unit(width)
  assert_unit(height)
  assert_just(bjust)
  assert_class(header_gp, "gpar")
  assert_class(body_gp, "gpar")
  assert_class(box_gp, "gpar")

  x <- prAsUnit(x)
  y <- prAsUnit(y)

  # Larger base padding for composite boxes with multi-line content
  txt_padding <- unit(4, "mm")
  margin_after_header <- unit(4, "mm")

  # Call the box function early to collect any suggested padding attributes
  rect <- do.call(box_fn, list(x = .5, y = .5, gp = box_gp))
  extra_pad <- attr(rect, "box_fn_padding")
  if (!is.null(extra_pad)) {
    tryCatch(
      {
        txt_padding <- txt_padding + extra_pad
      },
      error = function(e) {
        # ignore silently if attribute is not a proper unit
      }
    )
  }

  # Create header grob (centered at top with padding)
  header_txt <- textGrob(
    label = header,
    x = 0.5,
    y = unit(1, "npc") - txt_padding,
    just = c("center", "top"),
    gp = header_gp,
    name = "header"
  )

  # Create body grob (left-justified at bottom with padding)
  # Position at same padding as box edge for clean alignment
  body_left_margin <- txt_padding
  body_txt <- textGrob(
    label = body,
    x = body_left_margin,
    y = unit(0, "npc") + txt_padding,
    just = c("left", "bottom"),
    gp = body_gp,
    name = "body"
  )

  # Calculate size based on both text grobs
  if (missing(width)) {
    # Width is simply the maximum of text widths plus padding on both sides
    width <- unit.pmax(grobWidth(header_txt), grobWidth(body_txt)) + txt_padding + txt_padding
  } else {
    width <- prAsUnit(width)
  }

  if (missing(height)) {
    height <- grobHeight(header_txt) + grobHeight(body_txt) + txt_padding + txt_padding + margin_after_header
  } else {
    height <- prAsUnit(height)
  }

  vp_args <- list(
    x = x,
    y = y,
    width = width,
    height = height,
    just = bjust
  )

  gl <- grobTree(
    gList(
      rect,
      header_txt,
      body_txt
    ),
    vp = do.call(viewport, vp_args),
    name = name,
    cl = "box"
  )

  structure(gl,
    coords = prCreateBoxCoordinates(viewport_data = vp_args),
    viewport_data = vp_args
  )
}

library(testthat)
library(grid)

test_that("boxHeaderGrob creates a valid box grob", {
    box <- boxHeaderGrob(
        header = "Test Header",
        body = "Line 1\nLine 2\nLine 3"
    )

    expect_s3_class(box, "box")
    expect_s3_class(box, "gTree")
    expect_true(!is.null(attr(box, "coords")))
    expect_true(!is.null(attr(box, "viewport_data")))
})

test_that("boxHeaderGrob works with coords helper", {
    box <- boxHeaderGrob(
        header = "Header",
        body = "Body text",
        x = 0.3,
        y = 0.7
    )

    c <- coords(box)
    expect_s3_class(c, "coords")
    expect_true(!is.null(c$x))
    expect_true(!is.null(c$y))
    expect_true(!is.null(c$left))
    expect_true(!is.null(c$right))
    expect_true(!is.null(c$top))
    expect_true(!is.null(c$bottom))
})

test_that("boxHeaderGrob works with distance helper", {
    box1 <- boxHeaderGrob(
        header = "Box 1",
        body = "Content 1",
        y = 0.8
    )
    box2 <- boxHeaderGrob(
        header = "Box 2",
        body = "Content 2",
        y = 0.2
    )

    d <- distance(box1, box2, type = "v")
    expect_s3_class(d, "unit")
    expect_true(inherits(d, "Gmisc_unit"))
})

test_that("boxHeaderGrob works with connectGrob", {
    box1 <- boxHeaderGrob(
        header = "Start",
        body = "Initial step",
        y = 0.7
    )
    box2 <- boxHeaderGrob(
        header = "End",
        body = "Final step",
        y = 0.3
    )

    conn <- connectGrob(box1, box2, type = "vertical")
    expect_s3_class(conn, "connect_boxes")
})

test_that("boxHeaderGrob works with spreadVertical", {
    boxes <- list(
        early = boxHeaderGrob("Early", "Step 1\nStep 2"),
        late = boxHeaderGrob("Late", "Step A\nStep B")
    )

    result <- spreadVertical(boxes)
    expect_s3_class(result, "Gmisc_list_of_boxes")
    expect_length(result, 2)

    # Check that positions were updated
    c1 <- coords(result$early)
    c2 <- coords(result$late)
    expect_false(identical(
        convertY(c1$y, "npc", valueOnly = TRUE),
        convertY(c2$y, "npc", valueOnly = TRUE)
    ))
})

test_that("boxHeaderGrob works with alignHorizontal", {
    box1 <- boxHeaderGrob(
        header = "Box 1",
        body = "Content",
        x = 0.2,
        y = 0.7
    )
    box2 <- boxHeaderGrob(
        header = "Box 2",
        body = "Content",
        x = 0.8,
        y = 0.3
    )

    # alignHorizontal should successfully align two composite boxes
    result <- alignHorizontal(box1, box2)
    expect_s3_class(result, "Gmisc_list_of_boxes")
    expect_true(length(result) > 0)
})

test_that("boxHeaderGrob accepts custom styling", {
    box <- boxHeaderGrob(
        header = "Styled Header",
        body = "Styled body",
        header_gp = gpar(fontsize = 12, fontface = "bold", col = "blue"),
        body_gp = gpar(fontsize = 9, col = "darkgray"),
        box_gp = gpar(fill = "#E8F5E9", col = "#2E7D32", lwd = 2)
    )

    expect_s3_class(box, "box")
    # Verify grob tree contains header and body
    expect_equal(length(box$children), 3) # rect, header, body
})

test_that("boxHeaderGrob works in user's rehabilitation flowchart example", {
    # Simulate the user's use case
    early_gp_header <- list(
        box_gp = gpar(fill = "#E8F5E9", col = "#2E7D32", lwd = 1.4),
        header_gp = gpar(fontsize = 10.5, fontface = "bold"),
        body_gp = gpar(fontsize = 9)
    )

    box <- do.call(boxHeaderGrob, c(
        list(
            header = "Early rehabilitation",
            body = paste("0-1 weeks", "• Instruction", "• Pendulum + assisted ROM", sep = "\n")
        ),
        early_gp_header
    ))

    expect_s3_class(box, "box")

    # Should work with spread
    boxes <- list(
        box1 = box,
        box2 = boxHeaderGrob("Delayed", "0-3 weeks\n• Sling")
    )
    result <- spreadVertical(boxes)
    expect_length(result, 2)
})

#' Formatter wrapper for \code{\link[rmarkdown]{html_document}}, facilitates easier porting to docx
#'
#' This function adds the option of having adaptations needed for seemless integration with
#' MS Word for importing html-documents in the .docx-format. The advantage of html documents is
#' the ability to create advanced formatting frequently needed in medical publications
#' and that is available in the \code{\link[htmlTable]{htmlTable}} function. You can view
#' \href{https://gforge.se/2014/07/fast-track-publishing-using-rmarkdown/}{the series}
#' for more details regarding how to achieve
#' fast-track-publishing (ftp) together with knitr.
#'
#' If you want to get equations into Word the currently best way is to
#' use the \code{\link[rmarkdown]{word_document}} format.
#'
#' @param ... Passed onto \code{\link[rmarkdown]{html_document}}.
#' @param self_contained Overrides the default \code{TRUE} for
#'  \code{\link[rmarkdown]{html_document}} to \code{FALSE} as
#'  LibreOffice hangs on long lines such as the base64
#'  images included in the self-contained version.
#' @param mathjax The advanced mathjax does not work with with
#'  Word/LibreOffice.
#' @param theme No theme should be used for the output as the
#'  custom CSS should take care of everything.
#' @param highlight By default turn off highlighting as scripts
#'  are difficult to import. This does though work somewhat OK when
#'  copy-pasting from the web-browser.
#' @param css The CSS if other that the default within the package
#' @param h1_style You can choose any css style formatting here that you
#'  want to be applied to all h1 elements. Note: this is only applied
#'  if LibreOffice_adapt is \code{TRUE}.
#' @param other_h_style This is the formatting applied to any other
#'  h elements not included to the first. Note: this is only applied
#'  if LibreOffice_adapt is \code{TRUE}.
#' @param remove_scripts \code{TRUE} if <script></script> tags are
#'  to be removed. These are usually not compatible with Word-processors
#'  and should therefore in most cases be stripped from the document.
#' @param force_captions Since out.width and out.height remove the option of
#'  having captions this allows a workaround through some processing
#'  via the XML-package
#' @param css_max_width The max width of the body element. Defaults to "40em"
#'  if not specified. Any CSS-compliant width format works.
#' @return R Markdown output format to pass to \code{\link[rmarkdown]{render}}
#' @export
#' @author Max Gordon
#'
#' @examples
#' # Possible yaml configuration at the top of the Rmd doc
#' \dontrun{
#' ---
#' title: "Test"
#' author: "Max Gordon"
#' output:
#'   Gmisc::docx_document
#' ---
#' }
#' @importFrom rmarkdown html_document
docx_document <- function(...,
                          # Default options for ftp
                          self_contained = FALSE,
                          mathjax = NULL,
                          theme = NULL,
                          highlight = NULL,
                          # docx_specific
                          css = "rmarkdown/docx.css",
                          h1_style = "margin: 24pt 0pt 0pt 0pt;",
                          other_h_style = "margin: 10pt 0pt 0pt 0pt;",
                          remove_scripts = TRUE,
                          force_captions = FALSE,
                          css_max_width) {

  if (css == "rmarkdown/docx.css") {
    css <- system.file(css, package = "Gmisc")
    if (css == "")
      stop("Error locating the docx.css that should be included in the Gmisc package")
    if (!self_contained) {
      file.copy(from = css, to = "docx.css", overwrite = TRUE)
      css <- "docx.css"
      if (!missing(css_max_width)) {
        css <- prSetMaxWidth(max_width = css_max_width,
                             css_file = css)
      }
    } else {
      if (!missing(css_max_width)) {
        tmp_css <- tempfile()
        file.copy(from = css, to = tmp_css, overwrite = TRUE)
        css <- prSetMaxWidth(max_width = css_max_width,
                             css_file = tmp_css)
      }
    }
  } else if (!all(sapply(css, file.exists))) {
    alt_css <- list.files(pattern = ".css$")
    if (length(alt_css) > 0) {
      alt_css <- paste0("\n You do have alternative file name(s) in current directory that you may intend to use.",
                        " You may want to have a YAML section that looks something like:",
                        "\n---",
                        "\noutput:",
                        "\n  Gmisc::docx_document:",
                        "\n    css: \"", paste(alt_css, collapse = "\", \""), "\"",
                        "\n---")
    } else {
      alt_css <- ""
    }

    stop("You should have the default css for optimal docx compatibility.",
         " The one or more of the css-file(s) that you've specified can't be identified.",
         " The file(s) '", paste(css[!sapply(css, file.exists)],
                                collapse = "', '"), "'",
         " can't be found from the directory '", getwd(), "'",
         " - i.e. the directory where you have your .Rmd-file",
         alt_css)
  }

  # call the base html_document function
  output_ret_val <-
    html_document(...,
                  css = css,
                  mathjax = mathjax,
                  theme = theme,
                  highlight = highlight,
                  self_contained = self_contained)

  output_ret_val$post_processor_old <-
    output_ret_val$post_processor

  output_ret_val$post_processor <-
    post_processor <- function(metadata, input_file, output_file, clean, verbose,
                               old_post_processor = output_ret_val$post_processor_old) {
      # Call the original post-processor in order to limit the changes that this function
      # has on the original functionality
      output_file <-
        old_post_processor(
          metadata = metadata,
          input_file = input_file,
          output_file = output_file,
          clean = clean,
          verbose = verbose
        )

      # read the output file
      output_str <- readLines(output_file, warn = FALSE, encoding = "UTF-8")

      # Annoyingly it seems that Libre Office currently
      # 'forgets' the margin properties of the headers,
      # we therefore substitute these with a element specific
      # style option that works. Perhaps not that pretty but
      # it works and can be tweaked for most things.
      output_str <-
        prFtpHeaderStyle(output_str,
                         h1_style = h1_style,
                         other_h_style = h1_style)

      if (remove_scripts) {
        output_str <-
          prFtpScriptRemoval(output_str)


        output_str <-
          prFtpOtherRemoval(output_str)
      }

      output_str <-
        prFtpOtherChanges(output_str)

      writeLines(output_str, output_file, useBytes = TRUE)

      if (force_captions) {
        prCaptionFix(output_file)
      }
      return(output_file)
    }

  return(output_ret_val)
}

#' Helper to ftp_document
#'
#' @param output_str The output string from readLines()
#' @param h1_style You can choose any css style formatting here that you
#'  want to be applied to all h1 elements. Note: this is only applied
#'  if LibreOffice_adapt is \code{TRUE}.
#' @param other_h_style This is the formatting applied to any other
#'  h elements not included to the first. Note: this is only applied
#'  if LibreOffice_adapt is \code{TRUE}.
#' @return string
#' @keywords internal
prFtpHeaderStyle <- function(output_str, h1_style, other_h_style) {
  gsub(
    paste0('<h([0-9]+)',
           '(| ',
           '([ ]*class = "[^"]+"',
           '|[ ]*id = "[^"]+")+',
           ')[ ]*>'),
    paste0('<h\\1\\2 style = "', other_h_style, '">'),
    gsub(
      paste0('<h1+',
             '(| ',
             '([ ]*class = "[^"]+"',
             '|[ ]*id = "[^"]+")+',
             ')[ ]*>'),
      paste0('<h1\\1 style = "', h1_style, '">'),
      output_str
    )
  )
}

#' Removes the <script>*</scrip>
#'
#' @param output_str The input from readLines()
#' @return string Returns without the script elements
#' @keywords internal
prFtpScriptRemoval <- function(output_str) {
  start_scripts <- grep("<script", output_str)
  end_scripts <- grep("</script", output_str)
  if (length(start_scripts) == 0)
    return(output_str)

  rows_2_exclude <-
    unlist(lapply(1:length(start_scripts), FUN = function(x) -1*start_scripts[x]:end_scripts[x]))
  return(output_str[rows_2_exclude])
}


#' Removes other unwanted lines
#'
#' @param output_str The input from readLines()
#' @return string Returns without the unwanted lines
#' @keywords internal
prFtpOtherRemoval <- function(output_str) {
  lines_2_remove <-
    c(# html-validator complains
      "<meta http-equiv = \"Content-Style-Type\" content = \"text/css\" />",

      # Invalid formatting --s although this shouldn't be in the doc. start with
      "<!-- dynamically load mathjax for compatibility with --self-contained -->"
      )
  for (line in lines_2_remove) {
    rm_line <- grep(sprintf("^%s$", line),
                    output_str)
    if (length(rm_line) == 1)
      output_str <- output_str[-rm_line]
  }

  return(output_str)
}

#' Changes lines for XML-conformity
#'
#' @param output_str The input from readLines()
#' @return string Returns with changes
#' @keywords internal
prFtpOtherChanges <- function(output_str) {
  lines_2_change <-
    c(`<meta charset = "utf-8">` = '<meta charset = "utf-8" />')
  for (line in names(lines_2_change)) {
    ch_line <- grep(sprintf("^%s$", line),
                    output_str)
    if (length(ch_line) == 1)
      output_str[ch_line] <- lines_2_change[line]
  }

  return(output_str)
}

#' Fixes the caption for elements without caption
#'
#' @param outFile The name of the file
#' @return outFile The name of the file
#' @keywords internal
#' @import XML
prCaptionFix <- function(outFile) {
  # Encapsulate within a try since there is a high risk of error
  tryCatch({
    # Read in the full file through the html parser
    tmp <- XML::htmlParse(outFile, encoding = "utf-8", replaceEntities = FALSE)

    # The caption-less images are currently located under a p-element instead of a div
    caption_less_images <- xpathApply(tmp, "/html/body//p/img")
    for (i in 1:length(caption_less_images)) {
      old_node <- xmlParent(caption_less_images[[i]])
      img_clone <- xmlClone(caption_less_images[[i]])
      new_node <- newXMLNode("div",
                             img_clone,
                             newXMLNode("p",
                                        xmlAttrs(img_clone)["title"],
                                        attrs = c(class = "caption")),
                             attrs = c(class = "figure"))
      replaceNodes(oldNode = old_node,
                   newNode = new_node)
    }

    saveXML(tmp, encoding = "utf-8", file = outFile)
  }, error = function(err) warning("Could not force captions - error occurred: '", err, "'"))

  return(outFile)
}

#' Updates the css max-width
#'
#' Reads the file, changes the max-width
#' for the body element.
#'
#' @param max_width The new max-width css setting
#' @param css_file The CSS file name
#' @return \code{string} The file name
#' @keywords internal
prSetMaxWidth <- function(max_width, css_file) {
  content <- readLines(css_file)
  content <-
    sub(pattern = "max-width: 40em; /* Body */",
        replacement = sprintf("max-width: %s; /* Body */",
                               max_width),
        x = content,
        fixed = TRUE)
  writeLines(text = content,
             con = css_file)
  return(css_file)
}

# from dput(output_str)
output_str <- c(
  "<!DOCTYPE html>", "", "<html xmlns = \"http://www.w3.org/1999/xhtml\">",
  "", "<head>", "", "<meta charset = \"utf-8\">", "<meta http-equiv = \"Content-Type\" content = \"text/html; charset = utf-8\" />",
  "<meta http-equiv = \"Content-Style-Type\" content = \"text/css\" />",
  "<meta name = \"generator\" content = \"pandoc\" />", "", "<meta name = \"author\" content = \"MG\" />",
  "", "", "<title>Test</title>", "", "<script src = \"FTP_test_files/jquery-1.11.0/jquery.min.js\"></script>",
  "<meta name = \"viewport\" content = \"width = device-width, initial-scale = 1.0\" />",
  "<link href = \"FTP_test_files/bootstrap-2.3.2/css/bootstrap.min.css\" rel = \"stylesheet\" />",
  "<link href = \"FTP_test_files/bootstrap-2.3.2/css/bootstrap-responsive.min.css\" rel = \"stylesheet\" />",
  "<script src = \"FTP_test_files/bootstrap-2.3.2/js/bootstrap.min.js\"></script>",
  "", "<style type = \"text/css\">code{white-space: pre;}</style>",
  "<link rel = \"stylesheet\"", "      href = \"FTP_test_files/highlight/default.css\"",
  "      type = \"text/css\" />", "<script src = \"FTP_test_files/highlight/highlight.js\"></script>",
  "<style type = \"text/css\">", "  pre:not([class]) {", "    background-color: white;",
  "  }", "</style>", "<script type = \"text/javascript\">", "if (window.hljs && document.readyState && document.readyState === \"complete\") {",
  "   window.setTimeout(function() {", "      hljs.initHighlighting();",
  "   }, 0);", "}", "</script>", "", "", "<link rel = \"stylesheet\" href = \"custom.css\" type = \"text/css\" />",
  "", "</head>", "", "<body>", "", "<style type = \"text/css\">",
  ".main-container {", "  max-width: 940px;", "  margin-left: auto;",
  "  margin-right: auto;", "}", "</style>", "<div class = \"container-fluid main-container\">",
  "", "", "<div id = \"header\">", "<h1 class = \"title\" style = \"margin: 24pt 0pt 0pt 0pt;\">Test</h1>",
  "<h4 class = \"author\" style = \"margin: 10pt 0pt 0pt 0pt;\"><em>MG</em></h4>",
  "<h4 class = \"date\" style = \"margin: 10pt 0pt 0pt 0pt;\"><em>Sunday, July 13, 2014</em></h4>",
  "</div>", "", "", "<p>All analyses were performed using R (ver. 3.1.1)[R Core Team, 2013] and packages rms (ver. 4.2-0) [F. Harrell, 2014] for analysis, Gmisc for plot and table output (ver. 0.6.6), and knitr (ver ) [Xie, 2013] for reproducible research.</p>",
  "<div id = \"results\" class = \"section level1\">", "<h1 style = \"margin: 24pt 0pt 0pt 0pt;\">Results</h1>",
  "<p>We found 205 patients with malignant melanoma between the years 1962 and 1977. Patients were followed until the end of 1977, the median follow-up time was 5.5 years (range 0.0 to 15.2 years). Males were more common than females and had also a higher mortality rate.</p>",
  "<table class = 'gmisc_table' style = 'border-collapse: collapse;'  id = 'table_1'>",
  "    <thead>", "    <tr><td colspan = '6' style = 'text-align: left;'>",
  "    ", "Table 1: Baseline characteristics", "</td></tr>", "    <tr>",
  "        <th style = 'border-top: 2px solid grey;'></th>", "        <th colspan = '2' style = 'font-weight: 900; border-top: 2px solid grey; text-align: center;'>",
  " ", "</th><th style = 'border-top: 2px solid grey;; border-bottom: hidden;'>",
  " ", "</th>", "        <th colspan = '2' style = 'font-weight: 900; border-bottom: 1px solid grey; border-top: 2px solid grey;'>",
  "Death", "</th>", "    </tr>", "    <tr>", "        <th style = 'border-bottom: 1px solid grey; '>",
  " ", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "Total", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "Alive", "</th>", "        <th style = 'border-bottom: 1px solid grey;' colspan = '1'>",
  " ", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "Melanoma", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "Non-melanoma", "</th>", "    </tr>", "    </thead><tbody>",
  "    <tr style = 'background-color:#ffffff;'><td colspan = '6' style = 'font-weight: 900;'>",
  "Sex", "</td></tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "  Male", "</td>",
  "        <td style = 'text-align: right;'>", "126 (61%)", "</td>",
  "        <td style = 'text-align: right;'>", "91 (72%)", "</td>",
  "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "28 (22%)", "</td>", "        <td style = 'text-align: right;'>",
  "7 (6%)", "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "  Female", "</td>",
  "        <td style = 'text-align: right;'>", "79 (39%)", "</td>",
  "        <td style = 'text-align: right;'>", "43 (54%)", "</td>",
  "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "29 (37%)", "</td>", "        <td style = 'text-align: right;'>",
  "7 (9%)", "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'><td colspan = '6' style = 'font-weight: 900; '>",
  "Age<sup>†</sup>", "</td></tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "  Mean (SD)", "</td>",
  "        <td style = 'text-align: right;'>", "52 (± 17)", "</td>",
  "        <td style = 'text-align: right;'>", "50 (± 16)", "</td>",
  "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "55 (± 18)", "</td>", "        <td style = 'text-align: right;'>",
  "65 (± 11)", "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'><td colspan = '6' style = 'font-weight: 900; '>",
  "Ulceration", "</td></tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "  Absent", "</td>",
  "        <td style = 'text-align: right;'>", "115 (56%)", "</td>",
  "        <td style = 'text-align: right;'>", "92 (80%)", "</td>",
  "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "16 (14%)", "</td>", "        <td style = 'text-align: right;'>",
  "7 (6%)", "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "  Present", "</td>",
  "        <td style = 'text-align: right;'>", "90 (44%)", "</td>",
  "        <td style = 'text-align: right;'>", "42 (47%)", "</td>",
  "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "41 (46%)", "</td>", "        <td style = 'text-align: right;'>",
  "7 (8%)", "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'><td colspan = '6' style = 'font-weight: 900; '>",
  "Thickness<sup>‡</sup>", "</td></tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'border-bottom: 2px solid grey; text-align: left;'>",
  "  Mean (SD)", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "2.9 (±3.0)", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "2.2 (±2.3)", "</td>", "        <td style = 'border-bottom: 2px solid grey;' colspan = '1'>",
  " ", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "4.3 (±3.6)", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "3.7 (±3.6)", "</td>", "    </tr>", "    </tbody>", "    <tfoot><tr><td colspan = 6>",
  "    ", "<sup>†</sup> Age at the time of surgery. <br/><sup>‡</sup> Tumour thicknes, also known as Breslow thickness, measured in mm.",
  "</td></tr></tfoot>", "</table>", "", "<div id = \"main-results\" class = \"section level2\">",
  "<h2 style = \"margin: 10pt 0pt 0pt 0pt;\">Main results</h2>",
  "<table class = 'gmisc_table' style = 'border-collapse: collapse;'  id = 'table_2'>",
  "    <thead>", "    <tr><td colspan = '9' style = 'text-align: left;'>",
  "    ", "Table 2: Adjusted and unadjusted estimates for melanoma specific death.",
  "</td></tr>", "    <tr>", "        <th style = 'border-top: 2px solid grey;'></th>",
  "        <th colspan = '2' style = 'font-weight: 900; border-top: 2px solid grey; text-align: center;'>",
  " ", "</th><th style = 'border-top: 2px solid grey;; border-bottom: hidden;'>",
  " ", "</th>", "        <th colspan = '2' style = 'font-weight: 900; border-bottom: 1px solid grey; border-top: 2px solid grey;'>",
  "Crude", "</th><th style = 'border-top: 2px solid grey;; border-bottom: hidden;'>",
  " ", "</th>", "        <th colspan = '2' style = 'font-weight: 900; border-bottom: 1px solid grey; border-top: 2px solid grey;'>",
  "Adjusted", "</th>", "    </tr>", "    <tr>", "        <th style = 'font-weight: 900; border-bottom: 1px solid grey; '>",
  "Variable", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "Total", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "Event", "</th>", "        <th style = 'border-bottom: 1px solid grey;' colspan = '1'>",
  " ", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "HR", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "2.5 % to 97.5 %", "</th>", "        <th style = 'border-bottom: 1px solid grey;' colspan = '1'>",
  " ", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "HR", "</th>", "        <th style = 'border-bottom: 1px solid grey; text-align: center;'>",
  "2.5 % to 97.5 %", "</th>", "    </tr>", "    </thead><tbody>",
  "    <tr style = 'background-color:#ffffff;'><td colspan = '9' style = ''>",
  "Sex", "</td></tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "  Male", "</td>",
  "        <td style = 'text-align: right;'>", "126", "</td>", "        <td style = 'text-align: right;'>",
  "28 (22%)", "</td>", "        <td style = '' colspan = '1'>", " ",
  "</td>", "        <td style = 'text-align: right;'>", "1", "</td>",
  "        <td style = 'text-align: center;'>", "ref", "</td>", "        <td style = '' colspan = '1'>",
  " ", "</td>", "        <td style = 'text-align: right;'>", "1",
  "</td>", "        <td style = 'text-align: center;'>", "ref", "</td>",
  "    </tr>", "    <tr style = 'background-color:#ffffff;'>", "        <td style = 'text-align: left;'>",
  "  Female", "</td>", "        <td style = 'text-align: right;'>",
  "79", "</td>", "        <td style = 'text-align: right;'>", "29 (37%)",
  "</td>", "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "1.94", "</td>", "        <td style = 'text-align: center;'>",
  "1.15 to 3.26", "</td>", "        <td style = '' colspan = '1'>",
  " ", "</td>", "        <td style = 'text-align: right;'>", "1.54",
  "</td>", "        <td style = 'text-align: center;'>", "0.91 to 2.60",
  "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "Age", "</td>", "        <td style = 'text-align: right;'>",
  "52 (± 17)", "</td>", "        <td style = 'text-align: right;'>",
  "55 (± 18)", "</td>", "        <td style = '' colspan = '1'>", " ",
  "</td>", "        <td style = 'text-align: right;'>", "1.02", "</td>",
  "        <td style = 'text-align: center;'>", "1.00 to 1.04", "</td>",
  "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "1.01", "</td>", "        <td style = 'text-align: center;'>",
  "1.00 to 1.03", "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'><td colspan = '9' style = ' '>",
  "Ulceration", "</td></tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'text-align: left;'>", "  Absent", "</td>",
  "        <td style = 'text-align: right;'>", "115", "</td>", "        <td style = 'text-align: right;'>",
  "16 (14%)", "</td>", "        <td style = '' colspan = '1'>", " ",
  "</td>", "        <td style = 'text-align: right;'>", "1", "</td>",
  "        <td style = 'text-align: center;'>", "ref", "</td>", "        <td style = '' colspan = '1'>",
  " ", "</td>", "        <td style = 'text-align: right;'>", "1",
  "</td>", "        <td style = 'text-align: center;'>", "ref", "</td>",
  "    </tr>", "    <tr style = 'background-color:#ffffff;'>", "        <td style = 'text-align: left;'>",
  "  Present", "</td>", "        <td style = 'text-align: right;'>",
  "90", "</td>", "        <td style = 'text-align: right;'>", "41 (46%)",
  "</td>", "        <td style = '' colspan = '1'>", " ", "</td>", "        <td style = 'text-align: right;'>",
  "4.36", "</td>", "        <td style = 'text-align: center;'>",
  "2.44 to 7.77", "</td>", "        <td style = '' colspan = '1'>",
  " ", "</td>", "        <td style = 'text-align: right;'>", "3.20",
  "</td>", "        <td style = 'text-align: center;'>", "1.75 to 5.88",
  "</td>", "    </tr>", "    <tr style = 'background-color:#ffffff;'>",
  "        <td style = 'border-bottom: 2px solid grey; text-align: left;'>",
  "Breslow thickness", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "3 (± 3)", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "4 (± 4)", "</td>", "        <td style = 'border-bottom: 2px solid grey;' colspan = '1'>",
  " ", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "1.17", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: center;'>",
  "1.10 to 1.25", "</td>", "        <td style = 'border-bottom: 2px solid grey;' colspan = '1'>",
  " ", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: right;'>",
  "1.12", "</td>", "        <td style = 'border-bottom: 2px solid grey; text-align: center;'>",
  "1.04 to 1.20", "</td>", "    </tr>", "    </tbody>", "</table>",
  "", "<p>After adjusting for the three variables, age, sex, tumor thickness and ulceration, only the latter two remained significant (p-value &lt; 0.001 and 0.004), see table 1 and figure I.</p>",
  "<figure>", "<img src = 'FTP_test_files/figure-html/Regression_forestplot.png' style = 'display: block'>",
  "<figcaption>", "Figure I: A forest plot comparing the regression coefficients.",
  "</figcaption></figure>", "", "<p>There was no strong indication for non-linearity for any of the continuous variables although the impact of thickness did seem to lessen above 4 mm, see figure II.</p>",
  "<figure>", "<img src = 'FTP_test_files/figure-html/spline_plot.png' width = '384'height = ''width.px = '384'height.px = '384' style = 'display: block'>",
  "<figcaption>", "Figure II: The adjusted and unadjusted restricted cubic spline for tumor thickness. Solid line and confidence interval indicate the adjusted line while the dashed is the unadjusted line. The grey area at the bottom indicates the density.",
  "</figcaption></figure>",
  "", "", "</div>", "</div>", "", "",
  "</div>", "", "<script>", "", "// add bootstrap table styles to pandoc tables",
  "$(document).ready(function () {", "  $('tr.header').parent('thead').parent('table').addClass('table table-condensed');",
  "});", "", "</script>", "", "<!-- dynamically load mathjax for compatibility with --self-contained -->",
  "<script>", "  (function () {", "    var script = document.createElement(\"script\");",
  "    script.type = \"text/javascript\";", "    script.src  = \"https://c328740.ssl.cf1.rackcdn.com/mathjax/latest/MathJax.js?config = TeX-AMS-MML_HTMLorMML\";",
  "    document.getElementsByTagName(\"head\")[0].appendChild(script);",
  "  })();", "</script>", "", "</body>", "</html>"
)

test_that("Proper script removal", {
  expect_false(all(grepl("--s", prFtpOtherRemoval(output_str))))
  expect_false(all(grepl("Content-Style-Type", prFtpOtherRemoval(output_str))))
  expect_false(all(grepl("<script", prFtpScriptRemoval(output_str))))
})

#' A transition plot
#'
#' This plot's purpose is to illustrate how states change before and
#' after. In my research I use it before surgery and after surgery
#' but it can be used in any situation where you have a change from
#' one state to another
#'
#' @param transition_flow This should be a matrix with the size of the transitions.
#'  The unit for each cell should be number of observations, row/column-proportions
#'  will show incorrect sizes. The matrix needs to be square. The best way to generate
#'  this matrix is probably just do a \code{table(starting_state, end_state)}. The rows
#'  represent the starting positions, while the columns the end positions. I.e. the first
#'  rows third column is the number of observations that go from the first class to the
#'  third class.
#' @param type_of_arrow The types of arrow may be grid, simple, or gradient. Simple grid
#'  arrows are the \code{\link[grid:grid.bezier]{bezierGrob}} arrows (not that pretty),
#'  simple is the \code{\link{bezierArrowSmpl}} that I've created to get a more exact
#'  control of the arrow position and width, while gradient
#'  corresponds to \code{\link{bezierArrowGradient}}
#'  allowing the arrow to have a fill color that slowly turns into the color of the arrow.
#' @param box_txt The text to appear inside of the boxes. If you need line breaks
#'  then you need to manually add a \\n inside the string.
#' @param tot_spacing The proportion of the vertical space that is to be left
#'  empty. It is then split evenly between the boxes.
#' @param box_width The width of the box. By default the box is one fourth of
#'  the plot width.
#' @param fill_start_box The fill color of the start boxes. This can either
#'  be a single value or a vector if you desire different colors for each
#'  box. If you specify box_prop then this has to be a 2 column matrix.
#' @param txt_start_clr The text color of the start boxes. This can either
#'  be a single value or a vector if you desire different colors for each
#'  box. If you specify box_prop then this has to be a 2 column matrix.
#' @param fill_end_box The fill color of the end boxes. This can either
#'  be a single value or a vector if you desire different colors for each
#'  box. If you specify box_prop then this has to be a 2 column matrix.
#' @param txt_end_clr The text color of the end boxes. This can either
#'  be a single value or a vector if you desire different colors for each
#'  box. If you specify box_prop then this has to be a 2 column matrix.
#' @param cex The cex \code{\link{gpar}} of the text
#' @param min_lwd The minimum width of the line that we want to illustrate the
#'  tranisition with.
#' @param max_lwd The maximum width of the line that we want to illustrate the
#'  tranisition with.
#' @param lwd_prop_total The width of the lines may be proportional to either the
#'  other flows from that box, or they may be related to all flows. This is a boolean
#'  parameter that is set to true by default, i.e. relating to all flows.
#' @param arrow_clr The color of the arrows. Usually black, can be a vector indicating each arrow
#'  from first to last arrow (counting from the top). If the vector is of the same length as the
#'  boxes then all box arrows will have the same color (that is all the arrows stemming from the
#'  left boxes)
#' @param abs_arrow_width The width can either be absolute, i.e. each arrow headed for a box
#'  has the exact same width. The alternative is that the width is related to the line width.
#' @param overlap_bg_clr In order to enhance the 3D perspective and to make it easier
#'  to follow arrows the arrows have a background color to separate them from those underneath.
#' @param overlap_order The order from first->last for the lines. This means that the last
#'  line will be on top while the first one will appear at the bottom. This should be provided
#'  as a vector.
#' @param overlap_add_width The width of the white cross-over line. You can specify this as a scalar
#'  multiplication of the current line width. In case of non-grid arrows then you can also have this
#'  as a unit which is recommended as it looks better. If the scalar is < 1 then the overlap is ignored.
#' @param box_prop If you want the boxes to have proportions indicating some other factors then input
#'  a matrix with quantiles for the proportions. Note the size must be \code{nrow(transition_flow) x 2}.
#' @param mar A numerical vector of the form c(bottom, left, top, right) of the type \code{unit()}
#' @param main The title of the plot if any, default \code{NULL}
#' @param box_label A vector of length 2 if you want to label each box column
#' @param box_label_pos The position of the label, either \code{'top'} or \code{'bottom'}
#' @param box_label_cex The cex of the label, defaults to the default cex
#' @param color_bar If you have proportions inside the transition_flow variable
#'  then the color_bar will automatically appear at the bottom unless you set
#'  this to \code{FALSE}
#' @param color_bar_cex The size of the tick labels for the color bar
#' @param color_bar_labels The labels of the two proportions that make up the color bar.
#'  Defaults to the labels of the third dimension for the \code{transition_flow}
#'  argument.
#' @param color_bar_subspace If there is little or no difference
#'  at the low/high proportions of the spectrum then it
#'  can be of interest to focus the color change to the center
#'  leaving the tails constant
#' @param new_page If you want the plot to appear on a new blank page then set this to \code{TRUE}, by
#'  default it is \code{FALSE}.
#' @return void
#' @examples
#' # This example does not run since it
#' # takes a little while to assemble the
#' # arrows and RMD Check complains that this
#' # is more than allowed for
#' library(grid)
#' par_org <- par(ask = TRUE)
#' # Settings
#' no_boxes <- 3
#' # Generate test setting
#' transition_matrix <- matrix(NA, nrow = no_boxes, ncol = no_boxes)
#' transition_matrix[1, ] <- 200 * c(.5, .25, .25)
#' transition_matrix[2, ] <- 540 * c(.75, .10, .15)
#' transition_matrix[3, ] <- 340 * c(0, .2, .80)
#'
#' grid::grid.newpage()
#' transitionPlot(transition_matrix,
#'   box_txt = c("First", "Second", "Third"),
#'   type_of_arrow = "simple",
#'   min_lwd = unit(1, "mm"),
#'   max_lwd = unit(6, "mm"),
#'   overlap_add_width = unit(1, "mm")
#' )
#'
#'
#' # Setup proportions
#' box_prop <- cbind(c(1, 0, 0.5), c(.52, .2, .8))
#' # From the Set2 Colorbrewer
#' start_box_clr <- c("#8DA0CB", "#FC8D62")
#' # Darken the colors slightly
#' end_box_clr <- c(
#'   colorRampPalette(c(start_box_clr[1], "#000000"))(10)[2],
#'   colorRampPalette(c(start_box_clr[2], "#000000"))(10)[2]
#' )
#' # Create a new grid
#' grid::grid.newpage()
#' transitionPlot(transition_matrix,
#'   box_prop = box_prop,
#'   fill_start_box = start_box_clr, fill_end_box = end_box_clr,
#'   txt_start_clr = c("#FFFFFF", "#000000"), txt_end_clr = c("#FFFFFF", "#000000"),
#'   box_txt = c("First", "Second", "Third"),
#'   type_of_arrow = "gradient",
#'   min_lwd = unit(1, "mm"),
#'   max_lwd = unit(10, "mm"),
#'   overlap_add_width = unit(1, "mm")
#' )
#' par(par_org)
#' @import grid
#' @import magrittr
#' @importFrom grDevices grey as.raster
#' @export
transitionPlot <- function(transition_flow,
                           type_of_arrow = c("grid", "simple", "gradient"),
                           box_txt = rownames(transition_flow),
                           tot_spacing = 0.2,
                           box_width = 1 / 4,
                           fill_start_box = "darkgreen",
                           txt_start_clr = "white",
                           fill_end_box = fill_start_box,
                           txt_end_clr = txt_start_clr,
                           cex = 2,
                           min_lwd = if (type_of_arrow == "grid") 1 else unit(.1, "mm"),
                           max_lwd = if (type_of_arrow == "grid") 6 else unit(5, "mm"),
                           lwd_prop_total = TRUE,
                           arrow_clr = "#000000",
                           abs_arrow_width = FALSE,
                           overlap_bg_clr = "#FFFFFF",
                           overlap_order = 1:nrow(transition_flow),
                           overlap_add_width = if (type_of_arrow == "grid") 1.5 else unit(1, "mm"),
                           box_prop,
                           mar = unit(rep(3, times = 4), "mm"),
                           main = NULL,
                           box_label = NULL,
                           box_label_pos = "top",
                           box_label_cex = cex,
                           color_bar = TRUE,
                           color_bar_cex = cex * .33,
                           color_bar_labels,
                           color_bar_subspace = NULL,
                           new_page = FALSE) {
  # Just for convenience
  no_boxes <- nrow(transition_flow)


  # If the matrix is a 3D matrix then the third dimension gives the proportion
  if (length(dim(transition_flow)) > 2) {
    if (length(dim(transition_flow)) > 3) {
      stop(
        "Your transition matrix should be created through:",
        " table(var_a, var_b, var_c) providing a 3D-matrix",
        " you have provided a ", length(dim(transition_flow)), "D matrix."
      )
    }
    if (!missing(box_prop)) {
      stop("You can't have both box_prop and a three dimensional matrix as input")
    }
    if (dim(transition_flow)[3] != 2) {
      stop(
        "Your third dimension should be a proportion,",
        " i.e. a variable with two alternatives.",
        " You have provided a variable with ", dim(transition_flow)[3], " alternatives"
      )
    }

    prop_fn <- function(x) {
      if (x[1] == 0) {
        return(0)
      }
      if (x[2] == 0) {
        return(1)
      }
      return(x[1] / x[2])
    }
    no_1_start <- rowSums(transition_flow[, , 1])
    no_tot_start <- rowSums(transition_flow)
    no_1_end <- colSums(transition_flow[, , 1])
    no_tot_end <- rowSums(colSums(transition_flow[, , 1:2]))
    box_prop <- cbind(
      apply(cbind(no_1_start, no_tot_start), 1, prop_fn),
      apply(cbind(no_1_end, no_tot_end), 1, prop_fn)
    )
    transition_arrow_props <- transition_flow[, , 1] / (transition_flow[, , 1] + transition_flow[, , 2])

    if (color_bar == FALSE) {
      color_bar <- "none"
    } else if (!is.character(color_bar)) {
      color_bar <- "bottom"
    }

    if (missing(color_bar_labels) &&
      !is.null(dimnames(transition_flow))) {
      color_bar_labels <- dimnames(transition_flow)[[3]]
    }

    # Remove the third dimension
    transition_flow <- transition_flow[, , 1] + transition_flow[, , 2]
  } else if (!missing(box_prop)) {
    transition_arrow_props <- t(sapply(box_prop[, 1], function(x) rep(x, no_boxes)))
    color_bar <- "none"
  } else {
    transition_arrow_props <- matrix(1, ncol = no_boxes, nrow = no_boxes)
    color_bar <- "none"
  }

  if (length(arrow_clr) == no_boxes) {
    arrow_clr <- t(sapply(arrow_clr, FUN = function(x) {
      rep(x, ncol(transition_flow))
    }))
  } else if (length(arrow_clr) == 1) {
    arrow_clr <- rep(arrow_clr, no_boxes * ncol(transition_flow))
  }

  if (length(arrow_clr) != no_boxes * ncol(transition_flow)) {
    stop(
      "You have provided an invalid number of arrow colors,",
      " you have ", length(arrow_clr), " colors, while you should provide either 1, ",
      no_boxes, ", or ", no_boxes * ncol(transition_flow), " colors"
    )
  }

  if (length(overlap_order) != no_boxes) {
    stop(
      "You have the wrong number of overlap orders, you provided ",
      length(overlap_order), " while it should be ", no_boxes
    )
  } else if (all(overlap_order %in% 1:no_boxes) == FALSE) {
    stop(
      "Your overlap numbers contain numbers outside the rowrange of",
      " the transition rows, i.e. not between 1 and ", no_boxes
    )
  }

  type_of_arrow <- match.arg(type_of_arrow)
  if (type_of_arrow != "grid") {
    if (!"unit" %in% class(min_lwd) ||
      !"unit" %in% class(max_lwd)) {
      stop("Your line widths must be in units when you specify the alternative arrows, e.g. unit(10, 'pt')")
    }

    # We need to convert these into regular values in order to use
    # them later on in the calculations
    min_lwd <- convertUnit(min_lwd, unitTo = "npc", valueOnly = TRUE)
    max_lwd <- convertUnit(max_lwd, unitTo = "npc", valueOnly = TRUE)
  }

  # Do some sanity checking of the variables
  if (tot_spacing < 0 ||
    tot_spacing > 1) {
    stop(
      "Total spacing, the tot_spacing param,",
      " must be a fraction between 0-1,",
      " you provided ", tot_spacing
    )
  }

  if (box_width < 0 ||
    box_width > 1) {
    stop(
      "Box width, the box_width param,",
      " must be a fraction between 0-1,",
      " you provided ", box_width
    )
  }

  # If the text element is a vector then that means that
  # the names are the same prior and after
  if (is.null(box_txt)) {
    box_txt <- matrix("", ncol = 2, nrow = no_boxes)
  }
  if (is.null(dim(box_txt)) && is.vector(box_txt)) {
    if (length(box_txt) != no_boxes) {
      stop(
        "You have an invalid length of text description, the box_txt param,",
        " it should have the same length as the boxes, ", no_boxes, ",",
        " but you provided a length of ", length(box_txt)
      )
    } else {
      box_txt <- cbind(box_txt, box_txt)
    }
  } else if (nrow(box_txt) != no_boxes ||
    ncol(box_txt) != 2) {
    stop(
      "Your box text matrix doesn't have the right dimension, ",
      no_boxes, " x 2, it has: ",
      paste(dim(box_txt), collapse = " x ")
    )
  }


  if (missing(box_prop)) {
    # Make sure that the clrs correspond to the number of boxes
    fill_start_box <- rep(fill_start_box, length.out = no_boxes)
    txt_start_clr <- rep(txt_start_clr, length.out = no_boxes)
    fill_end_box <- rep(fill_end_box, length.out = no_boxes)
    txt_end_clr <- rep(txt_end_clr, length.out = no_boxes)
  } else {
    fill_start_box <- prTpGetBoxPropClr(fill_start_box,
      no_boxes = no_boxes
    )
    fill_end_box <- prTpGetBoxPropClr(fill_end_box,
      no_boxes = no_boxes
    )
    txt_start_clr <- prTpGetBoxPropClr(txt_start_clr,
      no_boxes = no_boxes,
      lengthOneOK = TRUE
    )
    txt_end_clr <- prTpGetBoxPropClr(txt_end_clr,
      no_boxes = no_boxes,
      lengthOneOK = TRUE
    )

    # Input checks
    if (is.matrix(box_prop) == FALSE) {
      stop("You have to provide the box_prop as a matrix corresponding to the boxes")
    } else if (nrow(box_prop) != no_boxes || ncol(box_prop) != 2) {
      stop(
        "Your box_prop matrix must have ", no_boxes, "x", 2,
        " dimensions, your matrix is currently of ",
        nrow(box_prop), "x", ncol(box_prop), " dimensions"
      )
    } else if (any(box_prop > 1 | box_prop < 0)) {
      stop("You have provided in box_prop invalid quantiles outside the 0-1 range")
    } else if (length(fill_start_box) == 0) {
      stop("You have provided invalid number of fill colors (fill_start_box) when used together with box_prop")
    } else if (length(fill_end_box) == 0) {
      stop("You have provided invalid number of fill colors (fill_end_box) when used together with box_prop")
    } else if (length(txt_start_clr) == 0) {
      stop("You have provided invalid number of text colors (txt_start_clr) when used together with box_prop")
    } else if (length(txt_end_clr) == 0) {
      stop("You have provided invalid number of text colors (txt_end_clr) when used together with box_prop")
    }
  }

  if (nrow(transition_flow) != ncol(transition_flow)) {
    stop(
      "Invalid input array, the matrix is not square but ",
      nrow(transition_flow), " x ", ncol(transition_flow)
    )
  }

  # Set the proportion of the start/end sizes of the boxes
  prop_start_sizes <- rowSums(transition_flow) / sum(transition_flow)
  prop_end_sizes <- colSums(transition_flow) / sum(transition_flow)

  if (sum(prop_end_sizes) == 0) {
    stop("You can't have all empty boxes after the transition")
  }


  if (new_page) grid.newpage()

  # For popViewport at the need to keep track of how
  # many levels we have added
  vp_depth <- 1
  # Add plot margin
  prPushMarginViewport(
    bottom = unit(prGetNpcValue(mar[1], "y"), "npc"),
    left = unit(prGetNpcValue(mar[2], "x"), "npc"),
    top = unit(prGetNpcValue(mar[3], "y"), "npc"),
    right = unit(prGetNpcValue(mar[4], "x"), "npc"),
    "main_margins"
  )

  if (!is.null(main) && nchar(main) > 0) {
    prGridPlotTitle(main, cex[1])
    vp_depth %<>% +2
  }

  if (!is.null(box_label) && length(box_label) == 2) {
    left <- prTpGetBoxPositions(
      side = "left", no = 1,
      transitions = transition_flow[1, ],
      prop_start_sizes = prop_start_sizes,
      prop_end_sizes = prop_end_sizes,
      tot_spacing = tot_spacing,
      box_width = box_width
    )
    right <- prTpGetBoxPositions(
      side = "right", no = 1,
      transitions = transition_flow[, 1],
      prop_start_sizes = prop_start_sizes,
      prop_end_sizes = prop_end_sizes,
      tot_spacing = tot_spacing,
      box_width = box_width
    )
    left_label <- textGrob(box_label[1],
      gp = gpar(cex = box_label_cex)
    )
    right_label <- textGrob(box_label[2],
      gp = gpar(cex = box_label_cex)
    )
    label_height <- prGetNpcValue(max(grobHeight(left_label), grobHeight(right_label)), "y")
    # Add ygjp space and some margin
    label_height <- unit(label_height * 2 + label_height * 0.1, "npc")
    width <- list(
      left = unit(left$right - left$left, "npc"),
      right = unit(right$right - right$left, "npc")
    )
    if (box_label_pos == "top") {
      gl <- grid.layout(
        nrow = 2, ncol = 3,
        heights = unit.c(
          label_height,
          unit(1, "npc") - label_height
        ),
        widths = unit.c(
          width$left,
          unit(1, "npc") -
            width$left -
            width$right,
          width$right
        )
      )
      label_row_no <- 1
      main_row_no <- 2
    } else {
      gl <- grid.layout(
        nrow = 2, ncol = 3,
        heights = unit.c(
          unit(1, "npc") - label_height,
          label_height
        ),
        widths = unit.c(
          width$left,
          unit(1, "npc") -
            width$left -
            width$right,
          width$right
        )
      )
      label_row_no <- 2
      main_row_no <- 1
    }

    # Set layout
    pushViewport(viewport(layout = gl, name = "Label_layout"))

    # Add labels
    pushViewport(viewport(layout.pos.row = label_row_no, layout.pos.col = 1, name = "Left_label"))
    grid.draw(left_label)
    popViewport()
    pushViewport(viewport(layout.pos.row = label_row_no, layout.pos.col = 3, name = "Right_label"))
    grid.draw(right_label)
    popViewport()

    # Set the graph viewport
    pushViewport(viewport(layout.pos.row = main_row_no, layout.pos.col = 1:3, name = "Main_exc_label"))
    vp_depth %<>% +2
  }

  if (color_bar != "none" &&
    type_of_arrow == "gradient") {
    if (color_bar == "bottom") {
      bar_height <- unit(.05, "npc")
      colorAxis <- xaxisGrob(
        at = c(0, .25, .5, .75, 1),
        label = sprintf("%d %%", c(0, .25, .5, .75, 1) * 100),
        main = FALSE, gp = gpar(cex = color_bar_cex)
      )

      # Add a little space to the actual height
      axis_height <- grobHeight(colorAxis) + unit(.01, "npc")
      bar_layout <- grid.layout(
        nrow = 3, ncol = 3,
        heights = unit.c(
          unit(1, "npc") -
            axis_height -
            bar_height,
          axis_height,
          bar_height
        ),
        widths = unit.c(
          unit(box_width, "npc"),
          unit(1, "npc") -
            unit(box_width * 2, "npc"),
          unit(box_width, "npc")
        )
      )

      pushViewport(viewport(layout = bar_layout, name = "Bar_layout"))

      pushViewport(viewport(
        layout.pos.row = 3,
        layout.pos.col = 2,
        name = "Color_bar"
      ))
      bar_clrs <- prTpGetColors(fill_start_box[1, ], space = color_bar_subspace)
      grid.raster(t(as.raster(bar_clrs)), width = 1, height = 1, interpolate = FALSE)
      grid.draw(colorAxis)
      if (!missing(color_bar_labels)) {
        # The height is actually oblivious to upper case and lower case letters
        lab_height <- convertY(grobHeight(textGrob("Ij")), "npc", valueOnly = TRUE)
        lab_cex_adjusted <- 1 / (lab_height * 2)

        if (missing(txt_start_clr)) {
          color_bar_txt_clr <- c("black", "black")
        } else if (ncol(txt_start_clr) == 1) {
          color_bar_txt_clr <- rep(txt_start_clr[1], 2)
        } else {
          color_bar_txt_clr <- txt_start_clr[1, ]
        }

        lab_margin <- .05
        left <- textGrob(color_bar_labels[1],
          x = 0 + lab_margin,
          just = "left",
          y = .5,
          gp = gpar(
            cex = lab_cex_adjusted,
            col = color_bar_txt_clr[1]
          )
        )
        right <- textGrob(color_bar_labels[2],
          x = 1 - lab_margin,
          just = "right",
          y = .5,
          gp = gpar(
            cex = lab_cex_adjusted,
            col = color_bar_txt_clr[2]
          )
        )
        grid.draw(left)
        grid.draw(right)
      }
      popViewport()

      pushViewport(viewport(
        layout.pos.row = 1,
        layout.pos.col = 1:3,
        name = "Main_exc_bar"
      ))
      vp_depth %<>% +2
    } else {
      stop("The color bar position you want, '", color_bar, "', is not yet supported")
    }
  }
  # Do the plot
  # Plot shadow boxes 2 % shifted of the box width
  shift <- box_width * .02
  vp1 <- viewport(x = 0.5 + shift, y = 0.5 - shift, height = 1 - shift * 2, width = 1 - shift * 2, name = "shadow_boxes")
  pushViewport(vp1)

  shadow_clr <- rep(grey(.8), length.out = no_boxes)
  prTpPlotBoxes(
    overlap_order = overlap_order,
    transition_flow = transition_flow,
    no_boxes = no_boxes,
    box_width = box_width,
    tot_spacing = tot_spacing,
    txt = matrix("", nrow = no_boxes, ncol = 2), # Don't print anything in the shadow boxes
    cex = cex,
    prop_start_sizes = prop_start_sizes,
    prop_end_sizes = prop_end_sizes,
    box_prop = box_prop,
    lwd_prop_total = lwd_prop_total,
    fill_start_clr = shadow_clr,
    fill_end_clr = shadow_clr,
    txt_start_clr = txt_start_clr,
    txt_end_clr = txt_end_clr,
    line_col = shadow_clr[1],
    plot_arrows = FALSE,
    proportion = FALSE
  )
  popViewport()

  # Plot real boxes
  vp1 <- viewport(
    x = 0.5 - shift, y = 0.5 + shift,
    height = 1 - shift * 2, width = 1 - shift * 2, name = "actual_boxes"
  )
  pushViewport(vp1)
  prTpPlotBoxes(
    overlap_order = overlap_order,
    transition_flow = transition_flow,
    no_boxes = no_boxes,
    box_width = box_width,
    tot_spacing = tot_spacing,
    txt = box_txt,
    cex = cex,
    prop_start_sizes = prop_start_sizes,
    prop_end_sizes = prop_end_sizes,
    box_prop = box_prop,
    lwd_prop_total = lwd_prop_total,
    fill_start_clr = fill_start_box,
    fill_end_clr = fill_end_box,
    txt_start_clr = txt_start_clr,
    txt_end_clr = txt_end_clr,
    min_lwd = min_lwd,
    max_lwd = max_lwd,
    overlap_add_width = overlap_add_width,
    overlap_bg_clr = overlap_bg_clr,
    type_of_arrow = type_of_arrow,
    abs_arrow_width = abs_arrow_width,
    arrow_clr = arrow_clr,
    transition_arrow_props = transition_arrow_props,
    color_bar_subspace = color_bar_subspace,
    plot_arrows = TRUE,
    proportion = TRUE
  )
  popViewport()

  # Exit margin viewport
  popViewport(vp_depth)
}

library(testthat)
context("transitionPlot basic functionality")

# simple 2x2 transition matrix
mat <- matrix(c(10, 5, 3, 2), nrow = 2)
rownames(mat) <- c("A", "B")
colnames(mat) <- c("A", "B")

# running should not error and return invisibly NULL
expect_silent(res <- transitionPlot(mat, type_of_arrow = "grid", new_page = FALSE))
expect_null(res)

# dimension check: non-square should error
mat2 <- matrix(1:6, nrow = 2)
expect_error(transitionPlot(mat2), "square")

# check 3D matrix error path
mat3 <- array(1:12, dim = c(2, 2, 3))
expect_error(transitionPlot(mat3), "third dimension")

#' Create a box with text
#'
#' Creates a \code{\link[grid:grid.grob]{grob}} box with text inside it.
#'
#' @param label The label to print - should be a number, text or expression.
#' @param y The y position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.
#' @param x The x position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.
#' @param width The box automatically adapts the size but you can force by specifying the width
#' @param height The box automatically adapts the size but you can force by specifying the height
#' @param just The justification for the text: left, center or right.
#' @param bjust The justification for the box: left, center, right, top or bottom.
#'  See the \code{just} option for the \code{\link[grid]{viewport}}
#' @param txt_gp The \code{\link[grid]{gpar}} style to apply to the text. Set \code{boxGrobTxt} option
#'  if you want to customize all the boxes at once.
#' @param txt_padding Padding between text and box border. Can be provided as a
#'  \code{\link[grid]{unit}} or numeric (interpreted as millimetres). You can set
#'  a global default with \code{options(boxGrobTxtPadding = ...)}.
#' @param box_gp The \code{\link[grid]{gpar}} style to apply to the box function of `box_fn` below.
#' @param box_fn Function to create box for the text. Parameters of `x=0.5`, `y=0.5` and `box_gp` will
#'  be passed to this function and return a \code{grob} object.
#' @seealso The package provides several convenience shape helpers that can be
#' passed to `boxGrob(..., box_fn = ...)`: \code{boxDiamondGrob},
#' \code{boxEllipseGrob}, \code{boxRackGrob}, \code{boxServerGrob},
#' \code{boxDatabaseGrob}, \code{boxDocumentGrob}, \code{boxDocumentsGrob}, and
#' \code{boxTapeGrob}. For examples see the vignette: \code{vignette("Grid-based_flowcharts", package = "Gmisc")}.
#' @param name a character identifier for the \code{grob}. Used to find the \code{grob} on the display
#'  list and/or as a child of another grob.
#' @param badge_label Optional badge text to display at the top of the box.
#' @param badge_position The position of the badge: "top" (currently only "top" supported).
#' @param badge_gp The \code{\link[grid]{gpar}} style to apply to the badge background.
#' @param badge_txt_gp The \code{\link[grid]{gpar}} style to apply to the badge text.
#'
#' @return A grob
#' @export
#'
#' @rdname box
#' @importFrom checkmate assert_class assert checkString checkNumeric
#' @family flowchart components
#' @order 1
#' @examples
#' # Note: grid functions are explicitly namespaced in examples to avoid
#' # relying on attaching the grid package in R CMD check.
#' grid::grid.newpage()
#' boxGrob("My box")
boxGrob <- function(label,
                    y = unit(.5, "npc"),
                    x = unit(.5, "npc"),
                    width,
                    height,
                    just = "center",
                    bjust = "center",
                    txt_gp = getOption("boxGrobTxt", default = gpar(
                      color = "black",
                      cex = 1
                    )),
                    txt_padding = getOption("boxGrobTxtPadding", default = unit(6 * ifelse(is.null(txt_gp$cex), 1, txt_gp$cex), "mm")),
                    box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                    box_fn = roundrectGrob,
                    name = NULL,
                    badge_label = NULL,
                    badge_position = "top",
                    badge_gp = gpar(fill = "steelblue", col = NA),
                    badge_txt_gp = gpar(col = "white", cex = 0.7)) {
  assert(
    checkString(label),
    checkNumeric(label),
    is.language(label)
  )
  assert_unit(y)
  assert_unit(x)
  assert_unit(width)
  assert_unit(height)
  if (is.numeric(txt_padding)) {
    txt_padding <- unit(txt_padding, "mm")
  }
  assert_unit(txt_padding)
  assert_just(just)
  assert_just(bjust)
  assert_class(txt_gp, "gpar")
  assert_class(box_gp, "gpar")


  x <- prAsUnit(x)
  y <- prAsUnit(y)

  # Call the box function early to collect any suggested padding attributes
  # (e.g., diamonds may request extra padding). This allows the padding to
  # influence text layout and the computed box width/height.
  rect <- do.call(box_fn, list(x = .5, y = .5, gp = box_gp))
  extra_pad <- attr(rect, "box_fn_padding")
  if (!is.null(extra_pad)) {
    tryCatch(
      {
        txt_padding <- txt_padding + extra_pad
      },
      error = function(e) {
        # ignore silently if attribute is not a proper unit
      }
    )
  }

  # Create text grob using the (possibly) adjusted padding
  txt <- textGrob(
    label = label,
    x = prGetX4Txt(just, txt_padding), y = .5,
    just = just, gp = txt_gp,
    name = "label"
  )

  if (missing(height)) {
    height <- grobHeight(txt) + txt_padding + txt_padding
  } else {
    height <- prAsUnit(height)
  }

  if (missing(width)) {
    width <- grobWidth(txt) + txt_padding + txt_padding
  } else {
    width <- prAsUnit(width)
  }

  vp_args <- list(
    x = x,
    y = y,
    width = width,
    height = height,
    just = bjust
  )

  # Build the inner gList. When a badge is requested, add badge grobs inside the
  # same viewport so that all x/y coordinates share the same coordinate system
  # (viewport npc + absolute mm offsets) — no compound-unit misalignment.
  inner <- if (!is.null(badge_label)) {
    badge_h   <- unit(4.5, "mm")
    badge_w   <- unit(11,  "mm")
    badge_pad <- unit(1.5, "mm")   # gap from the left edge of the box

    # Center of the badge pill in viewport coordinates.
    # unit(0, "npc") = left edge of viewport; badge_w * 0.5 + badge_pad moves
    # the center rightward so the pill's left side sits badge_pad from the box edge.
    b_x <- unit(0, "npc") + badge_w * 0.5 + badge_pad
    b_y <- unit(1, "npc")   # exactly the top border of the box

    badge_bg <- roundrectGrob(
      x = b_x, y = b_y,
      width = badge_w, height = badge_h,
      r = unit(2, "mm"),   # rounded corners — standard badge shape
      just = "center",
      gp = badge_gp
    )
    badge_lbl <- textGrob(
      label = as.character(badge_label),
      x = b_x, y = b_y,
      just = "center",
      gp = badge_txt_gp
    )
    gList(rect, txt, badge_bg, badge_lbl)
  } else {
    gList(rect, txt)
  }

  gl <- grobTree(
    inner,
    vp = do.call(viewport, vp_args),
    name = name,
    cl = "box"
  )

  structure(gl,
    coords = prCreateBoxCoordinates(viewport_data = vp_args),
    viewport_data = vp_args
  )
}

library(testthat)

test_that("Box auto width", {
  A <- boxGrob("A") %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  AA <- boxGrob("AA") %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  expect_lt(A, AA, label = "Box width should be defined from the width of the text")

  A <- boxPropGrob("A", "A", "A", prop = .5) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  AA <- boxPropGrob("AA", "AA", "AA", prop = .5) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  expect_lt(A, AA, label = "Prop box width should be defined from the width of the text")

  A <- boxPropGrob("AA", "AA", "AA", prop = .5) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  AA <- boxPropGrob("AA", "AA", "AA", prop = .2) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  expect_lt(A, AA, label = "Prop box width should be defined from the width of the text in the smallest box")
})

test_that("Box set width", {
  A <- boxGrob("A", width = unit(10, "mm")) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  AA <- boxGrob("AA", width = unit(10, "mm")) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  expect_equal(A, AA, label = "Box width should be defined from the width")

  A <- boxPropGrob("A", "A", "A", prop = .5, width = unit(10, "mm")) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  AA <- boxPropGrob("AA", "AA", "AA", prop = .5, width = unit(10, "mm")) %>% 
    grobWidth %>% 
    convertX(unitTo = "mm", valueOnly = TRUE)
  expect_equal(A, AA, label = "Prop box width should be defined from the width")
})

test_that("Box auto height", {
  A <- boxGrob("A") %>% 
    grobHeight %>% 
    convertY(unitTo = "mm", valueOnly = TRUE)
  AA <- boxGrob("A\nA") %>% 
    grobHeight %>% 
    convertY(unitTo = "mm", valueOnly = TRUE)
  expect_lt(A, AA, label = "Box height should be defined from the width of the text")
})

test_that("Box adjustment and coord consistency", {
  txt <- "AAAAAAAAAA\n  - BBBB"
  bx1 <- boxGrob(txt, x = .8)
  bx2 <- boxGrob(txt,
                 x = coords(bx1)$left, 
                 bjust = "left")
  measures <- rbind(bx1 = sapply(coords(bx1), function(x) convertUnit(x, unitTo = "npc", valueOnly = TRUE)),
                    bx2 = sapply(coords(bx2), function(x) convertUnit(x, unitTo = "npc", valueOnly = TRUE)))
  diff_per_measure <- measures[1,] - measures[2,]
  expect_equivalent(diff_per_measure,
                    rep(0, times = length(diff_per_measure)))
})

library(testthat)

test_that("N connector uses straight center branch for one-to-many", {
  start <- boxGrob("Start")
  ends <- list(boxGrob("L"), boxGrob("C"), boxGrob("R"))
  # spread ends symmetric around center
  ends <- spreadHorizontal(ends, from = unit(0.1, "npc"), to = unit(0.9, "npc"), type = "center")

  con <- connectGrob(start, ends, type = "N")
  # one of the returned grobs should be vertical (x coords equal)
  is_vertical <- vapply(con, function(g) {
    xvals <- convertX(attr(g, "line")$x, "npc", valueOnly = TRUE)
    max(xvals) - min(xvals) < 1e-6
  }, logical(1))
  expect_true(any(is_vertical))
})


test_that("N connector uses straight center branch for many-to-one", {
  ends <- list(boxGrob("L"), boxGrob("C"), boxGrob("R"))
  ends <- spreadHorizontal(ends, from = unit(0.1, "npc"), to = unit(0.9, "npc"), type = "center")
  target <- boxGrob("Target")

  con <- connectGrob(ends, target, type = "N")
  # last length(ends) elements are trunks in our implementation
  trunks <- tail(con, length(ends))
  is_vertical <- vapply(trunks, function(g) {
    xvals <- convertX(attr(g, "line")$x, "npc", valueOnly = TRUE)
    max(xvals) - min(xvals) < 1e-6
  }, logical(1))
  expect_true(any(is_vertical))
})

testthat::test_that("piped list + unit reference works (vignette case)", {
    library(grid)

    a_boxes <- paste("A", 1:3) |>
        lapply(function(x) boxGrob(x, box_gp = gpar(fill = "#E6F2FF"))) |>
        spreadHorizontal(from = unit(.1, "npc"), to = unit(1, "npc") - unit(1, "cm"))

    # This used to error with 'operator == not meaningful for units' during resolution
    expect_silent({
        res <- a_boxes |> alignVertical(position = "top", reference = unit(1, "npc"))
    })
})

library(testthat)
library(grid)

test_that("boxGrob respects global boxGrobTxtPadding option", {
  old <- options(boxGrobTxtPadding = unit(1, "mm"))
  on.exit(options(old), add = TRUE)

  b_small <- boxGrob("Padding test", txt_gp = gpar(cex = 1))
  w_small <- convertWidth(coords(b_small)$width, "mm", valueOnly = TRUE)

  options(boxGrobTxtPadding = unit(8, "mm"))
  b_large <- boxGrob("Padding test", txt_gp = gpar(cex = 1))
  w_large <- convertWidth(coords(b_large)$width, "mm", valueOnly = TRUE)

  expect_gt(w_large, w_small)
})

test_that("boxGrob accepts numeric txt_padding in millimeters", {
  b <- boxGrob("Padding test", txt_padding = 2)
  expect_s3_class(b, "box")
})

library(testthat)

test_that("alignHorizontal accepts piped list and uses first as reference", {
    b1 <- boxGrob("b1", x = .1, y = .5)
    b2 <- boxGrob("b2", x = .4, y = .5)
    b3 <- boxGrob("b3", x = .8, y = .5)

    aList <- list(b1, b2, b3)
    alignedList <- aList |> alignHorizontal(position = "center")

    cvt <- function(x) convertX(x, unitTo = "mm", valueOnly = TRUE)
    expect_equal(cvt(attr(alignedList[[1]], "viewport_data")$x), cvt(attr(alignedList[[2]], "viewport_data")$x))
    expect_equal(cvt(attr(alignedList[[1]], "viewport_data")$x), cvt(attr(alignedList[[3]], "viewport_data")$x))
})

library(testthat)

context("visual regression")

if (!requireNamespace("vdiffr", quietly = TRUE)) {
    skip("vdiffr not available")
}

# simple box grob change

test_that("boxGrob basic appearance", {
    grid::grid.newpage()
    g <- boxGrob("Test", x = .5, y = .5)
    vdiffr::expect_doppelganger("basic box", g)
})

library(testthat)

context("vdiffr connectors")

if (!requireNamespace("vdiffr", quietly = TRUE)) {
    skip("vdiffr not installed")
}

test_that("simple connector grob matches reference", {
    grid::grid.newpage()
    a <- boxGrob("A", x = .2, y = .5)
    b <- boxGrob("B", x = .8, y = .5)
    con <- connectGrob(a, b)
    vdiffr::expect_doppelganger("two-box connector", con)
})

testthat::test_that("alignVertical handles numeric reference 0 as coordinate, not list index", {
    a <- 5
    res <- testthat::expect_error(alignVertical(
        reference = 0, position = "bottom",
        bquote(alpha == theta[1] * .(a) + ldots) |> boxGrob(),
        paste("argument", sQuote("x"), "\nmust be non-zero") |> boxGrob()
    ), NA)
    testthat::expect_true(inherits(res, "Gmisc_list_of_boxes"))
})

testthat::test_that("print handles nested plain lists of boxes", {
    library(grid)
    a <- boxGrob("A", x = .2, y = .8)
    b <- boxGrob("B", x = .4, y = .6)

    # nested as plain lists
    nested <- list(grp = list(sub = list(a, b))) |>
        prExtendClass("Gmisc_list_of_boxes")

    # Should not error when printing
    testthat::expect_silent(print(nested))
})

#' Add [Hmisc::label()] to multiple columns
#'
#' Add label attribute using `dplyr` syntax using the [Hmisc::label()]
#'
#' @param x The data frame that we want to label
#' @param ... Variable names with their intended label, e.g. `mpg = "Miles per gallon"`.
#'
#' @return The original data.frame
#' @export
#'
#' @importFrom Hmisc label
#' @examples
#' library(magrittr)
#' data(mtcars)
#' mtcars_with_labels <- mtcars %>%
#'   set_column_labels(mpg = "Gas",
#'                     cyl = "Cylinders",
#'                     hp = "Strength")
#' Hmisc::label(mtcars_with_labels$mpg)
#' @family Hmisc helpers
set_column_labels <- function(x, ...) {
  labels <- list(...)
  stopifnot(is.data.frame(x))

  stopifnot(all(sapply(labels, is.character)))
  stopifnot(all(sapply(labels, length) == 1))
  stopifnot(all(names(labels) %in% colnames(x)))

  for (n in names(labels)) {
    label(x[[n]]) <- labels[[n]]
  }

  x
}

library(testthat)
context("yamlDump and Hmisc helpers")

# yamlDump behaviour - capture printed output
simple_list <- list(a = 1, b = list(c = 2))
out <- capture.output(yamlDump(simple_list))
expect_true(is.character(out))
expect_true(any(grepl("a: 1", out, fixed = TRUE)))
expect_true(any(grepl("c: 2", out, fixed = TRUE)))

# character input (JSON)
json <- '{"x": [1,2], "y": {"z":"foo"}}'
out2 <- capture.output(yamlDump(json))
expect_true(is.character(out2))
expect_true(any(grepl("x:", out2, fixed = TRUE)))
expect_true(any(grepl("z: foo", out2, fixed = TRUE)))

# set_column_labels & units
library(magrittr)
df <- data.frame(mpg = 1:3, cyl = 4:6)

labeled <- set_column_labels(df, mpg = "gas", cyl = "cylindrical")
expect_equal(Hmisc::label(labeled$mpg), "gas")
expect_equal(Hmisc::label(labeled$cyl), "cylindrical")

unitdf <- set_column_units(df, mpg = "mpg", cyl = "cyl")
expect_equal(Hmisc::units(unitdf$mpg), "mpg")
expect_equal(Hmisc::units(unitdf$cyl), "cyl")

#' Gets the maximum contributor variables from svd()
#'
#' This function is inspired by Jeff Leeks Data Analysis course where
#' he suggests that one way to use the \code{\link[base]{svd}} is to look
#' at the most influential rows for first columns in the V matrix.
#'
#' This function expands on that idea and adds the option of choosing
#' more than just the most contributing variable for each row. For instance
#' two variables may have a major impact on a certain component where the second
#' variable has 95% of the maximum variables value, since this variable is probably
#' important in that particular component it makes sense to include it
#' in the selection.
#'
#' It is of course useful when you have many continuous variables and you want
#' to determine a subgroup to look at, i.e. finding the needle in the haystack.
#'
#' @param mtrx A matrix or data frame with the variables. Note: if it contains missing
#'  variables make sure to impute prior to this function as the \code{\link[base]{svd}} can't
#'  handle missing values.
#' @param quantile The SVD D-matrix gives an estimate for the amount that is explained.
#'  This parameter is used for selecting the columns that have that quantile
#'  of explanation.
#' @param similarity_threshold A quantile for how close other variables have to be in value to
#'  maximum contributor of that particular column. If you only want the maximum value
#'  then set this value to 1.
#' @param plot_selection As this is all about variable exploring it is often interesting
#'  to see how the variables were distributed among the vectors
#' @param plot_threshold The threshold of the plotted bars, measured as
#'  percent explained by the D-matrix. By default it is set to 0.05.
#' @param varnames A vector with alternative names to the colnames
#' @return Returns a list with vector with the column numbers
#'  that were picked in the "most_influential" variable and the
#'  svd caluclation in the "svd"
#'
#' @example inst/examples/getSvdMostInfluential_example.R
#'
#' @import lattice
#' @importFrom grDevices colorRampPalette grey
#' @importFrom graphics par
#' @export
getSvdMostInfluential <- function(mtrx,
                                  quantile,
                                  similarity_threshold,
                                  plot_selection      = TRUE,
                                  plot_threshold      = 0.05,
                                  varnames            = NULL) {

  if (any(is.na(mtrx)))
    stop("Missing values are not allowed in this function. Impute prior to calling this function (try with mice or similar package).")

  if (quantile < 0 || quantile > 1)
    stop("The quantile mus be between 0-1")

  if (similarity_threshold < 0 || similarity_threshold > 1)
    stop("The similarity_threshold mus be between 0-1")

  svd_out <- svd(scale(mtrx))
  perc_explained <- svd_out$d^2/sum(svd_out$d^2) * 100
  # Select the columns that we want to look at
  cols_expl <- which(cumsum(perc_explained) <= quantile * 100)
  stopifnot(length(cols_expl) > 0)

  # Select the variables of interest
  getMostInfluentialVars <- function() {
    vars <- list()
    for (i in 1:length(perc_explained)) {
      v_abs <- abs(svd_out$v[,i])
      maxContributor <- which.max(v_abs)
      similarSizedContributors <- which(v_abs >= v_abs[maxContributor]*similarity_threshold)
      if (any(!similarSizedContributors %in% maxContributor)) {
        maxContributor <- similarSizedContributors[order(v_abs[similarSizedContributors], decreasing = TRUE)]
      }
      vars[[length(vars) + 1]] <- maxContributor
    }
    return(vars)
  }
  vars <- getMostInfluentialVars()

  plot_data <- prData2Plot(mtrx = mtrx,
                           vars = vars,
                           varnames = varnames,
                           plot_threshold = plot_threshold,
                           similarity_threshold = similarity_threshold,
                           perc_explained = perc_explained,
                           cols_expl = cols_expl)

  if (plot_selection) prPlotSvdSelection(plot_data)

  invisible(structure(list(most_influential = unique(unlist(vars[cols_expl])),
                           svd = svd_out,
                           plot_data = plot_data),
                      class = "Gmisc_svd_analysis"))
}

prGetMaxNo2Print <- function() getOption("Gmis_svd_max_no_to_print", 4)
prData2Plot <- function(mtrx,
                        vars,
                        varnames,
                        plot_threshold,
                        similarity_threshold,
                        perc_explained,
                        cols_expl) {
  if (plot_threshold < 0 || plot_threshold > 1)
    stop("The plot_threshold must be between 0-1")

  if (plot_threshold > similarity_threshold)
    stop(paste0("You can't plot less that you've chosen - it makes no sense",
                " - the plot (", plot_threshold, ")",
                " >",
                " similarity (", similarity_threshold, ")"))

  # Show all the bars that are at least at the threshold
  # level and group the rest into a "other category"
  percentage_threshold <- plot_threshold * 100
  bar_count <- length(perc_explained[perc_explained >= percentage_threshold]) + 1
  if (bar_count > length(perc_explained)) {
    bar_count <- length(perc_explained)
    plot_percent <- perc_explained
  }else{
    plot_percent <- perc_explained[perc_explained >= percentage_threshold][1:bar_count]
    plot_percent[bar_count] <- sum(perc_explained[perc_explained < percentage_threshold])
  }

  # Create transition colors
  selected_colors <- colorRampPalette(c("darkgreen", "#FFFFFF"))(bar_count + 2)[cols_expl]
  nonselected_colors <- colorRampPalette(c("darkgrey", "#FFFFFF"))(bar_count + 2)[(max(cols_expl) + 1):bar_count]

  names <- unlist(lapply(vars[1:bar_count], FUN = function(x) {
    if (is.null(varnames)) {
      varnames <- colnames(mtrx)
    }
    if (length(x) > prGetMaxNo2Print())
      ret <- paste(c(varnames[x[1:(prGetMaxNo2Print() - 1)]],
                     sprintf("+ %d other", length(x) + 1 - prGetMaxNo2Print())), collapse = "\n")
    else
      ret <- paste(varnames[x], collapse = "\n")

    return(ret)
  }))

  if (bar_count < length(perc_explained)) {
    names[bar_count] <- "Other"
    nonselected_colors[length(nonselected_colors)] <- grey(.5)
  }

  list(main = data.frame(names = names, percent = plot_percent, colors = c(selected_colors, nonselected_colors)),
       vars = vars)
}


prPlotSvdSelection <- function(data) {
  plot_data <- data$main

  max_vars <- lapply(data$vars[1:nrow(plot_data)], length) %>%
    lapply(min, prGetMaxNo2Print()) %>%
    unlist() %>%
    max()
  rotation <- 45 + (max_vars - 1) * (45 / prGetMaxNo2Print())

  m <- par(mar = c(8.1, 4.1, 4.1, 2.1))
  on.exit(par(mar = m))

  p1 <- barchart(percent ~ 1:nrow(plot_data),
                 data = plot_data,
                 horiz = FALSE,
                 ylab = "Percentage explained (%D)",
                 main = "SVD - the maximum contributors defined by V column",
                 xlab = "Pattern contributing variables",
                 col = plot_data$colors,
                 key = list(text = list(c("Selected", "Not selected")),
                            rectangles = list(col = c("darkgreen", "#777777"))),
                 scales = list(x = list(rot = rotation, labels = plot_data$names)))
  print(p1)
}

library(testthat)
context("getSvdMostInfluential")

# simple matrix with known structure
m <- diag(1, 5)
colnames(m) <- paste0("v", 1:5)

res <- getSvdMostInfluential(m,
    quantile = 0.5, similarity_threshold = 0.5,
    plot_selection = FALSE
)

# result should be the special S3 class and contain expected elements
expect_s3_class(res, "Gmisc_svd_analysis")
expect_true(is.list(res))
expect_true("most_influential" %in% names(res))
expect_true("svd" %in% names(res))

# check that most_influential is integer vector
expect_type(res$most_influential, "integer")

# ensure missing values produce an error
m2 <- m
m2[1, 1] <- NA
expect_error(getSvdMostInfluential(m2, plot_selection = FALSE), "Missing values are not allowed")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

library(Gmisc, quietly = TRUE)
library(glue)
library(htmlTable)
library(grid)

## ----flowchart-example, fig.height = 7, fig.width = 8-------------------------
# Shared styling
main_box_gp <- gpar(fill = "#ddeeff", col = "#336699", lwd = 1.5)
group_box_gp <- gpar(fill = "#e8f4e8", col = "#2e7d32", lwd = 1.5)
excl_box_gp  <- gpar(fill = "#fff8e1", col = "#cc8800", lwd = 1.2)
main_con_gp  <- gpar(col = "#336699", lwd = 1.5, fill = "#336699")
excl_con_gp  <- gpar(col = "#cc8800", lwd = 1.2, fill = "#cc8800")

grid.newpage()
flowchart(
  source = boxGrob(glue("Stockholm population\nn = {pop}", pop = txtInt(1632798)),
                   box_gp = main_box_gp),
  eligible = boxGrob(glue("Eligible\nn = {pop}", pop = txtInt(10032)),
                     box_gp = main_box_gp),
  included = boxGrob(glue("Randomized\nn = {incl}", incl = txtInt(122)),
                     box_gp = main_box_gp),
  groups = list(
    boxGrob(glue("Treatment A\nn = {n}", n = txtInt(43)),        box_gp = group_box_gp),
    boxGrob(glue("Treatment B\nn = {n}", n = txtInt(122-43-30)), box_gp = group_box_gp)
  )) |>
  spread(axis = "y") |>
  spread(subelement = "groups", axis = "x") |>
  equalizeWidths(subelement = list("source", "eligible", "included")) |>
  equalizeWidths(subelement = "groups") |>
  insert(list(excluded = boxHeaderGrob(
                header = glue("Excluded (n = {tot}):", tot = 30),
                body   = glue(" - not interested: {n1}\n - contra-indicated: {n2}",
                              n1 = 12, n2 = 18),
                bjust      = "left",
                box_gp     = excl_box_gp,
                header_gp  = gpar(col = "#cc8800", cex = 1))),
         after = "eligible",
         name  = "excluded") |>
  move(subelement = "excluded", x = .8) |>
  connect("eligible", "excluded", type = "L",    lty_gp = excl_con_gp, arrow_size = 3,
          label = "Excluded") |>
  connect("source",   "eligible", type = "vert", lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE) |>
  connect("eligible", "included", type = "vert", lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE) |>
  connect("included", "groups",   type = "N",    lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE)

## ----consort_example, fig.width = 9, fig.height = 6---------------------------
old_opts <- options(boxGrobTxtPadding = unit(3, "mm"))

box_fill     <- gpar(fill = "#ddeeff", col = "#336699", lwd = 1.5)
con_gp       <- gpar(col = "#336699", lwd = 1.5, fill = "#336699")
side_gp      <- gpar(col = "#cc8800", lwd = 1.2, fill = "#cc8800")
excl_fill    <- gpar(fill = "#fff8e1", col = "#cc8800", lwd = 1.2)
badge_gp     <- gpar(fill = "#336699", col = NA)
badge_txt_gp <- gpar(col = "white", cex = 0.65)

# Arms span the inner portion; lost boxes flank outside on each side.
# Using 0.28–0.72 keeps the arm centres far enough from the viewport edges
# that the lost boxes' right/left edges don't cross the arm centres.
left_from <- 0.28
left_to   <- 0.72
main_x    <- (left_from + left_to) / 2   # centre of the arms column (= 0.5)

grid.newpage()
flowchart(
  assessed = boxGrob(
    "Patients assessed for eligibility",
    x = main_x, box_gp = box_fill,
    badge_label = "840", badge_gp = badge_gp, badge_txt_gp = badge_txt_gp
  ),
  randomised = boxGrob(
    "Randomised",
    x = main_x, box_gp = box_fill,
    badge_label = "126", badge_gp = badge_gp, badge_txt_gp = badge_txt_gp
  ),
  arms = list(
    cast     = boxGrob("Randomised to\ncast immobilisation",
                       box_gp = box_fill,
                       badge_label = "62",
                       badge_gp = badge_gp, badge_txt_gp = badge_txt_gp),
    surgical = boxGrob("Randomised to\nsurgery",
                       box_gp = box_fill,
                       badge_label = "64",
                       badge_gp = badge_gp, badge_txt_gp = badge_txt_gp)
  ),
  # Lost-to-follow-up: one per arm — spread from 0 to 1 so they land
  # on the outside of each arm (left of cast, right of surgical)
  lost = list(
    lost_cast     = boxGrob("Lost to follow-up (n = 2)\n  1 No response\n  1 Other surgery",
                            just = "left", box_gp = excl_fill),
    lost_surgical = boxGrob("Lost to follow-up (n = 3)\n  2 No response\n  1 Other surgery",
                            just = "left", box_gp = excl_fill)
  ),
  analysis = list(
    analysis_cast     = boxGrob("Included in\nprimary analysis",
                                box_gp = box_fill,
                                badge_label = "60",
                                badge_gp = badge_gp, badge_txt_gp = badge_txt_gp),
    analysis_surgical = boxGrob("Included in\nprimary analysis",
                                box_gp = box_fill,
                                badge_label = "61",
                                badge_gp = badge_gp, badge_txt_gp = badge_txt_gp)
  )
) |>
  # Vertical spacing — extra margin to keep badges from being clipped at the top
  spread(axis = "y", margin = unit(5, "mm")) |>
  # Make arms and analysis boxes the same width so the same from/to spread
  # places their centres at matching x positions (otherwise the wider arm text
  # shifts centres relative to the narrower analysis text).
  # Alternative: skip equalizeWidths() and explicitly align x-centres,
  # e.g. with `align(axis = "x", reference = "analysis", subelement = "arms")`
  # (equivalent to `alignHorizontal(reference = "analysis", subelement = "arms")`).
  equalizeWidths(subelement = list("arms", "analysis")) |>
  # Arms and analysis in the inner span; lost boxes spread across full width
  # so that lost_cast lands left of cast and lost_surgical right of surgical
  spread(axis = "x", subelement = "arms",     from = left_from, to = left_to) |>
  spread(axis = "x", subelement = "analysis", from = left_from, to = left_to) |>
  spread(axis = "x", subelement = "lost",     from = 0,         to = 1) |>
  # Exclusion box: auto-positioned between assessed and randomised, then moved right
  insert(list(excluded = boxGrob(
    "Excluded (n = 714)\n  477 Stable ankle mortise\n   64 Incongruent ankle mortise\n   30 Previous serious trauma\n  143 Other reasons",
    just = "left", box_gp = excl_fill
  )), after = "assessed") |>
  move(subelement = "excluded", x = 0.85) |>
  # Main-flow connectors
  connect("assessed",   "randomised", type = "v", lty_gp = con_gp,  arrow_size = 3, smooth = TRUE) |>
  connect("randomised", "arms",       type = "N", lty_gp = con_gp,  arrow_size = 3, smooth = TRUE) |>
  connect("arms",       "analysis",   type = "v", lty_gp = con_gp,  arrow_size = 3) |>
  # type = "L": exits assessed's *bottom* then turns right — the "down then right" branch
  connect("assessed", "excluded", type = "L", lty_gp = side_gp, arrow_size = 3, smooth = TRUE) |>
  # Pairwise arm -> lost: sharp corners (smooth = FALSE) avoid a colour-transition
  # artefact where the orange arc would diverge from the shared blue vertical path
  # a few mm above the junction, making the line appear doubled.
  connect("arms", "lost", type = "L", lty_gp = side_gp, arrow_size = 3, smooth = TRUE)

options(old_opts)

## ----flowchart-group-width-and-padding, fig.height = 5.5, fig.width = 8-------
old_opts <- options(boxGrobTxtPadding = unit(2, "mm"))

flowchart(
  rando = glue("Randomised\nN = 100"),
  groups = list(
    glue("Group 1\nn = 50"),
    glue("Group 2\nn = 50")
  ),
  groups2 = list(
    glue("Analysed\nn = 49"),
    glue("Analysed\nn = 48")
  )
) |>
  spread(axis = "y", margin = unit(0.02, "npc")) |>
  spread(subelement = "groups", axis = "x", margin = unit(.05, "npc")) |>
  spread(subelement = "groups2", axis = "x", margin = unit(.05, "npc")) |>
  equalizeWidths(subelement = list("groups", "groups2")) |>
  connect("rando", "groups", type = "N") |>
  connect("groups", "groups2", type = "vertical")

options(old_opts)

## ----basic_box, fig.height = 1.5, fig.width = 3, message = FALSE--------------
grid.newpage()
txt <-
"Just a plain box
with some text
- Note that newline is OK"
boxGrob(txt)

## ----styled_box, fig.height = 3, fig.width = 3--------------------------------
grid.newpage()
boxGrob("A large\noffset\nyellow\nbox",
        width = .8, height = .8,
        x = 0, y = 0,
        bjust = c("left", "bottom"),
        txt_gp = gpar(col = "darkblue", cex = 2),
        box_gp = gpar(fill = "lightyellow", col = "darkblue"))

## ----prop_box, fig.height = 2, fig.width = 4----------------------------------
grid.newpage()
boxPropGrob("A box with proportions",
            "Left side", "Right side",
            prop = .7)

## ----fig.height = 3, fig.width = 4--------------------------------------------
grid.newpage()
smpl_bx <- boxGrob(
  label = "A simple box",
  x = .5,
  y = .9,
  just = "center")

prop_bx <- boxPropGrob(
  label = "A split box",
  label_left = "Left side",
  label_right = "Right side",
  x = .5,
  y = .3,
  prop = .3,
  just = "center")

plot(smpl_bx)
plot(prop_bx)

smpl_bx_coords <- coords(smpl_bx)
grid.circle(y = smpl_bx_coords$y,
            x = smpl_bx_coords$x,
            r = unit(2, "mm"),
            gp = gpar(fill = "#FFFFFF99", col = "black"))
grid.circle(y = smpl_bx_coords$bottom,
            x = smpl_bx_coords$right,
            r = unit(1, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = smpl_bx_coords$top,
            x = smpl_bx_coords$right,
            r = unit(1, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = smpl_bx_coords$bottom,
            x = smpl_bx_coords$left,
            r = unit(1, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = smpl_bx_coords$top,
            x = smpl_bx_coords$left,
            r = unit(1, "mm"),
            gp = gpar(fill = "orange"))

prop_bx_coords <- coords(prop_bx)
grid.circle(y = prop_bx_coords$y,
            x = prop_bx_coords$x,
            r = unit(2, "mm"),
            gp = gpar(fill = "#FFFFFF99", col = "black"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$right_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$right_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$left_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$left_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "orange"))

grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$right,
            r = unit(2, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$right,
            r = unit(2, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$left,
            r = unit(2, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$left,
            r = unit(2, "mm"),
            gp = gpar(fill = "orange"))

## ----extra_shapes, fig.height = 3, fig.width = 6------------------------------
# --- Branch labels + sharp diamond variant ---
grid.newpage()

# rounded and sharp diamond examples
d_rounded <- boxDiamondGrob("Decision", box_gp = gpar(fill = "#FFF4E6"))
d_sharp   <- boxDiamondGrob("Decision\n(sharp)", rounded = FALSE, box_gp = gpar(fill = "#FFF4E6"))

# outcomes
e <- boxEllipseGrob("Local", box_gp = gpar(fill = "#E6FFF4"))
r <- boxServerGrob("Server", box_gp = gpar(fill = "#E8F0FF"))

# arrange and draw
boxes <- list(decision = d_rounded, outcomes = list(e, r)) |>
  spreadHorizontal(from = unit(.1, "npc"), to = unit(.9, "npc"), subelement = "outcomes") |>
  spreadVertical() |>
  print()

# 1) quick many-to-many style connector (no labels)
con <- connectGrob(boxes$decision, boxes$outcomes, type = "N")
print(con)

# 2) explicit per-branch connectors with labels (preferred when you want text)
connectGrob(boxes$decision, boxes$outcomes[[1]], type = "N", label = "Local") |> print()
connectGrob(boxes$decision, boxes$outcomes[[2]], type = "N", label = "Server") |> print()

# 3) If you prefer the single connector and want labels on each branch:
#    place text at the midpoint of each returned grob (example)
con_list <- connectGrob(boxes$decision, boxes$outcomes, type = "N")
# Preferred: attach labels and let `print()` handle rendering
con_list <- setConnectorLabels(con_list, c("Local", "Server"))
print(con_list)

## ----standard_shapes, fig.height = 3.5, fig.width = 8-------------------------
# Arrange shapes in three rows for better readability
# 1) Grid-based objects (basic boxGrob / boxPropGrob / rect)
row1 <- list(
  boxGrob("Box (default)", box_gp = gpar(fill = "#EFEFEF"), y = unit(.85, "npc")),
  boxPropGrob("Prop", "Left", "Right", prop = .4, box_left_gp = gpar(fill = "#EFEFAF"), box_right_gp = gpar(fill = "#EFAFEF"), y = unit(.85, "npc")),
  boxGrob("Rectangle", box_fn = rectGrob, box_gp = gpar(fill = "#EFEFEF"), y = unit(.85, "npc"))
)

# 2) Gmisc row 1 (rounded/sharp diamond + ellipse + rack + server)
row2 <- list(
  boxDiamondGrob("Diamond\n(rounded)", box_gp = gpar(fill = "#FFF4E6"), y = unit(.55, "npc")),
  boxDiamondGrob("Diamond\n(sharp)", rounded = FALSE, box_gp = gpar(fill = "#FFF4E6"), y = unit(.55, "npc")),
  boxEllipseGrob("Ellipse", box_gp = gpar(fill = "#E6FFF4"), y = unit(.55, "npc")),
  boxRackGrob("Rack", box_gp = gpar(fill = "#E8F0FF"), y = unit(.55, "npc")),
  boxServerGrob("Server", box_gp = gpar(fill = "#E8F0FF"), y = unit(.55, "npc"))
)

# 3) Gmisc row 2 (database, document, documents, tape)
row3 <- list(
  boxDatabaseGrob("Database", box_gp = gpar(fill = "#DFF4E6"), y = unit(.25, "npc")),
  boxDocumentGrob("Document", box_gp = gpar(fill = "#FFF6E6"), y = unit(.25, "npc")),
  boxDocumentsGrob("Documents", box_gp = gpar(fill = "#FFF6E6"), y = unit(.25, "npc")),
  boxTapeGrob("Tape", box_gp = gpar(fill = "#E6F0FF"), y = unit(.25, "npc"))
)

# Spread each row across the horizontal span
spreadHorizontal(row1, from = unit(.05, "npc"), to = unit(.95, "npc"))
spreadHorizontal(row2, from = unit(.05, "npc"), to = unit(.95, "npc"))
spreadHorizontal(row3, from = unit(.05, "npc"), to = unit(.95, "npc"))

## ----"Connected boxes", fig.width = 7, fig.height = 5-------------------------
grid.newpage()
# Initiate the boxes that we want to connect
side <- boxPropGrob("Side", "Left", "Right",
                    prop = .3,
                    x = 0, y = .9,
                    bjust = c(0,1))
start <- boxGrob("Top",
                 x = .6, y = coords(side)$y,
                 box_gp = gpar(fill = "yellow"))
bottom <- boxGrob("Bottom",
                  x = .6, y = 0,
                  bjust = "bottom")


sub_side_left <- boxGrob("Left",
                         x = coords(side)$left_x,
                         y = 0,
                         bjust = "bottom")
sub_side_right <- boxGrob("Right",
                          x = coords(side)$right_x,
                          y = 0,
                          bjust = "bottom")

odd <- boxGrob("Odd\nbox",
               x = coords(side)$right,
               y = .5)

odd2 <- boxGrob("Also odd",
               x = coords(odd)$right +
                 distance(bottom, odd, type = "h", half = TRUE) -
                 unit(2, "mm"),
               y = 0,
               bjust = c(1,0))

exclude <- boxGrob("Exclude:\n - Too sick\n - Prev. surgery",
                   x = 1,
                   y = coords(bottom)$top +
                     distance(start, bottom, type = "v", half = TRUE),
                   just = "left", bjust = "right")

# Connect the boxes and print/plot them
connectGrob(start, bottom, "vertical")
connectGrob(start, side, "horizontal")
connectGrob(bottom, odd, "Z", "l")
connectGrob(odd, odd2, "N", "l")
connectGrob(side, sub_side_left, "v", "l")
connectGrob(side, sub_side_right, "v", "r")
connectGrob(start, exclude, "-",
            lty_gp = gpar(lwd = 2, col = "darkred", fill = "darkred"))

# Print the grobs
start
bottom
side
exclude
sub_side_left
sub_side_right
odd
odd2

## ----connect_multi, fig.width = 4, fig.height = 4-----------------------------
grid.newpage()

# Three upstream boxes + one side box
a_boxes <- paste("A", 1:3) |>
  lapply(\(x) boxGrob(x, box_gp = gpar(fill = "#E6F2FF"))) |>
  spreadHorizontal(from = unit(.1, "npc"), to = unit(1, "npc") - unit(1, "cm")) |>
  alignVertical(position="top",
                reference = unit(1, "npc")) |>
  print()

b_side <- boxGrob("B",  y = .70, box_gp = gpar(fill = "#FFF3BF")) |>
  moveBox(x = unit(1, "npc"),
          just = 1) |>
  print()

# Target box
c <- boxGrob("C", x = .50, box_gp = gpar(fill = "#D3F9D8"), width = unit(4, "cm")) |>
  moveBox(y = unit(0, "npc"),
          just = "bottom") |>
  print()


# Many -> one: merge on top with evenly distributed attachment points + margin
connectGrob(c(a_boxes, list(b_side)), c,
            type = "fan_in_top",
            margin = 4)

## ----horizontal_alignment, fig.width=10, fig.height=6-------------------------
align_1 <- boxGrob("Align 1",
                   y = .9,
                   x = 0,
                   bjust = c(0),
                   box_gp = gpar(fill = "#E6E8EF"))

align_2 <- boxPropGrob("Align 2",
                       "Placebo",
                       "Treatment",
                       prop = .7,
                       y = .8,
                       x = .5)

align_3 <- boxGrob("Align 3\nvertical\ntext",
                   y = 1,
                   x = 1,
                   bjust = c(1, 1),
                   box_gp = gpar(fill = "#E6E8EF"))

b1 <- boxGrob("B1",
              y = .3,
              x = .1,
              bjust = c(0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .6,
              x = .5)
b3 <- boxGrob("B3",
              y = .2,
              x = .8,
              bjust = c(0, 1))

grid.newpage()
align_1
alignHorizontal(reference = align_1,
                b1, b2, b3,
                position = "left")

align_2
alignHorizontal(reference = align_2,
                b1, b2, b3,
                position = "center",
                sub_position = "left")
alignHorizontal(reference = align_2,
                b1, b2, b3,
                position = "left",
                sub_position = "right")

align_3
alignHorizontal(reference = align_3,
                b1, b2, b3,
                position = "right")

## ----vertical_alignment, fig.width=10, fig.height=6---------------------------
align_1 <- boxGrob("Align 1\nvertical\ntext",
                   y = 1,
                   x = 1,
                   bjust = c(1, 1),
                   box_gp = gpar(fill = "#E6E8EF"))

align_2 <- boxPropGrob("Align 2",
                       "Placebo",
                       "Treatment",
                       prop = .7,
                       y = .5,
                       x = .6)

align_3 <- boxGrob("Align 3",
                   y = 0,
                   x = 0,
                   bjust = c(0, 0),
                   box_gp = gpar(fill = "#E6E8EF"))


b1 <- boxGrob("B1",
              y = .3,
              x = 0.1,
              bjust = c(0, 0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .6,
              x = .3)
b3 <- boxGrob("B3",
              y = .2,
              x = .85,
              bjust = c(0, 1))

grid.newpage()
align_1
alignVertical(reference = align_1,
              b1, b2, b3,
              position = "top")

align_2
alignVertical(reference = align_2,
              b1, b2, b3,
              position = "center")

align_3
alignVertical(reference = align_3,
              b1, b2, b3,
              position = "bottom")

## ----horizontal_spread, fig.width = 11, fig.height = 8------------------------
b1 <- boxGrob("B1", y = .85, x = .1, bjust = c(0, 0))
b2 <- boxGrob("B2", y = .65, x = .6)
b3 <- boxGrob("B3", y = .45, x = .6)
b4 <- boxGrob("B4 with long\ndescription", y = .7, x = .8)

from <- boxGrob("from",
                y = .25,
                x = .05,
                box_gp = gpar(fill = "darkgreen"),
                txt_gp = gpar(col = "white"))
to <- boxGrob("to this wide box",
              y = coords(from)$y,
              x = .95,
              bjust = "right",
              box_gp = gpar(fill = "darkred"),
              txt_gp = gpar(col = "white"))
txtOut <- function(txt, y_top) {
  grid.text(txt,
            x = unit(2, "mm"),
            y = y_top + unit(2, "mm"),
            just = c("left", "bottom"))
  grid.lines(y = y_top + unit(1, "mm"),
             gp = gpar(col = "grey"))
}

drawRow <- function(label, row_y, spread_args = list()) {
  row <- alignVertical(reference = row_y, b1, b2, b3, b4, position = "top")
  txtOut(label, coords(row[[1]])$top)
  do.call(spreadHorizontal, c(list(row), spread_args))
}

rowYs <- unit(c(.93, .76, .59, .42, .25, .12), "npc")

grid.newpage()

drawRow("Basic (viewport)", rowYs[1])
drawRow("From–to + margin (numeric = npc)", rowYs[2],
        spread_args = list(from = .2, to = .7, margin = .05))
drawRow("Only to (defaults from = 0)", rowYs[3],
        spread_args = list(to = .7))
drawRow("Only from (defaults to = 1)", rowYs[4],
        spread_args = list(from = .2))

# Row 5: Between boxes (box-to-box span)
row5_y <- rowYs[5]
row5 <- alignVertical(reference = row5_y, b1, b2, b3, b4, position = "top")
txtOut("Between boxes", coords(row5[[1]])$top)

span <- alignVertical(reference = row5_y, from  = from, to = to, position = "top")
span
spreadHorizontal(row5, from = span$from, to = span$to)

# Row 6: Reverse box order + center distribution
row6_y <- unit(.10, "npc")

bottom_from <- moveBox(from, x = coords(to)$right, y = 0, just = c(1, 0))
bottom_to <- moveBox(to, x = coords(from)$left, y = 0, just = c(0, 0))
bottom_from
bottom_to

row6 <- alignVertical(reference = bottom_from, b1, b2, b3, b4, position = "bottom")
txtOut("Reverse box order + center", coords(row6[[4]])$top)

spreadHorizontal(row6,
                 from = bottom_from,
                 to = bottom_to,
                 type = "center")



## ----vertical_spread, fig.width=6, fig.height=6-------------------------------
b1 <- boxGrob("B1",
              y = .8,
              x = 0.1,
              bjust = c(0, 0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .5,
              x = .5)
b3 <- boxGrob("B3",
              y = .2,
              x = .8)
b4 <- boxGrob("B4",
              y = .7,
              x = .8)


txtOut <- function(txt, refBx) {
  grid.text(txt,
            x = coords(refBx)$left - unit(2, "mm"),
            y = .5,
            just = c("center", "bottom"),
            rot = 90)
  grid.lines(x = coords(refBx)$left - unit(1, "mm"),
             gp = gpar(col = "grey"))
}

grid.newpage()
txtOut("Basic", b1)
alignHorizontal(reference = b1,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical()

txtOut("From-to", b2)
alignHorizontal(reference = b2,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical(from = .2,
                 to = .7)

txtOut("From-to with center and reverse the box order", b3)
alignHorizontal(reference = b3,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical(from = .7,
                 to = .2,
                 type = "center")

## ----complex_nested, fig.width = 7, fig.height = 8----------------------------
# Helper function to convert nested structure to grobs
make_boxes <- function(x) {
  if (is.list(x) && !inherits(x, "box_header")) {
    return(lapply(x, make_boxes))
  }

  if (inherits(x, "box_header")) {
    return(do.call(boxHeaderGrob, x))
  }

  # Simple text box fallback
  args <- attr(x, "args")
  if (is.null(args)) return(boxGrob(label = x))

  args$label <- x
  do.call(boxGrob, args)
}

# Define styling for different elements
arm_a_style <- list(
  header = gpar(fill = "#E8F5E9", col = "#2E7D32", lwd = 1.4),
  box = gpar(fill = "#F1F8E9", col = "#43A047")
)

arm_b_style <- list(
  header = gpar(fill = "#FFF8E1", col = "#EF6C00", lwd = 1.4),
  box = gpar(fill = "#FFFDE7", col = "#F9A825")
)

# Build flowchart structure
flowchart <- list(
  # Shared inclusion criteria
  criteria = structure(
    list(
      header = "Inclusion Criteria",
      body = paste(
        "• Adults aged 18-65",
        "• Confirmed diagnosis",
        "• Written informed consent",
        "• No contraindications",
        "• Available for 6-month follow-up",
        sep = "\n"
      ),
      box_gp = gpar(fill = "#E3F2FD", col = "#1E88E5", lwd = 1.4),
      body_gp = gpar(fontsize = 10)
    ),
    class = "box_header"
  ),

  # Two treatment arms
  arms = list(
    arm_a = list(
      # Arm header
      structure("Intensive Protocol", args = list(
        box_gp = arm_a_style$header,
        txt_gp = gpar(fontsize = 11, fontface = "bold")
      )),

      # Timeline boxes
      structure(list(
        header = "Week 0-1",
        body = "• Daily sessions\n• Supervised therapy\n",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Week 2-4",
        body = "• 3× weekly sessions\n• Progressive loading",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Week 5-8",
        body = "• Home program\n• Monthly check-ins\n• Return to activity",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header")
    ),

    arm_b = list(
      # Arm header
      structure("Standard Care", args = list(
        box_gp = arm_b_style$header,
        txt_gp = gpar(fontsize = 11, fontface = "bold")
      )),

      # Timeline boxes - different schedule
      structure(list(
        header = "Month 0",
        body = "• Initial consultation\n• Exercise booklet",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Month 3",
        body = "• Follow-up visit\n• Progress review",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Month 6",
        body = "• Final assessment\n• Discharge planning",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header")
    )
  )
)

# Convert to grobs and layout
grid.newpage()
boxes <- flowchart |>
  make_boxes() |>
  spreadVertical() |>
  spreadHorizontal(subelement = "arms", from = 0.15, to = 0.85) |>
  spreadVertical(subelement = c("arms", "arm_a"), from = 0.65) |>
  spreadVertical(subelement = c("arms", "arm_b"), from = 0.65) |>
  print()

# Connect criteria to both arms
connectGrob(boxes$criteria, boxes$arms, type = "N")

# Connect timeline within each arm
for (arm_name in names(boxes$arms)) {
  arm_boxes <- boxes$arms[[arm_name]]
  for (i in 2:length(arm_boxes)) {
    connectGrob(arm_boxes[[i-1]], arm_boxes[[i]], type = "v") |> print()
  }
}

## ----s3_api_example, fig.height=8, fig.width=6, eval=FALSE--------------------
# grid.newpage()
# 
# # Define the nodes
# b1 <- boxGrob("Start", y = 0.8)
# b2 <- boxGrob("Process", y = 0.5)
# b3 <- boxGrob("End", y = 0.2)
# 
# # Pipeline: list -> align -> connect -> print
# list(start = b1, process = b2, end = b3) |>
#   align(axis = "y") |>
#   spread(axis = "x") |>
#   connect("start", "process", type = "horizontal") |>
#   connect("process", "end", type = "horizontal") |>
#   print()

## ----math_expressions, fig.width=6, fig.height=3------------------------------
grid.newpage()
###############
# Expressions #
###############
# Font style
list(expression(bold("Bold text")),
     expression(italic("Italics text")),
     expression(paste("Mixed: ", italic("Italics"), " and ", bold("bold")))) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(1, "npc"),
                position = "top") |>
  spreadHorizontal()

# Math
list(expression(paste("y = ", beta[0], " + ", beta[1], X[1], " + ", beta[2], X[2]^2)),
     expression(paste(hat(mu) == sum(frac(x[i], n), i == 1, n))),
     expression(paste(int(a, b, f(x) * dx) == F(b) - F(a)))) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(0.5, "npc"),
                position = "center") |>
  spreadHorizontal()

##########
# Quotes #
##########
a = 5
list(bquote(alpha == theta[1] * .(a) + ldots),
     paste("argument", sQuote("x"), "\nmust be non-zero")) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(0, "npc"),
                position = "bottom") |>
  spreadHorizontal(from = .2, to = .8)

## ----basic_plot, fig.height = 2, fig.width = 2--------------------------------
# Load the grid library
# part of standard R libraries so no need installing
library(grid)

# Create a new graph
grid.newpage()

pushViewport(viewport(width = .5, height = .8))

grid.rect(gp = gpar(fill = "#D8F0D1"))

popViewport()

## ----relative_lines, fig.height = 3, fig.width = 3----------------------------
grid.newpage()
pushViewport(viewport(width = .5, height = .8, clip = "on"))
grid.rect(gp = gpar(lty = 2, fill = "lightyellow"))
lg <- linesGrob(x = unit(c(.2, 1), "npc"),
                y = unit(c(.2, 1), "npc"),
                gp = gpar(lwd = 2))
grid.draw(lg)
pushViewport(viewport(x = 0, y = .6, just = "left", width = .4, height = .4, angle = 20))
grid.rect(gp = gpar(fill = "lightblue")) # A translucent box to indicate the new viewport
grid.draw(lg)
popViewport()

## ----absolute_lines, fig.height = 3, fig.width = 3----------------------------
grid.newpage()
pushViewport(viewport(width = .5, height = .8, clip = "on"))
grid.rect(gp = gpar(lty = 2, fill = "lightyellow"))
lg <- linesGrob(x = unit(c(2, 10), "mm"),
                y = unit(c(2, 10), "mm"),
                gp = gpar(lwd = 2))
grid.draw(lg)
pushViewport(viewport(x = 0, y = .6, just = "left", width = .4, height = .4, angle = 20))
grid.rect(gp = gpar(fill = "lightblue")) # A translucent box to indicate the new viewport
grid.draw(lg)
popViewport()

## ----complex_example, fig.height = 9, fig.width = 9---------------------------
# Define the boxes
org_cohort <- glue("Proximal humerus fracture",
                   "  - \u2265 18 years",
                   "  - \u2264 4 weeks of trauma",
                   "  - Not pathological",
                   .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#E3F2FD"))

surgery <- glue("Surgery",
                "  - Direct (\u2248 4%)",
                "  - Delayed (\u2248 4%)",
                .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#F8BBD0"))

randomize <- boxGrob("Non-surgical\nRandomise",
                     box_gp = gpar(fill = "#FFF3E0"))

treatments <- list(early = boxGrob("Early rehab",
                                   box_gp = gpar(fill = "#DCEDC8")),
                   late = boxGrob("Late rehab",
                                  box_gp = gpar(fill = "#DCEDC8")),
                   obs = boxGrob("Observation",
                                 box_gp = gpar(fill = "#E0E0E0")))

early_followup <- glue("Early follow-up",
                       "  - 2 weeks [PNRS]",
                       "  - 4 weeks [PNRS]",
                       .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#E0F7FA"))

late_followup <- glue("Late follow-up",
                      "  - 2-10 months (random) [OSS, PNRS]",
                      "  - 1 year [OSS, PNRS, accelerometer]",
                      "  - 2 years [OSS, PNRS]",
                      "  - 5 years [OSS, PNRS]",
                      .sep = "\n") |>
  boxGrob(just = "left",
                       box_gp = gpar(fill = "#E0F7FA"))

# Create the flowchart
grid.newpage()
flowchart(start = org_cohort,
          step_1 = list(surgery = surgery,
                        `non-surgical` = randomize),
          treatment = treatments,
          early_followup = early_followup,
          followup = late_followup) |>
  spread(axis = "y") |>
  spread(axis = "x", subelement = "step_1") |>
  spread(axis = "x", subelement = "treatment", from = 0.35) |>
  align(axis = "x",
        reference = c("treatment", "late"),
        subelement = c("step_1", "non-surgical")) |>
  connect(from = "start", to = "step_1", type = "N") |>
  connect(from = "step_1$non-surgical", to = "treatment", type = "N") |>
  connect(from = "treatment", to = "early_followup", type = "fan_in_center") |>
  connect(from = "early_followup", to = "followup", type = "v") |>
  connect(from = "early_followup", to = "step_1$surgery", type = "Z",
          label = "Crossover\nto surgery") |>
  connect(from = "step_1$surgery", to = "followup", type = "L") |>
  print()

#' Retrieve basic description stats by header
#'
#' Helper for [getDescriptionStatsBy] that retrieves the basic
#' header names.
#'
#' @param already_table_format Just a boolean as we use this in the total column
#'
#' @return A vector with basic headers
#' @inheritParams getDescriptionStatsBy
prGetDescHeader <- function(by, html, header_count, already_table_format = FALSE) {
  if (already_table_format) {
    tbl_cnt <- by
  } else {
    tbl_cnt <- table(by)
  }
  if (is.null(header_count) || identical(header_count, FALSE)) {
    return(names(tbl_cnt))
  }

  if (is.character(header_count)) {
    if (!grepl("%s", header_count, fixed = TRUE)) {
      stop(
        "Your header_count must accept a string character",
        " or it will fail to add the count, i.e. use the ",
        " format: 'Text before %s text after'"
      )
    }
    cnt_str <-
      sprintf(
        header_count,
        txtInt(tbl_cnt)
      )
  } else {
    cnt_str <- paste("No.", txtInt(tbl_cnt))
  }

  return(mapply(txtMergeLines,
                names(tbl_cnt),
                cnt_str,
                html = html
  ))
}

#' A function for converting a show_missing variable.
#'
#' The variable is supposed to be directly compatible with
#' \code{\link[base]{table}}(..., useNA = show_missing). It throws an error
#' if not compatible. It is mostly useful for custom describe functions.
#'
#' \emph{Deprecated:} This function will be deprecated
#' as all functions now use the useNA style in order to comply
#' with standard R naming.
#'
#' @param show_missing Boolean or "no", "ifany", "always"
#' @return string
#'
#' @export
convertShowMissing <- function(show_missing) {
  if (missing(show_missing) ||
    show_missing == FALSE ||
    show_missing == "no") {
    return("no")
  }

  if (show_missing == TRUE) {
    return("ifany")
  }

  if (!show_missing %in% c("no", "ifany", "always")) {
    stop(
      "You have set an invalid option for show_missing variable",
      " '", show_missing, "' - it should be TRUE/FALSE",
      " or one of the options: no, ifany or always."
    )
  }

  if (length(show_missing) > 1) {
    stop("You have an invalid show_missing variable of more than one elements: ", length(show_missing))
  }

  return(show_missing)
}

#' Get default from/to values for an axis
#'
#' When axis is "x", the default is from 0 to 1 npc, i.e. left to right.
#' When axis is "y", the default is from 1 to 0 npc, i.e. top to bottom.
#'
#' @param axis Axis, either "x" or "y"
#' @return A list with `from` and `to` values as `unit` objects
prGetBoxAxisDefaults <- function(axis = c("x", "y")) {
  axis <- match.arg(axis)
  if (axis == "x") {
    list(from = unit(0, "npc"), to = unit(1, "npc"))
  } else {
    list(from = unit(1, "npc"), to = unit(0, "npc"))
  }
}

prNormalizeFromTo <- function(from, to, axis = c("x", "y")) {
  axis <- match.arg(axis)
  defaults <- prGetBoxAxisDefaults(axis)

  prAsNPC <- function(u) {
    if (is.null(u)) {
      return(NULL)
    }

    if (inherits(u, "unit")) {
      return(u)
    }

    if (is.numeric(u) && length(u) == 1) {
      return(prAsUnit(u, units = "npc"))
    }

    u
  }
  from <- prAsNPC(from)
  to <- prAsNPC(to)

  if (!is.null(from) && is.null(to)) {
    to <- defaults$to
  } else if (is.null(from) && !is.null(to)) {
    from <- defaults$from
  } else if (is.null(from) && is.null(to)) {
    from <- defaults$from
    to <- defaults$to
  }

  list(from = from, to = to)
}

#' Get the box coordinates
#'
#' Retrieves the boxes \code{"coords"} attribute.
#'
#' @param box The \code{\link{boxGrob}} or \code{\link{boxPropGrob}}
#' @return A list with the coordinates
#'
#' @importFrom checkmate assert_class
#' @family flowchart components
#' @export
#' @examples
#' box <- boxGrob("A test box")
#' coords(box)
coords <- function(box) {
  # Check if not already a coordinate element
  if (inherits(box, "coords")) {
    return(box)
  }

  assert_class(box, "box")
  attr(box, "coords")
}

library(grid)
grid.newpage()

pushViewport(viewport(y = 0.85, height = unit(.3, "npc")))
grid.rect(gp = gpar(col = "steelblue"))
grid.text("Example 1", y = .9, x = unit(10, "mm"), just = "left")
box <- boxGrob("A simple box", x = .5, y = .5) # Start at the middle
moveBox(box, x = -.2, space = "relative") # Move to the left
popViewport()


## Advanced example (skipped during R CMD check)
## Not run:
# Advanced example: create a nested list of treatment boxes, spread them horizontally,
# then move a single nested element (`Ibuprofen`) using `subelement` via a pipe.
pushViewport(viewport(y = 0.35, height = unit(.7, "npc") - unit(2, "mm")))
grid.rect(gp = gpar(col = "lightgreen"))

boxes <- list(
    population = boxGrob("Population"),
    treatment = list(
        Ibuprofen = boxGrob("Ibuprofen"),
        Paracetamol = boxGrob("Paracetamol"),
        Aspirin = boxGrob("Aspirin")
    ),
    followup = paste("Follow-up 4 weeks:",
        " - EQ-5D 5L",
        " - Lab",
        sep = "\n"
    ) |>
        boxGrob(just = "left")
) |>
    alignHorizontal(position = "center") |>
    spreadVertical() |>
    spreadHorizontal(
        from = unit(0.1, "npc"),
        to = unit(0.9, "npc"),
        type = "center",
        subelement = "treatment"
    ) |>
    moveBox(
        y = 0.1,
        space = "relative",
        subelement = c("treatment", "Ibuprofen")
    )

boxes
connectGrob(boxes$population, boxes$treatment, type = "N")
connectGrob(boxes$treatment, boxes$followup, type = "N")
## End(Not run)

#' Output boxes
#'
#' Outputs a list of boxes as produced by either the
#' spread or align functions for boxGrobs.
#'
#' @param x A list of a set of [`boxGrob`]/[`boxPropGrob`] to plot
#' @param ... Ignored argument
#' @export
print.Gmisc_list_of_boxes <- function(x, ...) {
  # Draw boxes
  for (box in x) {
    if (is.grob(box)) {
      grid.draw(box)
    } else if (inherits(box, "Gmisc_list_of_boxes")) {
      for (i in 1:length(box)) {
        print(box[[i]])
      }
    } else if (is.list(box)) {
      # treat plain lists containing boxes as nested Gmisc_list_of_boxes for printing
      print(prExtendClass(box, "Gmisc_list_of_boxes"))
    } else {
      stop("Element is not a grob or a Gmisc_list_of_boxes", class(box))
    }
  }

  # Draw stored connections
  conns <- attr(x, "connections")
  if (!is.null(conns)) {
    for (cg in conns) {
      grid.draw(cg)
    }
  }

  invisible(x)
}

#' @rdname getDescriptionStatsBy
#' @import htmlTable
#' @export
htmlTable.Gmisc_getDescriptionStatsBy <- function(x, ...) {
  if (attr(x, "multiple")) {
    htmlTable_args <- list(...)
    htmlTable_args <- htmlTable_args[names(htmlTable_args) %in% formalArgs(htmlTable)]
    if (length(htmlTable_args) > 0) {
      x$htmlTable_args <- htmlTable_args
    }

    merged_desc <- do.call(mergeDesc, x)

    # Retain htmlTable formatting if used htmlTable::addHtmlTableStyle()
    return(copyAllNewAttributes(from = x, to = merged_desc))
  }

  htmlTable_args <- list(...)
  htmlTable_args <- htmlTable_args[names(htmlTable_args) %in% formalArgs(htmlTable)]
  htmlTable_args$x <- x
  class(htmlTable_args$x) <- tail(class(htmlTable_args$x), -1)

  do.call(htmlTable, htmlTable_args)
}

#' @rdname getDescriptionStatsBy
#' @export
#' @importFrom knitr asis_output
#' @importFrom knitr knit_print
print.Gmisc_getDescriptionStatsBy <- function(x, ...) {
  htmlTable(x, ...) %>%
    print()
}

#' @rdname getDescriptionStatsBy
#' @export
knit_print.Gmisc_getDescriptionStatsBy <- function(x, ...) {
  htmlTable(x, ...) %>%
    knit_print()
}

#' @rdname getDescriptionStatsBy
#' @export
length.Gmisc_getDescriptionStatsBy <- function(x) {
  if (attr(x, "multiple")) {
    return(nrow(attr(x, "raw_data")))
  }

  length(attr(x, "raw_data"))
}

#' Deprecated function(s) in \pkg{Gmisc}
#'
#' The \code{forestplot2} has moved to \pkg{forestplot} package and changed name to \code{\link[forestplot]{forestplot}}
#'
#' @param ... Parameters to be passed to the new version of the function
#' @export
#' @keywords internal
#' @import forestplot
#' @rdname Gmisc-deprecated
#' @name Gmisc-deprecated
#' @docType package
forestplot2 <- function(...) {
  .Deprecated(new = "forestplot",
              package = "forestplot",
              old = "forestplot2",
              msg = "The forestplot2 has moved to a separate package as of 1.0, the 'forestplot' package")
  forestplot(...)
}

library(testthat)
context("deprecated functions")

# forestplot2 should run with minimal arguments without error
expect_error(suppressWarnings(forestplot2(labeltext = 1, mean = 1, lower = 0, upper = 2)), NA)

#' Creating of description statistics
#'
#' A function that returns a description statistic that can be used
#' for creating a publication "table 1" when you want it by groups.
#' The function identifies if the variable is a continuous, binary
#' or a factored variable. The format is inspired by NEJM, Lancet &
#' BMJ.
#'
#' @section Customizing statistics:
#'
#' You can specify what function that you want for statistic by providing a function
#' that takes two arguments \code{x} and \code{by} and returns a p-value. There are
#' a few functions already prepared for this see \code{\link{getPvalAnova}},
#' \code{\link{getPvalChiSq}}
#' \code{\link{getPvalFisher}}
#' \code{\link{getPvalKruskal}}
#' \code{\link{getPvalWilcox}}.
#' The default functions used are \code{getPvalFisher} and \code{getPvalWilcox} (unless the by
#' argument has more than three unique levels where it defaults to \code{getPvalAnova}).
#'
#' If you want the function to select functions depending on the type of input
#' you can provide a list with the names \code{'continuous'}, \code{'proportion'}, \code{'factor'} and
#' the function will choose accordingly. If you fail to define a certain category
#' it will default to the above.
#'
#' You can also use a custom function that returns a string with the attribute \code{'colname'}
#' set that will be appended to the results instead of the p-value column.
#'
#' @param x If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
#'          If it is a single variable it will be the core value that want the statistics for.
#'          In the print this is equivalent to the output of this function.
#' @param ... The variables that you want you statistic for. In the print all thes parameters
#'          are passed on as [htmlTable::htmlTable] arguments.
#' @param by The variable that you want to split into different columns
#' @param digits The number of decimals used
#' @param digits.nonzero The number of decimals used for values that are close to zero
#' @param html If HTML compatible output should be used. If \code{FALSE}
#'  it outputs LaTeX formatting
#' @param NEJMstyle Adds - no (\%) at the end to proportions
#' @param numbers_first If the number should be given or if the percentage
#'  should be presented first. The second is encapsulated in parentheses ().
#' @param useNA This indicates if missing should be added as a separate
#'  row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
#'  \emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.
#' @param useNA.digits The number of digits to use for the
#'  missing percentage, defaults to the overall \code{digits}.
#' @param continuous_fn The method to describe continuous variables. The
#'  default is \code{\link{describeMean}}.
#' @param prop_fn The method used to describe proportions, see \code{\link{describeProp}}.
#' @param factor_fn The method used to describe factors, see \code{\link{describeFactors}}.
#' @param statistics Add statistics, fisher test for proportions and Wilcoxon
#'  for continuous variables. See details below for more customization.
#' @param statistics.two_dec_lim The limit for showing two decimals. E.g.
#'  the p-value may be 0.056 and we may want to keep the two decimals in order
#'  to emphasize the proximity to the all-mighty 0.05 p-value and set this to
#'  \eqn{10^-2}. This allows that a value of 0.0056 is rounded to 0.006 and this
#'  makes intuitive sense as the 0.0056 level as this is well below
#'  the 0.05 value and thus not as interesting to know the exact proximity to
#'  0.05. \emph{Disclaimer:} The 0.05-limit is really silly and debated, unfortunately
#'  it remains a standard and this package tries to adapt to the current standards in order
#'  to limit publication associated issues.
#' @param statistics.sig_lim The significance limit for < sign, i.e. p-value 0.0000312
#'  should be < 0.0001 with the default setting.
#' @param statistics.suppress_warnings Hide warnings from the statistics function.
#' @param show_all_values Show all values in proportions. For factors with only two values
#'  it is most sane to only show one option as the other one will just be a complement
#'  to the first, i.e. we want to convey a proportion. For instance sex - if you know
#'  gender then automatically you know the distribution of the other sex as it's 100 \% - other \%.
#'  To choose which one you want to show then set the \code{default_ref} parameter.
#' @param hrzl_prop This is default FALSE and indicates
#'  that the proportions are to be interpreted in a vertical manner.
#'  If we want the data to be horizontal, i.e. the total should be shown
#'  and then how these differ in the different groups then set this to TRUE.
#' @param add_total_col This adds a total column to the resulting table.
#'  You can also specify if you want the total column "first" or "last"
#'  in the column order.
#' @param total_col_show_perc This is by default true but if
#'  requested the percentages are suppressed as this sometimes may be confusing.
#' @param use_units If the Hmisc package's units() function has been employed
#'  it may be interesting to have a column at the far right that indicates the
#'  unit measurement. If this column is specified then the total column will
#'  appear before the units (if specified as last). You can also set the value to
#'  \code{"name"} and the units will be added to the name as a parenthesis,
#'  e.g. Age (years).
#' @param units_column_name The name of the units column. Used if use_units = TRUE
#' @param percentage_sign If you want to suppress the percentage sign you
#'  can set this variable to FALSE. You can also choose something else that
#'  the default \% if you so wish by setting this variable.
#' @param header_count Set to \code{TRUE} if you want to add a header count,
#'  e.g. Smoking; No. 25 observations, where there is a new line after the
#'  factor name. If you want a different text for the second line you can
#'  specifically use the \code{\link[base]{sprintf}} formatting, e.g. "No. \%s patients".
#' @param missing_value Value that is substituted for empty cells. Defaults to "-"
#' @param names_of_missing Optional character vector containing the names of returned statistics,
#'  in case all returned values for a given \code{by} level are missing. Defaults to NULL
#' @param default_ref The default reference when dealing with proportions. When using
#'  `dplyr` syntax (`tidyselect`) you can specify a named vector/list for each column name.
#' @return Returns \code{matrix} if a single value was provided, otherwise a \code{list}
#'  of matrices with the class \code{"Gmisc_getDescriptionStatsBy"}.
#'
#' @example inst/examples/getDescriptionStatsBy_example.R
#'
#' @family descriptive functions
#'
#' @importFrom Hmisc label label<- units capitalize
#' @importFrom stats na.omit
#' @importFrom methods is
#'
#' @export
getDescriptionStatsBy <- function(x,
                                  ...,
                                  by,
                                  digits = 1,
                                  digits.nonzero = NA,
                                  html = TRUE,
                                  numbers_first = TRUE,
                                  statistics = FALSE,
                                  statistics.sig_lim = 10^-4,
                                  statistics.two_dec_lim = 10^-2,
                                  statistics.suppress_warnings = TRUE,
                                  useNA = c("ifany", "no", "always"),
                                  useNA.digits = digits,
                                  continuous_fn = describeMean,
                                  prop_fn = describeProp,
                                  factor_fn = describeFactors,
                                  show_all_values = FALSE,
                                  hrzl_prop = FALSE,
                                  add_total_col,
                                  total_col_show_perc = TRUE,
                                  use_units = FALSE,
                                  units_column_name = "Units",
                                  default_ref = NULL,
                                  NEJMstyle = FALSE,
                                  percentage_sign = TRUE,
                                  header_count = NULL,
                                  missing_value = "-",
                                  names_of_missing = NULL) {
  UseMethod("getDescriptionStatsBy")
}

#' @exportS3Method
#' @importFrom rlang expr enquo
#' @importFrom methods formalArgs
getDescriptionStatsBy.data.frame <- function(x,
                                             ...,
                                             by,
                                             digits = 1,
                                             digits.nonzero = NA,
                                             html = TRUE,
                                             numbers_first = TRUE,
                                             statistics = FALSE,
                                             statistics.sig_lim = 10^-4,
                                             statistics.two_dec_lim = 10^-2,
                                             statistics.suppress_warnings = TRUE,
                                             useNA = c("ifany", "no", "always"),
                                             useNA.digits = digits,
                                             continuous_fn = describeMean,
                                             prop_fn = describeProp,
                                             factor_fn = describeFactors,
                                             show_all_values = FALSE,
                                             hrzl_prop = FALSE,
                                             add_total_col,
                                             total_col_show_perc = TRUE,
                                             use_units = FALSE,
                                             units_column_name = "Units",
                                             default_ref = NULL,
                                             NEJMstyle = FALSE,
                                             percentage_sign = TRUE,
                                             header_count = NULL,
                                             missing_value = "-",
                                             names_of_missing = NULL) {
  Call = match.call()
  safeLoadPkg("tidyselect")
  vars <- tidyselect::eval_select(rlang::expr(c(...)), x)
  if (missing(by)) {
    if (length(vars) == 2) {
      by_var <- vars[2]
      by <- x[[by_var]]
      vars <- head(vars, -1)
    } else {
      stop("When providing a data.frame as input you must specify the by variables")
    }
  } else {
    by_var <- tidyselect::eval_select(rlang::enquo(by), x)
    by = x[[by_var]]
  }

  if (label(by) == "") {
    label(by) <- names(by_var)
  }

  base_call <- as.list(Call)
  base_call <- base_call[!names(base_call) %in% c("", "x")]
  base_call <- base_call[names(base_call) %in% formalArgs(getDescriptionStatsBy)]
  base_call$by = by

  get_single_result <- function(var, name) {
    base_call$x = x[[var]]
    column_name <- colnames(x)[var]
    # If the user has set the name in the input parameter then it is
    # highly likely that they want to override any label() name
    if (column_name != name && name != "" && !is.null(name) || label(base_call$x) == "") {
      label(base_call$x) <- name
    }

    if (column_name %in% names(default_ref)) {
      base_call$default_ref <- default_ref[[column_name]]
    } else if (!is.null(names(default_ref))) {
      base_call$default_ref <- NULL
    }
    do.call(getDescriptionStatsBy, base_call)
  }

  if (length(vars) == 1) {
    return(get_single_result(var = vars, name = names(vars)))
  }

  multiple_results <- mapply(FUN = get_single_result,
                             var = vars,
                             name = names(vars),
                             SIMPLIFY = FALSE,
                             USE.NAMES = FALSE)
  structure(multiple_results,
            class = c("Gmisc_getDescriptionStatsBy", class(multiple_results)),
            multiple = TRUE,
            raw_data = x)
}

#' @exportS3Method
getDescriptionStatsBy.default <- function(x,
                                          ...,
                                          by,
                                          digits = 1,
                                          digits.nonzero = NA,
                                          html = TRUE,
                                          numbers_first = TRUE,
                                          statistics = FALSE,
                                          statistics.sig_lim = 10^-4,
                                          statistics.two_dec_lim = 10^-2,
                                          statistics.suppress_warnings = TRUE,
                                          useNA = c("ifany", "no", "always"),
                                          useNA.digits = digits,
                                          continuous_fn = describeMean,
                                          prop_fn = describeProp,
                                          factor_fn = describeFactors,
                                          show_all_values = FALSE,
                                          hrzl_prop = FALSE,
                                          add_total_col,
                                          total_col_show_perc = TRUE,
                                          use_units = FALSE,
                                          units_column_name = "Units",
                                          default_ref = NULL,
                                          NEJMstyle = FALSE,
                                          percentage_sign = TRUE,
                                          header_count = NULL,
                                          missing_value = "-",
                                          names_of_missing = NULL) {
  vars <- list(...)
  if (missing(by)) {
    by = vars[[1]]
    vars[[1]] <- NULL
  }

  if (length(vars) > 0) {
    stop("As of version 3.0 of Gmisc the htmlTable handles unnamed parameters differently. Try to use named arguments instead of positional.")
  }

  useNA <- match.arg(useNA)
  if (!is.na(digits.nonzero)) {
    if (!is.numeric(digits.nonzero) ||
        floor(digits.nonzero) != digits.nonzero
    ) {
      stop("The digits.nonzero should be an integer, you provided: ", digits.nonzero)
    }
    if (digits.nonzero < digits) {
      stop("The digits.nonzero must be smaller than digits")
    }
  }

  if (!is.function(statistics)) {
    if (is.list(statistics) ||
        (statistics != FALSE)) {
      if (is.list(statistics)) {
        types <- c(
          "continuous",
          "proportion",
          "factor"
        )
        if (any(!names(statistics) %in% types)) {
          stop(
            "If you want to provide custom functions for generating statistics",
            " you must either provide a function or a list with the elements:",
            " '", paste(types, collapse = "', '"), "'"
          )
        }

        if (is.numeric(x) &&
            length(unique(x)) != 2) {
          statistics <- statistics[["continuous"]]
        } else if (length(unique(x)) == 2) {
          if ("proportion" %in% names(statistics)) {
            statistics <- statistics[["proportion"]]
          } else {
            statistics <- statistics[["factor"]]
          }
        } else {
          statistics <- statistics[["factor"]]
        }

        if (is.character(statistics)) {
          statistics <- get(statistics)
        }
      }

      if (!is.function(statistics)) {
        if (length(unique(x)) == 2) {
          statistics <- getPvalFisher
        } else if (is.numeric(x)) {
          if (length(unique(by)) == 2) {
            statistics <- getPvalWilcox
          } else {
            statistics <- getPvalAnova
          }
        } else {
          statistics <- getPvalFisher
        }
      }
    }
  }

  # Always have a total column if the description statistics
  # are presented in a horizontal fashion
  if (missing(add_total_col) &&
      hrzl_prop) {
    add_total_col <- TRUE
  }

  if (is.null(x)) {
    stop(
      "You haven't provided an x-value to do the statistics by.",
      " This error is most frequently caused by referencing an old",
      " variable name that doesn't exist anymore"
    )
  }
  if (is.null(by)) {
    stop(
      "You haven't provided a by-value to do the statistics by.",
      " This error is most frequently caused by referencing an old",
      " variable name that doesn't exist anymore"
    )
  }

  # If there is a label for the variable
  # that one should be used otherwise go
  # with the name of the variable
  if (label(x) == "") {
    name <- paste0(deparse(substitute(x)), collapse = "")
  } else {
    name <- label(x)
  }

  if (is.logical(x)) {
    x <- factor(x, levels = c(TRUE, FALSE))
  } else if (is.character(x) && any(table(x, by) == 0)) {
    x <- factor(x)
  }

  # Check missing -
  # Send a warning, since the user might be unaware of this
  # potentially disturbing fact. The dataset should perhaps by
  # subsetted by is.na(by) == FALSE
  if (any(is.na(by))) {
    warning(
      sprintf("Your 'by' variable has %d missing values", sum(is.na(by))),
      "\n   The corresponding 'x' and 'by' variables are automatically removed"
    )
    x <- x[!is.na(by)]
    if (inherits(x, "factor")) {
      x <- factor(x)
    }
    by <- by[!is.na(by)]
    if (inherits(by, "factor")) {
      by <- factor(by)
    }
  }

  if (useNA == "ifany" &&
      any(is.na(x))) {
    useNA <- "always"
  }

  # If all values are to be shown then simply use
  # the factors function, provided describeProp was specified
  if (show_all_values & deparse(substitute(prop_fn)) == "describeProp") {
    prop_fn <- describeFactors
  }

  if (is.numeric(x) || is(x, "difftime")) {
    t <- prNumericDescs(x = x,
                        by = by,
                        hrzl_prop = hrzl_prop,
                        continuous_fn = continuous_fn,
                        html = html,
                        digits = digits,
                        digits.nonzero = digits.nonzero,
                        numbers_first = numbers_first,
                        useNA = useNA,
                        useNA.digits = useNA.digits,
                        percentage_sign = percentage_sign,
                        missing_value = missing_value,
                        names_of_missing = names_of_missing)
  } else if ((!is.factor(x) &&
              length(unique(na.omit(x))) == 2) ||
             (is.factor(x) &&
              length(levels(x)) == 2) &&
             hrzl_prop == FALSE) {
    t <- prPropDescs(x = x,
                     by = by,
                     name = name,
                     default_ref = default_ref,
                     prop_fn = prop_fn,
                     html = html,
                     digits = digits,
                     digits.nonzero = digits.nonzero,
                     numbers_first = numbers_first,
                     useNA = useNA,
                     useNA.digits = useNA.digits,
                     percentage_sign = percentage_sign,
                     missing_value = missing_value,
                     names_of_missing = names_of_missing,
                     NEJMstyle = NEJMstyle)
  } else {
    # Make sure that the total isn't using proportions (happens with hrzl_prop)
    prop_fn <- factor_fn
    t <- prFactorDescs(x = x,
                       by = by,
                       factor_fn = factor_fn,
                       hrzl_prop = hrzl_prop,
                       html = html,
                       digits = digits,
                       digits.nonzero = digits.nonzero,
                       numbers_first = numbers_first,
                       useNA = useNA,
                       useNA.digits = useNA.digits,
                       percentage_sign = percentage_sign,
                       missing_value = missing_value,
                       names_of_missing = names_of_missing)
  }

  # Convert into a matrix
  results <- prAddEmptyVals(t, missing_value = missing_value) %>%
    unlist() %>%
    matrix(ncol = length(t)) %>%
    prFixDescRownames(t = t, name = name)

  attr(results, "column_names") <- prGetDescHeader(by = by, header_count = header_count, html = html)
  attr(results, "label") <- name

  if (!missing(add_total_col) && add_total_col != FALSE) {
    results %<>% prAddTotalDescColumn(x = x,
                                      by = by,
                                      numbers_first = numbers_first,
                                      total_col_show_perc = total_col_show_perc,
                                      show_all_values = show_all_values,
                                      useNA = useNA,
                                      useNA.digits = useNA.digits,
                                      html = html,
                                      digits = digits,
                                      continuous_fn = continuous_fn,
                                      factor_fn = factor_fn,
                                      prop_fn = prop_fn,
                                      percentage_sign = percentage_sign,
                                      header_count = header_count,
                                      add_total_col = add_total_col,
                                      default_ref = default_ref)
  }

  if (!isFALSE(use_units)) {
    results %<>% prAddDescUnitColumn(x = x,
                                     use_units = use_units,
                                     units_column_name = units_column_name)
  }

  if (is.function(statistics)) {
    results %<>% prAddDescStats(x = x,
                                by = by,
                                statistics = statistics,
                                statistics.suppress_warnings = statistics.suppress_warnings,
                                statistics.sig_lim = statistics.sig_lim,
                                statistics.two_dec_lim = statistics.two_dec_lim,
                                html = html)
  }

  colnames(results) <- attr(results, "column_names")

  # Even if one row has the same name this doesn't matter
  # at this stage as it is information that may or may
  # not be used later on
  label(results) <- attr(results, "label")

  structure(results,
            class = c("Gmisc_getDescriptionStatsBy", class(results)),
            multiple = FALSE,
            raw_data = list(x = x, by = by))
}

library("testthat")
library("stringr")


data("Loblolly")

set.seed(1)
Loblolly$young <- Loblolly$age < 10
Loblolly$young <- factor(Loblolly$young, label = c("Yes", "No"))
Loblolly$fvar <- factor(sample(letters[1:3], size = nrow(Loblolly), replace = TRUE))
Loblolly$young_w_missing <- Loblolly$young
Loblolly$young_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA
Loblolly$fvar_w_missing <- Loblolly$fvar
Loblolly$fvar_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA
Loblolly$height_w_missing <- Loblolly$height
Loblolly$height_w_missing[sample(1:nrow(Loblolly), size = 4)] <- NA

test_that("Statistics work as expected", {
  a <- getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
                             hrzl_prop = TRUE,
                             continuous_fn = describeMedian,
                             statistics = TRUE,
                             digits = 2, statistics.sig_lim = 10^-4
  )

  true_fisher_pval <- txtPval(fisher.test(Loblolly$fvar, Loblolly$young)$p.value,
                              statistics.sig_lim = 10^-4
  )

  expect_equivalent(
    as.character(a[1, "P-value"]),
    true_fisher_pval
  )

  a <- getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
                             hrzl_prop = TRUE,
                             continuous_fn = describeMedian,
                             statistics = list("factor" = getPvalFisher),
                             digits = 2, statistics.sig_lim = 10^-4
  )

  expect_equivalent(
    as.character(a[1, "P-value"]),
    true_fisher_pval
  )

  expect_error(getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
                                     hrzl_prop = TRUE,
                                     continuous_fn = describeMedian,
                                     statistics = list("factor" = getNonExistentPvalueFunction),
                                     digits = 2, statistics.sig_lim = 10^-4
  ))

  a <- getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
                             hrzl_prop = TRUE,
                             continuous_fn = describeMedian,
                             statistics = function(x, by) {
                               return(.2)
                             },
                             digits = 2, statistics.sig_lim = 10^-4
  )

  expect_equivalent(
    as.character(a[1, "P-value"]),
    "0.20", "Custom p-value function problem"
  )

  a <- getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
                             hrzl_prop = TRUE,
                             continuous_fn = describeMedian,
                             statistics = function(x, by) {
                               a <- "test"
                               attr(a, "colname") <- "test"
                               return(a)
                             },
                             digits = 2, statistics.sig_lim = 10^-4
  )

  expect_equivalent(
    as.character(a[1, "test"]),
    "test", "Errror when adding a string p-value alternative"
  )

  expect_error(getDescriptionStatsBy(Loblolly$fvar, Loblolly$young,
                                     hrzl_prop = TRUE,
                                     continuous_fn = describeMedian,
                                     statistics = function(x, by) {
                                       a <- "test"
                                       return(a)
                                     },
                                     digits = 2, statistics.sig_lim = 10^-4
  ),
  info = "The colname attribute must be present"
  )
})

data("mtcars")

### checks for issue #32: display of p-values for multi-row summaries
cars_missing <- mtcars
cars_missing$mpg[3] <- NA
test_that("p-values are displayed in multi-row summaries when rgroup and n.rgroup are specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "19.7 (&plusmn;1.5)",
      "0 (0%)", "15.1 (&plusmn;2.6)", "0 (0%)", "", ""
    ),
    .Dim = c(2L, 4L),
    .Dimnames = list(
      c("Mean (SD)", "Missing"),
      c("4", "6", "8", "P-value")
    ),
    rgroup = structure("Gas",
      add = structure(list(`1` = "&lt; 0.0001"),
        .Names = "1"
      )
    ),
    n.rgroup = 2,
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(getDescriptionStatsBy(
    x = cars_missing$mpg,
    by = cars_missing$cyl,
    statistics = TRUE
  ),
  htmlTable_args = list(rgroup = c("Gas"), n.rgroup = 2)
  )
  expect_equivalent(out, expected)
})

test_that("p-vlues are displayed in multi-row summaries when rgroup and n.rgroup are not specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "19.7 (&plusmn;1.5)",
      "0 (0%)", "15.1 (&plusmn;2.6)", "0 (0%)", "", ""
    ),
    .Dim = c(2L, 4L),
    .Dimnames = list(
      c("Mean (SD)", "Missing"),
      c("4", "6", "8", "P-value")
    ),
    rgroup = structure("cars_missing$mpg",
      add = structure(list(`1` = "&lt; 0.0001"),
        .Names = "1"
      )
    ),
    n.rgroup = 2L,
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(getDescriptionStatsBy(
    x = cars_missing$mpg,
    by = cars_missing$cyl,
    statistics = TRUE
  ))
  expect_equivalent(out, expected)
})

test_that("p-values are displayed in the rgroup title for both multi- and one-row summaries when rgroup and n.rgroup are specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "105.1 (&plusmn;26.9)",
      "19.7 (&plusmn;1.5)", "0 (0%)", "183.3 (&plusmn;41.6)",
      "15.1 (&plusmn;2.6)", "0 (0%)", "353.1 (&plusmn;67.8)", "", "", ""
    ),
    .Dim = 3:4,
    .Dimnames = list(
      c("Mean (SD)", "Missing", "cars_missing$disp"),
      c("4", "6", "8", "P-value")
    ),
    rgroup = structure(c("Gas", "Displacement"),
      add = list("&lt; 0.0001", "&lt; 0.0001")
    ),
    n.rgroup = c(2, 1),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(getDescriptionStatsBy(
    x = cars_missing$mpg,
    by = cars_missing$cyl,
    statistics = TRUE
  ),
  getDescriptionStatsBy(
    x = cars_missing$disp,
    by = cars_missing$cyl,
    statistics = TRUE
  ),
  htmlTable_args = list(
    rgroup = c("Gas", "Displacement"),
    n.rgroup = c(2, 1)
  )
  )
  expect_equivalent(out, expected)
})

test_that("p-values are displayed for both multi- and one-row summaries when rgroup and n.rgroup are not specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "105.1 (&plusmn;26.9)",
      "19.7 (&plusmn;1.5)", "0 (0%)", "183.3 (&plusmn;41.6)",
      "15.1 (&plusmn;2.6)", "0 (0%)", "353.1 (&plusmn;67.8)",
      "", "", "&lt; 0.0001"
    ),
    .Dim = 3:4,
    .Dimnames = list(
      c("Mean (SD)", "Missing", "cars_missing$disp"),
      c("4", "6", "8", "P-value")
    ),
    rgroup = structure(c("cars_missing$mpg", ""),
      add = structure(list(`1` = "&lt; 0.0001"), .Names = "1")
    ),
    n.rgroup = c(2, 1),
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(
    getDescriptionStatsBy(
      x = cars_missing$mpg,
      by = cars_missing$cyl,
      statistics = TRUE
    ),
    getDescriptionStatsBy(
      x = cars_missing$disp,
      by = cars_missing$cyl,
      statistics = TRUE
    )
  )
  expect_equivalent(out, expected)
})

test_that("p-vlues are displayed in multi-row summaries when rgroup and n.rgroup are not specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "19.7 (&plusmn;1.5)",
      "0 (0%)", "15.1 (&plusmn;2.6)", "0 (0%)", "", ""
    ),
    .Dim = c(2L, 4L),
    .Dimnames = list(
      c("Mean (SD)", "Missing"),
      c("4", "6", "8", "P-value")
    ),
    rgroup = structure("cars_missing$mpg",
                       add = structure(list(`1` = "&lt; 0.0001"),
                                       .Names = "1"
                       )
    ),
    n.rgroup = 2L,
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(getDescriptionStatsBy(
    x = cars_missing$mpg,
    by = cars_missing$cyl,
    statistics = TRUE
  ))
  expect_equivalent(out, expected)
})

test_that("p-values are displayed in the rgroup title for both multi- and one-row summaries when rgroup and n.rgroup are specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "105.1 (&plusmn;26.9)",
      "19.7 (&plusmn;1.5)", "0 (0%)", "183.3 (&plusmn;41.6)",
      "15.1 (&plusmn;2.6)", "0 (0%)", "353.1 (&plusmn;67.8)", "", "", ""
    ),
    .Dim = 3:4,
    .Dimnames = list(
      c("Mean (SD)", "Missing", "cars_missing$disp"),
      c("4", "6", "8", "P-value")
    ),
    rgroup = structure(c("Gas", "Displacement"),
                       add = list("&lt; 0.0001", "&lt; 0.0001")
    ),
    n.rgroup = c(2, 1),
    htmlTable_args = structure(list(), .Names = character(0)),
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(getDescriptionStatsBy(x = cars_missing$mpg,
                                         by = cars_missing$cyl,
                                         statistics = TRUE),
                   getDescriptionStatsBy(x = cars_missing$disp,
                                         by = cars_missing$cyl,
                                         statistics = TRUE),
                   htmlTable_args = list(rgroup = c("Gas", "Displacement"),
                                         n.rgroup = c(2, 1)))
  expect_equivalent(out, expected)
})


test_that("p-values are displayed for both multi- and one-row summaries when rgroup and n.rgroup are not specified", {
  expected <- structure(
    c(
      "27.1 (&plusmn;4.6)", "1 (9.1%)", "105.1 (&plusmn;26.9)",
      "19.7 (&plusmn;1.5)", "0 (0%)", "183.3 (&plusmn;41.6)",
      "15.1 (&plusmn;2.6)", "0 (0%)", "353.1 (&plusmn;67.8)",
      "", "", "&lt; 0.0001"),
    .Dim = 3:4,
    .Dimnames = list(c("Mean (SD)", "Missing", "cars_missing$disp"),
                     c("4", "6", "8", "P-value")),
    rgroup = structure(c("cars_missing$mpg", ""),
                       add = structure(list(`1` = "&lt; 0.0001"), .Names = "1")),
    n.rgroup = c(2, 1),
    class = c("descMrg", class(matrix(1)))
  )

  out <- mergeDesc(
    getDescriptionStatsBy(
      x = cars_missing$mpg,
      by = cars_missing$cyl,
      statistics = TRUE
    ),
    getDescriptionStatsBy(
      x = cars_missing$disp,
      by = cars_missing$cyl,
      statistics = TRUE
    )
  )
  expect_equivalent(out, expected)
})

library(testthat)
library(dplyr)

test_that("Check basic tidyselect functionality", {
  set.seed(1)

  loblolly <- datasets::Loblolly
  loblolly$young <- factor(loblolly$age < 10, labels = c("Yes", "No"))
  loblolly$fvar <- factor(sample(letters[1:3], size = nrow(loblolly), replace = TRUE))

  out <- loblolly |>
    getDescriptionStatsBy(
      height,
      age,
      by = young,
      statistics = TRUE,
      digits = 2,
      statistics.sig_lim = 1e-4
    )

  expect_length(out, nrow(loblolly))

  # Class structure can legitimately vary across R/dplyr/rlang versions.
  # Check something stable instead of "exactly 2 classes after removing one".
  expect_true(is.object(out))
  expect_gte(length(class(out)), 1)
})

test_that("Using default_ref with tidyselect", {
  set.seed(1)

  n <- 20
  d <- data.frame(
    a  = sample(LETTERS[1:2], size = n, replace = TRUE),
    b  = sample(LETTERS[1:2], size = n, replace = TRUE),
    by = sample(letters[1:2], size = n, replace = TRUE)
  ) |>
    mutate(
      across(-by, \(x) factor(x, levels = LETTERS[1:2])),
      across(by,  \(x) factor(x, levels = letters[1:2]))
    )

  out_standard <- d |>
    getDescriptionStatsBy(a, b, by = by)

  out_with_A <- d |>
    getDescriptionStatsBy(a, b, default_ref = "A", by = by)

  expect_identical(out_with_A, out_standard)

  out_with_B <- d |>
    getDescriptionStatsBy(a, b, default_ref = "B", by = by)

  expect_false(identical(out_with_B, out_standard))

  out_with_a_as_B <- d |>
    getDescriptionStatsBy(a, b, default_ref = c(a = "B"), by = by)

  expect_identical(out_with_a_as_B[[1]], out_with_B[[1]])
  expect_identical(out_with_a_as_B[[2]], out_standard[[2]])
})

org_par <- par(ask = TRUE)
set.seed(1345)
# Simulate data with a pattern
dataMatrix <- matrix(rnorm(15 * 160), ncol = 15)
colnames(dataMatrix) <- c(
    paste("Pos.3:", 1:3, sep = " #"),
    paste("Neg.Decr:", 4:6, sep = " #"),
    paste("No pattern:", 7:8, sep = " #"),
    paste("Pos.Incr:", 9:11, sep = " #"),
    paste("No pattern:", 12:15, sep = " #"))

for (i in 1:nrow(dataMatrix)) {
  # flip a coin
  coinFlip1 <- rbinom(1, size = 1, prob = 0.5)
  coinFlip2 <- rbinom(1, size = 1, prob = 0.5)
  coinFlip3 <- rbinom(1, size = 1, prob = 0.5)

  # if coin is heads add a common pattern to that row
  if (coinFlip1) {
    cols <- grep("Pos.3", colnames(dataMatrix))
    dataMatrix[i, cols] <- dataMatrix[i, cols] + 3
  }

  if (coinFlip2) {
    cols <- grep("Neg.Decr", colnames(dataMatrix))
    dataMatrix[i, cols] <- dataMatrix[i, cols] - seq(from = 5, to = 15, length.out = length(cols))
  }

  if (coinFlip3) {
    cols <- grep("Pos.Incr", colnames(dataMatrix))
    dataMatrix[i, cols] <- dataMatrix[i, cols] + seq(from = 3, to = 15, length.out = length(cols))
  }
}

# Illustrate data
heatmap(dataMatrix, Colv = NA, Rowv = NA, margins = c(7, 2), labRow = "")

svd_out <- svd(scale(dataMatrix))

library(lattice)
b_clr <- c("steelblue", "darkred")
key <- simpleKey(
  rectangles = TRUE, space = "top", points = FALSE,
  text = c("Positive", "Negative")
)
key$rectangles$col <- b_clr

b1 <- barchart(as.table(svd_out$v[, 1]),
               main = "First column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 1] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

b2 <- barchart(as.table(svd_out$v[, 2]),
               main = "Second column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 2] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

b3 <- barchart(as.table(svd_out$v[, 3]),
               main = "Third column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 3] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

b4 <- barchart(as.table(svd_out$v[, 4]),
               main = "Fourth column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 4] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

# Note that the fourth has the no pattern columns as the
# chosen pattern, probably partly because of the previous
# patterns already had been identified
print(b1, position = c(0, 0.5, .5, 1), more = TRUE)
print(b2, position = c(0.5, 0.5, 1, 1), more = TRUE)
print(b3, position = c(0, 0, .5, .5), more = TRUE)
print(b4, position = c(0.5, 0, 1, .5))

# Let's look at how well the SVD identifies
# the most influential columns
getSvdMostInfluential(dataMatrix,
                      quantile = .8,
                      similarity_threshold = .9,
                      plot_threshold = .05,
                      plot_selection = TRUE)
par(org_par)

#' Calculate evenly spaced attachment X positions on end box top
#'
#' Given a target `end` box and a number of `starts`, calculate `n` attachment
#' x positions along the end box top edge, applying `margin`. The returned
#' positions are converted to `npc` units to make the layout robust when the
#' plotting device or viewport is resized.
#'
#' @param starts A `list` of start boxes (only length is used).
#' @param end A `boxGrob` for which top-edge slots are required.
#' @param margin Margin applied to the left/right ends of the top edge. Numeric
#'   values are interpreted as millimetres.
#' @return A `unit` vector of length `length(starts)` with attachment x-positions
#'   in `npc` units.
#' @keywords internal
#' @noRd
pcCalculateXPositions <- function(starts, end, margin = unit(2, "mm")) {
  e <- coords(end)
  n <- length(starts)
  xs_end <- prEdgeSlots(e$left, e$right, n = n, margin = margin)
  # Convert attachment x positions to npc numeric values and wrap back to
  # npc units so downstream code can rely on `unit(..., 'npc')` values.
  unit(vapply(xs_end, \(u) prGetNpcValue(u, "x"), numeric(1)), "npc")
}

#' Generate connector line grobs for fan-in-on-top
#'
#' Create the stem and trunk line grobs that visually connect `starts` to the
#' top edge of `end`. Stems are vertical segments from each start to the shared
#' `bend_y`, and trunks are diagonal segments from the bend to the respective
#' end attachment slots. The returned list contains grobs with class
#' `connect_boxes` and the combined result has class `connect_boxes_list`.
#'
#' @param starts A `list` of start `boxGrob` objects.
#' @param end The end `boxGrob` object.
#' @param bend_y A `unit` giving the common bend y-coordinate.
#' @param xs_end A `unit` vector (npc) with target x-positions on the end box top.
#' @param lty_gp A `grid::gpar` describing line appearance.
#' @param arrow_obj A `grid::arrow` object for arrowheads on trunks.
#' @param subelmnt Optional sub-element specifier for `boxPropGrob` anchors.
#' @return A `list` of `connect_boxes` grobs with class `connect_boxes_list`.
#' @keywords internal
#' @noRd
prGenerateLines <- function(starts, end, bend_y, xs_end, lty_gp, arrow_obj, subelmnt) {
  s_coords <- lapply(starts, coords)
  e <- coords(end)

  sub_prefix <- if (missing(subelmnt)) "" else paste0(match.arg(subelmnt, c("right", "left")), "_")
  x_at <- function(pos, side = c("left", "right", "x")) {
    side <- match.arg(side)
    if (side == "x" && !is.null(pos[[paste0(sub_prefix, side)]])) {
      return(pos[[paste0(sub_prefix, side)]])
    }
    pos[[side]]
  }

  starts_above <- prStartsAbove(starts, end)

  mk <- function(x, y, arrow = NULL) {
    ux <- unit.c(x)
    uy <- unit.c(y)
    line <- list(x = ux, y = uy)
    g <- grid::linesGrob(x = ux, y = uy, gp = lty_gp, arrow = arrow)
    structure(g, line = line, class = unique(c("connect_boxes", class(g))))
  }

  # 1) stems: each start -> bend_y (no arrow)
  stem_grobs <- lapply(seq_along(s_coords), function(i) {
    s <- s_coords[[i]]
    x0 <- x_at(s, "x")
    # Ensure x0 is an npc unit for consistent behavior on resize
    if (!inherits(x0, "unit")) {
      x0 <- unit(x0, "npc")
    } else {
      x0 <- prGetNpcValue(x0, "x") |> unit("npc")
    }
    y0 <- if (starts_above) s$bottom else s$top

    mk(
      x = unit.c(x0, x0),
      y = unit.c(y0, bend_y),
      arrow = NULL
    )
  })

  # 2) trunks: assign end slots to starts by their left->right order so
  #    trunks meet the end's top edge in a visually consistent way. Also, when
  #    there is an odd number of starts and the middle one is aligned with the
  #    end (within tolerance) make that branch target the end center (straight).
  starts_x_units <- lapply(s_coords, \(s) x_at(s, "x"))
  starts_x_vals <- vapply(starts_x_units, \(u) prGetNpcValue(u, "x"), numeric(1))
  xs_end_vals <- vapply(xs_end, \(u) prGetNpcValue(u, "x"), numeric(1))
  ord <- order(starts_x_vals)

  # map left-to-right end slots to starts
  assigned_end_vals <- numeric(length(starts_x_vals))
  assigned_end_vals[ord] <- xs_end_vals

  # If centered straight branch desired, override middle assigned slot
  centered <- prNShouldUseCenteredBranch(starts, end)
  if (centered) {
    mid_sorted_idx <- ord[ceiling(length(ord) / 2)]
    targ_center <- prGetNpcValue(prConvert2Coords(end)$x, "x")
    # Prefer the target center when available; otherwise fall back to the
    # start's x coordinate to guarantee a vertical trunk when conversions fail.
    if (!is.na(targ_center)) {
      assigned_end_vals[mid_sorted_idx] <- targ_center
    } else if (!is.na(starts_x_vals[mid_sorted_idx])) {
      assigned_end_vals[mid_sorted_idx] <- starts_x_vals[mid_sorted_idx]
    }
  }

  trunk_grobs <- lapply(seq_along(starts_x_units), function(i) {
    x_start <- starts_x_units[[i]]
    x_end <- unit(assigned_end_vals[i], "npc")

    # If this is the centered branch, force the trunk to be vertical by
    # using the assigned end value for both start and end x coordinates.
    if (centered && i == mid_sorted_idx) {
      x_start <- x_end
    } else {
      # Convert start to npc unit as well. If it's a unit-like object, use
      # prGetNpcValue to obtain the numeric npc and wrap back to a unit.
      if (!inherits(x_start, "unit")) {
        x_start <- unit(x_start, "npc")
      } else {
        x_start <- prGetNpcValue(x_start, "x") |> unit("npc")
      }
    }

    mk(
      x = unit.c(x_start, x_end),
      y = unit.c(bend_y, e$top + unit(0.5, "mm")),
      arrow = arrow_obj
    )
  })

  # If we used a centered straight branch, make the corresponding stem
  # vertical as well by matching its x coords to the assigned trunk x value.
  if (centered) {
    mid_sorted_idx <- ord[ceiling(length(ord) / 2)]
    x_val_npc <- assigned_end_vals[mid_sorted_idx]
    if (!is.na(x_val_npc)) {
      x_unit <- unit(x_val_npc, "npc")
      old_stem <- stem_grobs[[mid_sorted_idx]]
      y_vals <- attr(old_stem, "line")$y
      stem_grobs[[mid_sorted_idx]] <- mk(x = unit.c(x_unit, x_unit), y = y_vals, arrow = NULL)
    }
  }

  structure(c(stem_grobs, trunk_grobs),
    class = c("connect_boxes_list", "list")
  )
}


#' Create a fan-in connector merging onto the top of an end box
#'
#' High-level helper that coordinates the calculation of the bend point and the
#' attachment positions and then generates the line grobs that join multiple
#' `starts` to a single `end` box using the fan-in-on-top layout.
#'
#' @param starts A `list` of `boxGrob` starts.
#' @param end A single `boxGrob` end target.
#' @param subelmnt Optional sub-element specifier for split boxes (`"left"` or
#'   `"right"`).
#' @param lty_gp A `grid::gpar` controlling line appearance.
#' @param arrow_obj A `grid::arrow` object for arrowheads.
#' @param margin Margin (mm) applied when computing end attachment slots.
#' @param split_pad Padding around the shared bend point (mm) used to avoid
#'   overlapping the end box.
#' @return A `connect_boxes_list` (list of grobs) representing the connector.
#' @keywords internal
#' @noRd
prConnectManyToOneFanTop <- function(
  starts,
  end,
  subelmnt,
  lty_gp,
  arrow_obj,
  margin = unit(2, "mm"),
  split_pad = unit(0, "mm")
) {
  # If `end` is container-like, attempt to find a reasonable boxed target.
  if (!inherits(end, "box")) {
    fb <- prFindFirstBox(end)
    if (!is.null(fb)) end <- fb
  }

  assert_class(end, "box")
  assert_class(lty_gp, "gpar")
  assert_class(arrow_obj, "arrow")
  if (is.numeric(margin)) margin <- unit(margin, "mm")
  if (is.numeric(split_pad)) split_pad <- unit(split_pad, "mm")

  bend_y <- prCalculateBendY(starts, end, edge = "top", margin = margin, split_pad = split_pad)
  xs_end <- pcCalculateXPositions(starts, end, margin)
  prGenerateLines(starts, end, bend_y, xs_end, lty_gp, arrow_obj, subelmnt)
}

#' A simple thing to keep the attributes
#'
#' Skips the attributes that the to object already has
#' to avoid overwriting dim and other important attributes
#'
#' @param from The from object
#' @param to The to object
#' @param attr2skip An optional lists of attributes
#'  that you may want to avoid having copied
#' @param attr2force An optional lists of attributes
#'  that you may want to force copy even if they already
#'  exist in the new object
#' @return \code{object} The \code{to} argument object
#'
#' @examples
#' a <- "test"
#' attr(a, 'wow') <- 1000
#' b <- a
#' b <- copyAllNewAttributes(a, b)
#' print(attr(b, 'wow'))
#'
#' @export
copyAllNewAttributes <- function(from, to, attr2skip = c(), attr2force = c()) {
  for (name in names(attributes(from))) {
    # Don't overwrite attributes
    if (!name %in% names(attributes(to)) &
          !name %in% attr2skip |
          name %in% attr2force)
      attr(to, name) <- attr(from, name)
  }
  return(to)
}

library("testthat")
context("copyAllNewAttributes")

test_that("Basic copy", {
  test_from <- 1
  attr(test_from, "t") <- 2

  test_2 <- 2
  test_2 <- copyAllNewAttributes(
    from = test_from,
    to = test_2
  )

  expect_equal(attributes(test_2), attributes(test_from))
})


test_that("A little more advanced copy", {
  test_from <- matrix(1, ncol = 2, nrow = 2)
  attr(test_from, "t") <- 2

  test_2 <- matrix(2, ncol = 3, nrow = 3)
  test_2 <- copyAllNewAttributes(
    from = test_from,
    to = test_2
  )

  expect_equal(attributes(test_2)$t, attributes(test_from)$t)
  expect_false(all(attributes(test_2)$dim == attributes(test_from)$dim))
})

test_that("An advanced copy", {
  test_from <- matrix(1, ncol = 2, nrow = 2)
  attr(test_from, "t") <- 2

  test_2 <- matrix(2, ncol = 3, nrow = 3)
  attr(test_2, "t") <- 1
  test_2 <- copyAllNewAttributes(
    from = test_from,
    to = test_2
  )
  expect_false(attributes(test_2)$t == attributes(test_from)$t)

  test_2 <- copyAllNewAttributes(
    from = test_from,
    to = test_2,
    attr2force = "t"
  )
  expect_true(attributes(test_2)$t == attributes(test_from)$t)
})

#' Create a box with a color split
#'
#' Creates a grob box with text inside it and a color split in the
#' horizontal axes that allow indicating different proportions. The
#' box can also have a title that spanse the two color areas and
#' that has its own background.
#'
#' @inheritParams boxGrob
#' @param label_left The label for the left area
#' @param label_right The label for the right area
#' @param prop The proportion to split along
#' @param txt_gp The \code{\link[grid]{gpar}} style to apply to the text. Set \code{boxPropGrobTxt} option
#'  if you want to customize all the boxes at once.
#' @param txt_left_gp The \code{\link[grid]{gpar}} style to apply to the left text. Set
#' \code{boxPropGrobLeftTxt} option if you want to customize all the boxes at once.
#' @param txt_right_gp The \code{\link[grid]{gpar}} style to apply to the right text. Set
#' \code{boxPropGrobRightTxt} option if you want to customize all the boxes at once.
#' @param box_left_gp The \code{\link[grid]{gpar}} style to apply to the left box. Set
#' \code{boxPropGrobLeft} option if you want to customize all the boxes at once.
#' @param box_right_gp The \code{\link[grid]{gpar}} style to apply to the right box. Set
#' \code{boxPropGrobRight} option if you want to customize all the boxes at once.
#' @param box_highlight_gp The \code{\link[grid]{gpar}} style to apply to the background
#' of the main label. Set \code{boxPropGrobHighlight} option if you want to customize
#' all the boxes at once.
#'
#' @return A box grob
#' @export
#'
#' @importFrom checkmate assert_class assert checkString checkNumeric assert_number
#' @family flowchart components
#' @examples
#' library(grid)
#' grid::grid.newpage()
#' boxPropGrob("Main label", "Left text", "Right text", prop = .3)
boxPropGrob <- function(label,
                        label_left,
                        label_right,
                        prop,
                        y = unit(.5, "npc"),
                        x = unit(.5, "npc"),
                        width,
                        height,
                        just = "center",
                        bjust = "center",
                        txt_gp = getOption("boxPropGrobTxt",
                          default = gpar(color = "black")
                        ),
                        txt_left_gp = getOption("boxPropGrobLeftTxt",
                          default = gpar(col = "black")
                        ),
                        txt_right_gp = getOption("boxPropGrobRightTxt",
                          default = gpar(col = "black")
                        ),
                        box_left_gp = getOption("boxPropGrobLeft",
                          default = gpar(fill = "#E6E8EF")
                        ),
                        box_right_gp = getOption("boxPropGrobRight",
                          default = gpar(fill = "#FFFDF6")
                        ),
                        box_highlight_gp = getOption("boxPropGrobHighlight",
                          default = gpar(fill = "#ffffff55", col = NA)
                        ),
                        name = NULL) {
  assert_label(label)
  assert_label(label_left)
  assert_label(label_right)
  assert_unit(y)
  assert_unit(x)
  assert_unit(width)
  assert_unit(height)
  assert_just(just)
  assert_just(bjust)
  assert_class(txt_gp, "gpar")
  assert_class(txt_left_gp, "gpar")
  assert_class(txt_right_gp, "gpar")
  assert_class(box_left_gp, "gpar")
  assert_class(box_right_gp, "gpar")
  assert_class(box_highlight_gp, "gpar")

  assert_number(prop, lower = 0, upper = 1)

  x <- prAsUnit(x)
  y <- prAsUnit(y)

  txt_padding <- unit(4, "mm")
  spacer <- list(
    x = unit(2, "mm"),
    y = unit(5, "mm")
  )

  label_measures <- list(
    width = 0,
    height = 0
  )

  main_label <- NULL
  if (!missing(label)) {
    tg <- textGrob(
      label = label,
      x = prGetX4Txt(just, txt_padding), y = .5,
      just = just,
      name = "label",
      gp = txt_gp
    )
    label_measures$height <- (prConvertHeightToMm(grobHeight(tg)) + 2)
    label_measures$width <- prConvertWidthToMm(grobWidth(tg))

    main_label <- grobTree(
      name = "main_label",
      gList(roundrectGrob(gp = box_highlight_gp), tg),
      vp = viewport(
        height = unit(label_measures$height, "mm"),
        y = 1,
        just = "top"
      )
    )
  }

  sublabel <- list()
  if (!missing(label_left)) {
    sublabel$left <- prBuildSubLabel(
      label = label_left,
      prop = prop,
      txt_gp = txt_left_gp,
      side = "left"
    )
  }

  if (!missing(label_right)) {
    sublabel$right <- prBuildSubLabel(
      label = label_right,
      prop = prop,
      txt_gp = txt_right_gp,
      side = "right"
    )
  }

  space_between_label_and_sublabel <- unit(0, "mm")
  if (missing(height)) {
    add_height <- 0
    if (length(sublabel) > 0) {
      add_height <- max(sapply(sublabel, function(x) attr(x, "height")))
    }

    if (label_measures$height > 0 && add_height > 0) {
      space_between_label_and_sublabel <- spacer$y
    }

    height <- unit(label_measures$height + add_height, "mm") + space_between_label_and_sublabel + txt_padding + txt_padding
  }

  if (length(sublabel) == 0) {
    sublabel <- NULL
  } else {
    sublabel <- do.call(gList, sublabel)
  }

  txt <- grobTree(
    gList(
      main_label,
      grobTree(
        sublabel,
        name = "sublabel",
        vp = viewport(
          y = 0, just = "bottom",
          height = unit(1, "npc") - unit(label_measures$height, "mm") - space_between_label_and_sublabel
        )
      )
    ),
    vp = viewport(
      height = unit(1, "npc") - txt_padding - txt_padding,
      width = unit(1, "npc") - txt_padding - txt_padding,
      clip = "on"
    ),
    name = name,
    cl = "boxProp"
  )


  # Calculate the width of the grob
  if (missing(width)) {
    total_sublabel_width <- 0
    if (length(sublabel) > 0) {
      total_sublabel_width <- sum(sapply(sublabel, function(x) attr(x, "width"))) +
        prConvertWidthToMm(spacer$x)
    }

    base_width <- max(label_measures$width, total_sublabel_width)

    # Due to the proportions we may need to force a larger window
    get_min_width <- function(sl) {
      if (is.null(sl)) {
        return(0)
      }

      attr(sl, "width") +
        prConvertWidthToMm(txt_padding) +
        prConvertWidthToMm(spacer$x)
    }

    min_left <- get_min_width(sublabel$left)
    if (base_width * prop < min_left) {
      base_width <- min_left / prop
    }

    min_right <- get_min_width(sublabel$right)
    if (base_width * (1 - prop) < min_right) {
      base_width <- min_right / (1 - prop)
    }

    width <- unit(base_width, "mm") + txt_padding + txt_padding
  }


  vp_args <- list(
    x = x,
    y = y,
    width = width,
    height = height,
    just = bjust
  )

  gl <- grobTree(
    roundrectGrob(
      gp = box_left_gp,
      width = width, x = 0, just = "left",
      vp = viewport(x = 0, just = "left", width = prop, clip = "on")
    ),
    roundrectGrob(
      gp = box_right_gp,
      width = width, x = 1, just = "right",
      vp = viewport(x = 1, just = "right", width = 1 - prop, clip = "on")
    ),
    txt,
    vp = do.call(viewport, vp_args),
    cl = "box"
  )

  xtr_coordinate_fns <- list(
    left_x = function(x, width, half_width) x - half_width + unit(prConvertWidthToMm(width) * prop / 2, "mm"),
    right_x = function(x, width, half_width) {
      x - half_width +
        unit(prConvertWidthToMm(width) * prop + prConvertWidthToMm(width) * (1 - prop) / 2, "mm")
    },
    prop_x = function(x, width, half_width) {
      x - half_width + unit(prConvertWidthToMm(width) * prop, "mm")
    }
  )

  structure(gl,
    coords = prCreateBoxCoordinates(viewport_data = vp_args, extra_coordinate_functions = xtr_coordinate_fns),
    extra_coordinate_functions = xtr_coordinate_fns,
    viewport_data = vp_args
  )
}


#' Add a sub-label to \code{boxPropGrob}
#'
#' @param prop The proportion
#' @param label The text of the label
#' @param txt_gp The style as defined by \code{\link[grid]{gpar}()}
#' @param side The side that the label belongs to
#' @return A \code{textGrob} with he additional attributes \code{width} and \code{height}.
prBuildSubLabel <- function(label,
                            prop,
                            txt_gp,
                            side = c("left", "right")) {
  side <- match.arg(side)
  if (side == "left") {
    x <- prop / 2
    width <- prop
  } else {
    x <- prop + (1 - prop) / 2
    width <- 1 - prop
  }

  vp <- viewport(
    x = x, width = width,
    y = 1,
    just = c("centre", "top"),
    name = paste0("vp_label_", side)
  )
  tg <- textGrob(
    label = label,
    x = .5, y = 0.5,
    just = "center",
    vp = vp,
    name = paste0("label_", side),
    gp = txt_gp
  )
  structure(tg,
    height = prConvertHeightToMm(grobHeight(tg) + unit(.5, units = "lines")),
    width = prConvertWidthToMm(grobWidth(tg) + unit(.5, units = "lines"))
  )
}

library(testthat)

test_that("Check that multi-line is higher than single line box", {
  b1 <- boxPropGrob(
    label = "Label",
    label_left = "Left",
    label_right = "Right",
    prop = 0.5
  )
  b2 <- boxPropGrob(
    label = "Label",
    label_left = "Left1\nLeft2",
    label_right = "Right",
    prop = 0.5
  )

  expect_gt(
    grobHeight(b2) |> prConvertHeightToMm(),
    grobHeight(b1) |> prConvertHeightToMm()
  )

  b3 <- boxPropGrob(
    label = "Label1\nlabel2",
    label_left = "Left",
    label_right = "Right",
    prop = 0.5
  )
  expect_gt(
    grobHeight(b3) |> prConvertHeightToMm(),
    grobHeight(b1) |> prConvertHeightToMm()
  )

  b4 <- boxPropGrob(
    label = "Label",
    label_left = "Left",
    label_right = "Right1\nRight2",
    prop = 0.5
  )
  expect_gt(
    grobHeight(b4) |> prConvertHeightToMm(),
    grobHeight(b1) |> prConvertHeightToMm()
  )
})

test_that("Check that not providing label reduces the height of the box", {
  b1 <- boxPropGrob(
    label = "Label",
    label_left = "Left", "Right",
    prop = 0.5
  )
  b2 <- boxPropGrob(
    label_left = "Left",
    label_right = "Right",
    prop = 0.5
  )

  expect_lt(
    grobHeight(b2) |> prConvertHeightToMm(),
    grobHeight(b1) |> prConvertHeightToMm()
  )

  b3 <- boxPropGrob(label = "Label", prop = 0.5)
  expect_lt(
    grobHeight(b3) |> prConvertHeightToMm(),
    grobHeight(b1) |> prConvertHeightToMm()
  )
})

test_that("Label with two lines increases the height", {
  b1 <- boxPropGrob(
    label = "A",
    label_left = "A",
    label_right = "A", prop = .5
  ) |>
    grobHeight() |>
    convertY(unitTo = "mm", valueOnly = TRUE)
  b2 <- boxPropGrob(
    label = "A\nA",
    label_left = "AA",
    label_right = "AA", prop = .5
  ) |>
    grobHeight() |>
    convertY(unitTo = "mm", valueOnly = TRUE)
  expect_gt(b2, b1, label = "Prop box height should be defined from the width of the text")
})

# This file contains all the helper functions that the outer exported
# functions utilize. I try to have a pr at the start of the name for all
# the private functions.
#
# Author: max
###############################################################################

#' Pushes viewport with margins
#'
#' A \code{\link[grid]{grid.layout}} object is used to
#' generate the margins. A second viewport selecting the
#' mid-row/col is used to create the effect of margins
#'
#' @param bottom The margin object, either in npc or a \code{\link[grid]{unit}} object,
#'  or a vector of 4 if you want the same margins allover.
#' @param left The margin object, either in npc or a \code{\link[grid]{unit}} object.
#' @param top The margin object, either in npc or a \code{\link[grid]{unit}} object
#' @param right The margin object, either in npc or a \code{\link[grid]{unit}} object
#' @param name The name of the last viewport
#' @return \code{2} - the number of pushed viewports
#'
#' @keywords internal
prPushMarginViewport <- function(bottom, left, top, right, name = "margin") {
  if (!is.unit(bottom)) {
    bottom <- unit(bottom, "npc")
  }

  if (missing(left) &&
    missing(top) &&
    missing(right)) {
    if (length(bottom) == 4) {
      left <- bottom[2]
      top <- bottom[3]
      right <- bottom[4]
      bottom <- bottom[1]
    } else {
      left <- bottom[1]
      top <- bottom[1]
      right <- bottom[1]
    }
  }

  if (!is.unit(top)) {
    top <- unit(top, "npc")
  }

  if (!is.unit(left)) {
    left <- unit(left, "npc")
  }

  if (!is.unit(right)) {
    right <- unit(right, "npc")
  }

  layout_name <- sprintf("margin_grid_%s", name)

  gl <- grid.layout(
    nrow = 3, ncol = 3,
    heights = unit.c(top, unit(1, "npc") - top - bottom, bottom),
    widths = unit.c(left, unit(1, "npc") - left - right, right)
  )

  pushViewport(viewport(layout = gl, name = layout_name))
  pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 2, name = name))
  return(2)
}

#' Adds a title to the plot
#'
#' Adds the title and generates a new
#' main viewport below the title
#'
#' @param title The title as accepted by \code{\link[grid:grid.text]{textGrob}}
#' @param base_cex The base cex used for the plot
#' @param cex_mult The multiplier of the base - i.e. the increase of the
#'  text size for the title as compared to the general
#' @param fontface The type of fontface
#' @param space_below The space below, defaults to 1/5 of the title height
#' @return \code{NULL} The function does not return a value
#'
#' @keywords internal
prGridPlotTitle <- function(title,
                            base_cex,
                            cex_mult = 1.2,
                            fontface = "bold",
                            space_below = NULL) {
  titleGrob <- textGrob(title,
    just = "center",
    gp = gpar(
      fontface = fontface,
      cex = base_cex * cex_mult
    )
  )

  # The y/g/j letters are not included in the height
  gh <- unit(convertUnit(grobHeight(titleGrob), "mm", valueOnly = TRUE) * 1.5, "mm")
  if (is.null(space_below)) {
    space_below <- unit(convertUnit(gh, "mm", valueOnly = TRUE) / 3, "mm")
  } else if (!is.unit(space_below)) {
    space_below <- unit(space_below, "npc")
  }

  gl <- grid.layout(
    nrow = 3, ncol = 1,
    heights = unit.c(gh, space_below, unit(1, "npc") - space_below - gh)
  )

  pushViewport(viewport(layout = gl, name = "title_layout"))
  pushViewport(viewport(layout.pos.row = 1, name = "title"))
  grid.draw(titleGrob)
  upViewport()

  pushViewport(viewport(layout.pos.row = 3, name = "main"))
}

#' Just a simple acces to the gp$cex parameter
#'
#' @param x The text-grob of interest
#' @return \code{numeric} The cex value, 1 if no cex was present
#' @keywords internal
prGetTextGrobCex <- function(x) {
  cex <- 1
  if (!is.null(x$gp$cex)) {
    cex <- x$gp$cex
  }

  return(cex)
}

#' Collapses a vector for throwing errors
#'
#' The function collapses a vector into an output useful when throwing
#' errors, e.g. 1:3 becomes '1', '2', '3'
#'
#' @param x The vector
prPasteVec <- function(x) {
  sprintf("'%s'", paste(x, collapse = "', '"))
}

#' @title Coerce value to a unit
#' @description Ensure a value is returned as a grid `unit`. If `val` is
#'   already a `unit` it is returned unchanged; otherwise it's wrapped as
#'   `unit(..., "npc")`.
#' @param val A numeric or `unit` object.
#' @return A `unit` object.
#' @inheritParams grid::unit
#' @keywords internal
#' @noRd
prAsUnit <- function(val, units = "npc") {
  if (is.null(val)) {
    stop("Cannot convert NULL to a unit", call. = FALSE)
  }

  if (is.unit(val)) {
    return(val)
  }

  return(unit(val, units = units))
}

#' @title Convert height to millimetres
#' @description Convert a grid height `unit` (or numeric) to millimetres.
#' @param val A `unit` or numeric value representing a height.
#' @return Numeric value in millimetres.
#' @keywords internal
#' @noRd
prConvertHeightToMm <- function(val) {
  convertHeight(val, unitTo = "mm", valueOnly = TRUE)
}

#' @title Convert width to millimetres
#' @description Convert a grid width `unit` (or numeric) to millimetres.
#' @param val A `unit` or numeric value representing a width.
#' @return Numeric value in millimetres.
#' @keywords internal
#' @noRd
prConvertWidthToMm <- function(val) {
  convertWidth(val, unitTo = "mm", valueOnly = TRUE)
}

#' @title Convert millimetres to NPC
#' @description Convert a numeric in millimetres to a `unit` in `npc`.
#' @param mm_val Numeric millimetres value.
#' @param axis Character 'x' or 'y' to choose convertX/convertY.
#' @return A `unit` in `npc`.
#' @keywords internal
#' @noRd
prConvertMmToNpc <- function(mm_val, axis = c("x", "y")) {
  axis <- match.arg(axis)
  if (!is.numeric(mm_val) || length(mm_val) != 1 || is.na(mm_val)) {
    return(unit(NA_real_, "npc"))
  }
  conv <- if (axis == "x") convertX else convertY
  v <- tryCatch(conv(unit(mm_val, "mm"), "npc", valueOnly = TRUE), error = function(e) NA_real_)
  unit(v, "npc")
}

#' @title Extract numeric NPC position
#' @description Obtain a numeric NPC position from a `unit` or numeric-like
#'   input. Attempts `convertX/convertY` first and falls back to numeric
#'   coercion when conversion fails. Returns `NA_real_` on failure.
#' @param u A `unit` or numeric-like value.
#' @param axis Character, either "x" or "y" to choose `convertX` or
#'   `convertY` respectively.
#' @return Numeric in NPC units or `NA_real_`.
#' @keywords internal
#' @noRd
prGetNpcValue <- function(u, axis = c("x", "y")) {
  axis <- match.arg(axis)
  conv <- if (axis == "x") convertX else convertY
  v <- tryCatch(conv(u, "npc", valueOnly = TRUE), error = function(e) NA_real_)
  if (!is.na(v)) {
    return(v)
  }

  v2 <- suppressWarnings(as.numeric(u))
  if (!is.na(v2) && length(v2) == 1 && v2 >= -1 && v2 <= 2) {
    return(v2)
  }
  NA_real_
}

#' @title Obtain size in NPC units
#' @description Convert a width/height `unit` or numeric-like value to a
#'   numeric expressed in NPC. Uses `convertWidth/convertHeight` and falls
#'   back to numeric coercion if needed.
#' @param u A `unit` or numeric-like value representing a size.
#' @param axis Character, either "x" (width) or "y" (height).
#' @return Numeric size in NPC or `NA_real_`.
#' @keywords internal
#' @noRd
prGetNpcSize <- function(u, axis = c("x", "y")) {
  axis <- match.arg(axis)
  conv <- if (axis == "x") convertWidth else convertHeight
  v <- tryCatch(conv(u, "npc", valueOnly = TRUE), error = function(e) NA_real_)
  if (!is.na(v)) {
    return(v)
  }
  v2 <- suppressWarnings(as.numeric(u))
  if (!is.na(v2) && length(v2) == 1 && v2 >= -1 && v2 <= 2) {
    return(v2)
  }
  NA_real_
}

library(testthat)

test_that("prGetNpcValue returns npc numeric for unit and numeric and NA for invalid", {
  # simple npc unit
  expect_equal(prGetNpcValue(unit(0.3, "npc"), "x"), 0.3)

  # numeric value should be accepted
  expect_equal(prGetNpcValue(0.25, "x"), 0.25)

  # string that cannot be coerced -> NA
  expect_true(is.na(prGetNpcValue("not-a-number", "x")))

  # y axis should use convertY (basic check with unit)
  expect_equal(prGetNpcValue(unit(0.4, "npc"), "y"), 0.4)
})

library(testthat)

test_that("prLineMidpoint returns npc units and midpoint for a simple straight segment", {
  line <- list(
    x = unit.c(unit(0.1, "npc"), unit(0.9, "npc")),
    y = unit.c(unit(0.5, "npc"), unit(0.5, "npc"))
  )

  mp <- prLineMidpoint(line)
  expect_s3_class(mp$x, "unit")
  expect_s3_class(mp$y, "unit")

  expect_equal(prGetNpcValue(mp$x, "x"), 0.5, tolerance = 1e-6)
  expect_equal(prGetNpcValue(mp$y, "y"), 0.5, tolerance = 1e-6)
})

library(testthat)

test_that("prGetNpcSize returns npc numeric for unit and numeric and NA for invalid", {
  # simple npc unit for width
  expect_equal(prGetNpcSize(unit(0.3, "npc"), "x"), 0.3)

  # numeric value should be accepted (interpreted as npc)
  expect_equal(prGetNpcSize(0.25, "x"), 0.25)

  # string that cannot be coerced -> NA
  expect_true(is.na(prGetNpcSize("not-a-number", "x")))

  # y axis should use convertHeight (basic check with unit)
  expect_equal(prGetNpcSize(unit(0.4, "npc"), "y"), 0.4)
})

#' A path join function
#'
#' This function joins strings into a valid path. It is a simple version of python's
#' \code{os.path.join} and fixes simple problems such as having/not having trailing /
#' in each section.
#'
#' @param ... A set of strings to join. Each may be a single string or a vector.
#'  If you provide vectors they can either be all of the same length
#'  or where there are two lengths where one is equal to 1.
#' @return \code{string} A string with the merged path
#'
#' @examples
#' pathJoin("my_base_path/helpers", "superfunction.R")
#' # 'my_base_path/helpers/superfunction.R'
#'
#' base_dir <- "/home/tester/images"
#' out <- data.frame(filename = c("file1.png", "file2.png", "file3.png")) |>
#'   dplyr::mutate(full_path = pathJoin(base_dir, filename))
#'
#' @importFrom stringr str_replace
#' @export
pathJoin <- function(...) {
  paths <- list(...)
  if (length(paths) == 1) {
    paths <- as.list(paths[[1]])
  }

  lengths <- sapply(paths, length, USE.NAMES = FALSE)
  if (length(unique(lengths)) > 1) {
    if (length(unique(lengths)) > 2) {
      stop("You can only have two lengths for joining paths")
    }
    if (!(1 %in% lengths)) {
      stop("If using different lengths at least one of the vectors must have the length of 1")
    }
  }

  clean_paths <- paths %>%
    lapply(function(x) {
      if (is.character(x)) {
        return(dropTrailingSlash(x))
      }
      lapply(x, dropTrailingSlash)
    })

  if (!all(lengths == lengths[1])) {
    ml <- max(lengths)
    for (i in 1:length(clean_paths)) {
      clean_paths[[i]] <- rep(clean_paths[[i]], length.out = ml)
    }
  }

  clean_paths |>
    data.frame() |>
    apply(MARGIN = 1, FUN = \(x) do.call(file.path, as.list(x))) |>
    str_replace("/[/]+", .Platform$file.sep)
}


dropTrailingSlash <- function(x) {
  str_replace(x, "[\\/]$", "")
}

library('testthat')
context('pathJoin')

test_that("pathJoin basics", {
  expect_equal(pathJoin("one", "two/three", "four"),
               file.path("one", "two/three", "four"))
  expect_equal(pathJoin("one", "two//three/", "four"),
               file.path("one", "two/three", "four"))
})

test_that("join single param", {
  expect_equal(pathJoin(LETTERS[1:3]),
               do.call(file.path, as.list(LETTERS[1:3])))
})

test_that("pathJoin with vector", {
  expect_equal(pathJoin("base/", c("//sub1", "sub2")),
               c("base/sub1", "base/sub2"))

  expect_equal(pathJoin(c("base1/", "base2"), c("//sub1", "sub2")),
               c("base1/sub1", "base2/sub2"))

  expect_equal(pathJoin(c("base1/", "base2"), c("//sub1", "sub2"), c("final")),
               c("base1/sub1/final", "base2/sub2/final"))

  expect_error(pathJoin(LETTERS[1:2], LETTERS[1:3]))
  expect_error(pathJoin(LETTERS[1:2], LETTERS[1:3], LETTERS[1]))
})

test_that("Works on a data.frame", {
  base_dir <- "/home/tester/images"
  out <- data.frame(filename = c("file1.png", "file2.png", "file3.png")) |>
    dplyr::mutate(full_path = pathJoin(base_dir, filename))
  expect_equal(out$full_path,
               c("file1.png", "file2.png", "file3.png") |> sapply(\(x) file.path(base_dir, x),
                                                                  USE.NAMES = FALSE))
})

library(grid)
grid.newpage()

# Initiate the boxes that we want to connect
boxes <- list(
  start = boxGrob("Top", x = .5, y = .8),
  end = boxGrob("Bottom", x = .5, y = .2),
  side = boxPropGrob("Side", "Left", "Right", prop = .3, x = .2, y = .8),
  exclude = boxGrob("Exclude:\n - Too sick\n - Prev. surgery", x = .8, y = .5, just = "left")
)

# Connect the boxes and print/plot them
connectGrob(boxes$start, boxes$end, "vertical")
connectGrob(boxes$start, boxes$side, "horizontal")
connectGrob(boxes$start, boxes$exclude, "L")

# We can also connect to/from lists
side_boxes <- list(
  left = boxGrob("Left", x = attr(boxes$side, "coords")$left_x, y = .5),
  right = boxGrob("Right", x = attr(boxes$side, "coords")$right_x, y = .5)
)

connectGrob(boxes$side, side_boxes$left, "v", "l")
connectGrob(boxes$side, side_boxes$right, "v", "r")

# Fan-in center example: multiple starts into one center bus and single trunk
list(
  boxes$start,
  boxGrob("S2", x = .3, y = .7),
  boxGrob("S3", x = .7, y = .7)
) |>
  connectGrob(boxes$end, type = "fan_in_center")

# Print the boxes
boxes
side_boxes

#' Plots a column of boxes
#'
#' Takes a set of box settings and plots them.
#'
#' @param box_positions The box positions
#' @param proportions The box vertical proportions (needed for the prop attribute)
#' @param fill The fill colors of length equal to the proportions
#' @param txt The texts of length equal to the proportions
#' @param txt_clr The text colors of length equal to the proportions
#' @param txt_gpar A list of gpar object for the text box
#' @param cex The fontsize multiplier
#'
#' @return \code{void}
#' @keywords internal
prTcPlotBoxColumn <- function(box_positions,
                              proportions,
                              fill,
                              txt,
                              txt_clr,
                              txt_gpar,
                              cex) {
  for (i in 1:length(box_positions)) {
    if (box_positions[[i]]$height == 0) {
      next
    }

    args <- list(
      bx = box_positions[[i]],
      bx_txt = txt[i],
      cex = cex,
      txt_gpar = txt_gpar[[i]],
      line_col = "black",
      lwd = 1
    )
    if (!is.null(attr(proportions, "prop"))) {
      args[["prop"]] <- attr(proportions, "prop")[i]
      args[["fill"]] <- fill[i, ]
      args[["txt_clr"]] <- txt_clr[i, ]
    } else {
      args[["fill"]] <- fill[i]
      args[["txt_clr"]] <- txt_clr[i]
    }

    do.call(prTpPlotBox, args)
  }
}

#' Plots the arrows within a single transition set
#'
#' @param trnstn_set The set of transitions to plot
#' @param left_box_clrs The colors of the left side boxes
#' @param type The type of arrow to be used
#' @param widths The arrow widths
#' @param clr The arrow colors (a matrix)
#' @param rez The arrow resolution
#' @param min_width The minimum line width
#' @param max_width The maximum line width
#' @param add_width Adds a certain width - useful for background arrows
#' @param origin_boxes The origin boxes' positions to the left
#' @param target_boxes The target boxes' positions to the right
#' @param max_flow The maximum flow if it is common for the entire image
#' @param abs_arrow_width If the arrow width should be the same for all arrows
#' @inheritParams prTcPlotBoxColumn
#' @keywords internal
prTcPlotArrows <- function(trnstn_set,
                           widths,
                           type = c("simple", "gradient"),
                           clr,
                           rez,
                           origin_boxes, target_boxes,
                           left_box_clrs, add_width,
                           max_flow, min_width, max_width,
                           abs_arrow_width = FALSE,
                           clr_bar_subspace) {
  if (length(dim(trnstn_set)) == 3) {
    transition_arrow_props <- trnstn_set[, , 1] / (trnstn_set[, , 1] + trnstn_set[, , 2])
    trnstn_set <- (trnstn_set[, , 1] + trnstn_set[, , 2])
  }
  for (org_row in 1:nrow(trnstn_set)) {
    origin <- origin_boxes[[org_row]]

    # Check what arrows that should be skipped
    no_arrows <- sapply(widths[[org_row]], function(x) is.null(x$lwd), USE.NAMES = FALSE)

    # Plot the widest arrow last
    for (target_row in order(trnstn_set[org_row, ])) {
      if (no_arrows[target_row]) {
        next
      }

      # Start with background arrow
      for (add_width in list(unit(2, "pt"), NA)) {
        no_trgt <- sapply(widths, function(x) is.null(x[[target_row]]$lwd), USE.NAMES = FALSE)

        target <- target_boxes[[target_row]]
        entry_length <- target$height / 3
        arrow_length <- target$width / 4

        adjusted_lwd <- widths[[org_row]][[target_row]]$adj_lwd
        x_ctrl_points <- c(
          origin$right,
          c(origin$right, target$left) %>%
            mean() %>%
            rep(times = 2),
          target$left
        )
        arrow_dodge <- seq(
          from = target$height / 6,
          to = -target$height / 6,
          length.out = sum(!no_trgt)
        )
        exit_pos <- org_row - sum(no_trgt[0:org_row])
        y_ctrl_points <- c(
          rep(origin$y, 2),
          rep(target$y + arrow_dodge[exit_pos], 2)
        )
        adjusted_lwd <- convertY(adjusted_lwd, unitTo = target$unit, valueOnly = TRUE)
        # The width can be wider using the special bezier arrows
        if (abs_arrow_width) {
          a_width <- entry_length * 1.5 / sum(!no_trgt)
        } else {
          a_width <-
            adjusted_lwd * 2
        }

        # Add line width addition if it is a background line
        if (!is.na(add_width)) {
          if (is.unit(add_width)) {
            a_width <- a_width + prGetNpcValue(add_width, "y")
            adjusted_lwd <- adjusted_lwd + prGetNpcValue(add_width, "y")
          } else {
            a_width <- a_width * add_width
            adjusted_lwd <- adjusted_lwd * add_width
          }
          arrow_clr <- "#FFFFFF"
        } else {
          arrow_clr <- clr[org_row, target_row]
        }


        if (a_width < adjusted_lwd) {
          sp_float_string <- sprintf("%%.%df", -floor(log10(adjusted_lwd - a_width)) + 1)
          warning(
            "The arrow width is smaller than the width of the line,",
            "thus not appearing as a regular arrow: ",
            sprintf(sp_float_string, a_width),
            " < ",
            sprintf(sp_float_string, adjusted_lwd)
          )

          # Looks really weird if this is allowed
          a_width <- adjusted_lwd
        }

        adjusted_lwd <- convertY(unit(adjusted_lwd, target$unit), unitTo = "mm")
        arrow_length <- convertY(unit(arrow_length, target$unit), unitTo = "mm")
        a_width <- convertY(unit(a_width, target$unit), unitTo = "mm")
        if (type == "simple" || !is.na(add_width)) {
          bezierArrowSmpl(
            x = x_ctrl_points,
            y = y_ctrl_points,
            width = adjusted_lwd,
            arrow = list(length = arrow_length, base = a_width),
            clr = arrow_clr,
            rez = rez
          ) %>%
            grid.draw()
        } else {
          if (NCOL(left_box_clrs) == 2) {
            current_grdt_clr <- prTpGetColors(
              colors = left_box_clrs[org_row, ],
              proportion = 1 - transition_arrow_props[org_row, target_row],
              space = clr_bar_subspace
            )
          } else {
            if (is.matrix(left_box_clrs)) {
              current_grdt_clr <- left_box_clrs[org_row, 1]
            } else {
              current_grdt_clr <- left_box_clrs[org_row]
            }
          }

          bezierArrowGradient(
            x = x_ctrl_points,
            y = y_ctrl_points,
            width = adjusted_lwd,
            arrow = list(length = arrow_length, base = a_width),
            clr = arrow_clr,
            rez = rez,
            grdt_type = "triangle",
            grdt_clr_prop = 0.5,
            grdt_prop = .8,
            grdt_decrease_prop = .5,
            grdt_clr = current_grdt_clr
          ) %>%
            grid.draw()
        }
      }
    }
  }
}


#' Checks and prepares colors
#'
#' The colors need to match the size of the transition matrix
#' and therefore we need to have this input check. The output is
#' always a matrix of the exact same dimensions as the transition
#' matrix.
#'
#' @param value The color values
#' @inheritParams prTcMatchClr
#' @return \code{matrix} Returns a matrix of the same size as the
#'  transition matrix
#' @keywords internal
prTcValidateAndPrepClr <- function(value, no_cols, no_rows, is3D) {
  if (!is.list(value)) {
    stop("Invalid value. Expecting a list with a vector/matrix matching the no rows in each column")
  }

  if (length(value) != no_cols) {
    stop("Invalid color length, got ", length(value), " expected ", no_cols)
  }

  for (i in 1:length(value)) {
    if (NROW(value[[i]]) != no_rows[i]) {
      if (NROW(value[[i]]) == 1) {
        if (is.matrix(value[[i]]) &&
          NCOL(value[[i]]) <= 2 &&
          is3D) {
          value[[i]] <-
            matrix(value[[i]], nrow = no_rows[i], ncol = 2, byrow = TRUE)
        } else if (length(value[[i]]) == 1) {
          value[[i]] <-
            rep(value[[i]], no_rows[i])
        } else {
          stop(
            "The number of colors don't match the transition object for column no. ", i,
            ": ", length(value[[i]]), "  != ", no_rows[i]
          )
        }
      } else {
        stop(
          "The number of colors don't match the transition object for column no. ", i,
          ": ", length(value[[i]]), "  != ", no_rows[i]
        )
      }
    }
  }

  return(value)
}

#' Finds the matching colors for the new addition based on the original set
#' of colors
#'
#' @param add The colors that we want to add
#' @param org The original colors
#' @param no_cols The no. of columns in the transition matrix
#' @param no_rows All the row counts in the transition plot
#' @param is3D Whether the transition matrix has 3 dimensions
#' @return A list with colors that is appropriate for the transition object
#' @keywords internal
prTcMatchClr <- function(add, org, no_cols, no_rows, is3D) {
  if (no_cols == 2) {
    if (missing(add) && length(org) == 2) {
      return(org)
    }

    if (is.list(add)) {
      if (inherits(add, "gpar")) {
        add <- rep(list(add), 2)
      } else if (length(add) == 1) {
        add <- rep(add, 2)
      } else if (length(add) != 2) {
        stop(
          "Invalid color list - expecting a list with 2 columns",
          " got one with ", length(add), " columns"
        )
      }
    } else if (is.matrix(add)) {
      if (ncol(add) == 2) {
        if (is3D) {
          add <- rep(list(add), 2)
        } else {
          add <- list(
            add[, 1],
            add[, 2]
          )
        }
      } else if (ncol(add) == 4 && is3D) {
        add <- list(
          add[, 1:2],
          add[, 3:4]
        )
      } else if (ncol(add) == 1) {
        if (length(add) == 2) {
          if (is3D) {
            add <-
              list(
                cbind(
                  rep(add[1, ], no_rows[1]),
                  rep(add[2, ], no_rows[1])
                ),
                cbind(
                  rep(add[1, ], no_rows[2]),
                  rep(add[2, ], no_rows[2])
                )
              )
          } else {
            add <-
              list(
                rep(add[1, ], no_rows[1]),
                rep(add[2, ], no_rows[2])
              )
          }
        } else if (length(add) == 1) {
          if (is3D) {
            add <-
              list(
                cbind(matrix(add[1, ],
                  nrow = no_rows[1],
                  ncol = 2
                )),
                cbind(matrix(add[1, ],
                  nrow = no_rows[1],
                  ncol = 2
                ))
              )
          } else {
            add <-
              list(
                rep(add[1, ], no_rows[1]),
                rep(add[1, ], no_rows[2])
              )
          }
        } else {
          stop("Invalid color argument")
        }
      }
    } else if (is.vector(add)) {
      if (length(add) == 2 && is3D) {
        add <- lapply(1:2, function(col) {
          cbind(
            rep(add[1], no_rows[col]),
            rep(add[2], no_rows[col])
          )
        })
      } else if (length(add) == 1) {
        if (is3D) {
          add <-
            list(
              matrix(add,
                nrow = no_rows[1],
                ncol = 2
              ),
              matrix(add,
                nrow = no_rows[2],
                ncol = 2
              )
            )
        } else {
          add <-
            list(
              rep(add, no_rows[1]),
              rep(add, no_rows[2])
            )
        }
      } else {
        add <- list(add)
      }
    } else {
      stop("Invalid color argument")
    }

    # If this is the initial value then simply use the default colors
    return(add)
  }

  # We need to handle the situation where a color is
  # added and the colors are already of the same dimension
  # as the transition matrix. This means that either we
  # switch entire matrix if the new colors have the correct
  # dimenension but otherwise we stick with the old colors
  if (all(length(org) == no_cols)) {
    if (missing(add)) {
      return(org)
    }
    # If the add is equal in dimentions to the target
    # dimentions then the reasonable way to go is to substitute
    # all colors
    if (is.matrix(add)) {
      if (ncol(add) != no_cols ||
        any(nrow(add) != unique(no_rows)) ||
        ((length(dim(add)) == 3) == is3D)) {
        stop("Can't add the color matrix as suggested since the dimensions don't match")
      }

      clrs <- list()
      for (i in 1:ncol(add)) {
        if (is3D) {
          clrs <- c(clrs, list(add[, i, ]))
        } else {
          clrs <- c(clrs, list(add[, i]))
        }
      }
      return(clrs)
    } else if (inherits(add, "gpar")) {
      stop("Adding gpar after-a-fact is not supported")
    } else if (is.list(add) &&
      length(add) != length(no_cols)) {
      stop("The list is not of the same length as the number of columns")
    }

    return(org)
  }

  if (missing(add)) {
    if (is3D) {
      # Doesn't work with magrittr as expected
      tmp <- tail(org, 1)[[1]]
      add <- cbind(
        rep(tmp[, 1], length.out = tail(no_rows, 1)),
        rep(tmp[, 2], length.out = tail(no_rows, 1))
      )
      rm(tmp)
    } else {
      add <- tail(org, 1)[[1]] %>%
        rep(length.out = tail(no_rows, 1))
    }
    add <- list(add)
  } else {
    if (inherits(add, "gpar")) {
      add <- list(rep(list(add), tail(no_rows, 1)))
    } else if (is.list(add)) {
      if (length(add) != 1) {
        stop(
          "Invalid color list - expecting a list with 1 column",
          " got one with ", length(add), " columns"
        )
      }
    } else if (is.matrix(add)) {
      if (ncol(add) == 2 && is3D) {
        add <- list(add)
      } else if (ncol(add) == 1 && !is3D) {
        add <- list(add[, 1])
      } else if (length(add) == 1) {
        if (is3D) {
          add <- list(cbind(matrix(add[1, ],
            nrow = tail(no_rows, 1),
            ncol = 2
          )))
        } else {
          add <- list(rep(add[1, ], tail(no_rows, 1)))
        }
      } else {
        stop("Invalid matrix color argument")
      }
    } else if (is.vector(add)) {
      if (length(add) == 2 && is3D) {
        add <- list(cbind(
          rep(add[1], tail(no_rows, 1)),
          rep(add[2], tail(no_rows, 1))
        ))
      } else if (length(add) == 1) {
        if (is3D) {
          add <- list(matrix(add,
            nrow = tail(no_rows, 1),
            ncol = 2
          ))
        } else {
          add <- list(rep(add, tail(no_rows, 1)))
        }
      } else {
        add <- list(add)
      }
    } else {
      stop("Invalid color argument")
    }
  }

  c(org, add)
}

#' Gets the color for the given box
#'
#' @param clr The color of the boxes, either
#'  a matrix, a vector of length two, or a single color
#' @param no_boxes The number of boxes
#' @param lengthOneOK If it is ok to have only one color
#' @return \code{matrix} With two columns and no_boxes rows.
#'  If function fails then return \code{NULL}
#'
#' @keywords internal
prTpGetBoxPropClr <- function(clr, no_boxes, lengthOneOK = FALSE) {
  if (is.matrix(clr)) {
    if (nrow(clr) == no_boxes &&
      ncol(clr) == 2) {
      return(clr)
    }
  } else if (length(clr) == 2 ||
    (lengthOneOK && length(clr) == 1)) {
    return(matrix(clr, ncol = 2, nrow = no_boxes, byrow = TRUE))
  }

  return(NULL)
}

#' Plots the box for the transition plot
#'
#' @param bx A list with the x, y, height and width parameters
#' @param bx_txt The box text
#' @param fill The fill color
#' @param txt_clr The text color
#' @param cex The font size
#' @param line_col The line color around the box
#' @param lwd The line width
#' @param prop Provide a proportion if the box should be split (0-1)
#' @return \code{void}
#'
#' @inheritParams prTpGetBoxSizedTextGrob
#' @keywords internal
prTpPlotBox <- function(bx, bx_txt, fill, txt_clr,
                        txt_gpar = NULL,
                        cex, line_col, lwd,
                        prop = NA) {
  pushViewport(viewport(
    y = bx$y, x = bx$x,
    height = bx$height, width = bx$width
  ))
  if (is.na(prop)) {
    grid.roundrect(gp = gpar(
      lwd = lwd,
      fill = fill,
      col = line_col
    ))

    if (bx_txt != "") {
      bx_grob <- prTpGetBoxSizedTextGrob(
        txt = bx_txt,
        txt_clr = txt_clr,
        txt_cex = cex,
        txt_gpar = txt_gpar
      )
      if (!is.null(bx_grob)) {
        grid.draw(bx_grob)
      }
    }
  } else {
    # Adapted from Paul Murray's example http://www.stat.auckland.ac.nz/~paul/RG2e/customgrid-nestedlay.R
    pushViewport(viewport(layout = grid.layout(
      nrow = 2,
      ncol = 1,
      heights = c(prop, 1 - prop)
    )))
    grid.roundrect(gp = gpar(
      lwd = lwd,
      fill = fill[1],
      col = NA
    ))

    bx_grob <- NULL
    if (bx_txt != "") {
      bx_grob <- prTpGetBoxSizedTextGrob(
        txt = bx_txt,
        txt_clr = txt_clr[1],
        txt_cex = cex,
        txt_gpar = txt_gpar
      )
      if (!is.null(bx_grob)) {
        grid.draw(bx_grob)
      }
    }

    pushViewport(viewport(layout.pos.row = 2, clip = "on"))
    if ((1 - prop) > 0) {
      grid.roundrect(y = .5 / (1 - prop), height = 1 / (1 - prop), gp = gpar(lwd = lwd, fill = fill[2], col = NA))
      if (bx_txt != "" && !is.null(bx_grob)) {
        # Should not autoadjust the cex but keep the previous one
        prev_cex <- attr(bx_grob, "adjusted_cex")
        bx_grob <- prTpGetBoxSizedTextGrob(
          txt = bx_txt,
          txt_clr = txt_clr[2],
          txt_cex = prev_cex,
          txt_gpar = txt_gpar,
          force_cex = TRUE,
          y = 0.5 / (1 - prop)
        )
        if (!is.null(bx_grob)) {
          grid.draw(bx_grob)
        }
      }
    }
    popViewport(2)
    grid.roundrect(gp = gpar(lwd = lwd, fill = NA, col = line_col))
  }
  popViewport()
}

#' Gets the text size for the box
#'
#' @param txt The text
#' @param txt_clr The color of the text
#' @param txt_cex The font size
#' @param txt_gpar The [grid::gpar()] object that is overridden with col & cex by other parameters
#' @param force_cex If font size should be forced
#' @param ... Other options
#'
#' @importFrom grid gpar
#' @keywords internal
prTpGetBoxSizedTextGrob <- function(txt,
                                    txt_clr,
                                    txt_cex,
                                    txt_gpar = NULL,
                                    force_cex = FALSE,
                                    ...) {
  if (is.null(txt_gpar)) {
    txt_gpar <- list()
  } else {
    stopifnot(is.list(txt_gpar))
  }
  txt_gpar$col <- txt_clr
  txt_gpar$cex <- txt_cex
  bx_grob <- textGrob(txt, gp = do.call(gpar, txt_gpar), ...)
  attr(bx_grob, "adjusted_cex") <- txt_cex
  if (force_cex) {
    return(bx_grob)
  }

  bx_height <- convertY(grobHeight(bx_grob), "npc", valueOnly = TRUE)
  # The box height is by definition 1 npc
  # We want to avoid anything that is bigger that
  # 95 % and that is includingt he yjp (the 2 - 1.5 is a little too small)
  if (.95 < bx_height * 2) {
    new_cex <- txt_cex * .95 / (bx_height * 2)
    # Don't go below 1/4 of the original text size
    if (new_cex < txt_cex * .25) {
      return(NULL)
    } else {
      txt_gpar$cex <- new_cex
      bx_grob <- textGrob(txt,
        gp = do.call(gpar, txt_gpar),
        ...
      )
    }
    attr(bx_grob, "adjusted_cex") <- new_cex
  }

  return(bx_grob)
}


#' Plots the arrows
#'
#' Outputs all the arrows from a box row
#'
#' @param type The type of arrow used
#' @param box_row Which box do the arrows originate from
#' @param max_flow The largest transition flow
#' @param min_lwd The minimum line width
#' @param max_lwd The maximum line width
#' @param clr The color of the line
#' @param box_clr The color of the box
#' @param transition_arrow_props The proportions of the different transitions if
#'  available.
#' @param prop_start_sizes The proportions to the left
#' @param prop_end_sizes The proportions to the right
#' @param tot_spacing Total spacing between boxes
#' @param box_width The box width
#' @param add_width Add a certain width
#' @param color_bar_subspace If there is little or no difference exists
#'  at the low/high proportions of the spectrum then it
#'  can be of interest to focus the color change to the center
#'  leaving the tails constant
#' @return \code{void}
#'
#' @keywords internal
prTpPlotArrows <- function(type,
                           box_row,
                           transition_flow,
                           max_flow,
                           min_lwd,
                           max_lwd,
                           clr,
                           box_clr,
                           transition_arrow_props,
                           prop_start_sizes,
                           prop_end_sizes,
                           tot_spacing,
                           box_width,
                           abs_arrow_width,
                           color_bar_subspace,
                           add_width = NA) {
  no_boxes <- nrow(transition_flow)
  bx_left <- prTpGetBoxPositions(
    no = box_row, side = "left",
    transitions = transition_flow[box_row, ],
    prop_start_sizes = prop_start_sizes,
    prop_end_sizes = prop_end_sizes,
    tot_spacing = tot_spacing,
    box_width = box_width
  )
  # Plot the widest arrow last
  for (flow in order(transition_flow[box_row, ])) {
    if (transition_flow[box_row, flow] > 0) {
      bx_right <- prTpGetBoxPositions(
        no = flow, side = "right",
        transitions = transition_flow[, box_row],
        prop_start_sizes = prop_start_sizes,
        prop_end_sizes = prop_end_sizes,
        tot_spacing = tot_spacing,
        box_width = box_width
      )

      # Calculate line width
      lwd <- max_lwd * transition_flow[box_row, flow] / max_flow
      if (lwd < min_lwd) {
        message(
          "The minimum width reached and the arrow at box no. '", box_row, "' to no. '", flow, "'",
          " will not be shown. This is due to the fact that the lwd will generate a falsely strong arrow."
        )
        next
      }

      adjusted_lwd <- lwd
      if (is.na(add_width) == FALSE) {
        if ("unit" %in% class(add_width)) {
          adjusted_lwd <- convertUnit(unit(lwd, "npc") + add_width, unitTo = "npc", valueOnly = TRUE)
        } else if (add_width > 1) {
          adjusted_lwd <- lwd * add_width
        } else {
          # Quit if the width isn't bigger as it won't show
          return()
        }
      }
      a_l <- (box_width / 4)
      x_ctrl_points <- c(bx_left$right, .5, .5, bx_right$left)
      y_ctrl_points <- c(
        bx_left$y_exit[flow], bx_left$y_exit[flow],
        bx_right$y_entry[box_row], bx_right$y_entry[box_row]
      )
      current_arrow_clr <- clr[(flow + (box_row - 1) * no_boxes)]
      if (type == "grid") {
        if (abs_arrow_width) {
          a_width <- bx_right$y_entry_height / no_boxes
        } else {
          # Not really sure but points seem to be a reasonable
          # unit for the lwd as a basis for this part
          a_width <- getGridVal(unit(lwd, "pt"), "npc") +
            bx_right$y_entry_height / (no_boxes + 1)
        }

        # Add line width addition if it is a background line
        if (!is.na(add_width)) {
          if (is.unit(add_width)) {
            a_width <- a_width + prGetNpcValue(add_width, "y")
          } else {
            a_width <- a_width * add_width
          }
        }

        a_angle <- atan(a_width / 2 / a_l) * 180 / pi
        # Need to adjust the end of the arrow as it otherwise overwrites part of the box
        # if it is thick
        x_ctrl_points[4] <- x_ctrl_points[4] - .00075 * adjusted_lwd
        grid.bezier(
          x = x_ctrl_points,
          y = y_ctrl_points,
          gp = gpar(lwd = adjusted_lwd, fill = current_arrow_clr, col = current_arrow_clr),
          arrow = arrow(type = "closed", angle = a_angle, length = unit(a_l, "npc"))
        )
      } else {
        # The width can be wider using the special bezier arrows
        if (abs_arrow_width) {
          a_width <- bx_right$y_entry_height * 1.5 / no_boxes
        } else {
          a_width <- # getGridVal(lwd, "npc") +
            bx_right$y_entry_height * transition_flow[box_row, flow] / max_flow * 2.2
          # Set a maximum size in proportion to the line
          if (getGridVal(lwd, "npc", axisTo = "y") * 1.66 < a_width) {
            a_width <- getGridVal(lwd, "npc", axisTo = "y") * 1.66
          }
        }

        # Add line width addition if it is a background line
        if (!is.na(add_width)) {
          if (is.unit(add_width)) {
            a_width <- a_width + prGetNpcValue(add_width, "y")
          } else {
            a_width <- a_width * add_width
          }
        }


        if (a_width < adjusted_lwd) {
          sp_float_string <- sprintf("%%.%df", -floor(log10(adjusted_lwd - a_width)) + 1)
          warning(
            "The arrow width is smaller than the width of the line,",
            "thus not appearing as a regular arrow: ",
            sprintf(sp_float_string, a_width),
            " < ",
            sprintf(sp_float_string, adjusted_lwd)
          )

          # Looks really weird if this is allowed
          a_width <- adjusted_lwd
        }

        if (type == "simple") {
          bz <- bezierArrowSmpl(
            x = x_ctrl_points,
            y = y_ctrl_points,
            width = adjusted_lwd,
            arrow = list(length = a_l, base = a_width),
            clr = current_arrow_clr
          )
          grid.draw(bz)
        } else if (type == "gradient") {
          if (length(box_clr) > 1) {
            # Invert order as that is the fill order
            current_grdt_clr <- prTpGetColors(
              colors = box_clr,
              proportion = 1 - transition_arrow_props[box_row, flow],
              space = color_bar_subspace
            )
          } else {
            current_grdt_clr <- box_clr
          }
          bz <- bezierArrowGradient(
            x = x_ctrl_points,
            y = y_ctrl_points,
            width = adjusted_lwd,
            arrow = list(length = a_l, base = a_width),
            clr = current_arrow_clr,
            grdt_type = "triangle",
            grdt_prop = .8,
            grdt_clr_prop = 0.5,
            grdt_decrease_prop = .5,
            grdt_clr = current_grdt_clr
          )
          grid.draw(bz)
        } else {
          stop("The arrow type ", type, " is not yet implemented, sorry.")
        }
      }
    }
  }
}

#' Plot boxes and arrows
#'
#' Outputs all the boxes and arrow
#'
#' @param no_boxes Number of boxes to plot
#' @param txt The text
#' @param prop_start_sizes The proportion of the different boxes
#'  to the left
#' @param prop_end_sizes The proportion of the different boxes
#'  to the right
#' @param fill_start_clr The color of the boxes to the left
#' @param fill_end_clr The color of the boxes to the right
#' @param lwd The line width
#' @param line_col The color of the line
#' @param abs_arrow_width The absolute width of the arrow
#' @param transition_arrow_props The proportions of the different transitions if
#'  available.
#' @param plot_arrows If we are plotting shadow boxes then
#'  arrows should not be plotted and this should be set to \code{FALSE}
#' @param proportion It there is a proportion
#' @return \code{void}
#'
#' @inheritParams transitionPlot
#' @inheritParams prTpPlotBox
#' @keywords internal
prTpPlotBoxes <- function(overlap_order,
                          transition_flow,
                          no_boxes,
                          box_width,
                          tot_spacing,
                          txt,
                          cex,
                          txt_gpar = NULL,
                          prop_start_sizes, prop_end_sizes,
                          box_prop,
                          lwd_prop_total = lwd_prop_total,
                          fill_start_clr, fill_end_clr,
                          txt_start_clr, txt_end_clr,
                          lwd = 2, line_col = "#000000",
                          min_lwd,
                          max_lwd,
                          overlap_add_width,
                          overlap_bg_clr,
                          type_of_arrow,
                          abs_arrow_width,
                          arrow_clr,
                          transition_arrow_props,
                          color_bar_subspace,
                          plot_arrows = TRUE, proportion = FALSE) {
  for (i in overlap_order) {
    if (prop_start_sizes[i] > 0) {
      bx_left <- prTpGetBoxPositions(
        no = i, side = "left",
        transitions = transition_flow[i, ],
        prop_start_sizes = prop_start_sizes,
        prop_end_sizes = prop_end_sizes,
        tot_spacing = tot_spacing,
        box_width = box_width
      )
      if (!missing(box_prop) & proportion) {
        fill_clr <- fill_start_clr[i, ]
        txt_clr <- txt_start_clr[i, ]
        prop <- box_prop[i, 1]
      } else {
        prop <- NA
        fill_clr <- fill_start_clr[i]
        txt_clr <- txt_start_clr[i]
      }

      if (plot_arrows) {
        # Plot arrows
        if (lwd_prop_total) {
          max_flow <- max(transition_flow)
        } else {
          max_flow <- sum(transition_flow[i, ])
        }

        # Do the background arrows
        prTpPlotArrows(
          type = ifelse(type_of_arrow == "grid", "grid", "simple"),
          box_row = i,
          transition_flow = transition_flow,
          max_flow = max_flow,
          min_lwd = min_lwd,
          max_lwd = max_lwd,
          clr = rep(overlap_bg_clr, no_boxes * ncol(transition_flow)),
          box_clr = overlap_bg_clr,
          prop_start_sizes = prop_start_sizes,
          prop_end_sizes = prop_end_sizes,
          tot_spacing = tot_spacing,
          box_width = box_width,
          abs_arrow_width = abs_arrow_width,
          add_width = overlap_add_width
        )

        # The actual arrows
        prTpPlotArrows(
          type = type_of_arrow,
          box_row = i,
          transition_flow = transition_flow,
          max_flow = max_flow,
          min_lwd = min_lwd,
          max_lwd = max_lwd,
          clr = arrow_clr,
          box_clr = fill_clr,
          transition_arrow_props = transition_arrow_props,
          prop_start_sizes = prop_start_sizes,
          prop_end_sizes = prop_end_sizes,
          tot_spacing = tot_spacing,
          box_width = box_width,
          abs_arrow_width = abs_arrow_width,
          color_bar_subspace = color_bar_subspace,
          add_width = NA
        )
      }

      prTpPlotBox(
        bx = bx_left,
        bx_txt = txt[i, 1],
        fill = fill_clr,
        txt_clr = txt_clr,
        cex = cex,
        line_col = line_col,
        lwd = lwd,
        prop = prop,
        txt_gpar = txt_gpar
      )
    }

    if (prop_end_sizes[i] > 0) {
      bx_right <- prTpGetBoxPositions(
        no = i, side = "right",
        transitions = transition_flow[, i],
        prop_start_sizes = prop_start_sizes,
        prop_end_sizes = prop_end_sizes,
        tot_spacing = tot_spacing,
        box_width = box_width
      )

      if (!missing(box_prop) & proportion) {
        fill_clr <- fill_end_clr[i, ]
        txt_clr <- txt_end_clr[i, ]
        prop <- box_prop[i, 2]
      } else {
        prop <- NA
        fill_clr <- fill_end_clr[i]
        txt_clr <- txt_end_clr[i]
      }

      prTpPlotBox(
        bx = bx_right,
        bx_txt = txt[i, 2],
        fill = fill_clr,
        txt_clr = txt_clr,
        cex = cex,
        line_col = line_col,
        lwd = lwd,
        prop = prop,
        txt_gpar = txt_gpar
      )
    }
  }
}

#' Gets the box position
#'
#' @param no The box number
#' @param side The right or left side
#' @param transitions The flows to or from
#' @param prop_start_sizes The size of the start boxes
#' @param prop_end_sizes The size of the end boxes
#' @return \code{list(top, left, bottom, right, width, height)}
#'
#' @inheritParams transitionPlot
#' @importFrom utils head tail
#' @keywords internal
prTpGetBoxPositions <- function(no, side,
                                transitions,
                                prop_start_sizes, prop_end_sizes,
                                tot_spacing,
                                box_width) {
  no_boxes <- max(length(prop_start_sizes), length(prop_end_sizes))
  empty_boxes <- ifelse(side == "left",
    sum(prop_start_sizes == 0),
    sum(prop_end_sizes == 0)
  )

  # Calculate basics
  space <- tot_spacing / (no_boxes - 1 - empty_boxes)

  # Basic box vars
  ret <- list(
    height = (1 - tot_spacing) * ifelse(side == "left",
      prop_start_sizes[no],
      prop_end_sizes[no]
    ),
    width = box_width
  )

  if (no == 1) {
    ret$top <- 1
  } else {
    ret$top <- 1 -
      ifelse(side == "left",
        sum(prop_start_sizes[1:(no - 1)]),
        sum(prop_end_sizes[1:(no - 1)])
      ) * (1 - tot_spacing) -
      space * (no - 1)
  }
  ret$bottom <- ret$top - ret$height
  ret$y <- mean(c(ret$top, ret$bottom))

  ret$y_exit <- rep(ret$y, times = no_boxes)
  ret$y_entry_height <- ret$height / 3
  ret$y_entry <- rep(NA, times = no_boxes)
  if (any(is.na(transitions)) || any(sum(transitions[-1]) == 0)) {
    # There are invalid transitions or it seems that
    # only one arrow exists
    if (any(is.na(transitions))) {
      ret$y_entry <- seq(
        to = ret$y - ret$height / 6,
        from = ret$y + ret$height / 6,
        length.out = no_boxes
      )
    } else {
      ret$y_entry <- rep(ret$y, times = no_boxes)
    }
  } else {
    # The entry point should be distributed according to arrow width
    # Unfortunately the arrow width also relates to the min_lwd and max_lwd
    # there is therefore some need for adjustments
    for (i in 1:no_boxes) {
      if (i == 1) {
        ret$y_entry[i] <- ret$y + ret$y_entry_height / 2
      } else if (sum(transitions[1:i]) == sum(transitions)) {
        ret$y_entry[i] <- ret$y - ret$y_entry_height / 2
      } else {
        # Do a proportion and remove half of the top/bottom
        # as these attach at the ends
        arrow_proportion <- sum(transitions[2:i]) /
          (sum(transitions) - (head(transitions, 1) + tail(transitions, 1)) / 2)

        # Narrow the space slightly
        narrower <- (no_boxes + 8) / (no_boxes + 10)

        ret$y_entry[i] <- ret$y + ret$height / 6 -
          ret$y_entry_height / no_boxes / 2 -
          ret$y_entry_height * narrower * arrow_proportion
      }
    }
    ret$y_entry <- seq(
      to = ret$y - ret$height / 6,
      from = ret$y + ret$height / 6,
      length.out = no_boxes
    )
  }

  # Now the x-axis
  if (side == "right") {
    ret$left <- 1 - box_width
    ret$right <- 1
  } else {
    ret$left <- 0
    ret$right <- box_width
  }

  txt_margin <- box_width / 10
  ret$txt_height <- ret$height - txt_margin * 2
  ret$txt_width <- box_width - txt_margin * 2

  ret$x <- mean(c(ret$left, ret$right))

  return(ret)
}

#' Gets a set of colors or just one color
#'
#' Used in order to illustrate the mix between two
#' proportions in the \code{\link{transitionPlot}}.
#'
#' @param colors A set of min. two colors that is used
#'  for \code{\link[grDevices:colorRamp]{colorRampPalette}}.
#' @param proportion A proportion or a set of proportions
#'  between 0 and 1. If you leave this out then the full color
#'  span will be returned.
#' @param space If there is little or no difference
#'  at the low/high proportions of the spectrum then it
#'  can be of interest to focus the color change to the center
#'  leaving the tails constant
#' @param no The color resolution to use
#' @return \code{character} The function can return both single colors
#'  or multiple colors as character vector (see \code{\link[grDevices]{rgb}})
#'
#' @importFrom utils tail
#' @keywords internal
prTpGetColors <- function(colors, proportion, space = NULL, no = 101) {
  start <- c()
  end <- c()
  if (!is.null(space)) {
    if (any(space > 1 | space < 0)) {
      stop("Your color subspace that you define should be between 0 and 1")
    }

    if (length(space) > 2) {
      stop(
        "The color subspace has to be a length of either one or two",
        " you have provided ", paste(space, collapse = ", "),
        " of length", length(space)
      )
    }

    if (length(space) == 2) {
      start_no <- ceiling(space[1] * no)
      end_no <- ceiling(space[2] * no)
      start <- rep(colors[1], times = start_no)
      end <- rep(tail(colors, 1), times = end_no)
      no <- end_no - start_no
      if (no < 0) {
        no <- 0
      }
    } else {
      tails_no <- ceiling(space * no)
      start <- rep(colors[1], times = tails_no)
      end <- rep(tail(colors, 1), times = tails_no)
      no <- no - 2 * tails_no
      if (no < 0) {
        no <- 0
      }
    }
  }

  if (no > 0) {
    clrs <- c(
      start,
      colorRampPalette(colors, space = "Lab")(no),
      end
    )
  } else {
    clrs <- c(start, end)
  }

  if (missing(proportion)) {
    return(clrs)
  }

  if (any(proportion < 0 | proportion > 1)) {
    stop(
      "You color proportion of interest must lie between 0 and 1",
      " you have provided: ", proportion
    )
  }

  return(clrs[1 + min(100, floor(length(clrs) * proportion))])
}

#' A dense time-span text
#'
#' When adding a time span text we often don't want to write *3 jun - 10 jun* but
#' shorten it to *3 - 10 jun* while retaining month and year info only if the span
#' crosses between months or years.
#'
#' @details
#'
#' There are options that can be set using the \code{\link[base:options]{options}}:
#'
#' * `Gmisc_time2spanTxt_day_month` The date with day + month as formatted by \code{\link[glue]{glue}} where the time is passed as `time`.
#' * `Gmisc_time2spanTxt_full_year` The full date with day + month + year as formatted by \code{\link[glue]{glue}} where the time is passed as `time`.
#' * `Gmisc_time2spanTxt_template` The merge of the stop & start elements using \code{\link[glue]{glue}}.
#'
#' @md
#' @param times The dates or POSIX timestamps to used for time span
#' @param day_month_glue_txt The \code{\link[glue]{glue}} string to format days and months with `time` as the time input
#' @param full_year_format The \code{\link[glue]{glue}} string to format the full year with `time` as the time input
#' @param start_stop_glue_txt The string used in the \code{\link[glue]{glue}} for putting the
#'  start and stop dates together into one string
#' @return \code{string} A string describing the time span
#'
#' @examples
#' time2spanTxt(as.POSIXct(c("2020-01-02", "2020-03-01", NA)))
#' # 2 jan to 1 mar
#'
#' @importFrom lubridate mday as_date year month
#' @importFrom stringr str_trim
#' @importFrom glue glue
#' @export
time2spanTxt <- function(times,
                         day_month_glue_txt = getOption("Gmisc_time2spanTxt_day_month",
                                                        default = "{mday(time)} {month(time, label = TRUE)}"),
                         full_year_format = getOption("Gmisc_time2spanTxt_full_year",
                                                      default = "{mday(time)} {month(time, label = TRUE)} {year(time)}"),
                         start_stop_glue_txt = getOption("Gmisc_time2spanTxt_template",
                                                         default = "{start} to {stop}")) {
  times <- as_date(times)
  times <- times[!is.na(times) & !is.infinite(times)]
  min_time <- min(times)
  max_time <- max(times)

  if (year(min_time) != year(max_time)) {
    start <- glue(full_year_format, time = min_time)
    stop <- glue(full_year_format, time = max_time)
  } else if (month(min_time) != month(max_time)) {
    start <- glue(day_month_glue_txt, time = min_time)
    stop <- glue(day_month_glue_txt, time = max_time)
  } else {
    start <- mday(min_time)
    stop <- glue(day_month_glue_txt, time = max_time)
  }

  glue(start_stop_glue_txt)
}

library('testthat')
context('pathJoin')

test_that("time2spanTxt basics", {
  expect_equal(time2spanTxt(as.POSIXct(c("2020-01-02", "2020-03-01", NA)),
                            day_month_glue_txt = "{mday(time)} {month(time, label = TRUE, locale = 'C')}",
                            start_stop_glue_txt = "{start}-{stop}"),
               "2 Jan-1 Mar")

  expect_equal(time2spanTxt(as.Date(c("2020-01-02", "2020-03-11", NA)),
                            day_month_glue_txt = "{mday(time)} {month(time, label = TRUE, locale = 'C')}",
                            start_stop_glue_txt = "{start}-{stop}"),
               "2 Jan-11 Mar")

  expect_equal(time2spanTxt(as.Date(c("2020-01-02", "2020-03-11", "2020-02-01")),
                            day_month_glue_txt = "{mday(time)} {month(time, label = TRUE, locale = 'C')}",
                            start_stop_glue_txt = "{start}-{stop}"),
               "2 Jan-11 Mar")
})

#' Outputs an object
#'
#' Manually viewing a list object can be tricky where the natural print
#' can be hard to work through. The config format *yaml* is increadibly
#' dense and useful not only for writing configs but also viewing them
#' which `yamlDump` helps with.
#'
#' @param x An object that \code{\link[yaml]{as.yaml}} accepts
#' @return \code{void}
#'
#' @examples
#' some_fancy_list <- list(complex = list(some_data = 1:3,
#'                                        other_data = list(name = "Max")),
#'                         simple = "awesome overview")
#' yamlDump(some_fancy_list)
#' #complex:
#' #  some_data:
#' #  - 1
#' #  - 2
#' #  - 3
#' #  other_data:
#' #    name: Max
#' #simple: awesome overview
#'
#' # If you got a character json you can also input it directly
#' # and the function will automatically convert it to a list
#' yamlDump('{"a":{"b":["1"]}}')
#'
#' @importFrom yaml as.yaml
#' @export
yamlDump <- function(x) {
  UseMethod("yamlDump")
}

#' @exportS3Method
yamlDump.default <- function(x) {
  as.yaml(x) %>% cat
}

#' @exportS3Method
yamlDump.character <- function(x) {
  safeLoadPkg("jsonlite")
  yamlDump(jsonlite::fromJSON(x, simplifyVector = FALSE))
}

#' @exportS3Method
yamlDump.json <- function(x) {
  yamlDump.character(x)
}

#' Connector Strategy Dispatcher
#'
#' @param start The start object(s)
#' @param end The end object(s)
#' @param type The connector type
#' @return A strategy object
#' @keywords internal
#' @noRd
prGetConnectorStrategy <- function(start, end, type) {
    is_start_list <- prIsBoxList(start)
    is_end_list <- prIsBoxList(end)

    if (is_start_list && is_end_list) {
        stop("Both 'start' and 'end' cannot be lists (not implemented).", call. = FALSE)
    }

    config <- if (is_start_list) {
        "ManyToOne"
    } else if (is_end_list) {
        "OneToMany"
    } else {
        "OneToOne"
    }

    # Map types to camelCase for class names
    type_map <- c(
        "vertical" = "Vertical",
        "horizontal" = "Horizontal",
        "L" = "L",
        "-" = "Dash",
        "Z" = "Z",
        "N" = "N",
        "fan_in_top" = "FanInTop",
        "fan_in_center" = "FanInCenter",
        "side" = "Side"
    )

    type_name <- type_map[type]
    if (is.na(type_name)) type_name <- "Default"

    class_name <- paste0(config, type_name, "ConnectorStrategy")

    structure(
        list(
            config = config,
            type = type
        ),
        class = c(class_name, "ConnectorStrategy")
    )
}

#' Calculate the connector grob(s) for a given strategy
#'
#' @param strategy The strategy object
#' @param ... Arguments passed to the strategy implementation
#' @return A single grob or a list of grobs
#' @keywords internal
#' @noRd
prCalculateConnector <- function(strategy, ...) {
    UseMethod("prCalculateConnector")
}

#' Default fallback for connector strategies
#' @noRd
#' @export
prCalculateConnector.ConnectorStrategy <- function(strategy, ...) {
    # If no specific strategy for the type+config combination exists,
    # we fall back to the config-level default handlers.
    args <- list(...)
    if (strategy$config == "ManyToOne") {
        return(prConnectManyToOneStandard(
            starts = args$start,
            end = args$end,
            type = strategy$type,
            subelmnt = args$subelmnt,
            lty_gp = args$lty_gp,
            arrow_obj = args$arrow_obj,
            smooth = args$smooth,
            corner_radius = args$corner_radius
        ))
    }

    if (strategy$config == "OneToMany") {
        return(prConnectOneToManyStandard(
            start = args$start,
            ends = args$end,
            type = strategy$type,
            subelmnt = args$subelmnt,
            lty_gp = args$lty_gp,
            arrow_obj = args$arrow_obj,
            smooth = args$smooth,
            corner_radius = args$corner_radius
        ))
    }

    # OneToOne default
    prConnect1(
        start = args$start,
        end = args$end,
        type = strategy$type,
        subelmnt = args$subelmnt,
        lty_gp = args$lty_gp,
        arrow_obj = args$arrow_obj,
        smooth = args$smooth,
        corner_radius = args$corner_radius,
        side = args$side,
        label = args$label,
        label_gp = args$label_gp,
        label_bg_gp = args$label_bg_gp,
        label_pos = args$label_pos,
        label_offset = args$label_offset,
        label_pad = args$label_pad
    )
}

#' Specific strategy for Many-to-One Fan-in Center
#' @noRd
#' @export
prCalculateConnector.ManyToOneFanInCenterConnectorStrategy <- function(strategy, ...) {
    args <- list(...)
    prConnectManyToOneFanCenter(
        starts = args$start,
        end = args$end,
        subelmnt = args$subelmnt,
        lty_gp = args$lty_gp,
        arrow_obj = args$arrow_obj,
        margin = args$margin,
        split_pad = args$split_pad
    )
}

#' Specific strategy for Many-to-One Fan-in Top
#' @noRd
#' @export
prCalculateConnector.ManyToOneFanInTopConnectorStrategy <- function(strategy, ...) {
    args <- list(...)
    prConnectManyToOneFanTop(
        starts = args$start,
        end = args$end,
        subelmnt = args$subelmnt,
        lty_gp = args$lty_gp,
        arrow_obj = args$arrow_obj,
        margin = args$margin,
        split_pad = args$split_pad
    )
}

#' Specific strategy for Many-to-One N
#' @noRd
#' @export
prCalculateConnector.ManyToOneNConnectorStrategy <- function(strategy, ...) {
    args <- list(...)
    prConnectManyToOneN(
        starts = args$start,
        end = args$end,
        type = strategy$type,
        subelmnt = args$subelmnt,
        lty_gp = args$lty_gp,
        arrow_obj = args$arrow_obj,
        split_pad = args$split_pad,
        smooth = args$smooth,
        corner_radius = args$corner_radius
    )
}

#' Specific strategy for One-to-Many N
#' @noRd
#' @export
prCalculateConnector.OneToManyNConnectorStrategy <- function(strategy, ...) {
    args <- list(...)
    prConnectOneToManyN(
        start = args$start,
        ends = args$end,
        type = strategy$type,
        subelmnt = args$subelmnt,
        lty_gp = args$lty_gp,
        arrow_obj = args$arrow_obj,
        split_pad = args$split_pad,
        smooth = args$smooth,
        corner_radius = args$corner_radius
    )
}

fastDoCall("complex", list(imaginary = 1:3))

## if we already have a list (e.g. a data frame)
## we need c() to add further arguments
tmp <- expand.grid(letters[1:2], 1:3, c("+", "-"))
fastDoCall("paste", c(tmp, sep = ""))

## examples of where objects will be found.
A <- 2
f <- function(x) print(x^2)
env <- new.env()
assign("A", 10, envir = env)
assign("f", f, envir = env)
f <- function(x) print(x)
f(A)                                      # 2
fastDoCall("f", list(A))                     # 2
fastDoCall("f", list(A), envir = env)        # 4
fastDoCall(f, list(A), envir = env)          # 2
fastDoCall("f", list(quote(A)), envir = env) # 100
fastDoCall(f, list(quote(A)), envir = env)   # 10
fastDoCall("f", list(as.name("A")), envir = env) # 100

eval(call("f", A))                      # 2
eval(call("f", quote(A)))               # 2
eval(call("f", A), envir = env)         # 4
eval(call("f", quote(A)), envir = env)  # 100

#' A wrapper around the \code{\link[htmlTable]{htmlTable}}/\code{\link[Hmisc]{latex}} function
#'
#' Calls the \code{\link[htmlTable]{htmlTable}}/\code{\link[Hmisc]{latex}} after extracting the
#' \code{rgroup}/\code{n.rgroup} arguments.
#'
#' @param object The \code{\link{mergeDesc}} object
#' @param ... Passed onto \code{\link[htmlTable]{htmlTable}}/\code{\link[Hmisc]{latex}}
#'
#' @rdname htmlTable_latex
#' @importFrom Hmisc latex
#' @keywords internal
#' @export
latex.descMrg <- function(object, ...) {
  dots <- list(...)
  if (!"rgroup" %in% names(dots)) {
    return(NextMethod(
      generic = NULL, object = object,
      rgroup = attr(object, "rgroup"),
      n.rgroup = attr(object, "n.rgroup"),
      ...
    ))
  }

  NextMethod(
    generic = NULL,
    object = object,
    ...
  )
}


#' @rdname htmlTable_latex
#' @export
#' @import htmlTable
#' @import magrittr
htmlTable.descMrg <- function(x, ...) {
  dots <- list(...)
  attr_args <- attr(x, "htmlTable_args")
  for (n in names(attr_args)) {
    if (!n %in% names(dots)) {
      dots[[n]] <- attr_args[[n]]
    }
  }

  # Merge calls
  class(x) <- class(x)[!(class(x) %in% "descMrg")]
  args <- c(
    list(x = x),
    dots
  )
  if (!"rgroup" %in% names(args)) {
    args[["rgroup"]] <- attr(x, "rgroup")
    args[["n.rgroup"]] <- attr(x, "n.rgroup")
  }

  fastDoCall(htmlTable, args)
}


#' @rdname htmlTable_latex
#' @export
#' @import htmlTable
print.descMrg <- function(x, ...) {
  htmlTable(x, ...) %>%
    print()
}


#' @rdname htmlTable_latex
#' @import htmlTable
#' @export
#' @importFrom knitr asis_output
#' @importFrom knitr knit_print
knit_print.descMrg <- function(x, ...) {
  htmlTable(x, ...) %>%
    knit_print()
}

#' Append to list of boxes (S3)
#'
#' Append an element to a list of boxes, preserving attributes/class.
#' Masks `base::append` to support S3 dispatch.
#'
#' @param x A `Gmisc_list_of_boxes` or `list`.
#' @param values The object(s) to append. If a single `boxGrob` is provided, it is wrapped in a list.
#' @param after Subscript, after which the values are to be appended.
#'
#' @return Updated list of boxes.
#' @export
#' @family flowchart components
append <- function(x, values, after = length(x)) {
  UseMethod("append")
}

#' @export
#' @rdname append
append.default <- function(x, values, after = length(x)) {
  base::append(x, values, after)
}

#' @export
#' @rdname append
append.Gmisc_list_of_boxes <- function(x, values, after = length(x)) {
  if (inherits(values, "box")) {
    values <- list(values)
  }

  ret <- base::append(x, values, after)

  # Restore attributes
  special_attrs <- setdiff(names(attributes(x)), c("names", "class"))
  for (at in special_attrs) {
    if (is.null(attr(ret, at))) {
      attr(ret, at) <- attr(x, at)
    }
  }

  prConvertListToBoxList(ret)
}

#' Insert element into a list of boxes (S3)
#'
#' Inserts a box into a list of boxes and positions it relative to the
#' surrounding boxes.
#'
#' @param x A `Gmisc_list_of_boxes`.
#' @param element The new box object to insert.
#' @param ... Not used.
#' @param after The name or index of the box after which to insert.
#' @param before The name or index of the box before which to insert.
#'
#' @return The updated list of boxes with the new element inserted.
#' @export
#' @family flowchart components
insert <- function(x, element, ..., after = NULL, before = NULL) {
  UseMethod("insert")
}

#' @export
#' @rdname insert
insert.default <- function(x, element, ..., after = NULL, before = NULL) {
  if (is.list(x) && !inherits(x, "box")) {
    return(insert(prConvertListToBoxList(x), element, ..., after = after, before = before))
  }
  stop("insert() expects a list of boxes as first argument")
}

#' @export
#' @rdname insert
insert.Gmisc_list_of_boxes <- function(x, element, ..., after = NULL, before = NULL) {
  if (!xor(is.null(after), is.null(before))) {
    stop("You must specify either 'after' or 'before' (but not both).")
  }

  element_name <- NULL
  if (inherits(element, "list") && length(element) == 1 && inherits(element[[1]], "box")) {
    element_name <- names(element)
    element <- element[[1]]
  }
  if (!inherits(element, "box")) {
    stop("inserted element must be a box")
  }

  # Find insertion index
  idx <- NULL
  if (!is.null(after)) {
    if (is.character(after)) {
      idx <- match(after, names(x))
      if (is.na(idx)) stop("Could not find box named '", after, "'")
    } else {
      idx <- after
    }
  } else {
    if (is.character(before)) {
      match_idx <- match(before, names(x))
      if (is.na(match_idx)) stop("Could not find box named '", before, "'")
      idx <- match_idx - 1
    } else {
      idx <- before - 1
    }
  }

  # Calculate bounds for positioning
  # If inserting after box A (index idx):
  # - Prev box is x[[idx]]
  # - Next box is x[[idx+1]] (if exists)

  # If inserting before box B (index match_idx):
  # - Prev box is x[[match_idx-1]] -> x[[idx]]
  # - Next box is x[[match_idx]] -> x[[idx+1]]

  prev_box <- if (idx > 0 && idx <= length(x)) x[[idx]] else NULL
  next_box <- if (idx + 1 <= length(x)) x[[idx + 1]] else NULL

  if (!is.null(prev_box) && !is.null(next_box)) {
    # Position between
    pc <- coords(prev_box)
    nc <- coords(next_box)

    # Determine orientation?
    # Heuristic: mostly different X -> horizontal split
    # Mostly different Y -> vertical split

    dx <- abs(convertX(pc$x - nc$x, "mm", valueOnly = TRUE))
    dy <- abs(convertY(pc$y - nc$y, "mm", valueOnly = TRUE))

    new_x <- unit(0.5, "npc") # default if fail
    new_y <- unit(0.5, "npc") # default if fail

    # Midpoint calculation
    # X
    px <- convertX(pc$x, "npc", valueOnly = FALSE)
    nx <- convertX(nc$x, "npc", valueOnly = FALSE)
    new_x <- px + (nx - px) * 0.5

    # Y
    py <- convertY(pc$y, "npc", valueOnly = FALSE)
    ny <- convertY(nc$y, "npc", valueOnly = FALSE)
    new_y <- py + (ny - py) * 0.5

    element <- moveBox(element, x = new_x, y = new_y, space = "absolute", just = "center")
  } else if (!is.null(prev_box)) {
    # Only prev box exists (append at end or start with no next)
    # Just place valid coords if not present?
    # User implies "mid position", if no two boxes, can't calc mid.
    # Maybe just keep existing coords if element has them, or default offset?
    # For now, do nothing if we can't interpolate.
  } else if (!is.null(next_box)) {
    # Only next box exists
  }

  # Perform insertion
  to_ins <- list(element)
  if (!is.null(element_name)) names(to_ins) <- element_name
  append(x, to_ins, after = idx)
}

library(testthat)

test_that("Flowchart S3 API structure", {
  # This test defines the expected behavior for the specific S3 API
  # requested: list() |> align() |> connect() |> print()

  # Mock components
  b1 <- boxGrob("Start")
  b2 <- boxGrob("End")

  # 1. Base list
  l <- list(start = b1, end = b2)
  expect_type(l, "list")

  # 2. align() generic
  # Should upgrade to Gmisc_list_of_boxes and set alignment
  l_aligned <- align(l, axis = "y")
  expect_s3_class(l_aligned, "Gmisc_list_of_boxes")

  # 3. connect() generic
  # Should NOT return a grob, but the list with connection metadata attached
  # The connectGrob uses coords which align sets.
  # So we connect AFTER alignment.
  l_conn <- connect(l_aligned, from = "start", to = "end", type = "vertical")
  expect_s3_class(l_conn, "Gmisc_list_of_boxes")
  conn_attr <- attr(l_conn, "connections")
  expect_type(conn_attr, "list")
  expect_equal(length(conn_attr), 1)

  # Check that the connection object (grob) is stored
  expect_s3_class(conn_attr[[1]], "connect_boxes")

  # 4. spread() generic
  # Should also accept a list and return Gmisc_list_of_boxes
  l_spread <- spread(l, axis = "x")
  expect_s3_class(l_spread, "Gmisc_list_of_boxes")
})

test_that("Class duplication check (user concern)", {
  # Mimicking the behavior of potential class addition
  obj <- list(a = 1)
  class(obj) <- c("Gmisc_list_of_boxes", "list")

  # If we extend it again
  obj2 <- prExtendClass(obj, "Gmisc_list_of_boxes")
  # Should be unique
  expect_equal(class(obj2), c("Gmisc_list_of_boxes", "list"))

  # If we use structure() blindly (which some code might do)
  obj3 <- structure(obj, class = c("Gmisc_list_of_boxes", class(obj)))
  # This WOULD duplicate
  expect_equal(class(obj3), c("Gmisc_list_of_boxes", "Gmisc_list_of_boxes", "list"))

  # The fix is to ensure all internal helpers use prExtendClass or unique()
})

test_that("Invalid input validation", {
  b1 <- boxGrob("Start")

  # Invalid list content
  bad_list <- list(start = b1, bad = "not a box")

  expect_error(align(bad_list, axis = "y"), "not valid boxes")
  expect_error(spread(bad_list, axis = "y"), "not valid boxes")
  expect_error(connect(bad_list, from = "start", to = "bad"), "not valid boxes")

  # Recursive invalid content
  recursive_bad <- list(start = b1, group = list(ok = b1, bad = 12345))
  expect_error(align(recursive_bad, axis = "y"), "group\\$bad")

  # Valid list content
  good_list <- list(start = b1, valid = list(b1)) # Nested list allowed handled by prConvert
  # Note: prConvertListToBoxList allows lists as elements.

  # Should not error
  expect_error(align(good_list, axis = "y"), NA)
})

test_that("S3 Mutations: move, append", {
  b1 <- boxGrob("Start")
  b2 <- boxGrob("End")

  l <- list(start = b1) |> align()
  expect_true(is.list(l))
  expect_true(inherits(l, "Gmisc_list_of_boxes"))

  message("L is list: ", is.list(l))
  message("L class: ", paste(class(l), collapse = ", "))

  # Append
  l_appended <- append(l, list(end = b2))
  expect_s3_class(l_appended, "Gmisc_list_of_boxes")
  expect_equal(length(l_appended), 2)
  expect_named(l_appended, c("start", "end"))

  # Ensure attributes preserved (align sets align_list attr? no, just coords in boxes)
  # But connections should be preserved
  attr(l, "connections") <- list("fake_conn")

  l_appended_conn <- append(l, list(end = b2))
  expect_false(is.null(attr(l_appended_conn, "connections")))

  # Move
  # Testing the move generic wrapper around moveBox
  l_moved <- move(l, x = .5, y = .5, subelement = "start")
  expect_s3_class(l_moved, "Gmisc_list_of_boxes")
  expect_false(is.null(attr(l_moved, "connections")))

  # Verify coords changed (moveBox works)
  # We can't easily check internal coords without checking the grob, but we trust moveBox.
  # Just checking structure here.
})

test_that("S3 Mutations: insert", {
  b1 <- boxGrob("Start", x = .1, y = .5)
  b2 <- boxGrob("End", x = .9, y = .5)

  l <- list(start = b1, end = b2) |> align()

  # Valid insertion between
  b_mid <- boxGrob("Middle")
  l_ins <- insert(l, b_mid, after = "start")

  # Check list structure
  expect_equal(length(l_ins), 3)
  expect_equal(names(l_ins), c("start", "", "end"))

  # Check midpoint calculation
  # b1 x=.1, b2 x=.9 -> mid should be .5
  # Note: we need to allow for some floating point tolerance and unit conversion
  mid_box <- l_ins[[2]]
  mid_coords <- coords(mid_box)

  # In test environment, grid units might need simple conversion verification
  # Since moveBox sets absolute viewport position.
  # We assume prConvertAllToNpc or similar helps, but moveBox sets "npc" units usually?
  # Actually moveBox(space="absolute") uses units provided.

  # Just verify x is approx 0.5
  x_val <- convertX(mid_coords$x, "npc", valueOnly = TRUE)
  expect_equal(x_val, 0.5, tolerance = 0.01)

  # Test insertion before
  l_ins2 <- insert(l, b_mid, before = "end")
  expect_equal(length(l_ins2), 3)
  expect_equal(convertX(coords(l_ins2[[2]])$x, "npc", valueOnly = TRUE), 0.5, tolerance = 0.01)

  # Test appending using insert (fail to calc midpoint, but inserts)
  # If we insert after end, prev=b2, next=NULL.
  # Code says: do nothing to coords.
  l_ins3 <- insert(l, b_mid, after = "end")
  expect_equal(length(l_ins3), 3)
})

test_that("Complex chaining example", {
  # Mock components
  org_cohort <- boxGrob("Stockholm", x = .5, y = .9)
  eligible <- boxGrob("Eligible", x = .5, y = .8)
  included <- boxGrob("Included", x = .5, y = .7)
  grp_a <- boxGrob("A", x = .4, y = .5)
  grp_b <- boxGrob("B", x = .6, y = .5)
  excluded <- boxGrob("Excluded", x = .8, y = .8) # to move

  # We use the pipeline exactly as user described (mocking spread functions slightly if needed, or using real ones)
  # Requires namespace access to internal S3 methods if not loaded, but test_file loads pkg

  res <- list(
    org_cohort = org_cohort,
    eligible = eligible,
    included = included,
    grps = list(grp_a, grp_b)
  ) |>
    spread(axis = "y") |>
    spread(subelement = "grps", axis = "x") |>
    insert(list(excluded = excluded), after = "eligible", name = "excluded") |>
    move(name = "excluded", x = .8) |>
    connect("org_cohort", "eligible", type = "vert") |>
    connect("eligible", "included", type = "vert") |>
    connect("included", "grps", type = "N") |>
    connect("eligible", "excluded", type = "L", label = "Excluded")

  expect_s3_class(res, "Gmisc_list_of_boxes")
  expect_true("excluded" %in% names(res))
  expect_equal(length(attr(res, "connections")), 4)

  # Verify move
  # moveBox logic with absolute space typically sets x to the value provided.
  ex_box <- res$excluded
  ex_coords <- coords(ex_box)
  expect_equal(convertX(ex_coords$x, "npc", valueOnly = TRUE), 0.8)

  # Debug connections count
  # We expect at least 4 connections (1 for each connect call)
  # Sometimes connect calls might produce multiple grobs depending on implementation details of 1-to-many
  conns <- attr(res, "connections")
  expect_gte(length(conns), 4)
})

test_that("connect() maps list-to-list targets pairwise", {
  fc <- flowchart(
    rando = "Randomised N = 100",
    groups = list(
      "Group1\nn = 50",
      "Group2\nn = 50"
    ),
    groups2 = list(
      "Analysed\nn = 49",
      "Analysed\nn = 48"
    )
  ) |>
    spread(axis = "y", margin = unit(0.02, "npc")) |>
    spread(subelement = "groups", axis = "x", margin = unit(.05, "npc")) |>
    spread(subelement = "groups2", axis = "x", margin = unit(.05, "npc")) |>
    connect("groups", "groups2", type = "vertical")

  conns <- attr(fc, "connections")
  expect_equal(length(conns), 1)
  expect_s3_class(conns[[1]], "connect_boxes_list")
  expect_equal(length(conns[[1]]), 2)

  end_x <- vapply(conns[[1]], function(g) {
    line <- attr(g, "line")
    convertX(line$x[length(line$x)], "npc", valueOnly = TRUE)
  }, numeric(1))

  target_x <- vapply(fc$groups2, function(b) {
    convertX(coords(b)$x, "npc", valueOnly = TRUE)
  }, numeric(1))

  expect_equal(end_x, target_x, tolerance = 1e-6)
})

test_that("connect() errors on unequal list-to-list lengths", {
  fc <- flowchart(
    from = list("A", "B"),
    to = list("C")
  )

  expect_error(
    connect(fc, "from", "to", type = "vertical"),
    "must have the same length"
  )
})

test_that("consort-style grouped pipeline with excluded branch remains stable", {
  library(grid)

  fc <- flowchart(
    rando = "Randomised N = 100",
    groups = list(
      "Group1\nn = 50",
      "Group2\nn = 50"
    ),
    groups2 = list(
      "Analysed\nn = 49",
      "Analysed\nn = 48"
    ),
    groups3 = list(
      "Excluded\nn = 1",
      "Excluded\nn = 2"
    )
  ) |>
    spread(axis = "y", margin = unit(0.02, "npc")) |>
    spread(subelement = "groups", axis = "x", margin = unit(.3, "npc")) |>
    spread(subelement = "groups2", axis = "x", margin = unit(.1, "npc")) |>
    spread(subelement = "groups3", axis = "x", margin = unit(.3, "npc")) |>
    move(name = "groups2", x = .6) |>
    connect("rando", "groups", type = "N") |>
    connect("groups", "groups2", type = "L") |>
    connect("groups", "groups3", type = "vert")

  expect_s3_class(fc, "Gmisc_list_of_boxes")
  expect_true(all(c("groups", "groups2", "groups3") %in% names(fc)))

  conns <- attr(fc, "connections")
  expect_type(conns, "list")
  expect_equal(length(conns), 3)

  expect_silent({
    aligned <- fc |>
      alignVertical("groups", subelement = "groups2")
  })

  expect_s3_class(aligned, "Gmisc_list_of_boxes")
  expect_true(all(c("groups", "groups2", "groups3") %in% names(aligned)))
  expect_equal(length(aligned$groups2), 2)
})

#' Add a p-value column to the results
#'
#' @param results The results that we want to add the column to
#'
#' @return results with added column
#' @inheritParams getDescriptionStatsBy
prAddDescStats <- function(results,
                           x,
                           by,
                           statistics,
                           statistics.suppress_warnings,
                           statistics.sig_lim,
                           statistics.two_dec_lim,
                           html) {
  org <- results
  column_names <- attr(results, "column_names")

  if (statistics.suppress_warnings) {
    pval <- suppressWarnings(statistics(x = x, by = by))
  } else {
    pval <- statistics(x = x, by = by)
  }

  if (is.numeric(pval) &&
      pval <= 1 &&
      pval >= 0) {
    pval <- txtPval(pval,
                    lim.sig = statistics.sig_lim,
                    lim.2dec = statistics.two_dec_lim,
                    html = html
    )
    results <- cbind(results, c(pval, rep("", nrow(results) - 1)))
    column_names <- c(column_names, "P-value")
  } else if (is.character(pval) && !is.null(attr(pval, "colname"))) {
    results <- cbind(results, c(pval, rep("", nrow(results) - 1)))
    column_names <- c(column_names, attr(pval, "colname"))
  } else {
    stop("Your statistics function should either return a numerical value from 0 to 1 or a character with the attribute 'colname'")
  }

  org %>%
    copyAllNewAttributes(results) %>%
    structure(column_names = column_names)
}

library(magrittr)
library(dplyr)
library(htmlTable)

data(mtcars)
mtcars %<>%
  mutate(am = factor(am, levels = 0:1, labels = c("Automatic", "Manual")),
         vs = factor(vs, levels = 0:1, labels = c("V-shaped", "straight")),
         drat_prop = drat > median(drat),
         drat_prop = factor(drat_prop,
                            levels = c(FALSE, TRUE),
                            labels = c("High ratio", "Low ratio")),
         carb_prop = carb > 2,
         carb_prop = factor(carb_prop,
                            levels = c(FALSE, TRUE),
                            labels = c("&le; 2", "&gt; 2")),
         across(c(gear, carb, cyl), factor))

# A simple bare-bone example
mtcars %>%
  getDescriptionStatsBy(`Miles per gallon` = mpg,
                        Weight = wt,
                        `Carborators &le; 2` = carb_prop,
                        by = am) %>%
  htmlTable(caption  = "Basic continuous stats from the mtcars dataset")
invisible(readline(prompt = "Press [enter] to continue"))

# For labeling & units we use set_column_labels/set_column_unit that use
# the Hmisc package annotation functions
mtcars %<>%
  set_column_labels(am = "Transmission",
                    mpg = "Gas",
                    wt = "Weight",
                    gear = "Gears",
                    disp = "Displacement",
                    vs = "Engine type",
                    drat_prop = "Rear axel ratio",
                    carb_prop = "Carburetors") %>%
  set_column_units(mpg = "Miles/(US) gallon",
                   wt = "10<sup>3</sup> lbs",
                   disp = "cu.in.")

mtcars %>%
  getDescriptionStatsBy(mpg,
                        wt,
                        `Gear&dagger;` = gear,
                        drat_prop,
                        carb_prop,
                        vs,
                        by = am,
                        header_count = TRUE,
                        use_units = TRUE,
                        show_all_values = TRUE)  %>%
  addHtmlTableStyle(pos.caption = "bottom") %>%
  htmlTable(caption  = "Stats from the mtcars dataset",
            tfoot = "&dagger; Number of forward gears")
invisible(readline(prompt = "Press [enter] to continue"))

# Using the default parameter we can
mtcars %>%
  getDescriptionStatsBy(mpg,
                        wt,
                        `Gear&dagger;` = gear,
                        drat_prop,
                        carb_prop,
                        vs,
                        by = am,
                        header_count = TRUE,
                        use_units = TRUE,
                        default_ref = c(drat_prop = "Low ratio",
                                        carb_prop = "&gt; 2"))  %>%
  addHtmlTableStyle(pos.caption = "bottom") %>%
  htmlTable(caption  = "Stats from the mtcars dataset",
            tfoot = "&dagger; Number of forward gears")
invisible(readline(prompt = "Press [enter] to continue"))

# We can also use lists
tll <- list()
tll[["Gear (3 to 5)"]] <- getDescriptionStatsBy(mtcars$gear, mtcars$am)
tll <- c(tll,
         list(getDescriptionStatsBy(mtcars$disp, mtcars$am)))

mergeDesc(tll,
          htmlTable_args = list(caption  = "Factored variables")) %>%
  htmlTable::addHtmlTableStyle(css.rgroup = "")
invisible(readline(prompt = "Press [enter] to continue"))

tl_no_units <- list()
tl_no_units[["Gas (mile/gallons)"]] <-
  getDescriptionStatsBy(mtcars$mpg, mtcars$am,
                        header_count = TRUE)
tl_no_units[["Weight (10<sup>3</sup> kg)"]] <-
  getDescriptionStatsBy(mtcars$wt, mtcars$am,
                        header_count = TRUE)
mergeDesc(tl_no_units,
          tll) %>%
  htmlTable::addHtmlTableStyle(css.rgroup = "")
invisible(readline(prompt = "Press [enter] to continue"))

# Other settings
mtcars$mpg[sample(1:NROW(mtcars), size = 5)] <- NA
getDescriptionStatsBy(mtcars$mpg,
                      mtcars$am,
                      statistics = TRUE)
invisible(readline(prompt = "Press [enter] to continue"))

# Do the horizontal version
getDescriptionStatsBy(mtcars$gear,
                      mtcars$am,
                      statistics = TRUE,
                      hrzl_prop = TRUE)
invisible(readline(prompt = "Press [enter] to continue"))

mtcars$wt_with_missing <- mtcars$wt
mtcars$wt_with_missing[sample(1:NROW(mtcars), size = 8)] <- NA
getDescriptionStatsBy(mtcars$wt_with_missing, mtcars$am, statistics = TRUE,
                      hrzl_prop = TRUE, total_col_show_perc = FALSE)
invisible(readline(prompt = "Press [enter] to continue"))

\dontrun{
  ## There is also a LaTeX wrapper
  tll <- list(
    getDescriptionStatsBy(mtcars$gear, mtcars$am),
    getDescriptionStatsBy(mtcars$col, mtcars$am))

  latex(mergeDesc(tll),
        caption  = "Factored variables",
        file = "")
}

#' Get statistics for missing data
#'
#' This function calculates the amount of missing per row for \code{\link{describeMean}},
#' \code{\link{describeMedian}} and custom description functions.
#' It will return invisibly when no missing values are present.
#'
#' @inheritParams describeMean
#' @return \code{vector} A vector with the missing estimate
#' @export
descGetMissing <- function(x,
                           html = TRUE,
                           number_first = TRUE,
                           percentage_sign = TRUE,
                           language = "en",
                           useNA.digits = 1,
                           ...) {
  if (!any(is.na(x))) {
    return(invisible())
  }
  df_arg_list <- list(
    x = is.na(x),
    html = html,
    number_first = number_first,
    percentage_sign = percentage_sign,
    language = language,
    digits = useNA.digits
  )
  dot_args <- list(...)
  for (n in names(dot_args)) {
    if (!n %in% names(df_arg_list)) {
      df_arg_list[[n]] <- dot_args[[n]]
    }
  }
  missing <- fastDoCall(describeFactors, df_arg_list)
  return(missing["TRUE", ])
}

fn <- system.file("extdata", "Full_test_suite.Rmd", package = "Gmisc")
if (!file.exists(fn))
  stop("Can't find the test file")

if (FALSE) {
  # This isn't allowed in read-only file-systems - see https://github.com/gforge/Gmisc/issues/36
  td <- tempdir()
  rmarkdown::render(fn, output_file = file.path(td, "tmp.html"))
  
  unlink(c(file.path(td, "docx.css"),
           file.path(td, "tmp.html"),
           file.path(td, "tmp_files")), 
         recursive = TRUE)
  
  additional_deletes <- c()
  fn <- system.file("extdata", "Full_test_suite_files", package = "Gmisc")
  if (dir.exists(fn))
    additional_deletes <- c(additional_deletes, fn)
  
  fn <- system.file("extdata", "Full_test_suite.html", package = "Gmisc")
  if (file.exists(fn))
    additional_deletes <- c(additional_deletes, fn)
  
  fn <- system.file("extdata", "docx.css", package = "Gmisc")
  if (file.exists(fn))
    additional_deletes <- c(additional_deletes, fn)
  
  if (length(additional_deletes) > 0)
    unlink(additional_deletes, recursive = TRUE)
}

#' Prepares a matrix for \code{htmlTable} from a list
#'
#' By putting all the output from the \code{\link{getDescriptionStatsBy}}
#' into a list, naming each element that we want in an \code{rgroup} we can
#' automatically merge everything and create an object ready for the
#' \code{\link[htmlTable]{htmlTable}}.
#'
#' @section The \code{rgroup} value:
#'
#' The value for the \code{rgroup} is by default the name of the list element. If you have
#' passed a list without a name for that particular element or if you have passed a
#' matrix it will look for a label set by the \pkg{Hmisc}\code{::\link[Hmisc]{label}} function.
#' For those elements that have only one row no \code{rgroup} is set, and the naming sequence
#' is the same as above but with an additional \code{\link[base:colnames]{rownames}} if the previous
#' two turn out empty. All this behavior is exemplified in the example.
#'
#' The \code{rgroup} value can be overridden by simply specifying a custom \code{rgroup} when
#' calling the \code{\link[htmlTable]{htmlTable}} function.
#'
#' @section The \code{colnames} of the matrix:
#'
#' The function chooses the \code{\link[base]{colnames}} from the first element in
#' the \code{tlist}.
#'
#' @param ... One or more elements coming from \code{\link{getDescriptionStatsBy}}.
#'  You can also provide pure output from the \code{\link{getDescriptionStatsBy}} function
#'  and have the function merge this together with the \code{...} argument.
#'  \emph{Note} that all elements must have the same \code{by} argument or you
#'  will not be able to merge it into a list.
#' @param htmlTable_args Any arguments that should be passed to
#'  \code{\link[htmlTable]{htmlTable}} function. The default is to remove
#'  any css formatting for the \code{rgroup}.
#' @return \code{matrix} Returns a matrix object of class descList
#' @export
#' @example inst/examples/getDescriptionStatsBy_example.R
#' @importFrom utils tail
#' @family table functions
mergeDesc <- function(..., htmlTable_args = list()) {
  tlist <- list()
  dots <- list(...)
  if (length(dots) > 0) {
    for (i in 1:length(dots)) {
      add_lst <- dots[[i]]
      if (!is.list(add_lst)) {
        add_lst <- list(add_lst)
        if (!is.null(names(dots)) &&
          names(dots)[i] != "") {
          names(add_lst) <- names(dots)[i]
        }
      }

      tlist <- append(
        tlist,
        add_lst
      )
    }
  }

  mx <- NULL
  rgroup <- n.rgroup <- c()
  pvals_rgroup <- list()
  for (i in 1:length(tlist)) {
    n <- names(tlist)[i]
    if (is.null(n) || n == "") {
      if (label(tlist[[i]]) != "") {
        n <- label(tlist[[i]])
      } else if (nrow(tlist[[i]]) == 1) {
        n <- rownames(tlist[[i]])
      } else {
        n <- ""
      }
    }

    mx <- rbind(
      mx,
      tlist[[i]]
    )
    if (nrow(tlist[[i]]) > 1) {
      rgname <- n
      rgno <- nrow(tlist[[i]])
      if (tolower(tail(colnames(mx), 1)) == "p-value") {
        pval <- list(mx[
          nrow(mx) - nrow(tlist[[i]]) + 1,
          ncol(mx)
        ])
        mx[
          nrow(mx) - nrow(tlist[[i]]) + 1,
          ncol(mx)
        ] <- ""
        names(pval)[1] <- as.character(i)
        pvals_rgroup <- c(
          pvals_rgroup,
          pval
        )
      }
    } else {
      rownames(mx)[NROW(mx)] <- n
      rgname <- ""
      rgno <- 1
    }

    if (rgno != "") {
      rgroup <- c(
        rgroup,
        rgname
      )
      n.rgroup <- c(
        n.rgroup,
        rgno
      )
    } else {
      if (length(rgroup) == 0) {
        rgroup <- ""
        n.rgroup <- rgno
      } else {
        if (tail(rgroup, 1) == rgname) {
          n.rgroup[length(n.rgroup)] <-
            n.rgroup[length(n.rgroup)] + rgno
        } else {
          rgroup <- c(
            rgroup,
            rgname
          )
          n.rgroup <- c(
            n.rgroup,
            rgno
          )
        }
      }
    }
  }

  colnames(mx) <- colnames(tlist[[1]])

  if (any(rgroup %in% names(htmlTable_args)) ||
    any(n.rgroup %in% names(htmlTable_args))) {
    stop(
      "You have provided rgroup elements within",
      " htmlTable_args argument and these will conflict",
      " with the autogenerated rgroups."
    )
  }
  if ("rgroup" %in% names(htmlTable_args) &
    "n.rgroup" %in% names(htmlTable_args)) {
    rgroup <- htmlTable_args$rgroup
    n.rgroup <- htmlTable_args$n.rgroup
    htmlTable_args$rgroup <- NULL
    htmlTable_args$n.rgroup <- NULL
    if ("P-value" %in% colnames(mx) & length(pvals_rgroup) != length(rgroup)) {
      replacement_pvals <- rep(NA, length(rgroup))
      replacement_pvals[n.rgroup > 1] <- pvals_rgroup
      replacement_pvals[n.rgroup == 1] <- mx[, "P-value"][mx[, "P-value"] != ""]
      mx[, "P-value"] <- ""
      pvals_rgroup <- replacement_pvals
    }
  }
  if (length(pvals_rgroup) > 0) {
    attr(rgroup, "add") <- pvals_rgroup
  }
  if (length(htmlTable_args) > 0 && is.null(names(htmlTable_args))) {
    stop("The htmlTable_args has to be a list or a vector with named elements")
  }
  if (!is.list(htmlTable_args)) {
    htmlTable_args <- as.list(htmlTable_args)
  }

  structure(mx,
    rgroup = rgroup,
    n.rgroup = n.rgroup,
    htmlTable_args = htmlTable_args,
    class = c("descMrg", class(mx))
  )
}

#' Collection of functions for plotting relations, generating tables, and more.
#'
#' This is a collection of functions that I've found useful in my research.
#' The package is inspired by Frank Harrell's \pkg{Hmisc} package. The main focus
#' is on tables, plots, and \pkg{knitr}-integration.
#'
#' @section Awesome tables:
#'
#' The \code{\link{getDescriptionStatsBy}} is a straightforward function that
#' aims at helping you to generate a descriptive table stratified by different
#' variables. In other words, the function returns everything you need for generating
#' a \emph{Table 1} ready for publication. This function is accompanied by the
#' \code{\link{describeMean}}, \code{\link{describeMedian}}, \code{\link{describeProp}},
#' and \code{\link{describeFactors}} functions.
#'
#' The \code{\link{mergeDesc}} allows you to merge a set of outputs \code{\link{getDescriptionStatsBy}}
#' into a \code{\link[htmlTable]{htmlTable}} with the rgroup arguments automatically
#' generated, see \code{vignette("descriptives", package = "Gmisc")} for a
#' detailed workflow description.
#'
#' @section Some fancy plots:
#'
#' The transition plot function, \code{\link{transitionPlot}}, is for
#' descriptive purposes. It tries to illustrate the size of change
#' between one state and the next, i.e. a transition. This is
#' basically a graph of based upon \code{table(var1, var2)}.
#'
#' The \href{https://en.wikipedia.org/wiki/Singular_value_decomposition}{Singular value decomposition}
#' is a common method for reducing the number of variables. Unfortunately
#' this compression can reduce the interpretability of the model. The \code{\link{getSvdMostInfluential}}
#' function tries to remedy that by identifying the most influential
#' elements from the \code{V}-matrix.
#'
#' @section Other stuff:
#'
#' The \code{\link{insertRowAndKeepAttr}} simply adds a row while remembering
#' all the attributes previously set by using the \code{\link{copyAllNewAttributes}}.
#' The \code{\link{mergeLists}} tries to merge lists that do not have identical
#' elements.
#'
#' @name Gmisc-package
"_PACKAGE"

# Helpers to resolve and validate .subelement targets for align* functions

# Find a target element by path in either the top-level list or the first nested
# container, returning a list with elements: target, container_is_first (logical)
prFindSubelementTarget <- function(boxes2align, path) {
    target <- get_list_element_by_path(boxes2align, path)
    container_is_first <- FALSE

    # Fallback 1: path may refer to named element at top-level with subpath
    if (is.null(target) && length(path) >= 2 && is.list(boxes2align) && !is.null(names(boxes2align)) && path[1] %in% names(boxes2align)) {
        candidate <- boxes2align[[path[1]]]
        if (!is.null(candidate) && is.list(candidate)) {
            target2 <- get_list_element_by_path(candidate, path[-1])
            if (!is.null(target2)) {
                return(list(target = target2, container_is_first = FALSE))
            }
        }
    }

    # Fallback 2: try inside the common nested first element (maintain old behavior)
    if (is.null(target) && length(boxes2align) > 0 && is.list(boxes2align[[1]]) && !inherits(boxes2align[[1]], "box")) {
        target2 <- get_list_element_by_path(boxes2align[[1]], path)
        if (!is.null(target2)) {
            return(list(target = target2, container_is_first = TRUE))
        }
    }

    list(target = target, container_is_first = container_is_first)
}

testthat::test_that("prFindSubelementTarget finds in top-level and nested container", {
    nested <- list(a = list(b = list("x")), c = 1)
    res1 <- Gmisc:::prFindSubelementTarget(nested, c("a", "b"))
    testthat::expect_true(!is.null(res1$target))
    testthat::expect_false(res1$container_is_first)

    # not found in top-level, but in nested first element
    containers <- list(list(sub = list("A")), other = list())
    res2 <- Gmisc:::prFindSubelementTarget(containers, c("sub"))
    testthat::expect_true(!is.null(res2$target))
    testthat::expect_true(res2$container_is_first)

    # missing path
    res3 <- Gmisc:::prFindSubelementTarget(nested, c("no", "pe"))
    testthat::expect_null(res3$target)
})

#' Label connector grobs
#'
#' Place text labels at the midpoint of connector grobs returned by
#' `connectGrob()` (useful for multi-target connectors where `label=` isn't
#' available). By default the function draws the labels; set `draw = FALSE` to
#' only return the `textGrob` objects for programmatic use.
#'
#' @param con A list of connector grobs produced by `connectGrob()` (typically
#'   a `connect_boxes_list`).
#' @param labels A character vector of labels (recycled if necessary).
#' @param x_offset,y_offset `grid::unit` offsets added to the label position.
#' @param gp A `grid::gpar` object controlling text appearance.
#' @param bg_gp A `grid::gpar` object controlling label background appearance.
#' @param pad A `grid::unit` padding to add around text inside the background box.
#' @return A `Gmisc_connector_label` S3 object (invisible). Use `print()` to draw the labels.
#' @examples
#' # Attach labels to a previously-created connector and draw them
#' a <- boxDiamondGrob("D")
#' b <- boxEllipseGrob("L")
#' c <- boxServerGrob("S")
#' boxes <- list(decision = a, outcomes = list(b, c)) |>
#'   spreadHorizontal(
#'     from = grid::unit(.1, "npc"),
#'     to = grid::unit(.9, "npc"),
#'     subelement = "outcomes"
#'   ) |>
#'   spreadVertical()
#'
#' connectGrob(boxes$decision, boxes$outcomes, type = "N") |>
#'   labelConnector(c("Local", "Server"))
#' @export
labelConnector <- function(
  con,
  labels,
  x_offset = unit(0, "mm"),
  y_offset = unit(0, "mm"),
  gp = gpar(cex = 0.9),
  bg_gp = gpar(fill = "white", col = NA),
  pad = unit(2, "mm")
) {
  if (!is.list(con)) stop("'con' must be a list of connector grobs")

  n <- length(con)
  if (n == 0) {
    out <- list(
      labels = character(0),
      texts = list(),
      bgs = list(),
      x = numeric(0),
      y = numeric(0),
      gp = gp,
      bg_gp = bg_gp,
      pad = pad,
      offsets = list(x_offset = x_offset, y_offset = y_offset)
    )
    class(out) <- "Gmisc_connector_label"
    return(invisible(out))
  }

  labels <- rep_len(as.character(labels), n)

  texts <- vector("list", n)
  bgs <- vector("list", n)
  xs <- numeric(n)
  ys <- numeric(n)

  for (i in seq_len(n)) {
    g <- con[[i]]
    if (is.null(attr(g, "line"))) stop("connector grob missing 'line' attribute")
    ln <- attr(g, "line")

    # convert coords to npc numeric values (works for mixed unit objects)
    xvals <- vapply(ln$x, \(u) prGetNpcValue(u, "x"), numeric(1))
    yvals <- vapply(ln$y, \(u) prGetNpcValue(u, "y"), numeric(1))

    # take midpoint (mean of numeric ranges) as label anchor
    x_mid <- mean(range(xvals, na.rm = TRUE))
    y_mid <- mean(range(yvals, na.rm = TRUE))

    x_unit <- unit(x_mid, "npc") + x_offset
    y_unit <- unit(y_mid, "npc") + y_offset

    tg <- textGrob(labels[i], x = x_unit, y = y_unit, gp = gp)

    # background rounded rect behind text
    w <- grobWidth(tg) + pad
    h <- grobHeight(tg) + pad
    bg <- roundrectGrob(x = x_unit, y = y_unit, width = w, height = h, r = unit(1, "mm"), gp = bg_gp)

    texts[[i]] <- tg
    bgs[[i]] <- bg
    xs[i] <- x_mid
    ys[i] <- y_mid
  }

  structure(
    list(
      labels = labels,
      texts = texts,
      bgs = bgs,
      x = xs,
      y = ys,
      gp = gp,
      bg_gp = bg_gp,
      pad = pad,
      offsets = list(x_offset = x_offset, y_offset = y_offset)
    ),
    class = c("Gmisc_connector_label", class(list()))
  )
}


#' Print a connector label object
#'
#' @param x A `Gmisc_connector_label` object
#' @param ... ignored
#' @export
print.Gmisc_connector_label <- function(x, ...) {
  # draw each background then text on top
  for (i in seq_along(x$texts)) {
    grid.draw(x$bgs[[i]])
    grid.draw(x$texts[[i]])
  }
  invisible(x)
}


#' Attach labels to a connector object
#'
#' Store labels and presentation options on a connector object so labels are
#' automatically drawn when the connector is printed.
#'
#' @param con A connector (a `connect_boxes` grob or `connect_boxes_list`).
#' @param labels Character vector of labels (recycled to match connector length).
#' @param gp A `gpar` for label text.
#' @param bg_gp A `gpar` for label background.
#' @param x_offset Unit offsets for label placement.
#' @param y_offset Unit offsets for label placement.
#' @return The connector object with attributes set (invisibly).
#' @export
setConnectorLabels <- function(
  con,
  labels,
  gp = gpar(cex = 0.9),
  bg_gp = gpar(fill = "white", col = NA),
  x_offset = unit(0, "mm"),
  y_offset = unit(0, "mm")
) {
  if (!is.list(con)) stop("'con' must be a connector list or single connector")
  n <- length(con)
  attr(con, "connector_labels") <- rep_len(as.character(labels), n)
  attr(con, "connector_label_gp") <- gp
  attr(con, "connector_label_bg_gp") <- bg_gp
  attr(con, "connector_label_offset") <- list(x_offset = x_offset, y_offset = y_offset)
  structure(
    con,
    class = "Gmisc_extended_connector_labels"
  )
}


#' @export
print.Gmisc_extended_connector_labels <- function(x, ...) {
  # Draw the underlying connector(s)
  if (is.list(x)) {
    for (g in x) grid.draw(g, ...)
  } else {
    grid.draw(x, ...)
  }

  # If labels are present, render them using the existing labelConnector helper
  labels <- attr(x, "connector_labels")
  if (!is.null(labels)) {
    gp <- attr(x, "connector_label_gp")
    if (is.null(gp)) gp <- gpar(cex = 0.9)
    bg_gp <- attr(x, "connector_label_bg_gp")
    if (is.null(bg_gp)) bg_gp <- gpar(fill = "white", col = NA)
    off <- attr(x, "connector_label_offset")
    if (is.null(off)) off <- list(x_offset = unit(0, "mm"), y_offset = unit(0, "mm"))

    lbl_obj <- labelConnector(
      x,
      labels = labels,
      x_offset = off$x_offset,
      y_offset = off$y_offset,
      gp = gp,
      bg_gp = bg_gp
    )
    print(lbl_obj)
  }

  invisible(x)
}

## -----------------------------------------------------------------------------
library(magrittr)
set.seed(1)
n <- 100
data <- 
  data.frame(
    Sex = sample(c("Male", "Female"),
                 size = n,
                 replace = TRUE,
                 prob = c(.4, .6)),
    Charnley_class = sample(c("A", "B", "C"), 
                            size = n, 
                            replace = TRUE))
getProbs <- function(Chrnl_name){
  prob <- data.frame(
    A = 1/3 +
      (data$Sex == "Male") * .25 +
      (data$Sex != "Male") * -.25 +
      (data[[Chrnl_name]] %in% "B") * -.5 +
      (data[[Chrnl_name]] %in% "C") * -2 ,
    B = 1/3 +
      (data$Sex == "Male") * .1 + 
      (data$Sex != "Male") * -.05 +
      (data[[Chrnl_name]] == "C") * -2,
    C = 1/3 +
      (data$Sex == "Male") * -.25 +
      (data$Sex != "Male") * .25)
  
  # Remove negative probabilities
  t(apply(prob, 1, function(x) {
    if (any(x < 0)){
      x <- x - min(x) + .05
      }
    x
    }))
}
  
Ch_classes <- c("Charnley_class")
Ch_classes %<>% c(sprintf("%s_%dyr", Ch_classes, c(1,2,6)))
for (i in 1:length(Ch_classes)){
  if (i == 1)
    next;

  data[[Ch_classes[i]]] <- 
    apply(getProbs(Ch_classes[i-1]), 1, function(p)
      sample(c("A", "B", "C"), 
             size = 1, 
             prob = p)) %>%
    factor(levels = c("A", "B", "C"))
}

## ----echo=FALSE---------------------------------------------------------------
knitr::opts_chunk$set(message = FALSE, 
                      warnings = FALSE)
knitr::opts_chunk$set(fig.height = 5, fig.width = 5)

## -----------------------------------------------------------------------------
library(Gmisc)
transitions <- table(data$Charnley_class, data$Charnley_class_1yr) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"))
transitions$render()

## -----------------------------------------------------------------------------
transitions <- table(data$Charnley_class, data$Charnley_class_1yr) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"),
                                # Control the box text via grid::gpar()
                                txt_gpar = grid::gpar(fontfamily = "serif"))
transitions$title <- "Charnley class in relation to THR"
transitions$arrow_type <- "simple"
transitions$box_label_pos <- "bottom"
transitions$render()

## ----color_transition---------------------------------------------------------
transitions <- table(data$Charnley_class, data$Charnley_class_1yr, data$Sex) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"))
transitions$title <- "Charnley class in relation to THR"
transitions$clr_bar <- "bottom"
transitions$render()

## ----multiple_transitions, fig.height = 5, fig.width = 7----------------------
transitions <- table(data$Charnley_class, data$Charnley_class_1yr, data$Sex) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"))
transitions$title <- "Charnley class in relation to THR"
transitions$arrow_type <- "simple"
table(data$Charnley_class_1yr, data$Charnley_class_2yr, data$Sex) %>%
  transitions$addTransitions(label = "2 years after")
library(grid)
transitions$max_lwd <- unit(.05, "npc")
transitions$render()

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Gets offsetted lines
#'
#' The function calculates new points according to the offset
#' that lie to the left/right of the provided line.
#'
#' @param x A numeric vector containing all the x-elements
#' @param y A numeric vector containing all the y-elements
#' @param offset The offset to add to the line, can be a vector if you
#'  want to use different offsets.
#' @param end_x The x end of the line where the arrow occurrs (if < 0 arrow is skipped)
#' @param end_y The y end of the line where the arrow occurrs (if < 0 arrow is skipped)
#' @param arrow_offset The offset to add to the arrow section if any (if <= 0 arrow is skipped)
#' @param rm_intersect Set to 0 if you want to skip intersection removal, 1 only to remove left or
#'  2 to only remove right. See details for why.
#'
#'  @section Remove intersections:
#'
#'  When the line is wide and the arrow has a narrow curve there may appear an empty triangle due
#'  to polygon cancellation (two polygons within the same are cancel out). This behaviour may be
#'  ugly and the function therefor tries to remove these.
#'
#'  \emph{Note:} it is expensive to check if there are the lineas may intersect at one point,
#'  remove those unexpected, and then adjust the line to the new situation so that the
#'  top and bottom lines match. It can also cause some unexpected behaviour why you may want to
#'  remove this feature if the arrow behaves erratically.
#'
#' @return \code{list(list(x = ..., y = ...))} Returns a list with the right/left
#'  lines that in turn lists with \emph{x} and \emph{y} elements
#' @useDynLib Gmisc
#' @importFrom Rcpp evalCpp
calculateLinesAndArrow <- function(x, y, offset, end_x = -1, end_y = -1, arrow_offset = -1, rm_intersect = 3L) {
    .Call('_Gmisc_calculateLinesAndArrow', PACKAGE = 'Gmisc', x, y, offset, end_x, end_y, arrow_offset, rm_intersect)
}

#' Generates a generalized Bézier line
#'
#' This is a general form of bezier line that can be used for cubic, quadratic,
#' and more advanced Bézier lines.
#'
#' @param ctrl_points The ctrl_points for the bezier control points. This should
#'  either be a \code{matrix} or a \code{data.frame}.
#' @param length_out The length of the return points, i.e. how fine
#'  detailed the points should be.
#'
#' @export
#' @examples
#' library(grid)
#' grid.newpage()
#' l <- gnrlBezierPoints(data.frame(x = c(.1, -.1, .7, 1, 1, 0.1),
#'                                  y = c(.9, 0, 1, .8, .4, .1)),
#'                       length_out = 100)
#' grid.lines(l[,1], l[,2], gp=gpar(col="#550000", lwd = 4))
#'
#' out_sizes <- 4:20
#' clrs <- colorRampPalette(c("orange", "darkblue"))(length(out_sizes))
#' for (i in out_sizes){
#'    l <- gnrlBezierPoints(data.frame(x = c(.1, -.1, .7, 1, 1, 0.1),
#'                                     y = c(.9, 0, 1, .8, .4, .1)),
#'                          length_out = i)
#'    grid.lines(l[,1], l[,2],
#'    gp=gpar(col=clrs[which(i == out_sizes)]))
#' }
gnrlBezierPoints <- function(ctrl_points, length_out = 100L) {
    .Call('_Gmisc_gnrlBezierPoints', PACKAGE = 'Gmisc', ctrl_points, length_out)
}

#' Gets the bezier points adjusted for an arrow
#'
#' @param x The x start and end points
#' @param y The spline control points
#' @param arrow_length The desired length of the arrow
#' @param length_out Increases the resolution for the final bezier
#'  points, i.e. generating more fine-grained intervals
#' @return list
#'
#' @import magrittr
#' @importFrom utils tail
getBezierAdj4Arrw <- function(x, y, arrow_length, length_out = 100) {
  org <- data.frame(x = x, y = y)
  true_bezier <- gnrlBezierPoints(cbind(x, y), length_out = length_out)
  true_bezier <- list(
    x = true_bezier[, 1],
    y = true_bezier[, 2]
  )
  true_bezier$distance <-
    with(
      true_bezier,
      sqrt((x - tail(x, 1))^2 + (y - tail(y, 1))^2)
    )
  if (missing(arrow_length) ||
    arrow_length == 0) {
    return(structure(true_bezier,
      true_bezier = true_bezier,
      cut_point = NA,
      spline_ctrl = org
    ))
  }

  # The distance between the tail and the second last spline point is indicative
  # of how strong the adjustment should be
  spl_len <- sqrt((tail(x, 1) - tail(x, 2)[1])^2 +
    (tail(y, 1) - tail(y, 2)[1])^2)
  if (spl_len < arrow_length * 3 || arrow_length == 0) {
    mult <- 1
  } else {
    mult <- spl_len / arrow_length
  }

  if (length(x) >= 4) {
    # The strength of the previous element is also of importance
    # A weak second element will cause the line to be more dependent
    # on the first element
    scnd_spl_len <- sqrt((tail(x, 3)[1] - tail(x, 4)[1])^2 +
      (tail(y, 3)[1] - tail(y, 4)[1])^2)
    if (scnd_spl_len < arrow_length * 3) {
      mult <- spl_len / (arrow_length * 2)
    }
  }

  cut_point <- which.min(abs(true_bezier$distance - arrow_length))

  dx <- tail(x, 1) - true_bezier$x[cut_point]
  dy <- tail(y, 1) - true_bezier$y[cut_point]

  # Fancy fixing of arrows fails. We therefore only to adjust last two points
  retain <- 1:(length(x) - 2)
  x <- x[retain]
  y <- y[retain]

  x <- c(x, true_bezier$x[cut_point] - dx * mult * .9)
  y <- c(y, true_bezier$y[cut_point] - dy * mult * .9)

  x <- c(x, true_bezier$x[cut_point])
  y <- c(y, true_bezier$y[cut_point])

  adjusted_bp <- gnrlBezierPoints(cbind(x, y), length_out = length_out)
  adjusted_bp <- list(
    x = adjusted_bp[, 1],
    y = adjusted_bp[, 2]
  )

  structure(adjusted_bp,
    true_bezier = true_bezier,
    cut_point = cut_point,
    spline_ctrl = list(x = x, y = y)
  )
}


#' Checks the input of a vector
#'
#' It checks that a vector makes sense in its
#' grid parameters. It also creates an origo
#' at point 0 if none is provided.
#'
#' @param x The x point of the vector
#' @param y The y point of the vector
#' @param x_origo The x origin if other than 0
#' @param y_origo The y origin if other than 0
#' @return list with the input variables checked and
#'  converted if necessary.
#'
#' @keywords internal
validateAndConvertVectorInputs <- function(x, y,
                                           x_origo, y_origo) {
  if (missing(x_origo) != missing(y_origo)) {
    stop("You must specify both origo points!")
  }

  if (missing(x) || missing(y)) {
    stop("You must specify both x and y points!")
  }

  # Just som sanity input check
  if (!identical(class(y), class(x))) {
    stop(
      "The x and y point don't have the same class,",
      " should be either numeric or units.",
      " Currently you have provided y = ", class(y), " & x = ", class(x)
    )
  }


  if (missing(x_origo)) {
    if (inherits(y, "unit")) {
      y_origo <- x_origo <- unit(0, attr(y, "unit"))
    } else {
      x_origo <- y_origo <- 0
    }
  }

  if (!setequal(class(y_origo), class(x_origo))) {
    stop(
      "The x and y point for the origo point don't have the same class,",
      " should be either numeric or units.",
      " Currently you have provided y = ", class(y_origo),
      " & x = ", class(x_origo)
    )
  }

  if (!setequal(class(y), class(y_origo))) {
    stop(
      "The angle won't make any sense if your x and y point",
      " don't have the same unit as the origo x and y point.",
      " Currently you have provided point class = ", class(y),
      " & origo class = ", class(y_origo)
    )
  }

  return(list(
    y = y,
    x = x,
    y_origo = y_origo,
    x_origo = x_origo
  ))
}

#' Translates "npc" widths into absolute units
#'
#' The "npc" \code{\link[grid]{unit}} is not absolute, i.e.
#' it depends on the relation between height and  width within
#' the current viewport. We therefore need to change these values
#' to the value most likely intended. This is believed to be the
#' main direction of the arrow as specified by the first and last
#' bezier control points \code{x} and \code{y}.
#'
#' @inheritParams bezierArrowSmpl
#' @param w The width that is to be changed
#' @return The width transformed into a \code{\link[grid]{unit}} of "mm"
#' @keywords internal
#' @importFrom utils tail
getAbsoluteWidth <- function(w, default.units, x = x, y = y) {
  if (!inherits(w, "unit")) {
    w <- unit(w, default.units)
  }

  horizontal <- (getGridVal(x[1], default.units = "mm", axisTo = "x") -
    getGridVal(tail(x, 1), default.units = "mm", axisTo = "x")) >
    (getGridVal(y[1], default.units = "mm", axisTo = "y") -
      getGridVal(tail(y, 1), default.units = "mm", axisTo = "y"))

  if (attr(w, "unit") == "npc") {
    if (!horizontal) {
      w <- unit(getGridVal(w, default.units = "mm", axisTo = "y"), "mm")
    } else {
      w <- unit(getGridVal(w, default.units = "mm", axisTo = "x"), "mm")
    }
  }
  return(w)
}

#' Gets an angle
#'
#' Uses a vector to get an angle by \code{\link{atan2}}.
#'
#' @param x The x point of the vector
#' @param y The y point of the vector
#' @param x_origo The x origin if other than 0
#' @param y_origo The y origin if other than 0
#' @return angle in radians (see \code{\link{atan2}})
#'
#' @keywords internal
getVectorAngle <- function(x, y,
                           x_origo = NA, y_origo = NA) {
  v <- validateAndConvertVectorInputs(
    x = x, y = y,
    x_origo = x_origo, y_origo = y_origo
  )

  if ("unit" %in% class(v$y)) {
    y_diff <- convertY(v$y - v$y_origo, unitTo = "mm", valueOnly = TRUE)
    x_diff <- convertX(v$x - v$x_origo, unitTo = "mm", valueOnly = TRUE)
  } else {
    y_diff <- v$y - v$y_origo
    x_diff <- v$x - v$x_origo
  }

  return(atan2(y = y_diff, x = x_diff))
}

#' Just a simple help with interpreting the radians
#'
#' @param angle_radian The angle in radians
#' @return boolean
#'
#' @keywords internal
isHorizontal <- function(angle_radian) {
  angle_prop <- abs(angle_radian / pi * 180)
  if (angle_prop < 45 || angle_prop > 135) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

# Start with the easy part, cutting off excess line
shortenLine <- function(x, y,
                        ref_x, ref_y,
                        internal.units,
                        axis) {
  if (axis == "y") {
    if (x[1] < x[2]) {
      keep <- which(x > ref_x)[1]:length(x)
    } else {
      keep <- which(x < ref_x)[1]:length(x)
    }
  } else {
    if (y[1] < y[2]) {
      keep <- which(y > ref_y)[1]:length(y)
    } else {
      keep <- which(y < ref_y)[1]:length(y)
    }
  }

  x <- c(rep(ref_x, keep[1] - 1 + 10), x[keep])
  y <- c(rep(ref_y, keep[1] - 1 + 10), y[keep])

  return(list(x = x, y = y))
}


# Adds the missing piece by generating another bezier curve
# for that specific section
extendLine <- function(x, y,
                       ref_x, ref_y,
                       internal.units,
                       axis) {
  distanceX <- x[1] - ref_x
  distanceY <- y[1] - ref_y

  # Generate a grob for the remaining spline
  ctrl_x <- c(x[1], ref_x, ref_x)
  ctrl_y <- c(y[1], ref_y, ref_y)

  add_bg_pt <- gnrlBezierPoints(cbind(ctrl_x, ctrl_y), length_out = 10)
  x <- c(rev(add_bg_pt[, 1]), x)
  y <- c(rev(add_bg_pt[, 2]), y)
  return(list(x = x, y = y))
}

align2Axis <- function(bp, lines, width, internal.units, axis) {
  if (!axis %in% c("x", "y")) {
    angle <- getVectorAngle(
      x_origo = bp$x[1],
      y_origo = bp$y[1],
      x = bp$x[2],
      y = bp$y[2]
    )
    if (isHorizontal(angle)) {
      axis <- "y"
    } else {
      axis <- "x"
    }
  }

  adaptLine_SL_ER <- function(lines,
                              org_left, org_right,
                              internal.units, axis) {
    # left is shorter due to the left skew
    lines$left <- shortenLine(
      x = lines$left$x, y = lines$left$y,
      ref_x = org_left[["x"]],
      ref_y = org_left[["y"]],
      internal.units = internal.units,
      axis = axis
    )
    lines$right <- extendLine(
      x = lines$right$x, y = lines$right$y,
      ref_x = org_right[["x"]],
      ref_y = org_right[["y"]],
      internal.units = internal.units,
      axis = axis
    )
    return(lines)
  }

  adaptLine_SR_EL <- function(lines,
                              org_left, org_right,
                              internal.units, axis) {
    # left is shorter due to the left skew
    lines$right <- shortenLine(
      x = lines$right$x, y = lines$right$y,
      ref_x = org_right[["x"]],
      ref_y = org_right[["y"]],
      internal.units = internal.units,
      axis = axis
    )
    lines$left <- extendLine(
      x = lines$left$x, y = lines$left$y,
      ref_x = org_left[["x"]],
      ref_y = org_left[["y"]],
      internal.units = internal.units,
      axis = axis
    )
    return(lines)
  }

  dy <- bp$y[2] - bp$y[1]
  dx <- bp$x[2] - bp$x[1]
  if (axis == "x") {
    # If the line is straight no changes necessary
    if (abs(dx) > .Machine$double.eps * 10^2) {
      if (dx > 0) {
        if (dy < 0) {
          org_left <- c(x = bp$x[1] + width / 2, y = bp$y[1])
          org_right <- c(x = bp$x[1] - width / 2, y = bp$y[1])
          lines <- adaptLine_SL_ER(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        } else {
          org_left <- c(x = bp$x[1] - width / 2, y = bp$y[1])
          org_right <- c(x = bp$x[1] + width / 2, y = bp$y[1])
          lines <- adaptLine_SR_EL(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        }
      } else {
        if (dy < 0) {
          org_left <- c(x = bp$x[1] + width / 2, y = bp$y[1])
          org_right <- c(x = bp$x[1] - width / 2, y = bp$y[1])
          lines <- adaptLine_SR_EL(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        } else {
          org_left <- c(x = bp$x[1] - width / 2, y = bp$y[1])
          org_right <- c(x = bp$x[1] + width / 2, y = bp$y[1])
          # Left turn
          lines <- adaptLine_SL_ER(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        }
      }
    }
  } else {
    if (abs(dy) > .Machine$double.eps) {
      if (dx > 0) {
        if (dy < 0) {
          org_left <- c(x = bp$x[1], y = bp$y[1] + width / 2)
          org_right <- c(x = bp$x[1], y = bp$y[1] - width / 2)
          lines <- adaptLine_SR_EL(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        } else {
          org_left <- c(x = bp$x[1], y = bp$y[1] + width / 2)
          org_right <- c(x = bp$x[1], y = bp$y[1] - width / 2)
          lines <- adaptLine_SL_ER(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        }
      } else {
        if (dy < 0) {
          org_left <- c(x = bp$x[1], y = bp$y[1] - width / 2)
          org_right <- c(x = bp$x[1], y = bp$y[1] + width / 2)
          lines <- adaptLine_SL_ER(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        } else {
          org_left <- c(x = bp$x[1], y = bp$y[1] - width / 2)
          org_right <- c(x = bp$x[1], y = bp$y[1] + width / 2)
          lines <- adaptLine_SR_EL(
            lines = lines,
            org_left = org_left,
            org_right = org_right,
            internal.units = internal.units,
            axis = axis
          )
        }
      }
    }
  }

  attr(lines, "axis") <- axis
  return(lines)
}
#' Gets grid value
#'
#' Returns the raw value in units if the provided is of type unit else
#' it returns the default unit.
#'
#' @param x Value
#' @param default.units The unit type
#' @param axisTo The axis that is used, useful for "npc" where
#'  there is a big difference in height and width
#' @return float
#'
#' @keywords internal
getGridVal <- function(x, default.units, axisTo = "x") {
  if ("unit" %in% class(x)) {
    return(convertUnit(x, unitTo = default.units, valueOnly = TRUE, axisTo = axisTo))
  } else {
    return(x)
  }
}

#' Converts an object to coordinates
#'
#' Sometimes we have an object that can be either a box,
#' a coordinate, a unit or a numerical value and all we
#' want is a `list` of coordinates that we can use for
#' calculating distance, alignment and other things.
#'
#' @param obj A single or a `list` of objects. Supported element types include:
#'   - `boxGrob` or `boxPropGrob`
#'   - output from `coords`
#'   - a `unit` (grid) or a numeric value coercible to an `npc` unit
#'
#'   When a `list` is provided the function recursively converts each element and
#'   returns bounding coordinates that encompass all elements. The returned
#'   units are in `npc` so the coordinates resize with the viewport.
#' @return A `list` with the points produced by `coords`. For list inputs the
#'   function returns merged bounding coordinates (class `box_coords`) with
#'   units in `npc`.
#' @details When given a list, `prConvert2Coords` computes the min/max of
#'   edges (or uses center +/- half sizes when edges are missing) to create a
#'   merged bounding box. Any additional coordinate names present on elements
#'   are merged heuristically (for example, `left*` -> min, `right*` -> max,
#'   otherwise averaged).
#' @examples
#' box1 <- boxGrob("A", x = .2, y = .8)
#' box2 <- boxGrob("B", x = .6, y = .4)
#' Gmisc:::prConvert2Coords(list(box1, box2))
#' @md
prConvert2Coords <- function(obj) {
  if (inherits(obj, "coords")) {
    return(obj)
  }

  # If a list is provided, recursively convert each element and merge edges
  if (inherits(obj, "list") && !is.grob(obj) && !is.unit(obj)) {
    if (length(obj) == 0) {
      stop("Expected a non-empty list of boxes/coords/units")
    }

    return(prConvertMultiple2Coords(obj))
  }

  if (inherits(obj, "box")) {
    return(coords(obj))
  }

  if (!is.unit(obj)) {
    if (!is.numeric(obj)) {
      stop(sprintf(
        "Expected a box object, coords, unit or number input; variable '%s' has value: %s",
        deparse(substitute(obj)), as.character(obj)
      ))
    }
    obj <- unit(obj, "npc")
  }

  list(
    y = obj,
    x = obj,
    top = obj,
    bottom = obj,
    left = obj,
    right = obj,
    height = 0,
    half_height = 0,
    width = 0,
    half_width = 0
  ) |>
    sapply(\(v) prAsUnit(v, units = "npc"), simplify = FALSE) |>
    structure(class = c("box_coords", "list"))
}


#' Helper to convert multiple boxes to merged bounding coordinates
#' for `prConvert2Coords()`.
#' @param obj A `list` of box-like objects.
#' @return A `list` of merged bounding coordinates with units in `npc`.
#' @noRd
prConvertMultiple2Coords <- function(obj) {
  coords_list <- lapply(obj, prConvert2Coords)

  # Per-element extractor: return numeric NPC value/size or NA_real_
  extractBoxGrobValue <- function(item, name, orient) {
    val <- item[[name]]
    if (is.null(val)) return(NA_real_)

    if (grepl("width|height|half", name)) {
      if (orient == "x") {
        return(prGetNpcSize(val, "x"))
      }
      return(prGetNpcSize(val, "y"))
    }

    if (orient == "x") return(prGetNpcValue(val, "x"))
    return(prGetNpcValue(val, "y"))
  }

  # Helper to extract numeric npc values for a given name and orientation
  getValues <- function(name, orient) {
    vals <- vapply(coords_list, function(c) extractBoxGrobValue(c, name, orient), numeric(1))
    vals <- vals[!is.na(vals)]
    if (length(vals) == 0) return(NULL)
    vals
  }

  # Primary edges (in npc)
  left_vals <- getValues("left", "x")
  right_vals <- getValues("right", "x")
  top_vals <- getValues("top", "y")
  bottom_vals <- getValues("bottom", "y")

  # If some primary edges are missing, try to compute from center +/- half sizes (all in npc)
  if (is.null(left_vals) || is.null(right_vals)) {
    x_vals <- getValues("x", "x")
    half_w_vals <- getValues("half_width", "x")
    if (is.null(left_vals) && !is.null(x_vals) && !is.null(half_w_vals)) left_vals <- x_vals - half_w_vals
    if (is.null(right_vals) && !is.null(x_vals) && !is.null(half_w_vals)) right_vals <- x_vals + half_w_vals
  }
  if (is.null(top_vals) || is.null(bottom_vals)) {
    y_vals <- getValues("y", "y")
    half_h_vals <- getValues("half_height", "y")
    if (is.null(top_vals) && !is.null(y_vals) && !is.null(half_h_vals)) top_vals <- y_vals + half_h_vals
    if (is.null(bottom_vals) && !is.null(y_vals) && !is.null(half_h_vals)) bottom_vals <- y_vals - half_h_vals
  }

  if (is.null(left_vals) || is.null(right_vals) || is.null(top_vals) || is.null(bottom_vals)) {
    stop("Could not determine bounding edges for provided list")
  }

  left_npc <- min(left_vals)
  right_npc <- max(right_vals)
  top_npc <- max(top_vals)
  bottom_npc <- min(bottom_vals)

  x_npc <- (left_npc + right_npc) / 2
  y_npc <- (top_npc + bottom_npc) / 2
  width_npc <- right_npc - left_npc
  height_npc <- top_npc - bottom_npc

  out <- list(
    left = left_npc,
    right = right_npc,
    top = top_npc,
    bottom = bottom_npc,
    x = x_npc,
    y = y_npc,
    width = width_npc,
    height = height_npc,
    half_width = width_npc / 2,
    half_height = height_npc / 2
  ) |>
    sapply(\(v) prAsUnit(v, units = "npc"), simplify = FALSE)

  # Merge any additional coordinates (e.g., left_x, right_x, prop_x) using heuristics
  all_names <- unique(unlist(lapply(coords_list, names)))
  extra_names <- setdiff(all_names, names(out))
  for (nm in extra_names) {
    orient <- ifelse(grepl("left|right|x|width", nm), "x", "y")
    # Reuse the `get_vals()` helper to extract numeric NPC values or sizes
    # for the extra coordinate name. `get_vals()` already handles width/height
    # detection and returns NULL when no values are present.
    vals <- getValues(nm, orient)
    vals <- vals[!is.na(vals)]
    if (length(vals) == 0) next

    if (grepl("left", nm)) {
      v <- min(vals)
    } else if (grepl("right", nm)) {
      v <- max(vals)
    } else if (grepl("top", nm)) {
      v <- max(vals)
    } else if (grepl("bottom", nm)) {
      v <- min(vals)
    } else {
      v <- mean(vals)
    }

    out[[nm]] <- unit(v, "npc")
  }

  return(structure(out, class = c("box_coords", "list")))
}

#' Spread boxes (S3)
#'
#' Spread a list of boxes vertically or horizontally. Designed for piping (`|>`).
#'
#' @param x A `list` of boxes.
#' @param ... Arguments passed to [`spreadVertical`] or [`spreadHorizontal`].
#' @param axis Orientation: `"y"`/`"vertical"` or `"x"`/`"horizontal"`.
#'
#' @return The list of boxes with updated positions (class `Gmisc_list_of_boxes`).
#' @export
#' @family flowchart components
spread <- function(x, ...) {
  UseMethod("spread")
}

#' @export
#' @rdname spread
spread.default <- function(x, ...) {
  if (is.list(x) && !inherits(x, "box")) {
    return(spread(prConvertListToBoxList(x), ...))
  }
  stop("spread() requires a list of boxes.")
}

#' @export
#' @rdname spread
spread.list <- function(x, ...) {
  spread.default(x, ...)
}

#' @export
#' @rdname spread
spread.Gmisc_list_of_boxes <- function(x, ..., axis = c("y", "x", "vertical", "horizontal")) {
  axis <- match.arg(axis)

  args <- list(...)
  call_args <- c(x, args)

  if (axis %in% c("y", "vertical")) {
    return(do.call(spreadVertical, call_args))
  } else {
    return(do.call(spreadHorizontal, call_args))
  }
}

# Internal helpers for fan_in_center connector -------------------------------------

# Compute bus extent based on starts x positions and end box center
prComputeFanCenterBus <- function(starts, end, subelmnt, margin = unit(2, "mm"), pad_mm = unit(1, "mm")) {
  s_coords <- lapply(starts, coords)
  e_coords <- prConvert2Coords(end)

  # subelement prefix for boxPropGrob anchors
  sub_prefix <- if (missing(subelmnt)) "" else paste0(match.arg(subelmnt, c("right", "left")), "_")
  x_at <- function(pos) {
    if (!is.null(pos[[paste0(sub_prefix, "x")]])) {
      return(pos[[paste0(sub_prefix, "x")]])
    }
    pos[["x"]]
  }

  # Get actual start x positions (not evenly-spaced slots) using the
  # centralized safe conversion helper. This avoids repeating tryCatch/as.numeric
  # patterns and provides a predictable NA fallback when conversion fails.
  starts_x_vals <- vapply(s_coords, \(s) prGetNpcValue(x_at(s), "x"), numeric(1))

  # Try to read end center x; fall back to sensible defaults if conversion
  # fails. Prefer end center, then span of valid starts, then center (0.5).
  end_center_x <- prGetNpcValue(e_coords$x, "x")

  valid_x <- starts_x_vals[!is.na(starts_x_vals)]
  if (is.na(end_center_x)) {
    if (length(valid_x) > 0) {
      end_center_x <- mean(range(valid_x))
    } else {
      end_center_x <- 0.5
    }
  }

  if (length(valid_x) == 0) {
    # No valid start x coordinates: use a small span around the end center
    bus_min_val <- max(0, end_center_x - 0.02)
    bus_max_val <- min(1, end_center_x + 0.02)
  } else {
    bus_min_val <- min(valid_x)
    bus_max_val <- max(valid_x)
  }

  list(
    bus_min = unit(bus_min_val, "npc"),
    bus_max = unit(bus_max_val, "npc"),
    trunk_x = unit(end_center_x, "npc"),
    starts_x_vals = starts_x_vals
  )
}

# Make vertical stem grobs for each start to bend_y
prMakeFanCenterStems <- function(starts, end, bend_y, starts_above, subelmnt, lty_gp) {
  s_coords <- lapply(starts, coords)
  sub_prefix <- if (missing(subelmnt)) "" else paste0(match.arg(subelmnt, c("right", "left")), "_")
  x_at <- function(pos) {
    if (!is.null(pos[[paste0(sub_prefix, "x")]])) {
      return(pos[[paste0(sub_prefix, "x")]])
    }
    pos[["x"]]
  }
  mk <- function(x, y) {
    ux <- unit.c(x)
    uy <- unit.c(y)
    g <- grid::linesGrob(x = ux, y = uy, gp = lty_gp)
    structure(g, line = list(x = ux, y = uy), class = unique(c("connect_boxes", class(g))))
  }
  lapply(seq_along(s_coords), function(i) {
    s <- s_coords[[i]]
    x0 <- x_at(s)
    if (!inherits(x0, "unit")) {
      x0 <- unit(x0, "npc")
    } else {
      x0 <- unit(prGetNpcValue(x0, "x"), "npc")
    }
    y0 <- if (starts_above) s$bottom else s$top
    mk(unit.c(x0, x0), unit.c(y0, bend_y))
  })
}

# Make the horizontal bus grob
prMakeFanCenterBus <- function(bus_min, bus_max, bend_y, lty_gp) {
  ux <- unit.c(bus_min, bus_max)
  uy <- unit.c(bend_y, bend_y)
  g <- grid::linesGrob(x = ux, y = uy, gp = lty_gp)
  structure(g, line = list(x = ux, y = uy), class = unique(c("connect_boxes", class(g))))
}

# Make a vertical trunk grob from trunk_x at bend_y to the end box center
prMakeFanCenterTrunk <- function(trunk_x, bend_y, end, starts_above, arrow_obj, lty_gp) {
  ecoords <- prConvert2Coords(end)

  # target y is box edge (top or bottom) plus small offset
  if (starts_above) {
    targ_y <- ecoords$top + unit(0.5, "mm")
  } else {
    targ_y <- ecoords$bottom - unit(0.5, "mm")
  }

  ux <- unit.c(trunk_x, trunk_x)
  uy <- unit.c(bend_y, targ_y)
  g <- grid::linesGrob(x = ux, y = uy, gp = lty_gp, arrow = arrow_obj)
  structure(g, line = list(x = ux, y = uy), class = unique(c("connect_boxes", class(g))))
}


#' Generate connector for fan-in-to-center
#'
#' Create a fan-in connector that merges stems onto a horizontal bus at a
#' shared bend height, and then a single centered trunk with an arrow pointing
#' to the center of the end box. Useful when a single arrow should represent
#' the merged flow.
#'
#' @param starts A list of start `boxGrob` objects.
#' @param end A single `boxGrob` end target.
#' @param subelmnt Optional sub-element specifier for split boxes (`"left"` or
#'   `"right"`).
#' @param lty_gp A `grid::gpar` controlling line appearance.
#' @param arrow_obj A `grid::arrow` object for arrowheads.
#' @param margin Margin (mm) applied when computing end attachment slots.
#' @param split_pad Padding around the shared bend point (mm) used to avoid
#'   overlapping the end box.
#' @return A `connect_boxes_list` (list of grobs) representing the connector.
#' @keywords internal
#' @noRd
prConnectManyToOneFanCenter <- function(
    starts,
    end,
    subelmnt,
    lty_gp,
    arrow_obj,
    margin = unit(2, "mm"),
    split_pad = unit(0, "mm")
) {
  # If `end` is container-like, attempt to find a reasonable boxed target.
  if (!inherits(end, "box")) {
    fb <- prFindFirstBox(end)
    if (!is.null(fb)) end <- fb
  }

  assert_class(end, "box")
  assert_class(lty_gp, "gpar")
  assert_class(arrow_obj, "arrow")
  if (is.numeric(margin)) margin <- unit(margin, "mm")
  if (is.numeric(split_pad)) split_pad <- unit(split_pad, "mm")

  # Calculate shared bend point
  bend_y <- prCalculateBendY(starts, end, margin = margin, split_pad = split_pad)

  # Compute bus extent and trunk x position (end box center)
  bus_res <- prComputeFanCenterBus(starts, end, subelmnt, margin = margin, pad_mm = unit(1, "mm"))
  bus_min <- bus_res$bus_min
  bus_max <- bus_res$bus_max
  trunk_x <- bus_res$trunk_x

  # Determine vertical relationship
  starts_above <- prStartsAbove(starts, end)

  # Make stems, bus and trunk
  stem_grobs <- prMakeFanCenterStems(starts, end, bend_y, starts_above, subelmnt, lty_gp)
  bus_grob <- prMakeFanCenterBus(bus_min, bus_max, bend_y, lty_gp)
  trunk_grob <- prMakeFanCenterTrunk(trunk_x, bend_y, end, starts_above, arrow_obj, lty_gp)

  structure(c(stem_grobs, list(bus_grob), list(trunk_grob)), class = c("connect_boxes_list", "list"))
}

# Internal helpers for align functions ----------------------------------------

# Normalize and validate the boxes argument for align* functions

# Helper: Is the object a list whose first element is itself a list
# but not a single box or grob? This captures the common "piped list"
# pattern where the caller supplied a list that got wrapped as an element
# inside `...` (e.g., `my_list |> alignHorizontal(reference=...)`).
prIsNestedNonBoxList <- function(x) {
    is.list(x) && length(x) >= 1 && is.list(x[[1]]) && !inherits(x[[1]], "box") && !is.grob(x[[1]])
}

# Helper: Is this a single-element wrapper (length == 1) that should be unwrapped
prIsSingleElementWrappedList <- function(x) {
    length(x) == 1 && prIsNestedNonBoxList(x)
}

# Helper: Does the list contain multiple top-level elements and the first
# element is itself a nested list container (not a box/grob)? This is used
# as the condition for searching inside the first container when resolving
# `subelement` paths (we only do this when there are multiple top-level
# elements to avoid changing behavior for single-element inputs).
prHasNestedFirstContainer <- function(x) {
    is.list(x) && length(x) > 1 && is.list(x[[1]]) && !inherits(x[[1]], "box") && !is.grob(x[[1]])
}

prNormalizeAndValidateBoxes <- function(boxes2align) {
    # If a single element list contains another list of boxes (e.g., result of piping),
    # unwrap it unless the inner element is itself a box/grob.
    if (prIsSingleElementWrappedList(boxes2align)) {
        boxes2align <- boxes2align[[1]]
    }

    # Ensure it's a list of at least one element
    assert_list(boxes2align, min.len = 1)

    # Validate members
    for (box in boxes2align) {
        prAssertBoxOrListOfBoxes(box)
    }

    boxes2align
}

prAssertBoxOrListOfBoxes <- function(box) {
    if (!inherits(box, "box") && !is.list(box) && !is.grob(box)) {
        if (inherits(box, "character")) {
            stop("Element must be a box or a list of boxes, got character: '", box, "'", call. = FALSE)
        }

        if (inherits(box, "numeric")) {
            stop("Element must be a box or a list of boxes, got numeric: ", box, call. = FALSE)
        }

        stop("Element must be a box or a list of boxes, got object of class ", paste(class(box), collapse = ", "), call. = FALSE)
    }
}

# Resolve a reference that may be provided as a path into boxes2align
prResolveReference <- function(reference, boxes2align) {
    if (is.atomic(reference) && length(boxes2align) > 0 && !inherits(reference, "box")) {
        maybe_ref <- get_list_element_by_path(boxes2align, reference)
        if (is.null(maybe_ref) && prIsNestedNonBoxList(boxes2align)) {
            maybe_ref <- get_list_element_by_path(boxes2align[[1]], reference)
        }
        if (!is.null(maybe_ref)) {
            return(maybe_ref)
        }
        # If reference is numeric but not a valid index, treat as coordinate (do not error).
        # Be careful: `grid::unit` objects may appear numeric-ish to some predicates but
        # are not numeric scalars and do not support `==` comparisons against numbers.
        # Ensure we only do numeric comparisons on bare numerics (not 'unit' objects).
        if (is.numeric(reference) && !inherits(reference, "unit") && length(reference) == 1 && (reference == 0 || reference > length(boxes2align))) {
            return(reference)
        }
    }
    reference
}

# Apply alignment given resolved boxes and ref positions
prApplyAlign <- function(boxes2align, ref_positions, position, axis = c("vertical", "horizontal")) {
    axis <- match.arg(axis)
    if (axis == "vertical") {
        ret <- lapply(boxes2align, function(box) {
            box_pos <- prConvert2Coords(box)
            if (position == "center") {
                new_y <- ref_positions$y
            } else if (position == "bottom") {
                new_y <- ref_positions$bottom + box_pos$half_height
            } else if (position == "top") {
                new_y <- ref_positions$top - box_pos$half_height
            } else {
                stop("Invalid position: ", position)
            }
            moveBox(box, y = new_y, just = c(NA, "center"))
        })
    } else {
        ret <- lapply(boxes2align, function(box) {
            box_pos <- prConvert2Coords(box)
            if (position == "center") {
                new_x <- ref_positions$x
            } else if (position == "left") {
                new_x <- ref_positions$left + box_pos$half_width
            } else if (position == "right") {
                new_x <- ref_positions$right - box_pos$half_width
            } else {
                stop("Invalid position: ", position)
            }
            moveBox(box, x = new_x, just = "center")
        })
    }
    prExtendClass(ret, "Gmisc_list_of_boxes")
}

testthat::test_that("prIsNestedNonBoxList and prIsSingleElementWrappedList work as expected", {
    # Nested list where first element is a list (should be TRUE)
    x1 <- list(list(a = 1), b = 2)
    testthat::expect_true(Gmisc:::prIsNestedNonBoxList(x1))
    testthat::expect_false(Gmisc:::prIsSingleElementWrappedList(x1))

    # Single-element wrapped list (should be TRUE)
    x2 <- list(list(a = 1))
    testthat::expect_true(Gmisc:::prIsNestedNonBoxList(x2))
    testthat::expect_true(Gmisc:::prIsSingleElementWrappedList(x2))

    # If the inner element is actually a box/grob, we shouldn't treat it as nested
    bx <- boxGrob("x")
    x3 <- list(bx)
    testthat::expect_false(Gmisc:::prIsNestedNonBoxList(x3))
    testthat::expect_false(Gmisc:::prIsSingleElementWrappedList(x3))

    # Multiple top-level elements where the first is a nested container
    x5 <- list(list(sub = list("A")), other = list())
    testthat::expect_true(Gmisc:::prIsNestedNonBoxList(x5))
    testthat::expect_true(Gmisc:::prHasNestedFirstContainer(x5))
    testthat::expect_false(Gmisc:::prIsSingleElementWrappedList(x5))

    # Non-list inner element
    x4 <- list(1)
    testthat::expect_false(Gmisc:::prIsNestedNonBoxList(x4))
    testthat::expect_false(Gmisc:::prIsSingleElementWrappedList(x4))
})

#' Create a flowchart object
#'
#' This function initializes a flowchart object, which is essentially a list of boxes
#' with the class `Gmisc_list_of_boxes`. It serves as the starting point for the
#' S3 pipeline, allowing for fluent chaining of [`align`], [`spread`], [`move`],
#' and [`connect`] operations.
#'
#' @param ... Either a set of [`boxGrob`] objects, text strings, or lists of these.
#'  Text strings will be automatically wrapped in [`boxGrob`] (or `default_box_fn`).
#' @param default_box_fn The function to use for converting text/content into a box
#'  (default: [`boxGrob`]).
#'
#' @return A `list` of class `Gmisc_list_of_boxes`.
#' @family flowchart components
#' @export
#' @examples
#' library(grid)
#' grid.newpage()
#'
#' # Create a flowchart with auto-conversion of text
#' fc <- flowchart(
#'   start = "Start",
#'   process = "Process",
#'   end = "End"
#' )
#'
#' # Use the pipeline
#' fc |>
#'   align(axis = "y") |>
#'   connect("start", "process", type = "vertical") |>
#'   connect("process", "end", type = "vertical") |>
#'   print()
flowchart <- function(..., default_box_fn = boxGrob) {
  args <- list(...)

  # If a single unnamed argument is passed and it's a list (but not a box),
  # treat it as the list itself rather than wrapping it
  if (length(args) == 1 &&
    is.list(args[[1]]) &&
    !inherits(args[[1]], "box") &&
    is.null(names(args))) {
    args <- args[[1]]
  }

  process_element <- function(el) {
    if (inherits(el, "box")) {
      return(el)
    }

    if (is.list(el)) {
      return(lapply(el, process_element))
    }

    # Assume content for a new box
    # Convert to box using default function
    default_box_fn(el)
  }

  converted_args <- lapply(args, process_element)
  prConvertListToBoxList(converted_args)
}

## ----include=FALSE------------------------------------------------------------
library(magrittr)
library(dplyr)
library(Gmisc)

# Style all the table output using htmlTable's theme handler
htmlTable::setHtmlTableTheme(css.rgroup = "")
set.seed(123)

## -----------------------------------------------------------------------------
library(Gmisc)
data("mtcars")
mtcars <- mtcars %>% 
  mutate(am = factor(am, levels = 0:1, labels = c("Automatic", "Manual")),
         gear = factor(gear),
         # Make up some data for making it slightly more interesting
         col = factor(sample(c("red", "black", "silver"),
                             size = NROW(mtcars), 
                             replace = TRUE))) %>% 
  set_column_labels(mpg = "Gas",
                    wt = "Weight",
                    am = "Transmission",
                    gear = "Gears",
                    col = "Car color") %>% 
  set_column_units(mpg = "Miles/(US) gallon",
                   wt = "10<sup>3</sup> lbs")

## -----------------------------------------------------------------------------
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am)

## -----------------------------------------------------------------------------
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am,
                        continuous_fn = describeMedian)

## -----------------------------------------------------------------------------
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        `Weight&dagger;` = wt,
                        am,
                        gear,
                        col,
                        by = am) %>% 
  htmlTable(caption  = "Basic descriptive statistics from the mtcars dataset",
            tfoot = "&dagger; The weight is in 10<sup>3</sup> kg")

## -----------------------------------------------------------------------------
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        `Weight&dagger;` = wt,
                        am,
                        gear,
                        col,
                        by = am,
                        digits = 0,
                        add_total_col = TRUE,
                        use_units = "name") %>% 
  addHtmlTableStyle(pos.caption = "bottom") %>% 
  htmlTable(caption  = "Basic descriptive statistics from the mtcars dataset",
            tfoot = "&dagger; The weight is in 10<sup>3</sup> kg")

## ----warning=FALSE------------------------------------------------------------
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am,
                        continuous_fn = describeMedian,
                        digits = 0,
                        header_count = TRUE,
                        statistics = TRUE) %>% 
  htmlTable(caption  = "Basic descriptive statistics from the mtcars dataset")

## ----warning=FALSE------------------------------------------------------------
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am,
                        continuous_fn = describeMedian,
                        digits = 0,
                        header_count = TRUE,
                        statistics = list(continuous = getPvalChiSq, 
                                          factor = getPvalChiSq, 
                                          proportion = getPvalFisher)) %>% 
  addHtmlTableStyle(pos.caption = "bottom") %>% 
  htmlTable(caption  = "P-values generated from a custom set of values")

## -----------------------------------------------------------------------------
getTable1Stats <- function(x, digits = 0, ...){
  getDescriptionStatsBy(x = x, 
                        by = mtcars$am,
                        digits = digits,
                        continuous_fn = describeMedian,
                        header_count = TRUE,
                        ...)
  
}

t1 <- list()
t1[["Gas"]] <-
  getTable1Stats(mtcars$mpg)
  
t1[["Weight&dagger;"]] <-
  getTable1Stats(mtcars$wt)

t1[["Color"]] <- 
  getTable1Stats(mtcars$col)

# If we want to use the labels set in the beginning
# we add an element without a name
t1 <- c(t1,
        list(getTable1Stats(mtcars$gear)))

mergeDesc(t1,
          htmlTable_args = list(caption  = "Basic descriptive statistics from the mtcars dataset",
                                tfoot = "&dagger; The weight is in 10<sup>3</sup> kg"))

#' Helper to [getDescriptionStatsBy()]
#'
#' @inheritParams getDescriptionStatsBy
#'
#' @return A [base::by] list
prNumericDescs <- function(x,
                           by,
                           hrzl_prop,
                           continuous_fn,
                           html,
                           digits,
                           digits.nonzero,
                           numbers_first,
                           useNA,
                           useNA.digits,
                           percentage_sign,
                           missing_value,
                           names_of_missing) {
  # If the numeric has horizontal_proportions then it's only so in the
  # missing category
  if (hrzl_prop) {
    t <- by(x, by,
            FUN = continuous_fn,
            html = html,
            digits = digits,
            digits.nonzero = digits.nonzero,
            number_first = numbers_first,
            useNA = useNA,
            useNA.digits = useNA.digits,
            horizontal_proportions = table(is.na(x), useNA = useNA),
            percentage_sign = percentage_sign
    )
  } else {
    t <- by(x, by,
            FUN = continuous_fn,
            html = html,
            digits = digits,
            digits.nonzero = digits.nonzero,
            number_first = numbers_first,
            useNA = useNA,
            useNA.digits = useNA.digits,
            percentage_sign = percentage_sign
    )
  }

  missing_t <- sapply(t, is.null)
  if (any(missing_t)) {
    substitute_t <- rep(missing_value, length(t[!missing_t][[1]]))
    names(substitute_t) <- names(t[!missing_t][[1]])
    for (i in seq_along(t[missing_t])) {
      t[missing_t][[i]] <- substitute_t
    }
  }

  if (all(unlist(sapply(t, is.na))) & !is.null(names_of_missing)) {
    substitute_t <- rep(missing_value, length(names_of_missing))
    names(substitute_t) <- names_of_missing
    substitute_list <- vector("list", length = length(t))
    names(substitute_list) <- names(t)
    for (i in seq_along(substitute_list)) {
      substitute_list[[i]] <- substitute_t
    }
    t <- substitute_list
  }

  if (length(t[[1]]) != 1) {
    fn_name <- deparse(substitute(continuous_fn))
    if (fn_name == "describeMean") {
      names(t[[1]][1]) <- "Mean"
    } else if (fn_name == "describeMedian") {
      names(t[[1]][1]) <- "Median"
    } else {
      names(t[[1]][1]) <- fn_name
    }
  }

  return(t)
}

library(grid)
grid.newpage()

# Create a set of boxes
start <- boxGrob("Top", x = .5, y = .8)
end <- boxGrob("Bottom", x = .5, y = .2)
side <- boxPropGrob("Side", "Left", "Right", prop = .3, x = .2, y = .8)
exclude <- boxGrob("Exclude:\n - Too sick\n - Prev. surgery", x = .8, y = .5, just = "left")

# We can chain the spread operations and print the result
spreadVertical(
  start = start,
  middle = list(side, exclude),
  end = end
) |>
  spreadHorizontal(subelement = "middle", from = 0.2, to = 0.8)

# Use device-level paging in interactive sessions so users can inspect the first plot
# This will prompt before creating the next page; we restore the previous setting afterwards
if (interactive()) {
  old_ask <- grDevices::devAskNewPage(TRUE)
  on.exit(grDevices::devAskNewPage(old_ask), add = TRUE)
  grid.newpage() # will prompt on interactive devices
} else {
  # non-interactive: just start a fresh page for the next example
  grid.newpage()
}

# Example: spread a nested subelement by deep path and print sequentially
list(grp = list(middle = list(side, exclude))) |>
  spreadHorizontal(subelement = c("grp", "middle"), from = 0.2, to = 0.8)

#' Move boxes (S3)
#'
#' Move a box or list of boxes. A wrapper for [`moveBox`] that supports
#' piping of `Gmisc_list_of_boxes` while preserving attributes.
#'
#' @param element A `list` of boxes.
#' @param ... Arguments passed to [`moveBox`].
#'
#' @return The moved list of boxes.
#' @export
#' @family flowchart components
move <- function(element, ...) UseMethod("move")

#' @export
#' @rdname move
move.default <- function(element, ...) {
  moveBox(element, ...)
}

#' @export
#' @rdname move
#' @param subelement The name of the element to move (maps to `subelement` in `moveBox`)
move.Gmisc_list_of_boxes <- function(element, subelement = NULL, ...) {
  # Capture attributes
  attrs <- attributes(element)

  args <- list(...)
  # Backward compatibility for 'name' if used (though it was just added in dev)
  if ("name" %in% names(args)) {
    # warning("Argument 'name' is deprecated, please use 'subelement' instead.")
    if (is.null(subelement)) subelement <- args$name
    args$name <- NULL
  }

  if (!is.null(subelement)) {
    args$subelement <- subelement
  }

  ret <- do.call(moveBox, c(list(element = element), args))

  # Restore attributes (specifically connections)
  # We only restore if they are missing in the result
  if (is.null(attr(ret, "connections")) && !is.null(attrs$connections)) {
    attr(ret, "connections") <- attrs$connections
  }

  prExtendClass(ret, "Gmisc_list_of_boxes")
}

#' Spread boxes
#'
#' Spreads a set of [`boxGrob`]/[`boxPropGrob`] in either horizontal or vertical
#' direction within a given span.
#'
#' The span can be defined explicitly using `from` / `to`, or implicitly by the
#' current viewport. Numeric values are interpreted as proportions of the viewport
#' (`npc` units).
#'
#' @param ... A set of boxes to spread. Can also be a \code{list} of boxes.
#' @param from Starting point of the span. Can be a box, a coordinate/unit, or a
#'   numeric value interpreted as `npc`. If only `from` is provided, `to` defaults
#'   to `1 npc`.
#' @param to Ending point of the span. Can be a box, a coordinate/unit, or a
#'   numeric value interpreted as `npc`. If only `to` is provided, `from` defaults
#'   to `0 npc`.
#' @param margin Optional padding applied at both ends of the span. Can be a
#'   [`grid::unit`] or a numeric value interpreted as `npc`. Applied whether the
#'   span comes from `from` / `to` or the viewport.
#' @param type If `between`, the space *between* boxes is identical. If `center`,
#'   the centers of the boxes are equally distributed across the span.
#' @param subelement If a `list` of boxes is provided, this parameter can be used
#'   to target a specific element (by name or index), or a deep path into nested
#'   lists (e.g., `c("detail", 1)`), for the spreading operation. You can also
#'   provide multiple targets by giving a list of paths (e.g., `list(c("detail", 1), c("followup", 1))`).
#'   The function will return the original list with the targeted element(s)
#'   replaced by their spread version(s).
#'
#' @return A `list` with the boxes that have been spread.
#'
#' @md
#' @name spread
#' @family flowchart components
#' @example inst/examples/spreadBox_ex.R


#' @rdname spread
#' @export
spreadVertical <- function(
  ...,
  from = NULL,
  to = NULL,
  margin = unit(0, "npc"),
  type = c("between", "center"),
  subelement = NULL
) {
  type <- match.arg(type)

  boxes2spread <- list(...)

  margin <- prAsUnit(margin, units = "npc")
  ft <- prNormalizeFromTo(from, to, axis = "y")
  from <- ft$from
  to <- ft$to

  if (length(boxes2spread) == 1 && is.list(boxes2spread) && !inherits(boxes2spread, "box")) {
    boxes2spread <- boxes2spread[[1]]
  }

  if (!is.null(subelement)) {
    paths <- if (is.list(subelement) && all(sapply(subelement, is.atomic))) subelement else list(subelement)

    # Resolve from/to if they are given as paths into the top-level list
    resolve_endpoint <- function(endpoint) {
      is_path_like <- is.list(endpoint) || (is.atomic(endpoint) && any(sapply(endpoint, is.character)))
      if (is_path_like && length(boxes2spread) > 0 && !inherits(endpoint, "box")) {
        ep <- get_list_element_by_path(boxes2spread, endpoint)
        if (is.null(ep) && length(boxes2spread) > 1 && is.list(boxes2spread[[1]]) && !inherits(boxes2spread[[1]], "box")) {
          ep <- get_list_element_by_path(boxes2spread[[1]], endpoint)
        }
        if (!is.null(ep)) {
          return(ep)
        }
      }
      endpoint
    }

    resolved_from <- resolve_endpoint(from)
    resolved_to <- resolve_endpoint(to)

    for (path in paths) {
      target <- get_list_element_by_path(boxes2spread, path)
      container_is_first <- FALSE

      if (is.null(target) && length(boxes2spread) > 1 && is.list(boxes2spread[[1]]) && !inherits(boxes2spread[[1]], "box")) {
        target <- get_list_element_by_path(boxes2spread[[1]], path)
        container_is_first <- TRUE
      }

      if (is.null(target)) {
        stop("The subelement '", paste(path, collapse = " -> "), "' was not found in the provided boxes.",
          call. = FALSE
        )
      }

      aligned <- spreadVertical(
        target,
        from = resolved_from,
        to = resolved_to,
        margin = margin,
        type = type
      )

      if (container_is_first) {
        boxes2spread[[1]] <- set_list_element_by_path(boxes2spread[[1]], path, aligned)
      } else {
        boxes2spread <- set_list_element_by_path(boxes2spread, path, aligned)
      }
    }

    return(prExtendClass(boxes2spread, "Gmisc_list_of_boxes"))
  }

  assert_list(boxes2spread, min.len = 1)
  for (box in boxes2spread) {
    prAssertBoxOrListOfBoxes(box)
  }

  span_info <- prGetSpanSpace(
    boxes2spread = boxes2spread,
    from = from,
    to = to,
    margin = margin,
    type = type,
    orientation = "vertical"
  )

  new_y_distances <- prGetNewDistances(
    span_info = span_info,
    type = type,
    orientation = "vertical"
  )

  prApplyBoxSpread(
    boxes2spread = boxes2spread,
    distances = new_y_distances,
    move_fn = function(box, pos) {
      moveBox(box, y = pos, space = "absolute", just = c(NA, "center"))
    }
  )
}

#' @rdname spread
#' @export
spreadHorizontal <- function(..., from = NULL, to = NULL, margin = unit(0, "npc"),
                             type = c("between", "center"), subelement = NULL) {
  type <- match.arg(type)

  boxes2spread <- list(...)

  margin <- prAsUnit(margin, units = "npc")
  ft <- prNormalizeFromTo(from, to, axis = "x")
  from <- ft$from
  to <- ft$to

  if (length(boxes2spread) == 1 && is.list(boxes2spread) && !inherits(boxes2spread, "box")) {
    boxes2spread <- boxes2spread[[1]]
  }

  if (!is.null(subelement)) {
    paths <- if (is.list(subelement) && all(sapply(subelement, is.atomic))) subelement else list(subelement)

    resolve_endpoint <- function(endpoint) {
      # Only attempt path resolution for character-like paths (e.g. c("detail", 1))
      is_path_like <- is.list(endpoint) || (is.atomic(endpoint) && any(sapply(endpoint, is.character)))
      if (is_path_like && length(boxes2spread) > 0 && !inherits(endpoint, "box")) {
        ep <- get_list_element_by_path(boxes2spread, endpoint)
        if (is.null(ep) && length(boxes2spread) > 1 && is.list(boxes2spread[[1]]) && !inherits(boxes2spread[[1]], "box")) {
          ep <- get_list_element_by_path(boxes2spread[[1]], endpoint)
        }
        if (!is.null(ep)) {
          return(ep)
        }
      }
      endpoint
    }

    resolved_from <- resolve_endpoint(from)
    resolved_to <- resolve_endpoint(to)

    for (path in paths) {
      target <- get_list_element_by_path(boxes2spread, path)
      container_is_first <- FALSE

      if (is.null(target) && length(boxes2spread) > 1 && is.list(boxes2spread[[1]]) && !inherits(boxes2spread[[1]], "box")) {
        target <- get_list_element_by_path(boxes2spread[[1]], path)
        container_is_first <- TRUE
      }

      if (is.null(target)) {
        stop("The subelement '", paste(path, collapse = " -> "), "' was not found in the provided boxes.",
          call. = FALSE
        )
      }

      aligned <- spreadHorizontal(
        target,
        from = resolved_from,
        to = resolved_to,
        margin = margin,
        type = type
      )

      if (container_is_first) {
        boxes2spread[[1]] <- set_list_element_by_path(boxes2spread[[1]], path, aligned)
      } else {
        boxes2spread <- set_list_element_by_path(boxes2spread, path, aligned)
      }
    }

    return(prExtendClass(boxes2spread, "Gmisc_list_of_boxes"))
  }

  assert_list(boxes2spread, min.len = 1)
  for (box in boxes2spread) {
    prAssertBoxOrListOfBoxes(box)
  }

  span_info <- prGetSpanSpace(
    boxes2spread = boxes2spread,
    from = from,
    to = to,
    margin = margin,
    type = type,
    orientation = "horizontal"
  )

  new_x_distances <- prGetNewDistances(
    span_info = span_info,
    type = type,
    orientation = "horizontal"
  )

  prApplyBoxSpread(
    boxes2spread = boxes2spread,
    distances = new_x_distances,
    move_fn = function(box, pos) {
      moveBox(box, x = pos, space = "absolute", just = "center")
    }
  )
}

prGetSpanSpace <- function(
  boxes2spread,
  from,
  to,
  margin = unit(0, "npc"),
  type,
  orientation = c("vertical", "horizontal")
) {
  orientation <- match.arg(orientation)
  type_size_key <- ifelse(orientation == "vertical", "height", "width")
  type_half_size_key <- paste0("half_", type_size_key)

  # ---- input checks / normalization ----
  stopifnot(is.list(boxes2spread), length(boxes2spread) >= 1)
  for (b in boxes2spread) {
    prAssertBoxOrListOfBoxes(b)
  }

  if (missing(type) || is.null(type)) stop("`type` must be provided.", call. = FALSE)
  if (!type %in% c("between", "center")) stop("`type` must be 'between' or 'center'.", call. = FALSE)

  if (is.numeric(margin)) {
    if (length(margin) != 1) stop("`margin` must be a scalar.", call. = FALSE)
    margin <- unit(margin, "npc")
  }
  if (!inherits(margin, "unit")) stop("`margin` must be a grid::unit or numeric.", call. = FALSE)

  axis <- if (orientation == "horizontal") "x" else "y"

  to_npc <- function(u) {
    unit(prGetNpcValue(u, axis), "npc")
  }

  # Normalize margin into npc and ensure non-negative
  margin_npc <- to_npc(margin)
  mn_val <- prGetNpcSize(margin_npc, axis)
  if (is.na(mn_val) || mn_val < 0) {
    stop("`margin` must be >= 0.", call. = FALSE)
  }

  # If only one endpoint is provided, default the other (wrappers may already do this)
  if (!is.null(from) && is.null(to)) to <- unit(1, "npc")
  if (is.null(from) && !is.null(to)) from <- unit(0, "npc")

  dist_sign <- NA
  include_first <- TRUE
  include_last <- TRUE

  # ---- compute span endpoints and in-between boxes ----
  if (!is.null(from) || !is.null(to)) {
    stopifnot(!is.null(from), !is.null(to))

    dist <- distance(box1 = from, box2 = to, type = orientation)
    dist_sign <- ifelse(isTRUE(attr(dist, "positive")), 1, -1)
    start_pos <- attr(dist, "from")
    end_pos <- attr(dist, "to")

    # Anchor at from: either a box or a coordinate/unit
    if (inherits(from, "box") || is.list(from)) {
      first <- from
      include_first <- FALSE
      if (type == "center") start_pos <- start_pos - dist_sign * prConvert2Coords(first)[[type_half_size_key]]
    } else {
      first <- boxes2spread[[1]]
      boxes2spread <- if (length(boxes2spread) == 1) list() else boxes2spread[-1]
      start_pos <- start_pos + dist_sign * prConvert2Coords(first)[[type_size_key]]
      dist <- dist - prConvert2Coords(first)[[type_size_key]]
    }

    # Anchor at to: either a box or a coordinate/unit
    if (inherits(to, "box") || is.list(to)) {
      last <- to
      include_last <- FALSE
      if (type == "center") end_pos <- end_pos + dist_sign * prConvert2Coords(last)[[type_half_size_key]]
    } else {
      if (length(boxes2spread) == 0) {
        stop("No boxes left to place between `from` and `to`.", call. = FALSE)
      }
      last <- tail(boxes2spread, 1)[[1]]
      boxes2spread <- if (length(boxes2spread) == 1) list() else boxes2spread[-length(boxes2spread)]
      end_pos <- end_pos - dist_sign * prConvert2Coords(last)[[type_size_key]]
      dist <- dist - prConvert2Coords(last)[[type_size_key]]
    }

    boxes_in_between <- boxes2spread
  } else {
    # Viewport mode
    if (length(boxes2spread) <= 1) stop("Can't spread a single box.", call. = FALSE)

    first <- boxes2spread[[1]]
    last <- tail(boxes2spread, 1)[[1]]

    # Anchor at the viewport margins (leave box half-sizes to later adjustments)
    if (orientation == "vertical") {
      start_pos <- unit(1, "npc") - margin_npc
      end_pos <- unit(0, "npc") + margin_npc
    } else {
      start_pos <- unit(0, "npc") + margin_npc
      end_pos <- unit(1, "npc") - margin_npc
    }

    dist <- distance(box1 = start_pos, box2 = end_pos, type = orientation)
    dist_sign <- ifelse(isTRUE(attr(dist, "positive")), 1, -1)

    boxes_in_between <- if (length(boxes2spread) > 2) {
      boxes2spread[2:(length(boxes2spread) - 1)]
    } else {
      list()
    }
  }

  # ---- apply margin for explicit from/to too ----
  # (viewport already included margin in start/end)
  if (!is.null(from) && !is.null(to)) {
    # shrink span on both ends by margin
    start_pos <- start_pos + dist_sign * margin_npc
    end_pos <- end_pos - dist_sign * margin_npc
    dist <- dist - 2 * margin_npc
  }

  # ---- compute available space in npc ----
  extra_space <- unit(0, "npc")
  if (type == "center") {
    extra_space <- to_npc(prConvert2Coords(first)[[type_half_size_key]] + prConvert2Coords(last)[[type_half_size_key]])
  }

  span_npc <- to_npc(dist) + extra_space

  # Must not be negative after margin/size adjustments
  sp_val <- prGetNpcSize(span_npc, axis)
  if (is.na(sp_val) || sp_val < 0) {
    stop(
      "No space left to spread boxes: span collapsed (from/to too close or margin too large).",
      call. = FALSE
    )
  }

  available_space <- span_npc

  if (type == "between" && length(boxes_in_between) > 0) {
    for (b in boxes_in_between) {
      available_space <- available_space - to_npc(prConvert2Coords(b)[[type_size_key]])
    }
    av_val <- prGetNpcSize(available_space, axis)
    if (is.na(av_val) || av_val < 0) {
      stop(
        "No space left to spread boxes: in-between boxes exceed the available span.",
        call. = FALSE
      )
    }
  }

  if (!include_first) first <- NULL
  if (!include_last) last <- NULL

  list(
    boxes_in_between = boxes_in_between,
    first = first,
    last = last,
    start_pos = start_pos,
    end_pos = end_pos,
    available_space = available_space, # npc
    sign = dist_sign
  )
}


prGetNewDistances <- function(span_info, type,
                              orientation = c("vertical", "horizontal")) {
  orientation <- match.arg(orientation)
  axis <- if (orientation == "vertical") "y" else "x"
  type_size_key <- ifelse(orientation == "vertical", "height", "width")
  type_half_size_key <- paste0("half_", type_size_key)

  to_npc_pos <- function(u) unit(prGetNpcValue(u, axis), "npc")
  to_npc_size <- function(u) unit(prGetNpcSize(u, axis), "npc")

  space_distance <- span_info$available_space / (length(span_info$boxes_in_between) + 1)

  new_coordinate <- NULL
  offset <- to_npc_pos(span_info$start_pos)

  if (!is.null(span_info$first)) {
    new_coordinate <- to_npc_pos(span_info$start_pos) - span_info$sign * to_npc_size(prConvert2Coords(span_info$first)[[type_half_size_key]])
    offset <- if (type == "center") new_coordinate else to_npc_pos(span_info$start_pos)
  }

  for (b in span_info$boxes_in_between) {
    if (type == "between") {
      new_position <- offset + span_info$sign * (space_distance + to_npc_size(prConvert2Coords(b)[[type_half_size_key]]))
      offset <- offset + span_info$sign * (space_distance + to_npc_size(prConvert2Coords(b)[[type_size_key]]))
    } else {
      new_position <- offset + span_info$sign * space_distance
      offset <- offset + span_info$sign * space_distance
    }
    new_coordinate <- if (is.null(new_coordinate)) new_position else unit.c(new_coordinate, new_position)
  }

  if (!is.null(span_info$last)) {
    last_position <- to_npc_pos(span_info$end_pos) + span_info$sign * to_npc_size(prConvert2Coords(span_info$last)[[type_half_size_key]])
    new_coordinate <- unit.c(new_coordinate, last_position)
  }

  new_coordinate
}


prApplyBoxSpread <- function(boxes2spread, distances, move_fn) {
  ret <- list()
  for (i in 1:length(boxes2spread)) {
    box <- move_fn(box = boxes2spread[[i]], pos = distances[i])

    if (is.null(names(boxes2spread))) {
      ret <- append(ret, list(box))
    } else {
      key <- names(boxes2spread)[i]
      if (key == "") {
        ret <- append(ret, list(box))
      } else {
        ret[[key]] <- box
      }
    }
  }

  prExtendClass(ret, "Gmisc_list_of_boxes")
}

testthat::test_that("alignHorizontal subelement replacement doesn't create self-referential boxes and connectors split", {
    library(grid)

    start <- boxGrob("start")
    rand <- boxGrob("rand")
    arms <- list(early = boxGrob("early"), late = boxGrob("late"))
    arm_text <- list(boxGrob("ate1"), boxGrob("ate2"))

    boxes <- spreadVertical(
        start = start,
        rand = rand,
        arms = arms,
        arm_text = arm_text,
        footer = boxGrob("footer")
    ) |>
        spreadHorizontal(subelement = "arms", from = 0.05, to = 0.95) |>
        spreadHorizontal(subelement = "arm_text", from = 0.05, to = 0.95) |>
        alignHorizontal(
            reference = c("arm_text", 1),
           subelement = c("arms", "early")
        ) |>
        alignHorizontal(
            reference = c("arm_text", 2),
           subelement = c("arms", "late")
        )

    # arms should be a plain list of boxes (not a self-referential copy of `boxes`)
    expect_false(identical(boxes$arms, boxes))
    expect_true(is.list(boxes$arms))
    expect_true(all(vapply(boxes$arms, inherits, logical(1), "box")))

    # Connecting from rand to arms (one-to-many) should produce a multi-connector
    expect_silent(con <- connectGrob(boxes$rand, boxes$arms, type = "N"))
    if (is.list(con)) {
        testthat::expect_s3_class(con, "connect_boxes_list")
    } else {
        testthat::expect_s3_class(con, "connect_boxes")
    }
})

testthat::test_that("connectGrob handles nested step_1 from vignette flow", {
  library(grid)

  org_cohort <- paste("Org cohort",
                      "  - N = 180 patients",
                      "  - Age 18-65 years",
                      sep = "\n"
  ) |> boxGrob(just = "left")
  surgery <- paste("Surgery",
                   "  - Direct (approx 10%)",
                   "  - Delayed (approx 10%)",
                   sep = "\n"
  ) |> boxGrob(just = "left")
  randomize <- boxGrob("Non-surgical\nRandomise")
  treatments <- list(early = boxGrob("Early rehab"), late = boxGrob("Late rehab"), obs = boxGrob("Observation"))
  early_followup <- paste("Early follow-up",
                          "  - 2 weeks [PNRS]",
                          "  - 4 weeks [PNRS]",
                          sep = "\n"
  ) |> boxGrob(just = "left")
  late_followup <- paste("Late follow-up",
                         "  - 3 months",
                         "  - 1 year",
                         "  - 2 years",
                         "  - 5 years",
                         sep = "\n"
  ) |> boxGrob(just = "left")

  boxes <- spreadVertical(
    start = org_cohort,
    step_1 = list(surgery = surgery, `non-surgical` = randomize),
    treatment = treatments,
    early_followup = early_followup,
    followup = late_followup
  ) |>
    spreadHorizontal(subelement = "step_1") |>
    spreadHorizontal(subelement = "treatment", from = 0.35) |>
    alignHorizontal(
      reference = c("treatment", "late"),
     subelement = c("step_1", "non-surgical")
    )

  # Should not error: connect from start to the step_1 group
  expect_silent(con <- connectGrob(boxes$start, boxes$step_1, type = "N"))
  # If returns a list of connectors, check class
  if (is.list(con)) {
    testthat::expect_s3_class(con, "connect_boxes_list")
  } else {
    testthat::expect_s3_class(con, "connect_boxes")
  }

  expect_silent(con <- connectGrob(boxes$step_1$`non-surgical`, boxes$treatment, type = "N"))
  if (is.list(con)) {
    testthat::expect_s3_class(con, "connect_boxes_list")
  } else {
    testthat::expect_s3_class(con, "connect_boxes")
  }
})

testthat::test_that("alignHorizontal pipeline with subelement works end-to-end", {
    library(grid)

    org_cohort <- paste("Proximal humerus fracture", "  - >= 18 years", "  - <= 4 weeks of trauma", "  - Not pathological", sep = "\n") |> boxGrob(just = "left")
    surgery <- paste("Surgery", "  - Direct (approx 4%)", "  - Delayed (approx 4%)", sep = "\n") |> boxGrob(just = "left")
    randomize <- boxGrob("Non-surgical\nRandomise")
    treatments <- list(early = boxGrob("Early rehab"), late = boxGrob("Late rehab"), obs = boxGrob("Observation"))
    early_followup <- paste("Early follow-up", "  - 2 weeks [PNRS]", "  - 4 weeks [PNRS]", sep = "\n") |> boxGrob(just = "left")
    late_followup <- paste("Late follow-up", "  - 2-10 months (random) [OSS, PNRS]", "  - 1 year [OSS, PNRS, accelerometer]", "  - 2 years [OSS, PNRS]", "  - 5 years [OSS, PNRS]", sep = "\n") |> boxGrob(just = "left")

    expect_silent({
        boxes <- spreadVertical(
            start = org_cohort,
            step_1 = list(surgery = surgery, `non-surgical` = randomize),
            treatment = treatments,
            early_followup = early_followup,
            followup = late_followup
        ) |>
            spreadHorizontal(subelement = "step_1") |>
            spreadHorizontal(subelement = "treatment", from = 0.35) |>
            alignHorizontal(
                reference = c("treatment", "late"),
               subelement = c("step_1", "non-surgical")
            )
    })
})

testthat::test_that("alignVertical accepts piped positional reference path", {
    library(grid)

    boxes <- flowchart(
        rando = "Randomised",
        groups = list("A", "B"),
        groups2 = list("C", "D")
    ) |>
        spread(axis = "y", margin = unit(0.02, "npc")) |>
        spread(subelement = "groups", axis = "x", margin = unit(.2, "npc")) |>
        spread(subelement = "groups2", axis = "x", margin = unit(.2, "npc"))

    expect_silent({
        aligned <- boxes |>
            alignVertical("groups", subelement = "groups2")
    })

    expect_s3_class(aligned, "Gmisc_list_of_boxes")
    expect_equal(length(aligned$groups2), 2)
})

testthat::test_that("connectGrob to group containers uses first box in each group", {
    library(grid)

    # Build grouped boxes: each group is a list of boxes (header then content boxes)
    header <- function(txt) structure(txt, args = list(box_gp = gpar(fill = "white")))
    content <- function(txt) structure(txt, args = list(box_gp = gpar(fill = "grey90")))

    boxes <- list(
        common = boxGrob("TOP"),
        groups = list(
            early = list(boxGrob("Early header"), boxGrob("E1"), boxGrob("E2")),
            late  = list(boxGrob("Late header"), boxGrob("L1"))
        )
    ) |>
        spreadVertical() |>
        spreadHorizontal(subelement = "groups", from = 0.2, to = 0.8)

    # Connect common to groups: expecting one connector per group (2 connectors)
    con <- connectGrob(boxes$common, boxes$groups, type = "N")
    testthat::expect_true(is.list(con) || inherits(con, "connect_boxes"))
    if (is.list(con)) {
        testthat::expect_length(con, length(boxes$groups))
    }

    # Check that each connector's endpoints point to each group's first box
    ends <- lapply(boxes$groups, function(g) prConvert2Coords(g[[1]]))
    # For N connector, assigned end x should match group's header x
    con_list <- if (is.list(con)) con else list(con)
    assigned_end_x <- vapply(con_list, function(g) convertX(attr(g, "line")$x[4], "npc", valueOnly = TRUE), numeric(1))
    expected_x <- vapply(ends, function(e) convertX(e$x, "npc", valueOnly = TRUE), numeric(1))
    testthat::expect_equal(sort(unname(assigned_end_x)), sort(unname(expected_x)))
})

library(testthat)

test_that("fan_in_center creates bus and single trunk arrow", {
    pop <- boxGrob("Population")
    treatments <- list(
        A = boxGrob("A"),
        B = boxGrob("B"),
        C = boxGrob("C")
    )
    followup <- boxGrob("Follow-up")

    boxes <- list(population = pop, treatment = treatments, followup = followup)
    boxes <- boxes |>
        spreadVertical() |>
        spreadHorizontal(from = unit(0.1, "npc"), to = unit(0.9, "npc"), type = "center", subelement = "treatment")

    con <- connectGrob(boxes$treatment, boxes$followup, type = "fan_in_center")
    expect_s3_class(con, "connect_boxes_list")

    n <- length(boxes$treatment)
    # Expect stems (n), bus (1), trunk (1)
    expect_equal(length(con), n + 2)

    bus <- con[[n + 1]]
    # bus should be horizontal: both y coords equal
    y_bus <- attr(bus, "line")$y
    y_vals_mm <- vapply(y_bus, function(u) convertY(u, "mm", valueOnly = TRUE), numeric(1))
    expect_equal(y_vals_mm[1], y_vals_mm[2])

    # bus should span the start boxes (plus padding)
    x_bus <- attr(bus, "line")$x
    x_bus_mm <- vapply(x_bus, function(u) convertX(if (!inherits(u, "unit")) unit(u, "npc") else u, "mm", valueOnly = TRUE), numeric(1))
    starts_x_mm <- vapply(lapply(boxes$treatment, coords), function(s) convertX(s$x, "mm", valueOnly = TRUE), numeric(1))
    expect_true((max(x_bus_mm) - min(x_bus_mm)) + 1e-6 >= (max(starts_x_mm) - min(starts_x_mm)))

    trunk <- con[[n + 2]]
    # trunk should have an arrow defined
    expect_true(!is.null(trunk$arrow))

    # trunk should be vertical (both x coords equal) and align with end box center
    trunk_x <- attr(trunk, "line")$x
    trunk_x_vals <- vapply(trunk_x, function(u) convertX(if (!inherits(u, "unit")) unit(u, "npc") else u, "npc", valueOnly = TRUE), numeric(1))
    expect_equal(trunk_x_vals[1], trunk_x_vals[2])

    # trunk should align with the end box center, not the bus center
    e_coords <- coords(boxes$followup)
    end_center_npc <- convertX(e_coords$x, "npc", valueOnly = TRUE)
    expect_equal(trunk_x_vals[1], end_center_npc)

    # trunk should point to the end box edge (top if starts are above, bottom otherwise)
    trunk_end_y <- attr(trunk, "line")$y[2]
    trunk_end_mm <- convertY(if (!inherits(trunk_end_y, "unit")) unit(trunk_end_y, "npc") else trunk_end_y, "mm", valueOnly = TRUE)
    end_top_mm <- convertY(e_coords$top + unit(0.5, "mm"), "mm", valueOnly = TRUE)
    end_bottom_mm <- convertY(e_coords$bottom - unit(0.5, "mm"), "mm", valueOnly = TRUE)
    starts_mean_y_mm <- mean(vapply(lapply(boxes$treatment, coords), function(s) convertY(s$y, "mm", valueOnly = TRUE), numeric(1)))
    end_y_mm <- convertY(e_coords$y, "mm", valueOnly = TRUE)
    if (starts_mean_y_mm > end_y_mm) {
        # starts are above -> trunk should target top edge
        expect_true(abs(trunk_end_mm - end_top_mm) < 1e-2)
    } else {
        # starts below -> trunk should target bottom edge
        expect_true(abs(trunk_end_mm - end_bottom_mm) < 1e-2)
    }
})

testthat::test_that("spreadHorizontal handles numeric endpoints with subelement name", {
  # Load package internals for this integration test
  # No package load needed; tests run in package context during R CMD check
  library(grid)

  randomize <- boxGrob("Randomize",
    box_gp = gpar(fill = "#ECEFF1", col = "#455A64"),
    txt_gp = gpar(fontsize = 11, fontface = "bold")
  )

  early <- boxGrob("Early rehabilitation",
    box_gp = gpar(fill = "#E8F5E9", col = "#2E7D32"),
    txt_gp = gpar(fontsize = 11, fontface = "bold")
  )

  late <- boxGrob("Delayed rehabilitation",
    box_gp = gpar(fill = "#FFF8E1", col = "#EF6C00"),
    txt_gp = gpar(fontsize = 11, fontface = "bold")
  )

  boxes <- spreadVertical(
    start = randomize,
    arms = list(early = early, late = late)
  )

  # Execute: we only assert that the specific integer-selection error is not raised
  res <- tryCatch(
    list(value = spreadHorizontal(boxes, subelement = "arms", from = 0.25, to = 0.75), error = NULL),
    error = function(e) list(value = NULL, error = e)
  )

  if (!is.null(res$error)) {
    msg <- conditionMessage(res$error)
    expect_false(grepl("attempt to select less than one element in integerOneIndex", msg, fixed = TRUE))
  } else {
    expect_true(is.list(res$value))
  }
})

library(testthat)

test_that("fan_in_top works with default margin and nested lists", {
    # Build a small example similar to the advanced example
    pop <- boxGrob("Population")
    treatments <- list(
        Ibuprofen = boxGrob("Ibuprofen"),
        Paracetamol = boxGrob("Paracetamol"),
        Aspirin = boxGrob("Aspirin")
    )
    followup <- paste("Follow-up 4 weeks:", " - EQ-5D 5L", " - Lab", sep = "\n") |> boxGrob(just = "left")

    boxes <- list(population = pop, treatment = treatments, followup = followup)
    boxes <- boxes |>
        spreadVertical() |>
        spreadHorizontal(from = unit(0.1, "npc"), to = unit(0.9, "npc"), type = "center", subelement = "treatment")

    # This should not error and should return a connect_boxes_list
    con <- connectGrob(boxes$treatment, boxes$followup, type = "fan_in_top")
    expect_s3_class(con, "connect_boxes_list")

    # Ensure trunks merge on the top edge with a spread (i.e., not all identical end x)
    n <- length(boxes$treatment)
    trunks <- tail(con, n)
    trunk_x_end_mm <- vapply(trunks, function(g) {
        x <- attr(g, "line")$x[2]
        if (!inherits(x, "unit")) x <- unit(x, "npc")
        convertX(x, "mm", valueOnly = TRUE)
    }, numeric(1))
    trunk_x_start_mm <- vapply(trunks, function(g) {
        x <- attr(g, "line")$x[1]
        if (!inherits(x, "unit")) x <- unit(x, "npc")
        convertX(x, "mm", valueOnly = TRUE)
    }, numeric(1))

    # end xs should be spread across the top edge
    expect_gt(max(trunk_x_end_mm) - min(trunk_x_end_mm), 0)

    # trunks should be diagonal (start x != end x) at least for one trunk
    expect_true(any(abs(trunk_x_end_mm - trunk_x_start_mm) > 1e-6))
})

library(testthat)

test_that("Box spread vertically with center", {
  box1 <- boxGrob("B1", x = .2, y = .8)
  box2 <- boxGrob("B2", x = .2, y = .8)
  box3 <- boxGrob("B3\ntricky", x = .2, y = .8)
  box4 <- boxGrob("B4", x = .2, y = .8)
  box5 <- boxGrob("B5", x = .2, y = .8)

  spread_boxes <- spreadVertical(box1,
    box2,
    box3,
    b = box4,
    c = box5,
    type = "center"
  )
  sapply(
    spread_boxes,
    function(b) coords(b)$x %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    expect_equivalent(.2)

  sapply(
    spread_boxes,
    function(b) coords(b)$y %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    length() %>%
    expect_equivalent(length(spread_boxes))
  convertY(coords(spread_boxes[[1]])$top, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 1,
      tolerance = 1e-3
    )

  convertY(coords(tail(spread_boxes, 1)[[1]])$bottom, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 0,
      tolerance = 1e-3
    )

  expect_equivalent(
    distance(spread_boxes[1:2], type = "v", center = TRUE),
    distance(spread_boxes[2:3], type = "v", center = TRUE)
  )

  expect_equivalent(
    distance(spread_boxes[3:4], type = "v", center = TRUE),
    distance(spread_boxes[4:5], type = "v", center = TRUE)
  )
})

test_that("Box spread vertically with between", {
  box1 <- boxGrob("B1", x = .2, y = .8)
  box2 <- boxGrob("B2\nmulti\n\n\nline", x = .2, y = .8)
  box3 <- boxGrob("B3", x = .2, y = .8)
  box4 <- boxGrob("B4", x = .2, y = .8)
  box5 <- boxGrob("B5\ntricky", x = .2, y = .8)

  spread_boxes <- spreadVertical(box1,
    box2,
    box3,
    b = box4,
    c = box5,
    type = "between"
  )
  sapply(
    spread_boxes,
    function(b) coords(b)$x %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    expect_equivalent(.2)

  sapply(
    spread_boxes,
    function(b) coords(b)$y %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    length() %>%
    expect_equivalent(length(spread_boxes))
  convertY(coords(spread_boxes[[1]])$top, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 1,
      tolerance = 1e-3
    )

  convertY(coords(tail(spread_boxes, 1)[[1]])$bottom, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 0,
      tolerance = 1e-3
    )

  expect_equivalent(
    distance(spread_boxes[1:2], type = "v"),
    distance(spread_boxes[2:3], type = "v")
  )

  expect_equivalent(
    distance(spread_boxes[3:4], type = "v"),
    distance(spread_boxes[4:5], type = "v")
  )
})

test_that("Box spread horizontally with center", {
  box1 <- boxGrob("B1 some long text", x = .2, y = .8)
  box2 <- boxGrob("B2", x = .2, y = .8)
  box3 <- boxGrob("B3", x = .2, y = .8)
  box4 <- boxGrob("B4", x = .2, y = .8)
  box5 <- boxGrob("B5", x = .2, y = .8)

  spread_boxes <- spreadHorizontal(box1,
    a = box2,
    box3,
    b = box4,
    box5,
    type = "center"
  )
  sapply(
    spread_boxes,
    function(b) coords(b)$y %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    expect_equivalent(.8)

  sapply(
    spread_boxes,
    function(b) coords(b)$x %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    length() %>%
    expect_equivalent(length(spread_boxes))

  convertX(coords(spread_boxes[[1]])$left, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 0,
      tolerance = 1e-3
    )

  convertX(coords(tail(spread_boxes, 1)[[1]])$right, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 1,
      tolerance = 1e-3
    )

  expect_equivalent(
    distance(spread_boxes[1:2], type = "h", center = TRUE),
    distance(spread_boxes[2:3], type = "h", center = TRUE)
  )

  expect_equivalent(
    distance(spread_boxes[3:4], type = "h", center = TRUE),
    distance(spread_boxes[4:5], type = "h", center = TRUE)
  )
})


test_that("Box spread horizontally with between", {
  box1 <- boxGrob("B1 some long text", x = .2, y = .8)
  box2 <- boxGrob("B2", x = .2, y = .8)
  box3 <- boxGrob("B3", x = .2, y = .8)
  box4 <- boxGrob("B4", x = .2, y = .8)
  box5 <- boxGrob("B5", x = .2, y = .8)

  spread_boxes <- spreadHorizontal(box1,
    a = box2,
    box3,
    b = box4,
    box5,
    type = "between"
  )
  sapply(
    spread_boxes,
    function(b) coords(b)$y %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    expect_equivalent(.8)

  sapply(
    spread_boxes,
    function(b) coords(b)$x %>% convertX(unitTo = "npc", valueOnly = TRUE)
  ) %>%
    unique() %>%
    length() %>%
    expect_equivalent(length(spread_boxes))

  convertX(coords(spread_boxes[[1]])$left, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 0,
      tolerance = 1e-3
    )

  convertX(coords(tail(spread_boxes, 1)[[1]])$right, unitTo = "npc", valueOnly = TRUE) %>%
    expect_equivalent(
      expected = 1,
      tolerance = 1e-3
    )

  expect_equivalent(
    distance(spread_boxes[1:2], type = "h"),
    distance(spread_boxes[2:3], type = "h")
  )

  expect_equivalent(
    distance(spread_boxes[3:4], type = "h"),
    distance(spread_boxes[4:5], type = "h")
  )
})

testthat::test_that("alignHorizontal accepts piped list with named reference argument", {
    library(grid)

    org_cohort <- paste("Proximal humerus fracture", "  - >= 18 years", "  - <= 4 weeks of trauma", "  - Not pathological", sep = "\n") |> boxGrob(just = "left")
    surgery <- paste("Surgery", "  - Direct (approx 4%)", "  - Delayed (approx 4%)", sep = "\n") |> boxGrob(just = "left")
    randomize <- boxGrob("Non-surgical\nRandomise")
    treatments <- list(early = boxGrob("Early rehab"), late = boxGrob("Late rehab"), obs = boxGrob("Observation"))
    early_followup <- paste("Early follow-up", "  - 2 weeks [PNRS]", "  - 4 weeks [PNRS]", sep = "\n") |> boxGrob(just = "left")
    late_followup <- paste("Late follow-up", "  - 2-10 months (random) [OSS, PNRS]", "  - 1 year [OSS, PNRS, accelerometer]", "  - 2 years [OSS, PNRS]", "  - 5 years [OSS, PNRS]", sep = "\n") |> boxGrob(just = "left")

    boxes_mid <- spreadVertical(
        start = org_cohort,
        step_1 = list(surgery = surgery, `non-surgical` = randomize),
        treatment = treatments,
        early_followup = early_followup,
        followup = late_followup
    ) |>
        spreadHorizontal(subelement = "step_1") |>
        spreadHorizontal(subelement = "treatment", from = 0.35)

    # The case that previously failed: a piped list and a named 'reference' arg
    expect_silent(result <- boxes_mid |> alignHorizontal(
        reference = c("treatment", "late"),
       subelement = c("step_1", "non-surgical")
    ))

    # Verify the targeted subelement exists and has been replaced (i.e., alignment returned a box)
    target <- Gmisc:::get_list_element_by_path(result, c("step_1", "non-surgical"))
    expect_true(!is.null(target))
})

#' Get statistics according to the type
#'
#' A simple function applied by the \code{\link{getDescriptionStatsBy}}
#' for the total column.
#'
#' @return A matrix or a vector depending on the settings
#'
#' @inheritParams getDescriptionStatsBy
#' @keywords internal
#' @importFrom stats na.omit
prGetStatistics <- function(x,
                            show_perc = FALSE,
                            html = TRUE,
                            digits = 1,
                            digits.nonzero = NA,
                            numbers_first = TRUE,
                            useNA = c("ifany", "no", "always"),
                            useNA.digits = digits,
                            show_all_values = FALSE,
                            continuous_fn = describeMean,
                            factor_fn = describeFactors,
                            prop_fn = factor_fn,
                            percentage_sign = TRUE,
                            default_ref = NULL) {
  # All the describe functions have the same interface
  # so it is useful to gather all the arguments here
  describe_args <- list(
    x = x,
    html = html,
    digits = digits,
    digits.nonzero = digits.nonzero,
    number_first = numbers_first,
    useNA = useNA,
    useNA.digits = useNA.digits,
    percentage_sign = percentage_sign,
    default_ref = default_ref
  )

  if (is.factor(x) ||
    is.logical(x) ||
    is.character(x)) {
    if ((is.factor(x) &&
      length(levels(x)) == 2) ||
      (!is.factor(x) &&
        length(unique(na.omit(x))) == 2)) {
      if (show_perc) {
        total_table <- fastDoCall(prop_fn, describe_args)
      } else {
        total_table <- table(x, useNA = useNA)
        names(total_table)[is.na(names(total_table))] <- "Missing"
        # Choose only the reference level
        if (show_all_values == FALSE) {
          total_table <- total_table[names(total_table) %in%
            c(levels(as.factor(x))[1], "Missing")]
        }
      }
    } else {
      if (show_perc) {
        total_table <- fastDoCall(factor_fn, describe_args)
      } else {
        total_table <- table(x, useNA = useNA) %>%
          txtInt()
        names(total_table)[is.na(names(total_table))] <- "Missing"
      }
    }
  } else {
    total_table <- fastDoCall(continuous_fn, describe_args)

    # If a continuous variable has two rows then it's assumed that the second is the missing
    if (length(total_table) == 2 &&
      show_perc == FALSE) {
      total_table[2] <- sum(is.na(x))
    }
  }
  return(total_table)
}

#' A helper function for the description stats
#'
#' @param x The variable of interest with the levels
#' @param default_ref The default reference, either first,
#'  the level name or a number within the levels. If left out
#'  it defaults to the first value.
#' @return \code{integer} The level number of interest
#'
#' @keywords internal
prDescGetAndValidateDefaultRef <- function(x, default_ref) {
  if (is.null(default_ref)) {
    # Use by default the first factor element (all chars are converted to a factor prior calculation)
    return(1)
  }

  if (is.character(default_ref)) {
    if (default_ref %in% levels(x)) {
      return(which(default_ref == levels(x)))
    }

    stop(
      "You have provided an invalid default reference, '",
      default_ref, "' can not be found among: ", paste(levels(x), collapse = ", ")
    )
  }

  if (!default_ref %in% 1:length(levels(x))) {
    stop(
      "You have provided an invalid default reference,",
      " it is ", default_ref, " while it should be between 1 and ", length(levels(x)),
      " as this is only used for factors."
    )
  }

  return(default_ref)
}

# Transitions
set.seed(1)
n <- 10
my_data <-
  data.frame(
    Var_a = sample(c(
      "Test 1",
      "Test 2",
      "Test 3"
    ),
    size = n,
    replace = TRUE,
    prob = 3:1
    ),
    Var_b = sample(c(
      "Test 1",
      "Test 2",
      "Test 3"
    ),
    size = n,
    replace = TRUE,
    prob = 1:3
    )
  )
mtrx <- with(
  my_data,
  table(Var_a, Var_b)
)

# Initialize the transition plot
transitions <- getRefClass("Transition")$new(mtrx,
                                             label = c("Before", "After"))

# Render the plot
transitions$render()

#' Connect boxes with arrows
#'
#' Creates connectors between boxes.
#'
#' The function supports:
#'
#' - **One-to-one**: a single start box connected to a single end box.
#' - **One-to-many**: a single start box connected to multiple end boxes.
#' - **Many-to-one**: multiple start boxes connected to a single end box.
#'
#' Many-to-many connections are **not supported**.
#'
#' If either `start` or `end` is a list, a list of connector grobs is returned
#' (one per connection). Otherwise a single connector grob is returned.
#'
#' Each connector stores its computed geometry in `attr(x, "line")`
#' (or for each element when a list is returned). This can be reused to construct
#' custom connectors using the calculated coordinates.
#'
#' ## Connector types
#'
#' `type` controls the connector shape:
#'
#' - `"vertical"`: straight vertical connector
#' - `"horizontal"`: straight horizontal connector
#' - `"L"`: vertical then horizontal (direction chosen automatically)
#' - `"-"`: straight horizontal connector at the end box y-position
#' - `"Z"`: horizontal connector with two 90-degree turns
#' - `"N"`: vertical connector with one horizontal segment
#'   When connecting to or from multiple boxes, all connectors share the same bend height.
#' - `"fan_in_top"`: many-to-one connector merging onto the *top edge* of the end box
#'   Attachment points are evenly distributed along the edge (with optional `margin`),
#'   and all connectors share a common bend height.
#' - `"fan_in_center"`: many-to-one connector that aggregates stems onto a horizontal
#'   bus (shared at the computed bend height) and then a single centered trunk with an
#'   arrow that points to the *center* of the end box. Useful when you want a single
#'   arrow to represent the merged flow (e.g. a middle trunk bus).
#'
#' For `type = "N"` and `type = "fan_in_top"` with multi-box connections, a shared
#' bend position is computed so that the horizontal segment aligns visually across
#' all connectors.
#'
#' ## Labels
#'
#' For one-to-one connectors you can add a text label (for example `"yes"` / `"no"`).
#' The label is placed near the midpoint of the connector.

#' The label is drawn with a white background for readability.
#' Use `label_pad` to control padding around the text and `label_offset` to move
#' the label away from the connector.
#'
#' ## Split boxes
#'
#' When connecting to or from a `boxPropGrob`, `subelmnt` controls whether the left
#' or right sub-box x-coordinate is used as the anchor point.
#'
#' @param start A `boxGrob`/`boxPropGrob`, or a list of boxes (many-to-one).
#' @param end A `boxGrob`/`boxPropGrob`, or a list of boxes (one-to-many).
#' @param type Connector type, see Details.
#' @param subelmnt For split boxes, which sub-element to anchor to: `"left"` or `"right"`.
#' @param lty_gp A [grid::gpar()] controlling line appearance. Can also be set globally via
#'   `options(connectGrob = ...)`.
#' @param arrow_obj Arrow specification created with [grid::arrow()]. Can also be set globally via
#'   `options(connectGrobArrow = ...)`.
#' @param arrow_size Optional numeric (mm). When supplied, overrides the arrow head length in
#'   `arrow_obj` without requiring the user to rebuild the entire arrow specification.
#' @param split_pad Padding around the shared bend point for multi-box connections.
#'   Numeric values are interpreted as millimeters.
#' @param margin For `type = "fan_in_top"`, the margin applied at the left and right ends of the
#'   end box top edge before distributing attachment points. Numeric values are interpreted
#'   as millimeters.
#' @param smooth Logical; if `TRUE` interior orthogonal corners are replaced by smooth
#'   Bézier arcs. Default `FALSE` preserves the existing sharp-corner behaviour.
#' @param corner_radius Radius of the rounded corner arc when `smooth = TRUE`.
#'   A [grid::unit()] or numeric (interpreted as millimeters). Default `unit(3, "mm")`.
#' @param side For `type = "side"`, which side of the start box to exit from.
#'   `"auto"` (default) picks the side that faces the end box.
#' @param label Optional text label for one-to-one connectors (e.g. `"yes"` / `"no"`).
#'   Only supported when both `start` and `end` are single boxes.
#' @param label_gp A [grid::gpar()] controlling label appearance.
#' @param label_pos Where to place the label along the connector: `"mid"`, `"near_start"`, or `"near_end"`.
#' @param label_offset Offset for the label away from the connector line.
#' @param label_pad Padding inside the label background. Numeric values are interpreted
#'   as millimeters.
#' @param label_bg_gp A [grid::gpar()] controlling label background appearance.
#'   Defaults to a white background with no border.
#'
#' @return
#' - One-to-one: a [grid::grob()] with class `"connect_boxes"`.
#' - One-to-many or many-to-one: a list of grobs with class `"connect_boxes_list"`.
#'
#' @family flowchart components
#' @export
#' @rdname connect
#' @example inst/examples/connectGrob_example.R
#' @md
connectGrob <- function(
  start,
  end,
  type = c("vertical", "horizontal", "L", "-", "Z", "N", "fan_in_top", "fan_in_center", "side"),
  subelmnt = c("right", "left"),
  lty_gp = getOption("connectGrob", default = gpar(fill = "black")),
  arrow_obj = getOption("connectGrobArrow", default = arrow(ends = "last", type = "closed")),
  arrow_size = NULL,
  split_pad = unit(2, "mm"),
  margin = unit(2, "mm"),
  smooth = FALSE,
  corner_radius = unit(3, "mm"),
  side = c("auto", "left", "right"),
  label = NULL,
  label_gp = grid::gpar(cex = 0.9),
  label_bg_gp = grid::gpar(fill = "white", col = NA),
  label_pad = unit(1.5, "mm"),
  label_pos = c("mid", "near_start", "near_end"),
  label_offset = unit(2, "mm")
) {
  type <- match.arg(type)
  label_pos <- match.arg(label_pos)
  side <- match.arg(side)

  if (!is.null(arrow_size)) {
    ends_map <- c("1" = "first", "2" = "last", "3" = "both")
    type_map <- c("1" = "open", "2" = "closed")
    arrow_obj <- arrow(
      ends = ends_map[as.character(arrow_obj$ends)],
      type = type_map[as.character(arrow_obj$type)],
      angle = arrow_obj$angle,
      length = unit(arrow_size, "mm")
    )
  }

  # Normalize possible single-element wrapped lists so strategy detection
  # correctly identifies lists of boxes (one-to-many / many-to-one).
  start <- prFlattenBoxListIfNeeded(start)
  end <- prFlattenBoxListIfNeeded(end)

  # Also unwrap a first nested container when it clearly contains boxes so that
  # the connector functions operate on the inner boxes rather than an outer
  # grouping wrapper (this handles cases produced by spread/align pipelines).
  start <- prMaybeUnwrapFirstContainerBoxes(start)

  # If the *original* end is a list of group containers (each a list of boxes),
  # the intention is usually to connect to the group's first element (header).
  # Convert each container to its first boxed element *before* any unwrapping
  # logic runs so we don't accidentally drop other groups.
  if (is.list(end) && length(end) > 0 && all(vapply(end, is.list, logical(1)))) {
    is_container_of_boxes <- vapply(end, function(el) {
      length(el) > 0 && all(vapply(el, inherits, logical(1), "box"))
    }, logical(1))
    if (all(is_container_of_boxes)) {
      end <- lapply(end, function(el) el[[1]])
    }
  }

  end <- prMaybeUnwrapFirstContainerBoxes(end)

  # Ensure elements are normalized so single-element wrappers are unwrapped
  # (e.g., list(box, list(box), box) -> list(box, box, box)).
  start <- prNormalizeBoxElements(start)
  end <- prNormalizeBoxElements(end)

  # Collapse single-element lists containing a box into the bare box so that
  # passing `list(box)` does not accidentally make both `start` and `end` be
  # lists (which is an unsupported many-to-many case).
  start <- prCollapseSingleBoxList(start)
  end <- prCollapseSingleBoxList(end)


  # If any top-level elements of `end` are containers (lists) take their
  # first element as the representative header when that first element is a
  # box. This is a permissive rule that handles mixed inputs produced by
  # spread/align pipelines without failing.
  # Only treat a *multi-element* list as a set of groups; if `end` is a
  # single-element list (e.g., a boxed element wrapped as a list) we should
  # not unwrap it here as that is used by other connector strategies.
  if (!inherits(end, "box") && is.list(end) && length(end) > 1 && any(vapply(end, is.list, logical(1)))) {
    end <- lapply(end, function(el) {
      if (is.list(el) && length(el) > 0 && inherits(el[[1]], "box")) {
        return(el[[1]])
      }
      el
    })
  }

  # If we are connecting *many-to-one* (multiple starts to a single end) it is
  # common for layout pipelines to produce a container-like structure for the
  # end (for example when using `.subelement` or grouped spreads). If such a
  # container accidentally reached this point, attempt to find the most
  # appropriate boxed element to use as the end target (prefer the first
  # boxed child). This keeps many-to-one strategies robust to mixed inputs.
  if ((prIsBoxList(start) || inherits(start, "Gmisc_list_of_boxes")) && !inherits(end, "box") && is.list(end) && !prIsBoxList(end)) {
    box_pos <- which(vapply(end, inherits, logical(1), "box"))
    if (length(box_pos) >= 1) {
      end <- end[[box_pos[1]]]
    }
  }


  strategy <- prGetConnectorStrategy(start, end, type)
  prCalculateConnector(
    strategy,
    start = start,
    end = end,
    subelmnt = subelmnt,
    lty_gp = lty_gp,
    arrow_obj = arrow_obj,
    split_pad = split_pad,
    margin = margin,
    smooth = smooth,
    corner_radius = corner_radius,
    side = side,
    label = label,
    label_gp = label_gp,
    label_bg_gp = label_bg_gp,
    label_pad = label_pad,
    label_pos = label_pos,
    label_offset = label_offset
  )
}


#' The print/plot calls the \code{\link[grid]{grid.draw}} function on the object
#' @param x The grob to print/plot
#' @param ... Passed to \code{\link[grid]{grid.draw}}
#' @rdname connect
#' @export
print.connect_boxes <- function(x, ...) {
  grid.draw(x, ...)
  # If labels attached, draw them on top
  labels <- attr(x, "connector_labels")
  if (!is.null(labels)) {
    gp <- attr(x, "connector_label_gp")
    if (is.null(gp)) gp <- gpar(cex = 0.9)
    bg_gp <- attr(x, "connector_label_bg_gp")
    if (is.null(bg_gp)) bg_gp <- gpar(fill = "white", col = NA)
    off <- attr(x, "connector_label_offset")
    if (is.null(off)) off <- list(x_offset = unit(0, "mm"), y_offset = unit(0, "mm"))
    labelConnector(list(x), labels = labels, x_offset = off$x_offset, y_offset = off$y_offset, gp = gp, bg_gp = bg_gp)
  }

  invisible(x)
}

#' @rdname connect
#' @export
plot.connect_boxes <- print.connect_boxes


#' @rdname connect
#' @param recording Passed to [grid::grid.draw()] when rendering each element in
#'  the list. Defaults to `TRUE` (record drawing for later replay).
#' @export
#' @importFrom grid grid.draw
grid.draw.connect_boxes_list <- function(x, recording = TRUE) {
  for (g in x) grid.draw(g, recording = recording)
  # If labels attached, draw them on top
  labels <- attr(x, "connector_labels")
  if (!is.null(labels)) {
    gp <- attr(x, "connector_label_gp")
    if (is.null(gp)) gp <- gpar(cex = 0.9)
    bg_gp <- attr(x, "connector_label_bg_gp")
    if (is.null(bg_gp)) bg_gp <- gpar(fill = "white", col = NA)
    off <- attr(x, "connector_label_offset")
    if (is.null(off)) off <- list(x_offset = unit(0, "mm"), y_offset = unit(0, "mm"))
    lbl_obj <- labelConnector(x, labels = labels, x_offset = off$x_offset, y_offset = off$y_offset, gp = gp, bg_gp = bg_gp)
    print(lbl_obj)
  }
}

#' @rdname connect
#' @export
print.connect_boxes_list <- function(x, ...) {
  grid.draw(x)
  invisible(x)
}

#' @rdname connect
#' @export
plot.connect_boxes_list <- print.connect_boxes_list

#' Add [Hmisc::unit()] to multiple columns
#'
#' Add label attribute using `dplyr` syntax using the [Hmisc::unit()]
#'
#' @param x The data frame that we want to define units on
#' @param ... Variable names with their intended unit, e.g. `hp = "Hp"`.
#'
#' @return The original data.frame
#' @export
#'
#' @importFrom Hmisc units
#' @examples
#' library(magrittr)
#' data(mtcars)
#' mtcars_with_units <- mtcars %>%
#'   set_column_units(wt = "1000 lbs")
#' Hmisc::units(mtcars_with_units$wt)
#' @family Hmisc helpers
set_column_units <- function(x, ...) {
  units_to_set <- list(...)
  stopifnot(is.data.frame(x))

  stopifnot(all(sapply(units_to_set, is.character)))
  stopifnot(all(sapply(units_to_set, length) == 1))
  stopifnot(all(names(units_to_set) %in% colnames(x)))

  for (n in names(units_to_set)) {
    units(x[[n]]) <- units_to_set[[n]]
  }

  x
}

#' Insert a row into a matrix
#'
#' Inserts a row and keeps the attributes \code{\link{copyAllNewAttributes}}
#'
#' @param m matrix
#' @param r row number where the new row should be inserted
#' @param v optional values for the new row
#' @param rName optional character string: the name of the new row.
#' @return \code{matrix} Returns a matrix with one more row than the provided matrix \code{m}
#'
#' @examples
#' test <- matrix(1:4, ncol = 2)
#' attr(test, "wow") <- 1000
#' test <- insertRowAndKeepAttr(test, 2)
#' print(attr(test, "wow"))
#' @export
#' @author Max Gordon, Arne Henningsen
insertRowAndKeepAttr <- function(m, r, v = NA, rName = "") {
  if (!inherits(m, "matrix")) {
    stop("argument 'm' must be a matrix")
  }
  if (r == as.integer(r)) {
    r <- as.integer(r)
  }
  else {
    stop("argument 'r' must be an integer")
  }
  if (length(r) != 1) {
    stop("argument 'r' must be a scalar")
  }
  if (r < 1) {
    stop("argument 'r' must be positive")
  }
  if (r > nrow(m) + 1) {
    stop(
      "argument 'r' must not be larger than the number of rows",
      " of matrix 'm' plus one"
    )
  }
  if (!is.character(rName)) {
    stop("argument 'rName' must be a character string")
  }
  if (length(rName) != 1) {
    stop("argument 'rName' must be a be a single character string")
  }
  nr <- nrow(m)
  nc <- ncol(m)
  rNames <- rownames(m)
  if (is.null(rNames) & rName != "") {
    rNames <- rep("", nr)
  }
  if (r == 1) {
    m2 <- rbind(matrix(v, ncol = nc), m)
    if (!is.null(rNames)) {
      rownames(m2) <- c(rName, rNames)
    }
  }
  else if (r == nr + 1) {
    m2 <- rbind(m, matrix(v, ncol = nc))
    if (!is.null(rNames)) {
      rownames(m2) <- c(rNames, rName)
    }
  }
  else {
    m2 <- rbind(
      m[1:(r - 1), , drop = FALSE], matrix(v, ncol = nc),
      m[r:nr, , drop = FALSE]
    )
    if (!is.null(rNames)) {
      rownames(m2) <- c(rNames[1:(r - 1)], rName, rNames[r:nr])
    }
  }

  return(copyAllNewAttributes(from = m, to = m2))
}

library("testthat")
context("insertRowAndKeepAttr")

test_that("Keeps attributes", {
  test <- matrix(1:4, ncol = 2)
  attr(test, "wow") <- 1000
  test <- insertRowAndKeepAttr(test, 1)

  expect_equal(attr(test, "wow"), 1000)
  expect_equal(nrow(test), 3)
})

test_that("Keeps class", {
  test <- matrix(1:4, ncol = 2)
  attr(test, "wow") <- 1000
  class(test) <- c("new_class", class(test))
  test <- insertRowAndKeepAttr(test, 1)

  expect_true("new_class" %in% class(test)[1])
})

library(lineprof)
library(Gmisc)
library(grid)
grid.newpage()
lineprof({
  arrowGrob <- bezierArrowSmpl(x = c(.1,.3,.6,.9),
                                   y = c(0.2, 0.2, 0.9, 0.9))
}) -> lp
grid.draw(arrowGrob)

lineprof::shine(lp)

# Time before optimization: 0.197 seconds
# Time after optimization: 0.029 - 0.005 seconds

lineprof({
  arrowGrob <- bezierArrowGradient(x = c(.1,.3,.6,.9),
                                   y = c(0.2, 0.2, 0.9, 0.9))
}) -> lp

lineprof::shine(lp)

# Time before optimization: 0.284 seconds
# Time after optimization: 0.011 - 0.006 seconds

#' @title Normalize box justification
#' @description Convert a variety of `bjust` specifications (character
#'   tokens like "left", "top", "center" or numeric vectors) into a
#'   numeric vector of length 2 representing x and y justification in
#'   [0,1]. Unspecified axis positions are returned as `NA_real_`.
#' @param bjust Character or numeric justification token(s) or `NULL`.
#' @return Numeric vector length 2: c(x, y) justification (values in [0,1]
#'   or `NA_real_` for unspecified axes).
#' @keywords internal
#' @noRd
prJustToNumeric <- function(bjust) {
  map_one <- function(s) {
    if (is.na(s)) {
      return(NA_real_)
    }
    if (is.character(s)) s <- tolower(s)
    if (is.character(s)) {
      if (s %in% c("left", "bottom")) {
        return(0)
      }
      if (s %in% c("center", "centre")) {
        return(0.5)
      }
      if (s %in% c("right", "top")) {
        return(1)
      }
      num <- suppressWarnings(as.numeric(s))
      if (!is.na(num)) {
        return(num)
      }
      return(NA_real_)
    }
    if (is.numeric(s)) {
      return(as.numeric(s))
    }
    NA_real_
  }

  if (is.null(bjust)) {
    return(c(0.5, 0.5))
  }
  if (is.numeric(bjust)) {
    if (length(bjust) == 1) {
      return(pmin(pmax(c(bjust, NA_real_), 0), 1))
    }
    vals <- as.numeric(bjust)
    if (length(vals) >= 2) vals <- vals[1:2]
    if (length(vals) == 1) vals <- c(vals, NA_real_)
    return(pmin(pmax(vals, 0), 1))
  }
  if (is.character(bjust)) {
    if (length(bjust) == 1) {
      s <- tolower(bjust)
      if (s %in% c("left", "right", "centre", "center")) {
        return(pmin(pmax(c(map_one(bjust), NA_real_), 0), 1))
      }
      if (s %in% c("top", "bottom")) {
        return(pmin(pmax(c(NA_real_, map_one(bjust)), 0), 1))
      }
      # generic single token (e.g. "center") -> apply to both
      v <- map_one(bjust)
      return(pmin(pmax(c(v, v), 0), 1))
    }
    # length >=2
    v1 <- map_one(bjust[1])
    v2 <- map_one(bjust[2])
    vals <- c(v1, v2)
    return(pmin(pmax(vals, 0), 1))
  }

  c(0.5, 0.5)
}

#' @title Fraction of a unit in NPC
#' @description Return a `unit(..., "npc")` that represents `x` divided by
#'   `divide_by`. `divide_by` may be numeric or a `unit` (in which case the
#'   unit is converted to NPC first). Useful for computing half/third
#'   fractions of a size in NPC units.
#' @param x A `unit` or numeric-like size to be expressed as a fraction.
#' @param divide_by Numeric or `unit` specifying the divisor.
#' @param axis Character, either "x" or "y" to indicate width/height.
#' @return A `unit` in NPC representing `x/divide_by`.
#' @keywords internal
#' @noRd
prUnitFraction <- function(x, divide_by, axis = c("x", "y")) {
  axis <- match.arg(axis)
  if (is.unit(divide_by)) {
    divide_by_val <- prGetNpcValue(divide_by, axis)
  } else if (is.numeric(divide_by)) {
    divide_by_val <- divide_by
  } else {
    stop("divide_by must be a numeric or unit object", call. = FALSE)
  }

  if (is.na(divide_by_val) || divide_by_val == 0) {
    stop("Cannot divide by NA or zero", call. = FALSE)
  }

  unit(prGetNpcSize(x, axis) / divide_by_val, "npc")
}

#' @title Axis-specific justification
#' @description Extract the numeric justification for a single axis from a
#'   `bjust` specification. Returns `NA_real_` when the axis is unspecified.
#' @param bjust Character or numeric justification token(s).
#' @param axis Character, either "x" or "y".
#' @return Numeric in [0,1] or `NA_real_`.
#' @keywords internal
#' @noRd
prJustForAxis <- function(bjust, axis = c("x", "y")) {
  axis <- match.arg(axis)
  bnum <- prJustToNumeric(bjust)
  if (axis == "x") {
    return(bnum[1])
  }
  bnum[2]
}

#' @title Adjust position by justification
#' @description Adjust a `pos` (grid `unit`) by the half-size offset implied
#'   by `bjust` for one axis. If the axis justification is unspecified
#'   (`NA`) the original `pos` is returned unchanged.
#' @param bjust Character or numeric justification token(s).
#' @param pos A grid `unit` representing the position to adjust.
#' @param size A grid `unit` or numeric-like size used to compute the offset.
#' @param axis Character, either "x" or "y".
#' @return Adjusted `unit` position.
#' @keywords internal
#' @noRd
prAdjustPos <- function(bjust, pos, size, axis = c("x", "y")) {
  axis <- match.arg(axis)
  just_val <- prJustForAxis(bjust, axis)
  if (is.na(just_val)) {
    return(pos)
  }

  if (!is.unit(size)) size <- unit(size, "npc")

  offset <- size * (0.5 - just_val)
  pos + offset
}

#' @title Compute x position for text within a box
#' @description Helper to compute an x position for text inside a box given a
#'   simple `just` token and a `txt_padding` unit.
#' @param just Character like "left", "right", or other.
#' @param txt_padding A `unit` used as padding from the box edge.
#' @return A `unit` representing the x coordinate.
#' @keywords internal
#' @noRd
prGetX4Txt <- function(just, txt_padding) {
  x <- .5
  if (just == "left") {
    x <- txt_padding
  } else if (just == "right") {
    x <- unit(1, "npc") - txt_padding
  }
  return(x)
}

#' @title Convert text to height (mm)
#' @description Compute the height in millimetres of a multi-line text
#'   string, using the number of lines and converting `lines` to mm.
#' @param str Character string (may contain newlines).
#' @return Numeric height in millimetres.
#' @keywords internal
#' @noRd
prConvTxt2Height <- function(str) {
  if (missing(str)) {
    return(0)
  }

  length(strsplit(as.character(str), "\n")[[1]]) |>
    unit("lines") |>
    prConvertHeightToMm()
}

# Helpers for deep-path retrieval / assignment in nested box lists
#' @title Safe deep-list retrieval
#' @description Retrieve a nested element from a list by a path specified as
#'   a vector/list of names or numeric indices. Returns `NULL` when the path
#'   is invalid or not applicable.
#' @param x A list-like object to index into.
#' @param path Character or numeric vector (or list) specifying the path.
#' @return The located element or `NULL` if not found/invalid.
#' @keywords internal
#' @noRd
get_list_element_by_path <- function(x, path) {
  if (is.null(path)) {
    return(NULL)
  }
  # Only atomic vectors (character/numeric) or lists are valid paths. For
  # other object types (e.g., a `unit`), return NULL rather than attempting to
  # coerce and index, which can produce confusing errors.
  if (!is.list(path)) {
    # If the path is a unit (or similar), treat as not a path.
    if (exists("is.unit", mode = "function") && is.unit(path)) {
      return(NULL)
    }
    if (!is.atomic(path)) {
      return(NULL)
    }
    path <- as.list(path)
  }

  for (seg in path) {
    # Accept numeric indices (including numeric-strings) or character names
    if (is.numeric(seg) || (is.character(seg) && grepl("^[0-9]+$", seg))) {
      idx <- as.integer(seg)
      # Numeric indices must be valid 1-based indices into the current list
      if (!is.list(x) || length(idx) == 0 || idx < 1 || idx > length(x)) {
        return(NULL)
      }
    } else if (is.character(seg)) {
      idx <- seg
      # If the name isn't present, treat as not-found
      if (!is.list(x) || is.null(names(x)) || !(idx %in% names(x))) {
        return(NULL)
      }
    } else {
      # Non-character, non-numeric segment (e.g., a unit object) is not a valid path
      return(NULL)
    }

    if (is.null(x)) {
      return(NULL)
    }
    x <- x[[idx]]
  }
  x
}

#' @title Safe deep-list assignment
#' @description Set a nested element inside a list by path, creating
#'   intermediate lists as needed.
#' @param x A list to modify.
#' @param path Character or numeric vector (or list) specifying the path.
#' @param value Value to assign at the specified path.
#' @return Modified list with `value` inserted.
#' @keywords internal
#' @noRd
set_list_element_by_path <- function(x, path, value) {
  if (!is.list(path)) path <- as.list(path)
  if (length(path) == 0) {
    return(x)
  }
  seg <- path[[1]]
  if (is.numeric(seg) || (is.character(seg) && grepl("^[0-9]+$", seg))) seg <- as.integer(seg)
  if (length(path) == 1) {
    x[[seg]] <- value
    return(x)
  }
  if (is.null(x[[seg]])) x[[seg]] <- list()
  x[[seg]] <- set_list_element_by_path(x[[seg]], path[-1], value)
  x
}

#' @title Extend object class
#' @description Prepend `new_class` to an object's class attribute, ensuring
#'   uniqueness.
#' @param obj An R object.
#' @param new_class Single string with the class name to add.
#' @return The object with updated class attribute.
#' @keywords internal
#' @noRd
prExtendClass <- function(obj, new_class) {
  if (!is.character(new_class) || length(new_class) != 1) {
    stop("new_class must be a single string")
  }

  class(obj) <- unique(c(new_class, class(obj)))
  obj
}

testthat::test_that("get/set list element by path works", {
    # Tests run inside package check; no need to load the package here

    l <- list(
        arms = list(
            early = list("E1", "E2"),
            late = list("L1")
        ),
        detail = list(list("D_early"), list("D_late"))
    )

    expect_equal(get_list_element_by_path(l, c("detail", 1))[[1]], "D_early")
    l2 <- set_list_element_by_path(l, c("detail", 1), list("NEW"))
    expect_equal(l2$detail[[1]][[1]], "NEW")

    # numeric given as character should also work
    expect_equal(get_list_element_by_path(l, c("detail", "1"))[[1]], "D_early")

    # Non-character atomic path (e.g., unit) should not error and return NULL
    library(grid)
    boxes <- list(start = list(a = 1), arms = list(early = "E"))
    expect_null(get_list_element_by_path(boxes, unit(0.25, "npc")))
})

library(testthat)

test_that("prExtendClass avoids duplicates", {
  obj <- list(a = 1)
  class(obj) <- c("my_class", "list")

  # First extension
  obj2 <- prExtendClass(obj, "my_class")
  expect_equal(class(obj2), c("my_class", "list"))

  # Second extension with new class
  obj3 <- prExtendClass(obj2, "new_class")
  expect_equal(class(obj3), c("new_class", "my_class", "list"))

  # Re-adding existing class
  obj4 <- prExtendClass(obj3, "my_class")
  expect_equal(class(obj4), c("my_class", "new_class", "list"))

  # But is "my_class" now first?
  # unique(c("my_class", "new_class", "my_class", "list")) -> "my_class", "new_class", "list"
  # Yes, unique keeps the first occurrence.
})

test_that("prExtendClass requires single string", {
  expect_error(prExtendClass(list(), c("A", "B")))
})

test_that("Reproduce Gmisc_list_of_boxes duplication scenario", {
  # Simulate how spread/align might chain
  # spreadVertical returns Gmisc_list_of_boxes
  # alignVertical checks and re-adds it?

  l <- list(dummy = 1)
  # Mimic spreadVertical
  l <- prExtendClass(l, "Gmisc_list_of_boxes")
  expect_equal(class(l), c("Gmisc_list_of_boxes", "list"))

  # Mimic alignVertical on the result
  l <- prExtendClass(l, "Gmisc_list_of_boxes")
  expect_equal(class(l), c("Gmisc_list_of_boxes", "list"))

  # What if we had duplicates initially?
  class(l) <- c("Gmisc_list_of_boxes", "Gmisc_list_of_boxes", "list")
  l <- prExtendClass(l, "Gmisc_list_of_boxes")
  expect_equal(class(l), c("Gmisc_list_of_boxes", "list"))
})

#' Get the distance between grid objects
#'
#' Retrieves the distance between two boxes as absolute \code{"mm"} units. The function also
#' accepts \code{\link{coords}} objects as well as a \code{\link[grid]{unit}} or a numeric input.
#'
#' @param box1 The first \code{\link{boxGrob}}. Can also be a \code{\link{coords}}
#'  object, a \code{\link[grid]{unit}} or a numeric. The latter is evaluated to
#'  a \code{unit} with \code{units="npc"}.
#' @param box2 The second object to calculate the distance to. Same type as for \code{box1}.
#' @param type Whether we should retrieve the vertical, horizontal or euclidean distance
#' @param half If set to true it returns half the distance. This is convenient
#'  when positioning boxes between each other.
#' @param center Calculate the distance from the center of each object
#' @return A \code{unit} in \code{"mm"} with an absolute value. The attribute
#'  \code{positive} indicates the direction of the value, i.e. if it is \code{TRUE} the
#'  distance was calculated from the first to the second, otherwise it is \code{FALSE}.
#'  For \code{euclidean} distance the \code{positive} attribute is \code{NA}. There is also the
#'  \code{from} and \code{to} attributes that has the coordinates that were used for the
#'  calculations, for \code{euclidean} distance this is \code{NA}.
#'
#' @family flowchart components
#' @importFrom checkmate assert_class assert checkNumeric checkClass checkTRUE checkList
#' @importFrom grid is.unit
#' @export
#' @examples
#' box1 <- boxGrob("A test box", y = .8)
#' box2 <- boxGrob("Another test box", y = .2)
#' distance(box1, box2, "v")
#' @rdname distance
distance <- function(box1,
                     box2,
                     type = c("vertical", "horizontal", "euclidean"),
                     half = FALSE,
                     center = FALSE) {
  assert_input <- function(v) {
    assert(
      checkClass(v, "box"),
      checkClass(v, "coords"),
      checkList(v, min.len = 1),
      checkNumeric(v),
      checkTRUE(is.unit(v))
    )
  }


  type <- match.arg(type)
  if (missing(box2) && is.list(box1) && length(box1) == 2) {
    box2 <- box1[[2]]
    box1 <- box1[[1]]
  }
  assert_input(box1)
  assert_input(box2)
  box_coords1 <- prConvert2Coords(box1)
  box_coords2 <- prConvert2Coords(box2)

  type <- match.arg(type)
  converter_fn <- if (type == "horizontal") prConvertWidthToMm else prConvertHeightToMm
  from <- NA
  to <- NA
  if (type == "vertical") {
    if (converter_fn(box_coords1$y) > converter_fn(box_coords2$y)) {
      if (center) {
        from <- box_coords1$y
        to <- box_coords2$y
      } else {
        from <- box_coords1$bottom
        to <- box_coords2$top
      }
    } else {
      if (center) {
        from <- box_coords1$y
        to <- box_coords2$y
      } else {
        from <- box_coords1$top
        to <- box_coords2$bottom
      }
    }
    ret <- converter_fn(to) - converter_fn(from)
  } else if (type == "horizontal") {
    if (prConvertWidthToMm(box_coords1$x) < prConvertWidthToMm(box_coords2$x)) {
      if (center) {
        from <- box_coords1$x
        to <- box_coords2$x
      } else {
        from <- box_coords1$right
        to <- box_coords2$left
      }
    } else {
      if (center) {
        from <- box_coords1$x
        to <- box_coords2$x
      } else {
        from <- box_coords1$left
        to <- box_coords2$right
      }
    }
    ret <- converter_fn(to) - converter_fn(from)
  } else if (type == "euclidean") {
    ydist <- distance(box1 = box1, box2 = box2, type = "vertical", center = center)
    xdist <- distance(box1 = box1, box2 = box2, type = "horizontal", center = center)
    ret <- sqrt(prConvertHeightToMm(ydist)^2 + prConvertWidthToMm(xdist)^2)
  } else {
    stop("Unreachable code")
  }

  if (ret < 0) {
    positive <- FALSE
    ret <- -1 * ret
  } else {
    positive <- TRUE
  }

  if (half) {
    ret <- ret / 2
  }

  ret <- unit(ret, "mm")
  structure(
    ret,
    class = c("Gmisc_unit", class(ret)),
    positive = positive,
    from = from,
    to = to,
    type = type,
    box_coords1 = box_coords1,
    box_coords2 = box_coords2,
    center = center
  )
}

#' @rdname distance
#' @param x A unit with from the \code{distance} function
#' @param ... Passed on to print
#' @export
print.Gmisc_unit <- function(x, ...) {
  base_txt <- as.character(x)
  repr <- paste(
    base_txt,
    paste0(" - positive = ", as.character(attr(x, "positive"))),
    paste0(" - from ", as.character(attr(x, "from"))),
    paste0(" - to   ", as.character(attr(x, "to"))),
    paste0(" - type: ", as.character(attr(x, "type"))),
    paste0(" - center: ", as.character(attr(x, "center"))),
    "",
    sep = "\n"
  )
  cat(repr)
  invisible(x)
}

par_org <- par(ask = TRUE)
library(grid)
pg <- bezierArrowSmpl(
  x = c(.2, .5, .5, .8),
  y = c(.2, .2, .8, .8),
  width = unit(6, "mm"),
  arrow = list(
    base = unit(.1, "npc"),
    length = unit(.1, "npc")
  )
)

plot.new()
grid.draw(pg)

plot.new()
pg <- bezierArrowSmpl(
  x = c(.2, .2, .8, .3),
  y = c(.3, .9, .8, .2)
)
grid.draw(pg)

plot.new()
pg <- bezierArrowSmpl(
  x = c(.3, .7, -.1, .25),
  y = c(.5, .1, .1, .5),
  align_2_axis = FALSE
)
grid.draw(pg)
par(par_org)

prLineMidpoint <- function(line) {
  stopifnot(is.list(line), inherits(line$x, "unit"), inherits(line$y, "unit"))
  stopifnot(length(line$x) == length(line$y), length(line$x) >= 2)

  x <- prConvertWidthToMm(line$x)
  y <- prConvertHeightToMm(line$y)

  dx <- diff(x)
  dy <- diff(y)
  seg <- sqrt(dx^2 + dy^2)
  total <- sum(seg)

  if (!is.finite(total) || total <= 0) {
    # Fallback: first point (return as npc units)
    return(list(x = prConvertMmToNpc(x[1], "x"), y = prConvertMmToNpc(y[1], "y")))
  }

  half <- total / 2
  cs <- c(0, cumsum(seg))
  i <- max(which(cs <= half))

  # If the selected segment has zero or non-finite length, fall back to the
  # start point of that segment. Otherwise compute the fractional position
  # along the segment (works for single-segment lines as well).
  if (!is.finite(seg[i]) || seg[i] <= 0) {
    return(list(x = prConvertMmToNpc(x[i], "x"), y = prConvertMmToNpc(y[i], "y")))
  }

  t <- (half - cs[i]) / seg[i]
  xm <- x[i] + t * (x[i + 1] - x[i])
  ym <- y[i] + t * (y[i + 1] - y[i])

  list(x = prConvertMmToNpc(xm, "x"), y = prConvertMmToNpc(ym, "y"))
}

prLineLabelPos <- function(line, prefer = c("auto", "horizontal", "longest")) {
  prefer <- match.arg(prefer)
  stopifnot(is.list(line), inherits(line$x, "unit"), inherits(line$y, "unit"))

  xs <- vapply(line$x, \(u) prGetNpcValue(u, "x"), numeric(1))
  ys <- vapply(line$y, \(u) prGetNpcValue(u, "y"), numeric(1))

  if (length(xs) != length(ys) || length(xs) < 2) {
    stop("Invalid line geometry for label placement", call. = FALSE)
  }

  # segment i is between i and i+1
  dx <- abs(diff(xs))
  dy <- abs(diff(ys))

  is_h <- dy < 1e-12
  is_v <- dx < 1e-12

  pick <- function() {
    if (prefer %in% c("auto", "horizontal") && any(is_h)) {
      # if there are multiple horizontals: pick the longest (in npc)
      return(which.max(dx * is_h))
    }
    if (prefer %in% c("auto", "longest")) {
      return(which.max(dx + dy))
    }
    # fallback: first segment
    1L
  }

  i <- pick()
  x_mid <- (xs[i] + xs[i + 1]) / 2
  y_mid <- (ys[i] + ys[i + 1]) / 2

  list(x = unit(x_mid, "npc"), y = unit(y_mid, "npc"))
}

prLabelWithBackground <- function(label, x, y,
                                  label_gp = gpar(cex = 1),
                                  pad = unit(1.5, "mm"),
                                  bg_gp = gpar(fill = "white", col = NA)) {
  tg <- textGrob(label, x = x, y = y, just = "center", gp = label_gp)

  w <- grobWidth(tg) + 2 * pad
  h <- grobHeight(tg) + 2 * pad

  bg <- rectGrob(x = x, y = y, width = w, height = h, just = "center", gp = bg_gp)

  grobTree(bg, tg)
}


prConnect1 <- function(
  start,
  end,
  type,
  subelmnt,
  lty_gp,
  arrow_obj,
  smooth = FALSE,
  corner_radius = unit(3, "mm"),
  side = c("auto", "left", "right"),
  label = NULL,
  label_gp = grid::gpar(cex = 0.9),
  label_bg_gp = grid::gpar(fill = "white", col = NA),
  label_pos = c("mid", "near_start", "near_end"),
  label_offset = unit(2, "mm"),
  label_pad = unit(1.5, "mm")
) {
  assert_class(start, "box")
  assert_class(end, "box")
  assert_class(lty_gp, "gpar")
  assert_class(arrow_obj, "arrow")
  if (!is.null(label)) assert_class(label_gp, "gpar")

  label_pos <- match.arg(label_pos)
  side <- match.arg(side)

  start <- coords(start)
  end <- coords(end)

  if (missing(subelmnt)) {
    subelmnt <- ""
  } else {
    subelmnt <- sprintf("%s_", match.arg(subelmnt, c("right", "left")))
  }

  getX4elmnt <- function(elmnt, side = c("left", "right", "x")) {
    side <- match.arg(side)
    if (side == "x" && !is.null(elmnt[[sprintf("%s%s", subelmnt, side)]])) {
      return(elmnt[[sprintf("%s%s", subelmnt, side)]])
    }
    elmnt[[side]]
  }

  line <- list(x = NULL, y = NULL)
  if (type %in% c("L", "-")) {
    if (type == "-") {
      line$y <- unit.c(end$y, end$y, end$y)
    } else {
      line$y <- unit.c(start$bottom, end$y, end$y)
    }
    start_x_mm    <- prConvertWidthToMm(getX4elmnt(start, "x"))
    end_x_mm      <- prConvertWidthToMm(getX4elmnt(end,   "x"))
    end_right_mm  <- prConvertWidthToMm(end$right)
    end_left_mm   <- prConvertWidthToMm(end$left)
    if (start_x_mm < end_x_mm) {
      # Start is to the left of end centre → approach from the left
      h_end <- end$left
      if (prConvertWidthToMm(h_end) <= start_x_mm) {
        # Overlap: end's left edge crosses start.x (box is wide).
        # Fall back to end centre so the horizontal segment still goes rightward.
        h_end <- getX4elmnt(end, "x")
      }
    } else if (end_right_mm < start_x_mm) {
      # Normal right-approach: end's right edge is clearly left of start → use it
      h_end <- end$right
    } else {
      # Overlap: end's right edge crosses start.x (box is wide).
      # Fall back to end centre so the horizontal segment still goes leftward.
      h_end <- getX4elmnt(end, "x")
    }
    line$x <- unit.c(getX4elmnt(start, "x"), getX4elmnt(start, "x"), h_end)
  } else if (type == "Z") {
    if (prConvertWidthToMm(start$x) < prConvertWidthToMm(end$x)) {
      line$x <- unit.c(
        start$right,
        start$right + distance(start, end, type = "h", half = TRUE),
        start$right + distance(start, end, type = "h", half = TRUE),
        end$left
      )
    } else {
      line$x <- unit.c(
        start$left,
        start$left - distance(start, end, type = "h", half = TRUE),
        start$left - distance(start, end, type = "h", half = TRUE),
        end$right
      )
    }
    line$y <- unit.c(start$y, start$y, end$y, end$y)
  } else if (type == "N") {
    dist_y <- distance(start, end, type = "v", half = TRUE)
    if (prConvertHeightToMm(start$y) < prConvertHeightToMm(end$y)) {
      line$y <- unit.c(start$top, start$top + dist_y, start$top + dist_y, end$bottom)
    } else {
      line$y <- unit.c(start$bottom, start$bottom - dist_y, start$bottom - dist_y, end$top)
    }
    line$x <- unit.c(
      getX4elmnt(start, "x"),
      getX4elmnt(start, "x"),
      getX4elmnt(end, "x"),
      getX4elmnt(end, "x")
    )
  } else if (type == "vertical") {
    line$x <- unit.c(getX4elmnt(start, "x"), getX4elmnt(end, "x"))
    if (prConvertHeightToMm(start$y) < prConvertHeightToMm(end$y)) {
      line$y <- unit.c(start$top, end$bottom)
    } else {
      line$y <- unit.c(start$bottom, end$top)
    }
  } else if (type == "side") {
    # Horizontal-first exit: leave from start side, travel horizontally,
    # then drop vertically to the end box.
    end_is_right <- prConvertWidthToMm(getX4elmnt(end, "x")) > prConvertWidthToMm(start$x)
    exit_x <- if (side == "right" || (side == "auto" && end_is_right)) {
      start$right
    } else {
      start$left
    }
    line$x <- unit.c(exit_x, getX4elmnt(end, "x"), getX4elmnt(end, "x"))
    if (prConvertHeightToMm(start$y) > prConvertHeightToMm(end$y)) {
      line$y <- unit.c(start$y, start$y, end$top)
    } else {
      line$y <- unit.c(start$y, start$y, end$bottom)
    }
  } else { # horizontal
    line$y <- unit.c(start$y, end$y)
    if (prConvertWidthToMm(getX4elmnt(start, "x")) < prConvertWidthToMm(getX4elmnt(end, "x"))) {
      line$x <- unit.c(start$right, end$left)
    } else {
      line$x <- unit.c(start$left, end$right)
    }
  }

  lg <- prRenderLine(
    x = line$x, y = line$y,
    smooth = smooth, corner_radius = corner_radius,
    gp = lty_gp, arrow = arrow_obj
  )


  if (!is.null(label) && nzchar(label)) {
    mid <- prLineLabelPos(line, prefer = "auto")
    lbl <- prLabelWithBackground(
      label = label,
      x = mid$x,
      y = mid$y,
      label_gp = label_gp,
      pad = label_pad,
      bg_gp = label_bg_gp
    )

    gt <- grobTree(lg, lbl)

    return(structure(
      gt,
      line = line,
      class = c("connect_boxes", class(gt))
    ))
  }

  structure(lg, line = line, class = c("connect_boxes", class(lg)))
}

#' Add a units column to the results
#'
#' @param results The results that we want to add the column to
#'
#' @return results with added column
#' @inheritParams getDescriptionStatsBy
prAddDescUnitColumn <- function(results, x, use_units, units_column_name) {
  org <- results
  label <- attr(results, "label")
  column_names <- attr(results, "column_names")
  if (isTRUE(use_units)) {
    if (units(x) != "") {
      unitcol <- rep(sprintf("%s", units(x)), times = NROW(results))
      unitcol[rownames(results) == "Missing"] <- ""
    } else {
      unitcol <- rep("", times = NROW(results))
    }
    if (length(unitcol) != nrow(results)) {
      stop(
        "There is an discrepancy in the number of rows in the units",
        " and the by results: ", length(unitcol), " units vs ", nrow(results), " results",
        "\n Units:", paste(unitcol, collapse = ", "),
        "\n Rows results:", paste(rownames(results), collapse = ", ")
      )
    }
    results <- cbind(results, unitcol)
    column_names <- c(column_names, units_column_name)
  } else if (use_units == "name") {
    if (units(x) != "") {
      label <- sprintf("%s (%s)", label, units(x))
    }
  }

  org %>%
    copyAllNewAttributes(results) %>%
    structure(label = label,
              column_names = column_names)
}

#' Calculate midpoint between two units
#'
#' Calculates the midpoint between two units. If the arguments are
#' numeric they are converted to npc units.
#'
#' @param u1 The first unit
#' @param u2 The second unit
#' @return A unit object
#' @export
#' @examples
#' MidDistanceX(grid::unit(.1, "npc"), grid::unit(.8, "npc"))
MidDistanceX <- function(u1, u2) {
  if (!grid::is.unit(u1)) u1 <- grid::unit(u1, "npc")
  if (!grid::is.unit(u2)) u2 <- grid::unit(u2, "npc")
  (u1 + u2) * 0.5
}

#' @rdname MidDistanceX
#' @export
MidDistanceY <- MidDistanceX

#' An R alternative to the lodash \code{get} in JavaScript
#'
#' This is a handy function for retrieving items deep in a nested structure
#' without causing error if not found
#'
#' @param sourceList The \code{list()}/\code{c()} that is to be searched for the element
#' @param path A string that can be separated by [,] or ., the string \code{"elementname1.1.elementname"}
#'  is equivalent to \code{"elementname1[[1]]]elementname"}. Note that the function doesn't check
#'  the validity of the path - it only separates and tries to address that element with `[[]]`.
#' @param default The value to return if the element isn't found
#'
#' @return Returns a sub-element from \code{sourceList} or the \code{default} value.
#' @importFrom stringr str_detect str_split str_replace_all
#' @examples
#' source <- list(a = list(b = 1, `odd.name` = 'I hate . in names', c(1,2,3)))
#' retrieve(source, "a.b")
#' retrieve(source, "a.b.1")
#' retrieve(source, "a.odd\\.name")
#' retrieve(source, "a.not_in_list")
#'
#' @family lodash similar functions
#' @export
retrieve <- function(sourceList, path, default = NA) {
  path <- str_split(path, "(?<!\\\\)[\\[\\].]+")[[1]]
  for (element in path) {
    element <- str_replace_all(element, "\\\\", "")
    if (str_detect(element, "^[0-9]+$")) {
      element <- as.numeric(element)
      if (length(sourceList) < element || element <= 0) {
        return(default)
      }
    } else if (!(element %in% names(sourceList))) {
      return(default)
    }

    sourceList <- sourceList[[element]]
  }
  return(sourceList)
}

#' An R alternative to the lodash \code{has} in JavaScript
#'
#' This is a handy function for checking if item exist in a nested structure
#'
#' @param sourceList The \code{list()}/\code{c()} that is to be searched for the element
#' @param path A string that can be separated by [,] or ., the string \code{"elementname1.1.elementname"}
#'  the validity of the path - it only separates and tries to address that element with `[[]]`.
#'
#' @return Returns a boolean.
#' @importFrom stringr str_detect str_split
#' @examples
#' has(list(a = list(b = 1)), "a.b")
#'
#' @family lodash similar functions
#' @export
has <- function(sourceList, path) {
  uniqueNotFoundId <- "__@GMISC_NOT_FOUND@__"
  value <- retrieve(sourceList, path, default = uniqueNotFoundId)
  if (length(value) > 1) {
    return(TRUE)
  }

  return(value != uniqueNotFoundId)
}

library('testthat')
context('retrieve and has')

test_that("Basic retrieve", { 
  var <- "a"
  expect_equal(retrieve(var, "1"), var)
  
  var <- list(b = 1:10)
  expect_equal(retrieve(var, "b"), 1:10)
  
  var <- list(b = list(c = 1, d = list(1,2,3,list(e = 22))))
  expect_equal(retrieve(var, "b.d.4.e"), 22)

  expect_true(is.na(retrieve(var, "b.ee")))

  var <- list(list(c = 1, d = 2))
  expect_equal(retrieve(var, "1.c"), 1)
})

test_that("Default values for retrieve", { 
  var <- "a"
  expect_equal(retrieve(var, "2"), NA)
  expect_equal(retrieve(var, "2", default = "missing"), "missing")
  
  var <- list(b = 1:10)
  expect_equal(retrieve(var, "a"), NA)
  expect_equal(retrieve(var, "a", default = "missing"), "missing")
  
  var <- list(a = NULL, b = 1:10)
  expect_equal(retrieve(var, "a"), NULL)
  expect_equal(retrieve(var, "a", default = "missing"), NULL)
})

test_that("Retrieve with special character", { 
  var <- list(b = list(c = 1, `a.d` = list(1,2,3,list(e = 22))))
  expect_equal(retrieve(var, "b.a\\.d.4.e"), 22)

  var <- list(b = list(c = 1, `a[evilname]d` = list(1,2,3,list(e = 22))))
  expect_equal(retrieve(var, "b.a\\[evilname\\]d.4.e"), 22)
})


test_that("Basic has", { 
  var <- "a"
  expect_true(has(var, "1"))
  
  var <- list(b = 1:10)
  expect_true(has(var, "b"))
  
  var <- list(b = list(c = 1, d = list(1,2,3,list(e = 22))))
  expect_true(has(var, "b.d.4.e"))
  
  expect_false(has(var, "b.ee"))
})

#' Additional box shape helpers for `boxGrob`
#'
#' These helper functions provide alternative box drawing functions that can be
#' passed to `boxGrob(..., box_fn = ...)` or used via convenience wrappers
#' (`boxDiamondGrob`, `boxEllipseGrob`, `boxRackGrob`). They are intentionally
#' lightweight and implemented using base `grid` primitives.
#'
#' @inheritParams boxGrob
#' @details Shape functions are called with `x` and `y` in [0,1] inside the box viewport and with `box_gp` for styling; convenience wrappers match the `boxGrob` signature.
#' @return A `grid` grob that draws the requested shape inside the box viewport.
#' @rdname boxShapes
#' @name boxShapes
#' @family flowchart components
#' @seealso \code{\link{boxGrob}} for usage and the `box_fn` argument
#' @examples
#' # These functions are designed to be passed to `boxGrob(..., box_fn = ...)`.
#' grid::grid.newpage()
#' d <- boxGrob("Decision", box_fn = diamond_rounded_box_fn)
#' e <- boxGrob("Start", box_fn = ellipse_box_fn)
#' r <- boxGrob("Server", box_fn = rack_box_fn)
#' spreadHorizontal(list(d, e, r))
NULL


#' Diamond (rounded or sharp)
#'
#' Two diamond variants are provided: a rounded diamond (default) implemented
#' with `xsplineGrob` for slightly softened corners, and a sharp diamond
#' implemented with `polygonGrob` for strict straight edges.
#'
#' @rdname boxShapes
#' @export
diamond_rounded_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  # Use slight inset from the viewport edge and a larger shape parameter
  # to avoid XSpline overshoot that can create small spikes when connectors
  # attach to the top/bottom of the box.
  inset <- 0.02
  grob <- xsplineGrob(
    x = c(.5, 1 - inset, .5, 0 + inset),
    y = c(1 - inset, .5, 0 + inset, .5),
    shape = rep(0.35, 4),
    open = FALSE,
    gp = gp
  )
  # Suggest additional padding for diamond shapes since text can sit close
  # to the acute corners; boxGrob will pick up this attribute and add it
  # to the text padding when sizing the box.
  attr(grob, "box_fn_padding") <- unit(3, "mm")
  grob
}

#' Sharp diamond (straight edges)
#'
#' @rdname boxShapes
#' @export
diamond_sharp_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  inset <- 0.02
  grob <- polygonGrob(x = c(.5, 1 - inset, .5, 0 + inset), y = c(1 - inset, .5, 0 + inset, .5), gp = gp)
  # Recommend a little extra padding for the sharp diamond variant as well
  attr(grob, "box_fn_padding") <- unit(3, "mm")
  grob
}

#' Convenience: create a diamond box with same signature as `boxGrob`
#'
#' @param rounded Logical; use rounded diamond (TRUE) or sharp diamond (FALSE).
#' @rdname boxShapes
#' @export
boxDiamondGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                           width, height, just = "center", bjust = "center",
                           txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                           box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                           rounded = TRUE,
                           name = NULL) {
  box_fn <- if (isTRUE(rounded)) diamond_rounded_box_fn else diamond_sharp_box_fn
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = box_fn, name = name
  )
}


#' Ellipse box
#'
#' Approximates an ellipse using a polygon with many points; it respects the
#' box viewport so it scales correctly.
#'
#' @rdname boxShapes
#' @export
ellipse_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  t <- seq(0, 2 * pi, length.out = 80)
  xs <- .5 + .48 * cos(t)
  ys <- .5 + .36 * sin(t)
  polygonGrob(x = xs, y = ys, gp = gp)
}


#' @rdname boxShapes
#' @export
boxEllipseGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                           width, height, just = "center", bjust = "center",
                           txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                           box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                           name = NULL) {
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = ellipse_box_fn, name = name
  )
}


#' Rack / server styled box
#'
#' A rectangular box with a slightly darker top band to mimic a server / disk rack
#' look. Implemented by composing a rounded rectangle and an overlaying top
#' strip. The strip is drawn without border to avoid double outlines.
#'
#' @rdname boxShapes
#' @export
rack_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  # base rounded rectangle
  base <- roundrectGrob(x = .5, y = .5, r = unit(3, "mm"), gp = gp)

  # compute a top strip fill (lightened version of box fill); set col = NA
  strip_fill <- if (is.null(gp$fill)) "grey90" else lighten_color(gp$fill, 0.9)
  top_strip <- rectGrob(x = .5, y = unit(0.78, "npc"), width = unit(0.95, "npc"), height = unit(0.22, "npc"), gp = gpar(fill = strip_fill, col = NA))

  gTree(children = gList(base, top_strip))
}

#' @rdname boxShapes
#' @export
boxRackGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                        width, height, just = "center", bjust = "center",
                        txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                        box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                        name = NULL) {
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = rack_box_fn, name = name
  )
}


#' Database / cylinder box
#'
#' Draws a cylinder-like database symbol (ellipse top, rectangular body, base ellipse).
#'
#' @rdname boxShapes
#' @export
database_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  t <- seq(0, 2 * pi, length.out = 64)
  top_h <- 0.83
  top_rvx <- 0.42
  top_rvy <- 0.09
  xs_top <- .5 + top_rvx * cos(t)
  ys_top <- top_h + top_rvy * sin(t)

  bottom_h <- 0.17
  bottom_rvy <- 0.09
  xs_bot <- .5 + top_rvx * cos(t)
  ys_bot <- bottom_h + bottom_rvy * sin(t)

  body <- rectGrob(x = .5, y = .5, width = unit(0.84, "npc"), height = unit(0.66, "npc"), gp = gp)
  top <- polygonGrob(x = xs_top, y = ys_top, gp = gp)
  bot_fill <- polygonGrob(x = xs_bot, y = ys_bot, gp = gpar(fill = gp$fill, col = NA))
  # Outline bottom to give the cylinder edge (use gp$col if available, otherwise black)
  col_edge <- if (!is.null(gp$col)) gp$col else "black"
  bot_edge <- linesGrob(x = xs_bot, y = ys_bot, gp = gpar(col = col_edge))

  gTree(children = gList(body, bot_fill, top, bot_edge))
}

#' @rdname boxShapes
#' @export
boxDatabaseGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                            width, height, just = "center", bjust = "center",
                            txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                            box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                            name = NULL) {
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = database_box_fn, name = name
  )
}


#' Document (wavy bottom) box
#'
#' A document icon with a wavy bottom edge, useful to indicate printed output.
#'
#' @rdname boxShapes
#' @export
document_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  n <- 80
  xs <- seq(0.08, 0.92, length.out = n)
  # top edge and sides
  top_y <- 0.88
  bottom_y <- 0.12
  amp <- 0.035
  wave <- bottom_y + amp * sin(seq(0, 2 * pi, length.out = n))

  xpoly <- c(xs, rev(xs))
  ypoly <- c(rep(top_y, n), rev(wave))
  polygonGrob(x = xpoly, y = ypoly, gp = gp)
}

#' @rdname boxShapes
#' @export
boxDocumentGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                            width, height, just = "center", bjust = "center",
                            txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                            box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                            name = NULL) {
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = document_box_fn, name = name
  )
}


#' Multiple documents (stack)
#'
#' Draws a stack of document shapes by repeating the same shape with offsets.
#'
#' @rdname boxShapes
#' @export
documents_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  n <- 80
  xs <- seq(0.08, 0.92, length.out = n)
  top_y_base <- 0.88
  bottom_y_base <- 0.12
  amp <- 0.035

  # Stack pages with a small diagonal offset so the stack is visible.
  # Back pages are drawn first (larger offset), front page last (no offset).
  n_pages <- 3
  shifts_x <- seq(0.03, 0, length.out = n_pages)
  shifts_y <- seq(-0.02, 0, length.out = n_pages)

  docs <- lapply(seq_len(n_pages), function(i) {
    sx <- shifts_x[i]
    sy <- shifts_y[i]
    top_y <- top_y_base + sy
    bottom_y <- bottom_y_base + sy
    wave <- bottom_y + amp * sin(seq(0, 2 * pi, length.out = n))
    xpoly <- c(xs + sx, rev(xs + sx))
    ypoly <- c(rep(top_y, n), rev(wave))
    polygonGrob(x = xpoly, y = ypoly, gp = gp)
  })

  gTree(children = do.call(gList, docs))
}

#' @rdname boxShapes
#' @export
boxDocumentsGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                             width, height, just = "center", bjust = "center",
                             txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                             box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                             name = NULL) {
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = documents_box_fn, name = name
  )
}


#' Tape / curved storage box
#'
#' Left/right curved edges to mimic tape-like icons.
#'
#' @rdname boxShapes
#' @export
tape_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  # Draw a tape-like rectangle with larger curved end caps (Option A).
  # Compose a central rectangle and two semi-elliptical caps so the ends are
  # more prominent and visually balanced with the body.
  body <- rectGrob(x = .5, y = .5, width = unit(0.74, "npc"), height = unit(0.64, "npc"), gp = gp)

  t <- seq(-pi / 2, pi / 2, length.out = 120)

  # Make caps larger horizontally and slightly taller
  cap_x_radius <- 0.12
  cap_y_radius <- 0.36

  # left cap (semi-ellipse) centered near left edge
  left_center <- 0.08
  left_x <- left_center + cap_x_radius * cos(t)
  left_y <- 0.5 + cap_y_radius * sin(t)
  left_cap <- polygonGrob(x = c(left_x, left_x[1]), y = c(left_y, left_y[1]), gp = gp)

  # right cap (mirror) centered near right edge
  right_center <- 0.92
  right_x <- right_center - cap_x_radius * cos(t)
  right_y <- 0.5 + cap_y_radius * sin(t)
  right_cap <- polygonGrob(x = c(right_x, right_x[1]), y = c(right_y, right_y[1]), gp = gp)

  gTree(children = gList(body, left_cap, right_cap))
}

#' @rdname boxShapes
#' @export
boxTapeGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                        width, height, just = "center", bjust = "center",
                        txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                        box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                        name = NULL) {
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = tape_box_fn, name = name
  )
}


#' Server (front-panel vents)
#'
#' A server-styled box with front-panel vent lines so it is visually distinct
#' from a rack (top strip) representation.
#'
#' @param x Horizontal position inside the box viewport (0-1).
#' @param y Vertical position inside the box viewport (0-1).
#' @param gp A `gpar` object with styling information (fill/col/etc).
#' @rdname boxShapes
#' @export
server_box_fn <- function(x = .5, y = .5, gp = gpar()) {
  # Base rounded rectangle
  base <- roundrectGrob(x = .5, y = .5, r = unit(3, "mm"), gp = gp)

  # vents: three small horizontal strips centered vertically on the front
  vent_fill <- (if (!is.null(gp$col)) gp$col else "#333333")
  vent_gp <- gpar(fill = "#FFFFFF", col = NA)

  vent_ys <- c(0.58, 0.5, 0.42)
  vents <- lapply(vent_ys, function(vy) {
    rectGrob(x = .5, y = vy, width = unit(0.6, "npc"), height = unit(0.05, "npc"), gp = gpar(fill = lighten_color(gp$fill, 0.92), col = NA))
  })

  gTree(children = do.call(gList, c(list(base), vents)))
}

#' Convenience: create a server-styled box
#'
#' @rdname boxShapes
#' @importFrom grDevices rgb
#' @export
boxServerGrob <- function(label, y = unit(.5, "npc"), x = unit(.5, "npc"),
                          width, height, just = "center", bjust = "center",
                          txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
                          box_gp = getOption("boxGrob", default = gpar(fill = "white")),
                          name = NULL) {
  boxGrob(
    label = label, y = y, x = x, width = width, height = height,
    just = just, bjust = bjust, txt_gp = txt_gp, box_gp = box_gp,
    box_fn = server_box_fn, name = name
  )
}


# Small helper to lighten a color (returns the hex for grid::gpar)
lighten_color <- function(col, factor = 0.9) {
  # factor <1 -> darker, >1 -> lighter (we use >1 to lighten)
  if (is.null(col)) {
    return(col)
  }
  if (is.na(col)) {
    return(col)
  }
  tryCatch(
    {
      rgb_vals <- grDevices::col2rgb(col) / 255
      rgb((1 - factor) * rgb_vals[1, ] + factor, (1 - factor) * rgb_vals[2, ] + factor, (1 - factor) * rgb_vals[3, ] + factor)
    },
    error = function(e) col
  )
}

library(testthat)

test_that("setConnectorLabels attaches attributes and printing returns object", {
    a <- boxDiamondGrob("D")
    b <- boxEllipseGrob("L")
    c <- boxServerGrob("S")

    boxes <- list(decision = a, outcomes = list(b, c)) |>
        spreadHorizontal(from = unit(.1, "npc"), to = unit(.9, "npc"), subelement = "outcomes") |>
        spreadVertical()

    con_list <- connectGrob(boxes$decision, boxes$outcomes, type = "N")
    expect_null(attr(con_list, "connector_labels"))

    con2 <- setConnectorLabels(con_list, c("Local", "Server"))
    expect_equal(attr(con2, "connector_labels"), c("Local", "Server"))
    expect_true(!is.null(attr(con2, "connector_label_gp")))

    # printing returns the object invisibly
    res <- print(con2)
    expect_identical(res, con2)
})

library(testthat)

test_that("labelConnector returns S3 object and print draws", {
    a <- boxDiamondGrob("D")
    b <- boxEllipseGrob("L")
    c <- boxServerGrob("S")

    boxes <- list(decision = a, outcomes = list(b, c)) |>
        spreadHorizontal(from = unit(.1, "npc"), to = unit(.9, "npc"), subelement = "outcomes") |>
        spreadVertical()

    con_list <- connectGrob(boxes$decision, boxes$outcomes, type = "N")

    lbl_obj <- labelConnector(con_list, c("Local", "Server"))
    expect_s3_class(lbl_obj, "Gmisc_connector_label")
    expect_equal(lbl_obj$labels, c("Local", "Server"))
    expect_length(lbl_obj$texts, 2)

    # print method should return invisibly and be the same object
    res <- print(lbl_obj)
    expect_identical(res, lbl_obj)
})

library(testthat)

test_that("box shapes construct without error", {
    expect_silent(db <- boxDatabaseGrob("DB"))
    expect_silent(doc <- boxDocumentGrob("Doc"))
    expect_silent(docs <- boxDocumentsGrob("Docs"))
    expect_silent(tp <- boxTapeGrob("Tape"))
    expect_silent(srv <- boxServerGrob("Server"))

    # server and rack should be visually distinct (not identical grobs)
    expect_silent(rack <- boxRackGrob("Rack"))
    expect_false(identical(rack, srv))

    # connecting them shouldn't error
    expect_silent(con <- connectGrob(db, list(doc, srv), type = "N"))
    expect_true(inherits(con, "connect_boxes_list"))
})

#' A simple bezier arrow
#'
#' This is an alternative to the grid packages \code{\link[grid:grid.bezier]{bezierGrob}}
#' with the advantage that it allows you to draw an arrow with a specific
#' unit width. Note, it has only a end-arrow at this point.
#'
#' @inheritParams grid::bezierGrob
#' @param width The width of the arrow, either a numeric single number or a unit. \strong{Note:}
#'  The arrow does not rely on \code{lwd} but on actual width.
#' @param clr The color of the arrow.
#' @param arrow This is a list with all the \strong{base} (width) and the desired
#' \strong{length} for the arrow. \strong{Note:} This differs from the original
#' \code{\link{bezierGrob}} function.
#' @param align_2_axis Indicates if the arrow should be vertically/horizontally
#'  aligned. This is useful for instance if the arrow attaches to a box.
#' @param rez The resolution of the arrow. This specifies how many points to retrieve from the
#'  \code{\link{gnrlBezierPoints}} function. Defaults to 200.
#' @param name A character identifier.
#' @return \code{grid::grob} A grob of the class polygonGrob with attributes that
#'  correspond to the bezier points.
#'
#' @inheritParams calculateLinesAndArrow
#' @examples
#' library(grid)
#' grid::grid.newpage()
#' arrowGrob <- bezierArrowSmpl(
#'   x = c(.1, .3, .6, .9),
#'   y = c(0.2, 0.2, 0.9, 0.9)
#' )
#' grid.draw(arrowGrob)
#' @import grid
#' @importFrom utils tail
#' @export
bezierArrowSmpl <- function(x = c(0.2, .7, .3, .9),
                            y = c(0.2, .2, .9, .9),
                            width = .05,
                            clr = "#000000",
                            default.units = "npc",
                            arrow = list(),
                            rez = 200,
                            align_2_axis = TRUE,
                            name = NULL,
                            rm_intersect = 3L,
                            gp = gpar(), vp = NULL) {
  if (!("unit" %in% class(x))) {
    x <- unit(x, default.units)
  }
  if (!("unit" %in% class(y))) {
    y <- unit(y, default.units)
  }

  width <- getAbsoluteWidth(
    w = width,
    default.units = default.units,
    x = x,
    y = y
  )
  if (!"base" %in% names(arrow)) {
    arrow$base <- unit(getGridVal(width, "mm") * 2, "mm")
  }
  if (!"length" %in% names(arrow)) {
    sqFn <- function(vals, axis) {
      (getGridVal(vals[1], default.units = "mm", axisTo = axis) -
        getGridVal(tail(vals, 1), default.units = "mm", axisTo = axis))^2
    }
    arrow$length <- unit(sqrt(sqFn(x, "x") + sqFn(y, "y")) * .1, "mm")
  }

  arrow$base <- getAbsoluteWidth(
    w = arrow$base,
    default.units = default.units,
    x = x,
    y = y
  )
  arrow$length <- getAbsoluteWidth(
    w = arrow$length,
    default.units = default.units,
    x = x,
    y = y
  )

  if (length(y) != length(x)) {
    stop(
      "You have provided unequal lengths to y and x - thus uninterpretable:",
      " y = ", length(y), " elements",
      " while x = ", length(x), " elements"
    )
  }

  ###############################################################################
  # Internally we want to avoid using the "npc" as this is not an absolute      #
  # measure and we therefore switch to "mm" that is consistent among the axes.  #
  # This compromises the portability of the grob but it is a price worth paying #
  # Note: All values are numeric beyone this point!                             #
  ###############################################################################
  internal.units <- "mm"
  x <- convertX(x, unitTo = internal.units, valueOnly = TRUE)
  y <- convertY(y, unitTo = internal.units, valueOnly = TRUE)
  width <- convertY(width, unitTo = internal.units, valueOnly = TRUE)
  arrow$length <- convertX(arrow$length, unitTo = internal.units, valueOnly = TRUE)
  arrow$base <- convertX(arrow$base, unitTo = internal.units, valueOnly = TRUE)

  # According to the original description they're all spline
  # control points but as I want the line to start and end
  # at specific points then this makes sense to me
  end_points <- list(
    start = list(
      x = x[1],
      y = y[1]
    ),
    end = list(
      x = tail(x, 1),
      y = tail(y, 1)
    )
  )

  new_bp <-
    getBezierAdj4Arrw(
      x = x, y = y,
      arrow_length = arrow$length,
      length_out = rez
    )

  # Get lengths
  new_bp$lengths <-
    with(
      new_bp,
      mapply(
        x1 = x[-length(x)],
        y1 = y[-length(y)],
        x2 = x[-1],
        y2 = y[-1],
        function(x1, y1, x2, y2) sqrt((x2 - x1)^2 + (y2 - y1)^2)
      )
    )

  # Add the arrow length to the last element
  new_bp$lengths[length(new_bp$lengths)] <-
    tail(new_bp$lengths, 1) +
    arrow$length

  lines <- calculateLinesAndArrow(
    x = new_bp$x, y = new_bp$y,
    offset = width / 2,
    end_x = end_points$end$x,
    end_y = end_points$end$y,
    arrow_offset = arrow$base / 2,
    rm_intersect = rm_intersect
  )

  if (align_2_axis != FALSE) {
    lines <- align2Axis(
      bp = new_bp,
      lines = lines,
      width = width,
      internal.units = internal.units,
      axis = align_2_axis
    )
    align_2_axis <- attr(lines, "axis")
  }

  # Change everything to default.units from internal
  lines <- lapply(
    lines,
    function(x) lapply(x, function(xx) unit(xx, internal.units))
  )
  lines$left$x <- convertX(lines$left$x, unitTo = default.units)
  lines$right$x <- convertX(lines$right$x, unitTo = default.units)

  lines$left$y <- convertY(lines$left$y, unitTo = default.units)
  lines$right$y <- convertY(lines$right$y, unitTo = default.units)

  new_bp <- lapply(new_bp, function(x) unit(x, internal.units))
  new_bp$x <- convertX(new_bp$x, unitTo = default.units)
  new_bp$y <- convertY(new_bp$y, unitTo = default.units)

  end_points$start$x <- convertX(unit(end_points$start$x, internal.units),
    unitTo = default.units
  )
  end_points$start$y <- convertY(unit(end_points$start$y, internal.units),
    unitTo = default.units
  )
  end_points$end$x <- convertX(unit(end_points$end$x, internal.units),
    unitTo = default.units
  )
  end_points$end$y <- convertY(unit(end_points$end$y, internal.units),
    unitTo = default.units
  )

  poly_x <- unit.c(
    lines$left$x,
    rev(lines$right$x)
  )
  poly_y <- unit.c(
    lines$left$y,
    rev(lines$right$y)
  )
  pg <- polygonGrob(
    x = poly_x,
    y = poly_y,
    gp = gpar(fill = clr, col = clr),
    name = name,
    vp = vp
  )

  # Add details that are used by the gradient version
  attr(pg, "center_points") <- new_bp
  attr(pg, "left_points") <- lines$left
  attr(pg, "right_points") <- lines$right
  attr(pg, "end_points") <- end_points
  attr(pg, "axis") <- align_2_axis

  return(pg)
}

#' Align boxes
#'
#' Aligns a set of [`boxGrob`]/[`boxPropGrob`] according to the first positional argument.
#'
#' @param reference A [`boxGrob`]/[`boxPropGrob`]/[`coords`] object or a [`unit`][grid::unit] or a
#'  numerical value that can be converted into a [`unit`][grid::unit] of `npc` type. If a numeric scalar is
#'  provided and is `0` or greater than the number of boxes, it is treated as a coordinate (no error is raised).
#' @param ... A set of boxes.
#' @param position How to align the boxes, differs slightly for vertical and horizontal alignment
#'  see the accepted arguments
#' @param subelement If a `list` of boxes is provided, this parameter can be used
#'   to target a specific element (by name or index), or a deep path into nested
#'   lists (e.g., `c("detail", 1)`), for the alignment operation. You can also
#'   provide multiple targets by giving a list of paths (e.g., `list(c("detail", 1), c("followup", 1))`).
#'   When a list of boxes is *piped* into `alignVertical()`/`alignHorizontal()` and a named `reference` is
#'   provided (e.g. `my_boxes |> alignHorizontal(reference = c("grp","sub"), .subelement = ...)`), the
#'   function will unwrap a single-element wrapper so nested `.subelement` targets are found as expected.
#'   The function will return the original list with the targeted element(s)
#'   replaced by their aligned version(s).
#' @return `list` with the boxes that are to be aligned
#'
#' @md
#' @export
#' @name align
#' @family flowchart components
#' @example inst/examples/alignBox_ex.R
#' @rdname align


alignVertical <- function(reference, ..., position = c("center", "top", "bottom"), subelement = NULL) {
  position <- match.arg(position)

  boxes2align <- list(...)


  # Handle pipeline where the piped object becomes the positional `reference`
  # but the user also provided a named `reference` (it ends up in `...`).
  # In that case, move the positional `reference` into `boxes2align` and use
  # the named `reference` value as the actual reference.
  if (!is.null(names(boxes2align)) &&
    "reference" %in% names(boxes2align) &&
    is.list(reference) &&
    !inherits(reference, "box")) {
    named_ref <- boxes2align$reference
    boxes2align$reference <- NULL
    boxes2align <- reference
    reference <- named_ref
  }

  # Support piping a list into alignVertical() while passing a positional
  # reference path, e.g. `boxes |> alignVertical("groups", subelement = "groups2")`.
  # In this shape, the piped object is captured as `reference` and the first
  # positional `...` argument is the intended reference.
  if (is.list(reference) &&
    !inherits(reference, "box") &&
    length(boxes2align) >= 1 &&
    is.null(names(boxes2align)) &&
    !inherits(boxes2align[[1]], "box") &&
    (is.atomic(boxes2align[[1]]) || is.list(boxes2align[[1]]))) {
    ref_arg <- boxes2align[[1]]
    boxes2align <- reference
    reference <- ref_arg
  }

  # Support piping a list into alignVertical(): if the user pipes a list as the
  # first argument and provides no '...' boxes, treat that list as the set of
  # boxes to align and use its first element as the reference.
  if (length(boxes2align) == 0 && is.list(reference) && !inherits(reference, "box")) {
    boxes2align <- reference
    reference <- boxes2align[[1]]
  }

  # If a list of boxes was piped into '...' as a single element (e.g., using
  # the base pipe with a named 'reference' argument), unwrap it so callers
  # can pass either `list_of_boxes |> alignVertical(...)` or `alignVertical(list_of_boxes)`.
  if (prIsSingleElementWrappedList(boxes2align)) {
    boxes2align <- boxes2align[[1]]
  }

  if (!is.null(subelement)) {
    # Normalize into a list of paths (each path is an atomic vector)
    paths <- if (is.list(subelement) && all(sapply(subelement, is.atomic))) subelement else list(subelement)

    for (path in paths) {
      # Find target using helper (search top-level and first nested container)
      # Try finding the target directly (top-level)
      target <- get_list_element_by_path(boxes2align, path)
      container_is_first <- FALSE

      # If not found, try inside the common nested first element
      if (is.null(target) && prHasNestedFirstContainer(boxes2align)) {
        target <- get_list_element_by_path(boxes2align[[1]], path)
        container_is_first <- TRUE
      }

      if (is.null(target)) {
        stop("The subelement '", paste(path, collapse = " -> "), "' was not found in the provided boxes.",
          call. = FALSE
        )
      }

      # Resolve the reference path against the original list so recursive calls get an object
      ref_for_call <- prResolveReference(reference, boxes2align)

      aligned <- alignVertical(
        reference = ref_for_call,
        target,
        position = position
      )

      # If the target was a single box, the recursive call returns a
      # single-element list-of-boxes; unwrap it so we don't insert a
      # list wrapper into the parent structure (which can create
      # self-referential/cyclic lists in subsequent operations).
      if (is.list(aligned) && length(aligned) == 1 && inherits(target, "box") && inherits(aligned[[1]], "box")) {
        aligned <- aligned[[1]]
      }

      if (container_is_first) {
        boxes2align[[1]] <- set_list_element_by_path(boxes2align[[1]], path, aligned)
      } else {
        boxes2align <- set_list_element_by_path(boxes2align, path, aligned)
      }
    }

    return(prExtendClass(boxes2align, "Gmisc_list_of_boxes"))
  }

  # Normalize and validate boxes now that subelement processing is done
  boxes2align <- prNormalizeAndValidateBoxes(boxes2align)

  # Resolve reference if it's a path into the boxes
  reference <- prResolveReference(reference, boxes2align)
  ref_positions <- prConvert2Coords(reference)

  # Apply vertical alignment
  return(prApplyAlign(boxes2align, ref_positions, position, axis = "vertical"))
}


#' @rdname align
#' @param sub_position When the box is a [`boxPropGrob`] it not only has the general `.positions` but
#'  also `left` and `right` which can be viewed as separate boxes that have simply been merged.
#' @md
#' @export
alignHorizontal <- function(
  reference,
  ...,
  position = c("center", "left", "right"),
  sub_position = c("none", "left", "right"),
  subelement = NULL
) {
  position <- match.arg(position)
  sub_position <- match.arg(sub_position)

  boxes2align <- list(...)


  # Handle pipeline where the piped object becomes the positional `reference`
  # but the user also provided a named `reference` (it ends up in `...`).
  # In that case, move the positional `reference` into `boxes2align` and use
  # the named `reference` value as the actual reference.
  if (!is.null(names(boxes2align)) && "reference" %in% names(boxes2align) && is.list(reference) && !inherits(reference, "box")) {
    named_ref <- boxes2align$reference
    boxes2align$reference <- NULL
    boxes2align <- reference
    reference <- named_ref
  }

  # Support piping a list into alignHorizontal() while passing a positional
  # reference path, e.g. `boxes |> alignHorizontal("groups", subelement = "groups2")`.
  if (is.list(reference) &&
    !inherits(reference, "box") &&
    length(boxes2align) >= 1 &&
    is.null(names(boxes2align)) &&
    !inherits(boxes2align[[1]], "box") &&
    (is.atomic(boxes2align[[1]]) || is.list(boxes2align[[1]]))) {
    ref_arg <- boxes2align[[1]]
    boxes2align <- reference
    reference <- ref_arg
  }

  # Support piping a list into alignHorizontal(): if the user pipes a list as the
  # first argument and provides no '...' boxes, treat that list as the set of
  # boxes to align and use its first element as the reference.
  if (length(boxes2align) == 0 && is.list(reference) && !inherits(reference, "box")) {
    boxes2align <- reference
    reference <- boxes2align[[1]]
  }

  # If a list of boxes was piped into '...' as a single element (e.g., using
  # the base pipe with a named 'reference' argument), unwrap it so callers
  # can pass either `list_of_boxes |> alignHorizontal(...)` or `alignHorizontal(list_of_boxes)`.
  if (prIsSingleElementWrappedList(boxes2align)) {
    boxes2align <- boxes2align[[1]]
  }

  if (!is.null(subelement)) {
    # Normalize into a list of paths (each path is an atomic vector)
    paths <- if (is.list(subelement) && all(sapply(subelement, is.atomic))) subelement else list(subelement)

    for (path in paths) {
      # Find target using helper (search top-level and first nested container)
      res <- prFindSubelementTarget(boxes2align, path)
      target <- res$target
      container_is_first <- res$container_is_first

      if (is.null(target)) {
        stop("The subelement '", paste(path, collapse = " -> "), "' was not found in the provided boxes.",
          call. = FALSE
        )
      }

      # Resolve the reference path against the original list so recursive calls get an object
      ref_for_call <- prResolveReference(reference, boxes2align)

      aligned <- alignHorizontal(
        reference = ref_for_call,
        target,
        position = position,
        sub_position = sub_position
      )

      # Unwrap single-element aligned box when target is a single box to
      # avoid inserting an unnecessary list wrapper into the parent.
      if (is.list(aligned) && length(aligned) == 1 && inherits(target, "box") && inherits(aligned[[1]], "box")) {
        aligned <- aligned[[1]]
      }

      if (container_is_first) {
        boxes2align[[1]] <- set_list_element_by_path(boxes2align[[1]], path, aligned)
      } else {
        boxes2align <- set_list_element_by_path(boxes2align, path, aligned)
      }
    }

    return(prExtendClass(boxes2align, "Gmisc_list_of_boxes"))
  }

  # Normalize and validate boxes now that subelement processing is done
  boxes2align <- prNormalizeAndValidateBoxes(boxes2align)

  # Resolve reference if it's a path into the boxes
  reference <- prResolveReference(reference, boxes2align)
  ref_positions <- prConvert2Coords(reference)
  if (sub_position != "none") {
    if (sub_position == "left") {
      assert_class(ref_positions$left_x, "unit")
      ref_positions$x <- ref_positions$left_x
      ref_positions$right <- ref_positions$prop_x
    } else {
      assert_class(ref_positions$right_x, "unit")
      ref_positions$x <- ref_positions$right_x
      ref_positions$left <- ref_positions$prop_x
    }
  }

  # Apply horizontal alignment
  return(prApplyAlign(boxes2align, ref_positions, position, axis = "horizontal"))
}

testthat::test_that("align accepts deep subelement paths and reports missing ones", {
    # No package load needed; tests run in package context during R CMD check

    nested <- list(a = list(b = list("x")), c = 1)

    expect_error(alignHorizontal(nested, subelement = c("no", "pe")), "The subelement 'no -> pe' was not found", fixed = TRUE)
    expect_error(alignVertical(nested, subelement = list(c("no", "pe"), c("a", "b"))), "The subelement 'no -> pe' was not found", fixed = TRUE)
})

testthat::test_that("align functions accept a single box as target (non-list) and return list", {
    library(grid)
    ref <- boxGrob("Ref", x = .5, y = .5)
    target <- boxGrob("T", x = .6, y = .2)

    res_h <- alignHorizontal(reference = ref, target, position = "left")
    testthat::expect_true(inherits(res_h, "Gmisc_list_of_boxes"))

    res_v <- alignVertical(reference = ref, target, position = "top")
    testthat::expect_true(inherits(res_v, "Gmisc_list_of_boxes"))
})

library(testthat)

context("prAddEmptyVals order and NA handling")

# exercise the internal helper directly

# create a fake list with row names out of order and an NA
t1 <- list(a = c(b = 1, a = 2), c = c(c = 3))
# call using exported function indirectly through getDescriptionStatsBy? we can
# call the helper by using :: but it's not exported; use getFromNamespace
prAddEmptyVals_fun <- getFromNamespace("prAddEmptyVals", "Gmisc")

res <- prAddEmptyVals_fun(t1, missing_value = 0)

# the names should preserve the order of appearance: b,a,c
expect_equal(names(res$a), c("b", "a", "c"))
expect_equal(names(res$c), c("b", "a", "c"))

# when NA present, ensure it is last
inch <- list(
    x = setNames(c(1, 2), c("head", NA)),
    y = setNames(3, "foot")
)
res2 <- prAddEmptyVals_fun(inch, missing_value = -1)
expect_equal(tail(names(res2$x), 1), NA_character_)
# when one element has NA name the helper adds the NA row for all entries
expect_equal(tail(names(res2$y), 1), NA_character_)

---
title: Document
output:
  Gmisc::docx_document:
    fig_caption: true
    force_captions: true
---

# Header 1

Test with header 1

Second paragraph

## Header 2

Test with header 2

Second paragraph

### Header 3

Test with header 3

Second paragraph

#### Header 4

Test with header 4

Second paragraph

##### Header 5

Test with header 5

Second paragraph

###### Header 6

Test with header 6

Second paragraph

# Tables

```{r htmlTable, results='asis', echo=FALSE, message=FALSE}
set.seed(1)
library(Gmisc)
library(magrittr)

f <- function(...) sample(..., 100, replace = TRUE)
data <- data.frame(age = rpois(100, 50),
                   cat_var = f(LETTERS[1:5]),
                   sex = f(c('Male','Female')),
                   race = f(c('Black','White','Asian')),
                   treat = factor(f(1:3),
                                  # The factor helps arranging the order
                                  labels = c('Treatment A', 'Treatment B', 'Placebo')))


getT1stat <- function(varname, digits = 0) {
  getDescriptionStatsBy(data[ , varname],
                        data$treat,
                        add_total_col = TRUE, 
                        show_all_values = TRUE,
                        hrzl_prop = FALSE,
                        statistics = FALSE,
                        html = TRUE,
                        digits = digits,
                        continuous_fn = describeMedian)
}


## table 1 stats
table_data <- list()
table_data[['Age']] <- getT1stat('age')
table_data[['Some categorical<br />&nbsp;&nbsp;variable']] <- getT1stat('cat_var')
table_data[['Sex']] <- getT1stat('sex')
table_data[['Race']] <- getT1stat('race')

## combine into matrix
output_data <- do.call(rbind, table_data)
rgroup <- names(table_data)
n.rgroup <- unname(sapply(rgroup, function(x) nrow(table_data[[x]])))

# add a column spanner for the status columns
cgroup <- c("", "Type of treatment<sup>&dagger;</sup>")
n.cgroup <- c(1, 3) 
colnames(output_data) <-
  c(paste0('Total<br />\n',
           '<span style="weight = normal; font-size: .6em;">',
            'n = ', nrow(data),
           '</span>'),
    paste0('Treated A<br />\n',
           '<span style="weight = normal; font-size: .6em;">',
            'n = ', sum(data$treat == 'Treatment A'),
           '</span>'),
    paste0('Treatment B&Dagger;<br />\n',
           '<span style="weight = normal; font-size: .6em;">',
            'n = ', sum(data$treat == 'Treatment B'),
           '</span>'),
    paste0('Placebo<br />',
           '<span style="weight = normal; font-size: .6em;">',
            'n = ', sum(data$treat == 'Placebo'),
           '</span>'))


output_data %>% 
  addHtmlTableStyle(align = 'rccc',
                    css.rgroup = '',
                    css.rgroup.sep = '',
                    col.rgroup = c('white','lightblue1')) %>% 
  htmlTable(rgroup = rgroup,
            n.rgroup = n.rgroup,
            cgroup = cgroup,
            n.cgroup = n.cgroup,
            tspanner = c("Base", "Other"),
            n.tspanner = c(sum(sapply(table_data, nrow)[1:2]),
                           sum(sapply(table_data, nrow)[3:4])),
            rowlabel = '',
            ctable = TRUE, # latex-style table lines
            caption = "Table 1: Patient demographics",
            tfoot = paste0(
              '<span style="font-size: .6em;">',
              'Abbreviations:',
              ' ECOG, Eastern Cooperative Oncology Group;',
              ' PS, performance score',
              '</span><br />\n',
              '<span style="font-size: .6em;">',
              '<sup>&dagger;</sup>',
              ' Note 1. Trial groups for a new wonder drug</span><br />\n',
              '<span style="font-size: .6em;">',
              '<sup>&Dagger;</sup>',
              ' Note 2. Twice the dosage of treatment A',
              '</span>'))
```

# Graph tests

Basic plot:

```{r Basic_graph}
plot(cars)
```

Height, width, and caption:

```{r Height_width_caption, fig.width=3, fig.height=3, fig.cap="Test: Height, width, and caption"}
plot(cars)
```

Height, width, caption, and dpi:

```{r Height_width_caption_dpi, fig.width=3, fig.height=3, dpi=300, fig.cap="Test: Height, width, caption, and dpi"}
plot(cars)
```


Height, width, caption, dpi, out.height, out.width:

```{r Height_width_caption_dpi_out.height_out.width, out.height=288, out.width=288, fig.width=3, fig.height=3, dpi=300, fig.cap="Test: Height, width, caption, dpi, out.height, out.width"}
plot(cars)
```

# Formula tests

$$y=X\beta$$

next:

$$\sum_{n}^{i=1}{x_i+1}$$


$$\frac{a}{(\sum_{n}^{i=1}{x_i+1})+k}$$

# Highlighting of code

```{r Highlighting, echo=TRUE}
a <- "Some text"
b <- 2
c <- b + 1234
d <- function(x){
  print(x)
}
```

---
title: "Introduction to the transitionPlot (deprecated)"
author: "Max Gordon"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Introduction to the transitionPlot (deprecated)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\usepackage[utf8]{inputenc}
---

Introduction
============

The transitionPlot function aims at illustrating the transition between classes. The original intent was to show how [self-administered the Charnley classification](https://www.tandfonline.com/doi/full/10.3109/17453674.2014.931199) behaves before and after surgery. The plot is a fancier version than what can be achieved using packages such as [*diagram*](https://CRAN.R-project.org/package=diagram) but at the cost of flexibility. The current function only allows to show a transition from one state to the next. 

**Note**: The new Transition class will superseed the transitionPlot. See `vignette("Introduction to the Transition-class")` for more info.

Generate some data
==================


We will start by simulating some data similar to my article. Each observation has a sex and a Charnley class (A, B, or C). The transition is then dependent on both the sex and the Charnley class.

```{r}
set.seed(1)
library(magrittr)
n <- 100
data <- 
  data.frame(
    Sex = sample(c("Male", "Female"),
                 size = n,
                 replace = TRUE),
    Charnley_class = sample(c("A", "B", "C"), 
                            size = n, 
                            replace = TRUE))
prob <- data.frame(
  A = 1/3 +
    (data$Sex == "Male") * .25 +
    (data$Sex != "Male") * -.25 +
    (data$Charnley_class %in% "B") * -.5 +
    (data$Charnley_class %in% "C") * -2 ,
  B = 1/3 +
    (data$Sex == "Male") * .1 + 
    (data$Sex != "Male") * -.05 +
    (data$Charnley_class == "C") * -.5,
  C = 1/3 +
    (data$Sex == "Male") * -.25 +
    (data$Sex != "Male") * .25)

# Remove negative probabilities
prob <- t(apply(prob, 1, function(x) {
  if (any(x < 0)){
    x <- x - min(x) + .05
    }
  x
  }))

data$Charnley_class_after <- 
  apply(prob, 1, function(p)
    sample(c("A", "B", "C"), 
           size = 1, 
           prob = p)) %>%
  factor(levels = c("A", "B", "C"))

# Create the transition matrix that 
# is the basis for the transition plot
trn_mtrx <-
  with(data,
       table(Charnley_class, 
             Charnley_class_after))
  
rm(prob)
```

Basic use
=========
```{r, echo=FALSE, }
knitr::opts_chunk$set(message=FALSE, 
                      warnings=FALSE)
```

The most simple use is to just supply the output from the `table()` call:

```{r, echo=FALSE}
knitr::opts_chunk$set(fig.height = 5, fig.width=5)
```

```{r}
library(Gmisc)
transitionPlot(trn_mtrx)
```

The out of the box arrows are rather in my opinion not well suited and I have therefore developed a few alternatives:

```{r}
transitionPlot(trn_mtrx, 
               type_of_arrow = "simple")
```

And a fancy variant that I call *gradient* arrows as they have a colored gradient inside that fades into the arrow color:

```{r}
transitionPlot(trn_mtrx, 
               type_of_arrow = "gradient")
```

Labels and stuff
================

```{r, echo=FALSE}
knitr::opts_chunk$set(fig.height = 6)
```

Adding title, box labels, and customizing box text can easily be done using the `main`, `box_label`, and `box_txt`. If you provide the `box_txt` a vector it assumes the same text for both left and right boxes but you can also have separate texts as in the example below:

```{r}
output_perc <- 
  function(txt, n) sprintf("%s\n[%.0f%%]", txt, n)
box_txt <- 
  cbind(mapply(output_perc, 
               txt = c("A", "B", "C"), 
               n = prop.table(rowSums(trn_mtrx))*100),
        mapply(output_perc, 
               txt = c("A", "B", "C"), 
               n = prop.table(colSums(trn_mtrx))*100))
transitionPlot(trn_mtrx, 
               main = "Charnley class changes",
               box_label = c("Before", "After"),
               box_txt = box_txt,
               cex = 1.2,
               type_of_arrow = "simple")
```

Using 3D matrices
=================

```{r, echo=FALSE}
knitr::opts_chunk$set(fig.height = 5.5)
```

We can enhance the [data-ink ratio](https://infovis-wiki.net/wiki/Data-Ink_Ratio) by adding a color split for the boxes. This works only for factors with 2 levels such as sex in our example.

```{r}
trn_mtrx_3D <-
  with(data,
       table(Charnley_class, 
             Charnley_class_after,
             Sex))

transitionPlot(trn_mtrx_3D, 
               fill_start_box = c("#5C246E", "#00688B"),
               type_of_arrow = "simple")
```

The gradient arrow is most useful in this setting. It mixes the two colors according to the particular transition composition and adds an explaining gradient bar:

```{r}
transitionPlot(trn_mtrx_3D, 
               fill_start_box = c("#5C246E", "#00688B"),
               type_of_arrow = "gradient")
```

Mixed text colors can be useful when one background color is brighter and white text becomes unreadable:

```{r}
transitionPlot(trn_mtrx_3D, 
               txt_start_clr = c("white", "black"),
               fill_start_box = c("#5C246E", "#BFEFFF"),
               type_of_arrow = "gradient")
```

---
title: "Building a flowchart"
author: "Max Gordon"
date: "28 May 2026"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Building a flowchart}
  %\VignetteEncoding{UTF-8}
  %\usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE)

library(Gmisc, quietly = TRUE)
library(glue)
library(htmlTable)
library(grid)
```

# Flowchart

A flowchart is a type of diagram that represents a workflow or process. In research we often want to explain how we recruited our patients, how many that were available from the start, how many that were excluded and how many were left at the final analysis. The **Gmisc** package provides a convenient set of functions for doing this using the R's built-in `grid` package with some bells and whistles. Below is a simple example that illustrates what we're aiming for.

```{r flowchart-example, fig.height = 7, fig.width = 8}
# Shared styling
main_box_gp <- gpar(fill = "#ddeeff", col = "#336699", lwd = 1.5)
group_box_gp <- gpar(fill = "#e8f4e8", col = "#2e7d32", lwd = 1.5)
excl_box_gp  <- gpar(fill = "#fff8e1", col = "#cc8800", lwd = 1.2)
main_con_gp  <- gpar(col = "#336699", lwd = 1.5, fill = "#336699")
excl_con_gp  <- gpar(col = "#cc8800", lwd = 1.2, fill = "#cc8800")

grid.newpage()
flowchart(
  source = boxGrob(glue("Stockholm population\nn = {pop}", pop = txtInt(1632798)),
                   box_gp = main_box_gp),
  eligible = boxGrob(glue("Eligible\nn = {pop}", pop = txtInt(10032)),
                     box_gp = main_box_gp),
  included = boxGrob(glue("Randomized\nn = {incl}", incl = txtInt(122)),
                     box_gp = main_box_gp),
  groups = list(
    boxGrob(glue("Treatment A\nn = {n}", n = txtInt(43)),        box_gp = group_box_gp),
    boxGrob(glue("Treatment B\nn = {n}", n = txtInt(122-43-30)), box_gp = group_box_gp)
  )) |>
  spread(axis = "y") |>
  spread(subelement = "groups", axis = "x") |>
  equalizeWidths(subelement = list("source", "eligible", "included")) |>
  equalizeWidths(subelement = "groups") |>
  insert(list(excluded = boxHeaderGrob(
                header = glue("Excluded (n = {tot}):", tot = 30),
                body   = glue(" - not interested: {n1}\n - contra-indicated: {n2}",
                              n1 = 12, n2 = 18),
                bjust      = "left",
                box_gp     = excl_box_gp,
                header_gp  = gpar(col = "#cc8800", cex = 1))),
         after = "eligible",
         name  = "excluded") |>
  move(subelement = "excluded", x = .8) |>
  connect("eligible", "excluded", type = "L",    lty_gp = excl_con_gp, arrow_size = 3,
          label = "Excluded") |>
  connect("source",   "eligible", type = "vert", lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE) |>
  connect("eligible", "included", type = "vert", lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE) |>
  connect("included", "groups",   type = "N",    lty_gp = main_con_gp, arrow_size = 3,
          smooth = TRUE)
```

# CONSORT-style flowchart

A CONSORT diagram represents patient flow through a clinical trial. This example
demonstrates the `flowchart()` pipe API at full scale, using named lists for parallel
boxes, `spread()` to position each column, and pairwise `connect()` for the arms.

Key points:

- `arms`, `lost`, and `analysis` are all **named lists** — `connect("arms", "lost", ...)`
  automatically creates one connector per matching pair (same list index).
- `type = "L"` exits the **bottom** of the assessed box and turns right, giving the
  classic "down then right" exclusion branch instead of a direct side-exit.
- `insert()` places the exclusion box into the list at the right vertical position;
  `move()` shifts it into the right-hand column.
- Badge labels extend above the box top and are not counted in the height used by
  `spread()`, so a `margin` of 10 mm is used to leave room at the top.

```{r consort_example, fig.width = 9, fig.height = 6}
old_opts <- options(boxGrobTxtPadding = unit(3, "mm"))

box_fill     <- gpar(fill = "#ddeeff", col = "#336699", lwd = 1.5)
con_gp       <- gpar(col = "#336699", lwd = 1.5, fill = "#336699")
side_gp      <- gpar(col = "#cc8800", lwd = 1.2, fill = "#cc8800")
excl_fill    <- gpar(fill = "#fff8e1", col = "#cc8800", lwd = 1.2)
badge_gp     <- gpar(fill = "#336699", col = NA)
badge_txt_gp <- gpar(col = "white", cex = 0.65)

# Arms span the inner portion; lost boxes flank outside on each side.
# Using 0.28–0.72 keeps the arm centres far enough from the viewport edges
# that the lost boxes' right/left edges don't cross the arm centres.
left_from <- 0.28
left_to   <- 0.72
main_x    <- (left_from + left_to) / 2   # centre of the arms column (= 0.5)

grid.newpage()
flowchart(
  assessed = boxGrob(
    "Patients assessed for eligibility",
    x = main_x, box_gp = box_fill,
    badge_label = "840", badge_gp = badge_gp, badge_txt_gp = badge_txt_gp
  ),
  randomised = boxGrob(
    "Randomised",
    x = main_x, box_gp = box_fill,
    badge_label = "126", badge_gp = badge_gp, badge_txt_gp = badge_txt_gp
  ),
  arms = list(
    cast     = boxGrob("Randomised to\ncast immobilisation",
                       box_gp = box_fill,
                       badge_label = "62",
                       badge_gp = badge_gp, badge_txt_gp = badge_txt_gp),
    surgical = boxGrob("Randomised to\nsurgery",
                       box_gp = box_fill,
                       badge_label = "64",
                       badge_gp = badge_gp, badge_txt_gp = badge_txt_gp)
  ),
  # Lost-to-follow-up: one per arm — spread from 0 to 1 so they land
  # on the outside of each arm (left of cast, right of surgical)
  lost = list(
    lost_cast     = boxGrob("Lost to follow-up (n = 2)\n  1 No response\n  1 Other surgery",
                            just = "left", box_gp = excl_fill),
    lost_surgical = boxGrob("Lost to follow-up (n = 3)\n  2 No response\n  1 Other surgery",
                            just = "left", box_gp = excl_fill)
  ),
  analysis = list(
    analysis_cast     = boxGrob("Included in\nprimary analysis",
                                box_gp = box_fill,
                                badge_label = "60",
                                badge_gp = badge_gp, badge_txt_gp = badge_txt_gp),
    analysis_surgical = boxGrob("Included in\nprimary analysis",
                                box_gp = box_fill,
                                badge_label = "61",
                                badge_gp = badge_gp, badge_txt_gp = badge_txt_gp)
  )
) |>
  # Vertical spacing — extra margin to keep badges from being clipped at the top
  spread(axis = "y", margin = unit(5, "mm")) |>
  # Make arms and analysis boxes the same width so the same from/to spread
  # places their centres at matching x positions (otherwise the wider arm text
  # shifts centres relative to the narrower analysis text).
  # Alternative: skip equalizeWidths() and explicitly align x-centres,
  # e.g. with `align(axis = "x", reference = "analysis", subelement = "arms")`
  # (equivalent to `alignHorizontal(reference = "analysis", subelement = "arms")`).
  equalizeWidths(subelement = list("arms", "analysis")) |>
  # Arms and analysis in the inner span; lost boxes spread across full width
  # so that lost_cast lands left of cast and lost_surgical right of surgical
  spread(axis = "x", subelement = "arms",     from = left_from, to = left_to) |>
  spread(axis = "x", subelement = "analysis", from = left_from, to = left_to) |>
  spread(axis = "x", subelement = "lost",     from = 0,         to = 1) |>
  # Exclusion box: auto-positioned between assessed and randomised, then moved right
  insert(list(excluded = boxGrob(
    "Excluded (n = 714)\n  477 Stable ankle mortise\n   64 Incongruent ankle mortise\n   30 Previous serious trauma\n  143 Other reasons",
    just = "left", box_gp = excl_fill
  )), after = "assessed") |>
  move(subelement = "excluded", x = 0.85) |>
  # Main-flow connectors
  connect("assessed",   "randomised", type = "v", lty_gp = con_gp,  arrow_size = 3, smooth = TRUE) |>
  connect("randomised", "arms",       type = "N", lty_gp = con_gp,  arrow_size = 3, smooth = TRUE) |>
  connect("arms",       "analysis",   type = "v", lty_gp = con_gp,  arrow_size = 3) |>
  # type = "L": exits assessed's *bottom* then turns right — the "down then right" branch
  connect("assessed", "excluded", type = "L", lty_gp = side_gp, arrow_size = 3, smooth = TRUE) |>
  # Pairwise arm -> lost: sharp corners (smooth = FALSE) avoid a colour-transition
  # artefact where the orange arc would diverge from the shared blue vertical path
  # a few mm above the junction, making the line appear doubled.
  connect("arms", "lost", type = "L", lty_gp = side_gp, arrow_size = 3, smooth = TRUE)

options(old_opts)
```

# Consistent grouped widths and global text padding

When building grouped flows (for example CONSORT-style diagrams), it is often useful to:

- reduce text padding globally for all `boxGrob()` calls
- make corresponding boxes in different levels share the same width

The snippet below demonstrates both via `options(boxGrobTxtPadding = ...)` and `equalizeWidths()`.

```{r flowchart-group-width-and-padding, fig.height = 5.5, fig.width = 8}
old_opts <- options(boxGrobTxtPadding = unit(2, "mm"))

flowchart(
  rando = glue("Randomised\nN = 100"),
  groups = list(
    glue("Group 1\nn = 50"),
    glue("Group 2\nn = 50")
  ),
  groups2 = list(
    glue("Analysed\nn = 49"),
    glue("Analysed\nn = 48")
  )
) |>
  spread(axis = "y", margin = unit(0.02, "npc")) |>
  spread(subelement = "groups", axis = "x", margin = unit(.05, "npc")) |>
  spread(subelement = "groups2", axis = "x", margin = unit(.05, "npc")) |>
  equalizeWidths(subelement = list("groups", "groups2")) |>
  connect("rando", "groups", type = "N") |>
  connect("groups", "groups2", type = "vertical")

options(old_opts)
```

# Basic components explained

There is a basic set of components that are used for generating flowcharts:

- Boxes, these are generated through the `boxGrob` and `boxPropGrob` functions.
- Arrows between boxes, these are generated through the `connectGrob` function.

These can be positioned directly or preferably manipulated according to the following principles:

- Spread - we want to use the full plot and either we position each element or we automatically spread them in a vertical or horizontal direction, `spreadHorizontal` and `spreadVertical` functions.
- Alignment of boxes - before or after spreading we may want to align boxes: `alignHorizontal` and `alignVertical` functions.

# A basic box

We can start with outputting a single box:

```{r basic_box, fig.height = 1.5, fig.width = 3, message = FALSE}
grid.newpage()
txt <-
"Just a plain box
with some text
- Note that newline is OK"
boxGrob(txt)
```

We can position and style this box as any element:

```{r styled_box, fig.height = 3, fig.width = 3}
grid.newpage()
boxGrob("A large\noffset\nyellow\nbox",
        width = .8, height = .8,
        x = 0, y = 0,
        bjust = c("left", "bottom"),
        txt_gp = gpar(col = "darkblue", cex = 2),
        box_gp = gpar(fill = "lightyellow", col = "darkblue"))
```

# A box with proportions

The `boxPropGrob` is for displaying proportions as the name indicates.

```{r prop_box, fig.height = 2, fig.width = 4}
grid.newpage()
boxPropGrob("A box with proportions",
            "Left side", "Right side",
            prop = .7)
```

# The box coordinates

The boxes have coordinates that allow you to easily draw lines to and from it. The coordinates are stored in the `coords` attribute. Below is an illustration of the coordinates for the two boxes:

```{r, fig.height = 3, fig.width = 4}
grid.newpage()
smpl_bx <- boxGrob(
  label = "A simple box",
  x = .5,
  y = .9,
  just = "center")

prop_bx <- boxPropGrob(
  label = "A split box",
  label_left = "Left side",
  label_right = "Right side",
  x = .5,
  y = .3,
  prop = .3,
  just = "center")

plot(smpl_bx)
plot(prop_bx)

smpl_bx_coords <- coords(smpl_bx)
grid.circle(y = smpl_bx_coords$y,
            x = smpl_bx_coords$x,
            r = unit(2, "mm"),
            gp = gpar(fill = "#FFFFFF99", col = "black"))
grid.circle(y = smpl_bx_coords$bottom,
            x = smpl_bx_coords$right,
            r = unit(1, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = smpl_bx_coords$top,
            x = smpl_bx_coords$right,
            r = unit(1, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = smpl_bx_coords$bottom,
            x = smpl_bx_coords$left,
            r = unit(1, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = smpl_bx_coords$top,
            x = smpl_bx_coords$left,
            r = unit(1, "mm"),
            gp = gpar(fill = "orange"))

prop_bx_coords <- coords(prop_bx)
grid.circle(y = prop_bx_coords$y,
            x = prop_bx_coords$x,
            r = unit(2, "mm"),
            gp = gpar(fill = "#FFFFFF99", col = "black"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$right_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$right_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$left_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$left_x,
            r = unit(1, "mm"),
            gp = gpar(fill = "orange"))

grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$right,
            r = unit(2, "mm"),
            gp = gpar(fill = "red"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$right,
            r = unit(2, "mm"),
            gp = gpar(fill = "purple"))
grid.circle(y = prop_bx_coords$bottom,
            x = prop_bx_coords$left,
            r = unit(2, "mm"),
            gp = gpar(fill = "blue"))
grid.circle(y = prop_bx_coords$top,
            x = prop_bx_coords$left,
            r = unit(2, "mm"),
            gp = gpar(fill = "orange"))
```

# Additional box shapes

You can create alternate box shapes by passing a custom `box_fn` to `boxGrob`,
or use the convenience helpers `boxDiamondGrob()`, `boxEllipseGrob()` and
`boxRackGrob()` that ship with the package.

```{r extra_shapes, fig.height = 3, fig.width = 6}
# --- Branch labels + sharp diamond variant ---
grid.newpage()

# rounded and sharp diamond examples
d_rounded <- boxDiamondGrob("Decision", box_gp = gpar(fill = "#FFF4E6"))
d_sharp   <- boxDiamondGrob("Decision\n(sharp)", rounded = FALSE, box_gp = gpar(fill = "#FFF4E6"))

# outcomes
e <- boxEllipseGrob("Local", box_gp = gpar(fill = "#E6FFF4"))
r <- boxServerGrob("Server", box_gp = gpar(fill = "#E8F0FF"))

# arrange and draw
boxes <- list(decision = d_rounded, outcomes = list(e, r)) |>
  spreadHorizontal(from = unit(.1, "npc"), to = unit(.9, "npc"), subelement = "outcomes") |>
  spreadVertical() |>
  print()

# 1) quick many-to-many style connector (no labels)
con <- connectGrob(boxes$decision, boxes$outcomes, type = "N")
print(con)

# 2) explicit per-branch connectors with labels (preferred when you want text)
connectGrob(boxes$decision, boxes$outcomes[[1]], type = "N", label = "Local") |> print()
connectGrob(boxes$decision, boxes$outcomes[[2]], type = "N", label = "Server") |> print()

# 3) If you prefer the single connector and want labels on each branch:
#    place text at the midpoint of each returned grob (example)
con_list <- connectGrob(boxes$decision, boxes$outcomes, type = "N")
# Preferred: attach labels and let `print()` handle rendering
con_list <- setConnectorLabels(con_list, c("Local", "Server"))
print(con_list)
```

# Standard flowchart shapes

Below are a few commonly-used flowchart shapes demonstrating their typical
appearance and usage.

```{r standard_shapes, fig.height = 3.5, fig.width = 8}
# Arrange shapes in three rows for better readability
# 1) Grid-based objects (basic boxGrob / boxPropGrob / rect)
row1 <- list(
  boxGrob("Box (default)", box_gp = gpar(fill = "#EFEFEF"), y = unit(.85, "npc")),
  boxPropGrob("Prop", "Left", "Right", prop = .4, box_left_gp = gpar(fill = "#EFEFAF"), box_right_gp = gpar(fill = "#EFAFEF"), y = unit(.85, "npc")),
  boxGrob("Rectangle", box_fn = rectGrob, box_gp = gpar(fill = "#EFEFEF"), y = unit(.85, "npc"))
)

# 2) Gmisc row 1 (rounded/sharp diamond + ellipse + rack + server)
row2 <- list(
  boxDiamondGrob("Diamond\n(rounded)", box_gp = gpar(fill = "#FFF4E6"), y = unit(.55, "npc")),
  boxDiamondGrob("Diamond\n(sharp)", rounded = FALSE, box_gp = gpar(fill = "#FFF4E6"), y = unit(.55, "npc")),
  boxEllipseGrob("Ellipse", box_gp = gpar(fill = "#E6FFF4"), y = unit(.55, "npc")),
  boxRackGrob("Rack", box_gp = gpar(fill = "#E8F0FF"), y = unit(.55, "npc")),
  boxServerGrob("Server", box_gp = gpar(fill = "#E8F0FF"), y = unit(.55, "npc"))
)

# 3) Gmisc row 2 (database, document, documents, tape)
row3 <- list(
  boxDatabaseGrob("Database", box_gp = gpar(fill = "#DFF4E6"), y = unit(.25, "npc")),
  boxDocumentGrob("Document", box_gp = gpar(fill = "#FFF6E6"), y = unit(.25, "npc")),
  boxDocumentsGrob("Documents", box_gp = gpar(fill = "#FFF6E6"), y = unit(.25, "npc")),
  boxTapeGrob("Tape", box_gp = gpar(fill = "#E6F0FF"), y = unit(.25, "npc"))
)

# Spread each row across the horizontal span
spreadHorizontal(row1, from = unit(.05, "npc"), to = unit(.95, "npc"))
spreadHorizontal(row2, from = unit(.05, "npc"), to = unit(.95, "npc"))
spreadHorizontal(row3, from = unit(.05, "npc"), to = unit(.95, "npc"))
```

# Connecting the boxes

In order to make connecting boxes with an arrow there is the `connectGrob` function. Here's an example of how you can use it for connecting a set of boxes:

```{r "Connected boxes", fig.width = 7, fig.height = 5}
grid.newpage()
# Initiate the boxes that we want to connect
side <- boxPropGrob("Side", "Left", "Right",
                    prop = .3,
                    x = 0, y = .9,
                    bjust = c(0,1))
start <- boxGrob("Top",
                 x = .6, y = coords(side)$y,
                 box_gp = gpar(fill = "yellow"))
bottom <- boxGrob("Bottom",
                  x = .6, y = 0,
                  bjust = "bottom")


sub_side_left <- boxGrob("Left",
                         x = coords(side)$left_x,
                         y = 0,
                         bjust = "bottom")
sub_side_right <- boxGrob("Right",
                          x = coords(side)$right_x,
                          y = 0,
                          bjust = "bottom")

odd <- boxGrob("Odd\nbox",
               x = coords(side)$right,
               y = .5)

odd2 <- boxGrob("Also odd",
               x = coords(odd)$right +
                 distance(bottom, odd, type = "h", half = TRUE) -
                 unit(2, "mm"),
               y = 0,
               bjust = c(1,0))

exclude <- boxGrob("Exclude:\n - Too sick\n - Prev. surgery",
                   x = 1,
                   y = coords(bottom)$top +
                     distance(start, bottom, type = "v", half = TRUE),
                   just = "left", bjust = "right")

# Connect the boxes and print/plot them
connectGrob(start, bottom, "vertical")
connectGrob(start, side, "horizontal")
connectGrob(bottom, odd, "Z", "l")
connectGrob(odd, odd2, "N", "l")
connectGrob(side, sub_side_left, "v", "l")
connectGrob(side, sub_side_right, "v", "r")
connectGrob(start, exclude, "-",
            lty_gp = gpar(lwd = 2, col = "darkred", fill = "darkred"))

# Print the grobs
start
bottom
side
exclude
sub_side_left
sub_side_right
odd
odd2
```

`connectGrob()` also supports connecting one box to many boxes, or many boxes to one box.
For merging many boxes into one, `type = "fan_in_top"` distributes the attachment points evenly along the top edge of the destination box, with an optional margin.

```{r connect_multi, fig.width = 4, fig.height = 4}
grid.newpage()

# Three upstream boxes + one side box
a_boxes <- paste("A", 1:3) |>
  lapply(\(x) boxGrob(x, box_gp = gpar(fill = "#E6F2FF"))) |>
  spreadHorizontal(from = unit(.1, "npc"), to = unit(1, "npc") - unit(1, "cm")) |>
  alignVertical(position="top",
                reference = unit(1, "npc")) |>
  print()

b_side <- boxGrob("B",  y = .70, box_gp = gpar(fill = "#FFF3BF")) |>
  moveBox(x = unit(1, "npc"),
          just = 1) |>
  print()

# Target box
c <- boxGrob("C", x = .50, box_gp = gpar(fill = "#D3F9D8"), width = unit(4, "cm")) |>
  moveBox(y = unit(0, "npc"),
          just = "bottom") |>
  print()


# Many -> one: merge on top with evenly distributed attachment points + margin
connectGrob(c(a_boxes, list(b_side)), c,
            type = "fan_in_top",
            margin = 4)
```

# Alignment

We frequently want to align boxes in either a horizontal or a vertical row. For this there are two functions, `alignHorizontal()` and `alignVertical()`.

```{r horizontal_alignment, fig.width=10, fig.height=6}
align_1 <- boxGrob("Align 1",
                   y = .9,
                   x = 0,
                   bjust = c(0),
                   box_gp = gpar(fill = "#E6E8EF"))

align_2 <- boxPropGrob("Align 2",
                       "Placebo",
                       "Treatment",
                       prop = .7,
                       y = .8,
                       x = .5)

align_3 <- boxGrob("Align 3\nvertical\ntext",
                   y = 1,
                   x = 1,
                   bjust = c(1, 1),
                   box_gp = gpar(fill = "#E6E8EF"))

b1 <- boxGrob("B1",
              y = .3,
              x = .1,
              bjust = c(0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .6,
              x = .5)
b3 <- boxGrob("B3",
              y = .2,
              x = .8,
              bjust = c(0, 1))

grid.newpage()
align_1
alignHorizontal(reference = align_1,
                b1, b2, b3,
                position = "left")

align_2
alignHorizontal(reference = align_2,
                b1, b2, b3,
                position = "center",
                sub_position = "left")
alignHorizontal(reference = align_2,
                b1, b2, b3,
                position = "left",
                sub_position = "right")

align_3
alignHorizontal(reference = align_3,
                b1, b2, b3,
                position = "right")
```

Here are similar examples of vertical alignment:

```{r vertical_alignment, fig.width=10, fig.height=6}
align_1 <- boxGrob("Align 1\nvertical\ntext",
                   y = 1,
                   x = 1,
                   bjust = c(1, 1),
                   box_gp = gpar(fill = "#E6E8EF"))

align_2 <- boxPropGrob("Align 2",
                       "Placebo",
                       "Treatment",
                       prop = .7,
                       y = .5,
                       x = .6)

align_3 <- boxGrob("Align 3",
                   y = 0,
                   x = 0,
                   bjust = c(0, 0),
                   box_gp = gpar(fill = "#E6E8EF"))


b1 <- boxGrob("B1",
              y = .3,
              x = 0.1,
              bjust = c(0, 0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .6,
              x = .3)
b3 <- boxGrob("B3",
              y = .2,
              x = .85,
              bjust = c(0, 1))

grid.newpage()
align_1
alignVertical(reference = align_1,
              b1, b2, b3,
              position = "top")

align_2
alignVertical(reference = align_2,
              b1, b2, b3,
              position = "center")

align_3
alignVertical(reference = align_3,
              b1, b2, b3,
              position = "bottom")
```

# Spreading

Similarly to alignment we often want to spread our boxes within a space so that we use all the available space in the `viewport`. This can be done through the `spreadHorizontal()` and `spreadVertical()`. You can both spread the entire span or only between a subspan that is defined using the `.to` and `.from` arguments.

Numeric `.from`, `.to`, and `.margin` values are interpreted as proportions of the current viewport (`npc`).
If only one of `.from` or `.to` is specified, the other defaults to the full span (`0 npc` or `1 npc`).
The `.margin` argument adds padding at both ends of the span (also when using `.from`/`.to`).

```{r horizontal_spread, fig.width = 11, fig.height = 8}
b1 <- boxGrob("B1", y = .85, x = .1, bjust = c(0, 0))
b2 <- boxGrob("B2", y = .65, x = .6)
b3 <- boxGrob("B3", y = .45, x = .6)
b4 <- boxGrob("B4 with long\ndescription", y = .7, x = .8)

from <- boxGrob("from",
                y = .25,
                x = .05,
                box_gp = gpar(fill = "darkgreen"),
                txt_gp = gpar(col = "white"))
to <- boxGrob("to this wide box",
              y = coords(from)$y,
              x = .95,
              bjust = "right",
              box_gp = gpar(fill = "darkred"),
              txt_gp = gpar(col = "white"))
txtOut <- function(txt, y_top) {
  grid.text(txt,
            x = unit(2, "mm"),
            y = y_top + unit(2, "mm"),
            just = c("left", "bottom"))
  grid.lines(y = y_top + unit(1, "mm"),
             gp = gpar(col = "grey"))
}

drawRow <- function(label, row_y, spread_args = list()) {
  row <- alignVertical(reference = row_y, b1, b2, b3, b4, position = "top")
  txtOut(label, coords(row[[1]])$top)
  do.call(spreadHorizontal, c(list(row), spread_args))
}

rowYs <- unit(c(.93, .76, .59, .42, .25, .12), "npc")

grid.newpage()

drawRow("Basic (viewport)", rowYs[1])
drawRow("From–to + margin (numeric = npc)", rowYs[2],
        spread_args = list(from = .2, to = .7, margin = .05))
drawRow("Only to (defaults from = 0)", rowYs[3],
        spread_args = list(to = .7))
drawRow("Only from (defaults to = 1)", rowYs[4],
        spread_args = list(from = .2))

# Row 5: Between boxes (box-to-box span)
row5_y <- rowYs[5]
row5 <- alignVertical(reference = row5_y, b1, b2, b3, b4, position = "top")
txtOut("Between boxes", coords(row5[[1]])$top)

span <- alignVertical(reference = row5_y, from  = from, to = to, position = "top")
span
spreadHorizontal(row5, from = span$from, to = span$to)

# Row 6: Reverse box order + center distribution
row6_y <- unit(.10, "npc")

bottom_from <- moveBox(from, x = coords(to)$right, y = 0, just = c(1, 0))
bottom_to <- moveBox(to, x = coords(from)$left, y = 0, just = c(0, 0))
bottom_from
bottom_to

row6 <- alignVertical(reference = bottom_from, b1, b2, b3, b4, position = "bottom")
txtOut("Reverse box order + center", coords(row6[[4]])$top)

spreadHorizontal(row6,
                 from = bottom_from,
                 to = bottom_to,
                 type = "center")


```

Vertical spreading follows the same pattern:

```{r vertical_spread, fig.width=6, fig.height=6}
b1 <- boxGrob("B1",
              y = .8,
              x = 0.1,
              bjust = c(0, 0))
b2 <- boxGrob("B2 with long\ndescription",
              y = .5,
              x = .5)
b3 <- boxGrob("B3",
              y = .2,
              x = .8)
b4 <- boxGrob("B4",
              y = .7,
              x = .8)


txtOut <- function(txt, refBx) {
  grid.text(txt,
            x = coords(refBx)$left - unit(2, "mm"),
            y = .5,
            just = c("center", "bottom"),
            rot = 90)
  grid.lines(x = coords(refBx)$left - unit(1, "mm"),
             gp = gpar(col = "grey"))
}

grid.newpage()
txtOut("Basic", b1)
alignHorizontal(reference = b1,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical()

txtOut("From-to", b2)
alignHorizontal(reference = b2,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical(from = .2,
                 to = .7)

txtOut("From-to with center and reverse the box order", b3)
alignHorizontal(reference = b3,
                b1, b2, b3, b4,
                position = "left") |>
  spreadVertical(from = .7,
                 to = .2,
                 type = "center")
```

# Complex nested flowcharts with boxHeaderGrob

The `boxHeaderGrob()` function creates boxes with centered headers and left-justified body text, perfect for flowcharts with structured information like timelines or protocols. Here's an example of a randomized trial flowchart with shared criteria and two treatment arms:

```{r complex_nested, fig.width = 7, fig.height = 8}
# Helper function to convert nested structure to grobs
make_boxes <- function(x) {
  if (is.list(x) && !inherits(x, "box_header")) {
    return(lapply(x, make_boxes))
  }

  if (inherits(x, "box_header")) {
    return(do.call(boxHeaderGrob, x))
  }

  # Simple text box fallback
  args <- attr(x, "args")
  if (is.null(args)) return(boxGrob(label = x))

  args$label <- x
  do.call(boxGrob, args)
}

# Define styling for different elements
arm_a_style <- list(
  header = gpar(fill = "#E8F5E9", col = "#2E7D32", lwd = 1.4),
  box = gpar(fill = "#F1F8E9", col = "#43A047")
)

arm_b_style <- list(
  header = gpar(fill = "#FFF8E1", col = "#EF6C00", lwd = 1.4),
  box = gpar(fill = "#FFFDE7", col = "#F9A825")
)

# Build flowchart structure
flowchart <- list(
  # Shared inclusion criteria
  criteria = structure(
    list(
      header = "Inclusion Criteria",
      body = paste(
        "• Adults aged 18-65",
        "• Confirmed diagnosis",
        "• Written informed consent",
        "• No contraindications",
        "• Available for 6-month follow-up",
        sep = "\n"
      ),
      box_gp = gpar(fill = "#E3F2FD", col = "#1E88E5", lwd = 1.4),
      body_gp = gpar(fontsize = 10)
    ),
    class = "box_header"
  ),

  # Two treatment arms
  arms = list(
    arm_a = list(
      # Arm header
      structure("Intensive Protocol", args = list(
        box_gp = arm_a_style$header,
        txt_gp = gpar(fontsize = 11, fontface = "bold")
      )),

      # Timeline boxes
      structure(list(
        header = "Week 0-1",
        body = "• Daily sessions\n• Supervised therapy\n",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Week 2-4",
        body = "• 3× weekly sessions\n• Progressive loading",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Week 5-8",
        body = "• Home program\n• Monthly check-ins\n• Return to activity",
        box_gp = arm_a_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header")
    ),

    arm_b = list(
      # Arm header
      structure("Standard Care", args = list(
        box_gp = arm_b_style$header,
        txt_gp = gpar(fontsize = 11, fontface = "bold")
      )),

      # Timeline boxes - different schedule
      structure(list(
        header = "Month 0",
        body = "• Initial consultation\n• Exercise booklet",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Month 3",
        body = "• Follow-up visit\n• Progress review",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header"),

      structure(list(
        header = "Month 6",
        body = "• Final assessment\n• Discharge planning",
        box_gp = arm_b_style$box,
        body_gp = gpar(fontsize = 9.5)
      ), class = "box_header")
    )
  )
)

# Convert to grobs and layout
grid.newpage()
boxes <- flowchart |>
  make_boxes() |>
  spreadVertical() |>
  spreadHorizontal(subelement = "arms", from = 0.15, to = 0.85) |>
  spreadVertical(subelement = c("arms", "arm_a"), from = 0.65) |>
  spreadVertical(subelement = c("arms", "arm_b"), from = 0.65) |>
  print()

# Connect criteria to both arms
connectGrob(boxes$criteria, boxes$arms, type = "N")

# Connect timeline within each arm
for (arm_name in names(boxes$arms)) {
  arm_boxes <- boxes$arms[[arm_name]]
  for (i in 2:length(arm_boxes)) {
    connectGrob(arm_boxes[[i-1]], arm_boxes[[i]], type = "v") |> print()
  }
}
```

# The S3 Layout API

The package provides a pipelined API where you can construct your flowchart as a single object (a list of boxes with connection attributes) and then print/plot it. This reduces the need for manual loops and `print()` calls for connectors.

```{r s3_api_example, fig.height=8, fig.width=6, eval=FALSE}
grid.newpage()

# Define the nodes
b1 <- boxGrob("Start", y = 0.8)
b2 <- boxGrob("Process", y = 0.5)
b3 <- boxGrob("End", y = 0.2)

# Pipeline: list -> align -> connect -> print
list(start = b1, process = b2, end = b3) |>
  align(axis = "y") |>
  spread(axis = "x") |>
  connect("start", "process", type = "horizontal") |>
  connect("process", "end", type = "horizontal") |>
  print()
```

# Math expressions in boxes

It is possible to use the R `expression` or the `bquote` functions to produce bold or italics text, or even formulas.

A few pointers on `expression`...

- expressions with multiple elements should be combined using `paste`. E.g. `expression(paste(beta, "1"))` would produce $\beta1$
- the behavior of `paste` when used in expression is more like the normal behavior or `paste0` (i.e. no separating space)
- Greek letters can be entered outside of quotes by typing the name e.g. `expression(beta)` will become $\beta$ and `expression(Gamma)` will become $\Gamma$ (note the case, not all Greek letters are available in upper case)
- superscripts are done via `expression(x^2)` and subscripts via `expression(x[2])`

```{r math_expressions, fig.width=6, fig.height=3}
grid.newpage()
###############
# Expressions #
###############
# Font style
list(expression(bold("Bold text")),
     expression(italic("Italics text")),
     expression(paste("Mixed: ", italic("Italics"), " and ", bold("bold")))) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(1, "npc"),
                position = "top") |>
  spreadHorizontal()

# Math
list(expression(paste("y = ", beta[0], " + ", beta[1], X[1], " + ", beta[2], X[2]^2)),
     expression(paste(hat(mu) == sum(frac(x[i], n), i == 1, n))),
     expression(paste(int(a, b, f(x) * dx) == F(b) - F(a)))) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(0.5, "npc"),
                position = "center") |>
  spreadHorizontal()

##########
# Quotes #
##########
a = 5
list(bquote(alpha == theta[1] * .(a) + ldots),
     paste("argument", sQuote("x"), "\nmust be non-zero")) |>
  lapply(boxGrob) |>
  alignVertical(reference = unit(0, "npc"),
                position = "bottom") |>
  spreadHorizontal(from = .2, to = .8)
```

See the `plotmath` help file for more details.

# Grid & some background info

The `grid` package is what makes R graphics great. All the popular tools with awesome graphics use the `grid` as the back-end, e.g. `ggplot2` and `lattice`. When I started working on the `forestplot` package I first encountered the `grid` and it was instant love. In this vignette I'll show how you can use the flowchart-functions in this package together with `grid` in order to generate a flowchart.

## Basics

The grid package splits the plot into views. You can define a `viewport` and it will work as an isolated part of the plot, ignorant of the world around it. You do this via `viewport()`, below I create a plot and add a rectangle to it:

```{r basic_plot, fig.height = 2, fig.width = 2}
# Load the grid library
# part of standard R libraries so no need installing
library(grid)

# Create a new graph
grid.newpage()

pushViewport(viewport(width = .5, height = .8))

grid.rect(gp = gpar(fill = "#D8F0D1"))

popViewport()
```

Important to note is that the grid allows you to define precise units or relative units.

## Relative units

- `npc` - ranges from 0-1 where 1 is 100% of the `viewport` width.
- `snpc` - similar to `npc` but is the same length in height/width.
- `lines` - the height of a line. The go-to method if you want to know the height of a few lines of text. It's relative to the `viewport`'s `fontsize` and `lineheight`.
- `char` - the `lines` without the `lineheight` part.

Below we draw a line with relative units in two nested viewports. Note that the to lines are generated from the exact same `grob` object but appear different depending on the `viewport` they are in:

```{r relative_lines, fig.height = 3, fig.width = 3}
grid.newpage()
pushViewport(viewport(width = .5, height = .8, clip = "on"))
grid.rect(gp = gpar(lty = 2, fill = "lightyellow"))
lg <- linesGrob(x = unit(c(.2, 1), "npc"),
                y = unit(c(.2, 1), "npc"),
                gp = gpar(lwd = 2))
grid.draw(lg)
pushViewport(viewport(x = 0, y = .6, just = "left", width = .4, height = .4, angle = 20))
grid.rect(gp = gpar(fill = "lightblue")) # A translucent box to indicate the new viewport
grid.draw(lg)
popViewport()
```

## Absolute units

- `mm` - probably my go-to unit when I want something absolute.
- `inch` - if you prefer inches I guess this is the go-to choice.

Below we draw a line with absolute units in two nested `viewports`. Note that the lines have the exact same length:

```{r absolute_lines, fig.height = 3, fig.width = 3}
grid.newpage()
pushViewport(viewport(width = .5, height = .8, clip = "on"))
grid.rect(gp = gpar(lty = 2, fill = "lightyellow"))
lg <- linesGrob(x = unit(c(2, 10), "mm"),
                y = unit(c(2, 10), "mm"),
                gp = gpar(lwd = 2))
grid.draw(lg)
pushViewport(viewport(x = 0, y = .6, just = "left", width = .4, height = .4, angle = 20))
grid.rect(gp = gpar(fill = "lightblue")) # A translucent box to indicate the new viewport
grid.draw(lg)
popViewport()
```

# A complex example

Here is a more complex example demonstrating the power of the grid-based flowcharts. This example uses `flowchart`, `spread`, `align` and `connect` functions to create a detailed clinical study flowchart.

```{r complex_example, fig.height = 9, fig.width = 9}
# Define the boxes
org_cohort <- glue("Proximal humerus fracture",
                   "  - \u2265 18 years",
                   "  - \u2264 4 weeks of trauma",
                   "  - Not pathological",
                   .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#E3F2FD"))

surgery <- glue("Surgery",
                "  - Direct (\u2248 4%)",
                "  - Delayed (\u2248 4%)",
                .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#F8BBD0"))

randomize <- boxGrob("Non-surgical\nRandomise",
                     box_gp = gpar(fill = "#FFF3E0"))

treatments <- list(early = boxGrob("Early rehab",
                                   box_gp = gpar(fill = "#DCEDC8")),
                   late = boxGrob("Late rehab",
                                  box_gp = gpar(fill = "#DCEDC8")),
                   obs = boxGrob("Observation",
                                 box_gp = gpar(fill = "#E0E0E0")))

early_followup <- glue("Early follow-up",
                       "  - 2 weeks [PNRS]",
                       "  - 4 weeks [PNRS]",
                       .sep = "\n") |>
  boxGrob(just = "left",
          box_gp = gpar(fill = "#E0F7FA"))

late_followup <- glue("Late follow-up",
                      "  - 2-10 months (random) [OSS, PNRS]",
                      "  - 1 year [OSS, PNRS, accelerometer]",
                      "  - 2 years [OSS, PNRS]",
                      "  - 5 years [OSS, PNRS]",
                      .sep = "\n") |>
  boxGrob(just = "left",
                       box_gp = gpar(fill = "#E0F7FA"))

# Create the flowchart
grid.newpage()
flowchart(start = org_cohort,
          step_1 = list(surgery = surgery,
                        `non-surgical` = randomize),
          treatment = treatments,
          early_followup = early_followup,
          followup = late_followup) |>
  spread(axis = "y") |>
  spread(axis = "x", subelement = "step_1") |>
  spread(axis = "x", subelement = "treatment", from = 0.35) |>
  align(axis = "x",
        reference = c("treatment", "late"),
        subelement = c("step_1", "non-surgical")) |>
  connect(from = "start", to = "step_1", type = "N") |>
  connect(from = "step_1$non-surgical", to = "treatment", type = "N") |>
  connect(from = "treatment", to = "early_followup", type = "fan_in_center") |>
  connect(from = "early_followup", to = "followup", type = "v") |>
  connect(from = "early_followup", to = "step_1$surgery", type = "Z",
          label = "Crossover\nto surgery") |>
  connect(from = "step_1$surgery", to = "followup", type = "L") |>
  print()
```

# Tips for debugging

If you find that your elements don't look as expected make sure that your not changing `viewport`/device. While most coordinates are relative some of them need to be fixed and therefore changing the `viewport` may impact where elements are rendered.

---
title: "CONSORT-Statement checklist"
lang: en
output:
  Gmisc::docx_document:
    fig_caption: true
---

```{r "Load libraries", echo=FALSE, message=FALSE, warning=FALSE}
library(Gmisc)
```

```{r "Load data"}

```

```{r "Munge data"}

```

# Abstract
1a  Identification as a randomised trial in the title  
1b	Structured summary of trial design, methods, results, and conclusions (for specific guidance see CONSORT for abstracts)  

# Introduction

## Background/rationale
Scientific background and explanation of rationale

## Objectives	
Specific objectives or hypotheses

# Methods

## Trial design
Description of trial design (such as parallel, factorial) including allocation ratio  
Important changes to methods after trial commencement (such as eligibility criteria), with reasons  

## Participants	
Eligibility criteria for participants  
Settings and locations where the data were collected  

## Interventions
The interventions for each group with sufficient details to allow replication, including how and when they were actually administered

## Outcomes
Completely defined pre-specified primary and secondary outcome measures, including how and when they were assessed  
Any changes to trial outcomes after the trial commenced, with reasons  

## Sample size
How sample size was determined  
When applicable, explanation of any interim analyses and stopping guidelines

## Randomisation:		

### Sequence generation
Method used to generate the random allocation sequence  
Type of randomisation; details of any restriction (such as blocking and block size)  

### Allocation concealment mechanism
Mechanism used to implement the random allocation sequence (such as sequentially numbered containers), describing any steps taken to conceal the sequence until interventions were assigned

### Implementation
Who generated the random allocation sequence, who enrolled participants, and who assigned participants to interventions

## Blinding	
If done, who was blinded after assignment to interventions (for example, participants, care providers, those assessing outcomes) and how  
If relevant, description of the similarity of interventions

## Statistical methods
Statistical methods used to compare groups for primary and secondary outcomes  
Methods for additional analyses, such as subgroup analyses and adjusted analyses  

# Results

## Participant flow (a diagram is strongly recommended)
For each group, the numbers of participants who were randomly assigned, received intended treatment, and were analysed for the primary outcome  
For each group, losses and exclusions after randomisation, together with reasons  

## Recruitment
Dates defining the periods of recruitment and follow-up  
Why the trial ended or was stopped

## Baseline data
A table showing baseline demographic and clinical characteristics for each group

## Numbers analysed
For each group, number of participants (denominator) included in each analysis and whether the analysis was by original assigned groups

## Outcomes and estimation
For each primary and secondary outcome, results for each group, and the estimated effect size and its precision (such as 95% confidence interval)  
For binary outcomes, presentation of both absolute and relative effect sizes is recommended

## Ancillary analyses
Results of any other analyses performed, including subgroup analyses and adjusted analyses, distinguishing pre-specified from exploratory

## Harms
All important harms or unintended effects in each group (for specific guidance see CONSORT for harms)

# Discussion
## Limitations
Trial limitations, addressing sources of potential bias, imprecision, and, if relevant, multiplicity of analyses

## Generalisability
Generalisability (external validity, applicability) of the trial findings

## Interpretation
Interpretation consistent with results, balancing benefits and harms, and considering other relevant evidence

# Other information

## Registration
Registration number and name of trial registry

## Protocol
Where the full trial protocol can be accessed, if available

## Funding
Sources of funding and other support (such as supply of drugs), role of funders

---
title: "Introduction to the Transition-class"
author: "Max Gordon"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Introduction to the Transition-class}
  %\usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

Introduction
============

The *Transition*-class is the advanced version of the transitionPlot that aims at illustrating transitions between classes over time. The original intent was to show how [self-administered the Charnley classification](https://www.tandfonline.com/doi/full/10.3109/17453674.2014.931199) behaves before and after surgery. The plot is a fancier version than what can be achieved using packages such as [*diagram*](https://CRAN.R-project.org/package=diagram), although it lacks some of the flexibility - most notable is the lack of ability to "go back in time". Thus the current function only allows to show transitions from one state to the next and there is no going back to previous columns.

Generate some data
==================


We will start by simulating some data similar to my article. Each observation has a sex and a Charnley class (A, B, or C). The transition is then dependent on both the sex and the Charnley class.

```{r}
library(magrittr)
set.seed(1)
n <- 100
data <- 
  data.frame(
    Sex = sample(c("Male", "Female"),
                 size = n,
                 replace = TRUE,
                 prob = c(.4, .6)),
    Charnley_class = sample(c("A", "B", "C"), 
                            size = n, 
                            replace = TRUE))
getProbs <- function(Chrnl_name){
  prob <- data.frame(
    A = 1/3 +
      (data$Sex == "Male") * .25 +
      (data$Sex != "Male") * -.25 +
      (data[[Chrnl_name]] %in% "B") * -.5 +
      (data[[Chrnl_name]] %in% "C") * -2 ,
    B = 1/3 +
      (data$Sex == "Male") * .1 + 
      (data$Sex != "Male") * -.05 +
      (data[[Chrnl_name]] == "C") * -2,
    C = 1/3 +
      (data$Sex == "Male") * -.25 +
      (data$Sex != "Male") * .25)
  
  # Remove negative probabilities
  t(apply(prob, 1, function(x) {
    if (any(x < 0)){
      x <- x - min(x) + .05
      }
    x
    }))
}
  
Ch_classes <- c("Charnley_class")
Ch_classes %<>% c(sprintf("%s_%dyr", Ch_classes, c(1,2,6)))
for (i in 1:length(Ch_classes)){
  if (i == 1)
    next;

  data[[Ch_classes[i]]] <- 
    apply(getProbs(Ch_classes[i-1]), 1, function(p)
      sample(c("A", "B", "C"), 
             size = 1, 
             prob = p)) %>%
    factor(levels = c("A", "B", "C"))
}
```

Basic use
=========
```{r, echo=FALSE}
knitr::opts_chunk$set(message = FALSE, 
                      warnings = FALSE)
knitr::opts_chunk$set(fig.height = 5, fig.width = 5)
```

The most simple use is to just supply the output from the `table()` call:

```{r}
library(Gmisc)
transitions <- table(data$Charnley_class, data$Charnley_class_1yr) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"))
transitions$render()
```

Adding customizations is rather straightforward by setting the different field arguments:

```{r}
transitions <- table(data$Charnley_class, data$Charnley_class_1yr) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"),
                                # Control the box text via grid::gpar()
                                txt_gpar = grid::gpar(fontfamily = "serif"))
transitions$title <- "Charnley class in relation to THR"
transitions$arrow_type <- "simple"
transitions$box_label_pos <- "bottom"
transitions$render()
```

## Adding a third dimension


Underlying proportions can be visualized by splitting the box colors and blending the colors in the gradient arrows. The color blend is explained in the colorbar element at the bottom:

```{r color_transition}
transitions <- table(data$Charnley_class, data$Charnley_class_1yr, data$Sex) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"))
transitions$title <- "Charnley class in relation to THR"
transitions$clr_bar <- "bottom"
transitions$render()
```

Multiple transitions
====================

The major advantage with the **Transition**-class is that it allows for transitions at multiple time points. We do this by calling the **addTransitions** function as shown below:

```{r multiple_transitions, fig.height = 5, fig.width = 7}
transitions <- table(data$Charnley_class, data$Charnley_class_1yr, data$Sex) %>%
  getRefClass("Transition")$new(label = c("Before surgery", "1 year after"))
transitions$title <- "Charnley class in relation to THR"
transitions$arrow_type <- "simple"
table(data$Charnley_class_1yr, data$Charnley_class_2yr, data$Sex) %>%
  transitions$addTransitions(label = "2 years after")
library(grid)
transitions$max_lwd <- unit(.05, "npc")
transitions$render()
```

---
title: "Descriptives statistics for Table 1"
author: "Max Gordon"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
    css: custom.css
vignette: >
  %\VignetteIndexEntry{Descriptives statistics for Table 1}
  %\usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

The purpose of the first table in a medical paper is most often to describe your population. In an RCT the table frequently compares the baseline characteristics between the randomized groups, while an observational study will often compare exposed with unexposed. In this vignette I will show how I use the functions to quickly generate a descriptive table.

```{r, include=FALSE}
library(magrittr)
library(dplyr)
library(Gmisc)

# Style all the table output using htmlTable's theme handler
htmlTable::setHtmlTableTheme(css.rgroup = "")
set.seed(123)
```

We will use the mtcars dataset and compare the groups with automatic transmission to those without. The units and labels are built upon the logic in the **Hmisc** package that allow us to specify attributes on columns. *Note* that this labeling is not needed, it just makes stuff nicer.

```{r}
library(Gmisc)
data("mtcars")
mtcars <- mtcars %>% 
  mutate(am = factor(am, levels = 0:1, labels = c("Automatic", "Manual")),
         gear = factor(gear),
         # Make up some data for making it slightly more interesting
         col = factor(sample(c("red", "black", "silver"),
                             size = NROW(mtcars), 
                             replace = TRUE))) %>% 
  set_column_labels(mpg = "Gas",
                    wt = "Weight",
                    am = "Transmission",
                    gear = "Gears",
                    col = "Car color") %>% 
  set_column_units(mpg = "Miles/(US) gallon",
                   wt = "10<sup>3</sup> lbs")
```

The basics of `getDescriptionStatsBy`
=====================================

The function `getDescriptionStatsBy` is a simple way to do basic descriptive statistics. Mandatory named column is *by*:

```{r}
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am)
```

If we prefer median we can simply specify the statistic used with continuous variables:

```{r}
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am,
                        continuous_fn = describeMedian)
```

Integration with `htmlTable`
----------------------------

Key to having good descriptive statistics is to be able to output it into a table. I usually rely on `htmlTable` for all my table requirements as it has a nice set of
advanced options that allow us to get publication ready tables that can simply be copy-pasted into our paper. *Note* that we here use html code &dagger; that we then explain
in the footer. If we specify a name to the parameters like this we override the labels previously set.


```{r}
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        `Weight&dagger;` = wt,
                        am,
                        gear,
                        col,
                        by = am) %>% 
  htmlTable(caption  = "Basic descriptive statistics from the mtcars dataset",
            tfoot = "&dagger; The weight is in 10<sup>3</sup> kg")
```

Extra everything
----------------

There is a large set of options for `getDescriptionStatsBy`, here is an example with some of them an some extra styling.

```{r}
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        `Weight&dagger;` = wt,
                        am,
                        gear,
                        col,
                        by = am,
                        digits = 0,
                        add_total_col = TRUE,
                        use_units = "name") %>% 
  addHtmlTableStyle(pos.caption = "bottom") %>% 
  htmlTable(caption  = "Basic descriptive statistics from the mtcars dataset",
            tfoot = "&dagger; The weight is in 10<sup>3</sup> kg")
```

P-values
========

Even though p-values are discouraged in the Table 1, they are [not uncommon](https://doi.org/10.1177/1741826711421688). I have therefore added basic statistics consisting of tests that default to Fisher's exact test for proportions and Wilcoxon rank sum test for continuous values. 

```{r, warning=FALSE}
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am,
                        continuous_fn = describeMedian,
                        digits = 0,
                        header_count = TRUE,
                        statistics = TRUE) %>% 
  htmlTable(caption  = "Basic descriptive statistics from the mtcars dataset")
```

Custom p-values
---------------

By popular demand I've expanded with the option of having custom p-values. All you need to do is to provide a function that takes two values and exports a single p-value. There are several prepared functions that you can use or use as a template for your own p-value function. They all start with `getPval..`, e.g. `getPvalKruskal`. You can either provide a single function or you can set the defaults depending on the variable type:


```{r, warning=FALSE}
mtcars %>% 
  getDescriptionStatsBy(mpg, 
                        wt,
                        am,
                        gear,
                        col,
                        by = am,
                        continuous_fn = describeMedian,
                        digits = 0,
                        header_count = TRUE,
                        statistics = list(continuous = getPvalChiSq, 
                                          factor = getPvalChiSq, 
                                          proportion = getPvalFisher)) %>% 
  addHtmlTableStyle(pos.caption = "bottom") %>% 
  htmlTable(caption  = "P-values generated from a custom set of values")
```

Using `mergeDesc`
========================

Prior to Gmisc v3.0 `mergeDesc` was the best way to quickly assemble a "Table 1":

```{r}
getTable1Stats <- function(x, digits = 0, ...){
  getDescriptionStatsBy(x = x, 
                        by = mtcars$am,
                        digits = digits,
                        continuous_fn = describeMedian,
                        header_count = TRUE,
                        ...)
  
}

t1 <- list()
t1[["Gas"]] <-
  getTable1Stats(mtcars$mpg)
  
t1[["Weight&dagger;"]] <-
  getTable1Stats(mtcars$wt)

t1[["Color"]] <- 
  getTable1Stats(mtcars$col)

# If we want to use the labels set in the beginning
# we add an element without a name
t1 <- c(t1,
        list(getTable1Stats(mtcars$gear)))

mergeDesc(t1,
          htmlTable_args = list(caption  = "Basic descriptive statistics from the mtcars dataset",
                                tfoot = "&dagger; The weight is in 10<sup>3</sup> kg"))
```

---
title: "STROBE-Statement checklist"
lang: en
output:
  Gmisc::docx_document:
    fig_caption: true
---

```{r "Load libraries", echo=FALSE, message=FALSE, warning=FALSE}
library(Gmisc)
```

```{r "Load data"}

```

```{r "Munge data"}

```

# Abstract
(a) Indicate the study’s design with a commonly used term in the title or the abstract
(b) Provide in the abstract an informative and balanced summary of what was done and what was found

# Introduction

## Background/rationale
Explain the scientific background and rationale for the investigation being reported

## Objectives	
State specific objectives, including any prespecified hypotheses

# Methods

##Study design
Present key elements of study design early in the paper

## Setting
Describe the setting, locations, and relevant dates, including periods of recruitment, exposure, follow-up, and data collection

## Participants	
(a) *Cohort study*—Give the eligibility criteria, and the sources and methods of selection of participants. Describe methods of follow-up  
*Case-control study*—Give the eligibility criteria, and the sources and methods of case ascertainment and control selection. Give the rationale for the choice of cases and controls  
*Cross-sectional study*—Give the eligibility criteria, and the sources and methods of selection of participants  
		
(b) *Cohort study*—For matched studies, give matching criteria and number of exposed and unexposed  
*Case-control study*—For matched studies, give matching criteria and the number of controls per case  

## Variables
Clearly define all outcomes, exposures, predictors, potential confounders, and effect modifiers. Give diagnostic criteria, if applicable

## Data sources/ measurement
For each variable of interest, give sources of data and details of methods of assessment (measurement). Describe comparability of assessment methods if there is more than one group

## Bias
Describe any efforts to address potential sources of bias

## Study size
Explain how the study size was arrived at

## Quantitative variables
Explain how quantitative variables were handled in the analyses. If applicable, describe which groupings were chosen and why

## Statistical methods
(a) Describe all statistical methods, including those used to control for confounding  
(b) Describe any methods used to examine subgroups and interactions  
(c) Explain how missing data were addressed  
(d) *Cohort study*—If applicable, explain how loss to follow-up was addressed  
*Case-control study*—If applicable, explain how matching of cases and controls was addressed  
*Cross-sectional study*—If applicable, describe analytical methods taking account of sampling strategy  
(e) Describe any sensitivity analyses

# Results

## Participants  
(a) Report numbers of individuals at each stage of study—eg numbers potentially eligible, examined for eligibility, confirmed eligible, included in the study, completing follow-up, and analysed  
(b) Give reasons for non-participation at each stage  
(c) Consider use of a flow diagram  

## Descriptive data	
(a) Give characteristics of study participants (eg demographic, clinical, social) and information on exposures and potential confounders  
(b) Indicate number of participants with missing data for each variable of interest  
(c) Cohort study—Summarise follow-up time (eg, average and total amount)  

## Outcome data
*Cohort study*—Report numbers of outcome events or summary measures over time  
*Case-control study*—Report numbers in each exposure category, or summary measures of exposure  
*Cross-sectional study*—Report numbers of outcome events or summary measures

## Main results	
(a) Give unadjusted estimates and, if applicable, confounder-adjusted estimates and their precision (eg, 95% confidence interval). Make clear which confounders were adjusted for and why they were included  
(b) Report category boundaries when continuous variables were categorized  
(c) If relevant, consider translating estimates of relative risk into absolute risk for a meaningful time period  

## Other analyses	
Report other analyses done—eg analyses of subgroups and interactions, and sensitivity analyses

# Discussion

## Key results
Summarise key results with reference to study objectives

## Limitations
Discuss limitations of the study, taking into account sources of potential bias or imprecision. Discuss both direction and magnitude of any potential bias

## Interpretation
Give a cautious overall interpretation of results considering objectives, limitations, multiplicity of analyses, results from similar studies, and other relevant evidence

## Generalisability
Discuss the generalisability (external validity) of the study results

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prAddTotalDescColumn.R
\name{prAddTotalDescColumn}
\alias{prAddTotalDescColumn}
\title{Add a total column to the results}
\usage{
prAddTotalDescColumn(
  results,
  x,
  by,
  numbers_first,
  total_col_show_perc,
  show_all_values,
  useNA,
  useNA.digits,
  html,
  digits,
  continuous_fn,
  factor_fn,
  prop_fn,
  percentage_sign,
  default_ref,
  header_count = NULL,
  add_total_col
)
}
\arguments{
\item{results}{The results that we want to add the column to}

\item{x}{If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
If it is a single variable it will be the core value that want the statistics for.
In the print this is equivalent to the output of this function.}

\item{by}{The variable that you want to split into different columns}

\item{numbers_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().}

\item{total_col_show_perc}{This is by default true but if
requested the percentages are suppressed as this sometimes may be confusing.}

\item{show_all_values}{Show all values in proportions. For factors with only two values
it is most sane to only show one option as the other one will just be a complement
to the first, i.e. we want to convey a proportion. For instance sex - if you know
gender then automatically you know the distribution of the other sex as it's 100 \% - other \%.
To choose which one you want to show then set the \code{default_ref} parameter.}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{digits}{The number of decimals used}

\item{continuous_fn}{The method to describe continuous variables. The
default is \code{\link{describeMean}}.}

\item{factor_fn}{The method used to describe factors, see \code{\link{describeFactors}}.}

\item{prop_fn}{The method used to describe proportions, see \code{\link{describeProp}}.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable.}

\item{default_ref}{The default reference when dealing with proportions. When using
`dplyr` syntax (`tidyselect`) you can specify a named vector/list for each column name.}

\item{header_count}{Set to \code{TRUE} if you want to add a header count,
e.g. Smoking; No. 25 observations, where there is a new line after the
factor name. If you want a different text for the second line you can
specifically use the \code{\link[base]{sprintf}} formatting, e.g. "No. \%s patients".}

\item{add_total_col}{This adds a total column to the resulting table.
You can also specify if you want the total column "first" or "last"
in the column order.}
}
\value{
results with added column
}
\description{
Add a total column to the results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\docType{package}
\name{Gmisc-deprecated}
\alias{Gmisc-deprecated}
\alias{forestplot2}
\title{Deprecated function(s) in \pkg{Gmisc}}
\usage{
forestplot2(...)
}
\arguments{
\item{...}{Parameters to be passed to the new version of the function}
}
\description{
The \code{forestplot2} has moved to \pkg{forestplot} package and changed name to \code{\link[forestplot]{forestplot}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prFixDescRownames.R
\name{prFixDescRownames}
\alias{prFixDescRownames}
\title{Fix rownames for descriptive results}
\usage{
prFixDescRownames(results, t, name)
}
\arguments{
\item{results}{A matrix with the results}

\item{t}{The [base::by()] output}

\item{name}{Name if row names are missing or the results is a single row}
}
\value{
The results with fixed names
}
\description{
Helper for [getDescriptionStatsBy] that fixes row names
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_labelConnector.R
\name{labelConnector}
\alias{labelConnector}
\title{Label connector grobs}
\usage{
labelConnector(
  con,
  labels,
  x_offset = unit(0, "mm"),
  y_offset = unit(0, "mm"),
  gp = gpar(cex = 0.9),
  bg_gp = gpar(fill = "white", col = NA),
  pad = unit(2, "mm")
)
}
\arguments{
\item{con}{A list of connector grobs produced by `connectGrob()` (typically
a `connect_boxes_list`).}

\item{labels}{A character vector of labels (recycled if necessary).}

\item{x_offset, y_offset}{`grid::unit` offsets added to the label position.}

\item{gp}{A `grid::gpar` object controlling text appearance.}

\item{bg_gp}{A `grid::gpar` object controlling label background appearance.}

\item{pad}{A `grid::unit` padding to add around text inside the background box.}
}
\value{
A `Gmisc_connector_label` S3 object (invisible). Use `print()` to draw the labels.
}
\description{
Place text labels at the midpoint of connector grobs returned by
`connectGrob()` (useful for multi-target connectors where `label=` isn't
available). By default the function draws the labels; set `draw = FALSE` to
only return the `textGrob` objects for programmatic use.
}
\examples{
# Attach labels to a previously-created connector and draw them
a <- boxDiamondGrob("D")
b <- boxEllipseGrob("L")
c <- boxServerGrob("S")
boxes <- list(decision = a, outcomes = list(b, c)) |>
  spreadHorizontal(
    from = grid::unit(.1, "npc"),
    to = grid::unit(.9, "npc"),
    subelement = "outcomes"
  ) |>
  spreadVertical()

connectGrob(boxes$decision, boxes$outcomes, type = "N") |>
  labelConnector(c("Local", "Server"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/figCaptionNo.R
\name{figCapNoLast}
\alias{figCapNoLast}
\title{Gets the last figure caption number}
\usage{
figCapNoLast(roman = getOption("fig_caption_no_roman", FALSE))
}
\arguments{
\item{roman}{Whether or not to use roman numbers instead
of Arabic. Can also be set through \code{options(fig_caption_no_roman = TRUE)}}
}
\description{
The function relies on \code{options("fig_caption_no")}
in order to keep track of the last number.
}
\examples{
org_opts <- options(fig_caption_no = 1)
figCapNoLast()
options(org_opts)
}
\seealso{
Other figure caption functions:
\code{\link[=figCapNo]{figCapNo()}},
\code{\link[=figCapNoNext]{figCapNoNext()}}
}
\concept{figure caption functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot.R
\name{transitionPlot}
\alias{transitionPlot}
\title{A transition plot}
\usage{
transitionPlot(
  transition_flow,
  type_of_arrow = c("grid", "simple", "gradient"),
  box_txt = rownames(transition_flow),
  tot_spacing = 0.2,
  box_width = 1/4,
  fill_start_box = "darkgreen",
  txt_start_clr = "white",
  fill_end_box = fill_start_box,
  txt_end_clr = txt_start_clr,
  cex = 2,
  min_lwd = if (type_of_arrow == "grid") 1 else unit(0.1, "mm"),
  max_lwd = if (type_of_arrow == "grid") 6 else unit(5, "mm"),
  lwd_prop_total = TRUE,
  arrow_clr = "#000000",
  abs_arrow_width = FALSE,
  overlap_bg_clr = "#FFFFFF",
  overlap_order = 1:nrow(transition_flow),
  overlap_add_width = if (type_of_arrow == "grid") 1.5 else unit(1, "mm"),
  box_prop,
  mar = unit(rep(3, times = 4), "mm"),
  main = NULL,
  box_label = NULL,
  box_label_pos = "top",
  box_label_cex = cex,
  color_bar = TRUE,
  color_bar_cex = cex * 0.33,
  color_bar_labels,
  color_bar_subspace = NULL,
  new_page = FALSE
)
}
\arguments{
\item{transition_flow}{This should be a matrix with the size of the transitions.
The unit for each cell should be number of observations, row/column-proportions
will show incorrect sizes. The matrix needs to be square. The best way to generate
this matrix is probably just do a \code{table(starting_state, end_state)}. The rows
represent the starting positions, while the columns the end positions. I.e. the first
rows third column is the number of observations that go from the first class to the
third class.}

\item{type_of_arrow}{The types of arrow may be grid, simple, or gradient. Simple grid
arrows are the \code{\link[grid:grid.bezier]{bezierGrob}} arrows (not that pretty),
simple is the \code{\link{bezierArrowSmpl}} that I've created to get a more exact
control of the arrow position and width, while gradient
corresponds to \code{\link{bezierArrowGradient}}
allowing the arrow to have a fill color that slowly turns into the color of the arrow.}

\item{box_txt}{The text to appear inside of the boxes. If you need line breaks
then you need to manually add a \\n inside the string.}

\item{tot_spacing}{The proportion of the vertical space that is to be left
empty. It is then split evenly between the boxes.}

\item{box_width}{The width of the box. By default the box is one fourth of
the plot width.}

\item{fill_start_box}{The fill color of the start boxes. This can either
be a single value or a vector if you desire different colors for each
box. If you specify box_prop then this has to be a 2 column matrix.}

\item{txt_start_clr}{The text color of the start boxes. This can either
be a single value or a vector if you desire different colors for each
box. If you specify box_prop then this has to be a 2 column matrix.}

\item{fill_end_box}{The fill color of the end boxes. This can either
be a single value or a vector if you desire different colors for each
box. If you specify box_prop then this has to be a 2 column matrix.}

\item{txt_end_clr}{The text color of the end boxes. This can either
be a single value or a vector if you desire different colors for each
box. If you specify box_prop then this has to be a 2 column matrix.}

\item{cex}{The cex \code{\link{gpar}} of the text}

\item{min_lwd}{The minimum width of the line that we want to illustrate the
tranisition with.}

\item{max_lwd}{The maximum width of the line that we want to illustrate the
tranisition with.}

\item{lwd_prop_total}{The width of the lines may be proportional to either the
other flows from that box, or they may be related to all flows. This is a boolean
parameter that is set to true by default, i.e. relating to all flows.}

\item{arrow_clr}{The color of the arrows. Usually black, can be a vector indicating each arrow
from first to last arrow (counting from the top). If the vector is of the same length as the
boxes then all box arrows will have the same color (that is all the arrows stemming from the
left boxes)}

\item{abs_arrow_width}{The width can either be absolute, i.e. each arrow headed for a box
has the exact same width. The alternative is that the width is related to the line width.}

\item{overlap_bg_clr}{In order to enhance the 3D perspective and to make it easier
to follow arrows the arrows have a background color to separate them from those underneath.}

\item{overlap_order}{The order from first->last for the lines. This means that the last
line will be on top while the first one will appear at the bottom. This should be provided
as a vector.}

\item{overlap_add_width}{The width of the white cross-over line. You can specify this as a scalar
multiplication of the current line width. In case of non-grid arrows then you can also have this
as a unit which is recommended as it looks better. If the scalar is < 1 then the overlap is ignored.}

\item{box_prop}{If you want the boxes to have proportions indicating some other factors then input
a matrix with quantiles for the proportions. Note the size must be \code{nrow(transition_flow) x 2}.}

\item{mar}{A numerical vector of the form c(bottom, left, top, right) of the type \code{unit()}}

\item{main}{The title of the plot if any, default \code{NULL}}

\item{box_label}{A vector of length 2 if you want to label each box column}

\item{box_label_pos}{The position of the label, either \code{'top'} or \code{'bottom'}}

\item{box_label_cex}{The cex of the label, defaults to the default cex}

\item{color_bar}{If you have proportions inside the transition_flow variable
then the color_bar will automatically appear at the bottom unless you set
this to \code{FALSE}}

\item{color_bar_cex}{The size of the tick labels for the color bar}

\item{color_bar_labels}{The labels of the two proportions that make up the color bar.
Defaults to the labels of the third dimension for the \code{transition_flow}
argument.}

\item{color_bar_subspace}{If there is little or no difference
at the low/high proportions of the spectrum then it
can be of interest to focus the color change to the center
leaving the tails constant}

\item{new_page}{If you want the plot to appear on a new blank page then set this to \code{TRUE}, by
default it is \code{FALSE}.}
}
\value{
void
}
\description{
This plot's purpose is to illustrate how states change before and
after. In my research I use it before surgery and after surgery
but it can be used in any situation where you have a change from
one state to another
}
\examples{
# This example does not run since it
# takes a little while to assemble the
# arrows and RMD Check complains that this
# is more than allowed for
library(grid)
par_org <- par(ask = TRUE)
# Settings
no_boxes <- 3
# Generate test setting
transition_matrix <- matrix(NA, nrow = no_boxes, ncol = no_boxes)
transition_matrix[1, ] <- 200 * c(.5, .25, .25)
transition_matrix[2, ] <- 540 * c(.75, .10, .15)
transition_matrix[3, ] <- 340 * c(0, .2, .80)

grid::grid.newpage()
transitionPlot(transition_matrix,
  box_txt = c("First", "Second", "Third"),
  type_of_arrow = "simple",
  min_lwd = unit(1, "mm"),
  max_lwd = unit(6, "mm"),
  overlap_add_width = unit(1, "mm")
)


# Setup proportions
box_prop <- cbind(c(1, 0, 0.5), c(.52, .2, .8))
# From the Set2 Colorbrewer
start_box_clr <- c("#8DA0CB", "#FC8D62")
# Darken the colors slightly
end_box_clr <- c(
  colorRampPalette(c(start_box_clr[1], "#000000"))(10)[2],
  colorRampPalette(c(start_box_clr[2], "#000000"))(10)[2]
)
# Create a new grid
grid::grid.newpage()
transitionPlot(transition_matrix,
  box_prop = box_prop,
  fill_start_box = start_box_clr, fill_end_box = end_box_clr,
  txt_start_clr = c("#FFFFFF", "#000000"), txt_end_clr = c("#FFFFFF", "#000000"),
  box_txt = c("First", "Second", "Third"),
  type_of_arrow = "gradient",
  min_lwd = unit(1, "mm"),
  max_lwd = unit(10, "mm"),
  overlap_add_width = unit(1, "mm")
)
par(par_org)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_s3_spread.R, R/boxGrobs_spread.R
\name{spread}
\alias{spread}
\alias{spread.default}
\alias{spread.list}
\alias{spread.Gmisc_list_of_boxes}
\alias{spreadVertical}
\alias{spreadHorizontal}
\title{Spread boxes (S3)}
\usage{
spread(x, ...)

\method{spread}{default}(x, ...)

\method{spread}{list}(x, ...)

\method{spread}{Gmisc_list_of_boxes}(x, ..., axis = c("y", "x", "vertical", "horizontal"))

spreadVertical(
  ...,
  from = NULL,
  to = NULL,
  margin = unit(0, "npc"),
  type = c("between", "center"),
  subelement = NULL
)

spreadHorizontal(
  ...,
  from = NULL,
  to = NULL,
  margin = unit(0, "npc"),
  type = c("between", "center"),
  subelement = NULL
)
}
\arguments{
\item{x}{A `list` of boxes.}

\item{...}{A set of boxes to spread. Can also be a \code{list} of boxes.}

\item{axis}{Orientation: `"y"`/`"vertical"` or `"x"`/`"horizontal"`.}

\item{from}{Starting point of the span. Can be a box, a coordinate/unit, or a
numeric value interpreted as \code{npc}. If only \code{from} is provided, \code{to} defaults
to \verb{1 npc}.}

\item{to}{Ending point of the span. Can be a box, a coordinate/unit, or a
numeric value interpreted as \code{npc}. If only \code{to} is provided, \code{from} defaults
to \verb{0 npc}.}

\item{margin}{Optional padding applied at both ends of the span. Can be a
\code{\link[grid:unit]{grid::unit}} or a numeric value interpreted as \code{npc}. Applied whether the
span comes from \code{from} / \code{to} or the viewport.}

\item{type}{If \code{between}, the space \emph{between} boxes is identical. If \code{center},
the centers of the boxes are equally distributed across the span.}

\item{subelement}{If a \code{list} of boxes is provided, this parameter can be used
to target a specific element (by name or index), or a deep path into nested
lists (e.g., \code{c("detail", 1)}), for the spreading operation. You can also
provide multiple targets by giving a list of paths (e.g., \code{list(c("detail", 1), c("followup", 1))}).
The function will return the original list with the targeted element(s)
replaced by their spread version(s).}
}
\value{
The list of boxes with updated positions (class `Gmisc_list_of_boxes`).

A \code{list} with the boxes that have been spread.
}
\description{
Spread a list of boxes vertically or horizontally. Designed for piping (`|>`).

Spreads a set of \code{\link{boxGrob}}/\code{\link{boxPropGrob}} in either horizontal or vertical
direction within a given span.
}
\details{
The span can be defined explicitly using \code{from} / \code{to}, or implicitly by the
current viewport. Numeric values are interpreted as proportions of the viewport
(\code{npc} units).
}
\examples{
library(grid)
grid.newpage()

# Create a set of boxes
start <- boxGrob("Top", x = .5, y = .8)
end <- boxGrob("Bottom", x = .5, y = .2)
side <- boxPropGrob("Side", "Left", "Right", prop = .3, x = .2, y = .8)
exclude <- boxGrob("Exclude:\n - Too sick\n - Prev. surgery", x = .8, y = .5, just = "left")

# We can chain the spread operations and print the result
spreadVertical(
  start = start,
  middle = list(side, exclude),
  end = end
) |>
  spreadHorizontal(subelement = "middle", from = 0.2, to = 0.8)

# Use device-level paging in interactive sessions so users can inspect the first plot
# This will prompt before creating the next page; we restore the previous setting afterwards
if (interactive()) {
  old_ask <- grDevices::devAskNewPage(TRUE)
  on.exit(grDevices::devAskNewPage(old_ask), add = TRUE)
  grid.newpage() # will prompt on interactive devices
} else {
  # non-interactive: just start a fresh page for the next example
  grid.newpage()
}

# Example: spread a nested subelement by deep path and print sequentially
list(grp = list(middle = list(side, exclude))) |>
  spreadHorizontal(subelement = c("grp", "middle"), from = 0.2, to = 0.8)
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_spread_npc_norm_helpers.R
\name{prGetBoxAxisDefaults}
\alias{prGetBoxAxisDefaults}
\title{Get default from/to values for an axis
Get default from/to values for an axis}
\usage{
prGetBoxAxisDefaults(axis = c("x", "y"))
}
\arguments{
\item{axis}{Axis, either "x" or "y"}
}
\value{
A list with `from` and `to` values as `unit` objects
}
\description{
When axis is "x", the default is from 0 to 1 npc, i.e. left to right.
When axis is "y", the default is from 1 to 0 npc, i.e. top to bottom.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/private_functions.R
\name{prPushMarginViewport}
\alias{prPushMarginViewport}
\title{Pushes viewport with margins}
\usage{
prPushMarginViewport(bottom, left, top, right, name = "margin")
}
\arguments{
\item{bottom}{The margin object, either in npc or a \code{\link[grid]{unit}} object,
or a vector of 4 if you want the same margins allover.}

\item{left}{The margin object, either in npc or a \code{\link[grid]{unit}} object.}

\item{top}{The margin object, either in npc or a \code{\link[grid]{unit}} object}

\item{right}{The margin object, either in npc or a \code{\link[grid]{unit}} object}

\item{name}{The name of the last viewport}
}
\value{
\code{2} - the number of pushed viewports
}
\description{
A \code{\link[grid]{grid.layout}} object is used to
generate the margins. A second viewport selecting the
mid-row/col is used to create the effect of margins
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flowchart.R
\name{flowchart}
\alias{flowchart}
\title{Create a flowchart object}
\usage{
flowchart(..., default_box_fn = boxGrob)
}
\arguments{
\item{...}{Either a set of [`boxGrob`] objects, text strings, or lists of these.
Text strings will be automatically wrapped in [`boxGrob`] (or `default_box_fn`).}

\item{default_box_fn}{The function to use for converting text/content into a box
(default: [`boxGrob`]).}
}
\value{
A `list` of class `Gmisc_list_of_boxes`.
}
\description{
This function initializes a flowchart object, which is essentially a list of boxes
with the class `Gmisc_list_of_boxes`. It serves as the starting point for the
S3 pipeline, allowing for fluent chaining of [`align`], [`spread`], [`move`],
and [`connect`] operations.
}
\examples{
library(grid)
grid.newpage()

# Create a flowchart with auto-conversion of text
fc <- flowchart(
  start = "Start",
  process = "Process",
  end = "End"
)

# Use the pipeline
fc |>
  align(axis = "y") |>
  connect("start", "process", type = "vertical") |>
  connect("process", "end", type = "vertical") |>
  print()
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezierArrowSmpl.R
\name{bezierArrowSmpl}
\alias{bezierArrowSmpl}
\title{A simple bezier arrow}
\usage{
bezierArrowSmpl(
  x = c(0.2, 0.7, 0.3, 0.9),
  y = c(0.2, 0.2, 0.9, 0.9),
  width = 0.05,
  clr = "#000000",
  default.units = "npc",
  arrow = list(),
  rez = 200,
  align_2_axis = TRUE,
  name = NULL,
  rm_intersect = 3L,
  gp = gpar(),
  vp = NULL
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of
    spline control points.}

\item{y}{A numeric vector or unit object specifying y-locations of
    spline control points.}

\item{width}{The width of the arrow, either a numeric single number or a unit. \strong{Note:}
The arrow does not rely on \code{lwd} but on actual width.}

\item{clr}{The color of the arrow.}

\item{default.units}{A string indicating the default units to use
    if \code{x} or \code{y}
    are only given as numeric vectors.}

\item{arrow}{This is a list with all the \strong{base} (width) and the desired
\strong{length} for the arrow. \strong{Note:} This differs from the original
\code{\link{bezierGrob}} function.}

\item{rez}{The resolution of the arrow. This specifies how many points to retrieve from the
\code{\link{gnrlBezierPoints}} function. Defaults to 200.}

\item{align_2_axis}{Indicates if the arrow should be vertically/horizontally
aligned. This is useful for instance if the arrow attaches to a box.}

\item{name}{A character identifier.}

\item{rm_intersect}{Set to 0 if you want to skip intersection removal, 1 only to remove left or
 2 to only remove right. See details for why.

 @section Remove intersections:

 When the line is wide and the arrow has a narrow curve there may appear an empty triangle due
 to polygon cancellation (two polygons within the same are cancel out). This behaviour may be
 ugly and the function therefor tries to remove these.

 \emph{Note:} it is expensive to check if there are the lineas may intersect at one point,
 remove those unexpected, and then adjust the line to the new situation so that the
 top and bottom lines match. It can also cause some unexpected behaviour why you may want to
 remove this feature if the arrow behaves erratically.}

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
\code{grid::grob} A grob of the class polygonGrob with attributes that
 correspond to the bezier points.
}
\description{
This is an alternative to the grid packages \code{\link[grid:grid.bezier]{bezierGrob}}
with the advantage that it allows you to draw an arrow with a specific
unit width. Note, it has only a end-arrow at this point.
}
\examples{
library(grid)
grid::grid.newpage()
arrowGrob <- bezierArrowSmpl(
  x = c(.1, .3, .6, .9),
  y = c(0.2, 0.2, 0.9, 0.9)
)
grid.draw(arrowGrob)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/private_functions.R
\name{prGridPlotTitle}
\alias{prGridPlotTitle}
\title{Adds a title to the plot}
\usage{
prGridPlotTitle(
  title,
  base_cex,
  cex_mult = 1.2,
  fontface = "bold",
  space_below = NULL
)
}
\arguments{
\item{title}{The title as accepted by \code{\link[grid:grid.text]{textGrob}}}

\item{base_cex}{The base cex used for the plot}

\item{cex_mult}{The multiplier of the base - i.e. the increase of the
text size for the title as compared to the general}

\item{fontface}{The type of fontface}

\item{space_below}{The space below, defaults to 1/5 of the title height}
}
\value{
\code{NULL} The function does not return a value
}
\description{
Adds the title and generates a new
main viewport below the title
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionClass_helpers.R
\name{prTcValidateAndPrepClr}
\alias{prTcValidateAndPrepClr}
\title{Checks and prepares colors}
\usage{
prTcValidateAndPrepClr(value, no_cols, no_rows, is3D)
}
\arguments{
\item{value}{The color values}

\item{no_cols}{The no. of columns in the transition matrix}

\item{no_rows}{All the row counts in the transition plot}

\item{is3D}{Whether the transition matrix has 3 dimensions}
}
\value{
\code{matrix} Returns a matrix of the same size as the
 transition matrix
}
\description{
The colors need to match the size of the transition matrix
and therefore we need to have this input check. The output is
always a matrix of the exact same dimensions as the transition
matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_labelConnector.R
\name{setConnectorLabels}
\alias{setConnectorLabels}
\title{Attach labels to a connector object}
\usage{
setConnectorLabels(
  con,
  labels,
  gp = gpar(cex = 0.9),
  bg_gp = gpar(fill = "white", col = NA),
  x_offset = unit(0, "mm"),
  y_offset = unit(0, "mm")
)
}
\arguments{
\item{con}{A connector (a `connect_boxes` grob or `connect_boxes_list`).}

\item{labels}{Character vector of labels (recycled to match connector length).}

\item{gp}{A `gpar` for label text.}

\item{bg_gp}{A `gpar` for label background.}

\item{x_offset}{Unit offsets for label placement.}

\item{y_offset}{Unit offsets for label placement.}
}
\value{
The connector object with attributes set (invisibly).
}
\description{
Store labels and presentation options on a connector object so labels are
automatically drawn when the connector is printed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_boxGrob.R, R/boxGrobs_S3_fns.R
\name{boxGrob}
\alias{boxGrob}
\alias{print.box}
\alias{plot.box}
\alias{widthDetails.box}
\alias{heightDetails.box}
\title{Create a box with text}
\usage{
boxGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  txt_padding = getOption("boxGrobTxtPadding", default = unit(6 *
    ifelse(is.null(txt_gp$cex), 1, txt_gp$cex), "mm")),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  box_fn = roundrectGrob,
  name = NULL,
  badge_label = NULL,
  badge_position = "top",
  badge_gp = gpar(fill = "steelblue", col = NA),
  badge_txt_gp = gpar(col = "white", cex = 0.7)
)

\method{print}{box}(x, ...)

\method{plot}{box}(x, ...)

\method{widthDetails}{box}(x)

\method{heightDetails}{box}(x)
}
\arguments{
\item{label}{The label to print - should be a number, text or expression.}

\item{y}{The y position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.}

\item{x}{The x position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.}

\item{width}{The box automatically adapts the size but you can force by specifying the width}

\item{height}{The box automatically adapts the size but you can force by specifying the height}

\item{just}{The justification for the text: left, center or right.}

\item{bjust}{The justification for the box: left, center, right, top or bottom.
See the \code{just} option for the \code{\link[grid]{viewport}}}

\item{txt_gp}{The \code{\link[grid]{gpar}} style to apply to the text. Set \code{boxGrobTxt} option
if you want to customize all the boxes at once.}

\item{txt_padding}{Padding between text and box border. Can be provided as a
\code{\link[grid]{unit}} or numeric (interpreted as millimetres). You can set
a global default with \code{options(boxGrobTxtPadding = ...)}.}

\item{box_gp}{The \code{\link[grid]{gpar}} style to apply to the box function of `box_fn` below.}

\item{box_fn}{Function to create box for the text. Parameters of `x=0.5`, `y=0.5` and `box_gp` will
be passed to this function and return a \code{grob} object.}

\item{name}{a character identifier for the \code{grob}. Used to find the \code{grob} on the display
list and/or as a child of another grob.}

\item{badge_label}{Optional badge text to display at the top of the box.}

\item{badge_position}{The position of the badge: "top" (currently only "top" supported).}

\item{badge_gp}{The \code{\link[grid]{gpar}} style to apply to the badge background.}

\item{badge_txt_gp}{The \code{\link[grid]{gpar}} style to apply to the badge text.}

\item{...}{Passed to \code{\link[grid]{grid.draw}}}
}
\value{
A grob
}
\description{
Creates a \code{\link[grid:grid.grob]{grob}} box with text inside it.
}
\section{The \code{plot}/\code{print}}{


To output the \code{\link[grid:grid.grob]{grob}} objects to the plot either call \code{plot}
on the object or \code{print} it. Note that R automatically prints any object that is outputted
to the console. The function calls in turn the \code{\link[grid]{grid.draw}} function on the object.
}

\section{S3 from the \pkg{grid} package}{


Width and height functions address the \code{\link{coords}} attribute for the corresponding information.
The \code{\link[grid:widthDetails]{widthDetails}} and \code{\link[grid:widthDetails]{heightDetails}}
that provide information on an object.
}

\examples{
# Note: grid functions are explicitly namespaced in examples to avoid
# relying on attaching the grid package in R CMD check.
grid::grid.newpage()
boxGrob("My box")
}
\seealso{
The package provides several convenience shape helpers that can be
passed to `boxGrob(..., box_fn = ...)`: \code{boxDiamondGrob},
\code{boxEllipseGrob}, \code{boxRackGrob}, \code{boxServerGrob},
\code{boxDatabaseGrob}, \code{boxDocumentGrob}, \code{boxDocumentsGrob}, and
\code{boxTapeGrob}. For examples see the vignette: \code{vignette("Grid-based_flowcharts", package = "Gmisc")}.

Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{assert_unit}
\alias{assert_unit}
\alias{assert_label}
\alias{assert_just}
\title{Checks if unit or number 0-1}
\usage{
assert_unit(var, allow_missing = TRUE)

assert_label(var, allow_missing = TRUE)

assert_just(var, allow_missing = TRUE)
}
\arguments{
\item{var}{Variable 2 check}

\item{allow_missing}{Allow missing variables}
}
\value{
void

void

void
}
\description{
Checks if unit or number 0-1

Assert that it's a string or number

Assert valid color
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeDesc.R
\name{mergeDesc}
\alias{mergeDesc}
\title{Prepares a matrix for \code{htmlTable} from a list}
\usage{
mergeDesc(..., htmlTable_args = list())
}
\arguments{
\item{...}{One or more elements coming from \code{\link{getDescriptionStatsBy}}.
You can also provide pure output from the \code{\link{getDescriptionStatsBy}} function
and have the function merge this together with the \code{...} argument.
\emph{Note} that all elements must have the same \code{by} argument or you
will not be able to merge it into a list.}

\item{htmlTable_args}{Any arguments that should be passed to
\code{\link[htmlTable]{htmlTable}} function. The default is to remove
any css formatting for the \code{rgroup}.}
}
\value{
\code{matrix} Returns a matrix object of class descList
}
\description{
By putting all the output from the \code{\link{getDescriptionStatsBy}}
into a list, naming each element that we want in an \code{rgroup} we can
automatically merge everything and create an object ready for the
\code{\link[htmlTable]{htmlTable}}.
}
\section{The \code{rgroup} value}{


The value for the \code{rgroup} is by default the name of the list element. If you have
passed a list without a name for that particular element or if you have passed a
matrix it will look for a label set by the \pkg{Hmisc}\code{::\link[Hmisc]{label}} function.
For those elements that have only one row no \code{rgroup} is set, and the naming sequence
is the same as above but with an additional \code{\link[base:colnames]{rownames}} if the previous
two turn out empty. All this behavior is exemplified in the example.

The \code{rgroup} value can be overridden by simply specifying a custom \code{rgroup} when
calling the \code{\link[htmlTable]{htmlTable}} function.
}

\section{The \code{colnames} of the matrix}{


The function chooses the \code{\link[base]{colnames}} from the first element in
the \code{tlist}.
}

\examples{
library(magrittr)
library(dplyr)
library(htmlTable)

data(mtcars)
mtcars \%<>\%
  mutate(am = factor(am, levels = 0:1, labels = c("Automatic", "Manual")),
         vs = factor(vs, levels = 0:1, labels = c("V-shaped", "straight")),
         drat_prop = drat > median(drat),
         drat_prop = factor(drat_prop,
                            levels = c(FALSE, TRUE),
                            labels = c("High ratio", "Low ratio")),
         carb_prop = carb > 2,
         carb_prop = factor(carb_prop,
                            levels = c(FALSE, TRUE),
                            labels = c("&le; 2", "&gt; 2")),
         across(c(gear, carb, cyl), factor))

# A simple bare-bone example
mtcars \%>\%
  getDescriptionStatsBy(`Miles per gallon` = mpg,
                        Weight = wt,
                        `Carborators &le; 2` = carb_prop,
                        by = am) \%>\%
  htmlTable(caption  = "Basic continuous stats from the mtcars dataset")
invisible(readline(prompt = "Press [enter] to continue"))

# For labeling & units we use set_column_labels/set_column_unit that use
# the Hmisc package annotation functions
mtcars \%<>\%
  set_column_labels(am = "Transmission",
                    mpg = "Gas",
                    wt = "Weight",
                    gear = "Gears",
                    disp = "Displacement",
                    vs = "Engine type",
                    drat_prop = "Rear axel ratio",
                    carb_prop = "Carburetors") \%>\%
  set_column_units(mpg = "Miles/(US) gallon",
                   wt = "10<sup>3</sup> lbs",
                   disp = "cu.in.")

mtcars \%>\%
  getDescriptionStatsBy(mpg,
                        wt,
                        `Gear&dagger;` = gear,
                        drat_prop,
                        carb_prop,
                        vs,
                        by = am,
                        header_count = TRUE,
                        use_units = TRUE,
                        show_all_values = TRUE)  \%>\%
  addHtmlTableStyle(pos.caption = "bottom") \%>\%
  htmlTable(caption  = "Stats from the mtcars dataset",
            tfoot = "&dagger; Number of forward gears")
invisible(readline(prompt = "Press [enter] to continue"))

# Using the default parameter we can
mtcars \%>\%
  getDescriptionStatsBy(mpg,
                        wt,
                        `Gear&dagger;` = gear,
                        drat_prop,
                        carb_prop,
                        vs,
                        by = am,
                        header_count = TRUE,
                        use_units = TRUE,
                        default_ref = c(drat_prop = "Low ratio",
                                        carb_prop = "&gt; 2"))  \%>\%
  addHtmlTableStyle(pos.caption = "bottom") \%>\%
  htmlTable(caption  = "Stats from the mtcars dataset",
            tfoot = "&dagger; Number of forward gears")
invisible(readline(prompt = "Press [enter] to continue"))

# We can also use lists
tll <- list()
tll[["Gear (3 to 5)"]] <- getDescriptionStatsBy(mtcars$gear, mtcars$am)
tll <- c(tll,
         list(getDescriptionStatsBy(mtcars$disp, mtcars$am)))

mergeDesc(tll,
          htmlTable_args = list(caption  = "Factored variables")) \%>\%
  htmlTable::addHtmlTableStyle(css.rgroup = "")
invisible(readline(prompt = "Press [enter] to continue"))

tl_no_units <- list()
tl_no_units[["Gas (mile/gallons)"]] <-
  getDescriptionStatsBy(mtcars$mpg, mtcars$am,
                        header_count = TRUE)
tl_no_units[["Weight (10<sup>3</sup> kg)"]] <-
  getDescriptionStatsBy(mtcars$wt, mtcars$am,
                        header_count = TRUE)
mergeDesc(tl_no_units,
          tll) \%>\%
  htmlTable::addHtmlTableStyle(css.rgroup = "")
invisible(readline(prompt = "Press [enter] to continue"))

# Other settings
mtcars$mpg[sample(1:NROW(mtcars), size = 5)] <- NA
getDescriptionStatsBy(mtcars$mpg,
                      mtcars$am,
                      statistics = TRUE)
invisible(readline(prompt = "Press [enter] to continue"))

# Do the horizontal version
getDescriptionStatsBy(mtcars$gear,
                      mtcars$am,
                      statistics = TRUE,
                      hrzl_prop = TRUE)
invisible(readline(prompt = "Press [enter] to continue"))

mtcars$wt_with_missing <- mtcars$wt
mtcars$wt_with_missing[sample(1:NROW(mtcars), size = 8)] <- NA
getDescriptionStatsBy(mtcars$wt_with_missing, mtcars$am, statistics = TRUE,
                      hrzl_prop = TRUE, total_col_show_perc = FALSE)
invisible(readline(prompt = "Press [enter] to continue"))

\dontrun{
  ## There is also a LaTeX wrapper
  tll <- list(
    getDescriptionStatsBy(mtcars$gear, mtcars$am),
    getDescriptionStatsBy(mtcars$col, mtcars$am))

  latex(mergeDesc(tll),
        caption  = "Factored variables",
        file = "")
}
}
\concept{table functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docx_document.R
\name{prFtpOtherRemoval}
\alias{prFtpOtherRemoval}
\title{Removes other unwanted lines}
\usage{
prFtpOtherRemoval(output_str)
}
\arguments{
\item{output_str}{The input from readLines()}
}
\value{
string Returns without the unwanted lines
}
\description{
Removes other unwanted lines
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_align.R, R/boxGrobs_s3_align.R
\name{align}
\alias{align}
\alias{alignVertical}
\alias{alignHorizontal}
\alias{align.default}
\alias{align.list}
\alias{align.Gmisc_list_of_boxes}
\title{Align boxes}
\usage{
alignVertical(
  reference,
  ...,
  position = c("center", "top", "bottom"),
  subelement = NULL
)

alignHorizontal(
  reference,
  ...,
  position = c("center", "left", "right"),
  sub_position = c("none", "left", "right"),
  subelement = NULL
)

align(x, ...)

\method{align}{default}(x, ...)

\method{align}{list}(x, ...)

\method{align}{Gmisc_list_of_boxes}(x, ..., axis = c("y", "x", "vertical", "horizontal"))
}
\arguments{
\item{reference}{A \code{\link{boxGrob}}/\code{\link{boxPropGrob}}/\code{\link{coords}} object or a \code{\link[grid:unit]{unit}} or a
numerical value that can be converted into a \code{\link[grid:unit]{unit}} of \code{npc} type. If a numeric scalar is
provided and is \code{0} or greater than the number of boxes, it is treated as a coordinate (no error is raised).}

\item{...}{Arguments passed to [`alignVertical`] or [`alignHorizontal`].}

\item{position}{How to align the boxes, differs slightly for vertical and horizontal alignment
see the accepted arguments}

\item{subelement}{If a \code{list} of boxes is provided, this parameter can be used
to target a specific element (by name or index), or a deep path into nested
lists (e.g., \code{c("detail", 1)}), for the alignment operation. You can also
provide multiple targets by giving a list of paths (e.g., \code{list(c("detail", 1), c("followup", 1))}).
When a list of boxes is \emph{piped} into \code{alignVertical()}/\code{alignHorizontal()} and a named \code{reference} is
provided (e.g. \code{my_boxes |> alignHorizontal(reference = c("grp","sub"), .subelement = ...)}), the
function will unwrap a single-element wrapper so nested \code{.subelement} targets are found as expected.
The function will return the original list with the targeted element(s)
replaced by their aligned version(s).}

\item{sub_position}{When the box is a \code{\link{boxPropGrob}} it not only has the general \code{.positions} but
also \code{left} and \code{right} which can be viewed as separate boxes that have simply been merged.}

\item{x}{A `list` of boxes.}

\item{axis}{Orientation: `"y"`/`"vertical"` or `"x"`/`"horizontal"`.}
}
\value{
\code{list} with the boxes that are to be aligned

The aligned list of boxes (class `Gmisc_list_of_boxes`).
}
\description{
Aligns a set of \code{\link{boxGrob}}/\code{\link{boxPropGrob}} according to the first positional argument.

Align a list of boxes vertically or horizontally. Designed for piping (`|>`).
}
\examples{
library(grid)
grid.newpage()

# Create a reference box
box <- boxGrob("A cool reference box",
  x = .5, y = .8,
  box_gp = gpar(fill = "#ADB5C7")
)

# Create a group of boxes to align
boxes <- list(
  another_box = boxGrob("A horizontal box", x = .1, y = .5),
  yet_another_box = boxGrob("Another horizontal box", x = .8, y = .3)
)

# Align the group and then individual boxes within that group
# (do not pipe the list into the function, as the first argument is `reference`)
aligned_boxes <- alignHorizontal(boxes, reference = box, position = "right") |>
  alignVertical(reference = .5, position = "center")

# Example: align a nested element inside a complex list using a deep path
complex_list <- list(
  arms = list(
    early = list(boxGrob("Early", x = .2, y = .4)),
    late = list(boxGrob("Late", x = .8, y = .2))
  ),
  detail = list(
    list(boxGrob("D_early", x = .1, y = .6)),
    list(boxGrob("D_late", x = .9, y = .1))
  )
)

# Align the first detail element to the early arm by deep path
complex_list <- complex_list |>
  alignHorizontal(
    reference = c("arms", "early"),
    position = "center",
    subelement = c("detail", 1)
  )

# Print the reference and the aligned boxes
box
aligned_boxes
complex_list
}
\seealso{
Other flowchart components:
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docx_document.R
\name{prFtpOtherChanges}
\alias{prFtpOtherChanges}
\title{Changes lines for XML-conformity}
\usage{
prFtpOtherChanges(output_str)
}
\arguments{
\item{output_str}{The input from readLines()}
}
\value{
string Returns with changes
}
\description{
Changes lines for XML-conformity
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier_private_functions.R
\name{getBezierAdj4Arrw}
\alias{getBezierAdj4Arrw}
\title{Gets the bezier points adjusted for an arrow}
\usage{
getBezierAdj4Arrw(x, y, arrow_length, length_out = 100)
}
\arguments{
\item{x}{The x start and end points}

\item{y}{The spline control points}

\item{arrow_length}{The desired length of the arrow}

\item{length_out}{Increases the resolution for the final bezier
points, i.e. generating more fine-grained intervals}
}
\value{
list
}
\description{
Gets the bezier points adjusted for an arrow
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/copyAllNewAttributes.R
\name{copyAllNewAttributes}
\alias{copyAllNewAttributes}
\title{A simple thing to keep the attributes}
\usage{
copyAllNewAttributes(from, to, attr2skip = c(), attr2force = c())
}
\arguments{
\item{from}{The from object}

\item{to}{The to object}

\item{attr2skip}{An optional lists of attributes
that you may want to avoid having copied}

\item{attr2force}{An optional lists of attributes
that you may want to force copy even if they already
exist in the new object}
}
\value{
\code{object} The \code{to} argument object
}
\description{
Skips the attributes that the to object already has
to avoid overwriting dim and other important attributes
}
\examples{
a <- "test"
attr(a, 'wow') <- 1000
b <- a
b <- copyAllNewAttributes(a, b)
print(attr(b, 'wow'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/descGetMissing.R
\name{descGetMissing}
\alias{descGetMissing}
\title{Get statistics for missing data}
\usage{
descGetMissing(
  x,
  html = TRUE,
  number_first = TRUE,
  percentage_sign = TRUE,
  language = "en",
  useNA.digits = 1,
  ...
)
}
\arguments{
\item{x}{The variable that you want the statistics for}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{number_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().
This is only used together with the useNA variable.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable. Note, this is
only used when combined with the missing information.}

\item{language}{The ISO-639-1 two-letter code for the language of
interest. Currently only english is distinguished from the ISO
format using a ',' as the separator in the \code{\link[htmlTable]{txtInt}}
function.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{...}{Passed on to \code{\link{describeFactors}}}
}
\value{
\code{vector} A vector with the missing estimate
}
\description{
This function calculates the amount of missing per row for \code{\link{describeMean}},
\code{\link{describeMedian}} and custom description functions.
It will return invisibly when no missing values are present.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionClass_helpers.R
\name{prTcMatchClr}
\alias{prTcMatchClr}
\title{Finds the matching colors for the new addition based on the original set
of colors}
\usage{
prTcMatchClr(add, org, no_cols, no_rows, is3D)
}
\arguments{
\item{add}{The colors that we want to add}

\item{org}{The original colors}

\item{no_cols}{The no. of columns in the transition matrix}

\item{no_rows}{All the row counts in the transition plot}

\item{is3D}{Whether the transition matrix has 3 dimensions}
}
\value{
A list with colors that is appropriate for the transition object
}
\description{
Finds the matching colors for the new addition based on the original set
of colors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier_private_functions.R
\name{validateAndConvertVectorInputs}
\alias{validateAndConvertVectorInputs}
\title{Checks the input of a vector}
\usage{
validateAndConvertVectorInputs(x, y, x_origo, y_origo)
}
\arguments{
\item{x}{The x point of the vector}

\item{y}{The y point of the vector}

\item{x_origo}{The x origin if other than 0}

\item{y_origo}{The y origin if other than 0}
}
\value{
list with the input variables checked and
 converted if necessary.
}
\description{
It checks that a vector makes sense in its
grid parameters. It also creates an origo
at point 0 if none is provided.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_print.Gmisc_list_of_boxes.R
\name{print.Gmisc_list_of_boxes}
\alias{print.Gmisc_list_of_boxes}
\title{Output boxes}
\usage{
\method{print}{Gmisc_list_of_boxes}(x, ...)
}
\arguments{
\item{x}{A list of a set of [`boxGrob`]/[`boxPropGrob`] to plot}

\item{...}{Ignored argument}
}
\description{
Outputs a list of boxes as produced by either the
spread or align functions for boxGrobs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{calculateLinesAndArrow}
\alias{calculateLinesAndArrow}
\title{Gets offsetted lines}
\usage{
calculateLinesAndArrow(
  x,
  y,
  offset,
  end_x = -1,
  end_y = -1,
  arrow_offset = -1,
  rm_intersect = 3L
)
}
\arguments{
\item{x}{A numeric vector containing all the x-elements}

\item{y}{A numeric vector containing all the y-elements}

\item{offset}{The offset to add to the line, can be a vector if you
want to use different offsets.}

\item{end_x}{The x end of the line where the arrow occurrs (if < 0 arrow is skipped)}

\item{end_y}{The y end of the line where the arrow occurrs (if < 0 arrow is skipped)}

\item{arrow_offset}{The offset to add to the arrow section if any (if <= 0 arrow is skipped)}

\item{rm_intersect}{Set to 0 if you want to skip intersection removal, 1 only to remove left or
 2 to only remove right. See details for why.

 @section Remove intersections:

 When the line is wide and the arrow has a narrow curve there may appear an empty triangle due
 to polygon cancellation (two polygons within the same are cancel out). This behaviour may be
 ugly and the function therefor tries to remove these.

 \emph{Note:} it is expensive to check if there are the lineas may intersect at one point,
 remove those unexpected, and then adjust the line to the new situation so that the
 top and bottom lines match. It can also cause some unexpected behaviour why you may want to
 remove this feature if the arrow behaves erratically.}
}
\value{
\code{list(list(x = ..., y = ...))} Returns a list with the right/left
 lines that in turn lists with \emph{x} and \emph{y} elements
}
\description{
The function calculates new points according to the offset
that lie to the left/right of the provided line.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot_helpers.R
\name{prTpPlotArrows}
\alias{prTpPlotArrows}
\title{Plots the arrows}
\usage{
prTpPlotArrows(
  type,
  box_row,
  transition_flow,
  max_flow,
  min_lwd,
  max_lwd,
  clr,
  box_clr,
  transition_arrow_props,
  prop_start_sizes,
  prop_end_sizes,
  tot_spacing,
  box_width,
  abs_arrow_width,
  color_bar_subspace,
  add_width = NA
)
}
\arguments{
\item{type}{The type of arrow used}

\item{box_row}{Which box do the arrows originate from}

\item{max_flow}{The largest transition flow}

\item{min_lwd}{The minimum line width}

\item{max_lwd}{The maximum line width}

\item{clr}{The color of the line}

\item{box_clr}{The color of the box}

\item{transition_arrow_props}{The proportions of the different transitions if
available.}

\item{prop_start_sizes}{The proportions to the left}

\item{prop_end_sizes}{The proportions to the right}

\item{tot_spacing}{Total spacing between boxes}

\item{box_width}{The box width}

\item{color_bar_subspace}{If there is little or no difference exists
at the low/high proportions of the spectrum then it
can be of interest to focus the color change to the center
leaving the tails constant}

\item{add_width}{Add a certain width}
}
\value{
\code{void}
}
\description{
Outputs all the arrows from a box row
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier_private_functions.R
\name{getAbsoluteWidth}
\alias{getAbsoluteWidth}
\title{Translates "npc" widths into absolute units}
\usage{
getAbsoluteWidth(w, default.units, x = x, y = y)
}
\arguments{
\item{w}{The width that is to be changed}

\item{default.units}{A string indicating the default units to use
    if \code{x} or \code{y}
    are only given as numeric vectors.}

\item{x}{A numeric vector or unit object specifying x-locations of
    spline control points.}

\item{y}{A numeric vector or unit object specifying y-locations of
    spline control points.}
}
\value{
The width transformed into a \code{\link[grid]{unit}} of "mm"
}
\description{
The "npc" \code{\link[grid]{unit}} is not absolute, i.e.
it depends on the relation between height and  width within
the current viewport. We therefore need to change these values
to the value most likely intended. This is believed to be the
main direction of the arrow as specified by the first and last
bezier control points \code{x} and \code{y}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prAddDescStats.R
\name{prAddDescStats}
\alias{prAddDescStats}
\title{Add a p-value column to the results}
\usage{
prAddDescStats(
  results,
  x,
  by,
  statistics,
  statistics.suppress_warnings,
  statistics.sig_lim,
  statistics.two_dec_lim,
  html
)
}
\arguments{
\item{results}{The results that we want to add the column to}

\item{x}{If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
If it is a single variable it will be the core value that want the statistics for.
In the print this is equivalent to the output of this function.}

\item{by}{The variable that you want to split into different columns}

\item{statistics}{Add statistics, fisher test for proportions and Wilcoxon
for continuous variables. See details below for more customization.}

\item{statistics.suppress_warnings}{Hide warnings from the statistics function.}

\item{statistics.sig_lim}{The significance limit for < sign, i.e. p-value 0.0000312
should be < 0.0001 with the default setting.}

\item{statistics.two_dec_lim}{The limit for showing two decimals. E.g.
the p-value may be 0.056 and we may want to keep the two decimals in order
to emphasize the proximity to the all-mighty 0.05 p-value and set this to
\eqn{10^-2}. This allows that a value of 0.0056 is rounded to 0.006 and this
makes intuitive sense as the 0.0056 level as this is well below
the 0.05 value and thus not as interesting to know the exact proximity to
0.05. \emph{Disclaimer:} The 0.05-limit is really silly and debated, unfortunately
it remains a standard and this package tries to adapt to the current standards in order
to limit publication associated issues.}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}
}
\value{
results with added column
}
\description{
Add a p-value column to the results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrieve_and_has.R
\name{has}
\alias{has}
\title{An R alternative to the lodash \code{has} in JavaScript}
\usage{
has(sourceList, path)
}
\arguments{
\item{sourceList}{The \code{list()}/\code{c()} that is to be searched for the element}

\item{path}{A string that can be separated by [,] or ., the string \code{"elementname1.1.elementname"}
the validity of the path - it only separates and tries to address that element with `[[]]`.}
}
\value{
Returns a boolean.
}
\description{
This is a handy function for checking if item exist in a nested structure
}
\examples{
has(list(a = list(b = 1)), "a.b")

}
\seealso{
Other lodash similar functions:
\code{\link[=retrieve]{retrieve()}}
}
\concept{lodash similar functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_labelConnector.R
\name{print.Gmisc_connector_label}
\alias{print.Gmisc_connector_label}
\title{Print a connector label object}
\usage{
\method{print}{Gmisc_connector_label}(x, ...)
}
\arguments{
\item{x}{A `Gmisc_connector_label` object}

\item{...}{ignored}
}
\description{
Print a connector label object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/descriptionStats.R
\name{describeMean}
\alias{describeMean}
\title{Describe the mean}
\usage{
describeMean(
  x,
  html = TRUE,
  digits = 1,
  digits.nonzero = NA,
  number_first = TRUE,
  useNA = c("ifany", "no", "always"),
  useNA.digits = digits,
  percentage_sign = TRUE,
  plusmin_str,
  language = "en",
  ...
)
}
\arguments{
\item{x}{The variable that you want the statistics for}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{digits}{The number of decimals used}

\item{digits.nonzero}{The number of decimals used for values that are close to zero}

\item{number_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().
This is only used together with the useNA variable.}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable. Note, this is
only used when combined with the missing information.}

\item{plusmin_str}{Provide if you want anything other than the plus minus sign
suited for the given output format.}

\item{language}{The ISO-639-1 two-letter code for the language of
interest. Currently only english is distinguished from the ISO
format using a ',' as the separator in the \code{\link[htmlTable]{txtInt}}
function.}

\item{...}{Passed on to \code{\link{describeFactors}}}
}
\value{
\code{string} Returns a string formatted for either LaTeX or HTML
}
\description{
A function that returns a description of a continuous variable using
the mean together with the standard deviation. The standard deviation
is used as it is "industry standard" to use mean with standard deviation
and not because it's the only option.
}
\examples{
describeMean(1:10)
describeMean(c(1:10, NA), useNA = "always")
describeMean(c(1:10, NA), useNA = "no")
}
\seealso{
\code{\link{getDescriptionStatsBy}}

Other descriptive functions:
\code{\link[=describeFactors]{describeFactors()}},
\code{\link[=describeMedian]{describeMedian()}},
\code{\link[=describeProp]{describeProp()}},
\code{\link[=getDescriptionStatsBy]{getDescriptionStatsBy()}},
\code{\link[=getPvalWilcox]{getPvalWilcox()}}
}
\concept{descriptive functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/private_functions.R
\name{prPasteVec}
\alias{prPasteVec}
\title{Collapses a vector for throwing errors}
\usage{
prPasteVec(x)
}
\arguments{
\item{x}{The vector}
}
\description{
The function collapses a vector into an output useful when throwing
errors, e.g. 1:3 becomes '1', '2', '3'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionClass_helpers.R
\name{prTcPlotArrows}
\alias{prTcPlotArrows}
\title{Plots the arrows within a single transition set}
\usage{
prTcPlotArrows(
  trnstn_set,
  widths,
  type = c("simple", "gradient"),
  clr,
  rez,
  origin_boxes,
  target_boxes,
  left_box_clrs,
  add_width,
  max_flow,
  min_width,
  max_width,
  abs_arrow_width = FALSE,
  clr_bar_subspace
)
}
\arguments{
\item{trnstn_set}{The set of transitions to plot}

\item{widths}{The arrow widths}

\item{type}{The type of arrow to be used}

\item{clr}{The arrow colors (a matrix)}

\item{rez}{The arrow resolution}

\item{origin_boxes}{The origin boxes' positions to the left}

\item{target_boxes}{The target boxes' positions to the right}

\item{left_box_clrs}{The colors of the left side boxes}

\item{add_width}{Adds a certain width - useful for background arrows}

\item{max_flow}{The maximum flow if it is common for the entire image}

\item{min_width}{The minimum line width}

\item{max_width}{The maximum line width}

\item{abs_arrow_width}{If the arrow width should be the same for all arrows}
}
\description{
Plots the arrows within a single transition set
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_boxPropGrob.R
\name{prBuildSubLabel}
\alias{prBuildSubLabel}
\title{Add a sub-label to \code{boxPropGrob}}
\usage{
prBuildSubLabel(label, prop, txt_gp, side = c("left", "right"))
}
\arguments{
\item{label}{The text of the label}

\item{prop}{The proportion}

\item{txt_gp}{The style as defined by \code{\link[grid]{gpar}()}}

\item{side}{The side that the label belongs to}
}
\value{
A \code{textGrob} with he additional attributes \code{width} and \code{height}.
}
\description{
Add a sub-label to \code{boxPropGrob}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_boxHeaderGrob.R
\name{boxHeaderGrob}
\alias{boxHeaderGrob}
\title{Create a composite box with header and body text}
\usage{
boxHeaderGrob(
  header,
  body,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  bjust = "center",
  header_gp = getOption("boxHeaderGrobTxt", default = gpar(color = "black", cex = 1)),
  body_gp = getOption("boxBodyGrobTxt", default = gpar(color = "black", cex = 0.9)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  box_fn = roundrectGrob,
  name = NULL
)
}
\arguments{
\item{header}{The header text (centered, typically bold).}

\item{body}{The body text (left-justified, typically with bullets or multiple lines).}

\item{y}{The y position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.}

\item{x}{The x position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.}

\item{width}{The box automatically adapts the size but you can force by specifying the width}

\item{height}{The box automatically adapts the size but you can force by specifying the height}

\item{bjust}{The justification for the box: left, center, right, top or bottom.
See the \code{just} option for the \code{\link[grid]{viewport}}}

\item{header_gp}{The \code{\link[grid]{gpar}} style to apply to the header text.
Defaults to bold text at slightly larger size.}

\item{body_gp}{The \code{\link[grid]{gpar}} style to apply to the body text.}

\item{box_gp}{The \code{\link[grid]{gpar}} style to apply to the box function of `box_fn` below.}

\item{box_fn}{Function to create box for the text. Parameters of `x=0.5`, `y=0.5` and `box_gp` will
be passed to this function and return a \code{grob} object.}

\item{name}{a character identifier for the \code{grob}. Used to find the \code{grob} on the display
list and/or as a child of another grob.}
}
\value{
A grob with class "box" compatible with all Gmisc flowchart helpers
 (\code{\link{coords}}, \code{\link{distance}}, \code{\link{connectGrob}},
 \code{\link{spreadVertical}}, \code{\link{alignHorizontal}}, etc.)
}
\description{
Creates a \code{\link[grid:grid.grob]{grob}} box with a centered header and
left-justified body text. Useful for flowchart elements that combine a title
with bullet points or multi-line descriptions.
}
\examples{
library(grid)
grid.newpage()

# Simple example with header and bullets
box1 <- boxHeaderGrob(
  header = "Early rehabilitation",
  body = paste("0-1 weeks", "• Instruction", "• Pendulum + assisted ROM", sep = "\n"),
  header_gp = gpar(fontsize = 11, fontface = "bold"),
  body_gp = gpar(fontsize = 9),
  box_gp = gpar(fill = "#E8F5E9", col = "#2E7D32")
)
grid.draw(box1)

# Works with all helpers
box2 <- boxHeaderGrob(
  header = "Delayed rehabilitation",
  body = "0-3 weeks\n• Sling immobilisation",
  y = 0.3,
  box_gp = gpar(fill = "#FFF8E1", col = "#EF6C00")
)

# Distance calculation works
d <- distance(box1, box2, type = "v")

# Connect boxes
grid.draw(box2)
connectGrob(box1, box2, type = "vertical")
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docx_document.R
\name{docx_document}
\alias{docx_document}
\title{Formatter wrapper for \code{\link[rmarkdown]{html_document}}, facilitates easier porting to docx}
\usage{
docx_document(
  ...,
  self_contained = FALSE,
  mathjax = NULL,
  theme = NULL,
  highlight = NULL,
  css = "rmarkdown/docx.css",
  h1_style = "margin: 24pt 0pt 0pt 0pt;",
  other_h_style = "margin: 10pt 0pt 0pt 0pt;",
  remove_scripts = TRUE,
  force_captions = FALSE,
  css_max_width
)
}
\arguments{
\item{...}{Passed onto \code{\link[rmarkdown]{html_document}}.}

\item{self_contained}{Overrides the default \code{TRUE} for
\code{\link[rmarkdown]{html_document}} to \code{FALSE} as
LibreOffice hangs on long lines such as the base64
images included in the self-contained version.}

\item{mathjax}{The advanced mathjax does not work with with
Word/LibreOffice.}

\item{theme}{No theme should be used for the output as the
custom CSS should take care of everything.}

\item{highlight}{By default turn off highlighting as scripts
are difficult to import. This does though work somewhat OK when
copy-pasting from the web-browser.}

\item{css}{The CSS if other that the default within the package}

\item{h1_style}{You can choose any css style formatting here that you
want to be applied to all h1 elements. Note: this is only applied
if LibreOffice_adapt is \code{TRUE}.}

\item{other_h_style}{This is the formatting applied to any other
h elements not included to the first. Note: this is only applied
if LibreOffice_adapt is \code{TRUE}.}

\item{remove_scripts}{\code{TRUE} if <script></script> tags are
to be removed. These are usually not compatible with Word-processors
and should therefore in most cases be stripped from the document.}

\item{force_captions}{Since out.width and out.height remove the option of
having captions this allows a workaround through some processing
via the XML-package}

\item{css_max_width}{The max width of the body element. Defaults to "40em"
if not specified. Any CSS-compliant width format works.}
}
\value{
R Markdown output format to pass to \code{\link[rmarkdown]{render}}
}
\description{
This function adds the option of having adaptations needed for seemless integration with
MS Word for importing html-documents in the .docx-format. The advantage of html documents is
the ability to create advanced formatting frequently needed in medical publications
and that is available in the \code{\link[htmlTable]{htmlTable}} function. You can view
\href{https://gforge.se/2014/07/fast-track-publishing-using-rmarkdown/}{the series}
for more details regarding how to achieve
fast-track-publishing (ftp) together with knitr.
}
\details{
If you want to get equations into Word the currently best way is to
use the \code{\link[rmarkdown]{word_document}} format.
}
\examples{
# Possible yaml configuration at the top of the Rmd doc
\dontrun{
---
title: "Test"
author: "Max Gordon"
output:
  Gmisc::docx_document
---
}
}
\author{
Max Gordon
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_units.R
\name{MidDistanceX}
\alias{MidDistanceX}
\alias{MidDistanceY}
\title{Calculate midpoint between two units}
\usage{
MidDistanceX(u1, u2)

MidDistanceY(u1, u2)
}
\arguments{
\item{u1}{The first unit}

\item{u2}{The second unit}
}
\value{
A unit object
}
\description{
Calculates the midpoint between two units. If the arguments are
numeric they are converted to npc units.
}
\examples{
MidDistanceX(grid::unit(.1, "npc"), grid::unit(.8, "npc"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/figCaptionNo.R
\name{figCapNo}
\alias{figCapNo}
\title{Adds a figure caption number}
\usage{
figCapNo(
  str,
  roman = getOption("fig_caption_no_roman", default = FALSE),
  sprintf_str = getOption("fig_caption_no_sprintf", default = "Fig. \%s: \%s")
)
}
\arguments{
\item{str}{The string that is to be prepended with string}

\item{roman}{Whether or not to use roman numbers instead
of Arabic. Can also be set through \code{options(fig_caption_no_roman = TRUE)}}

\item{sprintf_str}{An \code{\link[base:sprintf]{sprintf}} formatted
string where the first argument is reserved for the string
generated by the counter and the second one is for the caption
text. Can also be set through \code{options(fig_caption_no_sprintf = TRUE)}}
}
\description{
The function relies on \code{options("fig_caption_no")}
in order to keep track of the last number. If
you want to force the caption function to skip
captions while still using it in the knitr fig.cap option
then simply set \code{options(fig_caption_no = FALSE)}
}
\examples{
\dontrun{
```{r, fig.cap = pigCapNo("My nice plot")}
plot(1:10 + rnorm(10), 1:10)
```
}
org_opts <- options(fig_caption_no = 2,
                    fig_caption_no_sprintf = "Figure \%s: \%s")
figCapNo("A plot with caption number = 3")

org_opts <- options(fig_caption_no = TRUE)
figCapNo("A plot with caption number = 1")

# Use default setting
options(fig_caption_no_sprintf = NULL)
figCapNo("A plot with caption number = 2")

# Return the original settings
options(org_opts)
}
\seealso{
Other figure caption functions:
\code{\link[=figCapNoLast]{figCapNoLast()}},
\code{\link[=figCapNoNext]{figCapNoNext()}}
}
\concept{figure caption functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/descriptionStats.R
\name{describeFactors}
\alias{describeFactors}
\title{Describes factor variables}
\usage{
describeFactors(
  x,
  html = TRUE,
  digits = 1,
  digits.nonzero = NA,
  number_first = TRUE,
  useNA = c("ifany", "no", "always"),
  useNA.digits = digits,
  horizontal_proportions,
  percentage_sign = TRUE,
  language = "en",
  ...
)
}
\arguments{
\item{x}{The variable that you want the statistics for}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{digits}{The number of decimals used}

\item{digits.nonzero}{The number of decimals used for values that are close to zero}

\item{number_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().
This is only used together with the useNA variable.}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{horizontal_proportions}{Is only active if useNA since this is
the only case of a proportion among continuous variables. This is default NULL and indicates
that the proportions are to be interpreted in a vertical manner.
If we want the data to be horizontal, i.e. the total should be shown
and then how these differ in the different groups then supply the
function with the total number in each group, i.e. if done in a by
manner as in \code{\link{getDescriptionStatsBy}} it needs to provide
the number before the by() command.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable. Note, this is
only used when combined with the missing information.}

\item{language}{The ISO-639-1 two-letter code for the language of
interest. Currently only english is distinguished from the ISO
format using a ',' as the separator in the \code{\link[htmlTable]{txtInt}}
function.}

\item{...}{Passed on to \code{\link[htmlTable]{txtInt}}}
}
\value{
A string formatted for printing either latex by  HTML
}
\description{
A function that returns a description of proportions in a
factor that contains the number of times a level occurs and the percentage
}
\examples{
set.seed(1)
describeFactors(sample(50, x = c("A", "B", "C"), replace = TRUE))

n <- 500
my_var <- factor(sample(size = n, x = c("A", "B", "C", NA), replace = TRUE))
my_exp <- rbinom(n = n, size = 1, prob = 0.2)
total <- table(my_var, useNA = "ifany")
by(my_var,
  INDICES = my_exp,
  FUN = describeFactors,
  useNA = "ifany",
  horizontal_proportions = total
)
}
\seealso{
\code{\link{getDescriptionStatsBy}}

Other descriptive functions:
\code{\link[=describeMean]{describeMean()}},
\code{\link[=describeMedian]{describeMedian()}},
\code{\link[=describeProp]{describeProp()}},
\code{\link[=getDescriptionStatsBy]{getDescriptionStatsBy()}},
\code{\link[=getPvalWilcox]{getPvalWilcox()}}
}
\concept{descriptive functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot_helpers.R
\name{prTpPlotBox}
\alias{prTpPlotBox}
\title{Plots the box for the transition plot}
\usage{
prTpPlotBox(
  bx,
  bx_txt,
  fill,
  txt_clr,
  txt_gpar = NULL,
  cex,
  line_col,
  lwd,
  prop = NA
)
}
\arguments{
\item{bx}{A list with the x, y, height and width parameters}

\item{bx_txt}{The box text}

\item{fill}{The fill color}

\item{txt_clr}{The text color}

\item{txt_gpar}{The [grid::gpar()] object that is overridden with col & cex by other parameters}

\item{cex}{The font size}

\item{line_col}{The line color around the box}

\item{lwd}{The line width}

\item{prop}{Provide a proportion if the box should be split (0-1)}
}
\value{
\code{void}
}
\description{
Plots the box for the transition plot
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier_private_functions.R
\name{isHorizontal}
\alias{isHorizontal}
\title{Just a simple help with interpreting the radians}
\usage{
isHorizontal(angle_radian)
}
\arguments{
\item{angle_radian}{The angle in radians}
}
\value{
boolean
}
\description{
Just a simple help with interpreting the radians
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_helpers.R
\name{prGetStatistics}
\alias{prGetStatistics}
\title{Get statistics according to the type}
\usage{
prGetStatistics(
  x,
  show_perc = FALSE,
  html = TRUE,
  digits = 1,
  digits.nonzero = NA,
  numbers_first = TRUE,
  useNA = c("ifany", "no", "always"),
  useNA.digits = digits,
  show_all_values = FALSE,
  continuous_fn = describeMean,
  factor_fn = describeFactors,
  prop_fn = factor_fn,
  percentage_sign = TRUE,
  default_ref = NULL
)
}
\arguments{
\item{x}{If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
If it is a single variable it will be the core value that want the statistics for.
In the print this is equivalent to the output of this function.}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{digits}{The number of decimals used}

\item{digits.nonzero}{The number of decimals used for values that are close to zero}

\item{numbers_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{show_all_values}{Show all values in proportions. For factors with only two values
it is most sane to only show one option as the other one will just be a complement
to the first, i.e. we want to convey a proportion. For instance sex - if you know
gender then automatically you know the distribution of the other sex as it's 100 \% - other \%.
To choose which one you want to show then set the \code{default_ref} parameter.}

\item{continuous_fn}{The method to describe continuous variables. The
default is \code{\link{describeMean}}.}

\item{factor_fn}{The method used to describe factors, see \code{\link{describeFactors}}.}

\item{prop_fn}{The method used to describe proportions, see \code{\link{describeProp}}.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable.}

\item{default_ref}{The default reference when dealing with proportions. When using
`dplyr` syntax (`tidyselect`) you can specify a named vector/list for each column name.}
}
\value{
A matrix or a vector depending on the settings
}
\description{
A simple function applied by the \code{\link{getDescriptionStatsBy}}
for the total column.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezierArrowGradient.R
\name{bezierArrowGradient}
\alias{bezierArrowGradient}
\title{A bezier arrow with gradient}
\usage{
bezierArrowGradient(
  x = c(0.2, 0.7, 0.3, 0.9),
  y = c(0.2, 0.2, 0.9, 0.9),
  width = 0.05,
  clr = "#000000",
  default.units = "npc",
  align_2_axis = TRUE,
  grdt_type = c("triangle", "rectangle"),
  grdt_prop = 0.8,
  grdt_decrease_prop = 0.5,
  grdt_clr_prop = 0.7,
  grdt_line_width,
  grdt_clr = "#2F4F2F",
  vp = NULL,
  gp = gpar(),
  rm_intersect = 3L,
  ...
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of
    spline control points.}

\item{y}{A numeric vector or unit object specifying y-locations of
    spline control points.}

\item{width}{The width of the arrow, either a numeric single number or a unit. \strong{Note:}
The arrow does not rely on \code{lwd} but on actual width.}

\item{clr}{The color of the arrow. This is the main color of the arrow and not the gradient color.}

\item{default.units}{A string indicating the default units to use
    if \code{x} or \code{y}
    are only given as numeric vectors.}

\item{align_2_axis}{Indicates if the arrow should be vertically/horizontally
aligned. This is useful for instance if the arrow attaches to a box.}

\item{grdt_type}{The type of growth and gradient that is to be used,
currently it only supports triangle (I'm considering adding bezier curves
but currently I'm a little tired of coding)}

\item{grdt_prop}{The proportion of the full length that should be a the gradient. The gradient
consists of three things:
(1) the central band,
(2) the slimming of the central band,
(3) the color shift into the arrow color.
\emph{Note} that the the slimming and color proportions can be overlapping.}

\item{grdt_decrease_prop}{The proportion of the gradient that should be decreasing,
i.e. narrowing according to the \code{grdt_type} argument.}

\item{grdt_clr_prop}{The proportion of the gradient that should be converging to the arrow color.}

\item{grdt_line_width}{The width of the border line. If not specified it defaults to 5 \% of
the original width, note the gradient's width is thus 90 \%.}

\item{grdt_clr}{The color of the gradient.}

\item{vp}{A Grid viewport object (or NULL).}

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{rm_intersect}{Set to 0 if you want to skip intersection removal, 1 only to remove left or
 2 to only remove right. See details for why.

 @section Remove intersections:

 When the line is wide and the arrow has a narrow curve there may appear an empty triangle due
 to polygon cancellation (two polygons within the same are cancel out). This behaviour may be
 ugly and the function therefor tries to remove these.

 \emph{Note:} it is expensive to check if there are the lineas may intersect at one point,
 remove those unexpected, and then adjust the line to the new situation so that the
 top and bottom lines match. It can also cause some unexpected behaviour why you may want to
 remove this feature if the arrow behaves erratically.}

\item{...}{Passed on to \code{\link{bezierArrowSmpl}}}
}
\value{
A grob of \code{\link[grid:grid.grob]{gList}}-type
}
\description{
This is an experimental addition to the original \code{\link{bezierArrowSmpl}}
with the addition of a gradient in the center of the arrow that fades.
}
\section{Note}{

The triangle section of the arrow is not currently included in the gradient.
}

\examples{
library(grid)
grid::grid.newpage()
arrowGrob <- bezierArrowGradient(
  x = c(.1, .3, .6, .9),
  y = c(0.2, 0.2, 0.9, 0.9)
)
grid.draw(arrowGrob)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{gnrlBezierPoints}
\alias{gnrlBezierPoints}
\title{Generates a generalized Bézier line}
\usage{
gnrlBezierPoints(ctrl_points, length_out = 100L)
}
\arguments{
\item{ctrl_points}{The ctrl_points for the bezier control points. This should
either be a \code{matrix} or a \code{data.frame}.}

\item{length_out}{The length of the return points, i.e. how fine
detailed the points should be.}
}
\description{
This is a general form of bezier line that can be used for cubic, quadratic,
and more advanced Bézier lines.
}
\examples{
library(grid)
grid.newpage()
l <- gnrlBezierPoints(data.frame(x = c(.1, -.1, .7, 1, 1, 0.1),
                                 y = c(.9, 0, 1, .8, .4, .1)),
                      length_out = 100)
grid.lines(l[,1], l[,2], gp=gpar(col="#550000", lwd = 4))

out_sizes <- 4:20
clrs <- colorRampPalette(c("orange", "darkblue"))(length(out_sizes))
for (i in out_sizes){
   l <- gnrlBezierPoints(data.frame(x = c(.1, -.1, .7, 1, 1, 0.1),
                                    y = c(.9, 0, 1, .8, .4, .1)),
                         length_out = i)
   grid.lines(l[,1], l[,2],
   gp=gpar(col=clrs[which(i == out_sizes)]))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insertRowAndKeepAttr.R
\name{insertRowAndKeepAttr}
\alias{insertRowAndKeepAttr}
\title{Insert a row into a matrix}
\usage{
insertRowAndKeepAttr(m, r, v = NA, rName = "")
}
\arguments{
\item{m}{matrix}

\item{r}{row number where the new row should be inserted}

\item{v}{optional values for the new row}

\item{rName}{optional character string: the name of the new row.}
}
\value{
\code{matrix} Returns a matrix with one more row than the provided matrix \code{m}
}
\description{
Inserts a row and keeps the attributes \code{\link{copyAllNewAttributes}}
}
\examples{
test <- matrix(1:4, ncol = 2)
attr(test, "wow") <- 1000
test <- insertRowAndKeepAttr(test, 2)
print(attr(test, "wow"))
}
\author{
Max Gordon, Arne Henningsen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastDoCall.R
\name{fastDoCall}
\alias{fastDoCall}
\title{An alternative to the internal \code{do.call}}
\usage{
fastDoCall(what, args, quote = FALSE, envir = parent.frame())
}
\arguments{
\item{what}{either a function or a non-empty character string naming the
    function to be called.}

\item{args}{a \emph{list} of arguments to the function call.  The
    \code{names} attribute of \code{args} gives the argument names.}

\item{quote}{a logical value indicating whether to quote the
    arguments.}

\item{envir}{an environment within which to evaluate the call.  This
    will be most useful if \code{what} is a character string and
    the arguments are symbols or quoted expressions.}
}
\description{
The \code{\link[base]{do.call}} can be somewhat slow, especially when working
with large objects. This function is based upon the suggestions from Hadley
Wickham on the R mailing list.
Also thanks to \emph{Tommy} at StackOverflow for
\href{https://stackoverflow.com/questions/10022436/do-call-in-combination-with}{suggesting}
how to handle double and triple colon operators, \code{::}, further
enhancing the function.
}
\section{Note}{


While the function attempts to do most of what \code{\link[base]{do.call}}
can it has limitations. It can currently not parse the example code from the
original function: \code{do.call(paste, list(as.name("A"), as.name("B")), quote = TRUE)}
and the functionality of \code{quote} has not been thoroughly tested.
}

\examples{
fastDoCall("complex", list(imaginary = 1:3))

## if we already have a list (e.g. a data frame)
## we need c() to add further arguments
tmp <- expand.grid(letters[1:2], 1:3, c("+", "-"))
fastDoCall("paste", c(tmp, sep = ""))

## examples of where objects will be found.
A <- 2
f <- function(x) print(x^2)
env <- new.env()
assign("A", 10, envir = env)
assign("f", f, envir = env)
f <- function(x) print(x)
f(A)                                      # 2
fastDoCall("f", list(A))                     # 2
fastDoCall("f", list(A), envir = env)        # 4
fastDoCall(f, list(A), envir = env)          # 2
fastDoCall("f", list(quote(A)), envir = env) # 100
fastDoCall(f, list(quote(A)), envir = env)   # 10
fastDoCall("f", list(as.name("A")), envir = env) # 100

eval(call("f", A))                      # 2
eval(call("f", quote(A)))               # 2
eval(call("f", A), envir = env)         # 4
eval(call("f", quote(A)), envir = env)  # 100
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_equalizeWidths.R
\name{equalizeWidths}
\alias{equalizeWidths}
\title{Equalize box widths}
\usage{
equalizeWidths(x, subelement = NULL, width = NULL)
}
\arguments{
\item{x}{A list of boxes (or nested lists of boxes). Can also be a single box.}

\item{subelement}{Optional target(s) inside \code{x}.
Can be a single path (e.g. \code{c("groups", 1)}) or a list of paths
(e.g. \code{list(c("groups", 1), c("groups2", 1))}).
If \code{NULL}, all top-level boxes in \code{x} are used.}

\item{width}{Optional width to apply. If \code{NULL}, the maximum current width
among selected boxes is used.}
}
\value{
The updated object with equalized widths.
}
\description{
Sets selected boxes to a shared width. This is useful when aligning flowchart
levels so corresponding boxes have consistent visual width and center points.
}
\examples{
fc <- flowchart(
  groups = list("Group 1", "Group 2"),
  groups2 = list("Analysed", "Analysed")
)

fc |>
  equalizeWidths(subelement = list(c("groups", 1),
                                   c("groups2", 1),
                                   c("groups", 2),
                                   c("groups2", 2)))

# Global fixed width via explicit argument
fc |> equalizeWidths(subelement = "groups", width = grid::unit(30, "mm"))
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeDesc_S3_fns.R
\name{latex.descMrg}
\alias{latex.descMrg}
\alias{htmlTable.descMrg}
\alias{print.descMrg}
\alias{knit_print.descMrg}
\title{A wrapper around the \code{\link[htmlTable]{htmlTable}}/\code{\link[Hmisc]{latex}} function}
\usage{
\method{latex}{descMrg}(object, ...)

\method{htmlTable}{descMrg}(x, ...)

\method{print}{descMrg}(x, ...)

\method{knit_print}{descMrg}(x, ...)
}
\arguments{
\item{object}{The \code{\link{mergeDesc}} object}

\item{...}{Passed onto \code{\link[htmlTable]{htmlTable}}/\code{\link[Hmisc]{latex}}}
}
\description{
Calls the \code{\link[htmlTable]{htmlTable}}/\code{\link[Hmisc]{latex}} after extracting the
\code{rgroup}/\code{n.rgroup} arguments.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_column_labels.R
\name{set_column_labels}
\alias{set_column_labels}
\title{Add [Hmisc::label()] to multiple columns}
\usage{
set_column_labels(x, ...)
}
\arguments{
\item{x}{The data frame that we want to label}

\item{...}{Variable names with their intended label, e.g. `mpg = "Miles per gallon"`.}
}
\value{
The original data.frame
}
\description{
Add label attribute using `dplyr` syntax using the [Hmisc::label()]
}
\examples{
library(magrittr)
data(mtcars)
mtcars_with_labels <- mtcars \%>\%
  set_column_labels(mpg = "Gas",
                    cyl = "Cylinders",
                    hp = "Strength")
Hmisc::label(mtcars_with_labels$mpg)
}
\seealso{
Other Hmisc helpers:
\code{\link[=set_column_units]{set_column_units()}}
}
\concept{Hmisc helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docx_document.R
\name{prFtpScriptRemoval}
\alias{prFtpScriptRemoval}
\title{Removes the <script>*</scrip>}
\usage{
prFtpScriptRemoval(output_str)
}
\arguments{
\item{output_str}{The input from readLines()}
}
\value{
string Returns without the script elements
}
\description{
Removes the <script>*</scrip>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_boxShapes.R
\name{boxShapes}
\alias{boxShapes}
\alias{diamond_rounded_box_fn}
\alias{diamond_sharp_box_fn}
\alias{boxDiamondGrob}
\alias{ellipse_box_fn}
\alias{boxEllipseGrob}
\alias{rack_box_fn}
\alias{boxRackGrob}
\alias{database_box_fn}
\alias{boxDatabaseGrob}
\alias{document_box_fn}
\alias{boxDocumentGrob}
\alias{documents_box_fn}
\alias{boxDocumentsGrob}
\alias{tape_box_fn}
\alias{boxTapeGrob}
\alias{server_box_fn}
\alias{boxServerGrob}
\title{Additional box shape helpers for `boxGrob`}
\usage{
diamond_rounded_box_fn(x = 0.5, y = 0.5, gp = gpar())

diamond_sharp_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxDiamondGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  rounded = TRUE,
  name = NULL
)

ellipse_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxEllipseGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  name = NULL
)

rack_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxRackGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  name = NULL
)

database_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxDatabaseGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  name = NULL
)

document_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxDocumentGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  name = NULL
)

documents_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxDocumentsGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  name = NULL
)

tape_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxTapeGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  name = NULL
)

server_box_fn(x = 0.5, y = 0.5, gp = gpar())

boxServerGrob(
  label,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxGrobTxt", default = gpar(color = "black", cex = 1)),
  box_gp = getOption("boxGrob", default = gpar(fill = "white")),
  name = NULL
)
}
\arguments{
\item{x}{Horizontal position inside the box viewport (0-1).}

\item{y}{Vertical position inside the box viewport (0-1).}

\item{gp}{A `gpar` object with styling information (fill/col/etc).}

\item{label}{The label to print - should be a number, text or expression.}

\item{width}{The box automatically adapts the size but you can force by specifying the width}

\item{height}{The box automatically adapts the size but you can force by specifying the height}

\item{just}{The justification for the text: left, center or right.}

\item{bjust}{The justification for the box: left, center, right, top or bottom.
See the \code{just} option for the \code{\link[grid]{viewport}}}

\item{txt_gp}{The \code{\link[grid]{gpar}} style to apply to the text. Set \code{boxGrobTxt} option
if you want to customize all the boxes at once.}

\item{box_gp}{The \code{\link[grid]{gpar}} style to apply to the box function of `box_fn` below.}

\item{rounded}{Logical; use rounded diamond (TRUE) or sharp diamond (FALSE).}

\item{name}{a character identifier for the \code{grob}. Used to find the \code{grob} on the display
list and/or as a child of another grob.}
}
\value{
A `grid` grob that draws the requested shape inside the box viewport.
}
\description{
These helper functions provide alternative box drawing functions that can be
passed to `boxGrob(..., box_fn = ...)` or used via convenience wrappers
(`boxDiamondGrob`, `boxEllipseGrob`, `boxRackGrob`). They are intentionally
lightweight and implemented using base `grid` primitives.

Two diamond variants are provided: a rounded diamond (default) implemented
with `xsplineGrob` for slightly softened corners, and a sharp diamond
implemented with `polygonGrob` for strict straight edges.

Approximates an ellipse using a polygon with many points; it respects the
box viewport so it scales correctly.

A rectangular box with a slightly darker top band to mimic a server / disk rack
look. Implemented by composing a rounded rectangle and an overlaying top
strip. The strip is drawn without border to avoid double outlines.

Draws a cylinder-like database symbol (ellipse top, rectangular body, base ellipse).

A document icon with a wavy bottom edge, useful to indicate printed output.

Draws a stack of document shapes by repeating the same shape with offsets.

Left/right curved edges to mimic tape-like icons.

A server-styled box with front-panel vent lines so it is visually distinct
from a rack (top strip) representation.
}
\details{
Shape functions are called with `x` and `y` in [0,1] inside the box viewport and with `box_gp` for styling; convenience wrappers match the `boxGrob` signature.
}
\examples{
# These functions are designed to be passed to `boxGrob(..., box_fn = ...)`.
grid::grid.newpage()
d <- boxGrob("Decision", box_fn = diamond_rounded_box_fn)
e <- boxGrob("Start", box_fn = ellipse_box_fn)
r <- boxGrob("Server", box_fn = rack_box_fn)
spreadHorizontal(list(d, e, r))
}
\seealso{
\code{\link{boxGrob}} for usage and the `box_fn` argument

Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathJoin.R
\name{pathJoin}
\alias{pathJoin}
\title{A path join function}
\usage{
pathJoin(...)
}
\arguments{
\item{...}{A set of strings to join. Each may be a single string or a vector.
If you provide vectors they can either be all of the same length
or where there are two lengths where one is equal to 1.}
}
\value{
\code{string} A string with the merged path
}
\description{
This function joins strings into a valid path. It is a simple version of python's
\code{os.path.join} and fixes simple problems such as having/not having trailing /
in each section.
}
\examples{
pathJoin("my_base_path/helpers", "superfunction.R")
# 'my_base_path/helpers/superfunction.R'

base_dir <- "/home/tester/images"
out <- data.frame(filename = c("file1.png", "file2.png", "file3.png")) |>
  dplyr::mutate(full_path = pathJoin(base_dir, filename))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docx_document.R
\name{prFtpHeaderStyle}
\alias{prFtpHeaderStyle}
\title{Helper to ftp_document}
\usage{
prFtpHeaderStyle(output_str, h1_style, other_h_style)
}
\arguments{
\item{output_str}{The output string from readLines()}

\item{h1_style}{You can choose any css style formatting here that you
want to be applied to all h1 elements. Note: this is only applied
if LibreOffice_adapt is \code{TRUE}.}

\item{other_h_style}{This is the formatting applied to any other
h elements not included to the first. Note: this is only applied
if LibreOffice_adapt is \code{TRUE}.}
}
\value{
string
}
\description{
Helper to ftp_document
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy.R,
%   R/getDescriptionStatsBy_S3_fns.R
\name{getDescriptionStatsBy}
\alias{getDescriptionStatsBy}
\alias{htmlTable.Gmisc_getDescriptionStatsBy}
\alias{print.Gmisc_getDescriptionStatsBy}
\alias{knit_print.Gmisc_getDescriptionStatsBy}
\alias{length.Gmisc_getDescriptionStatsBy}
\title{Creating of description statistics}
\usage{
getDescriptionStatsBy(
  x,
  ...,
  by,
  digits = 1,
  digits.nonzero = NA,
  html = TRUE,
  numbers_first = TRUE,
  statistics = FALSE,
  statistics.sig_lim = 10^-4,
  statistics.two_dec_lim = 10^-2,
  statistics.suppress_warnings = TRUE,
  useNA = c("ifany", "no", "always"),
  useNA.digits = digits,
  continuous_fn = describeMean,
  prop_fn = describeProp,
  factor_fn = describeFactors,
  show_all_values = FALSE,
  hrzl_prop = FALSE,
  add_total_col,
  total_col_show_perc = TRUE,
  use_units = FALSE,
  units_column_name = "Units",
  default_ref = NULL,
  NEJMstyle = FALSE,
  percentage_sign = TRUE,
  header_count = NULL,
  missing_value = "-",
  names_of_missing = NULL
)

\method{htmlTable}{Gmisc_getDescriptionStatsBy}(x, ...)

\method{print}{Gmisc_getDescriptionStatsBy}(x, ...)

\method{knit_print}{Gmisc_getDescriptionStatsBy}(x, ...)

\method{length}{Gmisc_getDescriptionStatsBy}(x)
}
\arguments{
\item{x}{If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
If it is a single variable it will be the core value that want the statistics for.
In the print this is equivalent to the output of this function.}

\item{...}{The variables that you want you statistic for. In the print all thes parameters
are passed on as [htmlTable::htmlTable] arguments.}

\item{by}{The variable that you want to split into different columns}

\item{digits}{The number of decimals used}

\item{digits.nonzero}{The number of decimals used for values that are close to zero}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{numbers_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().}

\item{statistics}{Add statistics, fisher test for proportions and Wilcoxon
for continuous variables. See details below for more customization.}

\item{statistics.sig_lim}{The significance limit for < sign, i.e. p-value 0.0000312
should be < 0.0001 with the default setting.}

\item{statistics.two_dec_lim}{The limit for showing two decimals. E.g.
the p-value may be 0.056 and we may want to keep the two decimals in order
to emphasize the proximity to the all-mighty 0.05 p-value and set this to
\eqn{10^-2}. This allows that a value of 0.0056 is rounded to 0.006 and this
makes intuitive sense as the 0.0056 level as this is well below
the 0.05 value and thus not as interesting to know the exact proximity to
0.05. \emph{Disclaimer:} The 0.05-limit is really silly and debated, unfortunately
it remains a standard and this package tries to adapt to the current standards in order
to limit publication associated issues.}

\item{statistics.suppress_warnings}{Hide warnings from the statistics function.}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{continuous_fn}{The method to describe continuous variables. The
default is \code{\link{describeMean}}.}

\item{prop_fn}{The method used to describe proportions, see \code{\link{describeProp}}.}

\item{factor_fn}{The method used to describe factors, see \code{\link{describeFactors}}.}

\item{show_all_values}{Show all values in proportions. For factors with only two values
it is most sane to only show one option as the other one will just be a complement
to the first, i.e. we want to convey a proportion. For instance sex - if you know
gender then automatically you know the distribution of the other sex as it's 100 \% - other \%.
To choose which one you want to show then set the \code{default_ref} parameter.}

\item{hrzl_prop}{This is default FALSE and indicates
that the proportions are to be interpreted in a vertical manner.
If we want the data to be horizontal, i.e. the total should be shown
and then how these differ in the different groups then set this to TRUE.}

\item{add_total_col}{This adds a total column to the resulting table.
You can also specify if you want the total column "first" or "last"
in the column order.}

\item{total_col_show_perc}{This is by default true but if
requested the percentages are suppressed as this sometimes may be confusing.}

\item{use_units}{If the Hmisc package's units() function has been employed
it may be interesting to have a column at the far right that indicates the
unit measurement. If this column is specified then the total column will
appear before the units (if specified as last). You can also set the value to
\code{"name"} and the units will be added to the name as a parenthesis,
e.g. Age (years).}

\item{units_column_name}{The name of the units column. Used if use_units = TRUE}

\item{default_ref}{The default reference when dealing with proportions. When using
`dplyr` syntax (`tidyselect`) you can specify a named vector/list for each column name.}

\item{NEJMstyle}{Adds - no (\%) at the end to proportions}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable.}

\item{header_count}{Set to \code{TRUE} if you want to add a header count,
e.g. Smoking; No. 25 observations, where there is a new line after the
factor name. If you want a different text for the second line you can
specifically use the \code{\link[base]{sprintf}} formatting, e.g. "No. \%s patients".}

\item{missing_value}{Value that is substituted for empty cells. Defaults to "-"}

\item{names_of_missing}{Optional character vector containing the names of returned statistics,
in case all returned values for a given \code{by} level are missing. Defaults to NULL}
}
\value{
Returns \code{matrix} if a single value was provided, otherwise a \code{list}
 of matrices with the class \code{"Gmisc_getDescriptionStatsBy"}.
}
\description{
A function that returns a description statistic that can be used
for creating a publication "table 1" when you want it by groups.
The function identifies if the variable is a continuous, binary
or a factored variable. The format is inspired by NEJM, Lancet &
BMJ.
}
\section{Customizing statistics}{


You can specify what function that you want for statistic by providing a function
that takes two arguments \code{x} and \code{by} and returns a p-value. There are
a few functions already prepared for this see \code{\link{getPvalAnova}},
\code{\link{getPvalChiSq}}
\code{\link{getPvalFisher}}
\code{\link{getPvalKruskal}}
\code{\link{getPvalWilcox}}.
The default functions used are \code{getPvalFisher} and \code{getPvalWilcox} (unless the by
argument has more than three unique levels where it defaults to \code{getPvalAnova}).

If you want the function to select functions depending on the type of input
you can provide a list with the names \code{'continuous'}, \code{'proportion'}, \code{'factor'} and
the function will choose accordingly. If you fail to define a certain category
it will default to the above.

You can also use a custom function that returns a string with the attribute \code{'colname'}
set that will be appended to the results instead of the p-value column.
}

\examples{
library(magrittr)
library(dplyr)
library(htmlTable)

data(mtcars)
mtcars \%<>\%
  mutate(am = factor(am, levels = 0:1, labels = c("Automatic", "Manual")),
         vs = factor(vs, levels = 0:1, labels = c("V-shaped", "straight")),
         drat_prop = drat > median(drat),
         drat_prop = factor(drat_prop,
                            levels = c(FALSE, TRUE),
                            labels = c("High ratio", "Low ratio")),
         carb_prop = carb > 2,
         carb_prop = factor(carb_prop,
                            levels = c(FALSE, TRUE),
                            labels = c("&le; 2", "&gt; 2")),
         across(c(gear, carb, cyl), factor))

# A simple bare-bone example
mtcars \%>\%
  getDescriptionStatsBy(`Miles per gallon` = mpg,
                        Weight = wt,
                        `Carborators &le; 2` = carb_prop,
                        by = am) \%>\%
  htmlTable(caption  = "Basic continuous stats from the mtcars dataset")
invisible(readline(prompt = "Press [enter] to continue"))

# For labeling & units we use set_column_labels/set_column_unit that use
# the Hmisc package annotation functions
mtcars \%<>\%
  set_column_labels(am = "Transmission",
                    mpg = "Gas",
                    wt = "Weight",
                    gear = "Gears",
                    disp = "Displacement",
                    vs = "Engine type",
                    drat_prop = "Rear axel ratio",
                    carb_prop = "Carburetors") \%>\%
  set_column_units(mpg = "Miles/(US) gallon",
                   wt = "10<sup>3</sup> lbs",
                   disp = "cu.in.")

mtcars \%>\%
  getDescriptionStatsBy(mpg,
                        wt,
                        `Gear&dagger;` = gear,
                        drat_prop,
                        carb_prop,
                        vs,
                        by = am,
                        header_count = TRUE,
                        use_units = TRUE,
                        show_all_values = TRUE)  \%>\%
  addHtmlTableStyle(pos.caption = "bottom") \%>\%
  htmlTable(caption  = "Stats from the mtcars dataset",
            tfoot = "&dagger; Number of forward gears")
invisible(readline(prompt = "Press [enter] to continue"))

# Using the default parameter we can
mtcars \%>\%
  getDescriptionStatsBy(mpg,
                        wt,
                        `Gear&dagger;` = gear,
                        drat_prop,
                        carb_prop,
                        vs,
                        by = am,
                        header_count = TRUE,
                        use_units = TRUE,
                        default_ref = c(drat_prop = "Low ratio",
                                        carb_prop = "&gt; 2"))  \%>\%
  addHtmlTableStyle(pos.caption = "bottom") \%>\%
  htmlTable(caption  = "Stats from the mtcars dataset",
            tfoot = "&dagger; Number of forward gears")
invisible(readline(prompt = "Press [enter] to continue"))

# We can also use lists
tll <- list()
tll[["Gear (3 to 5)"]] <- getDescriptionStatsBy(mtcars$gear, mtcars$am)
tll <- c(tll,
         list(getDescriptionStatsBy(mtcars$disp, mtcars$am)))

mergeDesc(tll,
          htmlTable_args = list(caption  = "Factored variables")) \%>\%
  htmlTable::addHtmlTableStyle(css.rgroup = "")
invisible(readline(prompt = "Press [enter] to continue"))

tl_no_units <- list()
tl_no_units[["Gas (mile/gallons)"]] <-
  getDescriptionStatsBy(mtcars$mpg, mtcars$am,
                        header_count = TRUE)
tl_no_units[["Weight (10<sup>3</sup> kg)"]] <-
  getDescriptionStatsBy(mtcars$wt, mtcars$am,
                        header_count = TRUE)
mergeDesc(tl_no_units,
          tll) \%>\%
  htmlTable::addHtmlTableStyle(css.rgroup = "")
invisible(readline(prompt = "Press [enter] to continue"))

# Other settings
mtcars$mpg[sample(1:NROW(mtcars), size = 5)] <- NA
getDescriptionStatsBy(mtcars$mpg,
                      mtcars$am,
                      statistics = TRUE)
invisible(readline(prompt = "Press [enter] to continue"))

# Do the horizontal version
getDescriptionStatsBy(mtcars$gear,
                      mtcars$am,
                      statistics = TRUE,
                      hrzl_prop = TRUE)
invisible(readline(prompt = "Press [enter] to continue"))

mtcars$wt_with_missing <- mtcars$wt
mtcars$wt_with_missing[sample(1:NROW(mtcars), size = 8)] <- NA
getDescriptionStatsBy(mtcars$wt_with_missing, mtcars$am, statistics = TRUE,
                      hrzl_prop = TRUE, total_col_show_perc = FALSE)
invisible(readline(prompt = "Press [enter] to continue"))

\dontrun{
  ## There is also a LaTeX wrapper
  tll <- list(
    getDescriptionStatsBy(mtcars$gear, mtcars$am),
    getDescriptionStatsBy(mtcars$col, mtcars$am))

  latex(mergeDesc(tll),
        caption  = "Factored variables",
        file = "")
}
}
\seealso{
Other descriptive functions:
\code{\link[=describeFactors]{describeFactors()}},
\code{\link[=describeMean]{describeMean()}},
\code{\link[=describeMedian]{describeMedian()}},
\code{\link[=describeProp]{describeProp()}},
\code{\link[=getPvalWilcox]{getPvalWilcox()}}
}
\concept{descriptive functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_prConvert2Coords.R
\name{prConvert2Coords}
\alias{prConvert2Coords}
\title{Converts an object to coordinates}
\usage{
prConvert2Coords(obj)
}
\arguments{
\item{obj}{A single or a \code{list} of objects. Supported element types include:
\itemize{
\item \code{boxGrob} or \code{boxPropGrob}
\item output from \code{coords}
\item a \code{unit} (grid) or a numeric value coercible to an \code{npc} unit
}

When a \code{list} is provided the function recursively converts each element and
returns bounding coordinates that encompass all elements. The returned
units are in \code{npc} so the coordinates resize with the viewport.}
}
\value{
A \code{list} with the points produced by \code{coords}. For list inputs the
function returns merged bounding coordinates (class \code{box_coords}) with
units in \code{npc}.
}
\description{
Sometimes we have an object that can be either a box,
a coordinate, a unit or a numerical value and all we
want is a \code{list} of coordinates that we can use for
calculating distance, alignment and other things.
}
\details{
When given a list, \code{prConvert2Coords} computes the min/max of
edges (or uses center +/- half sizes when edges are missing) to create a
merged bounding box. Any additional coordinate names present on elements
are merged heuristically (for example, \verb{left*} -> min, \verb{right*} -> max,
otherwise averaged).
}
\examples{
box1 <- boxGrob("A", x = .2, y = .8)
box2 <- boxGrob("B", x = .6, y = .4)
Gmisc:::prConvert2Coords(list(box1, box2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionClass_helpers.R
\name{prTcPlotBoxColumn}
\alias{prTcPlotBoxColumn}
\title{Plots a column of boxes}
\usage{
prTcPlotBoxColumn(
  box_positions,
  proportions,
  fill,
  txt,
  txt_clr,
  txt_gpar,
  cex
)
}
\arguments{
\item{box_positions}{The box positions}

\item{proportions}{The box vertical proportions (needed for the prop attribute)}

\item{fill}{The fill colors of length equal to the proportions}

\item{txt}{The texts of length equal to the proportions}

\item{txt_clr}{The text colors of length equal to the proportions}

\item{txt_gpar}{A list of gpar object for the text box}

\item{cex}{The fontsize multiplier}
}
\value{
\code{void}
}
\description{
Takes a set of box settings and plots them.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSvdMostInfluential.R
\name{getSvdMostInfluential}
\alias{getSvdMostInfluential}
\title{Gets the maximum contributor variables from svd()}
\usage{
getSvdMostInfluential(
  mtrx,
  quantile,
  similarity_threshold,
  plot_selection = TRUE,
  plot_threshold = 0.05,
  varnames = NULL
)
}
\arguments{
\item{mtrx}{A matrix or data frame with the variables. Note: if it contains missing
variables make sure to impute prior to this function as the \code{\link[base]{svd}} can't
handle missing values.}

\item{quantile}{The SVD D-matrix gives an estimate for the amount that is explained.
This parameter is used for selecting the columns that have that quantile
of explanation.}

\item{similarity_threshold}{A quantile for how close other variables have to be in value to
maximum contributor of that particular column. If you only want the maximum value
then set this value to 1.}

\item{plot_selection}{As this is all about variable exploring it is often interesting
to see how the variables were distributed among the vectors}

\item{plot_threshold}{The threshold of the plotted bars, measured as
percent explained by the D-matrix. By default it is set to 0.05.}

\item{varnames}{A vector with alternative names to the colnames}
}
\value{
Returns a list with vector with the column numbers
 that were picked in the "most_influential" variable and the
 svd caluclation in the "svd"
}
\description{
This function is inspired by Jeff Leeks Data Analysis course where
he suggests that one way to use the \code{\link[base]{svd}} is to look
at the most influential rows for first columns in the V matrix.
}
\details{
This function expands on that idea and adds the option of choosing
more than just the most contributing variable for each row. For instance
two variables may have a major impact on a certain component where the second
variable has 95% of the maximum variables value, since this variable is probably
important in that particular component it makes sense to include it
in the selection.

It is of course useful when you have many continuous variables and you want
to determine a subgroup to look at, i.e. finding the needle in the haystack.
}
\examples{
org_par <- par(ask = TRUE)
set.seed(1345)
# Simulate data with a pattern
dataMatrix <- matrix(rnorm(15 * 160), ncol = 15)
colnames(dataMatrix) <- c(
    paste("Pos.3:", 1:3, sep = " #"),
    paste("Neg.Decr:", 4:6, sep = " #"),
    paste("No pattern:", 7:8, sep = " #"),
    paste("Pos.Incr:", 9:11, sep = " #"),
    paste("No pattern:", 12:15, sep = " #"))

for (i in 1:nrow(dataMatrix)) {
  # flip a coin
  coinFlip1 <- rbinom(1, size = 1, prob = 0.5)
  coinFlip2 <- rbinom(1, size = 1, prob = 0.5)
  coinFlip3 <- rbinom(1, size = 1, prob = 0.5)

  # if coin is heads add a common pattern to that row
  if (coinFlip1) {
    cols <- grep("Pos.3", colnames(dataMatrix))
    dataMatrix[i, cols] <- dataMatrix[i, cols] + 3
  }

  if (coinFlip2) {
    cols <- grep("Neg.Decr", colnames(dataMatrix))
    dataMatrix[i, cols] <- dataMatrix[i, cols] - seq(from = 5, to = 15, length.out = length(cols))
  }

  if (coinFlip3) {
    cols <- grep("Pos.Incr", colnames(dataMatrix))
    dataMatrix[i, cols] <- dataMatrix[i, cols] + seq(from = 3, to = 15, length.out = length(cols))
  }
}

# Illustrate data
heatmap(dataMatrix, Colv = NA, Rowv = NA, margins = c(7, 2), labRow = "")

svd_out <- svd(scale(dataMatrix))

library(lattice)
b_clr <- c("steelblue", "darkred")
key <- simpleKey(
  rectangles = TRUE, space = "top", points = FALSE,
  text = c("Positive", "Negative")
)
key$rectangles$col <- b_clr

b1 <- barchart(as.table(svd_out$v[, 1]),
               main = "First column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 1] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

b2 <- barchart(as.table(svd_out$v[, 2]),
               main = "Second column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 2] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

b3 <- barchart(as.table(svd_out$v[, 3]),
               main = "Third column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 3] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

b4 <- barchart(as.table(svd_out$v[, 4]),
               main = "Fourth column",
               horizontal = FALSE, col = ifelse(svd_out$v[, 4] > 0,
                                                b_clr[1], b_clr[2]
               ),
               ylab = "Impact value",
               scales = list(x = list(rot = 55, labels = colnames(dataMatrix), cex = 1.1)),
               key = key
)

# Note that the fourth has the no pattern columns as the
# chosen pattern, probably partly because of the previous
# patterns already had been identified
print(b1, position = c(0, 0.5, .5, 1), more = TRUE)
print(b2, position = c(0.5, 0.5, 1, 1), more = TRUE)
print(b3, position = c(0, 0, .5, .5), more = TRUE)
print(b4, position = c(0.5, 0, 1, .5))

# Let's look at how well the SVD identifies
# the most influential columns
getSvdMostInfluential(dataMatrix,
                      quantile = .8,
                      similarity_threshold = .9,
                      plot_threshold = .05,
                      plot_selection = TRUE)
par(org_par)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPval.R
\name{getPvalWilcox}
\alias{getPvalWilcox}
\alias{getPvalAnova}
\alias{getPvalFisher}
\alias{getPvalChiSq}
\alias{getPvalKruskal}
\title{P-value extractors for \code{\link{getDescriptionStatsBy}}}
\usage{
getPvalWilcox(x, by)

getPvalAnova(x, by)

getPvalFisher(x, by)

getPvalChiSq(x, by)

getPvalKruskal(x, by)
}
\arguments{
\item{x}{The main variable of interest}

\item{by}{The variable for the stratification}
}
\value{
\code{numeric} Returns the p-value from that particular test
}
\description{
These functions are the base functions for getting the description
p-values. You can provide your own functions but all functions should
take two arguments and return a p-value (numeric, non-formatted)
}
\section{getPvalWilcox}{


Performs a two-sample two-sided Wilcoxon test (also known as the Mann-Whitney test),
see \code{\link[stats]{wilcox.test}}.
}

\section{getPvalAnova}{


Performs a standard Analysis of Variance model
through \code{\link[stats]{anova}(\link[stats]{lm}(x ~ by))}
}

\section{getPvalFisher}{

Performs Fisher's exact test through the \code{\link[stats]{fisher.test}}.
}

\section{getPvalChiSq}{


Performs a standard Chi-Squares analysis
through \code{\link[stats]{chisq.test}}
}

\section{getPvalKruskal}{


Performs a  Kruskal-Wallis rank sum test
through \code{\link[stats]{kruskal.test}}
}

\examples{
set.seed(123)
getPvalFisher(
  sample(letters[1:3], size = 100, replace = TRUE),
  sample(LETTERS[1:3], size = 100, replace = TRUE)
)
getPvalWilcox(
  rnorm(100),
  sample(LETTERS[1:2], size = 100, replace = TRUE)
)
}
\seealso{
Other descriptive functions:
\code{\link[=describeFactors]{describeFactors()}},
\code{\link[=describeMean]{describeMean()}},
\code{\link[=describeMedian]{describeMedian()}},
\code{\link[=describeProp]{describeProp()}},
\code{\link[=getDescriptionStatsBy]{getDescriptionStatsBy()}}
}
\concept{descriptive functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/private_functions.R
\name{prGetTextGrobCex}
\alias{prGetTextGrobCex}
\title{Just a simple acces to the gp$cex parameter}
\usage{
prGetTextGrobCex(x)
}
\arguments{
\item{x}{The text-grob of interest}
}
\value{
\code{numeric} The cex value, 1 if no cex was present
}
\description{
Just a simple acces to the gp$cex parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gmisc.R
\docType{package}
\name{Gmisc-package}
\alias{Gmisc}
\alias{Gmisc-package}
\title{Collection of functions for plotting relations, generating tables, and more.}
\description{
This is a collection of functions that I've found useful in my research.
The package is inspired by Frank Harrell's \pkg{Hmisc} package. The main focus
is on tables, plots, and \pkg{knitr}-integration.
}
\section{Awesome tables}{


The \code{\link{getDescriptionStatsBy}} is a straightforward function that
aims at helping you to generate a descriptive table stratified by different
variables. In other words, the function returns everything you need for generating
a \emph{Table 1} ready for publication. This function is accompanied by the
\code{\link{describeMean}}, \code{\link{describeMedian}}, \code{\link{describeProp}},
and \code{\link{describeFactors}} functions.

The \code{\link{mergeDesc}} allows you to merge a set of outputs \code{\link{getDescriptionStatsBy}}
into a \code{\link[htmlTable]{htmlTable}} with the rgroup arguments automatically
generated, see \code{vignette("descriptives", package = "Gmisc")} for a
detailed workflow description.
}

\section{Some fancy plots}{


The transition plot function, \code{\link{transitionPlot}}, is for
descriptive purposes. It tries to illustrate the size of change
between one state and the next, i.e. a transition. This is
basically a graph of based upon \code{table(var1, var2)}.

The \href{https://en.wikipedia.org/wiki/Singular_value_decomposition}{Singular value decomposition}
is a common method for reducing the number of variables. Unfortunately
this compression can reduce the interpretability of the model. The \code{\link{getSvdMostInfluential}}
function tries to remedy that by identifying the most influential
elements from the \code{V}-matrix.
}

\section{Other stuff}{


The \code{\link{insertRowAndKeepAttr}} simply adds a row while remembering
all the attributes previously set by using the \code{\link{copyAllNewAttributes}}.
The \code{\link{mergeLists}} tries to merge lists that do not have identical
elements.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://gforge.se}
  \item Report bugs at \url{https://github.com/gforge/Gmisc/issues}
}

}
\author{
\strong{Maintainer}: Max Gordon \email{max@gforge.se} (\href{https://orcid.org/0000-0002-8080-5815}{ORCID})

Authors:
\itemize{
  \item Max Gordon \email{max@gforge.se} (\href{https://orcid.org/0000-0002-8080-5815}{ORCID})
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot_helpers.R
\name{prTpGetBoxPropClr}
\alias{prTpGetBoxPropClr}
\title{Gets the color for the given box}
\usage{
prTpGetBoxPropClr(clr, no_boxes, lengthOneOK = FALSE)
}
\arguments{
\item{clr}{The color of the boxes, either
a matrix, a vector of length two, or a single color}

\item{no_boxes}{The number of boxes}

\item{lengthOneOK}{If it is ok to have only one color}
}
\value{
\code{matrix} With two columns and no_boxes rows.
 If function fails then return \code{NULL}
}
\description{
Gets the color for the given box
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/time2spanTxt.R
\name{time2spanTxt}
\alias{time2spanTxt}
\title{A dense time-span text}
\usage{
time2spanTxt(
  times,
  day_month_glue_txt = getOption("Gmisc_time2spanTxt_day_month", default =
    "{mday(time)} {month(time, label = TRUE)}"),
  full_year_format = getOption("Gmisc_time2spanTxt_full_year", default =
    "{mday(time)} {month(time, label = TRUE)} {year(time)}"),
  start_stop_glue_txt = getOption("Gmisc_time2spanTxt_template", default =
    "{start} to {stop}")
)
}
\arguments{
\item{times}{The dates or POSIX timestamps to used for time span}

\item{day_month_glue_txt}{The \code{\link[glue]{glue}} string to format days and months with \code{time} as the time input}

\item{full_year_format}{The \code{\link[glue]{glue}} string to format the full year with \code{time} as the time input}

\item{start_stop_glue_txt}{The string used in the \code{\link[glue]{glue}} for putting the
start and stop dates together into one string}
}
\value{
\code{string} A string describing the time span
}
\description{
When adding a time span text we often don't want to write \emph{3 jun - 10 jun} but
shorten it to \emph{3 - 10 jun} while retaining month and year info only if the span
crosses between months or years.
}
\details{
There are options that can be set using the \code{\link[base:options]{options}}:
\itemize{
\item \code{Gmisc_time2spanTxt_day_month} The date with day + month as formatted by \code{\link[glue]{glue}} where the time is passed as \code{time}.
\item \code{Gmisc_time2spanTxt_full_year} The full date with day + month + year as formatted by \code{\link[glue]{glue}} where the time is passed as \code{time}.
\item \code{Gmisc_time2spanTxt_template} The merge of the stop & start elements using \code{\link[glue]{glue}}.
}
}
\examples{
time2spanTxt(as.POSIXct(c("2020-01-02", "2020-03-01", NA)))
# 2 jan to 1 mar

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeLists.R
\name{mergeLists_internal}
\alias{mergeLists_internal}
\title{A helper function for mergeLists that does the actual work.}
\usage{
mergeLists_internal(o_element, n_element)
}
\arguments{
\item{o_element}{Old element}

\item{n_element}{New element to add to the old element}
}
\value{
list with the two merged
}
\description{
A helper function for mergeLists that does the actual work.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prFactorDescs.R
\name{prFactorDescs}
\alias{prFactorDescs}
\title{Helper to [getDescriptionStatsBy()]}
\usage{
prFactorDescs(
  x,
  by,
  factor_fn,
  hrzl_prop,
  html,
  digits,
  digits.nonzero,
  numbers_first,
  useNA,
  useNA.digits,
  percentage_sign,
  missing_value,
  names_of_missing
)
}
\arguments{
\item{x}{If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
If it is a single variable it will be the core value that want the statistics for.
In the print this is equivalent to the output of this function.}

\item{by}{The variable that you want to split into different columns}

\item{factor_fn}{The method used to describe factors, see \code{\link{describeFactors}}.}

\item{hrzl_prop}{This is default FALSE and indicates
that the proportions are to be interpreted in a vertical manner.
If we want the data to be horizontal, i.e. the total should be shown
and then how these differ in the different groups then set this to TRUE.}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{digits}{The number of decimals used}

\item{digits.nonzero}{The number of decimals used for values that are close to zero}

\item{numbers_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable.}

\item{missing_value}{Value that is substituted for empty cells. Defaults to "-"}

\item{names_of_missing}{Optional character vector containing the names of returned statistics,
in case all returned values for a given \code{by} level are missing. Defaults to NULL}
}
\value{
A [base::by] list
}
\description{
Helper to [getDescriptionStatsBy()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertShowMissing.R
\name{convertShowMissing}
\alias{convertShowMissing}
\title{A function for converting a show_missing variable.}
\usage{
convertShowMissing(show_missing)
}
\arguments{
\item{show_missing}{Boolean or "no", "ifany", "always"}
}
\value{
string
}
\description{
The variable is supposed to be directly compatible with
\code{\link[base]{table}}(..., useNA = show_missing). It throws an error
if not compatible. It is mostly useful for custom describe functions.
}
\details{
\emph{Deprecated:} This function will be deprecated
as all functions now use the useNA style in order to comply
with standard R naming.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prPropDescs.R
\name{prPropDescs}
\alias{prPropDescs}
\title{Helper to [getDescriptionStatsBy()]}
\usage{
prPropDescs(
  x,
  by,
  name,
  default_ref,
  prop_fn,
  html,
  digits,
  digits.nonzero,
  numbers_first,
  useNA,
  useNA.digits,
  percentage_sign,
  missing_value,
  names_of_missing,
  NEJMstyle
)
}
\arguments{
\item{x}{If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
If it is a single variable it will be the core value that want the statistics for.
In the print this is equivalent to the output of this function.}

\item{by}{The variable that you want to split into different columns}

\item{name}{The name of the row}

\item{default_ref}{The default reference when dealing with proportions. When using
`dplyr` syntax (`tidyselect`) you can specify a named vector/list for each column name.}

\item{prop_fn}{The method used to describe proportions, see \code{\link{describeProp}}.}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{digits}{The number of decimals used}

\item{digits.nonzero}{The number of decimals used for values that are close to zero}

\item{numbers_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable.}

\item{missing_value}{Value that is substituted for empty cells. Defaults to "-"}

\item{names_of_missing}{Optional character vector containing the names of returned statistics,
in case all returned values for a given \code{by} level are missing. Defaults to NULL}

\item{NEJMstyle}{Adds - no (\%) at the end to proportions}
}
\value{
A [base::by] list
}
\description{
Helper to [getDescriptionStatsBy()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prGetDescHeader.R
\name{prGetDescHeader}
\alias{prGetDescHeader}
\title{Retrieve basic description stats by header}
\usage{
prGetDescHeader(by, html, header_count, already_table_format = FALSE)
}
\arguments{
\item{by}{The variable that you want to split into different columns}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{header_count}{Set to \code{TRUE} if you want to add a header count,
e.g. Smoking; No. 25 observations, where there is a new line after the
factor name. If you want a different text for the second line you can
specifically use the \code{\link[base]{sprintf}} formatting, e.g. "No. \%s patients".}

\item{already_table_format}{Just a boolean as we use this in the total column}
}
\value{
A vector with basic headers
}
\description{
Helper for [getDescriptionStatsBy] that retrieves the basic
header names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_column_units.R
\name{set_column_units}
\alias{set_column_units}
\title{Add [Hmisc::unit()] to multiple columns}
\usage{
set_column_units(x, ...)
}
\arguments{
\item{x}{The data frame that we want to define units on}

\item{...}{Variable names with their intended unit, e.g. `hp = "Hp"`.}
}
\value{
The original data.frame
}
\description{
Add label attribute using `dplyr` syntax using the [Hmisc::unit()]
}
\examples{
library(magrittr)
data(mtcars)
mtcars_with_units <- mtcars \%>\%
  set_column_units(wt = "1000 lbs")
Hmisc::units(mtcars_with_units$wt)
}
\seealso{
Other Hmisc helpers:
\code{\link[=set_column_labels]{set_column_labels()}}
}
\concept{Hmisc helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot_helpers.R
\name{prTpGetBoxSizedTextGrob}
\alias{prTpGetBoxSizedTextGrob}
\title{Gets the text size for the box}
\usage{
prTpGetBoxSizedTextGrob(
  txt,
  txt_clr,
  txt_cex,
  txt_gpar = NULL,
  force_cex = FALSE,
  ...
)
}
\arguments{
\item{txt}{The text}

\item{txt_clr}{The color of the text}

\item{txt_cex}{The font size}

\item{txt_gpar}{The [grid::gpar()] object that is overridden with col & cex by other parameters}

\item{force_cex}{If font size should be forced}

\item{...}{Other options}
}
\description{
Gets the text size for the box
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/descriptionStats.R
\name{describeProp}
\alias{describeProp}
\title{A function that returns a description proportion that contains
the number and the percentage}
\usage{
describeProp(
  x,
  html = TRUE,
  digits = 1,
  digits.nonzero = NA,
  number_first = TRUE,
  useNA = c("ifany", "no", "always"),
  useNA.digits = digits,
  default_ref = NULL,
  percentage_sign = TRUE,
  language = "en",
  ...
)
}
\arguments{
\item{x}{The variable that you want the statistics for}

\item{html}{If HTML compatible output should be used. If \code{FALSE}
it outputs LaTeX formatting}

\item{digits}{The number of decimals used}

\item{digits.nonzero}{The number of decimals used for values that are close to zero}

\item{number_first}{If the number should be given or if the percentage
should be presented first. The second is encapsulated in parentheses ().
This is only used together with the useNA variable.}

\item{useNA}{This indicates if missing should be added as a separate
row below all other. See \code{\link[base]{table}} for \code{useNA}-options.
\emph{Note:} defaults to ifany and not "no" as \code{\link[base]{table}} does.}

\item{useNA.digits}{The number of digits to use for the
missing percentage, defaults to the overall \code{digits}.}

\item{default_ref}{The default reference, either first,
the level name or a number within the levels. If left out
it defaults to the first value.}

\item{percentage_sign}{If you want to suppress the percentage sign you
can set this variable to FALSE. You can also choose something else that
the default \% if you so wish by setting this variable. Note, this is
only used when combined with the missing information.}

\item{language}{The ISO-639-1 two-letter code for the language of
interest. Currently only english is distinguished from the ISO
format using a ',' as the separator in the \code{\link[htmlTable]{txtInt}}
function.}

\item{...}{Passed on to \code{\link{describeFactors}}}
}
\value{
\code{string} A string formatted for either LaTeX or HTML
}
\description{
A function that returns a description proportion that contains
the number and the percentage
}
\examples{
describeProp(factor(sample(50, x = c("A", "B", NA), replace = TRUE)))
}
\seealso{
Other descriptive functions:
\code{\link[=describeFactors]{describeFactors()}},
\code{\link[=describeMean]{describeMean()}},
\code{\link[=describeMedian]{describeMedian()}},
\code{\link[=getDescriptionStatsBy]{getDescriptionStatsBy()}},
\code{\link[=getPvalWilcox]{getPvalWilcox()}}
}
\concept{descriptive functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeLists.R
\name{mergeLists}
\alias{mergeLists}
\title{Merging of multiple lists}
\usage{
mergeLists(
  ...,
  lapplyOutput = NULL,
  sortNames = getOption("Gmisc.mergeList.sort", default = TRUE)
)
}
\arguments{
\item{...}{Any number of lists that you want to merge}

\item{lapplyOutput}{The \code{\link[base]{lapply}} function outputs a number
of lists and this is for specifically merging all of those.}

\item{sortNames}{Set to false if you don't want the names to be sorted.
This can also be done via the option `Gmisc.mergeList.sort`.}
}
\value{
Returns a list with all the given lists.
}
\description{
The merge allows for a recursive component
where the lists are compared on the subelement.
If one does not contain that element it will get NA
in for those parameters.
}
\examples{
v1 <- list("a" = c(1, 2), b = "test 1", sublist = list(one = 20:21, two = 21:22))
v2 <- list("a" = c(3, 4), b = "test 2", sublist = list(one = 10:11, two = 11:12, three = 1:2))
mergeLists(v1, v2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docx_document.R
\name{prSetMaxWidth}
\alias{prSetMaxWidth}
\title{Updates the css max-width}
\usage{
prSetMaxWidth(max_width, css_file)
}
\arguments{
\item{max_width}{The new max-width css setting}

\item{css_file}{The CSS file name}
}
\value{
\code{string} The file name
}
\description{
Reads the file, changes the max-width
for the body element.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_s3_append.R
\name{append}
\alias{append}
\alias{append.default}
\alias{append.Gmisc_list_of_boxes}
\title{Append to list of boxes (S3)}
\usage{
append(x, values, after = length(x))

\method{append}{default}(x, values, after = length(x))

\method{append}{Gmisc_list_of_boxes}(x, values, after = length(x))
}
\arguments{
\item{x}{A `Gmisc_list_of_boxes` or `list`.}

\item{values}{The object(s) to append. If a single `boxGrob` is provided, it is wrapped in a list.}

\item{after}{Subscript, after which the values are to be appended.}
}
\value{
Updated list of boxes.
}
\description{
Append an element to a list of boxes, preserving attributes/class.
Masks `base::append` to support S3 dispatch.
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_distance.R
\name{distance}
\alias{distance}
\alias{print.Gmisc_unit}
\title{Get the distance between grid objects}
\usage{
distance(
  box1,
  box2,
  type = c("vertical", "horizontal", "euclidean"),
  half = FALSE,
  center = FALSE
)

\method{print}{Gmisc_unit}(x, ...)
}
\arguments{
\item{box1}{The first \code{\link{boxGrob}}. Can also be a \code{\link{coords}}
object, a \code{\link[grid]{unit}} or a numeric. The latter is evaluated to
a \code{unit} with \code{units="npc"}.}

\item{box2}{The second object to calculate the distance to. Same type as for \code{box1}.}

\item{type}{Whether we should retrieve the vertical, horizontal or euclidean distance}

\item{half}{If set to true it returns half the distance. This is convenient
when positioning boxes between each other.}

\item{center}{Calculate the distance from the center of each object}

\item{x}{A unit with from the \code{distance} function}

\item{...}{Passed on to print}
}
\value{
A \code{unit} in \code{"mm"} with an absolute value. The attribute
 \code{positive} indicates the direction of the value, i.e. if it is \code{TRUE} the
 distance was calculated from the first to the second, otherwise it is \code{FALSE}.
 For \code{euclidean} distance the \code{positive} attribute is \code{NA}. There is also the
 \code{from} and \code{to} attributes that has the coordinates that were used for the
 calculations, for \code{euclidean} distance this is \code{NA}.
}
\description{
Retrieves the distance between two boxes as absolute \code{"mm"} units. The function also
accepts \code{\link{coords}} objects as well as a \code{\link[grid]{unit}} or a numeric input.
}
\examples{
box1 <- boxGrob("A test box", y = .8)
box2 <- boxGrob("Another test box", y = .2)
distance(box1, box2, "v")
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prAddEmptyVals.R
\name{prAddEmptyVals}
\alias{prAddEmptyVals}
\title{Convert the by-list into a matrix compatible format}
\usage{
prAddEmptyVals(t, missing_value)
}
\arguments{
\item{t}{Output from [prNumericDescs], [prPropDescs], or [prFactorDescs].}

\item{missing_value}{Value that is substituted for empty cells. Defaults to "-"}
}
\value{
A fixed list
}
\description{
Helper for [getDescriptionStatsBy] that fixes empty values in matrix
so that they are compatible with the matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier_private_functions.R
\name{getVectorAngle}
\alias{getVectorAngle}
\title{Gets an angle}
\usage{
getVectorAngle(x, y, x_origo = NA, y_origo = NA)
}
\arguments{
\item{x}{The x point of the vector}

\item{y}{The y point of the vector}

\item{x_origo}{The x origin if other than 0}

\item{y_origo}{The y origin if other than 0}
}
\value{
angle in radians (see \code{\link{atan2}})
}
\description{
Uses a vector to get an angle by \code{\link{atan2}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_s3_append.R
\name{insert}
\alias{insert}
\alias{insert.default}
\alias{insert.Gmisc_list_of_boxes}
\title{Insert element into a list of boxes (S3)}
\usage{
insert(x, element, ..., after = NULL, before = NULL)

\method{insert}{default}(x, element, ..., after = NULL, before = NULL)

\method{insert}{Gmisc_list_of_boxes}(x, element, ..., after = NULL, before = NULL)
}
\arguments{
\item{x}{A `Gmisc_list_of_boxes`.}

\item{element}{The new box object to insert.}

\item{...}{Not used.}

\item{after}{The name or index of the box after which to insert.}

\item{before}{The name or index of the box before which to insert.}
}
\value{
The updated list of boxes with the new element inserted.
}
\description{
Inserts a box into a list of boxes and positions it relative to the
surrounding boxes.
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_boxPropGrob.R
\name{boxPropGrob}
\alias{boxPropGrob}
\title{Create a box with a color split}
\usage{
boxPropGrob(
  label,
  label_left,
  label_right,
  prop,
  y = unit(0.5, "npc"),
  x = unit(0.5, "npc"),
  width,
  height,
  just = "center",
  bjust = "center",
  txt_gp = getOption("boxPropGrobTxt", default = gpar(color = "black")),
  txt_left_gp = getOption("boxPropGrobLeftTxt", default = gpar(col = "black")),
  txt_right_gp = getOption("boxPropGrobRightTxt", default = gpar(col = "black")),
  box_left_gp = getOption("boxPropGrobLeft", default = gpar(fill = "#E6E8EF")),
  box_right_gp = getOption("boxPropGrobRight", default = gpar(fill = "#FFFDF6")),
  box_highlight_gp = getOption("boxPropGrobHighlight", default = gpar(fill = "#ffffff55",
    col = NA)),
  name = NULL
)
}
\arguments{
\item{label}{The label to print - should be a number, text or expression.}

\item{label_left}{The label for the left area}

\item{label_right}{The label for the right area}

\item{prop}{The proportion to split along}

\item{y}{The y position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.}

\item{x}{The x position to put the box at. Can be either in \code{npc} (i.e. 0-1) or a \code{\link[grid]{unit}}.}

\item{width}{The box automatically adapts the size but you can force by specifying the width}

\item{height}{The box automatically adapts the size but you can force by specifying the height}

\item{just}{The justification for the text: left, center or right.}

\item{bjust}{The justification for the box: left, center, right, top or bottom.
See the \code{just} option for the \code{\link[grid]{viewport}}}

\item{txt_gp}{The \code{\link[grid]{gpar}} style to apply to the text. Set \code{boxPropGrobTxt} option
if you want to customize all the boxes at once.}

\item{txt_left_gp}{The \code{\link[grid]{gpar}} style to apply to the left text. Set
\code{boxPropGrobLeftTxt} option if you want to customize all the boxes at once.}

\item{txt_right_gp}{The \code{\link[grid]{gpar}} style to apply to the right text. Set
\code{boxPropGrobRightTxt} option if you want to customize all the boxes at once.}

\item{box_left_gp}{The \code{\link[grid]{gpar}} style to apply to the left box. Set
\code{boxPropGrobLeft} option if you want to customize all the boxes at once.}

\item{box_right_gp}{The \code{\link[grid]{gpar}} style to apply to the right box. Set
\code{boxPropGrobRight} option if you want to customize all the boxes at once.}

\item{box_highlight_gp}{The \code{\link[grid]{gpar}} style to apply to the background
of the main label. Set \code{boxPropGrobHighlight} option if you want to customize
all the boxes at once.}

\item{name}{a character identifier for the \code{grob}. Used to find the \code{grob} on the display
list and/or as a child of another grob.}
}
\value{
A box grob
}
\description{
Creates a grob box with text inside it and a color split in the
horizontal axes that allow indicating different proportions. The
box can also have a title that spanse the two color areas and
that has its own background.
}
\examples{
library(grid)
grid::grid.newpage()
boxPropGrob("Main label", "Left text", "Right text", prop = .3)
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_prCreateBoxCoordinates.R
\name{prCreateBoxCoordinates}
\alias{prCreateBoxCoordinates}
\title{Creates coordinates for box}
\usage{
prCreateBoxCoordinates(viewport_data, extra_coordinate_functions = NULL)
}
\arguments{
\item{viewport_data}{The arguments that will be used for generating the \code{viewport}}

\item{extra_coordinate_functions}{A list with named functions if we want additional
parameters}
}
\value{
\code{list} of class \code{coords}
}
\description{
Creates coordinates for box
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_helpers.R
\name{prDescGetAndValidateDefaultRef}
\alias{prDescGetAndValidateDefaultRef}
\title{A helper function for the description stats}
\usage{
prDescGetAndValidateDefaultRef(x, default_ref)
}
\arguments{
\item{x}{The variable of interest with the levels}

\item{default_ref}{The default reference, either first,
the level name or a number within the levels. If left out
it defaults to the first value.}
}
\value{
\code{integer} The level number of interest
}
\description{
A helper function for the description stats
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_s3_move.R
\name{move}
\alias{move}
\alias{move.default}
\alias{move.Gmisc_list_of_boxes}
\title{Move boxes (S3)}
\usage{
move(element, ...)

\method{move}{default}(element, ...)

\method{move}{Gmisc_list_of_boxes}(element, subelement = NULL, ...)
}
\arguments{
\item{element}{A `list` of boxes.}

\item{...}{Arguments passed to [`moveBox`].}

\item{subelement}{The name of the element to move (maps to `subelement` in `moveBox`)}
}
\value{
The moved list of boxes.
}
\description{
Move a box or list of boxes. A wrapper for [`moveBox`] that supports
piping of `Gmisc_list_of_boxes` while preserving attributes.
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_move.R
\name{moveBox}
\alias{moveBox}
\title{Move a boxGrob}
\usage{
moveBox(
  element,
  x = NULL,
  y = NULL,
  space = c("absolute", "relative"),
  just = NULL,
  subelement = NULL
)
}
\arguments{
\item{element}{A \code{\link{boxGrob}}/\code{\link{boxPropGrob}} object.}

\item{x}{A \code{\link[grid:unit]{unit}} element or a numeric that can be converted to a \code{npc} unit object.}

\item{y}{A \code{\link[grid:unit]{unit}} element or a numeric that can be converted to a \code{npc} unit object.}

\item{space}{We can provide \code{absolute} that confers the box absolute position within the parent
\code{\link[grid:viewport]{viewport}}. If \code{relative} the movement is related to the current position.}

\item{just}{The justification of an argument as used by \code{\link[grid:viewport]{viewport}} some tiny differences:
(1) you only want to change the justification in the vertical direction you can retain the
existing justification by using \code{NA}, e.g. \code{c(NA, 'top')}, (2) if you specify only one string
and that string is either \code{top} or \code{bottom} it will assume vertical justification.}

\item{subelement}{If a \code{list} of boxes is provided, this can be a name, index, a deep path (e.g. \code{c("detail", 1)}) or a
vector of names/indices to target a single nested element to move. You can also pass
multiple targets as a list of paths (e.g. \code{list(c("detail", 1), c("followup", 1))}).
The function will return the original list with the targeted element(s) replaced by their moved version(s).}
}
\value{
The element with updated viewport and coordinates
}
\description{
Moves a \code{\link{boxGrob}}/\code{\link{boxPropGrob}} by modifying it's \code{\link[grid:viewport]{viewport}}.
This can be useful if you want to create a series of boxes whose position are relative
to each other and depend on each box's width/height.
}
\examples{
library(grid)
grid.newpage()

pushViewport(viewport(y = 0.85, height = unit(.3, "npc")))
grid.rect(gp = gpar(col = "steelblue"))
grid.text("Example 1", y = .9, x = unit(10, "mm"), just = "left")
box <- boxGrob("A simple box", x = .5, y = .5) # Start at the middle
moveBox(box, x = -.2, space = "relative") # Move to the left
popViewport()


## Advanced example (skipped during R CMD check)
## Not run:
# Advanced example: create a nested list of treatment boxes, spread them horizontally,
# then move a single nested element (`Ibuprofen`) using `subelement` via a pipe.
pushViewport(viewport(y = 0.35, height = unit(.7, "npc") - unit(2, "mm")))
grid.rect(gp = gpar(col = "lightgreen"))

boxes <- list(
    population = boxGrob("Population"),
    treatment = list(
        Ibuprofen = boxGrob("Ibuprofen"),
        Paracetamol = boxGrob("Paracetamol"),
        Aspirin = boxGrob("Aspirin")
    ),
    followup = paste("Follow-up 4 weeks:",
        " - EQ-5D 5L",
        " - Lab",
        sep = "\n"
    ) |>
        boxGrob(just = "left")
) |>
    alignHorizontal(position = "center") |>
    spreadVertical() |>
    spreadHorizontal(
        from = unit(0.1, "npc"),
        to = unit(0.9, "npc"),
        type = "center",
        subelement = "treatment"
    ) |>
    moveBox(
        y = 0.1,
        space = "relative",
        subelement = c("treatment", "Ibuprofen")
    )

boxes
connectGrob(boxes$population, boxes$treatment, type = "N")
connectGrob(boxes$treatment, boxes$followup, type = "N")
## End(Not run)
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot_helpers.R
\name{prTpGetColors}
\alias{prTpGetColors}
\title{Gets a set of colors or just one color}
\usage{
prTpGetColors(colors, proportion, space = NULL, no = 101)
}
\arguments{
\item{colors}{A set of min. two colors that is used
for \code{\link[grDevices:colorRamp]{colorRampPalette}}.}

\item{proportion}{A proportion or a set of proportions
between 0 and 1. If you leave this out then the full color
span will be returned.}

\item{space}{If there is little or no difference
at the low/high proportions of the spectrum then it
can be of interest to focus the color change to the center
leaving the tails constant}

\item{no}{The color resolution to use}
}
\value{
\code{character} The function can return both single colors
 or multiple colors as character vector (see \code{\link[grDevices]{rgb}})
}
\description{
Used in order to illustrate the mix between two
proportions in the \code{\link{transitionPlot}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_coords.R
\name{coords}
\alias{coords}
\title{Get the box coordinates}
\usage{
coords(box)
}
\arguments{
\item{box}{The \code{\link{boxGrob}} or \code{\link{boxPropGrob}}}
}
\value{
A list with the coordinates
}
\description{
Retrieves the boxes \code{"coords"} attribute.
}
\examples{
box <- boxGrob("A test box")
coords(box)
}
\seealso{
Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=connectGrob]{connectGrob()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionClass.R
\docType{class}
\name{Transition-class}
\alias{Transition-class}
\alias{Transition}
\title{A reference class for generating transition plots}
\description{
This class simplifies the creating of transition plots. It also
allows for advanced multi-column transitions.
}
\details{
Transition plots are a type of \emph{Sankey diagrams}. These are a specific type
of flow diagram, in which the width of the arrows is shown proportionally
to the flow quantity. See \href{https://en.wikipedia.org/wiki/Sankey_diagram}{Wikipedia}
for details.
}
\section{Fields}{

\describe{
\item{\code{id}}{Optional id. The render uses named viewports that require a unique id if multiple transition plots
are combined. In order to avoid having overlapping graphs we need to generate a unique id for each viewport
and thus this variable exists. If left empty it will create a counter that is stored in the \code{\link[base]{options}}
(\code{"Gmisc.transitionClassCounter"}) and each viewport will have the name preceded with \code{tc_[0-9]+}. Set this
if you intend to use \code{\link[grid:viewports]{seekViewport}}.}

\item{\code{transitions}}{This is a >= 3 dimensional array with the transitions. Should not be directly accessed.}

\item{\code{box_width}}{The box width}

\item{\code{box_txt}}{The texts of each box}

\item{\code{box_label}}{Box labels}

\item{\code{box_label_pos}}{The label's positions, either "top"/"bottom"}

\item{\code{box_label_cex}}{The size of the box labels}

\item{\code{box_cex}}{The font-size multiplier for the text within the boxes}

\item{\code{arrow_type}}{The type of arrow to use, defaults to "gradient", but can also be "simple".
The corresponding functions are \code{\link{bezierArrowGradient}}, and
\code{\link{bezierArrowSmpl}}. \emph{Note} The \code{bezierGrob} has been deprecated
as it is no longer faster than the bezier arrows and there is a difference in design.}

\item{\code{arrow_clr}}{The arrow color}

\item{\code{arrow_rez}}{The resolution of the arrow}

\item{\code{vertical_space}}{The space between the boxes}

\item{\code{fill_clr}}{The box fill color}

\item{\code{clr_bar}}{Shows a color bar if there are proportions. Can be \code{"none"}, \code{"top"}, \code{"bottom"}}

\item{\code{clr_bar_clrs}}{Extracts the colors for the color bar from the \code{fill_clr} if none is provided}

\item{\code{clr_bar_cex}}{The size of the ticks in the color bar}

\item{\code{clr_bar_subspace}}{If little or no difference exists at the low/high proportions of
the spectrum then it can be of interest to focus the color change to the center leaving the tails constant}

\item{\code{clr_bar_labels}}{The labels of the color bars. Defaults to the dim names for the proportions.}

\item{\code{txt_clr}}{The text color within the boxes}

\item{\code{txt_gpar}}{Similar to `txt_clr` but for more advanced styling with fontfamily (see [grid::gpar()]). *Note* that
col & cex are overridden.}

\item{\code{title}}{The plot title if any}

\item{\code{title_cex}}{The font-size multiplier for the title}

\item{\code{skip_shadows}}{Skip the shadow effect on the boxes}

\item{\code{mar}}{The margins for the plot.}

\item{\code{min_lwd}}{The minimum line width that is still shown. The pixels will most likely
not have the same fine resolution as the data and you therefore may want to hide
lines that are smaller than a certain amount.}

\item{\code{max_lwd}}{The maximum line width to show}

\item{\code{lwd_prop_type}}{The line can either be proportional to the \code{"set"} of transitions
(group of two box columns), to \code{"all"} transitions, or to each \code{"box"}. It defaults
to \code{"all"}.}

\item{\code{data}}{Internal storage variable. Should not be accessed directly.}
}}

\section{Methods}{

\describe{
\item{\code{addBoxStyle(fill, txt, gpar)}}{Adds colors or extends existing one so that they match the transition matrix.
The fill corresponds to the fill_clr and txt corresponds to the txt_clr. If
the colors are missing and the transitions consist of only two columns the default
colors will be used. If the matrix is being extended and these values are missing the
values from the previous last column will be used for the default columns.}

\item{\code{addTransitions(mtrx, label, txt, fill_clr, txt_clr, txt_gpar)}}{Add a transition matrix. The input has to be a numerical matrix between 2 and 3 dimensions.
If you don't provide the txt field the box' text field will be deduced from the transition matrix'
dimnames. The fill_clr and txt_clr are passed on to the \code{addBoxStyle} function.}

\item{\code{arrowWidths(set_no, add_width)}}{Retrieves the details regarding arrow sizes for each arrow within the transition
group}

\item{\code{boxPositions(col)}}{The box positions as a list with scalars for the positions:
\enumerate{
 \item \emph{x} The center x-position
 \item \emph{y} The center y-position
 \item \emph{right} The right edge
 \item \emph{left} The left edge
 \item \emph{top} The top edge
 \item \emph{bottom} The bottom edge
 \item \emph{height} The box height
 \item \emph{width} The box width
 \item \emph{unit} The unit used for the values (npc)
}}

\item{\code{boxSizes(col)}}{Gets the size of the boxes. The \code{col} argument should
be either an integer or 'last'}

\item{\code{getDim()}}{Gets the current dimensions of the transitions}

\item{\code{getTransitionSet(no, reduce_dim = FALSE)}}{Gets a specific set of transitions. If the \code{reduce_dim} is set
to TRUE it will only return a 2-dimensional matrix even if the original
has a 3rd proportions dimension}

\item{\code{getYProps(col)}}{Gets the proportions after removing the \code{vertical_space}
between the boxes}

\item{\code{initialize(transitions, label, txt, fill_clr, txt_clr, txt_gpar, id, ...)}}{Set up a Transition object. The \code{transitions} should be a 2D or 3D matrix
as defined in the \code{$addTransitions} section and not as later internally stored.}

\item{\code{noCols()}}{Gets the number of columns, i.e. the number of transitions}

\item{\code{noRows(no)}}{Gets the number of boxes in each row.
If multiple rows the number of rows may differ
betwen each transition matrix we therefore need
to specify what transitions that we refer to.
If no value is specified it returns all of them.}

\item{\code{render(new_page = TRUE)}}{Call this to render the full graph. The \code{new_page} argument
is for creating a new plot, set this to \code{FALSE}
if you want to combine this plot with another or if you have
additional viewports that you intend to use.}

\item{\code{trnstnSizes(set_no)}}{Gets the transitions per box as a 2D matrix. For the proportions
it also adds an attribute \code{attr('props', prop_mtrx)} that
is a 2D matrix with the corresponding proportions.}
}}

\examples{
# Transitions
set.seed(1)
n <- 10
my_data <-
  data.frame(
    Var_a = sample(c(
      "Test 1",
      "Test 2",
      "Test 3"
    ),
    size = n,
    replace = TRUE,
    prob = 3:1
    ),
    Var_b = sample(c(
      "Test 1",
      "Test 2",
      "Test 3"
    ),
    size = n,
    replace = TRUE,
    prob = 1:3
    )
  )
mtrx <- with(
  my_data,
  table(Var_a, Var_b)
)

# Initialize the transition plot
transitions <- getRefClass("Transition")$new(mtrx,
                                             label = c("Before", "After"))

# Render the plot
transitions$render()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docx_document.R
\name{prCaptionFix}
\alias{prCaptionFix}
\title{Fixes the caption for elements without caption}
\usage{
prCaptionFix(outFile)
}
\arguments{
\item{outFile}{The name of the file}
}
\value{
outFile The name of the file
}
\description{
Fixes the caption for elements without caption
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot_helpers.R
\name{prTpGetBoxPositions}
\alias{prTpGetBoxPositions}
\title{Gets the box position}
\usage{
prTpGetBoxPositions(
  no,
  side,
  transitions,
  prop_start_sizes,
  prop_end_sizes,
  tot_spacing,
  box_width
)
}
\arguments{
\item{no}{The box number}

\item{side}{The right or left side}

\item{transitions}{The flows to or from}

\item{prop_start_sizes}{The size of the start boxes}

\item{prop_end_sizes}{The size of the end boxes}

\item{tot_spacing}{The proportion of the vertical space that is to be left
empty. It is then split evenly between the boxes.}

\item{box_width}{The width of the box. By default the box is one fourth of
the plot width.}
}
\value{
\code{list(top, left, bottom, right, width, height)}
}
\description{
Gets the box position
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescriptionStatsBy_prAddDescUnitColumn.R
\name{prAddDescUnitColumn}
\alias{prAddDescUnitColumn}
\title{Add a units column to the results}
\usage{
prAddDescUnitColumn(results, x, use_units, units_column_name)
}
\arguments{
\item{results}{The results that we want to add the column to}

\item{x}{If a data.frame it will be used as the data source for the variables in the \code{...} parameter.
If it is a single variable it will be the core value that want the statistics for.
In the print this is equivalent to the output of this function.}

\item{use_units}{If the Hmisc package's units() function has been employed
it may be interesting to have a column at the far right that indicates the
unit measurement. If this column is specified then the total column will
appear before the units (if specified as last). You can also set the value to
\code{"name"} and the units will be added to the name as a parenthesis,
e.g. Age (years).}

\item{units_column_name}{The name of the units column. Used if use_units = TRUE}
}
\value{
results with added column
}
\description{
Add a units column to the results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxGrobs_connect.R, R/boxGrobs_s3_connect.R
\name{connectGrob}
\alias{connectGrob}
\alias{print.connect_boxes}
\alias{plot.connect_boxes}
\alias{grid.draw.connect_boxes_list}
\alias{print.connect_boxes_list}
\alias{plot.connect_boxes_list}
\alias{connect}
\alias{connect.default}
\alias{connect.Gmisc_list_of_boxes}
\title{Connect boxes with arrows}
\usage{
connectGrob(
  start,
  end,
  type = c("vertical", "horizontal", "L", "-", "Z", "N", "fan_in_top", "fan_in_center",
    "side"),
  subelmnt = c("right", "left"),
  lty_gp = getOption("connectGrob", default = gpar(fill = "black")),
  arrow_obj = getOption("connectGrobArrow", default = arrow(ends = "last", type =
    "closed")),
  arrow_size = NULL,
  split_pad = unit(2, "mm"),
  margin = unit(2, "mm"),
  smooth = FALSE,
  corner_radius = unit(3, "mm"),
  side = c("auto", "left", "right"),
  label = NULL,
  label_gp = grid::gpar(cex = 0.9),
  label_bg_gp = grid::gpar(fill = "white", col = NA),
  label_pad = unit(1.5, "mm"),
  label_pos = c("mid", "near_start", "near_end"),
  label_offset = unit(2, "mm")
)

\method{print}{connect_boxes}(x, ...)

\method{plot}{connect_boxes}(x, ...)

\method{grid.draw}{connect_boxes_list}(x, recording = TRUE)

\method{print}{connect_boxes_list}(x, ...)

\method{plot}{connect_boxes_list}(x, ...)

connect(x, ...)

\method{connect}{default}(x, ...)

\method{connect}{Gmisc_list_of_boxes}(x, from = NULL, to = NULL, ...)
}
\arguments{
\item{start}{A \code{boxGrob}/\code{boxPropGrob}, or a list of boxes (many-to-one).}

\item{end}{A \code{boxGrob}/\code{boxPropGrob}, or a list of boxes (one-to-many).}

\item{type}{Connector type, see Details.}

\item{subelmnt}{For split boxes, which sub-element to anchor to: \code{"left"} or \code{"right"}.}

\item{lty_gp}{A \code{\link[grid:gpar]{grid::gpar()}} controlling line appearance. Can also be set globally via
\code{options(connectGrob = ...)}.}

\item{arrow_obj}{Arrow specification created with \code{\link[grid:arrow]{grid::arrow()}}. Can also be set globally via
\code{options(connectGrobArrow = ...)}.}

\item{arrow_size}{Optional numeric (mm). When supplied, overrides the arrow head length in
\code{arrow_obj} without requiring the user to rebuild the entire arrow specification.}

\item{split_pad}{Padding around the shared bend point for multi-box connections.
Numeric values are interpreted as millimeters.}

\item{margin}{For \code{type = "fan_in_top"}, the margin applied at the left and right ends of the
end box top edge before distributing attachment points. Numeric values are interpreted
as millimeters.}

\item{smooth}{Logical; if \code{TRUE} interior orthogonal corners are replaced by smooth
Bézier arcs. Default \code{FALSE} preserves the existing sharp-corner behaviour.}

\item{corner_radius}{Radius of the rounded corner arc when \code{smooth = TRUE}.
A \code{\link[grid:unit]{grid::unit()}} or numeric (interpreted as millimeters). Default \code{unit(3, "mm")}.}

\item{side}{For \code{type = "side"}, which side of the start box to exit from.
\code{"auto"} (default) picks the side that faces the end box.}

\item{label}{Optional text label for one-to-one connectors (e.g. \code{"yes"} / \code{"no"}).
Only supported when both \code{start} and \code{end} are single boxes.}

\item{label_gp}{A \code{\link[grid:gpar]{grid::gpar()}} controlling label appearance.}

\item{label_bg_gp}{A \code{\link[grid:gpar]{grid::gpar()}} controlling label background appearance.
Defaults to a white background with no border.}

\item{label_pad}{Padding inside the label background. Numeric values are interpreted
as millimeters.}

\item{label_pos}{Where to place the label along the connector: \code{"mid"}, \code{"near_start"}, or \code{"near_end"}.}

\item{label_offset}{Offset for the label away from the connector line.}

\item{x}{A `list` of boxes (will be converted to `Gmisc_list_of_boxes` if needed).}

\item{...}{Arguments passed on to [`connectGrob`].}

\item{recording}{Passed to [grid::grid.draw()] when rendering each element in
the list. Defaults to `TRUE` (record drawing for later replay).}

\item{from}{The name (string) or index of the start box in `x`. Multiple values allowed.}

\item{to}{The name (string) or index of the end box in `x`. Multiple values allowed.}
}
\value{
\itemize{
\item One-to-one: a \code{\link[grid:grob]{grid::grob()}} with class \code{"connect_boxes"}.
\item One-to-many or many-to-one: a list of grobs with class \code{"connect_boxes_list"}.
}

The original list `x` (upgraded to `Gmisc_list_of_boxes`) with a new
  connection appended to its `"connections"` attribute. When printed, these
  connections are drawn.
}
\description{
Creates connectors between boxes.

A convenient way to connect boxes in a `Gmisc_list_of_boxes` or
simple `list` context, designed for piping (`|>`).
}
\details{
The function supports:
\itemize{
\item \strong{One-to-one}: a single start box connected to a single end box.
\item \strong{One-to-many}: a single start box connected to multiple end boxes.
\item \strong{Many-to-one}: multiple start boxes connected to a single end box.
}

Many-to-many connections are \strong{not supported}.

If either \code{start} or \code{end} is a list, a list of connector grobs is returned
(one per connection). Otherwise a single connector grob is returned.

Each connector stores its computed geometry in \code{attr(x, "line")}
(or for each element when a list is returned). This can be reused to construct
custom connectors using the calculated coordinates.
\subsection{Connector types}{

\code{type} controls the connector shape:
\itemize{
\item \code{"vertical"}: straight vertical connector
\item \code{"horizontal"}: straight horizontal connector
\item \code{"L"}: vertical then horizontal (direction chosen automatically)
\item \code{"-"}: straight horizontal connector at the end box y-position
\item \code{"Z"}: horizontal connector with two 90-degree turns
\item \code{"N"}: vertical connector with one horizontal segment
When connecting to or from multiple boxes, all connectors share the same bend height.
\item \code{"fan_in_top"}: many-to-one connector merging onto the \emph{top edge} of the end box
Attachment points are evenly distributed along the edge (with optional \code{margin}),
and all connectors share a common bend height.
\item \code{"fan_in_center"}: many-to-one connector that aggregates stems onto a horizontal
bus (shared at the computed bend height) and then a single centered trunk with an
arrow that points to the \emph{center} of the end box. Useful when you want a single
arrow to represent the merged flow (e.g. a middle trunk bus).
}

For \code{type = "N"} and \code{type = "fan_in_top"} with multi-box connections, a shared
bend position is computed so that the horizontal segment aligns visually across
all connectors.
}

\subsection{Labels}{

For one-to-one connectors you can add a text label (for example \code{"yes"} / \code{"no"}).
The label is placed near the midpoint of the connector.
The label is drawn with a white background for readability.
Use \code{label_pad} to control padding around the text and \code{label_offset} to move
the label away from the connector.
}

\subsection{Split boxes}{

When connecting to or from a \code{boxPropGrob}, \code{subelmnt} controls whether the left
or right sub-box x-coordinate is used as the anchor point.
}
}
\examples{
library(grid)
grid.newpage()

# Initiate the boxes that we want to connect
boxes <- list(
  start = boxGrob("Top", x = .5, y = .8),
  end = boxGrob("Bottom", x = .5, y = .2),
  side = boxPropGrob("Side", "Left", "Right", prop = .3, x = .2, y = .8),
  exclude = boxGrob("Exclude:\n - Too sick\n - Prev. surgery", x = .8, y = .5, just = "left")
)

# Connect the boxes and print/plot them
connectGrob(boxes$start, boxes$end, "vertical")
connectGrob(boxes$start, boxes$side, "horizontal")
connectGrob(boxes$start, boxes$exclude, "L")

# We can also connect to/from lists
side_boxes <- list(
  left = boxGrob("Left", x = attr(boxes$side, "coords")$left_x, y = .5),
  right = boxGrob("Right", x = attr(boxes$side, "coords")$right_x, y = .5)
)

connectGrob(boxes$side, side_boxes$left, "v", "l")
connectGrob(boxes$side, side_boxes$right, "v", "r")

# Fan-in center example: multiple starts into one center bus and single trunk
list(
  boxes$start,
  boxGrob("S2", x = .3, y = .7),
  boxGrob("S3", x = .7, y = .7)
) |>
  connectGrob(boxes$end, type = "fan_in_center")

# Print the boxes
boxes
side_boxes
}
\seealso{
[`connectGrob`]

Other flowchart components:
\code{\link[=align]{align()}},
\code{\link[=append]{append()}},
\code{\link[=boxGrob]{boxGrob()}},
\code{\link[=boxHeaderGrob]{boxHeaderGrob()}},
\code{\link[=boxPropGrob]{boxPropGrob()}},
\code{\link{boxShapes}},
\code{\link[=coords]{coords()}},
\code{\link[=distance]{distance()}},
\code{\link[=equalizeWidths]{equalizeWidths()}},
\code{\link[=flowchart]{flowchart()}},
\code{\link[=insert]{insert()}},
\code{\link[=move]{move()}},
\code{\link[=moveBox]{moveBox()}},
\code{\link[=spread]{spread()}}
}
\concept{flowchart components}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transitionPlot_helpers.R
\name{prTpPlotBoxes}
\alias{prTpPlotBoxes}
\title{Plot boxes and arrows}
\usage{
prTpPlotBoxes(
  overlap_order,
  transition_flow,
  no_boxes,
  box_width,
  tot_spacing,
  txt,
  cex,
  txt_gpar = NULL,
  prop_start_sizes,
  prop_end_sizes,
  box_prop,
  lwd_prop_total = lwd_prop_total,
  fill_start_clr,
  fill_end_clr,
  txt_start_clr,
  txt_end_clr,
  lwd = 2,
  line_col = "#000000",
  min_lwd,
  max_lwd,
  overlap_add_width,
  overlap_bg_clr,
  type_of_arrow,
  abs_arrow_width,
  arrow_clr,
  transition_arrow_props,
  color_bar_subspace,
  plot_arrows = TRUE,
  proportion = FALSE
)
}
\arguments{
\item{overlap_order}{The order from first->last for the lines. This means that the last
line will be on top while the first one will appear at the bottom. This should be provided
as a vector.}

\item{transition_flow}{This should be a matrix with the size of the transitions.
The unit for each cell should be number of observations, row/column-proportions
will show incorrect sizes. The matrix needs to be square. The best way to generate
this matrix is probably just do a \code{table(starting_state, end_state)}. The rows
represent the starting positions, while the columns the end positions. I.e. the first
rows third column is the number of observations that go from the first class to the
third class.}

\item{no_boxes}{Number of boxes to plot}

\item{box_width}{The width of the box. By default the box is one fourth of
the plot width.}

\item{tot_spacing}{The proportion of the vertical space that is to be left
empty. It is then split evenly between the boxes.}

\item{txt}{The text}

\item{cex}{The cex \code{\link{gpar}} of the text}

\item{txt_gpar}{The [grid::gpar()] object that is overridden with col & cex by other parameters}

\item{prop_start_sizes}{The proportion of the different boxes
to the left}

\item{prop_end_sizes}{The proportion of the different boxes
to the right}

\item{box_prop}{If you want the boxes to have proportions indicating some other factors then input
a matrix with quantiles for the proportions. Note the size must be \code{nrow(transition_flow) x 2}.}

\item{lwd_prop_total}{The width of the lines may be proportional to either the
other flows from that box, or they may be related to all flows. This is a boolean
parameter that is set to true by default, i.e. relating to all flows.}

\item{fill_start_clr}{The color of the boxes to the left}

\item{fill_end_clr}{The color of the boxes to the right}

\item{txt_start_clr}{The text color of the start boxes. This can either
be a single value or a vector if you desire different colors for each
box. If you specify box_prop then this has to be a 2 column matrix.}

\item{txt_end_clr}{The text color of the end boxes. This can either
be a single value or a vector if you desire different colors for each
box. If you specify box_prop then this has to be a 2 column matrix.}

\item{lwd}{The line width}

\item{line_col}{The color of the line}

\item{min_lwd}{The minimum width of the line that we want to illustrate the
tranisition with.}

\item{max_lwd}{The maximum width of the line that we want to illustrate the
tranisition with.}

\item{overlap_add_width}{The width of the white cross-over line. You can specify this as a scalar
multiplication of the current line width. In case of non-grid arrows then you can also have this
as a unit which is recommended as it looks better. If the scalar is < 1 then the overlap is ignored.}

\item{overlap_bg_clr}{In order to enhance the 3D perspective and to make it easier
to follow arrows the arrows have a background color to separate them from those underneath.}

\item{type_of_arrow}{The types of arrow may be grid, simple, or gradient. Simple grid
arrows are the \code{\link[grid:grid.bezier]{bezierGrob}} arrows (not that pretty),
simple is the \code{\link{bezierArrowSmpl}} that I've created to get a more exact
control of the arrow position and width, while gradient
corresponds to \code{\link{bezierArrowGradient}}
allowing the arrow to have a fill color that slowly turns into the color of the arrow.}

\item{abs_arrow_width}{The absolute width of the arrow}

\item{arrow_clr}{The color of the arrows. Usually black, can be a vector indicating each arrow
from first to last arrow (counting from the top). If the vector is of the same length as the
boxes then all box arrows will have the same color (that is all the arrows stemming from the
left boxes)}

\item{transition_arrow_props}{The proportions of the different transitions if
available.}

\item{color_bar_subspace}{If there is little or no difference
at the low/high proportions of the spectrum then it
can be of interest to focus the color change to the center
leaving the tails constant}

\item{plot_arrows}{If we are plotting shadow boxes then
arrows should not be plotted and this should be set to \code{FALSE}}

\item{proportion}{It there is a proportion}
}
\value{
\code{void}
}
\description{
Outputs all the boxes and arrow
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier_private_functions.R
\name{getGridVal}
\alias{getGridVal}
\title{Gets grid value}
\usage{
getGridVal(x, default.units, axisTo = "x")
}
\arguments{
\item{x}{Value}

\item{default.units}{The unit type}

\item{axisTo}{The axis that is used, useful for "npc" where
there is a big difference in height and width}
}
\value{
float
}
\description{
Returns the raw value in units if the provided is of type unit else
it returns the default unit.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrieve_and_has.R
\name{retrieve}
\alias{retrieve}
\title{An R alternative to the lodash \code{get} in JavaScript}
\usage{
retrieve(sourceList, path, default = NA)
}
\arguments{
\item{sourceList}{The \code{list()}/\code{c()} that is to be searched for the element}

\item{path}{A string that can be separated by [,] or ., the string \code{"elementname1.1.elementname"}
is equivalent to \code{"elementname1[[1]]]elementname"}. Note that the function doesn't check
the validity of the path - it only separates and tries to address that element with `[[]]`.}

\item{default}{The value to return if the element isn't found}
}
\value{
Returns a sub-element from \code{sourceList} or the \code{default} value.
}
\description{
This is a handy function for retrieving items deep in a nested structure
without causing error if not found
}
\examples{
source <- list(a = list(b = 1, `odd.name` = 'I hate . in names', c(1,2,3)))
retrieve(source, "a.b")
retrieve(source, "a.b.1")
retrieve(source, "a.odd\\\\.name")
retrieve(source, "a.not_in_list")

}
\seealso{
Other lodash similar functions:
\code{\link[=has]{has()}}
}
\concept{lodash similar functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yamlDump.R
\name{yamlDump}
\alias{yamlDump}
\title{Outputs an object}
\usage{
yamlDump(x)
}
\arguments{
\item{x}{An object that \code{\link[yaml]{as.yaml}} accepts}
}
\value{
\code{void}
}
\description{
Manually viewing a list object can be tricky where the natural print
can be hard to work through. The config format *yaml* is increadibly
dense and useful not only for writing configs but also viewing them
which `yamlDump` helps with.
}
\examples{
some_fancy_list <- list(complex = list(some_data = 1:3,
                                       other_data = list(name = "Max")),
                        simple = "awesome overview")
yamlDump(some_fancy_list)
#complex:
#  some_data:
#  - 1
#  - 2
#  - 3
#  other_data:
#    name: Max
#simple: awesome overview

# If you got a character json you can also input it directly
# and the function will automatically convert it to a list
yamlDump('{"a":{"b":["1"]}}')

}
